/*
 * $Id: //depot/prod/newport_news/main/modules/jndmp/src/main/java/com/netapp/ndmp/test/NdmpTest.java#4 $
 *
 * Copyright (c) 2010 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.ndmp.test;

import java.net.UnknownHostException;

import org.acplt.oncrpc.XdrAble;

import com.netapp.ndmp.NdmpException;
import com.netapp.ndmp.NdmpMessageException;
import com.netapp.ndmp.NdmpSession;
import com.netapp.ndmp.NdmpUtil;
import com.netapp.ndmp.rpcgen.ndmp_Snapmirror_message;
import com.netapp.ndmp.rpcgen.ndmp_Snapvault_primary_message;
import com.netapp.ndmp.rpcgen.ndmp_Snapvault_secondary_message;
import com.netapp.ndmp.rpcgen.ndmp_config_get_host_info_reply;
import com.netapp.ndmp.rpcgen.ndmp_config_get_server_info_reply;
import com.netapp.ndmp.rpcgen.ndmp_message;
import com.netapp.ndmp.rpcgen.ndmp_sm_get_info_reply;
import com.netapp.ndmp.rpcgen.ndmp_snap_dir_list_reply;
import com.netapp.ndmp.rpcgen.ndmp_snap_dir_list_request;
import com.netapp.ndmp.rpcgen.ndmp_snap_message;
import com.netapp.ndmp.rpcgen.ndmp_snap_qtree_contents_reply;
import com.netapp.ndmp.rpcgen.ndmp_snap_qtree_contents_reply_v4;
import com.netapp.ndmp.rpcgen.ndmp_snap_qtree_contents_request;
import com.netapp.ndmp.rpcgen.ndmp_snap_qtree_contents_request_v4;
import com.netapp.ndmp.rpcgen.ndmp_sv_error;
import com.netapp.ndmp.rpcgen.ndmp_sv_status;
import com.netapp.ndmp.rpcgen.ndmp_sv_status_v4;
import com.netapp.ndmp.rpcgen.ndmp_svp_get_status_reply;
import com.netapp.ndmp.rpcgen.ndmp_svp_get_status_reply_v4;
import com.netapp.ndmp.rpcgen.ndmp_svp_get_status_request;
import com.netapp.ndmp.rpcgen.ndmp_svs_get_status_reply;
import com.netapp.ndmp.rpcgen.ndmp_svs_get_status_reply_v4;
import com.netapp.ndmp.rpcgen.ndmp_svs_get_status_request;
import com.netapp.ndmp.rpcgen.qtree_info;
import com.netapp.ndmp.rpcgen.snap_info;
import com.netapp.ndmp.rpcgen.snap_list_q;

public class NdmpTest {
    private static final String NDMP_LOGIN = "root";
    private static final String NDMP_PASSWORD = "";

    private NdmpTest() { }

    private static void getHostInfo(NdmpSession s) throws NdmpException {

        ndmp_config_get_host_info_reply hostInfoReply = new ndmp_config_get_host_info_reply();
        s.sendMessage(ndmp_message.NDMP_CONFIG_GET_HOST_INFO, null, hostInfoReply);
        System.out.printf("Host Info:%n  ID: %s%n  Name: %s%n  OS: %s%n  Version: %s%n",
                hostInfoReply.hostid, hostInfoReply.hostname,
                hostInfoReply.os_type, hostInfoReply.os_vers);
    }

    private static void getServerInfo(NdmpSession s) throws NdmpException {

        ndmp_config_get_server_info_reply serverInfoReply = new ndmp_config_get_server_info_reply();
        s.sendMessage(ndmp_message.NDMP_CONFIG_GET_SERVER_INFO, null, serverInfoReply);
        System.out.printf("Server Info:%n  Product Name: %s%n  Revision Number: %s%n  Vendor Name: %s%n",
                serverInfoReply.product_name, serverInfoReply.revision_number,
                serverInfoReply.vendor_name);
    }

    private static void getSvpStatus(NdmpSession s) throws NdmpException {

        ndmp_svp_get_status_request statusRequst = new ndmp_svp_get_status_request();
        XdrAble statusReply;
        if (4 <= s.getExtensionVersion(NdmpSession.SNAP_EXT_CLASS)) {
            statusReply = new ndmp_svp_get_status_reply_v4();
        } else {
            statusReply = new ndmp_svp_get_status_reply();
        }
        statusRequst.vol = "";
        try {
            s.sendMessage(ndmp_Snapvault_primary_message.NDMP_SVP_GET_STATUS, statusRequst, statusReply);
        } catch (NdmpMessageException e) {
            if (ndmp_sv_error.NDMP_SV_NOT_LICENSED != e.getErrorCode()) {
                throw e;
            }
            System.out.printf("Snapvault primary not licensed%n");
            return;
        }
        int relationshipCount;
        if (statusReply instanceof ndmp_svp_get_status_reply_v4) {
            relationshipCount = ((ndmp_svp_get_status_reply_v4) statusReply).svp_status_list.length;
        } else {
            relationshipCount = ((ndmp_svp_get_status_reply) statusReply).svp_status_list.length;
        }
        System.out.printf("Snapvault primary relationships: %d%n", relationshipCount);
        if (statusReply instanceof ndmp_svp_get_status_reply_v4) {
            for (ndmp_sv_status_v4 sv : ((ndmp_svp_get_status_reply_v4) statusReply).svp_status_list) {
                System.out.printf("\tSV %s:%s %s:%s%n", sv.srcsys, sv.srcpath, sv.dstsys, sv.dstpath);
            }
        } else {
            for (ndmp_sv_status sv : ((ndmp_svp_get_status_reply) statusReply).svp_status_list) {
                System.out.printf("\tSV %s:%s %s:%s%n", sv.srcsys, sv.srcpath, sv.dstsys, sv.dstpath);
            }
        }
    }

    private static void getSvsStatus(NdmpSession s) throws NdmpException {

        ndmp_svs_get_status_request statusRequst = new ndmp_svs_get_status_request();
        XdrAble statusReply;
        if (4 <= s.getExtensionVersion(NdmpSession.SNAP_EXT_CLASS)) {
            statusReply = new ndmp_svs_get_status_reply_v4();
        } else {
            statusReply = new ndmp_svs_get_status_reply();
        }
        statusRequst.vol = "";
        try {
            s.sendMessage(ndmp_Snapvault_secondary_message.NDMP_SVS_GET_STATUS, statusRequst, statusReply);
        } catch (NdmpMessageException e) {
            if (ndmp_sv_error.NDMP_SV_NOT_LICENSED != e.getErrorCode()) {
                throw e;
            }
            System.out.printf("Snapvault seconday not licensed%n");
            return;
        }
        int relationshipCount;
        if (statusReply instanceof ndmp_svs_get_status_reply_v4) {
            relationshipCount = ((ndmp_svs_get_status_reply_v4) statusReply).svs_status_list.length;
        } else {
            relationshipCount = ((ndmp_svs_get_status_reply) statusReply).svs_status_list.length;
        }
        System.out.printf("Snapvault seconday relationships: %d%n", relationshipCount);
        if (statusReply instanceof ndmp_svs_get_status_reply_v4) {
            for (ndmp_sv_status_v4 sv : ((ndmp_svs_get_status_reply_v4) statusReply).svs_status_list) {
                System.out.printf("\tSV %s:%s %s:%s%n", sv.srcsys, sv.srcpath, sv.dstsys, sv.dstpath);
            }
        } else {
            for (ndmp_sv_status sv : ((ndmp_svs_get_status_reply) statusReply).svs_status_list) {
                System.out.printf("\tSV %s:%s %s:%s%n", sv.srcsys, sv.srcpath, sv.dstsys, sv.dstpath);
            }
        }
    }

    private static void testFiler(String hostname) throws NdmpException, UnknownHostException {

        NdmpSession s = new NdmpSession(hostname, 30000, 0);
        s.connect(NDMP_LOGIN, NDMP_PASSWORD);

        System.out.printf("Snap Extension version: %d%n", s.getExtensionVersion(NdmpSession.SNAP_EXT_CLASS));

        getHostInfo(s);
        getServerInfo(s);
        getSvpStatus(s);
        getSvsStatus(s);

        s.close();
        System.out.println();
    }

    private static void testQtreeContents() throws NdmpException, UnknownHostException {

        NdmpSession s = new NdmpSession("manowar.rtp.netapp.com", 30000, 0);
        s.connect(NDMP_LOGIN, NDMP_PASSWORD);

        System.out.printf("Snap Extension version: %d%n", s.getExtensionVersion(NdmpSession.SNAP_EXT_CLASS));

        String volPath = "/vol/ben_test";

        XdrAble qtreeContentsRequest;
        XdrAble qtreeContentsReply;
        if (4 <= s.getExtensionVersion(NdmpSession.SNAP_EXT_CLASS)) {
            qtreeContentsRequest = new ndmp_snap_qtree_contents_request_v4();
            ((ndmp_snap_qtree_contents_request_v4) qtreeContentsRequest).volpath = volPath;
            qtreeContentsReply = new ndmp_snap_qtree_contents_reply_v4();
        } else {
            qtreeContentsRequest = new ndmp_snap_qtree_contents_request();
            ((ndmp_snap_qtree_contents_request) qtreeContentsRequest).pathname = volPath;
            qtreeContentsReply = new ndmp_snap_qtree_contents_reply();
        }
        s.sendMessage(ndmp_snap_message.NDMP_SNAP_GET_QTREE_CONTENTS, qtreeContentsRequest, qtreeContentsReply);
        snap_list_q[] list;
        if (qtreeContentsReply instanceof ndmp_snap_qtree_contents_reply_v4) {
            list = ((ndmp_snap_qtree_contents_reply_v4) qtreeContentsReply).list;
        } else {
            list = ((ndmp_snap_qtree_contents_reply) qtreeContentsReply).list;
        }
        if (1 != list.length) {
            System.out.printf("Wrong number of volumes: %d%n", list.length);
            return;
        }
        snap_list_q slq = list[0];
        System.out.printf("Volname: %s%n", slq.volname);
        for (snap_info si : slq.info) {
            System.out.printf("SnapName: %s Date: %d%n", si.snapname, si.date);
            for (qtree_info qi : si.q_info) {
                System.out.printf("Name: %s Source: %s Contents: %d Date: %d%n",
                        qi.name, qi.source, qi.contents, qi.date);
            }
        }

        s.close();
        System.out.println();
    }

    private static void testMyServer() throws UnknownHostException, NdmpException {

        NdmpSession s = new NdmpSession("127.0.0.1", 30000, 0);
        s.connect(NDMP_LOGIN, NDMP_PASSWORD);
        System.out.printf("Snap Extension version: %d%n", s.getExtensionVersion(NdmpSession.SNAP_EXT_CLASS));
        s.close();
    }

    private static void testKfs() throws UnknownHostException, NdmpException {

        NdmpSession s = new NdmpSession("manowar.rtp.netapp.com", 12005, 30000, 0);
        s.connect(NDMP_LOGIN, NDMP_PASSWORD);
        System.out.printf("Snap Extension version: %d%n", s.getExtensionVersion(NdmpSession.SNAP_EXT_CLASS));
        getSvpStatus(s);
        getSvsStatus(s);
        s.close();
    }

    private static void testSvp() throws UnknownHostException, NdmpException {

        NdmpSession s = new NdmpSession("manowar.rtp.netapp.com", 30000, 0);
        s.connect(NDMP_LOGIN, NDMP_PASSWORD);
        getSvpStatus(s);
        s.close();
    }

    private static void testSnapDirList() throws UnknownHostException, NdmpException {

        NdmpSession s = new NdmpSession("10.61.165.24", 30000, 0);
        s.connect(NDMP_LOGIN, NDMP_PASSWORD);

        ndmp_snap_dir_list_request listRequst = new ndmp_snap_dir_list_request();
        listRequst.pathname = "/vol/vol0/etcx";
        listRequst.flags = 0;
        listRequst.max_entries = 10;
        ndmp_snap_dir_list_reply listReply = new ndmp_snap_dir_list_reply();

        s.sendMessage(ndmp_snap_message.NDMP_SNAP_DIR_LIST, listRequst, listReply);

        System.out.printf("%s%n", NdmpUtil.getXml(listReply));

        s.close();
    }

    private static void testSmGetInfo() throws UnknownHostException, NdmpException {

        NdmpSession s = new NdmpSession("manowar.rtp.netapp.com", 30000, 0);
        s.connect(NDMP_LOGIN, NDMP_PASSWORD);

        ndmp_sm_get_info_reply infoReply = new ndmp_sm_get_info_reply();

        s.sendMessage(ndmp_Snapmirror_message.NDMP_SM_GET_INFO, null, infoReply);

        System.out.printf("%s%n", NdmpUtil.getXml(infoReply));

        s.close();
    }

    public static void main(String[] args) throws NdmpException, UnknownHostException {


        testFiler("asdf.rtp.netapp.com");
        /*
        testFiler("10.61.167.31");
        testFiler("10.61.167.32");
        testFiler("10.61.167.33");
*/
//        testQtreeContents();
//        testMyServer();
//        testKfs();
//        testSnapDirList();
//        testSmGetInfo();
//        testSvp();
    }
}
