package com.netapp.offtap.ndmpcopy;

import java.net.UnknownHostException;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpLog4j;
import com.netapp.offtap.ndmpcopy.messages.connect.NdmpInvalidAuthException;
import com.netapp.offtap.ndmpcopy.ndmpcopy.NdmpCopyData;
import com.netapp.offtap.ndmpcopy.ndmpcopy.NdmpCopyThread;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdArgs;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdException;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdSemantics;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpFilerInfo;
import com.netapp.offtap.ndmpcopy.util.NdmpFatalException;
import com.netapp.offtap.ndmpcopy.util.NdmpPseudoStream;

public class NDMPCopy {

	public static void copy(String srcHostname, String srcPath, String srcUsername, String srcPassword,
			String dstHostname, String dstPath, String dstUsername, String dstPassword) throws NdmpFatalException, NdmpCopyCmdException, UnknownHostException, NdmpInvalidAuthException{
		NdmpCopyCmdArgs cmdArgs = new NdmpCopyCmdArgs();
		NdmpCopyData copyData = new NdmpCopyData();
		
		NdmpFilerInfo srcFiler = new NdmpFilerInfo(srcHostname, srcPath, srcUsername, srcPassword, "md5");
		NdmpFilerInfo dstFiler = new NdmpFilerInfo(dstHostname, dstPath, dstUsername, dstPassword, "md5");
		cmdArgs.filerInfo[NdmpCopyCmdArgs.srcFiler] = srcFiler;
		cmdArgs.filerInfo[NdmpCopyCmdArgs.dstFiler] = dstFiler;
		
		copyData.debugLog = new NdmpLog4j(NDMPCopy.class);
		copyData.out = new NdmpPseudoStream();
		cmdArgs = NdmpCopyCmdSemantics.checkSemantics (cmdArgs);

		copyData.cmdArgs = cmdArgs;
		NdmpCopyThread thread = new NdmpCopyThread(copyData);
		
		thread.start();
		while(!thread.isDone()){
			copyData.out.printToStream(System.out);
		}
		
		//Only handles fatal exceptions... not the greatest
		Exception exception = thread.getException();
		if(null != exception){
			if(exception instanceof NdmpFatalException){
				if(exception.getMessage().contains("Authentication failed")){
					throw new NdmpInvalidAuthException(exception);
				} else {
					throw (NdmpFatalException) exception;
				}
			} else if(exception instanceof UnknownHostException){
				throw (UnknownHostException) exception;
			}
		}
		
	}
	
	   
//	Let's not use Aeonflux objects and save ourselves the circular dependency, shall we?
//	public static void copy(NtapController srcFiler, String srcPath, NtapController dstFiler, String dstPath){
//	    copy(srcFiler.getConnectIpAddress(), srcPath, srcFiler.getConnectUserName(), srcFiler.getConnectPassword(),
//	            dstFiler.getConnectIpAddress(), dstPath, dstFiler.getConnectUserName(), dstFiler.getConnectPassword());
//	}
}
