package com.netapp.offtap.ndmpcopy;
  

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLogFactory;
import com.netapp.offtap.ndmpcopy.ndmpcopy.NdmpCopyData;
import com.netapp.offtap.ndmpcopy.ndmpcopy.NdmpCopyThread;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdArgs;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdException;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdParser;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdSemantics;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdSyntax;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;
import com.netapp.offtap.ndmpcopy.util.NdmpPseudoStream;

/**
 * All commands from the console are routed through this interface (and
 * JAF). 
 */
public class NdmpCopyAdmin {

	private	static	int	ndmpcopyID = 0;
	private static	Object	copyMutex = new Object ();

	/**
	 * All commands are routed through this main() call
	 */
	public static void main (String args[]) {

		NdmpCopyCmdArgs		cmdArgs = null;
		NdmpCopyData		copyData = null;
		NdmpCopyCmdParser	cmdParser = null;
		NdmpCopyThread		copyThread = null;
		
		try {

			NdmpPseudoStream out = new NdmpPseudoStream ();		
			NdmpPseudoStream debugOut = new NdmpPseudoStream ();

			if (args.length < 1) {
				NdmpCopyCmdSyntax.printUsage ();
				return ;
			}

			cmdParser = new NdmpCopyCmdParser (args);
			try {
				cmdArgs = NdmpCopyCmdSyntax.checkSyntax (cmdParser);

				if (null == cmdArgs)
					return ;

				cmdArgs = NdmpCopyCmdSemantics.checkSemantics (cmdArgs);

			} catch (NdmpCopyCmdException e) {
				if (null != e)
					System.out.println (e.getMessage ());
				
				NdmpCopyCmdSyntax.printUsage ();
				return ;
			}		

			//Initialize the copyData.
			copyData = new NdmpCopyData (cmdArgs);
			
			synchronized (copyMutex) {
				copyData.copyID = ndmpcopyID++;
				ndmpcopyID = ndmpcopyID % 10000;
			}

			copyData.debugLog = NdmpDebugLogFactory.createDebugLog (
						NdmpCons.NDMP_DEBUG_PATH, copyData.copyID,
						copyData.cmdArgs.debugLevel, debugOut);

			copyData.out = out;

			copyData.debugLog.println(10, "Starting ndmpcopy operation");
			System.out.println ("Ndmpcopy: Starting copy [ " + 
				copyData.copyID + " ] ...");

			// this should pass exceptions up here !!
			copyThread = new NdmpCopyThread (copyData);
			copyThread.start();
		
			if (null == copyThread) {
				copyData.debugLog.println(10, "Error instantiating NdmpCopyThread");
				System.out.println ("Ndmpcopy: Could not start");
				copyData.debugLog.cleanup();
				return ;	
			}

			// loop till the interrupt or completion
			while (!copyThread.isDone ()) {
				System.out.flush();
				try {
					Thread.currentThread ().sleep (500);
				} catch (InterruptedException ignored) {
				}

				// this will flush the buffers for console output
				// as well as debug log output
				out.printToStream (System.out);
				debugOut.printToStream (System.out);
			}

			// this is where the copy gets the AXE
			while (!copyThread.isDone ()) {
				copyThread.killIt ();
				try {
					Thread.currentThread ().sleep (500);
				} catch (InterruptedException ignored) {
				}
			}
			out.printToStream (System.out);
			debugOut.printToStream (System.out);
			
			copyData.debugLog.println (10, "Ndmpcopy done");
			System.out.println ("Ndmpcopy: Done");	
		} catch (Exception e) {
			// error message here
			if (null != e && null != e.getMessage ())
				copyData.debugLog.println (10, e, "Unexpected exception. Ndmpcopy aborted");
			else
				copyData.debugLog.println (10, "Ndmpcopy aborted");

			System.out.println ("Ndmpcopy: Aborted");
		}

		System.out.flush();
		copyData.debugLog.cleanup();
		return ;
	}
}
