package com.netapp.offtap.ndmpcopy.messagereader;

import com.netapp.offtap.ndmpcopy.messages.NdmpMessage;


/**
 * Internal queue structure used to pipe multiple messages from several ndmpd
 * sessions to a single control thread.
 */
public class NdmpMsgQueue { 
	
	private int queueLen, maxQueueLen;
	private QNode head, tail;

	private int threadlistCapacity = 10;	/* Capacity of the vectors */

	/**
	 * Constructs a message queue
	 * <p>
	 * @param len The maximum size of the given queue.  
	 */
	public NdmpMsgQueue (int len) { 
		queueLen = 0;
		maxQueueLen = len;
		
		head = tail = null;
	}

	/**
	 * Gets the next object from the message queue and doesn't block
	 * <p>
	 * @return Returns the next object in the queue.
	 */
	public QNode getNoSleep () {
		QNode data = null;

		synchronized (this) {
			if (head != null) {
				data = head;
				head = head.next;
				queueLen--;
				if (head == null) {
					tail = null;
					queueLen = 0;
				}
			}
		}
		return data;
	}

	/**
	 * Puts in an object into the queue and doesn't block
	 * <p>
	 * @param data The data element to put into the queue
	 * @return Whether the put operation succeeded
	 */
	public boolean putNoSleep (QNode qNode) {
	
		synchronized (this) {
			if (null == qNode)
				return false;

			if (queueLen > maxQueueLen) {
				return false;
			}

			if (tail != null) {
				tail.next =  qNode;
				tail = tail.next;
			} else 
				head = tail = qNode;
			
			queueLen++;
		}

		return true;
	}

	/**
	 * Puts in an object into the queue and doesn't block
	 * <p>
	 * @param data The data element to put into the queue
	 * @return Whether the put operation succeeded
	 */
	public boolean putNoSleep (NdmpMessage msg) {

		synchronized (this) {
			if (null == msg)
				return false;

			if (queueLen > maxQueueLen) {
				return false;
			}

			QNode qNode = new QNode (msg);
		
			if (tail != null) {
				tail.next =  qNode;
				tail = tail.next;
			} else 
				head = tail = qNode;
			
			queueLen++;
		}

		return true;
	}	
}
