
package com.netapp.offtap.ndmpcopy.messages;

//import netapp.cmds.ndmpc.*;
import java.io.*;
import java.util.*;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLog;
import com.netapp.offtap.ndmpcopy.messages.*;
import com.netapp.offtap.ndmpcopy.streams.*;

/**
 * This public class forms the base class from which all other NdmpdMessage
 * public classes extend.  The only variables in this class deal with the
 * header of the message.  The subpublic classes will contain methods to
 * extract the message body and executing it.
 */
public abstract class NdmpMessageData {
	
	/**
	 *  Read in the body, which is different across all NDMP messages.
	 *
	 * @param in The control input stream from which to read the rest
	 *         of the message
	 */
	public abstract void read (NdmpInput in) throws IOException ;

	/**
	 *  Writes out the NdmpdMessage body to the output stream.
	 *  Synchronization on the output stream is assumed to be handled
	 *  by the caller of the function.
	 */
	public abstract void write (NdmpOutput output) throws IOException ;

	/**
	 * Printing the body is used for debugging purposes.
	 * <p>
	 * @param debugLog The log to write to
	 */
	public abstract void writeLog (NdmpDebugLog debugLog) ;

	//public void println (OutputStream os) throws IOException ;

	public boolean isError () { return true; }

	public boolean isError (int errorcode) { return true; }
}
