package com.netapp.offtap.ndmpcopy.messages.config;

//import netapp.cmds.ndmpc.*;
import java.io.IOException;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLog;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageData;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageDataReply;
import com.netapp.offtap.ndmpcopy.streams.NdmpInput;
import com.netapp.offtap.ndmpcopy.streams.NdmpOutput;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;


/**
 * Retrieves information about the requested authentication type.  The message
 * is generally used for MD5 authentication, since this is the way in which we
 * construct and send back the challenge.
 */
public class NdmpMessageConfigGetAuthAttrRReply 
								extends NdmpMessageDataReply {

	public int authtype;
	
	public byte[] 	challenge;

	/**
	 * Basic constructor
	 *
	 */
	public NdmpMessageConfigGetAuthAttrRReply () {
	}

	public NdmpMessageConfigGetAuthAttrRReply (int authtype, 
						byte[] challenge) {
		this.authtype	= authtype;
		this.challenge	= challenge;
	}

	 public NdmpMessageConfigGetAuthAttrRReply (NdmpMessageData msg){
	 	//Dummy function
		// This is only impl for an NDMP server and will probably have
		// another parameter specifying the auth attr.
	}
	public void read (NdmpInput input) throws IOException {
		super.read (input);
		
		authtype 	= input.getLong();

		switch (authtype) {
			default:
				break;
			case NdmpCons.NDMP_AUTH_NONE:
			case NdmpCons.NDMP_AUTH_TEXT:
				break;
			case NdmpCons.NDMP_AUTH_MD5:
				challenge = input.getBytes(64);
				break;
		}
	}

	public void write (NdmpOutput output) throws IOException {
		// Only needed for server.
	}

	public void writeLog (NdmpDebugLog debugLog) {
		String challengeString = 
			"Challenge : 64 bytes in 16 x 4 format: \r\n \t";

		for (int i = 0; (i < 64 ) && 
				(NdmpCons.NDMP_AUTH_MD5 == authtype); 
						challengeString += "\r\n \t") 
			for (int j = i; i-j < 16; i++) 
				challengeString += (challenge[i] + " ");
		
		debugLog.println (50, challengeString);
		super.writeLog (debugLog);
	}


}
