package com.netapp.offtap.ndmpcopy.messages.config;

//import netapp.cmds.ndmpc.*;
import java.io.IOException;
import java.util.Properties;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLog;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageDataReply;
import com.netapp.offtap.ndmpcopy.streams.NdmpInput;
import com.netapp.offtap.ndmpcopy.streams.NdmpOutput;


/**
 * Retrieves information about the dumplets which the server supports.
 */
public class NdmpMessageConfigGetBuTypeInfoRReply 
		extends NdmpMessageDataReply {
	 
	protected NdmpButypeInfo[]	butypeInfo = null;

	/**
	 * Basic constructor
	 *
	 */
	public NdmpMessageConfigGetBuTypeInfoRReply () {
	}

	public void read (NdmpInput input) throws IOException {
		super.read (input);

		int infoCount = input.getLong();
		if (0 < infoCount)	
			butypeInfo = new NdmpButypeInfo [infoCount];
		
		for (int i = 0; i < infoCount; i++) {
			butypeInfo[i] = new NdmpButypeInfo();
			butypeInfo[i].butype_name = input.getString();
			butypeInfo[i].default_env = new Properties ();
			int j = input.getLong();
			while (j-- > 0) 
				butypeInfo[i].default_env.put (input.getString(), 
						input.getString());

			butypeInfo[i].attrs = input.getLong();
		}
	}

	public void write (NdmpOutput output) throws IOException {
		//For server.
	}

	public void writeLog (NdmpDebugLog debugLog) {
		String debugstring = "";
		int	infoCount = (null == butypeInfo)?0:butypeInfo.length;

		debugLog.println (50, "Total of " + infoCount + " butype infos.");
		for (int i = 0; i < infoCount; i++) {
			debugstring +=  "BuType Info["+i+"] :" +
			" Name : "+ butypeInfo[i].butype_name + "\r\n" + 
			"\t Env Var : "+ butypeInfo[i].default_env.toString() + "\r\n" +
			"\t Attr : " + butypeInfo[i].attrs + "\r\n";
		}
		debugLog.println (50, debugstring);
		super.writeLog (debugLog);
	}

	/**
	 * Check if the given env var is supported for the "dump" butype.
	 */
	public boolean checkEnv (String name) {
		int infoCount = (null == butypeInfo) ? 0 : butypeInfo.length ;

		for (int i=0; i<infoCount; i++) {
			if (0 == butypeInfo[i].butype_name.compareTo("dump"))
				if (null != butypeInfo[i].default_env.getProperty(name))
					return true;
				else
					return false;
		}
		return false;
	}
}
