package com.netapp.offtap.ndmpcopy.messages.config;

//import netapp.cmds.ndmpc.*;
import java.io.IOException;
import java.util.Properties;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLog;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageDataReply;
import com.netapp.offtap.ndmpcopy.streams.NdmpInput;
import com.netapp.offtap.ndmpcopy.streams.NdmpOutput;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;


/**
 * Retrieves information about the filesystem.  The public classes used are
 * based on the <code>DiskUsage</code> public classes.
 */
public class NdmpMessageConfigGetFsInfoRReply extends NdmpMessageDataReply {
	protected	NdmpFsInfo[]	fsInfo;	// List of fs Info.

	/**
	 * Basic constructor
	 */
	public NdmpMessageConfigGetFsInfoRReply () { 
	}

	public NdmpMessageConfigGetFsInfoRReply (int errorcode, 
										NdmpFsInfo[] fsInfo){
		super (errorcode);
		this.fsInfo		= fsInfo;
	}

	public void read (NdmpInput input) throws IOException {
		super.read (input);

		int len = input.getLong();

		if (0 < len)
			fsInfo = new NdmpFsInfo[len];

		for (int i = 0; i < len; i++) {
			fsInfo[i].invalid	= input.getLong();
			fsInfo[i].fs_type	= input.getString();

			fsInfo[i].fs_logical_device		= input.getString();
			fsInfo[i].fs_physical_device	= input.getString();
			
			fsInfo[i].total_size	= NdmpCons.toLong(input.getLong(), 
										input.getLong());
			fsInfo[i].used_size		= NdmpCons.toLong(input.getLong(), 
										input.getLong());
			fsInfo[i].avail_size	= NdmpCons.toLong(input.getLong(), 
										input.getLong());
			fsInfo[i].total_inode	= NdmpCons.toLong(input.getLong(), 
										input.getLong());
			fsInfo[i].used_inode	= NdmpCons.toLong(input.getLong(), 
										input.getLong());
										
			fsInfo[i].fs_env	= new Properties ();

			int j = input.getLong();
			while (j-- > 0)
				fsInfo[i].fs_env.put(input.getString(), input.getString());
			fsInfo[i].fs_status = input.getString();
		}
	}

	public void write (NdmpOutput output) throws IOException {
		// Dummy, needed by server only.
	}

	public void writeLog (NdmpDebugLog debugLog) {
		String	debugstring = "";
		int		fsinfocount = (null == fsInfo)?0:fsInfo.length;
		
		debugstring +=  ("There are " + fsinfocount + " enteries for FS Info."
						+ "\r\n"); 
		// This may be odd but is faster and easier to read from log file.
		// Now the complete message per session will be dumped.
		// Plus we make only one function call.
		for (int i = 0; i < fsinfocount; i++) {
			debugstring += "FS Info ["+i+"] :" +
			" Invalid : " + fsInfo[i].invalid + "\r\n" +
			"\t fs_type : " + fsInfo[i].fs_type + "\r\n" +
			"\t fs_logical_device : " + fsInfo[i].fs_logical_device + "\r\n" +
			"\t fs_physical_device : "+ fsInfo[i].fs_physical_device +"\r\n" +
			"\t total_size : " + fsInfo[i].total_size + "\r\n" +
			"\t used_size : " + fsInfo[i].used_size + "\r\n" +
			"\t avail_size : " + fsInfo[i].avail_size + "\r\n" +
			"\t total_inode : " + fsInfo[i].total_inode + "\r\n" +
			"\t used_inode : " + fsInfo[i].used_inode + "\r\n" +
			"\t fs_env : " + fsInfo[i].fs_env.toString() + "\r\n" +
			"\t fs_status : " + fsInfo[i].fs_status + "\r\n" ;
		}
		debugLog.println (50, debugstring);
		super.writeLog (debugLog);
	}
}
