package com.netapp.offtap.ndmpcopy.messages.connect;

//import netapp.cmds.ndmpc.*;
import java.io.*;
import java.util.*;

import com.netapp.offtap.ndmpcopy.debuglog.*;
import com.netapp.offtap.ndmpcopy.messages.*;
import com.netapp.offtap.ndmpcopy.streams.*;
import com.netapp.offtap.ndmpcopy.util.*;

/**
 * This message is used to authenticate the NDMP client to the NDMP server
 * Only the request messages within the CONFIG and CONNECT interfaces
 * may be processed on a connection that has not yet been authenticated
 * <p>
 * <pre>
 * Message structure:
 *  ndmp_auth_type (int)    type
 *  Three possible cases
 *  NDMP_AUTH_NONE - none
 *  NDMP_AUTH_TEXT -
 *          string  user
 *          string  password
 *  NDMP_AUTH_MD5 -
 *          string  user
 *          opaque  auth_digest
 * </pre>
 */
public class NdmpMessageConnectClientAuthRequest extends NdmpMessageData {

	protected NdmpAuthData authData;

	public NdmpMessageConnectClientAuthRequest () {
		authData = null; 		//Invalid auth Data.
	}

	public NdmpMessageConnectClientAuthRequest (NdmpAuthData authtData) {
		this.authData = authtData;
	}

	public void read (NdmpInput input) throws IOException {
		authData = new NdmpAuthData ();
		
		authData.authtype = input.getLong ();
		switch (authData.authtype) {
			case NdmpCons.NDMP_AUTH_NONE:
			default:
					break ;
			case NdmpCons.NDMP_AUTH_TEXT:
					authData.auth_id		= input.getString();
					authData.auth_passwd	= input.getString();
					break;
			case NdmpCons.NDMP_AUTH_MD5:
					authData.auth_id        = input.getString();
					authData.auth_digest	= input.getBytes (
											authData.auth_digest.length);
					break;
		}
	}
	
	public void write (NdmpOutput output) throws IOException {
		if (null == authData)
			return;
		
		output.putLong (authData.authtype);
		switch (authData.authtype) {
			case NdmpCons.NDMP_AUTH_TEXT:
				output.putString (authData.auth_id);
				output.putString (authData.auth_passwd);
				break;
		
			case NdmpCons.NDMP_AUTH_MD5:
				output.putString (authData.auth_id);
				output.putFixedLenBytes (authData.auth_digest,
							authData.auth_digest.length);
				break;
		}
	}

	public void writeLog (NdmpDebugLog debugLog) {
		if (null == authData)
			return;

		String	debugstring = "Auth Type: " + authData.authtype + "\r\n";
		switch (authData.authtype) {
			case NdmpCons.NDMP_AUTH_TEXT:
				debugstring += "\t Auth ID : " + authData.auth_id +
					"\t Passwd : " + authData.auth_passwd;
			break;
			
			case NdmpCons.NDMP_AUTH_MD5:
				debugstring += "\t Auth ID : " + authData.auth_id + "\r\n" +
					"\t auth_digest : ";
				for (int i = 0; i < authData.auth_digest.length; i++)
					debugstring += " " + authData.auth_digest[i];
				break;
		}
		debugLog.println (50, debugstring);
	}

}
