package com.netapp.offtap.ndmpcopy.messages.data;

//import netapp.cmds.ndmpc.*;
import java.io.IOException;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLog;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageDataReply;
import com.netapp.offtap.ndmpcopy.streams.NdmpInput;
import com.netapp.offtap.ndmpcopy.streams.NdmpOutput;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;

/**
 * Puts the data server into the listen state and prepares it for being
 * connected to by the another data/mover server.  This is one of the
 * new messages in V3.
 */
public class NdmpMessageDataListenRReply extends NdmpMessageDataReply {

	public int	dataConnectionType;
	public int	tcpIpAddr;
	
	public short	tcpPort;
	
	/**
	 * Basic constructor
	 */
	public NdmpMessageDataListenRReply () {
	}

	public NdmpMessageDataListenRReply (int dataConnectionType, 
					int tcpIpAddr) {
		this.dataConnectionType = dataConnectionType;
		this.tcpIpAddr	= tcpIpAddr;
	}

	public void read (NdmpInput input) throws IOException {
		super.read (input);

		dataConnectionType = input.getLong();
		switch (dataConnectionType) {
			default:
			case NdmpCons.NDMP_ADDR_LOCAL:
				break;
			case NdmpCons.NDMP_ADDR_TCP:
				tcpIpAddr	= input.getLong();
				tcpPort		= input.getShort();
				break;
			case NdmpCons.NDMP_ADDR_FC:
				input.getLong();	// we dont support these
				break;
			case NdmpCons.NDMP_ADDR_IPC:
				input.getBytes();
				break;
		}
	}

	public void write (NdmpOutput output) throws IOException {
	}

	public void writeLog (NdmpDebugLog debugLog) {
		debugLog.println (50, "Data Listen Reply Connection Type: "+
			dataConnectionType);
		switch (dataConnectionType) {
			case NdmpCons.NDMP_ADDR_TCP:
				debugLog.println (50, "Data Listen, Tcp-ip Addr "+
					tcpIpAddr + "\t Tcp-ipPort " + tcpPort);
				break;
		}
		super.writeLog (debugLog);
	}
}
