package com.netapp.offtap.ndmpcopy.messages.data.v4;

//import netapp.cmds.ndmpc.*;
import java.io.*;
import java.util.*;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLog;
import com.netapp.offtap.ndmpcopy.messages.*;
import com.netapp.offtap.ndmpcopy.streams.*;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;

public class NdmpMessageDataConnectRequestV4 extends 
							NdmpMessageData{


	protected	NdmpTcpAddress[]	tcp_addr;
	protected	byte[]		comm_data;		// for ipc
	private		int			dataConnectionType;
	
	public NdmpMessageDataConnectRequestV4 () {
	}
	
	 public NdmpMessageDataConnectRequestV4 (int dataConnectionType,
                                    int ipAddr, short port) {
        this.dataConnectionType = dataConnectionType;
		this.tcp_addr = new NdmpTcpAddress [1];
		this.tcp_addr [0] = new NdmpTcpAddress (ipAddr, port);
    }
	
	public NdmpMessageDataConnectRequestV4 (int dataConnectionType,
				NdmpTcpAddress[] tcp_addr) {
		this.dataConnectionType = dataConnectionType;
		this.tcp_addr = tcp_addr;
		this.comm_data = null; // since we dont use it.
	}

	public void write (NdmpOutput output) throws IOException {
		output.putLong (dataConnectionType);
		switch (dataConnectionType) {
			case NdmpCons.NDMP_ADDR_TCP:
				output.putLong (tcp_addr.length);
				for (int i = 0; i < tcp_addr.length; i++) {
					output.putLong (tcp_addr [i].ip_addr);
					output.putShort (tcp_addr [i].port);
					int	envEntries = 0;
					if (null != tcp_addr [i].addr_env)
						envEntries = tcp_addr [i].addr_env.size ();
					output.putLong (envEntries);
					if (0 == envEntries)
						continue;
					Enumeration keyList = tcp_addr [i].addr_env.keys ();
					Properties addr_env = tcp_addr [i].addr_env;
					while (keyList.hasMoreElements ()) {
						String key = (String) keyList.nextElement ();
						output.putString (key);
						output.putString ((String) addr_env.getProperty (key));
					}
				}
				break;
			default:
				throw new IOException ("Invalid addr type");
		}				
	}
	public void read (NdmpInput input) throws IOException {
		// Dummy only needed for server.	
	}

	public void writeLog (NdmpDebugLog debugLog) {
		String	debugString = new String ("Data Connection Type : " 
			+ dataConnectionType +
			"\n\r\t\tThere are " + tcp_addr.length +
			" enteries for tcp_addr ");
		for (int i = 0; i < tcp_addr.length; i++) {
			int addr = tcp_addr [i].ip_addr;
	        char  ar [] = new char [4];
	        ar [0] =  (char) (addr & 0x000000ff);
	        ar [1] =  (char)((addr & 0x0000ff00) >>8);
	        ar [2] =  (char )((addr & 0x00ff0000) >> 16);
	        ar [3] =  (char )((addr & 0xff000000) >> 24);

			debugString += ("\n\r\t\t Ip Addr = " + tcp_addr [i].ip_addr +
				" :  " + (int) ar [0] + "." + (int) ar [1] + "." + 
				(int) ar [2] + "." + (int) ar [3] +
				" \t Ip Port #" + tcp_addr [i].port + "\n\r" +
				"\t\t Env Vars: " + 
				((tcp_addr [i].addr_env == 
					null)? "" : tcp_addr [i].addr_env.toString ()));
		}		
		debugLog.println (50, debugString);	
	}		
}
