package com.netapp.offtap.ndmpcopy.messages.data.v4;

//import netapp.cmds.ndmpc.*;
import java.io.IOException;
import java.util.Properties;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLog;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageDataReply;
import com.netapp.offtap.ndmpcopy.streams.NdmpInput;
import com.netapp.offtap.ndmpcopy.streams.NdmpOutput;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;

public class NdmpMessageDataListenRReplyV4 extends 
			NdmpMessageDataReply {

	public	NdmpTcpAddress[]	connect_addr; 
	public	int	dataConnectionType;
	
	/**
	 * Basic constructor
	 */
	public NdmpMessageDataListenRReplyV4 () {
	}

	public NdmpMessageDataListenRReplyV4 (int dataConnectionType, 
					NdmpTcpAddress[] connect_addr) {
		this.dataConnectionType = dataConnectionType;
		this.connect_addr = connect_addr;
	}
	
	public NdmpMessageDataListenRReplyV4 (int dataConnectionType,
                                    int ipAddr, short port) {
        this.dataConnectionType = dataConnectionType;
        this.connect_addr = new NdmpTcpAddress [1];
        this.connect_addr [0] = new NdmpTcpAddress (ipAddr, port);
    }

	public void read (NdmpInput input) throws IOException {
		super.read (input);

		dataConnectionType = input.getLong();
		switch (dataConnectionType) {
			default:
				throw new IOException ("Invalid addr type used.");
				
			case NdmpCons.NDMP_ADDR_LOCAL:
				break;
			case NdmpCons.NDMP_ADDR_TCP:
				int	tcpEntries = input.getLong ();
				connect_addr = new NdmpTcpAddress [tcpEntries];
				
				for (int i = 0; i < tcpEntries; i++) {
					connect_addr [i] = new NdmpTcpAddress ();
					connect_addr [i].ip_addr = input.getLong ();
					connect_addr [i].port = input.getShort ();
					int	envEntries = input.getLong ();
					if (0 == envEntries)
						continue;
					connect_addr [i].addr_env = new Properties ();
					Properties envVar = connect_addr [i].addr_env;
					for (int j = 0; j < envEntries; j++) 
						envVar.put ((String)input.getString (),
								(String)input.getString ());
				}
				break;
		}		

	}

	public void write (NdmpOutput output) throws IOException {
		// needed for serever only..
	}

	public void writeLog (NdmpDebugLog debugLog) {
		String  debugString  = new String ("Data Connection Type : " 
			+ dataConnectionType +
        	"\n\r\t\tThere are " + connect_addr.length +
            " enteries for connect_addr ");
        for (int i = 0; i < connect_addr.length; i++) {
			int addr = connect_addr [i].ip_addr;
            char  ar [] = new char [4];
            ar [0] =  (char) (addr & 0x000000ff);
            ar [1] =  (char)((addr & 0x0000ff00) >>8);
            ar [2] =  (char )((addr & 0x00ff0000) >> 16);
            ar [3] =  (char )((addr & 0xff000000) >> 24);

            debugString += ("\n\r\t\t Ip Addr = " + connect_addr [i].ip_addr +
				" :  " + (int) ar [0] + "." + (int) ar [1] + "." + (int) ar [2] + "." + (int) ar [3] +
                " \t Ip Port #" + connect_addr [i].port + "\n\r" +
                "\t\t Env Vars: " +
				((connect_addr [i].addr_env == 
					null)? "" : connect_addr [i].addr_env.toString ()));
        }
        debugLog.println (50, debugString);
	
		super.writeLog (debugLog);
	}
}
