
package com.netapp.offtap.ndmpcopy.ndmpcopy;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Properties;

import com.netapp.offtap.ndmpcopy.messagereader.NdmpMessageQueueOut;
import com.netapp.offtap.ndmpcopy.messagereader.QueueNode;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessage;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageDataReply;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageFactory;
import com.netapp.offtap.ndmpcopy.messages.config.NdmpMessageConfigGetAuthAttrRReply;
import com.netapp.offtap.ndmpcopy.messages.config.NdmpMessageConfigGetBuTypeInfoRReply;
import com.netapp.offtap.ndmpcopy.messages.config.NdmpMessageConfigGetHostInfoRReply;
import com.netapp.offtap.ndmpcopy.messages.connect.NdmpAuthData;
import com.netapp.offtap.ndmpcopy.messages.connect.NdmpInvalidAuthException;
import com.netapp.offtap.ndmpcopy.messages.data.NdmpFileInfo;
import com.netapp.offtap.ndmpcopy.messages.data.NdmpMessageDataListenRReply;
import com.netapp.offtap.ndmpcopy.messages.log.NdmpMessageLogMessageRequest;
import com.netapp.offtap.ndmpcopy.messages.notify.NdmpMessageNotifyConnectedRequest;
import com.netapp.offtap.ndmpcopy.messages.notify.NdmpMessageNotifyDataHaltedRequest;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdArgs;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpFilerInfo;
import com.netapp.offtap.ndmpcopy.util.MD5;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;
import com.netapp.offtap.ndmpcopy.util.NdmpFatalException;
import com.netapp.offtap.ndmpcopy.util.NdmpTimeOutException;





public class NdmpCopyOperation {
	private boolean alreadyshutdown = false;
	protected	NdmpCopyData		copyData;
	
	protected int	threadState;	// 0 = ideal, 1 = setting connections
					// 2 = waiting for NdmpCopyOperation ..
	protected	final	static	int	InitState	= 0;
	protected	final	static	int	RunState	= 1;
	protected	final	static	int	WaitState	= 2;

	public		final	static	int		BaseCopyVersion = 3;

	public static NdmpCopyOperation	createCopyOp (NdmpCopyData copyData) 
			throws NdmpFatalException {

		String	classname = "NdmpCopyOperation";
		String	packagename = NdmpCopyOperation.class.getPackage().getName() + "."; 
		Class  copyOp = null;

		for (int i = copyData.version; (i > BaseCopyVersion) && 
				(null == copyOp); --i) {
			try {
				copyOp = Class.forName (packagename +
					"v" + i + "." + classname + "V" + i);
			} catch (ClassNotFoundException e) {
				copyOp = null;
			}
		}

		if (null == copyOp) {
			// Use the default
			try {
				copyOp = Class.forName (packagename + classname); 
			} catch (ClassNotFoundException e) {
				copyData.debugLog.println (20, e,"Class NdmpCopyOperation not found");
				throw new NdmpFatalException("Initialization failed");
			}		
		}

		Constructor copyOpConst;
		NdmpCopyOperation newCopyOp;

		Class[] argsType = { NdmpCopyData.class};
		Object[] args = { copyData};

		try {
			copyOpConst = copyOp.getConstructor(argsType);
		} catch (NoSuchMethodException e) {
			copyData.debugLog.println (20, e,
				"Failed to get constructor for copyOp");
			throw new NdmpFatalException("Initialization failed");
		}

		if (null == copyOpConst) {
			copyData.debugLog.println (20, 
				"Got null constructor for copyOp");
			throw new NdmpFatalException("Initialization failed");
		}

		try {
			newCopyOp = (NdmpCopyOperation) copyOpConst.newInstance (args);
		
		} catch (InstantiationException e) {
			copyData.debugLog.println (20, 
				"Failed to create instance for copyOp");
			throw new NdmpFatalException("Initialization failed");
		
		} catch (IllegalAccessException e) {
			copyData.debugLog.println (20, 
				"Failed to create instance for copyOp");
			throw new NdmpFatalException("Initialization failed");
		
		} catch (IllegalArgumentException e) {
			copyData.debugLog.println (20, 
				"Failed to create instance for copyOp");
			throw new NdmpFatalException("Initialization failed");
		
		} catch (InvocationTargetException e) {
			copyData.debugLog.println (20, 
				"Failed to create instance for copyOp");
			throw new NdmpFatalException("Initialization failed");
		}

		return newCopyOp;
	}
	
	public NdmpCopyOperation (NdmpCopyData	copyData) {
		this.copyData	= copyData;
		threadState = InitState;
	}

	
	public void rocknRoll () throws NdmpFatalException {
		
		/**
		 * This is V3
		 * (I) Load the Guns
		 * 1> Authenticate
		 * 2> Make sure we'r talkn to Netapp 
		 * 3> Set up the dump name
		 * 4> Set up the dump dates
		 * 5> Set up the EXCLUDE (system files)
		 *
		 * (II) Aim
		 * This will usually be dependent on the target, NDMP version
		 * a> Set up the data connection
		 * z> start the message reader threads ;; alwayz in the end
		 *
		 * (III) Shoot
		 * i> start the dump
		 * ii> start the restore
		 */

		//(I)
		setupCopyData ();

		//(II)
		setupCopyConnection ();
		//(III)
		startCopyOperation ();

		//we'r done.
	}

	public void setupCopyData () throws NdmpFatalException {

		// starting the message readers
		copyData.filerData [NdmpCopyCmdArgs.srcFiler].msgReader.start ();
		copyData.filerData [NdmpCopyCmdArgs.dstFiler].msgReader.start ();
		
		//<1>
		authenticate (NdmpCopyCmdArgs.srcFiler);
		copyData.out.println ("Ndmpcopy: " + 
			copyData.cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].filerName +
			": " + "Connect: Authentication successful");
		authenticate (NdmpCopyCmdArgs.dstFiler);
	 	copyData.out.println ("Ndmpcopy: " +
			copyData.cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].filerName +
			": " + "Connect: Authentication successful");
		
		//<2>
		checkFilerType (NdmpCopyCmdArgs.srcFiler);
		checkFilerType (NdmpCopyCmdArgs.dstFiler);

		//<3>
		addDumpName ();

		//<4>
//		if (envSupported ("BASE_DATE", NdmpCopyCmdArgs.srcFiler))
//			addDumpDates ();
		if (true == copyData.cmdArgs.incrForever) {
			copyData.cmdArgs.envVar.put ("REPLICATE", "Y");
                }

		//<5>
		addExcludeFiles ();

		//<6>
		//Have a drink!
	}

	public void setupCopyConnection () throws NdmpFatalException {
	
		//<a>
		NdmpMessageFactory msgFact = 
					copyData.filerData [NdmpCopyCmdArgs.dstFiler].msgFactory;

		Class[]	argT = {int.class};
		Object[] arg = {new Integer (NdmpCons.NDMP_ADDR_TCP)};
		
		NdmpMessage msg = msgFact.createRequestMessage (
					NdmpCons.NDMP_DATA_LISTEN, argT, arg);
		
		if (null == msg) {
			copyData.debugLog.println (20, "Request message " +
				"creation failed for DataListen");
			abort("Connection setup for transfer failed");
		}
				
		NdmpMessage rmsg = sendMessage (NdmpCopyCmdArgs.dstFiler, msg, 
			NdmpCons.NDMP_DATA_LISTEN);
		if(null == rmsg)
			abort("Connection setup for transfer failed");

		NdmpMessageDataListenRReply	msgData = null;
		try {
			msgData = (NdmpMessageDataListenRReply) rmsg.getMsgData ();
		} catch (ClassCastException e) {
			copyData.debugLog.println(20, e, "Unexpected reply in " +
				" response to " + msg.msgClassName() + " from " +
				"destination");
			abort(e, "Connection setup for transfer failed");
		}		

		Class[]	argtT = {int.class, int.class, short.class};
		Object[] argt = {new Integer (msgData.dataConnectionType), 
						new Integer (msgData.tcpIpAddr),
						new Short (msgData.tcpPort)};
	
		msgFact = copyData.filerData [NdmpCopyCmdArgs.srcFiler].msgFactory;
		
		// data connect message
		msg = msgFact.createRequestMessage (NdmpCons.NDMP_DATA_CONNECT, argtT,
									argt);
		if (null == msg) {
			copyData.debugLog.println (20, "Request message " +
				"creation failed for DataConnect");
			abort("Connection setup for transfer failed");
		}
		
		rmsg = null;
		rmsg = sendMessage (NdmpCopyCmdArgs.srcFiler, msg, 
						NdmpCons.NDMP_DATA_CONNECT);
		if(null == rmsg)
			abort("Connection setup for transfer failed");
	}

	public void startCopyOperation () throws NdmpFatalException {
	
		//<i>
		startDump (NdmpCopyCmdArgs.srcFiler);
		
		//<ii>
		startRestore (NdmpCopyCmdArgs.dstFiler);
		
		processMessages ();
	}	

	public void authenticate (int filer) throws NdmpFatalException {
		/**
		 * If its md5, get the auth attr type then make the
		 * auth data. send the auth request.
		 */
		NdmpAuthData authData = null;
		NdmpFilerInfo filerInfo = copyData.cmdArgs.filerInfo [filer];
		NdmpMessage msg = null;
		NdmpMessageFactory msgFact = copyData.filerData [filer].msgFactory;

		if (null == filerInfo || null == msgFact) {
			copyData.debugLog.println (20, "Null info for " +
				NdmpCopyCmdArgs.filer (filer)); 
			abort("Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}

		try {
		 	authData = new NdmpAuthData (filerInfo.authType, filerInfo.usrName,
								filerInfo.usrPasswd);
		} catch (NdmpInvalidAuthException e) {
			copyData.debugLog.println (20, e,
				"Creation of auth data failed for " + NdmpCopyCmdArgs.filer (filer)); 
			abort("Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}
	
		doMd5authentication (filer, authData);

		Class[] argT = {NdmpAuthData.class};
		Object[] arg = {authData};

		msg = msgFact.createRequestMessage (
			NdmpCons.NDMP_CONNECT_CLIENT_AUTH, argT, arg);

		if (null == msg) {
			copyData.debugLog.println(20, "Failed to create " +
				"CONNECT_CLIENT_AUTH message");
			abort("Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}
				
		msg = sendMessage (filer, msg, NdmpCons.NDMP_CONNECT_CLIENT_AUTH);
		if(null == msg)
			abort("Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 

		return ;
	}

	public void doMd5authentication (int filer, NdmpAuthData authData) 
			throws NdmpFatalException {

		NdmpFilerInfo filerInfo = copyData.cmdArgs.filerInfo [filer];
		NdmpMessage msg = null;
		NdmpMessageFactory msgFact = copyData.filerData [filer].msgFactory;
		
		if (NdmpCons.NDMP_AUTH_MD5 != authData.authtype) {
			copyData.debugLog.println(50, "Using text authentication for " +
				NdmpCopyCmdArgs.filer(filer));
			return ;
		}
			
		copyData.debugLog.println(50, "Using md5 authentication for " +
			NdmpCopyCmdArgs.filer(filer));
			
		// Get the auth attr type
		Class[] argT = { int.class };
		Object[] arg = { new Integer (authData.authtype) };
		
		msg = msgFact.createRequestMessage (
			NdmpCons.NDMP_CONFIG_GET_AUTH_TYPE_ATTR, argT, arg);
		
		if (null == msg) {
			copyData.debugLog.println(20, "Failed to create " +
				"GET_AUTH_TYPE_ATTR message");
			abort("Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}
		
		msg = sendMessage (filer, msg, NdmpCons.NDMP_CONFIG_GET_AUTH_TYPE_ATTR);
		if(null == msg)
			abort("Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 

		NdmpMessageConfigGetAuthAttrRReply msgData = null;
		byte challenge[];
		
		try {
			msgData = (NdmpMessageConfigGetAuthAttrRReply)
							msg.getMsgData ();
		} catch (ClassCastException e) {
			copyData.debugLog.println(20, e, "Unexpected reply in " +
				" response to " + msg.msgClassName() + " from " +
				NdmpCopyCmdArgs.filer(filer));
			abort(e, "Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}

		if(null == msgData) {
			copyData.debugLog.println(20, "Null data in " +
				"GET_AUTH_TYPE_ATTR reply message");
			abort("Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}
	
		challenge = msgData.challenge;
		if (null == challenge) {
			copyData.debugLog.println(20, "Null challenge from " +
				NdmpCopyCmdArgs.filer (filer));
			abort("Authentication failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}
				
		// The ndmp v4 spec describes how to construct the md5
		// digest under the description for NDMP_CONNECT_CLIENT_AUTH.
		byte buf[] = new byte [128];
		byte pwd[] = authData.auth_passwd.getBytes();
		
		for (int i = 0; i < buf.length; i++)
			buf[i] = 0;

		// The ndmp spec requires that passwords greater than 32 bytes
		// in length be truncated. i.e the password can be at most 32
		// bytes.
		int pwd_len = 0;
		if (pwd.length > 32) {
			pwd_len = 32;
		} else {
			pwd_len = pwd.length;
		}

		System.arraycopy(pwd, 0, buf, 0, pwd_len);
		System.arraycopy(challenge, 0, buf,
			buf.length - pwd_len - challenge.length, challenge.length);
		System.arraycopy(pwd, 0, buf,
			buf.length - pwd_len, pwd_len);
		
		MD5 md5 = new MD5 ();

		authData.auth_digest = md5.digest (buf, 0, buf.length);
	}
	
	public void checkFilerType (int filer) throws NdmpFatalException {
		if (!copyData.cmdArgs.filerInfo [filer].authNeeded)
			return ;
		/**
		 * Send a CONFIG_GET_HOST_INFO message and see if it return 
		 * NetApp or not..
		 */
		NdmpMessage msg = sendMessage (filer, 
					NdmpCons.NDMP_CONFIG_GET_HOST_INFO);		
		if(null == msg)
			abort("Initialization failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		
		NdmpMessageConfigGetHostInfoRReply msgData = null;

		try {
			msgData = (NdmpMessageConfigGetHostInfoRReply) msg.getMsgData ();
		} catch (ClassCastException e) {
			copyData.debugLog.println(20, e, "Unexpected reply in " +
				" response to " + msg.msgClassName() + " from " +
				NdmpCopyCmdArgs.filer(filer));
			abort(e, "Initialization failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}
		
                if ( !( msgData.ostype.startsWith(NdmpCons.OS_TYPE_NETAPP) &&
                        msgData.osvers.startsWith(NdmpCons.OS_VERS_NETAPP) ) &&
                     !( msgData.ostype.startsWith(NdmpCons.OS_TYPE_IBM) &&
                        msgData.osvers.startsWith(NdmpCons.OS_VERS_IBM) ) )
			abort(NdmpCopyCmdArgs.filer (filer) + " is not a " +
				"netapp host");
	}

	public boolean envSupported (String name, int filer) 
			throws NdmpFatalException {

		// send the configgetbutypeinfo request
		NdmpMessage msg = sendMessage (filer, 
					NdmpCons.NDMP_CONFIG_GET_BUTYPE_INFO);		
		if(null == msg)
			abort("Initialization failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		
		NdmpMessageConfigGetBuTypeInfoRReply msgData = null;

		try {
			msgData = (NdmpMessageConfigGetBuTypeInfoRReply) msg.getMsgData ();
		} catch (ClassCastException e) {
			copyData.debugLog.println(20, e, "Unexpected reply in " +
				" response to " + msg.msgClassName() + " from " +
				NdmpCopyCmdArgs.filer(filer));
			abort(e, "Initialization failed for " +
				NdmpCopyCmdArgs.filer(filer)); 
		}

		if (msgData.checkEnv (name)) {
			copyData.debugLog.println(50, "Env variable " +
				name + " supported on " + 
				NdmpCopyCmdArgs.filer(filer));
			return true;
		}
		
		copyData.debugLog.println(50, "Env variable " +
			name + " NOT supported on " + 
			NdmpCopyCmdArgs.filer(filer));
		return false;
	}

	public void addDumpName () throws NdmpFatalException {
		 /**
		  * We will generate a unique name here for this ndmpcopy operation.
		  * Note: Any change to this algorithm will cause ndmpcopy to be
		  * incompatible with earlier releases.
		  *
		  * The name is based on:
		  * <source-filer>:<path> <destination-filer>:<destination-path>
		  */
		
		NdmpFilerInfo filerInfo = 
			copyData.cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler];
		
		String  tname = new String (filerInfo.pathName);
		if (!tname.endsWith ("/"))
			tname += "/";
			
		if (tname.length() > NdmpCons.MAX_PATH_SIZE)
			tname = tname.substring (0, NdmpCons.MAX_PATH_SIZE);
			
		String	tname1 = new String (filerInfo.filerIP);
		if (tname1.length() > NdmpCons.MAX_NAME_SIZE)
			tname1 = tname1.substring (0, NdmpCons.MAX_NAME_SIZE);

		filerInfo = copyData.cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler];

		tname += filerInfo.pathName;
		/*
		 * Fix for burt107302. The '.snapshot' text needs to be parsed
		 * out of the DMP_NAME since the DMP_NAME is where the bitmap
		 * files will be created and we cannot create a directory named
		 * .snapshot anywhere on the filesystem.
		 *
		 * Fix for burt233537. Along with the '.snapshot' text, the name
		 * of the snapshot will also have to be parsed out 
		 *
		 *
		 */
		int idx;
		if (-1 != (idx = tname.indexOf("/.snapshot"))) {
			int idx2 = tname.indexOf('/', idx+1);

			if (-1 != idx2) {
				/* Now parse out the snapshot name */
				idx2 = tname.indexOf('/', idx2+1);
				if(-1 != idx2)
				      tname = tname.substring(0, idx) + tname.substring(idx2);
				else
				      tname = tname.substring(0, idx); 
			}
		}
		if (!tname.endsWith ("/"))
		            tname += "/";
		if (tname.length () > (2 * NdmpCons.MAX_PATH_SIZE))
			tname = tname.substring (0, (2 * NdmpCons.MAX_PATH_SIZE));
			
		String dname = tname1 + filerInfo.filerIP;
		if (dname.length ()  > (2 * NdmpCons.MAX_NAME_SIZE))
			dname = dname.substring (0, (2 * (NdmpCons.MAX_NAME_SIZE)));
		tname += dname;

		/**
		 * For DMP_NAME we need to send an ascii human-readable string
		 * Details in burt 61590
		 */
		String dmpName = NdmpCons.DMP_NAME_PREFIX + tname;
		copyData.cmdArgs.envVar.put ("DMP_NAME", dmpName);
		copyData.cmdArgs.envVar.put ("UPDATE", "Y");

		/**
		 * For dump dates file
		 * we will store the #s only, md5 hash and length of the tname.
		 */
		byte[] buf = tname.getBytes ();
		MD5 md5 = new MD5 ();
		
		// md5 digest method returns a 16 byte digest
		byte[] digest = md5.digest (buf, 0, buf.length);
		copyData.dumpName = digest;
		// log this
		copyData.debugLog.println(50, "Hash key: " + tname);
		copyData.debugLog.println(50, "Added env var DMP_NAME");
		copyData.debugLog.println(50, "Added env var UPDATE");

	}

//	public void addDumpDates () throws NdmpFatalException {
//
//		if (0 == copyData.cmdArgs.level && !copyData.cmdArgs.incrForever) {
//			copyData.cmdArgs.envVar.put ("BASE_DATE", "0");
//			copyData.debugLog.println(50, "Added env var BASE_DATE");
//			return ;
//		}
//
//		long date = 0;
//		byte [] hash = copyData.dumpName;
//		NdmpMetaDDateRecord  arec = new NdmpMetaDDateRecord (-1, true, 
//					hash, copyData.cmdArgs.level, date, 
//					copyData.dumpName.length, null,copyData.cmdArgs.incrForever);
//	
//		NdmpMetaDDateRecord arec1 = 
//			(NdmpMetaDDateRecord) NdmpMetaFileFactory.findRecord (1, arec);
//		
//		if (null == arec1) { // no recs!
//			copyData.debugLog.println(20, "No Level 0 entry found for this" +
//				" dump, forcing a level 0 dump");
//			copyData.cmdArgs.envVar.put ("BASE_DATE", "0");
//			copyData.cmdArgs.envVar.remove ("LEVEL");
//			copyData.cmdArgs.envVar.put ("LEVEL", "0");
//			copyData.cmdArgs.level = 0;
//			
//			copyData.debugLog.println(50, "Added env var BASE_DATE");
//			copyData.debugLog.println(50, "Added env var LEVEL");
//			return;
//		}
//
//		/* 
//		 * This can happen when a level 1 or level 2 ndmpcopy
//		 * has already been run and now an incremental copy
//		 * is being attempted or vice versa. We do not currently 
//		 * allow incr-forevers to be mixed with lev 1 or lev 2 copies
//		*/
//		if (!arec1.validFlag) {
//			abort("The option '-i' is mutually exclusive with options '-l 1' or '-l 2'");
//		}
//
//		long dateLong = arec1.dumpDate;
//		dateLong >>= 32;
//
//		if (dateLong >= 3 && !copyData.cmdArgs.incrForever) {
//			abort("Cannot go higher than level 2 incremental");
//		}
//
//		String date1 = Long.toString (arec1.dumpDate);
//		copyData.cmdArgs.envVar.put ("BASE_DATE", date1); 
//		copyData.debugLog.println(50, "Added env var BASE_DATE");
//
//		return ;
//	}

	public void addExcludeFiles () throws NdmpFatalException {
		//Dummy function
	}	

	/*
	 * Create a message for filer filer with msg ID, msgID and
	 * send it to the serevr (filer) then read the response
	 * and check for errors if ok send the message back.
	 */
	public NdmpMessage sendMessage (int filer,  int msgID) 
					throws NdmpFatalException {
		NdmpMessage msg = createMessage (filer, msgID);
		return sendMessage (filer, msg, msgID);
	}

	/*
	 * This method only checks if a notify shutdown message
	 * from the filer has already been received. It does not
	 * wait if there are no messages in the queue.
	 */
	public boolean rcvNotifyShutdown(int filer) {
		NdmpMessage msg = copyData.msgQueue.getNoSleep (filer);
		
		if (null != msg) {
			msg.writeLog(copyData.debugLog);
			if (notifyShutdown(msg)) {
				return true;
			}
			
			// Not a notify connected with shutdown
			// We should not have received this message...
			copyData.debugLog.println(20, "No message expected, but received " +
				msg.msgClassName() + " from " + NdmpCopyCmdArgs.filer (filer));
			return false;
		}

		return false;
	}

	/*
	 * Checks if a message is NOTIFY_CONNECTED with SHUTDOWN. Only
	 * then it returns true.
	 */
	public boolean notifyShutdown(NdmpMessage msg) {
		NdmpMessageNotifyConnectedRequest msgData = null;
		boolean retval = false;
		
		try {
			msgData = (NdmpMessageNotifyConnectedRequest)
					msg.getMsgData ();
			if (msgData.reasoncode == NdmpCons.NDMP_SHUTDOWN) {
				// shutdown confirmed 
				retval = true;
			}
		}catch (ClassCastException e) {
		}

		return retval;
	}
		
		 
	public NdmpMessage createMessage (int filer, int msgID) 
			throws NdmpFatalException {	 

		NdmpMessage msg = 
			copyData.filerData [filer].msgFactory.createRequestMessage (
				msgID, null, null);
		if (null == msg) {
			copyData.debugLog.println(20,"Failed to create message " + 
				msgID + " for " + NdmpCopyCmdArgs.filer (filer));
			abort(null);
		}
	
		return msg;
	}	

	public NdmpMessage sendMessage (int filer, NdmpMessage msg, int msgID) 
			throws NdmpFatalException {

		// Check if we already have received a notify connected from the
		// filer, in case it shutdown.
		if (rcvNotifyShutdown(filer)) {
			abort("Connection shutdown by " + NdmpCopyCmdArgs.filer(filer) +
				" " + copyData.cmdArgs.filerInfo[NdmpCopyCmdArgs.srcFiler].filerName);
		}

		try {				
			msg.write (copyData.filerData [filer].output);
		} catch (IOException e) {
			copyData.debugLog.println(20, "I/O error writing request " +
				msg.msgClassName() + " to " +
				NdmpCopyCmdArgs.filer(filer));
			abort(e, "I/O error on socket to " + 
				NdmpCopyCmdArgs.filer (filer));
		}

		msg.writeLog (copyData.debugLog);
		NdmpMessage rmsg = null;
		try {
			while ( true ) {
				rmsg = copyData.msgQueue.get (filer); //blocks

				/* BURT250387: We could get LOG_MESSAGE requests
				 * which are valid and so continue reading for
				 * messages if the current msg is a log msg
				 */
				if ( rmsg.isMethod (NdmpCons.NDMP_LOG_MESSAGE) ) {
					NdmpMessageLogMessageRequest msgD = null;
					try {
					    msgD = (NdmpMessageLogMessageRequest)rmsg.getMsgData();
					} catch (ClassCastException e) {
					    copyData.debugLog.println(50, e, "Unexpected format " +
					            " for " + msg.msgClassName() + " from " +
					            NdmpCopyCmdArgs.filer(filer));
					    msgD = null;
					}
					if (null != msgD)
				 	    println (msg, msgD.entry, filer);
				}
				else
					break;
			}
		} catch (NdmpTimeOutException e) {
			abort(e, "Timed out waiting for response from " +
				NdmpCopyCmdArgs.filer (filer));
		}
		if (null == rmsg) {
			copyData.debugLog.println(20,"Null message from " +
				NdmpCopyCmdArgs.filer (filer) + " in response to " +
				msg.msgClassName());
			abort(null);
		}

		rmsg.writeLog (copyData.debugLog);
		
		if (rmsg.isHeaderError ()) {
			NdmpMessageDataReply msgData = null;

			if (!rmsg.isMethod (msgID)) {
				
				// Check if we received a notify connected with shutdown
				if (notifyShutdown(rmsg)) {
					abort("Connection shutdown by " + NdmpCopyCmdArgs.filer(filer) +
						" " + copyData.cmdArgs.filerInfo[NdmpCopyCmdArgs.srcFiler].
						filerName);
				} 
				
				// Otherwise, we have an unexpected message...
				copyData.debugLog.println(20, "Received message " + rmsg.msgClassName() + 
					" instead of " +msg.msgClassName() +
					" from " + NdmpCopyCmdArgs.filer (filer));
				abort(null);
			}	

			try {
				msgData = (NdmpMessageDataReply) rmsg.getMsgData ();
			} catch (ClassCastException e) {
				copyData.debugLog.println(20, e, "Unexpected reply in " +
					" response to " + msg.msgClassName() + " from " +
					NdmpCopyCmdArgs.filer(filer));
				abort(null);
			}		
			
			// By not aborting here we expect the callers of sendMessage
			// to handle this situation. we send back a null message in
			// this case. 
			// XXX: Ideally the caller of send message should dig into
			// the messages for errors. Right now, this is a very detailed
			// task. So the alternative is to have the callers of sendmessage
			// check for a null reply message and throw a sensible error
			// to the user.
			if  (! msgData.isError(NdmpCons.NDMP_NO_ERR)) {
				// this is an error within the message reply
				copyData.debugLog.println(20, "Body error " +
					msgData.msgBodyError() + " in reply message " +
					rmsg.msgClassName() + " from " + 
					NdmpCopyCmdArgs.filer (filer));

				rmsg = null;
			}
		} else {
			// we handle this in the same way as above.
			copyData.debugLog.println(20, "Header error " +
				rmsg.msgError() + " in reply message " +
				rmsg.msgClassName() + " from " +
				NdmpCopyCmdArgs.filer(filer));
			rmsg = null;
		}
				
		return rmsg;
	}		
		
	/**
	 * sendMessageNoWait - dont wait for reply
	 */
	public void sendMessageNoWait (int filer, int msgID) 
			throws NdmpFatalException {
		
		NdmpMessage msg = createMessage (filer, msgID);							
		try {
			msg.write (copyData.filerData [filer].output);
		} catch (IOException e) {
			abort(e, "I/O error on socket to " + 
				NdmpCopyCmdArgs.filer (filer));
		}								

		msg.writeLog (copyData.debugLog);
	}

	public void startDump (int filer) throws NdmpFatalException {
		NdmpMessage msg = null, rmsg = null;

		Class[]	argT = {String.class, Properties.class};
		Object[] arg = {"dump", copyData.cmdArgs.envVar};
		
		msg = copyData.filerData [filer].msgFactory.createRequestMessage (
				NdmpCons.NDMP_DATA_START_BACKUP, argT, arg);
		
		if (null == msg) {
			copyData.debugLog.println(20, "Failed to create a " +
				"DATA_START_BACKUP message for " + 
				NdmpCopyCmdArgs.filer (filer));
			abort(null);
		}
		
		rmsg = sendMessage (filer, msg, NdmpCons.NDMP_DATA_START_BACKUP);
		if(null == rmsg)
			abort("Failed to start dump on " +
				NdmpCopyCmdArgs.filer(filer));
	}

	public void startRestore (int filer) throws NdmpFatalException {
		NdmpMessage msg = null, rmsg = null;
		
		NdmpFileInfo[] finfo = new NdmpFileInfo[1];
		
		finfo[0] = new NdmpFileInfo ();
		finfo[0].originalFilename = "/";
		finfo[0].destinationPath = 
			copyData.cmdArgs.filerInfo [filer].pathName + "/";
		finfo[0].newFilename = finfo[0].otherFilename = "";
		finfo[0].node = finfo[0].fhInfo = 0;
		
		Class[]	argT = {Properties.class, NdmpFileInfo[].class, 
									String.class};
		Object[] arg = {copyData.cmdArgs.envVar, finfo, "dump"};
		
		msg = copyData.filerData [filer].msgFactory.createRequestMessage (
				NdmpCons.NDMP_DATA_START_RECOVER, argT, arg);
		
		if (null == msg) {
			copyData.debugLog.println(20, "Failed to create a " +
				"DATA_START_RECOVER message for " + 
				NdmpCopyCmdArgs.filer (filer));
			abort(null);
		}
		
		rmsg = sendMessage (filer, msg, NdmpCons.NDMP_DATA_START_RECOVER);
		if(null == rmsg)
			abort("Failed to start restore on " +
				NdmpCopyCmdArgs.filer(filer));
	}
		
	public void processMessages () throws NdmpFatalException {
		/**
		 * Now wait and watch...
		 * Keep on reading the messages we receive from both filers and
		 * quit after Notify from both has arrived...
		 */

		NdmpMessage msg = null, rmsg = null;
		NdmpMessageDataReply msgData = null;
		NdmpMessageQueueOut msgQueue = copyData.msgQueue;
		QueueNode msgQ = null;
		 
		boolean srcHalt = false, dstHalt = false;
		boolean copyFailed = false, srcAborted = false, dstAborted = false;
		int filer = -1;	
		long starttime, endtime;
		 
		starttime = System.currentTimeMillis();	 
		copyData.debugLog.println (50, "Transfer started");
		copyData.debugLog.println(50, "Now waiting for source and " +
		 		" destination to finish.");

		while (!srcHalt || !dstHalt) {
		 	if (!srcHalt && !dstHalt) {	
			 	try {
			  		msgQ = msgQueue.get ();
				} catch (NdmpTimeOutException e) {
					copyData.debugLog.println (20, e, 
						"Inactivity timeout on both control " +
						"connections");
					break;
				}
	
				if (null == msgQ) {
					copyData.debugLog.println(20,
						"Received a null message from queue");
					break;
				}

				filer = msgQ.readerID;
				msg   = msgQ.getMsg ();
			}
			else {
				if (srcHalt) 
					filer = NdmpCopyCmdArgs.dstFiler;
				else
					filer = NdmpCopyCmdArgs.srcFiler;
				try {
					msg = msgQueue.get (filer);
				} catch (NdmpTimeOutException e) {
					copyData.debugLog.println (20, e, 
					       "Inactivity time out on connection to " +
					       copyData.cmdArgs.filer(filer));
					break;
				}
			}	
			
			if (null == msg) {
					copyData.debugLog.println(20,
						"Received a null message from queue");
					break;
			}
	
			msg.writeLog (copyData.debugLog);
			if (msg.isMethod (NdmpCons.NDMP_DATA_ABORT)) {
				
				// reply for the abort we sent earlier
				NdmpMessageDataReply rMsgData = 
					(NdmpMessageDataReply) msg.getMsgData();

				if (rMsgData.errorcode != NdmpCons.NDMP_NO_ERR) 
					copyData.debugLog.println (20, "Attempt to abort " + 
						" operation unsuccessful for " + 
						NdmpCopyCmdArgs.filer(filer));
				else {
					copyData.debugLog.println (50, "Attempt to abort " + 
						" operation successful for " + 
						NdmpCopyCmdArgs.filer(filer));

					if (NdmpCopyCmdArgs.srcFiler == filer) 
						srcHalt = true;
					else	
						dstHalt = true;
				}

			} else if (msg.isMethod (NdmpCons.NDMP_NOTIFY_DATA_HALTED)) {

				// check if the data operation was successful
				NdmpMessageNotifyDataHaltedRequest rMsgData =
					(NdmpMessageNotifyDataHaltedRequest) msg.getMsgData();

				// something failed - stop the other operation
				if (rMsgData.reasoncode != NdmpCons.NDMP_DATA_HALT_SUCCESSFUL) {
					copyFailed = true;
					println (msg, NdmpCopyCmdArgs.filerType(filer) + 
						" failed", filer);
				} else 
					println (msg, NdmpCopyCmdArgs.filerType(filer) + 
						" successful", filer);

				if (NdmpCopyCmdArgs.srcFiler == filer) 
					srcHalt = true;
				else	
					dstHalt = true;

			} else if (msg.isMethod (NdmpCons.NDMP_NOTIFY_CONNECTED)) {

				// check if the data operation was successful
				NdmpMessageNotifyConnectedRequest rMsgData =
					(NdmpMessageNotifyConnectedRequest) msg.getMsgData();
				
				// something failed - stop the other operation
				if (rMsgData.reasoncode == NdmpCons.NDMP_SHUTDOWN) {
					copyFailed = true;
					println (msg, "Ndmp server shutting down " +
						"connection", filer);
					
					if (NdmpCopyCmdArgs.srcFiler == filer) 
						srcHalt = true;
					else	
						dstHalt = true;
				}

			} else if (msg.isMethod (NdmpCons.NDMP_LOG_MESSAGE)) {

				NdmpMessageLogMessageRequest msgD = null;
				try {
					msgD = (NdmpMessageLogMessageRequest)msg.getMsgData();
				} catch (ClassCastException e) {
					copyData.debugLog.println(50, e, "Unexpected format " +
						" for " + msg.msgClassName() + " from " +
						NdmpCopyCmdArgs.filer(filer));
					msgD = null;
				}
				if (null != msgD)
					println (msg, msgD.entry, filer);
 
			} else if (!(msg.isMethod (NdmpCons.NDMP_NOTIFY_DATA_READ))) {
				
				copyData.debugLog.println (50, "Unexpected message " +
					msg.msgClassName() + " from " +
					NdmpCopyCmdArgs.filer(filer));
			} 

			if (copyFailed) {
				if (srcHalt && !dstHalt && !dstAborted) {
					dstAborted = true;
					// send abort to destination filer only once
					sendMessageNoWait (NdmpCopyCmdArgs.dstFiler,
						NdmpCons.NDMP_DATA_ABORT);
				} else if (dstHalt && !srcHalt && !srcAborted) {
					srcAborted = true;
					// send abort to source filer only once
					sendMessageNoWait (NdmpCopyCmdArgs.srcFiler,
						NdmpCons.NDMP_DATA_ABORT);
				}
			}
		}
	
		if (srcHalt && dstHalt && !copyFailed) {
			endtime = System.currentTimeMillis();	 
//			if (!updateDumpDates ()) {
//				copyData.debugLog.println(20, "Dumpdate records updation failed");
//				copyData.out.println("Ndmpcopy: Incrementals based on this transfer" +
//					" will fail");
//			}
			shutdownCopyOp ();
			inactivateMessageReader(NdmpCopyCmdArgs.srcFiler);
			inactivateMessageReader(NdmpCopyCmdArgs.dstFiler);
			copyData.out.println("Ndmpcopy: Transfer successful [ " +
				timetaken (endtime-starttime) + " ]");
			copyData.debugLog.println (20, "Transfer successful");
		} else {
			shutdownCopyOp ();
			inactivateMessageReader(NdmpCopyCmdArgs.srcFiler);
			inactivateMessageReader(NdmpCopyCmdArgs.dstFiler);
			abort("Transfer failed");
		}
	}
	
	/* BURT 241035: This function moves the reader thread to an idle 
	 * state so that it does not read for messages anymore. 
	 */
	public void inactivateMessageReader (int filer) {
                try {
                    if (null != copyData.filerData [filer].msgReader)
                        copyData.filerData [filer].msgReader.idleThread ();
                } catch (NullPointerException npex) {
		}
        }


//	public boolean updateDumpDates () throws NdmpFatalException {
//
//		NdmpMessage msg = sendMessage (NdmpCopyCmdArgs.srcFiler, 
//					NdmpCons.NDMP_DATA_GET_ENV);
//		if(null == msg) {
//			copyData.debugLog.println(20, "Failed getting DataGetEnv from source");
//			return false;
//		}
//
//		NdmpMessageDataGetEnvRReply msgData = null ;
//		try {
//			msgData = (NdmpMessageDataGetEnvRReply) msg.getMsgData ();
//		}catch (ClassCastException e) {
//			copyData.debugLog.println(20, e, "Unexpected reply in " +
//				" response to " + msg.msgClassName() + " from " +
//				"source");
//			return false;
//		}
//		if (null == msgData) {
//			copyData.debugLog.println(20, "Recieved a null message from source");
//			return false;
//		}
//
//		String date = (String) msgData.getValue ("DUMP_DATE");
//		
//		if (null == date) { //support /etc/dumpdates instead for versions prior to 6.2
//			copyData.debugLog.println (20, "Source does not support DUMP_DATE " +
//				"mechanism");
//			return true;
//		}
//            
//                if ( date.length () < 1) {
//			copyData.debugLog.println (20, "Invalid DUMP_DATE from source"); 
//			return false;
//		}
//		long dateVal = 0;
//			
//		try {
//			dateVal = Long.parseLong (date);
//		} catch (ClassCastException e) {
//			copyData.debugLog.println(20, e, "Invalid DUMP_DATE from source");
//			return false;
//		}
//
//		
//		NdmpMetaDDateRecord arec = new NdmpMetaDDateRecord (0, true,
//			copyData.dumpName, copyData.cmdArgs.level,
//			dateVal, copyData.dumpName.length, null, copyData.cmdArgs.incrForever);
//		
//		/* If a '-i' tranfer has been performed without a preceding
//		 * level 0 tranfer, then we will store '-i' as a level 0.
//		 * If we don't do this, then the first level 'i' tranfer will 
//		 * be stored as 'i' and when the next level 'i' tranfer is
//		 * attempted, the call to findRecord() will return null since
//		 * the native code looks for a preceding level 0 record entry which 
//		 * would not be present. Hence the subsequent level 'i' transfer will
//		 * again default to a level 0 tranfer which would be incorrect.
//		 * To avoid this, in the absence of any level 0 transfers
//		 * we store the first level 'i' transfer as a level 0
//		 */ 
//		if ( copyData.cmdArgs.incrForever && (null == NdmpMetaFileFactory.findRecord(1, arec))) {
//			arec.incrForever = false;
//			arec.dumpLevel = 0;
//		}
//
//		if (NdmpMetaFileFactory.updateRecord(1, arec)) {
//			copyData.debugLog.println(50, "Dumpdate records updated");
//
//			/* If this is a level 0 tranfer, then reset the metafile entries
//		 	 * for levels 1, 2 and 'i' (incr-forever). This is required for BURT 233537
//		 	 * because when a dump of a snapshot is performed, the creation date
//		 	 * of the snapshot is used as the dumpDate. Thus it would not be  
//		 	 * possible to reset a ndmpcopy of a snapshot with a level 0 copy as 
//		 	 * the dumpDate for that snapshot always remains the same. 
//			 * For ex: The following sequence would fail, if we don't reset the
//			 * entries in the file
//			 *  ndmpcopy -l 0 /vol/srcvol/.snapshot/snap0 /vol/dstvol
//			 *  ndmpcopy -l 1 /vol/srcvol/.snapshot/snap0 /vol/dstvol
//			 *  ndmpcopy -l 0 /vol/srcvol/.snapshot/snap0 /vol/dstvol
//			 *  ndmpcopy -l 1 /vol/srcvol/.snapshot/snap0 /vol/dstvol <-- Fails
//		 	*/
//			if (0 == copyData.cmdArgs.level && !copyData.cmdArgs.incrForever) {
//				NdmpMetaDDateRecord  rec1 = new NdmpMetaDDateRecord (-1, true, 
//								copyData.dumpName, 1, 0, 
//								copyData.dumpName.length, null,false);
//
//				NdmpMetaDDateRecord  rec2 = new NdmpMetaDDateRecord (-1, true, 
//								copyData.dumpName, 2, 0, 
//								copyData.dumpName.length, null,false);
//
//				NdmpMetaDDateRecord  rec3 = new NdmpMetaDDateRecord (-1, true, 
//								copyData.dumpName, 0, 0, 
//								copyData.dumpName.length, null,true);
//	
//				NdmpMetaDDateRecord frec1 = 
//					(NdmpMetaDDateRecord) NdmpMetaFileFactory.findMatchRecord (1, rec1);
//				if (null != frec1) {
//					frec1.dumpDate = 0;
//					if (! NdmpMetaFileFactory.modifyRecord(1, frec1)) {
//						copyData.debugLog.println(50, "Dumpdate updation for level 1 record failed");
//					}
//				}
//
//				NdmpMetaDDateRecord frec2 = 
//					(NdmpMetaDDateRecord) NdmpMetaFileFactory.findMatchRecord (1, rec2);
//				if (null != frec2) {
//					frec2.dumpDate = 0;
//					if ( ! NdmpMetaFileFactory.modifyRecord(1, frec2)) {
//						copyData.debugLog.println(50, "Dumpdate updation for level 2 record failed");
//					}
//				}
//
//				NdmpMetaDDateRecord frec3 = 
//					(NdmpMetaDDateRecord) NdmpMetaFileFactory.findMatchRecord (1, rec3);
//				if (null != frec3) {
//					frec3.dumpDate = 0;
//					if ( ! NdmpMetaFileFactory.modifyRecord(1, frec3)) {
//						copyData.debugLog.println(50, "Dumpdate updation for level 'i' record failed");
//					}
//				}
//			}
//			return true;
//		} 
//			
//		return false;
//	}

	public void shutdownCopyOp () {
		if(alreadyshutdown)
			return;
		alreadyshutdown = true;

		
		try {
			sendMessageNoWait (NdmpCopyCmdArgs.srcFiler, 
				NdmpCons.NDMP_CONNECT_CLOSE);
		}catch (NdmpFatalException ignored) {
			//log it.
			copyData.debugLog.println (40, "Failed to send CONNECT_CLOSE to source");
		}
		try {
			sendMessageNoWait (NdmpCopyCmdArgs.dstFiler,
				NdmpCons.NDMP_CONNECT_CLOSE);
		}catch (NdmpFatalException ignored) {
			// log it
			copyData.debugLog.println (40, "Failed to send CONNECT_CLOSE to destination");
		}
	}	

	public void println (NdmpMessage msg, String msgString, int filer) {
		if (null == msgString || msgString.length () < 1)
			return ;
		copyData.out.println ("Ndmpcopy: " +
			copyData.cmdArgs.filerInfo [filer].filerName + ": " +
			msg.msgInterfaceName () + ": " + msgString);
	}
	
	public void println (String msgString, int filer) {
		if (null == msgString || msgString.length () < 1)
			return ;
		copyData.out.println ("Ndmpcopy: " +
			copyData.cmdArgs.filerInfo [filer].filerName + ": " +
			NdmpCopyCmdArgs.filerType (filer) + ": " + msgString);
	}
	
	public void abort (String errMsg) throws NdmpFatalException {
		if (null == errMsg)
			errMsg = "Fatal error";

		copyData.debugLog.println (20, errMsg);
		throw new NdmpFatalException (errMsg);
	}

	//public void abort (Exception exception) throws NdmpFatalException {
	public void abort (Exception exception, String err) throws NdmpFatalException {
		String errMsg = null;

		if (null != exception)
			errMsg = exception.getMessage ();
		// this is a sugar coated message  we want thrown up to the user 
		// note that this overrides the exception message
		if(null != err)
			errMsg = err;
		if (null == errMsg)
			errMsg = "Fatal error";
	 
		copyData.debugLog.println (20, exception, errMsg);

		throw new NdmpFatalException (errMsg); 
	}		

	public String timetaken (long timeinmillis) {
		StringBuffer timetaken = new StringBuffer();
		long timeinseconds = timeinmillis / 1000;
		int numdays, numhours, numminutes, numseconds;

		numdays = (int) timeinseconds / (24 * 60 * 60);
		timeinseconds = timeinseconds % (24 * 60 * 60);

		numhours = (int) timeinseconds / (60 * 60);
		timeinseconds = timeinseconds % (60 * 60);

		numminutes = (int) timeinseconds / 60;
		numseconds = (int) timeinseconds % 60;

		if (numdays > 0)
			timetaken = timetaken.append(numdays + " days ");
		
		if (numhours > 0)
			timetaken = timetaken.append(numhours + " hours ");
		
		if (numminutes > 0)
			timetaken = timetaken.append(numminutes + " minutes ");
		
		if (numseconds > 0)
			timetaken = timetaken.append(numseconds + " seconds");

		return timetaken.toString();
	}
}
