package com.netapp.offtap.ndmpcopy.ndmpcopy;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebugLog;
import com.netapp.offtap.ndmpcopy.messagereader.NdmpMessageQueue;
import com.netapp.offtap.ndmpcopy.messagereader.NdmpMessageReader;
import com.netapp.offtap.ndmpcopy.messagereader.NdmpQueueException;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessage;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageDataReply;
import com.netapp.offtap.ndmpcopy.messages.NdmpMessageFactory;
import com.netapp.offtap.ndmpcopy.messages.notify.NdmpMessageNotifyConnectedRequest;
import com.netapp.offtap.ndmpcopy.ndmpcopycmd.NdmpCopyCmdArgs;
import com.netapp.offtap.ndmpcopy.streams.NdmpInput;
import com.netapp.offtap.ndmpcopy.streams.NdmpOutput;
import com.netapp.offtap.ndmpcopy.util.InvalidVersionException;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;
import com.netapp.offtap.ndmpcopy.util.NdmpFatalException;
import com.netapp.offtap.ndmpcopy.util.NdmpKillThreadException;





public class NdmpCopyThread extends Thread {
	protected NdmpCopyOperation copyOp;
	protected NdmpCopyData copyData;			
	protected NdmpMessageQueue msgQueue;
	
	private Exception exception = null;

	private	int threadState;
	private	static final int ALIVE = 0;
	private	static final int DIE = 1;
	private	static final int DEAD = 2;

	public NdmpCopyThread (NdmpCopyData copyData) {
		super("ndmpcopythread");

		this.copyData = copyData;
		this.copyOp = null;
		threadState = ALIVE;
//		start ();
	}

	/**
	 * Never used
	 */
	public NdmpCopyThread () {
		super("ndmpcopythread");
		copyData = null;
		copyOp = null;
		threadState = ALIVE;
	}

	public boolean isDone () {
		if (DEAD == threadState)
			return true;
		return false;
	}

	public void killIt () {

		// burt 261345: Avoid race with cleanup()
		synchronized (this) {
			// if not alive, it means killIt was already done
			if(ALIVE != threadState) 
				return;
			threadState = DIE;
		}
		copyData.debugLog.println(10, "Abort request from user");
		copyData.out.println ("Ndmpcopy: User requesting abort ...");
		if (null != msgQueue)
			msgQueue.invalidateQueue ();	
		this.interrupt ();
		
	}
	
	public void run () {
		// Set up the connection and get the version #
		InetAddress srcInetAddr, dstInetAddr;
		Socket srcSocket = null, dstSocket = null;
		NdmpInput srcInput = null, dstInput = null;
		NdmpOutput srcOutput = null, dstOutput = null;
		NdmpMessageFactory srcMsgFactory = null,
			dstMsgFactory = null;
		NdmpDebugLog debugLog = copyData.debugLog;
		NdmpMessageReader srcMsgReader = null,
			dstMsgReader = null;
		NdmpMessage srcMsg = null, dstMsg = null;
		
		try {
			try {
				NdmpCopyCmdArgs	cmdArgs = copyData.cmdArgs;
				if (ALIVE == threadState) {
					srcInetAddr = dstInetAddr = null;
					srcSocket = dstSocket = null;

					/* we need a check for catching illegally 
					 * formatted IP addresses */

					try {
						srcInetAddr = InetAddress.getByName(cmdArgs.
							filerInfo[NdmpCopyCmdArgs.srcFiler].filerName);
					} catch (UnknownHostException e) {
						abort(e, "Hostname resolution for source " + 
							cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].
							filerName + " failed");
					}

					try{
						srcSocket = new Socket (srcInetAddr, NdmpCons.NDMP_PORT_NUMBER);
					} catch (IOException e) {
						System.out.println( "Ndmpcopy: Socket connection to source " + 
							cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].
							filerName + " failed");
						abort(e, "Issue 'ndmpd on' on the source filer to enable NDMP request then retry the connection");
					}
					
					Thread.currentThread ().yield ();
					if (ALIVE != threadState) {
						srcSocket.close();
						throw new NdmpKillThreadException ("Received interrupt. Aborting ...");
					}
			
					try {
						dstInetAddr = InetAddress.getByName(cmdArgs.
							filerInfo[NdmpCopyCmdArgs.dstFiler].filerName);
					} catch (UnknownHostException e) {
						srcSocket.close();
						abort(e, "Hostname resolution for destination " + 
							cmdArgs.filerInfo[NdmpCopyCmdArgs.dstFiler].
							filerName + " failed");
					}

					try{
						dstSocket = new Socket (dstInetAddr, NdmpCons.NDMP_PORT_NUMBER);
					} catch (IOException e) {
						System.out.println("Ndmpcopy: Socket connection to destination " + 
							cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].filerName +
							" failed"); 
						srcSocket.close();
						abort(e, "Issue 'ndmpd on' on the destination filer to enable NDMP request then retry the connection");
					}
					
					/*
					 * BURT164440
					 * Set SO_LINGER socket option on output so that
					 * the last message can be sent out before socket
					 * is closed. The timeout value is in seconds.
					 */
					dstSocket.setSoLinger (true, 1);
					dstSocket.setTcpNoDelay (true);
						
					Thread.currentThread ().yield ();
					if (ALIVE != threadState) {
						srcSocket.close();
						dstSocket.close();
						throw new NdmpKillThreadException ("Received interrupt. Aborting ...");
					}

					try {
						srcInput = new NdmpInput (srcSocket.getInputStream());
						srcOutput = new NdmpOutput (srcSocket.getOutputStream());
					} catch (IOException e) {
						debugLog.println(20, "IOException while setting " +
							"up source streams");
						srcSocket.close();
						dstSocket.close();
						abort(e, "Trouble during set up for source " +
							cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].
							filerName);
					}	

					try {
						dstInput = new NdmpInput (dstSocket.getInputStream());
						dstOutput = new NdmpOutput (dstSocket.getOutputStream());

					} catch (IOException e) {
						debugLog.println(20, "IOException while setting " +
							"up destination streams");
						srcSocket.close();
						dstSocket.close();
						abort(e, "Trouble during set up for destination " +
							cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].
							filerName);
					}	
					
					Thread.currentThread ().yield ();
					if (ALIVE != threadState) {
						srcSocket.close();
						dstSocket.close();
						throw new NdmpKillThreadException ("Received interrupt. Aborting ...");
					}

					// At this point, all network connection setup has been done
					// The filerIP will be used as part of the hash sent to the
					// metadata file
					cmdArgs.filerInfo[NdmpCopyCmdArgs.srcFiler].filerIP = 
							srcInetAddr.getHostAddress();
					cmdArgs.filerInfo[NdmpCopyCmdArgs.dstFiler].filerIP = 
							dstInetAddr.getHostAddress();

					srcMsgFactory = NdmpMessageFactory.createMsgFactory (debugLog);
					dstMsgFactory = NdmpMessageFactory.createMsgFactory (debugLog);
					
					// This creates an NdmpMessageQueue which can hold upto a max 
					// of 100 unprocessed messages in each queue, one for the 
					// source and one for the destination.
					msgQueue = new NdmpMessageQueue (100);
				
					try {
						srcMsgReader = new NdmpMessageReader (srcInput, srcMsgFactory,
									msgQueue, copyData.debugLog, 
									NdmpCopyCmdArgs.srcFiler);
					} catch (NdmpQueueException e) {
						copyData.debugLog.println(20, "Error while setting up source " +
							"message reader");
						srcSocket.close();
						dstSocket.close();
						abort (e, "Trouble during setup for source " +
							cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].
							filerName );
					}
					
					try {
						dstMsgReader = new NdmpMessageReader (dstInput, dstMsgFactory,
									msgQueue, copyData.debugLog,
									NdmpCopyCmdArgs.dstFiler);
					} catch (NdmpQueueException e) {
						copyData.debugLog.println(20, "Error while setting up " +
							"destination message reader");
						srcSocket.close();
						dstSocket.close();
						abort (e, "Trouble during setup for destination " +
							cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].
							filerName );
					}

					NdmpFilerData[]	fData = new NdmpFilerData [2];

					fData [NdmpCopyCmdArgs.srcFiler] = new NdmpFilerData ( srcInput,
						srcOutput, srcMsgReader, srcMsgFactory);
									
					fData [NdmpCopyCmdArgs.dstFiler] = new NdmpFilerData (dstInput,
						dstOutput, dstMsgReader, dstMsgFactory);

					copyData.filerData = fData;
					copyData.msgQueue = msgQueue;
								
					/* 
					 * Servers should have sent us NOTIFY_CONNECTED messages for 
					 * protocol versions
					 */
					Thread.currentThread ().yield ();
					if (ALIVE != threadState)
						throw new NdmpKillThreadException ("Received interrupt. Aborting ...");

					try {
						srcMsg = srcMsgReader.readMessage (); 
					} catch (IOException e) {
						abort(e, "I/O error on socket to source " +
							cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].
							filerName);
					}
					srcMsg.writeLog(copyData.debugLog);
					
					try {
						dstMsg = dstMsgReader.readMessage ();
					} catch (IOException e) {
						abort(e, "I/O error on socket to destination " +
							cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].
							filerName);
					}
					dstMsg.writeLog(copyData.debugLog);
				
					NdmpMessageNotifyConnectedRequest srcData = null, dstData = null;
					try {
						srcData = (NdmpMessageNotifyConnectedRequest) srcMsg.getMsgData();
					}catch (ClassCastException e){
						copyData.debugLog.println(20, e, "Unexpected reply in " +
							" response to " + srcMsg.msgClassName() + " from source");
						abort (null);
					}

					/*
					 * Check if the source returned us a CONNECTION_REFUSED. 
					 */
					if(srcData.reasoncode == NdmpCons.NDMP_CONNECTED) {
						copyData.debugLog.println(50, "Connected to source"); 
						copyData.out.println("Ndmpcopy: " +
							cmdArgs.filerInfo[NdmpCopyCmdArgs.srcFiler].filerName +
							": Notify: Connection established"); 
					} else if(srcData.reasoncode == NdmpCons.NDMP_REFUSED) {
						abort("Connection refused by source " +
							cmdArgs.filerInfo[NdmpCopyCmdArgs.srcFiler].
							filerName);
					} else if(srcData.reasoncode == NdmpCons.NDMP_SHUTDOWN) {
						abort("Connection shutdown by source " +
							cmdArgs.filerInfo[NdmpCopyCmdArgs.srcFiler].
							filerName);
					}
			
					try {
						dstData = (NdmpMessageNotifyConnectedRequest) dstMsg.getMsgData();
					}catch (ClassCastException e){
						copyData.debugLog.println(20, e, "Unexpected reply in " +
							" response to " + dstMsg.msgClassName() + " from destination");
						abort (null);
					}
					
					/*
					 * Check if the destination returned us a CONNECTION_REFUSED. 
					 */
					if(dstData.reasoncode == NdmpCons.NDMP_CONNECTED) {
						copyData.debugLog.println(50, "Connected to destination"); 
						copyData.out.println("Ndmpcopy: " + 
							cmdArgs.filerInfo[NdmpCopyCmdArgs.dstFiler].filerName +
							": Notify: Connection established"); 
					} else if(dstData.reasoncode == NdmpCons.NDMP_REFUSED) {
						abort("Connection refused by destination " +
							cmdArgs.filerInfo[NdmpCopyCmdArgs.dstFiler].
							filerName);
					} else if(dstData.reasoncode == NdmpCons.NDMP_SHUTDOWN) {
						abort("Connection shutdown by destination " +
							cmdArgs.filerInfo[NdmpCopyCmdArgs.dstFiler].
							filerName);
					}

					int ver = NdmpCons.NDMP_MAX_VERSION;
					
					Thread.currentThread ().yield ();
					if (ALIVE != threadState)
						throw new NdmpKillThreadException ("Received interrupt. Aborting ...");

					try {
						ver = findVersion (srcData.version, dstData.version);
						debugLog.println (20,"Auto negotiated to version " +
							ver);
					} catch (InvalidVersionException e) {
						// burt 54790
						abort (e, null);
					} catch (NdmpFatalException e) {
						abort (e, null);
					} catch (IOException e) {
						abort (e, "Failure during version auto negotiation");
					}	
					
					copyData.version = ver;
					int filer = NdmpCopyCmdArgs.srcFiler;	
					copyData.filerData [filer].msgFactory = new NdmpMessageFactory (
						copyData.filerData [filer].msgFactory, ver, debugLog);
					srcMsgReader.changeMsgFactory (
						copyData.filerData [filer].msgFactory);	

					filer = NdmpCopyCmdArgs.dstFiler;					
					copyData.filerData [filer].msgFactory = new NdmpMessageFactory (
						copyData.filerData [filer].msgFactory, ver, debugLog);
					dstMsgReader.changeMsgFactory (
						copyData.filerData [filer].msgFactory);

					Thread.currentThread ().yield ();
					if (ALIVE != threadState)
						throw new NdmpKillThreadException ("Received interrupt. Aborting ...");

					// create the Ndmpcopy operation object and we'r done....
					copyOp = (NdmpCopyOperation) NdmpCopyOperation.createCopyOp (copyData);
					
					if (ALIVE != threadState)
						throw new NdmpKillThreadException ("Received interrupt. Aborting ...");

					copyOp.rocknRoll ();
					// Done
				} else {
					copyData.debugLog.println(20, "NdmpCopyThread state is not ALIVE");
					copyData.out.println ("Ndmpcopy: Fatal error");
				}
			
			} catch (NdmpKillThreadException e) {
				// User abort
				if (null != e && null != e.getMessage ())
					copyData.out.println ("Ndmpcopy: " + e.getMessage ());
				
				exception = e;
			
			} catch (NdmpFatalException e) {
				
				if (ALIVE != threadState) {
					// User aborted us, and something threw an exception
					copyData.out.println ("Ndmpcopy: Received interrupt. Aborting ...");
				} else if (null != e && null != e.getMessage ()) {
					// This is a real problem, so let the user know
					copyData.out.println ("Ndmpcopy: " + e.getMessage ());
				} else {
					// No information in the exception
					copyData.out.println ("Ndmpcopy: Fatal error");
				}

				exception = e;
			}
		}catch (Exception e) {
			//e.printStackTrace (System.out);
			if (null != e && null != e.getMessage ())
				debugLog.println (10, e, "Unexpected exception");
			copyData.out.println ("Ndmpcopy: Fatal error");
			
			exception = e;
		}

		cleanup();
		return;
	}	

	protected void cleanup () {
		copyData.debugLog.println(50, "Cleaning up");
		try{
			if (null != msgQueue)
				msgQueue.invalidateQueue ();
			if (null != copyOp)
				copyOp.shutdownCopyOp ();
			cleanupFiler (NdmpCopyCmdArgs.srcFiler);
			cleanupFiler (NdmpCopyCmdArgs.dstFiler);
			copyData.debugLog.println(50, "Finished cleaning up");
		} catch (Exception e){
			copyData.debugLog.println(30, e, "Exception during cleanup"); 
		}  
		
		synchronized (this) {
			// Mark the copy thread DEAD, so the ADMIN thread releases resources
			threadState = DEAD;
		}
	}

	protected void cleanupFiler (int filer) {
		if (null == copyData)
			return;

		try {
			if (null != copyData.filerData [filer].msgReader)
				copyData.filerData [filer].msgReader.killThread ();
			if (null != copyData.filerData [filer].input)	
				copyData.filerData [filer].input.close ();
			if (null != copyData.filerData [filer].output)
				copyData.filerData [filer].output.close ();
		} catch (IOException ioex) {
		} catch (NullPointerException npex) {
		} finally {
			if (null != copyData.filerData [filer].msgReader)
				copyData.filerData [filer].msgReader = null;	
			if (null != copyData.filerData [filer].input)	
				copyData.filerData [filer].input = null;	
			if (null != copyData.filerData [filer].output)
				copyData.filerData [filer].output = null;
		}
	}	 

	public int findVersion (int srcVersion, int dstVersion) 
			throws InvalidVersionException, NdmpFatalException,
			IOException {
												
		// In minimal iterations find out which version to use.
		
		/**
		 * Algo at a glance.
		 * sv - src version 
		 * dv - destination version
		 * findversion (sv, dv)
		 * begin
		 * 	if Max Version == sv == dv
		 *		return Max Version ;; nothing to do
		 *	start from Max version till both sv and dv are equal
		 * end
		 */
		if ((0 == (srcVersion - dstVersion)) && 
			srcVersion == NdmpCons.NDMP_MAX_VERSION) {
			return NdmpCons.NDMP_MAX_VERSION;
		}
			
		int tmpV = NdmpCons.NDMP_MAX_VERSION;
		int dstV = dstVersion;
		int i = 0;

		do {
			for (i = 0; i < 2; i++) {
				tmpV = requestVersion (i, tmpV);
				if (tmpV == dstV)
					break;
				dstV = tmpV;
			} 
		}while (i > 1);
		return tmpV;
	}

	// returns the version am NDMP server agrees upon.
	public int  requestVersion (int filer, int reqVersion) 
			throws InvalidVersionException, NdmpFatalException, IOException {

		filer %= 2;		
		if (NdmpCons.NDMP_MAX_VERSION < reqVersion)
			throw new InvalidVersionException ("Version " + reqVersion +
				" greater than maximum supported version " +
				NdmpCons.NDMP_MAX_VERSION);
				
		while (reqVersion >= NdmpCons.NDMP_MIN_VERSION) {
			NdmpMessage msg = 
				copyData.filerData [filer].msgFactory.createConnectOpenMsg (
					(short)	reqVersion);
			try {										
				msg.write (copyData.filerData [filer].output);
				msg.writeLog (copyData.debugLog);
			}catch (IOException e) {
				copyData.debugLog.println (30, "IOException while sending " +
					"ConnectOpenRequest to " + NdmpCopyCmdArgs.filer(filer));
				abort(null);
			}
			
			// readMessage throws an IOException which is caught during the call to 
			// requestVersion
			NdmpMessage rmsg = copyData.filerData [filer].msgReader.readMessage ();

			rmsg.writeLog(copyData.debugLog);
			
			// consistent with sendMessage function in NdmpCopyOperation
			// isHeaderError() returns TRUE if the error is NDMP_NO_ERR
			// XXX: ideally we would want this to be called isNoHeaderError
			if (rmsg.isHeaderError()) {
				// should only return non null msg
				NdmpMessageDataReply msgData = null;

				try {
					msgData = (NdmpMessageDataReply) rmsg.getMsgData ();	
				}catch (ClassCastException e) {
					copyData.debugLog.println(30, e, "Unexpected reply in " +
						" response to " + msg.msgClassName() + " from " +
						NdmpCopyCmdArgs.filer(filer));
				}

				// if all is well, return this version
				if (msgData.isError(NdmpCons.NDMP_NO_ERR))
					return reqVersion;
				
				copyData.debugLog.println(30, "Body error " +
					msgData.msgBodyError() + " in reply message " + 
					rmsg.msgClassName() + " from " +
					NdmpCopyCmdArgs.filer (filer));
			} else 
				copyData.debugLog.println(30, "Header error " +
					rmsg.msgError() + " in reply message " + 
					rmsg.msgClassName() + " from " +
					NdmpCopyCmdArgs.filer (filer));

			reqVersion--;		// reduce the request version.
		}	
		throw new InvalidVersionException ("Server does not support ndmp version " +
				+ NdmpCons.NDMP_MIN_VERSION + " or higher");
	}	

	public void abort (String errMsg) throws NdmpFatalException {
		if (null == errMsg)
			errMsg = "Fatal error";

		copyData.debugLog.println (10, errMsg);
		throw new NdmpFatalException (errMsg);
	}

	//public void abort (Exception exception) throws NdmpFatalException {
	public void abort (Exception exception, String err) throws NdmpFatalException {
		String errMsg = null;

		if (null != exception)
			errMsg = exception.getMessage ();
		// this is a sugar coated message  we want thrown up to the user 
		// note that this overrides the exception message
		if(null != err)
			errMsg = err;
		if (null == errMsg)
			errMsg = "Fatal error";
	 
		copyData.debugLog.println (10, exception, errMsg);
		//XXX Intercepting the exception seems like a really bad idea
		throw new NdmpFatalException (errMsg); 
	}

	public Exception getException() {
		return exception;
	}

	public void setException(Exception Exception) {
		this.exception = exception;
	}		
}
