
package com.netapp.offtap.ndmpcopy.ndmpcopycmd;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebug;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;


public class NdmpCopyCmdParser {

	private	String[] args;
	private	int	argIdx;		/* keeps track of current arg */
	private	int	curPos;		/* keeps track of current position
							 * within current arg 
							 */
	
	public NdmpCopyCmdParser (String[] args) {
		this.args = args;
		curPos = 0;
		argIdx = 0;
	}
	
	/**
	 * nextToken
	 *
	 * This will return the next token in the current argument.
	 *
	 * The string returned is a substring, from the current
	 * position as stored in curPos, upto (but not including)
	 * the position where the first delim character is
	 * encountered.
	 * 
	 * If end of string is encountered, then the entire substring
	 * starting from the curPos is returned.
	 *
	 * If the delim passed in is the NULL_DELIM character, then
	 * the delim matching is skipped, and the entire substring
	 * starting from curPos is returned.
	 */
	public String nextToken (char delim)
			throws NdmpCopyCmdException {

		/* debugging info */
		String methodname = "nextToken";
		String retStr = "";

		// start from the position we last left //
		int startPos = curPos;
		NdmpDebug.dbgPrintln(methodname + " : starting @ pos " + startPos +
			" in current arg :" + args[argIdx]);

		if (startPos >= args[argIdx].length()) {
			/* reached end of current arg */
			NdmpDebug.dbgPrintln(methodname + " : reached end of current arg : " +
				args[argIdx]);
			return retStr;
		}

		/* 
		 * Avoid using the in-built java String class methods, because
		 * we dont want any exceptions here.
		 */
		char[] theArgs = args[argIdx].toCharArray();
		int theArgslen = theArgs.length;
		int endPos = startPos;
		while (endPos < theArgslen) {
			if ((NdmpCons.NULL_DELIM != delim) && 
					(delim == theArgs[endPos])) {
				NdmpDebug.dbgPrintln(methodname + " : matched delim " +
					delim + " in arg :" + args[argIdx]);
				break;
			}
			endPos++;
		}

		retStr = String.valueOf(theArgs, startPos, (endPos - startPos));
		retStr = retStr.trim();
		
		NdmpDebug.dbgPrintln(methodname + " : return string :" + retStr);

		/* skip delimiter */
		if (endPos < theArgslen) {
			endPos++;
		} 

		/*
		 * we want to remember the current position in case someone
		 * asks for the next token in the same arg.
		 */
		curPos = endPos;
		NdmpDebug.dbgPrintln(methodname + " : leaving @ curPos : " + curPos +
			" in current arg :" + args[argIdx]);

		return retStr;
	}
	
	/**
	 * This sets the next arg in the args array to be the
	 * current arg. The method nextToken, will by default
	 * only look at the current arg. Once it is done
	 * processing the current arg, the setNextArg method should
	 * be used to move to the next arg in the args list.
	 */
	public boolean setNextArg () {
		String methodname = "setNextArg";

		if ((argIdx < 0) || (argIdx >= args.length - 1)) {
			/* set to args.length, so getCurArg will fail */
			argIdx = args.length;
			return false;
		}

		argIdx++;
		curPos = 0;
		
		NdmpDebug.dbgPrintln(methodname + " : set current arg :" +
			args[argIdx]);

		return true;
	}
	
	/**
	 * Returns the current arg.
	 */
	public String getCurArg() {
		String methodname = "getCurArg";
	
		if ((argIdx < 0) || (argIdx >= args.length)) {
			return null;
		}

		NdmpDebug.dbgPrintln(methodname + " : returning cur arg :" +
			args[argIdx]);

		return args[argIdx];
	}
}
