
package com.netapp.offtap.ndmpcopy.ndmpcopycmd;

import java.util.Properties;

import com.netapp.offtap.ndmpcopy.util.NdmpCons;


public class NdmpCopyCmdSemantics {

	/** 
	 * The user specified values for any of these env vars are
	 * blocked from being passed on to dump/restore.
	 */
	protected static String[] BlockedEnvVarList = { "HIST", "DIRECT",
				"BASE_DATE", "DUMP_DATE", "DMP_NAME",
				"EXCLUDE_PATH", "FILESYSTEM",
				"LEVEL"};

	//These files wont be back'd up unless force to do so..
	protected static String[] ExcludeFiles = { "/etc" };

	public static NdmpCopyCmdArgs checkSemantics (NdmpCopyCmdArgs cmdArgs)
						throws NdmpCopyCmdException {
		/**
		 * 1> If source and destination filer and paths are same
		 *	just return.
		 * 2> Ignore the env vars that could cause trouble 
		 * 3> Auth types are text and md5
		 * 4> level = 0 - 2
		 * 5> FILESYSTEM env var should have a valid value.
		 * 6> Exclude system files to be back'd up, unless specified
		 *    using '-f' force all files to be dump'd.
		 * 7> Check for Source filerName = dst filer name = hostname
		 *	  if any of src/dst equals hostname that filer doesnt need
		 *	  any authentication.
		 */

		
		// <1>
		cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].filerName = 
			checkFilerName (
				cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].filerName);
		cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].filerName = 		
			checkFilerName (
				cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].filerName);
				
		checkFilerPath (cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].filerName,
			cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].filerName,
			cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].pathName,
			cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].pathName);

		//<2>
		cmdArgs.envVar = checkEnvVars (cmdArgs.envVar);

		//<3>
		cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].authType = checkAuthType (
			cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].authType);
		cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].authType = checkAuthType (
			cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].authType);

		//<4>
		cmdArgs.envVar = checkLevel (cmdArgs.levelSpec, cmdArgs.level, 
			cmdArgs.envVar);

		//<5>
		cmdArgs.envVar = checkFileSystem (cmdArgs.envVar, 
			cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].pathName);
		
		//<6>
		cmdArgs.envVar = checkForcedOption (cmdArgs.forced, cmdArgs.envVar);
			
		//<7>
		cmdArgs = checkFilerAuth (cmdArgs);	
		return cmdArgs;
	}

	protected static NdmpCopyCmdArgs checkFilerAuth (NdmpCopyCmdArgs cmdArgs) 
			throws NdmpCopyCmdException {

		cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler] =
			checkAuthNeeded (cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler]);
		cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler] =
			checkAuthNeeded (cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler]);

		String sfname = cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler].filerName;
		String dfname = cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler].filerName;
	
		NdmpFilerInfo sfInfo = cmdArgs.filerInfo [NdmpCopyCmdArgs.srcFiler];
		NdmpFilerInfo dfInfo = cmdArgs.filerInfo [NdmpCopyCmdArgs.dstFiler];
	
		if (0 == sfname.compareTo (dfname)) {
			if (null == sfInfo.usrName) {
				if (null == dfInfo.usrName)
					throw new NdmpCopyCmdException ("Invalid username specified");
				else {
					sfInfo.usrName = dfInfo.usrName;
					sfInfo.usrPasswd = dfInfo.usrPasswd;
				}
			} else if (null == dfInfo.usrName) {
				dfInfo.usrName = sfInfo.usrName;
				dfInfo.usrPasswd = sfInfo.usrPasswd;
			}
		}
		return cmdArgs;
	}

	protected static String checkFilerName (String filerName) {
		if ((null == filerName) || (filerName.equals("")))
			//filerName = ONTAP.getLocalHostName ();
			//TODO: Handle this more gracefully
			throw new IllegalArgumentException("You have to supply both " +
					"a source and destination filer hostname");
			
		return filerName;	
	}


	protected static void checkFilerPath (String srcFilerName, 
			String dstFilerName, String srcFilerPath,
			String	dstFilerPath) throws NdmpCopyCmdException {
							
		if ((null == srcFilerPath) || (srcFilerPath.equals("")))
		 	throw new NdmpCopyCmdException ("Invalid source path");
	
		if ((null == dstFilerPath) || (dstFilerPath.equals("")))
			throw new NdmpCopyCmdException ("Invalid destination path");

		//removing the trailing /
		if (srcFilerPath.endsWith("/"))
			srcFilerPath = srcFilerPath.substring(0, srcFilerPath.length()-1);
		if (dstFilerPath.endsWith("/"))
			dstFilerPath = dstFilerPath.substring(0, dstFilerPath.length()-1);

		boolean pathEqual = (0 == dstFilerPath.compareTo (srcFilerPath)); 
		
		if (null == srcFilerName)
			throw new NdmpCopyCmdException ("Invalid source filer name");	
		if (null == dstFilerName)
			throw new NdmpCopyCmdException ("Invalid destination filer name");

		if (0 == srcFilerName.compareTo (dstFilerName))	//same filer
				if (pathEqual)	// same path
					throw new NdmpCopyCmdException ("Same source and "+
					"destination path");

		return ;
	}

	protected static Properties checkEnvVars (Properties envVar) {
		
		if (null == envVar) {
			envVar = NdmpCopyCmdArgs.getDefaultEnvVarList();
		}
			
		for (int i = 0; i < BlockedEnvVarList.length; i++) {
			envVar.remove(BlockedEnvVarList [i]);
		}

		return envVar;
	}

	protected static Properties checkLevel (boolean levelSpec, 
			int level, Properties envVar) throws NdmpCopyCmdException {

		if (!levelSpec) {
			level = 0;
		}

		if (level < 0 || level > 2)
			throw new NdmpCopyCmdException ("Invalid  level " + level);

		if (null == envVar) {
			envVar = NdmpCopyCmdArgs.getDefaultEnvVarList();
		}

		envVar.put ("LEVEL", String.valueOf (level));
		return envVar;
	}
	
	protected static Properties checkFileSystem (Properties envVar, 
			String pathName) {

		if (null == envVar) {
			envVar = NdmpCopyCmdArgs.getDefaultEnvVarList();
		}
		
		envVar.put ("FILESYSTEM", pathName);
		return envVar;
	}	

	protected static String checkAuthType (String authType)
			throws NdmpCopyCmdException {

		if (null == authType) {
			authType = new String ("md5"); //default auth type
			return authType;
		}
		
		return authType;
	}		

	protected static Properties checkForcedOption (boolean forced, 
			Properties envVar) {

		if (null == envVar) {
			envVar = NdmpCopyCmdArgs.getDefaultEnvVarList();
		}
			
		if (!forced) {
			String val = new String ("");
			
			for (int i = 0; i < ExcludeFiles.length; i++) {
				if (i != 0)
					val += NdmpCons.EXCLUDE_PATH_SEPARATOR;
				val += ExcludeFiles [i];
			}
			envVar.put ("EXCLUDE_PATH", val);
		}

		return envVar;
	}	

	protected static NdmpFilerInfo checkAuthNeeded (NdmpFilerInfo fInfo) {

//		String hostName = ONTAP.getLocalHostName ();
		String hostName = "";

		if (fInfo.filerName.compareTo (hostName) != 0) {
			fInfo.authNeeded = true;
			return fInfo;
		}

//		fInfo.authNeeded = false;
//		
//		/* check if user name is still null */
//		if (fInfo.usrName != null) 
//			return fInfo;
//
//		String rootPasswd = null;
//
//		if (ONTAP.isDefaultVfiler()) {
//			rootPasswd = ONTAP.rootPasswd ();
//		} else {
//			rootPasswd = NdmpServer.generatePassword("root");
//			if (rootPasswd == null) {
//				// This should never occur, but if it does,
//				// default to empty password.
//				rootPasswd = "";
//			}
//		}
//
//		fInfo.usrName = new String ("root");
//		fInfo.usrPasswd = new String (rootPasswd);
//
//		rootPasswd = null;
		
		return fInfo;
	}	
}
