package com.netapp.offtap.ndmpcopy.ndmpcopycmd;

import java.util.Properties;

import com.netapp.offtap.ndmpcopy.debuglog.NdmpDebug;
import com.netapp.offtap.ndmpcopy.util.NdmpCons;



public class NdmpCopyCmdSyntax {

	private static String[] validAuthTypes = {"text", "md5"};

	/* Incremental-forever mode is supported only when the
	   privilege is set to diag */
	private static boolean diagMode = false;
	

	public static NdmpCopyCmdArgs checkSyntax (NdmpCopyCmdParser parser) 
						throws NdmpCopyCmdException {
		int filerID;
		int filerIndex, i = 0;
		NdmpCopyCmdArgs cmdArgs = null;
		String token = null;

		diagMode = false;
//		if ((ONTAP.cmd_get_priv() & ONTAP.SH_PRIV_DIAG) != 0) {
//		    diagMode = true;
//		}

		/* debugging info */
		String methodname = "checkSyntax";
		cmdArgs = new NdmpCopyCmdArgs ();

		/**
		 * -st <md5|text> -dt <md5|text>
		 *
		 * -sa <user>:[<passwd>] -da <user>:[<passwd>]
		 */
		do {
			token = parser.getCurArg();
			if ((null == token) || (token.equals("")))
				throw new NdmpCopyCmdException ("Invalid syntax");

			NdmpDebug.dbgPrintln(methodname + ": saw arg :" + token);

			if ('-' != token.charAt (0))
				break;

			if (token.length () < 2)
				throw new NdmpCopyCmdException ("Invalid option "+
					token);

			if (2 == token.length ()) {
				switch (token.charAt (1)) {
					case	'd' :
						/**
						 * -d turns debugging on (no args)
						 */
						cmdArgs.debugLevel = 50;
						break;
				
					case	'l' :	
						/**
						 * -l level #
						 */
						if(cmdArgs.incrForever) {
						    throw new NdmpCopyCmdException (
							"Invalid syntax. -l option cannot be specified with -i option");
						}
						if (false == parser.setNextArg()) {
						    throw new NdmpCopyCmdException (
							"Invalid syntax. Level number expected");
						}

						String level = parser.getCurArg();
						NdmpDebug.dbgPrintln(methodname + 
							": saw level = " + level);

						try {
							cmdArgs.level = Integer.parseInt (level);
						}catch (NumberFormatException e) {
							throw new NdmpCopyCmdException (
								"Invalid level number " + level);
						}

						cmdArgs.levelSpec = true;
						break;		

					case	'i' :
						/**
						 * -i incremental transfer (no args)
						 */
						if (diagMode) {
							if (cmdArgs.levelSpec) {
						    		throw new NdmpCopyCmdException (
								"Invalid syntax. -l option cannot be specified with -i option");
							}
							cmdArgs.incrForever = true;
						}
						else {
						    	throw new NdmpCopyCmdException (
							"-i option can be used only in the diag mode");
						}
						break;

					case	'e' :	
						/**
						 * -e Env Var List
						 * -e VAR = VALUE, VAR1 = VALUE1...
						 */
						if (false == parser.setNextArg())
						    throw new NdmpCopyCmdException (
							"Invalid syntax. Env variable" +
							"list expected");

						cmdArgs.envVar = checkEnvListSyntax(parser);
						break;
						
					case	'f' :	
						/**
						 * -f forced (no args)
						 */
						cmdArgs.forced = true;
						break;

					case	'h' :	
						printHelpMessage ();
						return null;
							
					default :
						throw new NdmpCopyCmdException (
							"Invalid option "+token);
				}
			} else if (3 == token.length()) { 
				switch (token.charAt (1)) {
					case	's' :
						filerIndex = NdmpCopyCmdArgs.srcFiler;
						break;
					case	'd' :
						filerIndex = NdmpCopyCmdArgs.dstFiler;
						break;
					default :
						throw new NdmpCopyCmdException (
							"Invalid option "+token);
				}
				switch (token.charAt (2)) {
					case	't' :
						if (false == parser.setNextArg()) {
						    throw new NdmpCopyCmdException (
							"Invalid syntax. Authentication type " +
							" expected after " + token);
						}
						String authType = parser.getCurArg();
						NdmpDebug.dbgPrintln(methodname + 
							": saw authtype :" + authType);

						for (i = 0; i < validAuthTypes.length; i++) {
							if (authType.equals(validAuthTypes[i])) {
								break;
							}
						}
						
						if (i >= validAuthTypes.length) {
							throw new NdmpCopyCmdException (
								"Unknown "+((filerIndex == 
								NdmpCopyCmdArgs.srcFiler) ?
								"source" : "destination") +
								" authentication type " + authType);
						}

						cmdArgs.filerInfo[filerIndex].authType = authType;
						break;
					
					case	'a' :
						if (false == parser.setNextArg()) {
							throw new NdmpCopyCmdException ("Invalid syntax. "+
							"Authentication information expected after " +
							token);
						}
						String ntok = parser.nextToken(':');
						if (ntok.equals(""))
							throw new NdmpCopyCmdException (
								"Invalid syntax. Filer username " +
								"expected after " + token);
						
						NdmpDebug.dbgPrintln(methodname + 
							": saw username :" + ntok);
							
						cmdArgs.filerInfo [filerIndex].usrName = ntok;
						
						ntok = parser.nextToken(NdmpCons.NULL_DELIM);
						NdmpDebug.dbgPrintln(methodname + 
							": saw password :" + ntok);
						// At this moment fsinfo struct only keeps the
						// first 16 characters of the password.
						if(ntok.length() > NdmpCons.NDMP_PASSWD_SIGNIFICANT_LEN)
							ntok = ntok.substring(0, NdmpCons.NDMP_PASSWD_SIGNIFICANT_LEN);
						// we allow the empty string as password //
						cmdArgs.filerInfo [filerIndex].usrPasswd = ntok;
						break;

					default :
						throw new NdmpCopyCmdException (
							"Invalid option "+token);
				}
			} else {
				throw new NdmpCopyCmdException (
					"Invalid option "+token);
			}

		} while (false != parser.setNextArg());

		if (null == parser.getCurArg())
			throw new NdmpCopyCmdException ("Invalid syntax. "+
			"Filer name/path expected");
		checkFilerInfo(cmdArgs.filerInfo[NdmpCopyCmdArgs.srcFiler], parser);
		
		if (false == parser.setNextArg()) {
			throw new NdmpCopyCmdException ("Invalid syntax. "+
			"Destination filer name/path expected");
		}
		checkFilerInfo(cmdArgs.filerInfo[NdmpCopyCmdArgs.dstFiler], parser);

		if (false != parser.setNextArg()) {
			throw new NdmpCopyCmdException ("Invalid syntax. " +
				"Unexpected argument :" + parser.getCurArg());
		}
	
		return cmdArgs;
	}

	protected static void checkFilerInfo (NdmpFilerInfo filerInfo,
			NdmpCopyCmdParser parser) throws NdmpCopyCmdException {
			
		/**
		 * The Syntax is:
		 * [filer name:] <filer-path> 
		 */
		
		String fname, fpath;
		/* debugging info */
		String methodname = "checkFilerInfo";

		fname = parser.nextToken(':');
		if (fname.equals(""))
			throw new NdmpCopyCmdException ("Invalid syntax. "+
			"Filer name/path expected");
		
		NdmpDebug.dbgPrintln(methodname + 
			": saw filername :" + fname);
		
		fpath = parser.nextToken(NdmpCons.NULL_DELIM);
		NdmpDebug.dbgPrintln(methodname + 
			": saw filerpath :" + fpath);
		if (fpath.equals("")) {
			filerInfo.pathName = fname;
			filerInfo.filerName = null;
		} else { 
			filerInfo.filerName = fname;
			filerInfo.pathName = fpath;
		}
	}

	protected static Properties checkEnvListSyntax(NdmpCopyCmdParser parser) 
			throws NdmpCopyCmdException {

		/**
		 * Syntax expected is:
		 *
		 * -e <name>:<val>[,<name>:<val>]
		 */

		/* debugging info */
		String methodname = "checkEnvListSyntax";
		Properties env = NdmpCopyCmdArgs.getDefaultEnvVarList();
		boolean found = false;

		while (true) {
			String envName = parser.nextToken('=');
			if (envName.equals("")) {
				if (found == false) {
					// error 
					throw new NdmpCopyCmdException("Invalid syntax. Empty env " +
						"name encountered");
				} else //end of the env var list 
					break;
			}
			NdmpDebug.dbgPrintln(methodname + ": saw envname :" + envName);

			String envVal = parser.nextToken(',');
			if (envVal.equals("")) {
				// error
				throw new NdmpCopyCmdException("Invalid syntax. Env value " +
					"eexpected after " + envName);
			}
			NdmpDebug.dbgPrintln(methodname + ": saw envval :" + envVal);

			/* Adding the pair to the env var list */
			env.put (envName, envVal);
			found = true;
		}

		return env;
	}

	public static void printUsage () {
	
		diagMode = false;
//		if ((ONTAP.cmd_get_priv() & ONTAP.SH_PRIV_DIAG) != 0) {
//		    diagMode = true;
//		}

		System.out.println("usage:");
		System.out.println ("ndmpcopy [<options>] <source> <destination>");
		System.out.println ();
		System.out.println ("<source> and <destination> are of the form [<filer>:]<path>");
		System.out.println ("options:");
		System.out.println ("\t[-sa <username>:<password>]");
		System.out.println ("\t[-da <username>:<password>]"); 
		System.out.println ("\t\tsource/destination filer authentication");
		System.out.println ("\t[-st { text | md5 }]");
		System.out.println ("\t[-dt { text | md5 }]"); 
		System.out.println ("\t\tsource/destination filer authentication type");
		System.out.println ("\t\tdefault is md5");
		System.out.println ("\t[-l { 0 | 1 | 2 }]");
		System.out.println ("\t\tincremental level");
		System.out.println ("\t\tdefault is 0");
		/* The incr-forever option is only available in the diag mode */
		if (diagMode) {
			System.out.println ("\t[-i]");
			System.out.println ("\t\tincrement forever");
			System.out.println ("\t\tThis option cannot be used in conjunction with -l option");
		}
		System.out.println ("\t[-d]");
		System.out.println ("\t\tdebug mode");
		System.out.println ("\t[-f]");
		System.out.println ("\t\tforce flag, to copy system files");
		System.out.println ("\t[-h]"); 
		System.out.println ("\t\tdisplay this message");
	}
	
	public static void printHelpMessage () {
		printUsage();
	}
}
