package com.netapp.offtap.ndmpcopy.streams;


import java.io.IOException;
import java.io.InputStream;

import com.netapp.offtap.ndmpcopy.xdr.rpc.XDRRMInputStream;


/**
 * NdmpInput
 *
 *  This class acts a wrapper class for the XDRRMInputStream.  Most of the 
 *  get functions make calls to the corresponding XDR get function.  
 *
 *  XDR has a concept of what a long and a int are which does not necessarily
 *  correspond with what Java believes they are.  The methods to this
 *  object are named based on the concept of what the XDR object thinks
 *  are that size.
 *                      XDR size
 *      int             32 bits
 *      unsigned int    32 bits
 *      long            32 bits
 *      unsigned long   32 bits
 *      enum            32 bits
 *      opaque          array of bytes
 */
public class NdmpInput {

	private XDRRMInputStream input;

	public NdmpInput (InputStream in) {
		input = new XDRRMInputStream(in);
	}

	/**
	 * setNewInputStream
	 *
	 *  In case of the control connection breaking, this allows the session
	 *  thread to reconnect to the client by creating new input stream.
	 *
	 */
	public void setNewInputStream (InputStream in) {
		this.input = new XDRRMInputStream(in);
	}

	/**
	 *  Closes off the input stream
	 */
	public void close () throws IOException {
		input.close();
	}
	
	public void resetStream () throws IOException { 
		input.resetStream();
	}

	public byte getByte () throws IOException {
		return input.getByte();
	}

	public short getShort () throws IOException {
		return input.getShort();
	}

	public int getLong () throws IOException {
		return input.getLong();
	}

	public long getHyper () throws IOException {
		return input.getHyper();
	}

	public String getString () throws IOException {
		return input.getString();
	}

	public byte[] getBytes () throws IOException {
		return input.getBytes();
	}

	public byte[] getBytes (int byte_len) throws IOException {
		return input.getBytes(byte_len);
	}
}

