package com.netapp.offtap.ndmpcopy.streams;

import java.io.IOException;
import java.io.OutputStream;

import com.netapp.offtap.ndmpcopy.xdr.rpc.XDRRMOutputStream;


/**
 * Handle the traffic originating from the NDMP Server to the Client.
 * There are two types of traffic:
 *	Replys to messages received from the client
 *	Requests sent on the Log, Notify and File History interfaces.
 *
 * This object works by knowing what type of replys it needs to
 * do and what type of requests it can start.  The other parts
 * of the server decide when or why to send such a request but
 * this object will do what it is told when it is told to do it.
 */

public class NdmpOutput {
	private		int sequence;					// one per instance 
	
	private XDRRMOutputStream output;
	
	private final	int	NDMP_HEADERSIZE = 24;	// size of NMPD header 



	public NdmpOutput (OutputStream out) {
		output = new XDRRMOutputStream(out);
		sequence = 0;
	}

	/**
	 *  In case of the control connection breaking, this allows the session
	 *  thread to reconnect to the client by creating new .
	 *
	public void setNewOutputStreams (OutputStream out) {
		this.output = new XDRRMOutputStream(out);
	}

	/**
	 *  Closes off the output stream
	 */
	public void close () throws IOException {
		output.close();
	}
	
	/* Synchronization of this method is the responsibility of the caller */
	public int getSequenceNumber () {
		sequence += 1;
		return sequence;
	}

	public void markStartRec () throws IOException {
		output.markStartRec();
	}

	public void markEndRec () throws IOException {
		output.markEndRec();
	}

	public void putByte (byte b) throws IOException {
		output.putByte(b);
	}

	public void putShort (short i) throws IOException {
		output.putShort(i);
	}

	public void putLong (int i) throws IOException {
		output.putLong(i);
	}

	public void putHyper (long i) throws IOException {
		output.putHyper(i);
	}

	public void putString (String s) throws IOException {
		output.putString(s);
	}

	public void putBytes (byte b[]) throws IOException {
		output.putBytes(b);
	}

	public void putBytes (byte b[], int len) throws IOException {
		output.putBytes(b, len);
	}

	public void putFixedLenBytes (byte b[], int len) throws IOException {
		output.putFixedLenBytes(b, len);
	}
}
