package com.netapp.offtap.ndmpcopy.util;

/*
 * Constants for NDMP
 */
public class NdmpCons {

/* request or reply */
public final static int NDMP_MESSAGE_REQUEST		= 0;
public final static int NDMP_MESSAGE_REPLY			= 1;

/* NDMP Error numbers */
public final static int NDMP_NO_ERR					= 0;
public final static int NDMP_NOT_SUPPORTED_ERR		= 1;
public final static int NDMP_DEVICE_BUSY_ERR		= 2;
public final static int NDMP_DEVICE_OPENED_ERR		= 3;
public final static int NDMP_NOT_AUTHORIZED_ERR		= 4;
public final static int NDMP_PERMISSION_ERR			= 5;
public final static int NDMP_DEV_NOT_OPEN_ERR		= 6;
public final static int NDMP_IO_ERR					= 7;
public final static int NDMP_TIMEOUT_ERR			= 8;
public final static int NDMP_ILLEGAL_ARGS_ERR		= 9;
public final static int NDMP_NO_TAPE_LOADED_ERR		= 10;
public final static int NDMP_WRITE_PROTECT_ERR		= 11;
public final static int NDMP_EOF_ERR				= 12;
public final static int NDMP_EOM_ERR				= 13;
public final static int NDMP_FILE_NOT_FOUND_ERR		= 14;
public final static int NDMP_BAD_FILE_ERR			= 15;
public final static int NDMP_NO_DEVICE_ERR			= 16;
public final static int NDMP_NO_BUS_ERR				= 17;
public final static int NDMP_XDR_DECODE_ERR			= 18;
public final static int NDMP_ILLEGAL_STATE_ERR		= 19;
public final static int NDMP_UNDEFINED_ERR			= 20;
public final static int NDMP_XDR_ENCODE_ERR			= 21;
public final static int NDMP_NO_MEM_ERR				= 22;
public final static int NDMP_CONNECT_ERR			= 23;

public final static String OS_TYPE_IBM                         = "IBM";
public final static String OS_VERS_IBM                         = "Data ONTAP";
public final static String OS_TYPE_NETAPP                      = "NetApp";
public final static String OS_VERS_NETAPP                      = "NetApp";

/* Strings for the errors */
public final static String[] ErrStrings = { "NDMP_NO_ERR",
									 "NDMP_NOT_SUPPORTED_ERR",
									 "NDMP_DEVICE_BUSY_ERR",
									 "NDMP_DEVICE_OPENED_ERR",
									 "NDMP_NOT_AUTHORIZED_ERR",
									 "NDMP_PERMISSION_ERR",
									 "NDMP_DEV_NOT_OPEN_ERR",
									 "NDMP_IO_ERR",
									 "NDMP_TIMEOUT_ERR",
									 "NDMP_ILLEGAL_ARGS_ERR",
									 "NDMP_NO_TAPE_LOADED_ERR",
									 "NDMP_WRITE_PROTECT_ERR",
									 "NDMP_EOF_ERR",
									 "NDMP_EOM_ERR",
									 "NDMP_FILE_NOT_FOUND_ERR",
									 "NDMP_BAD_FILE_ERR",
									 "NDMP_NO_DEVICE_ERR",
									 "NDMP_NO_BUS_ERR",
									 "NDMP_XDR_DECODE_ERR",
									 "NDMP_ILLEGAL_STATE_ERR",
									 "NDMP_UNDEFINED_ERR",
									 "NDMP_XDR_ENCODE_ERR",
									 "NDMP_NO_MEM_ERR",
									 "NDMP_CONNECT_ERR" };

public final static String	ProtocolName 	= "Ndmp";
public final static String  NameDelimiter 	= "_";
public final static String	ClassName		= "Message";


public final static int		MAX_RETRY_COUNT		= 50;

/**
 * A Note on the Naming convention:
 * These names are also used for creating an Ndmp Message class
 * and also in printing the Name of an Ndmp message in debug Log.
 *
 *  <Ndmp><Delimiter><Interface Name><Delimiter><Message Name> 
 * In case of a class Name (Ndmp Message class Name) the 
 * value of delimiter is "" (nothing), while in case we print it in
 * a debug log file the Delimiter is "_", just to ease in reading.
 *
 * The Names : First Letter caps for all the words in the spec after
 * a '_', and the rest are small till a next '_' comes..
 * 
 * e.g. If spec defines something lke NDMP_CONFIG_GET_FS_INFO
 *  ==> NdmpConfigGetFsInfo.
 * Remove '_' and change the case of first letter after '_'.
 */
 
public final static String[] InterfaceName = {
								"Config",	//0x1
								"Scsi",		//0x2
								"Tape",		//0x3
								"Data",		//0x4
								"Notify",	//0x5
								"Log",		//0x6
								"Fh",		//0x7
								"",			//0x8 reserved
								"Connect",	//0x9
								"Mover"};	//0xA

public final static String[][] MsgName = {
					{"GetHostInfo", "",	//0x101 res.
					"GetConnectionType", "GetAuthAttr", 
					"GetBuTypeInfo", "GetFsInfo",
					"GetTapeInfo", "GetScsiInfo",
					"GetServerInfo", },
					
					//Scsi
					{"Open", "Close", "GetState", "SetTarget",
					"ResetDevice", "ResetBus", "ExecuteCdb" },
					
					//Tape
					{"Open", "Close", "GetState", "Mtio",
					"Write", "Read", "", "ExecuteCdb" },

					//Data
					{"GetState", "StartBackup", "StartRecover",
					"Abort", "GetEnv", "", "", "Stop", "",
					"Listen", "Connect" },

					//Notify
					{"", "DataHalted", "Connected", "MoverHalted",
					"MoverPaused", "DataRead"  },

					//Log
					{"", "", "Files", "Message" },

					//File
					{"", "", "", "AddFile", "AddDir", "AddNode" },

					//reserverd
					{"" },

					//Connect
					{"Open", "ClientAuth", "Close", "ServerAuth" },

					//Mover
					{"GetState", "Listen", "Continue", "Abort", "Stop",
					"SetWindow", "Read", "Close", "SetRecordSize",
					"Connect" }
					};


public final static String[] MsgTypeName = {
					"Request",		//NDMP_MESSAGE_REQUEST
					"RReply"		//NDMP_MESSAGE_REPLY
					};

/*
 * Interface Messages
 */

/* Connect Interface */
public final static int NDMP_CONNECT_OPEN			= 0x900;
public final static int NDMP_CONNECT_CLIENT_AUTH	= 0x901;
public final static int NDMP_CONNECT_AUTH			= 0x901;
public final static int NDMP_CONNECT_CLOSE			= 0x902;
public final static int NDMP_CONNECT_SERVER_AUTH	= 0x903;

/* Config Interface */
public final static int NDMP_CONFIG_GET_HOST_INFO			= 0x100;
public final static int NDMP_CONFIG_GET_BUTYPE_ATTR			= 0x101;
public final static int NDMP_CONFIG_GET_CONNECTION_TYPE 	= 0x102;	// V3
public final static int NDMP_CONFIG_GET_MOVER_TYPE			= 0x102;	// V2
public final static int NDMP_CONFIG_GET_AUTH_TYPE_ATTR		= 0x103;
public final static int NDMP_CONFIG_GET_BUTYPE_INFO			= 0x104;
public final static int NDMP_CONFIG_GET_FS_INFO				= 0x105;
public final static int NDMP_CONFIG_GET_TAPE_INFO			= 0x106;
public final static int NDMP_CONFIG_GET_SCSI_INFO			= 0x107;
public final static int NDMP_CONFIG_GET_SERVER_INFO			= 0x108;

/* SCSI Interface */
public final static int NDMP_SCSI_OPEN				= 0x200;
public final static int NDMP_SCSI_CLOSE				= 0x201;
public final static int NDMP_SCSI_GET_STATE			= 0x202;
public final static int NDMP_SCSI_SET_TARGET		= 0x203;
public final static int NDMP_SCSI_RESET_DEVICE		= 0x204;
public final static int NDMP_SCSI_RESET_BUS			= 0x205;
public final static int NDMP_SCSI_EXECUTE_CDB		= 0x206;

/* Tape Interface */
public final static int NDMP_TAPE_OPEN				= 0x300;
public final static int NDMP_TAPE_CLOSE				= 0x301;
public final static int NDMP_TAPE_GET_STATE			= 0x302;
public final static int NDMP_TAPE_MTIO				= 0x303;
public final static int NDMP_TAPE_WRITE				= 0x304;
public final static int NDMP_TAPE_READ				= 0x305;
public final static int NDMP_TAPE_RESVD1			= 0x306;
public final static int NDMP_TAPE_SET_RECORD_SIZE	= 0x306;
public final static int NDMP_TAPE_EXECUTE_CDB		= 0x307;

/* Data Interface */
public final static int NDMP_DATA_GET_STATE			= 0x400;
public final static int NDMP_DATA_START_BACKUP		= 0x401;
public final static int NDMP_DATA_START_RECOVER		= 0x402;
public final static int NDMP_DATA_ABORT				= 0x403;
public final static int NDMP_DATA_GET_ENV			= 0x404;
public final static int NDMP_DATA_RESVD1			= 0x405;
public final static int NDMP_DATA_RESVD2			= 0x406;
public final static int NDMP_DATA_STOP				= 0x407;
public final static int NDMP_DATA_CONTINUE			= 0x408;	
public final static int NDMP_DATA_LISTEN			= 0x409;
public final static int NDMP_DATA_CONNECT			= 0x40a;

/* Notify Interface */
public final static int NDMP_NOTIFY_PAUSED			= 0x500;	// V1 only 
public final static int NDMP_NOTIFY_DATA_HALTED		= 0x501;
public final static int NDMP_NOTIFY_CONNECTED		= 0x502;
public final static int NDMP_NOTIFY_MOVER_HALTED	= 0x503;
public final static int NDMP_NOTIFY_MOVER_PAUSED	= 0x504;
public final static int NDMP_NOTIFY_DATA_READ		= 0x505;

/* Log Interface */
public final static int NDMP_LOG_LOG				= 0x600;
public final static int NDMP_LOG_DEBUG				= 0x601;
public final static int NDMP_LOG_FILE				= 0x602;
public final static int NDMP_LOG_MESSAGE			= 0x603;

/* File History Interface */
public final static int NDMP_FH_ADD_UNIX_PATH		= 0x700;
public final static int NDMP_FH_ADD_UNIX_DIR		= 0x701;
public final static int NDMP_FH_ADD_UNIX_NODE		= 0x702;
public final static int NDMP_FH_ADD_FILE			= 0x703;
public final static int NDMP_FH_ADD_DIR				= 0x704;
public final static int NDMP_FH_ADD_NODE			= 0x705;

/* Mover Interface */
public final static int NDMP_MOVER_GET_STATE		= 0xa00;
public final static int NDMP_MOVER_LISTEN			= 0xa01;
public final static int NDMP_MOVER_CONTINUE			= 0xa02;
public final static int NDMP_MOVER_ABORT			= 0xa03;
public final static int NDMP_MOVER_STOP 			= 0xa04;
public final static int NDMP_MOVER_SET_WINDOW 		= 0xa05;
public final static int NDMP_MOVER_READ 			= 0xa06;
public final static int NDMP_MOVER_CLOSE 			= 0xa07;
public final static int NDMP_MOVER_SET_RECORD_SIZE	= 0xa08;
public final static int NDMP_MOVER_CONNECT			= 0xa09;

/* Reserved for the vendor specific usage */
public final static int NDMP_VENDORS_BASE			= 0xf000;

/* Reserved for Prototyping */
public final static int NDMP_RESERVED				= 0xff00;


/* NDMP Connect Constants */
public final static int NDMP_CONNECTED				= 0;
public final static int NDMP_SHUTDOWN				= 1;
public final static int NDMP_REFUSED				= 2;

public final static int NDMP_AUTH_NONE				= 0;
public final static int NDMP_AUTH_TEXT				= 1;
public final static int NDMP_AUTH_MD5				= 2;

public final static int NDMP_AUTH_DIGEST_LEN		= 16;
public final static int NDMP_PASSWD_SIGNIFICANT_LEN     = 16;

/* NDMP Config Constants */
public final static int NDMP_NO_BACKUP_FILELIST		= 0x0001;
public final static int NDMP_NO_BACKUP_FHINFO		= 0x0002;
public final static int NDMP_NO_RECOVER_FILELIST	= 0x0004;
public final static int NDMP_NO_RECOVER_FHINFO		= 0x0008;
public final static int NDMP_NO_RECOVER_RESVD		= 0x0010;
public final static int NDMP_NO_RECOVER_INC_ONLY	= 0x0020;

public final static int NDMP_BUTYPE_BACKUP_FILE_HISTORY		= 0x00000001;
public final static int NDMP_BUTYPE_BACKUP_FILELIST			= 0x00000002;
public final static int NDMP_BUTYPE_RECOVER_FILELIST		= 0x00000004;
public final static int NDMP_BUTYPE_BACKUP_DIRECT			= 0x00000008;
public final static int NDMP_BUTYPE_RECOVER_DIRECT			= 0x00000010;
public final static int NDMP_BUTYPE_BACKUP_INCREMENTAL		= 0x00000020;
public final static int NDMP_BUTYPE_RECOVER_INCREMENTAL		= 0x00000040;
public final static int NDMP_BUTYPE_BACKUP_UTF8				= 0x00000080;
public final static int NDMP_BUTYPE_RECOVER_UTF8			= 0x00000100;

public final static int NDMP_FS_INFO_TOTAL_SIZE_INVALID		= 0x00000001;
public final static int NDMP_FS_INFO_USED_SIZE_INVALID		= 0x00000002;
public final static int NDMP_FS_INFO_AVAIL_SIZE_INVALID		= 0x00000004;
public final static int NDMP_FS_INFO_TOTAL_INODES_INVALID	= 0x00000008;
public final static int NDMP_FS_INFO_USED_INODES_INVALID	= 0x00000010;

public final static int NDMP_TAPE_ATTR_REWIND				= 1;
public final static int NDMP_TAPE_ATTR_UNLOAD				= 2;

/* NDMP Data Constants */
public final static int NDMP_DATA_STATE_IDLE				= 0;
public final static int NDMP_DATA_STATE_ACTIVE				= 1;
public final static int NDMP_DATA_STATE_HALTED				= 2;
public final static int NDMP_DATA_STATE_LISTEN				= 3;
public final static int NDMP_DATA_STATE_CONNECTED			= 4;
public final static int NDMP_DATA_STATE_PAUSED_v1			= 2;	// V1 only
public final static int NDMP_DATA_STATE_HALTED_v1			= 3;	// V1 only

public final static int NDMP_DATA_STATE_EST_BYTES_REMAIN	= 0x00000001;
public final static int NDMP_DATA_STATE_EST_TIME_REMAIN		= 0x00000002;

public final static int NDMP_DATA_OP_NOACTION				= 0;
public final static int NDMP_DATA_OP_BACKUP					= 1;
public final static int NDMP_DATA_OP_RESTORE				= 2;

public final static int NDMP_DATA_HALT_NA					= 0;
public final static int NDMP_DATA_HALT_SUCCESSFUL 			= 1;
public final static int NDMP_DATA_HALT_ABORTED				= 2;
public final static int NDMP_DATA_HALT_INTERNAL_ERROR		= 3;
public final static int NDMP_DATA_HALT_CONNECT_ERROR 		= 4;
public final static int NDMP_DATA_HALT_MEDIA_ERROR_v1		= 3;	// V1 only
public final static int NDMP_DATA_HALT_INTERNAL_ERROR_v1	= 4;	// V1 only

/* NDMP Mover Constants */
public final static int NDMP_MOVER_STATE_IDLE		= 0;
public final static int NDMP_MOVER_STATE_LISTEN		= 1;
public final static int NDMP_MOVER_STATE_ACTIVE		= 2;
public final static int NDMP_MOVER_STATE_PAUSED		= 3;
public final static int NDMP_MOVER_STATE_HALTED		= 4;

public final static int NDMP_MOVER_PAUSE_NA				= 0;
public final static int NDMP_MOVER_PAUSE_EOM			= 1;
public final static int NDMP_MOVER_PAUSE_EOF			= 2;
public final static int NDMP_MOVER_PAUSE_SEEK			= 3;
public final static int NDMP_MOVER_PAUSE_MEDIA_ERROR	= 4;
public final static int NDMP_MOVER_PAUSE_EOW			= 5;

public final static int NDMP_MOVER_HALT_NA					= 0;
public final static int NDMP_MOVER_HALT_CONNECTION_CLOSED	= 1;
public final static int NDMP_MOVER_HALT_ABORTED				= 2;
public final static int NDMP_MOVER_HALT_INTERNAL_ERROR		= 3;
public final static int NDMP_MOVER_HALT_CONNECT_ERROR		= 4;

public final static int NDMP_MOVER_MODE_READ			= 0;
public final static int NDMP_MOVER_MODE_WRITE			= 1;
public final static int NDMP_MOVER_MODE_DATA			= 2;

public final static int NDMP_ADDR_LOCAL				= 0;
public final static int NDMP_ADDR_TCP				= 1;
public final static int NDMP_ADDR_FC				= 2;
public final static int NDMP_ADDR_IPC				= 3;

/* NDMP SCSI Constants */
public final static int NDMP_SCSI_DATA_IN		= 1;
public final static int NDMP_SCSI_DATA_OUT		= 2;

public final static int NDMP_SCSI_CDB_TIMEOUT  = 900; // Max scsi cdb timeout in sec

/* NDMP Tape Constants */
public final static int NDMP_TAPE_NOREWIND							= 0x0008;
public final static int NDMP_TAPE_WR_PROT							= 0x0010;
public final static int NDMP_TAPE_ERROR								= 0x0020;
public final static int NDMP_TAPE_UNLOAD							= 0x0040;

// Time out for Tape CDB in secs.
public final static int NDMP_TAPE_CDB_TIMEOUT					= 900;

public final static int NDMP_TAPE_READ_MODE						= 0;
public final static int NDMP_TAPE_RDWR_MODE						= 1;

/* 
 * Open up the tape drive regardless of whether there is a tape in there 
 * This is not in the official protocol, so I chose a number where we
 * won't reuse it
 */
public final static int NDMP_TAPE_NDELAY_MODE					= 0x7fffffff;

public final static int NDMP_TAPE_STATE_NUM_INVALID				= 0x00000001;
public final static int NDMP_TAPE_STATE_SOFT_ERRORS_INVALID		= 0x00000002;
public final static int NDMP_TAPE_STATE_BLOCK_SIZE_INVALID		= 0x00000004;
public final static int NDMP_TAPE_STATE_BLOCKNO_INVALID			= 0x00000008;
public final static int NDMP_TAPE_STATE_TOTAL_SPACE_INVALID		= 0x00000010;
public final static int NDMP_TAPE_STATE_SPACE_REMAIN_INVALID	= 0x00000020;
public final static int NDMP_TAPE_STATE_PARTITION_INVALID		= 0x00000040;
public final static int NDMP_TAPE_STATE_NOREWIND				= 0x0008;
public final static int NDMP_TAPE_STATE_WR_PROT					= 0x0010;
public final static int NDMP_TAPE_STATE_ERROR					= 0x0020;
public final static int NDMP_TAPE_STATE_UNLOAD					= 0x0040;

public final static int NDMP_MTIO_FSF							= 0x00000000;
public final static int NDMP_MTIO_BSF							= 0x00000001;
public final static int NDMP_MTIO_FSR							= 0x00000002;
public final static int NDMP_MTIO_BSR							= 0x00000003;
public final static int NDMP_MTIO_REW							= 0x00000004;
public final static int NDMP_MTIO_EOF							= 0x00000005;
public final static int NDMP_MTIO_OFF							= 0x00000006;

// Implementation specific constants
public final static String NDMP_VENDOR_NAME		= "Network Appliance";
public final static String NDMP_PRODUCT_NAME = "Network Appliance NDMP server";
public final static String NDMP_REVISION_NUMBER	= "3.0";
public final static String NDMP_VERSION			= "NDMP_VERSION";
public final static int[] NDMP_AUTH_TYPES		= { NDMP_AUTH_TEXT, 
													NDMP_AUTH_MD5 };

public final static int[] NDMP_ADDR_TYPES_V1		= { NDMP_ADDR_LOCAL };
public final static int[] NDMP_ADDR_TYPES_V2		= { NDMP_ADDR_LOCAL, 
														NDMP_ADDR_TCP };
public final static int[] NDMP_ADDR_TYPES_V3		= { NDMP_ADDR_LOCAL, 
														NDMP_ADDR_TCP };

public final static int NDMP_PORT_NUMBER			= 10000;
public final static String NDMP_DEBUG_PATH			= "/etc/log/";
public final static String NDMP_DUMPLETS_PATH		= "/etc/java/ndmpd/";
public final static int NDMP_TAPE_RECORD_SIZE_MAX	= 1048576 * 2;	// 2MB
public final static int NDMP_RECORDSIZE_NOT_SET		= 0;
public final static short NDMP_DEFAULT_VERSION		= 3;

public final static short NDMP_MAX_VERSION			= 4;
public final static	short NDMP_MIN_VERSION			= 3;


public final static short TP_BSIZE				= 1024;	// Tape Block Size

public final static int	MAX_PATH_SIZE			= 1024;

public final static int MAX_NAME_SIZE			= 64;

public final static String DMP_NAME_PREFIX 		= "ndmpcopy:";
public final static String EXCLUDE_PATH_SEPARATOR	= "\n";

/**
 * Turn this to true, for ndmpcopy console debug logs.
 * Only on DEBUG builds.
 */
public final static boolean NDMPCOPY_DEBUG		= false;	
public final static char NULL_DELIM = 0;	


/**
 * Helper functions to convert from longlong to ndmp_u_quad (the ndmpd
 * representation to handle 64 bit numbers)
 */
public static int getHighByte (long val) {
	return (new Long(val >> 32)).intValue() & 0xffffffff;
}
public static  int getLowByte (long val) {
	return ((new Long(val)).intValue()) & 0xffffffff;
}
public static long toLong (int high, int low) {
	return ((((new Integer(high)).longValue() << 32) & ((long)0xffffffff << 32))
	| (((new Integer(low)).longValue() << 32 >>> 32) & (long)0xffffffff));
}

public static String msgError (int error) {
	try {
		return ErrStrings[error];
	} catch (ArrayIndexOutOfBoundsException e) {
		return "invalid/unrecognized error" ;
	}
}

public static String msgName (int method, int error) {
	try {
		return (ProtocolName + NameDelimiter + InterfaceName [(method >>8) - 1] +
			NameDelimiter + MsgName [(method >> 8) - 1][(method & 0x00FF)]);
	} catch (ArrayIndexOutOfBoundsException e) {
		// Check for Vendor Spec. Messages....
		return "invalid/unrecognized message name";
	}
}

public static String msgClassName (int method) {
	try {
		return (ProtocolName + ClassName + InterfaceName [(method >>8) - 1] +
			MsgName [(method >> 8) - 1][(method & 0x00FF)]);
	} catch (ArrayIndexOutOfBoundsException e) {
		return "invalid/unrecognized message classname";
	}
}

public static String msgClassName (int method, int msgtype) {
	return (msgClassName (method) + msgClassTypeName (msgtype));
}

public static String msgClassTypeName (int msgtype) {
	try {
		return MsgTypeName [msgtype];
	} catch (ArrayIndexOutOfBoundsException e) {
		return "invalid/unrecognized message type";
	}
}	

public static String msgInterfaceName (int method) {
	try {
		return InterfaceName [(method >> 8) - 1];
	}
	catch (ArrayIndexOutOfBoundsException e) {
		return "invalid/unrecognized message interface";
	}
}
}
