/*
 * $Id: //depot/prod/ontap/main/prod/common/java/netapp/onc/rpc/XDRRMOutputStream.java#2 $
 *
 * Copyright (c) 1998 Network Appliance, Inc. All Rights Reserved.
 */

package com.netapp.offtap.ndmpcopy.xdr.rpc;

import java.io.IOException;
import java.io.OutputStream;

import com.netapp.offtap.ndmpcopy.xdr.xdr.XDROutputStream;


/**
 * General purpose marshal/unmashal filter based on ONC-XDR encodings.
 */
public class XDRRMOutputStream extends XDROutputStream {
	
	private byte[]	markbuffer;		// XDR record marker buffer
	public XDRRMOutputStream (OutputStream outs) {
		super(outs);
		markbuffer = new byte[4];
	}

	public XDRRMOutputStream (OutputStream outs, int bufsize) {
		super(outs, bufsize);
		markbuffer = new byte[4];
	}

		
	/**
	 * Mark a fragment that we are sending. 
	 */
	private void writeMarker(int last, int val) throws IOException {
		if (last != 0) {
			val |= 0x80000000;
		}
		markbuffer[0] = (byte)((val >> 24) & 0xff);
		markbuffer[1] = (byte)((val >> 16) & 0xff);
		markbuffer[2] = (byte)((val >> 8) & 0xff);
		markbuffer[3] = (byte)((val >> 0) & 0xff);
		out.write(markbuffer, 0, 4);	// put out marker
	}

	/**
	 * Mark the start of a record
	 * Tells this module that we are starting a new
	 * record so the record marking code will know
	 * how much is part of this record.
	 */
	public void markStartRec() throws IOException {
		if (index != 0)
			markEndRec();	// the last of the fragment remaining
	}

	/**
	 * Mark the end of a record
	 * This is called when the caller has finished sending
	 * a record and desires to send the record.  We have the
	 * remainder of this record in buffer and we need to put
	 * the marker on the front and send it out.  If we have
	 * fragmented it, flushBuf would have been called and
	 * it would have put out a fragment.  The remainder here
	 * is the last part of the fragment.
	 */
	public void markEndRec() throws IOException {
		writeMarker(1, index);
		out.write(outbuffer, 0, index);	// put out data
		index = 0;
	}

	/**
	 * Flush the output stream, and reset the index
	 * This is only called when we end up fragmenting
	 * a record.  This routine will always mark this
	 * chunk as not-the-last of a fragment.
	 */
	protected void flushBuf() throws IOException {
		writeMarker(0, index);
		out.write(outbuffer, 0, index);
		index = 0;
	}
}
