/*
 * $Id: //depot/prod/ontap/main/prod/common/java/netapp/onc/xdr/XDRInputStream.java#2 $
 *
 * Copyright (c) 1998 Network Appliance, Inc. All Rights Reserved.
 */

package com.netapp.offtap.ndmpcopy.xdr.xdr;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * General purpose marshal/unmashal filter based on ONC-XDR encodings.
 */
public class XDRInputStream {
	private BufferedInputStream 	input;		// Whence come the bytes
	private int 			BufSize = 4096; // Default read size


	public XDRInputStream(InputStream stream) {
		input = new BufferedInputStream(stream, BufSize);
	}

	public XDRInputStream(InputStream stream, int bufsize) {
		BufSize = bufsize;
		input = new BufferedInputStream(stream, BufSize);
	}
		
	/**
	 * Closes off the input stream
	 */
	public void close () throws IOException {
		input.close();
	}

	protected void readData(byte[] buf, int offset, int num_bytes) 
	    throws IOException 
	{
		int	bytes_read, bytes_left;

		bytes_left = num_bytes;

		while (bytes_left != 0) {
			bytes_read = input.read(buf, offset, bytes_left);
			if (bytes_read == -1)
				throw new IOException("Socket closed.");

			offset += bytes_read; 
			bytes_left -= bytes_read;
		}
	}


	/**
	 * Read in the next string of bytes from the buffer
	 */
	public byte[] getBytes(int byte_len) throws IOException {
		int	bytes_read, toss_bytes;
		byte[]	buffer;
		byte[]	toss_buffer;

		if (byte_len == 0)
			return null;

		// Get array of bytes
		buffer = new byte[byte_len];
		readData(buffer, 0, byte_len);


		// Make sure we're on a XDR UNIT
		toss_bytes = XDRConvert.toAlignBytes(byte_len);
		if (toss_bytes != 0) {
			toss_buffer = new byte[toss_bytes];
			readData(toss_buffer, 0, toss_bytes);
		}

		return (buffer);
	}
		
	/**
	 * Read in a long from the marshal buffer
	 */ 
	public int getLong() throws IOException {
		byte[]	buffer = new byte[XDRCons.XDR_LONG_BYTES];

		readData(buffer, 0, XDRCons.XDR_LONG_BYTES);
		return (XDRConvert.toLong(buffer));
	}

	/**
	 * Read in a hyper from the marshal buffer
	 */ 
	public long getHyper() throws IOException {
		byte[]	buffer = new byte[XDRCons.XDR_HYPER_BYTES];
	
		readData(buffer, 0, XDRCons.XDR_HYPER_BYTES);
		return (XDRConvert.toHyper(buffer));
	}
			

	/**
	 * Read in a short from the marshal buffer
	 */ 
	public short getShort() throws IOException {
		byte[]	buffer = new byte[XDRCons.XDR_SHORT_BYTES];

		readData(buffer, 0, XDRCons.XDR_SHORT_BYTES);
		return (XDRConvert.toShort(buffer));
	}

	/**
	 * Read in a byte from the marshal buffer
	 */ 
	public byte getByte() throws IOException {
		byte[]	buffer = new byte[XDRCons.XDR_BYTE_BYTES];

		readData(buffer, 0, XDRCons.XDR_BYTE_BYTES);

		return (XDRConvert.toByte(buffer));
	}
}
