/*
 * $Id: //depot/prod/ontap/main/prod/common/java/netapp/onc/xdr/XDROutputStream.java#3 $
 *
 * Copyright (c) 1998 Network Appliance, Inc. All Rights Reserved.
 */

package com.netapp.offtap.ndmpcopy.xdr.xdr;

import java.io.IOException;
import java.io.OutputStream;

/**
 * General purpose marshal/unmashal filter based on ONC-XDR encodings.
 */
public class XDROutputStream {
	protected OutputStream 	out;		// Gotta send the XDR somewhere
	protected byte 		outbuffer[];	// XDR buffer
	protected int 		index = 0;
	protected int 		BufSize = 256;	// Default size

	public XDROutputStream(OutputStream outs) 
	{
		out = outs;
		outbuffer = new byte[BufSize];
	}

	public XDROutputStream (OutputStream outs, int bufsize) 
	{
		out = outs;
		BufSize = bufsize;
		outbuffer = new byte[BufSize];
	}

	/**
	 * Closes off the stream
	 */
	public void close () throws IOException {
		out.close();
	}

	/**
	 * Return the amount of data in the current marshalled stream
	 */
	public int length() 
	{
		return (index);
	}

	/**
	 * Return the amount of space available in the buffer
	 */
	public int spaceLeft() 
	{
		return outbuffer.length - index;
	}

	/**
	 * Skip ahead this many bytes in the marhsal buffer
	 * (changes the length() of the buffer as side effect)
	 */
	public void skip(int n) throws IOException 
	{
		while ((index + n) > outbuffer.length) {
			n -= (outbuffer.length - index);
			index = outbuffer.length;
			flushBuf(); 
		}
		index += n;
	}

	/**
	 * Return the raw XDR marhsalled buffer.
	 */
	public byte[] getBuf() 
	{
		return (outbuffer);
	}

		
	/**
	 * Reset out index pointer
	 */
	public void reset() 
	{
		index = 0;
	}


        public void putBool(boolean val) throws IOException 
	{
        	if (val)
                	putLong(1);
                else
                	putLong(0);
        }

	/**
	 * Put a stream of bytes to marshal buffer
	 */
	public void putBytes(byte val[]) throws IOException 
	{
		if (val == null)
			putBytes(val, 0);
		else
			putBytes(val, val.length);
	}

	/**
	 * Put a stream of bytes to marshal buffer
	 */
	public void putBytes(byte val[], int len) throws IOException 
	{

		putLong(len);
		putFixedLenBytes(val, len);
	}

	public void putFixedLenBytes(byte val[], int len) throws IOException
	{
		int 	retval, nwritten, max_to_write;

		nwritten = 0;
		while (nwritten < len) { 
			//Copy an XDR allowable subset of data
			retval = XDRConvert.fromBytesPart(val, nwritten, 
			    (len - nwritten), outbuffer, index);
			index += retval;
			nwritten += retval;
			// We've filled the buffer to brim...
			if (nwritten < len)
				flushBuf();
		}
	}
		

	/**
	 * Put a network hyper (64 bits) to the marshal buffer
	 */
	public void putHyper(long val) throws IOException {
		if ((index + XDRCons.XDR_HYPER_BYTES) > outbuffer.length)
			flushBuf();
		index = XDRConvert.fromHyper(val, outbuffer, index);
	}

	/**
	 * Put a network long to the marshal buffer
	 */
	public void putLong(int val) throws IOException {
		if ((index + XDRCons.XDR_LONG_BYTES) > outbuffer.length)
			flushBuf();
		index = XDRConvert.fromLong(val, outbuffer, index);
	}

	/**
	 * Put a network short to the marshal buffer
	 */
	public void putShort(short val) throws IOException {
		if ((index + XDRCons.XDR_SHORT_BYTES) > outbuffer.length)
			flushBuf();
		index = XDRConvert.fromShort(val, outbuffer, index);
	}

	/**
	 * Put a byte out to the marshal buffer
	 */
	public void putByte(byte val) throws IOException {
		if ((index + XDRCons.XDR_BYTE_BYTES) > outbuffer.length)
			flushBuf();
		index = XDRConvert.fromByte(val, outbuffer, index);
	}

	/**
	 * Put a XDR string to the marshal buffer
	 */
	public void putString(String s) throws IOException {
		putBytes(s.getBytes());
	}

        /**
         * align the output buffer
         */
        public void align() throws IOException {
		switch (index % XDRCons.XDR_ALIGN) {
		case 1:
			outbuffer[index++] = 0;
                        // fall through
		case 2:
			outbuffer[index++] = 0;
                        // fall through
		case 3:
			outbuffer[index++] = 0;
                        // fall through
		default:
			break;
		}
        }

	/**
	 * Flush the output stream, and reset the index
	 */
	protected void flushBuf() throws IOException {
		// Tell our OutputStream to write out this buffer
		out.write(outbuffer, 0, index);
		index = 0;
	}
}
