package com.netapp.offtap3;

import java.util.Arrays;
import java.util.EnumSet;
import java.util.Set;

public enum ControllerResource implements Resource {
    // these first two are special:
    // they have very basic information required for functional controllers
    // and so they will always be loaded and always loaded before other resources start loading
    system_info,
    licenses,

    // common
    protocols,
    aggregates,
    system_model,
    zapi_access,
    volume_maximums,

    interfaces(StorageSystem.adminsvm, StorageSystem.datasvm, StorageSystem.pfiler, StorageSystem.vfiler),

    space_usage(StorageSystem.DATA_SERVING),
    cifs_shares(StorageSystem.DATA_SERVING),
    volumes(StorageSystem.DATA_SERVING),
    luns(StorageSystem.DATA_SERVING),
    iscsi(StorageSystem.DATA_SERVING),
    igroups(StorageSystem.DATA_SERVING),
    snapmirror(StorageSystem.DATA_SERVING),
    root_volume(StorageSystem.DATA_SERVING),
    vstorage_status(StorageSystem.DATA_SERVING),

    port_sets(StorageSystem.datasvm, StorageSystem.pfiler),

    // cDOT specific
    svms(StorageSystem.adminsvm),
    cluster_status(StorageSystem.adminsvm),
    cluster_state_reason(StorageSystem.adminsvm),
    cluster_peers(StorageSystem.adminsvm),
    svm_peers(StorageSystem.datasvm),
    qos(StorageSystem.datasvm),
    lun_bindings(StorageSystem.datasvm),
    export_policies(StorageSystem.datasvm),
    unix_users_groups(StorageSystem.datasvm),

    // 7-mode specific
    vfiler_status(StorageSystem.vfiler),
    vfilers(StorageSystem.pfiler, StorageSystem.vfiler),
    global_status(StorageSystem.pfiler),
    primary_snapvaults(StorageSystem.pfiler, StorageSystem.vfiler),
    secondary_snapvaults(StorageSystem.pfiler, StorageSystem.vfiler),

    cf_info(StorageSystem.pfiler, StorageSystem.vfiler, StorageSystem.nodesvm),

    flash_cache(StorageSystem.nodesvm),

    snmp(StorageSystem.pfiler, StorageSystem.adminsvm),
    asup(StorageSystem.pfiler, StorageSystem.nodesvm),
    fcp(StorageSystem.pfiler, StorageSystem.vfiler, StorageSystem.adminsvm, StorageSystem.nodesvm);

    private final EnumSet<StorageSystem> supportedPlatforms;

    ControllerResource() {
        supportedPlatforms = EnumSet.allOf(StorageSystem.class);
    }

    ControllerResource(StorageSystem... supports) {
        supportedPlatforms = EnumSet.copyOf(Arrays.asList(supports));
    }

    ControllerResource(EnumSet<StorageSystem> supported) { supportedPlatforms = supported; }

    public boolean supports(Set<StorageSystem> type) {
        return supportedPlatforms.containsAll(type);
    }
}
