package com.netapp.offtap3;

import java.util.List;

public interface File extends Refreshable, HasLoader {

    /**
     * Returns the full path of the file
     *
     * @return the full path of the file
     */
    String getFullPath();

    String read(long offset, long length) throws FileOperationException;
    String read(long offset, long length, String streamName) throws FileOperationException;
    String readBin(long offset, long length) throws FileOperationException;

    void write(String data) throws FileOperationException;
    void write(String data, long offset, boolean overwrite) throws FileOperationException;
    void write(String data, long offset, boolean overwrite, String streamName) throws FileOperationException;

    void write(String fileData, long offset, boolean overwrite, boolean update) throws FileOperationException;
    void write(String fileData, long offset, boolean overwrite, boolean update, String streamName) throws FileOperationException;

    void setFileReservation(boolean on) throws FileOperationException;

    List<? extends File> listFiles() throws FileOperationException;

    File createFile(String relativePath);

    void mkdir() throws FileOperationException;
    void mkdir(String permissions) throws FileOperationException;

    void rmdir() throws FileOperationException;
    void rmdir(boolean removeContents) throws FileOperationException;

    void delete() throws FileOperationException;

    File getParentDir();

    void delete(boolean isCloneFile) throws FileOperationException;

    void truncate(Size size) throws FileOperationException;

    String reclaimSpace() throws FileOperationException;

    void move(File newLocation) throws FileOperationException;

    void copy(File newLocation) throws FileOperationException;

    /**
     * @return the base name of the file
     */
    @FileLoader(FileResource.file_metadata)
    String getName();

    /**
     * @return the ACL type of the file
     */
    @FileLoader(FileResource.file_metadata)
    String getAclType();

    /**
     * @return the file type of the file
     */
    @FileLoader(FileResource.file_metadata)
    String getFileType();

    /**
     * @return the accessed timestamp of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getAccessedTimestamp();

    /**
     * @return the changed timestamp of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getChangedTimestamp();

    /**
     * @return the creation timestamp of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getCreationTimestamp();

    /**
     * @return the file size of the file
     */
    @FileLoader(FileResource.file_metadata)
    Size getFileSize();

    /**
     * @return the group ID of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getGroupID();

    /**
     * @return the number of hard links of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getHardLinksCount();

    /**
     * @return the inode number of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getInodeNumber();

    /**
     * @return the modified timestamp of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getModifiedTimestamp();

    /**
     * @return the owner ID of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getOwnerId();

    /**
     * @return the file permission bits of the file
     */
    @FileLoader(FileResource.file_metadata)
    int getPerm();

    @FileLoader(FileResource.file_metadata)
    Size getUsed();

    long getUniqueBytes() throws FileOperationException;

    @FileLoader(FileResource.file_metadata)
    boolean isDirectory() throws FileOperationException;

    // don't cache this with a loader.  run every time.
    boolean exists();

    @Override
    File refresh();
}
