/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/HasUnixUsersAndGroups.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3;

import java.util.Map;

public interface HasUnixUsersAndGroups {
    Map<String, UnixGroup> getUnixGroups();
    Map<String, UnixUser> getUnixUsers();

    /**
     * Creates a new unix group.
     * @param name The name of the group.
     * @param id The id of the group.
     * @return The group object that was just created.
     * @throws UserAndGroupOperationException If the name or id already exists or there is some other failure.
     */
    UnixGroup createUnixGroup(String name, int id) throws UserAndGroupOperationException;

    /**
     * Creates a new unix user. A default private group with the same name as the user and the closest available id
     * will automatically be created.
     * @param name The name of the user.
     * @param id The id of the user.
     * @return The user object that was just created.
     * @throws UserAndGroupOperationException If the name or id already exist, a group with this name already exists,
     * or some other failure.
     */
    UnixUser createUnixUser(String name, int id) throws UserAndGroupOperationException;

    /**
     * Creates a new unix user with the specified group as its primary group.
     * @param name The name of the user.
     * @param id The id of the user.
     * @param group The primary group for the new user.
     * @return The user object that was just created.
     * @throws UserAndGroupOperationException If the name or id already exists or there is some other failure.
     */
    UnixUser createUnixUser(String name, int id, UnixGroup group) throws UserAndGroupOperationException;
}
