/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3;

/**
 * Inteface to indicate that this controller may support the VMWare vStorage API. Because some controllers may only
 * support vStorage at certain versions, it's possible that a controller that implements this interface does not
 * actually support the vStorage APIs.
 */
public interface HasVStorageSupport {
    /**
     * Returns true if this controller supports the VMWare vStorage API and it is enabled. Returns false otherwise.
     */
    @ControllerLoader(ControllerResource.vstorage_status)
    boolean isVStorageApiEnabled();

    /**
     * Returns true if this controller provides support for the VMWare vStorage API.
     */
    boolean isVStorageApiSupported();

    /**
     * Enables or disables the VMWare vStorage API based on the <code>enabled</code> value.
     * @throws java.lang.UnsupportedOperationException if isVStorageApiSupported() returns false.
     */
    void modifyVStorageApiEnabled(boolean enabled);
}
