/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3;

import java.util.List;
import java.util.Set;

import com.netapp.offtap3.ontap.IGroupOperationException;
import com.netapp.offtap3.ontap.OntapIGroup;

public interface IGroup extends Refreshable {
    /**
     * Returns the protocol associated with this initiator group. e.g. iscsi or fcp
     */
    String getProtocol();

    /**
     * Returns the operating system type associated with this initiator group.
     */
    String getOsType();

    String getUuid();

    Set<String> getInitiatorIds();

    /**
     * Add the specified list of initiators. Duplicates will be ignored.
     */
    void addInitiators(Set<String> initiators) throws IGroupOperationException;

    /**
     * Adds the specified initiator to this group. If the initiator is already in this group, nothing happens.
     */
    void addInitiator(String initiator) throws IGroupOperationException;

    /**
     * Removes all of the specified initiators from this group.
     * @param force Forcibly remove all of the initiators even if they are mapped to an existing lun.
     */
    void removeInitiators(Set<String> initiators, boolean force) throws IGroupOperationException;

    /**
     * This method performs actions on the server to remove any initiators that are not in the list and add any iniators
     * that are in the list but were not previously. The end result is that the initiators associated with this group
     * on the server will exactly mirror those specified in the supplied list.
     * @return True if the initiator list was able to be modified as requested.
     */
    boolean setInitiators(Set<String> initiators) throws IGroupOperationException;

    void rename(String newName) throws IGroupOperationException;

    /**
     * Destroys this initiator group.
     * @param force Forcibly remove the group and all of the initiators even if they are mapped to an existing lun.
     */
    void destroy(boolean force) throws IGroupOperationException;

    /**
     * Destroys this initiator group.
     */
    void destroy() throws IGroupOperationException;

    @Override
    IGroup refresh();
}
