package com.netapp.offtap3;


import java.util.Set;

public interface UnixUser extends HasName {
    /**
     * Returns the id of the user.
     */
    int getId();

    /**
     * Returns the primary group of the user. This group is used as the group for any files or directories created
     * by this user.
     */
    UnixGroup getPrimaryGroup();

    /**
     * Returns a set of secondary groups to which this user is a member. This user will have access to files or
     * directories that the secondary group has access to.
     */
    Set<UnixGroup> getSecondaryGroups();

    /**
     * Adds this user to the specified group as a secondary member. This user will have access to files or directories
     * that the secondary group has access to.
     * @throws UserAndGroupOperationException
     */
    void addSecondaryGroup(UnixGroup group) throws UserAndGroupOperationException;

    /**
     * Removes this user from the specified group as a secondary member.
     * @throws UserAndGroupOperationException
     */
    void removeSecondaryGroup(UnixGroup group) throws UserAndGroupOperationException;

    /**
     * Destroys the user. If the user had any secondary groups, it is removed from those groups. If the user's primary
     * group was a group with the same name as the user and there are no other members of that group, that group will
     * be destroyed as well.
     * @throws UserAndGroupOperationException
     */
    void destroy() throws UserAndGroupOperationException;
}
