/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/factories/ControllerConnectionFactory.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.factories;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.apache.log4j.Logger;

import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.internal.ControllerConnectionImpl;

public class ControllerConnectionFactory {
    private static final Logger LOGGER = Logger.getLogger(ControllerConnectionFactory.class);
    private static ControllerConnectionFactory instance;
    private static final Object INSTANCE_CREATION_LOCK = new Object();

    private ControllerConnectionFactory() {
    }

    public static ControllerConnectionFactory getInstance() {
        synchronized (INSTANCE_CREATION_LOCK) {
            if (instance == null) {
                instance = new ControllerConnectionFactory();
            }
        }
        return instance;
    }

    public ControllerConnection createConnection(OfftapPreferences preferences) {
        return new ControllerConnectionImpl(preferences);
    }

    public <C extends ControllerConnection> C createConnection(OfftapPreferences preferences, ControllerType<C> controllerType) {
        Class<C> connectionClass = controllerType.getConnectionClass();
        try {
            Constructor<C> constructor = connectionClass.getConstructor(OfftapPreferences.class);
            return constructor.newInstance(preferences);
        } catch (NoSuchMethodException | InvocationTargetException | InstantiationException | IllegalAccessException e) {
            LOGGER.error("Error while attempting to create connection for connection type " + controllerType + "!", e);
            throw new RuntimeException(e);
        }
    }
}
