package com.netapp.offtap3.factories;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.factories.platform.PlatformDataLoader;
import com.netapp.offtap3.internal.mars.MarsConnection;
import com.netapp.offtap3.internal.mars.MarsPlatformDataLoader;
import com.netapp.offtap3.internal.ontap.controller.OntapConnection;
import com.netapp.offtap3.internal.ontap.dataloader.OntapDataLoader;

public final class ControllerType<C extends ControllerConnection> {
    public static final ControllerType<OntapConnection> ONTAP = new ControllerType<>(OntapDataLoader.class, OntapConnection.class);
    public static final ControllerType<MarsConnection> MARS = new ControllerType<>(MarsPlatformDataLoader.class, MarsConnection.class);

    private static Set<ControllerType<? extends ControllerConnection>> values;

    static {
        Set<ControllerType<? extends ControllerConnection>> mutableValues = new HashSet<>();
        mutableValues.add(ONTAP);
        mutableValues.add(MARS);
        values = Collections.unmodifiableSet(mutableValues);
    }

    private Class<? extends PlatformDataLoader> dataLoaderClass;
    private Class<C> connectionClass;

    private ControllerType(Class<? extends PlatformDataLoader> dataLoaderClass, Class<C> connectionClass) {
        this.dataLoaderClass = dataLoaderClass;
        this.connectionClass = connectionClass;
    }

    public Class<? extends PlatformDataLoader> getPlatformLoaderClass() {
        return dataLoaderClass;
    }

    public Class<C> getConnectionClass() {
        return connectionClass;
    }

    public static Set<ControllerType<? extends ControllerConnection>> values() {
        return values;
    }
}