package com.netapp.offtap3.factories;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;

import java.util.Set;

public interface DataLoader<T> {
    /**
     * Returns true if this data loader has completed loading its data into the target object.
     * @param resources The Set of resources for which a completed load is checked.
     * @return True if the load has been completed for ALL resources in the Set.
     */
    boolean isLoaded(Set<Resource> resources);

    /**
     * Initiates a load of the data associated with the specified resources into the loadTarget.
     * @param loadTarget The target object where the data is stored.
     * @param loadType The type of load being performed.
     * @param resources The Set of resources which will be loaded.
     * @throws com.netapp.offtap3.factories.DataLoaderException
     */
    void load(T loadTarget, LoadType loadType, Set<Resource> resources);

    /**
     * Initiates a refresh of the data on the target object for the specified resource and refreshTarget object. If
     * possible, the refreshTarget object should be used to scope the data loading to only the data needed to refresh
     * that particular object. Otherwise, the data loader may refresh all data associated with the specified resource.
     * @param loadTarget The target object where the data is stored.
     * @param resource The resource for the reloadTarget.
     * @param refreshTarget The Refreshable object that specifically needs to be refreshed.
     * @throws com.netapp.offtap3.factories.DataLoaderException
     */
    Refreshable refresh(T loadTarget, Resource resource, Refreshable refreshTarget);

    /**
     * Invalidates the current loaded state. After calling this method, {@link #isLoaded(java.util.Set)} will return
     * false until {@link #load(Object, com.netapp.offtap3.factories.DataLoader.LoadType, java.util.Set)} is called.
     */
    void invalidate();

    /**
     * Indicates the type of load being performed. This can be used by the data loader to perform more efficient
     * loading based on the context.
     */
    enum LoadType {initial, lazy, refresh
    }
}
