package com.netapp.offtap3.factories;


import com.netapp.offtap3.ConnectionException;
import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.factories.platform.PlatformDataLoader;
import com.netapp.offtap3.internal.Messages;

public class PlatformDataLoaderFactory {

    /**
     * Attempts to connect to the specified connection using each of the platform loaders available. If
     *
     * @param controllerConnection The connection used to connect to the platform.
     * @return A PlatformDataLoader capable of loading a controller using the specified connection.
     * @throws ConnectionException If a loader could connect but received an error indicating the connection failed for
     *                             another reason.
     * @throws DataLoaderException If no loader could be found that was able to handle the connection at all.
     */
    public static PlatformDataLoader buildPlatformDataLoader(ControllerConnection controllerConnection) throws ConnectionException, DataLoaderException {
        PlatformDataLoader loaderToUse = null;

        try {
            if (controllerConnection.getControllerType() != null) {
                loaderToUse = controllerConnection.getControllerType().getPlatformLoaderClass().newInstance();
            } else {
                for (ControllerType<?> controllerType : ControllerType.values()) {
                    PlatformDataLoader loader = controllerType.getPlatformLoaderClass().newInstance();

                    if (loader.canHandle(controllerConnection)) {
                        loaderToUse = loader;
                        break;
                    }
                }
            }
        } catch (InstantiationException | IllegalAccessException e) {
            throw new RuntimeException(e);
        }

        if (loaderToUse == null) {
            throw new DataLoaderException(
                    Messages.getInstance().getMessage("platform.loader.missing", controllerConnection));
        }

        return loaderToUse;
    }
}
