package com.netapp.offtap3.internal;

import com.google.common.base.Objects;
import com.netapp.offtap3.HasLoader;
import com.netapp.offtap3.Refreshable;

import java.io.Serializable;
import java.util.Date;

public abstract class AbstractRefreshable implements Serializable, Comparable<AbstractRefreshable>, Refreshable {
    private String name;
    private Date lastLoad;
    private HasProxy<? extends HasLoader> loader;

    public AbstractRefreshable(HasProxy loader) {
        this(null, new Date(), loader);
    }

    @SuppressWarnings("unchecked")
    protected AbstractRefreshable(String name, Date lastLoad, HasProxy loader) {
        this.name = name;
        this.lastLoad = lastLoad;
        this.loader = loader;
    }

    @Override
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Override
    public Date getLastLoad() {
        return lastLoad;
    }

    public void setLastLoad(Date lastLoad) {
        this.lastLoad = lastLoad;
    }

    @Override
    public int compareTo(AbstractRefreshable o) {
        return getName().compareTo(o.getName());
    }

    @Override
    public Refreshable refresh() {
        return getLoader().refresh(this);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Refreshable) {
            Refreshable other = (Refreshable) obj;

            return Objects.equal(getName(), other.getName()) && Objects.equal(getLastLoad(), other.getLastLoad());
        }
        return false;
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(this.name, this.lastLoad);
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this).addValue(name).addValue(lastLoad).toString();
    }

    @Override
    public HasLoader getLoader() {
        return loader.getProxy();
    }
}
