
package com.netapp.offtap3.internal;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import com.google.common.base.Objects;

import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.DiscoveryProgressListener;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.ProxyType;
import com.netapp.offtap3.factories.ControllerType;

public class ControllerConnectionImpl implements ControllerConnection {
    private static final long serialVersionUID = 2243690949955431393L;
    private String ipAddress;
    private String userName = "root";
    private String password = "";
    private int port = 0;
    private boolean ssl;
    private OfftapPreferences prefs;
    private Set<DiscoveryProgressListener> discoveryProgressListeners = new HashSet<DiscoveryProgressListener>();
	private String filerIdOfTarget;
	private int zapiConnectionTimeout;
    private int zapiTimeout;
	private boolean allowRetry = true;
    private ControllerType controllerType;

    public ControllerConnectionImpl(OfftapPreferences prefs){
        this.prefs = prefs;
    }
    
    public ControllerConnectionImpl(ControllerConnection controllerConnection) {
        this.ipAddress = controllerConnection.getIpAddress();
        this.userName = controllerConnection.getUserName();
        this.password = controllerConnection.getPassword();
        this.ssl = controllerConnection.isSSL();
        setPort(controllerConnection.getPort());
        this.prefs = controllerConnection.getOfftapPreferences();
        this.discoveryProgressListeners.addAll(controllerConnection.getDiscoveryProgressListeners());
        this.filerIdOfTarget = controllerConnection.getFilerIdOfTarget();
        this.zapiConnectionTimeout = controllerConnection.getZapiConnectionTimeout();
        this.zapiTimeout = controllerConnection.getZapiTimeout();
        this.allowRetry = controllerConnection.isAllowRetry();
        this.controllerType = controllerConnection.getControllerType();
    }
    
    @Override
    public String getIpAddress() {
        return ipAddress;
    }
    @Override
    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
    }
    @Override
    public String getUserName() {
        return userName;
    }
    @Override
    public void setUserName(String userName) {
        this.userName = userName;
    }
    @Override
    public String getPassword() {
        return password;
    }
    @Override
    public void setPassword(String password) {
        this.password = password;
    }
    @Override
    public int getPort() {
        return port;
    }
    @Override
    public void setPort(int port) {
        if (port <= 0) {
            if (isSSL()) {
                this.port = 443;
            } else {
                this.port = 80;
            }
        } else {
            this.port = port;
        }
    }
    @Override
    public boolean isSSL() {
        return ssl;
    }
    @Override
    public void setSSL(boolean ssl) {
        this.ssl = ssl;
    }
    @Override
    public OfftapPreferences getOfftapPreferences() {
        return prefs;
    }
    @Override
    public void setOfftapPreferences(OfftapPreferences prefs) {
        this.prefs = prefs;
    }

    @Override
    public Set<DiscoveryProgressListener> getDiscoveryProgressListeners(ProxyType proxyType) {
        Set<DiscoveryProgressListener> listenersForProxy = new HashSet<DiscoveryProgressListener>();
        for (DiscoveryProgressListener listener : discoveryProgressListeners) {
            if (listener.getSupportedProxyTypes().contains(proxyType)) {
                listenersForProxy.add(listener);
            }
        }
        return listenersForProxy;
    }

    @Override
    public void addDiscoveryProgressListener(DiscoveryProgressListener listener) {
        discoveryProgressListeners.add(listener);
    }

    @Override
    public void removeDiscoveryProgressListener(DiscoveryProgressListener listener) {
        discoveryProgressListeners.remove(listener);
    }

    @Override
    public Set<DiscoveryProgressListener> getDiscoveryProgressListeners() {
        return Collections.unmodifiableSet(discoveryProgressListeners);
    }

    @Override
    public int hashCode() {
        return ((ipAddress == null) ? 0 : ipAddress.hashCode());
    }

    @Override
    public boolean equals(Object obj) {
        boolean result = obj instanceof ControllerConnectionImpl && toString().equals(obj.toString());
        return result;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
                .add("ipAddress", ipAddress)
                .add("userName", userName)
                .add("port", port)
                .add("ssl", ssl)
                .toString();
    }

    @Override
    public String getFilerIdOfTarget() {
		return filerIdOfTarget;
	}

	public void setFilerIdOfTarget(String filerIdOfTarget) {
		this.filerIdOfTarget = filerIdOfTarget;
	}

    @Override
    public int getZapiConnectionTimeout() {
        return zapiConnectionTimeout;
    }

    public void setZapiConnectionTimeout(int zapiConnectionTimeout) {
        this.zapiConnectionTimeout = zapiConnectionTimeout;
    }

    @Override
    public int getZapiTimeout() {
		return zapiTimeout;
	}

	public void setZapiTimeout(int zapiTimeout) {
		this.zapiTimeout = zapiTimeout;
	}

    @Override
	public boolean isAllowRetry() {
		return allowRetry;
	}

	public void setAllowRetry(boolean allowRetry) {
		this.allowRetry = allowRetry;
	}

    @Override
    public void setControllerType(ControllerType controllerType) {
        this.controllerType = controllerType;
    }

    @Override
    public ControllerType getControllerType() {
        return controllerType;
    }
}
