package com.netapp.offtap3.internal;

import com.netapp.offtap3.ControllerLoader;
import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.ProxyType;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoader;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.utils.Utils.VscThreadFactory;

import org.apache.log4j.Logger;

import java.lang.reflect.Method;
import java.util.Collections;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


public abstract class ControllerProxy<I, T extends HasProxy<I>> extends OfftapProxy<I, T> {

    private static final Logger logger = Logger.getLogger(ControllerProxy.class);

    public ControllerProxy() {
        super(ProxyType.CONTROLLER);
    }

    @Override
    protected Resource getResourceAnnotation(Method method) {
        ControllerLoader loader = method.getAnnotation(ControllerLoader.class);
        return (loader == null) ? null : loader.value();
    }

    @Override
    protected ExecutorService getExecutorService() {
        return Executors.newFixedThreadPool(getOfftapPreferences().getMaxRunningControllerThreads(), new VscThreadFactory("controllerProxyPool"));
    }

    @Override
    protected void initialLoad() throws DataLoaderException {
        // for controllers to be useful and sensible,
        // make sure these always get run FIRST
        executeDataLoader(DataLoader.LoadType.initial, Collections.<Resource>singleton(ControllerResource.system_info));
        executeDataLoader(DataLoader.LoadType.initial, Collections.<Resource>singleton(ControllerResource.licenses));

        super.initialLoad();
    }
}
