package com.netapp.offtap3.internal;

import com.netapp.offtap3.DiscoveryProgressJob;
import com.netapp.offtap3.DiscoveryProgressListener;

import java.util.Collection;
import java.util.concurrent.atomic.AtomicInteger;


public class DiscoveryProgressJobImpl implements DiscoveryProgressJob {

    private String name;
    private String status;
    private final int defaultIncrement;
    private AtomicInteger percentDone = new AtomicInteger();

    private Collection<DiscoveryProgressListener> listeners;

    public DiscoveryProgressJobImpl(int totalIncrements, Collection<DiscoveryProgressListener> listeners) {
        if (totalIncrements == 0) {
            defaultIncrement = 100;
        } else {
            defaultIncrement = 100 / totalIncrements;
        }
        this.listeners = listeners;
    }

    @Override
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void increment() {
        increment(defaultIncrement);
    }

    public void increment(int inc) {
        percentDone.addAndGet(inc);
    }

    public int getPercentDone() {
        return percentDone.get();
    }

    @Override
    public String getStatusMessage() {
        return status;
    }

    public void notifyStart() {

        for (DiscoveryProgressListener listener : listeners) {
            listener.started(this);
        }
    }

    public void notifyUpdate() {
        increment();
        for (DiscoveryProgressListener listener : listeners) {
            listener.updateProgress(this);
        }
    }

    public void notifyFinished() {
        for (DiscoveryProgressListener listener : listeners) {
            listener.finished(this);
        }
    }
}
