/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal;

import java.util.HashSet;
import java.util.Set;

import com.google.common.base.Objects;
import com.netapp.offtap3.DataController;
import com.netapp.offtap3.IGroup;

public abstract class IGroupImpl<T extends AbstractController<C>, C extends DataController> extends AbstractRefreshable implements IGroup, HasController<C, T> {
    protected final T controller;
    protected Set<String> initiatorIds = new HashSet<>();
    private String protocol;
    private String osType;
    private String uuid;

    public IGroupImpl(T controller) {
        super(controller);
        this.controller = controller;
    }

    @Override
    public C getController() {
        return controller.getProxy();
    }

    @Override
    public T getControllerImpl() {
        return controller;
    }

    @Override
    public String getProtocol() {
        return protocol;
    }

    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }

    @Override
    public String getOsType() {
        return osType;
    }

    public void setOsType(String osType) {
        this.osType = osType;
    }

    @Override
    public String getUuid() {
		return uuid;
	}

    public void setUuid(String uuid) {
		this.uuid = uuid;
	}

    @Override
    public Set<String> getInitiatorIds() {
        return initiatorIds;
    }

    public void setInitiatorIds(Set<String> initiatorIds) {
        this.initiatorIds = initiatorIds;
    }

    @Override
    public boolean setInitiators(Set<String> initiators) {
        Set<String> currentInitiatorIds = getInitiatorIds();
        Set<String> toAdd = new HashSet<>();
        Set<String> toRemove = new HashSet<>();
        for (String currWanted : initiators) {
            if (!currentInitiatorIds.contains(currWanted)) {
                toAdd.add(currWanted);
            }
        }

        for (String currentHave : currentInitiatorIds) {
            if (!initiators.contains(currentHave)) {
                toRemove.add(currentHave);
            }
        }

        if (toAdd.size() > 0) {
            addInitiators(new HashSet<>(toAdd));
        }

        if (toRemove.size() > 0) {
            removeInitiators(new HashSet<>(toRemove), true);
        }
        return toAdd.size() > 0 || toRemove.size() > 0;
    }

    @Override
    public IGroup refresh() {
        return (IGroup)super.refresh();
    }

    @Override
    public boolean equals(Object o) {
        return o instanceof IGroupImpl && toString().equals(o.toString());
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(getName(), getProtocol(), getOsType());
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
                      .add("name", getName())
                      .add("protocol", getProtocol())
                      .add("osType", getOsType())
                      .add("uuid", getUuid())
                      .toString();
    }
}
