/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal;

import java.util.Set;

import com.netapp.offtap3.IGroup;
import com.netapp.offtap3.Lun;
import com.netapp.offtap3.Size;
import com.netapp.offtap3.ontap.OntapIGroup;

public abstract class LunImpl extends AbstractRefreshable implements Lun {
    private String path;
    private String uuid;
    private String serial;
    private String type;
    private boolean online;
    private Size totalSize = Size.ZERO;
    private Size usedSize = Size.ZERO;
    private boolean readOnly;
    private Set<? extends IGroup> igroups;

    public LunImpl(HasProxy loader) {
        super(loader);
    }

    public void setPath(String path) {
        this.path = path;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public void setSerial(String serial) {
        this.serial = serial;
    }

    public void setType(String type) {
        this.type = type;
    }

    public void setOnline(boolean online) {
        this.online = online;
    }

    public void setTotalSize(long totalSize) {
        this.totalSize = Size.createB(totalSize);
    }

    public void setUsedSize(long usedSize) {
        this.usedSize = Size.createB(usedSize);
    }

    public void setReadOnly(boolean readOnly) {
        this.readOnly = readOnly;
    }

    public void setIgroups(Set<? extends IGroup> igroups) {
        this.igroups = igroups;
    }

    @Override
    public String getSerial() {
        return serial;
    }

    @Override
    public String getUuid() {
        return uuid;
    }

    @Override
    public String getType() {
        return type;
    }

    @Override
    public Size getTotalSize() {
        return totalSize;
    }

    @Override
    public Size getUsedSize() {
        return usedSize;
    }

    @Override
    public Size getFreeSize() {
        return totalSize.minus(usedSize);
    }

    @Override
    public String getPath() {
        return path;
    }

    @Override
    public boolean isOnline() {
        return online;
    }

    @Override
    public boolean isReadOnly() {
        return readOnly;
    }

    @Override
    public Set<? extends IGroup> getIgroups() {
        return igroups;
    }

    @Override
    public Lun refresh() {
        return (Lun) super.refresh();
    }

    @Override
    public String toString() {
        return "LunImpl{" +
                "path='" + path + '\'' +
                ", uuid='" + uuid + '\'' +
                ", serial='" + serial + '\'' +
                ", type='" + type + '\'' +
                ", online=" + online +
                ", totalSize=" + totalSize +
                ", usedSize=" + usedSize +
                ", readOnly=" + readOnly +
                "} " + super.toString();
    }
}
