/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/UnixGroupImpl.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal;

import com.netapp.offtap3.UnixGroup;
import com.netapp.offtap3.UnixUser;
import com.netapp.offtap3.UserAndGroupOperationException;
import com.netapp.offtap3.internal.ontap.HasOntapController;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.Map;

public class UnixGroupImpl implements UnixGroup, HasOntapController<DataSvm, DataSvmImpl> {
    private static final Logger LOGGER = Logger.getLogger(UnixGroupImpl.class);

    private int id;
    private String name;
    private Map<String, UnixUser> users = new SafeConcurrentHashMap<String, UnixUser>();
    private DataSvmImpl dataSvm;

    public UnixGroupImpl(DataSvmImpl dataSvm, String name, int id) {
        this.id = id;
        this.name = name;
        this.dataSvm = dataSvm;
    }

    @Override
    public int getId() {
        return id;
    }

    @Override
    public Map<String, UnixUser> getUsers() {
        return Collections.unmodifiableMap(users);
    }

    public void setUsers(Map<String, UnixUser> users) {
        this.users = users;
    }

    /**
     * Adds a user object to the internal maps of users but does not affect the users associated with this group on
     * the server. This method should be used only for the construction of the UnixGroup object.
     */
    public void addUser(UnixUser user) {
        users.put(user.getName(), user);
    }

    public void removeUser(String userName) {
        users.remove(userName);
    }

    @Override
    public void destroy() throws UserAndGroupOperationException {
        LOGGER.debug("destroy: " + getName());

        for (UnixUser user : users.values()) {
            if (user.getPrimaryGroup().equals(this)) {
                throw new UserAndGroupOperationException("Cannot destroy group " + getName() + " because it is still the primary group for other users.");
            }
        }

        NaElement input = new NaElement("name-mapping-unix-group-destroy");
        input.addNewChild("group-name", getName());
        try {
            NaElement output = dataSvm.invokeZapi(input, ZapiCache.NO_CACHE);
            if ("passed".equals(output.getAttr("status"))) {
                dataSvm.getUnixGroups().remove(name);
            }
            LOGGER.info("Destroyed " + getName() + ":" + output.getAttr("status") + ".");
        } catch (ApiFailedException e) {
            String message = "Unable to destroy unix group " + getName() + " on data svm " + dataSvm.getName() + ".";
            LOGGER.error("destroy: " + message);
            throw new UserAndGroupOperationException(message, e);
        }
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public DataSvm getController() {
        return dataSvm.getProxy();
    }

    @Override
    public DataSvmImpl getControllerImpl() {
        return dataSvm;
    }
}
