/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars;

import org.apache.log4j.Logger;

import com.fasterxml.jackson.core.type.TypeReference;
import com.netapp.offtap3.Size;
import com.netapp.offtap3.internal.HasController;
import com.netapp.offtap3.internal.LunImpl;
import com.netapp.offtap3.internal.mars.webmapi.model.common.ResponseContentType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.LunResizeInputType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.ResourceRenameInputType;
import com.netapp.offtap3.mars.LunMaxExtentSize;
import com.netapp.offtap3.mars.MarsApiFailedException;
import com.netapp.offtap3.mars.MarsController;
import com.netapp.offtap3.mars.MarsLun;
import com.netapp.offtap3.ontap.LunOperationException;

public class MarsLunImpl extends LunImpl implements MarsLun, HasController<MarsController, MarsControllerImpl> {
    private static final Logger logger = Logger.getLogger(MarsLunImpl.class);
    private MarsControllerImpl controller;
    private Size blockSize;
    private LunMaxExtentSize maxExtentSize;

    public MarsLunImpl(MarsControllerImpl controller) {
        super(controller);
        this.controller = controller;
    }

    @Override
    public Size getBlockSize() {
        return blockSize;
    }

    public void setBlockSize(long bytes) {
        blockSize = Size.createB(bytes);
    }

    @Override
    public LunMaxExtentSize getMaxExtentSize() {
        return maxExtentSize;
    }

    public void setMaxExtentSize(LunMaxExtentSize maxExtentSize) {
        this.maxExtentSize = maxExtentSize;
    }

    @Override
    public void offline() throws LunOperationException {
        logger.debug("Offlining lun " + getName());
        try {
            getControllerImpl().getConnection().put("luns/" + getName() + "/action/offline", null);
        } catch (MarsApiFailedException e) {
            throw new LunOperationException("An error occurred while attempting to offline the lun " + getName() + ": " + e.getMessage(), e);
        }
    }

    @Override
    public void online() {
        logger.debug("Onlining lun " + getName());
        try {
            getControllerImpl().getConnection().put("luns/" + getName() + "/action/online", null);
        } catch (MarsApiFailedException e) {
            throw new LunOperationException("An error occurred while attempting to online the lun " + getName() + ": " + e.getMessage(), e);
        }
    }

    @Override
    public void destroy() {
        logger.debug("Destroying lun " + getName());
        try {
            // Always force the destruction.
            getControllerImpl().getConnection().delete("luns/" + getName(), "force=true");
        } catch (MarsApiFailedException e) {
            throw new LunOperationException("An error occurred while attempting to destroy the lun " + getName() + ": " + e.getMessage(), e);
        }
    }

    @Override
    public Size resize(Size newSize) {
        logger.debug("Resizing lun " + getName() + " to " + newSize);
        LunResizeInputType input = new LunResizeInputType();
        input.setName(getName());
        input.setSize(String.valueOf(newSize.toBytes()));
        try {
            getControllerImpl().getConnection().put("luns/action/resize", input, new TypeReference<ResponseContentType>() {});
            setTotalSize(newSize.toBytes());
            return newSize;
        } catch (MarsApiFailedException e) {
            throw new LunOperationException("An error occurred while attempting to offline the lun " + getName() + ": " + e.getMessage(), e);
        }
    }

    @Override
    public void rename(String newName) {
        logger.debug("Renaming lun " + getName() + " to " + newName);
        ResourceRenameInputType input = new ResourceRenameInputType();
        input.setName(getName());
        input.setNewName(newName);
        try {
            getControllerImpl().getConnection().put("luns/action/rename", input, new TypeReference<ResponseContentType>() {});
            setName(newName);
        } catch (MarsApiFailedException e) {
            throw new LunOperationException("An error occurred while renaming the lun " + getName() + " to " + newName + ": " + e.getMessage(), e);
        }
    }

    @Override
    public MarsController getController() {
        return controller.getProxy();
    }

    @Override
    public MarsControllerImpl getControllerImpl() {
        return controller;
    }

    @Override
    public MarsLun refresh() {
        return (MarsLun) super.refresh();
    }

    @Override
    public String toString() {
        return "MarsLunImpl{" +
                "blockSize=" + blockSize +
                "} " + super.toString();
    }
}
