/*
 * $Id$
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars;

import org.apache.log4j.Logger;

import com.netapp.offtap3.mars.MarsApiFailedException;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ConnectionException;
import com.netapp.offtap3.Controller;
import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.Version;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.factories.platform.PlatformDataLoader;

public class MarsPlatformDataLoader implements PlatformDataLoader {
    private static final Logger logger = Logger.getLogger(MarsPlatformDataLoader.class);

    private MarsConnection connection;

    @Override
    public boolean canHandle(ControllerConnection controllerConnection) throws ConnectionException {
        connection = new MarsConnection(controllerConnection);
        try {
            return connection.login() != null;
        } catch (MarsApiFailedException e) {
            logger.trace("Caught exception while checking if controller with ip address " + controllerConnection
                    .getIpAddress() + " was a Mars controller. Exception message = " + e.getMessage());
            logger.trace("Full stack trace of Mars connection attempt.", e);
        }
        return false;
    }

    @Override
    public Controller load(ControllerConnection controllerConnection) throws DataLoaderException {
        if (connection == null) {
            try {
                if (!canHandle(controllerConnection)) {
                    throw new DataLoaderException(
                            "Error: the specified connection information cannot be used to connect to a Mars controller.");
                }
            } catch (ConnectionException e) {
                throw new DataLoaderException(e);
            }
        }
        MarsControllerImpl controller = new MarsControllerImpl(connection);
        return controller.getProxy();
    }

    @Override
    public Version getVersion(ControllerConnection controllerConnection) throws ConnectionException {
        return null;
    }
}
