/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.dataloader;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;

import org.apache.log4j.Logger;

import com.fasterxml.jackson.core.type.TypeReference;
import com.netapp.offtap3.IGroup;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.mars.MarsControllerImpl;
import com.netapp.offtap3.internal.mars.MarsIGroupImpl;
import com.netapp.offtap3.internal.mars.MarsPlatformDataLoader;
import com.netapp.offtap3.internal.mars.webmapi.model.common.ContainerType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.ResponseContentType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.IGroupInfoType;
import com.netapp.offtap3.internal.mars.webmapi.util.MarsError;
import com.netapp.offtap3.mars.MarsApiFailedException;

public class MarsIGroupDataLoader extends AbstractDataLoader<MarsControllerImpl> {
    private static final Logger LOGGER = Logger.getLogger(MarsPlatformDataLoader.class);

    @Override
    protected boolean doLoad(MarsControllerImpl controller, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        return doLoad(controller, loadType, resources, null);
    }

    @Override
    protected Refreshable doRefresh(MarsControllerImpl controller, Resource resource, Refreshable refreshTarget) {
        MarsIGroupImpl marsIGroup = (MarsIGroupImpl) refreshTarget;
        LOGGER.debug("Mars igroup refresh for igroup " + marsIGroup.getName() + " with uuid " + marsIGroup.getUuid());
        doLoad(controller, LoadType.refresh, Collections.singleton(resource), marsIGroup);
       IGroup igroup = controller.getIGroup(marsIGroup.getName());
        LOGGER.debug("Returning refreshed mars igroup :" + igroup);
        return igroup;
    }

    protected boolean doLoad(MarsControllerImpl marsController, LoadType loadType, Set<Resource> resources, MarsIGroupImpl existingIgroup) throws DataLoaderException {
        try {
            List<IGroupInfoType> igroups = Collections.emptyList();
            List<IGroup> igroupList;
            if (existingIgroup != null) {
                igroupList = new ArrayList<>(marsController.getIGroups());
                for (Iterator<IGroup> iter = igroupList.iterator(); iter.hasNext();) {
                    if (iter.next().getName().equals(existingIgroup.getName())) {
                        iter.remove();
                    }
                }
                try {
                    ResponseContentType<IGroupInfoType> result = marsController.getConnection()
                                                                               .get("igroups/" + existingIgroup
                                                                                            .getName(),
                                                                                    new TypeReference<ResponseContentType<IGroupInfoType>>() {
                                                                                    });
                    igroups = Collections.singletonList(result.getResult());
                } catch (MarsApiFailedException e) {
                    if (MarsError.IGROUP_SHOW_IGROUP_NOT_FOUND == e.getError()) {
                        LOGGER.debug("Did not find igroup " + existingIgroup.getName() + " after refresh.");
                        igroups = Collections.emptyList();
                    } else {
                        throw e;
                    }
                }
            } else {
                ResponseContentType<ContainerType<IGroupInfoType>> result = marsController.getConnection().get("igroups",
                        new TypeReference<ResponseContentType<ContainerType<IGroupInfoType>>>() {
                        });

                ContainerType<IGroupInfoType> igroupContainer = result.getResult();
                igroupList = new CopyOnWriteArrayList<>();
                if (igroupContainer != null && igroupContainer.getInstancesCount() > 0) {
                    igroups = igroupContainer.getInstances();
                }
            }

            for (IGroupInfoType igroupInfo : igroups) {
                MarsIGroupImpl igroupImpl = new MarsIGroupImpl(marsController);
                igroupImpl.setName(igroupInfo.getName());
                igroupImpl.setUuid(igroupInfo.getUuid());
                igroupImpl.setOsType(igroupInfo.getOstype().toString());
                Set<String> initiatorIds = new HashSet<>();
                Collections.addAll(initiatorIds, igroupInfo.getInitiators());
                // Mars only supports fiber channel.
                igroupImpl.setProtocol("fcp");
                igroupImpl.setInitiatorIds(initiatorIds);

                igroupList.add(igroupImpl);
            }
            marsController.setIGroups(igroupList);
        } catch (MarsApiFailedException e) {
            LOGGER.error("An error occurred while loading the Mars igroups.", e);
            throw new DataLoaderException(e);
        }
        return true;
    }
}
