/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.dataloader;

import java.util.Set;

import com.fasterxml.jackson.core.type.TypeReference;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.mars.MarsControllerImpl;
import com.netapp.offtap3.internal.mars.webmapi.model.common.ContainerType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.ResponseContentType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.ControllerInfoType;

public class MarsModelDataLoader extends AbstractDataLoader<MarsControllerImpl> {
    @Override
    protected boolean doLoad(MarsControllerImpl marsController, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        try {
            ResponseContentType<ContainerType<ControllerInfoType>> response = marsController.getConnection().get("hardware/controllers", new TypeReference<ResponseContentType<ContainerType<ControllerInfoType>>>() {});
            if (response.getError() != null) {
                throw new DataLoaderException(response.getError().getErrorMessage() + " - Caused by: " + response.getError().getCauseMessage());
            }
            ContainerType<ControllerInfoType> controllerContainer = response.getResult();
            if (controllerContainer.getInstancesCount() > 0) {
                marsController.setModel(controllerContainer.getInstances().get(0).getModelName());
            }
        } catch (Exception e) {
            throw new DataLoaderException(e);
        }
        return true;
    }
}
