/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.dataloader;

import java.util.Set;

import org.apache.log4j.Logger;

import com.fasterxml.jackson.core.type.TypeReference;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.Size;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.SpaceUsageImpl;
import com.netapp.offtap3.internal.mars.MarsControllerImpl;
import com.netapp.offtap3.internal.mars.webmapi.model.common.ResponseContentType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.StatsCounterDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.StatsInstanceDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.StatsInstanceValueType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.StatsShowInputType;
import com.netapp.offtap3.internal.mars.webmapi.model.domain.StatsShowOutputType;

public class MarsSpaceUsageDataLoader extends AbstractDataLoader<MarsControllerImpl> {
    private static final Logger LOGGER = Logger.getLogger(MarsSpaceUsageDataLoader.class);

    @Override
    protected boolean doLoad(MarsControllerImpl marsController, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        try {
            StatsShowInputType statsInput = new StatsShowInputType();
            statsInput.setObject("capacity");

            ResponseContentType<StatsShowOutputType> response = marsController.getConnection().post(
                    "stats", statsInput, new TypeReference<ResponseContentType<StatsShowOutputType>>() {
            });
            StatsShowOutputType statsOutput = response.getResult();
            StatsInstanceDataType[] stats = statsOutput.getCounterValue().getObjectValue();
            if (stats != null) {
                long storable = 0;
                long stored = 0;
                long provisioned = 0;
                for (StatsInstanceDataType stat : stats) {
                    if ("cluster".equals(stat.getInstanceName())) {
                        for (StatsInstanceValueType statValue : stat.getInstanceValues()) {
                            StatsCounterDataType[] statsDataArray = statValue.getCounterValues();
                            for (StatsCounterDataType statsData : statsDataArray) {
                                // The counters are in KB. We'll convert to bytes and store in a long.
                                if (statsData.getCounterValue() != null && statsData.getCounterValue().length > 0) {
                                    if ("stored".equals(statsData.getCounterName())) {
                                        LOGGER.debug("Found stored value of " + statsData.getCounterValue()[0]);
                                        stored = (long) statsData.getCounterValue()[0].doubleValue() * 1024;
                                    }
                                    if ("storable".equals(statsData.getCounterName())) {
                                        LOGGER.debug("Found storable value of " + statsData.getCounterValue()[0]);
                                        storable = (long) statsData.getCounterValue()[0].doubleValue() * 1024;
                                    }
                                    if ("provisioned".equals(statsData.getCounterName())) {
                                        LOGGER.debug("Found provisioned value of " + statsData.getCounterValue()[0]);
                                        provisioned = (long) statsData.getCounterValue()[0].doubleValue() * 1024;
                                    }
                                }
                            }
                        }
                    }
                }
                LOGGER.debug("Loading space usage, received capacity stats: stored = " + stored + ", storable = " + storable);

                if (storable > 0) {
                    SpaceUsageImpl spaceUsage = marsController.getSpaceUsage();
                    spaceUsage.setCapacity(Size.createB(storable));
                    spaceUsage.setUsed(Size.createB(stored));
                    spaceUsage.setFree(Size.createB(storable - stored));
                    spaceUsage.setAllocated(Size.createB(provisioned));
                }
            }
        } catch (Exception e) {
            throw new DataLoaderException(e);
        }
        return true;
    }
}
