/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.common;

import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * BulkResultType defines type to contain results of bulk operations.
 * <P>
 * T is generic type, should be mentioned for each individual case
 * 
 * @author Jing Shi
 * @version 1.0, Jan 9, 2014
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class BulkResultType<T> extends BaseDataType implements WebData, WebOutput {
    /**
     * total count of items
     */
    @XmlElement(required = true, nillable = true)
    private Integer itemTotalCount = null;

    /**
     * count of item results
     */
    @XmlElement(required = true, nillable = true)
    private Integer itemResultsCount = null;

    /**
     * count of item errors
     */
    @XmlElement(required = true, nillable = true)
    private Integer itemErrorsCount = null;

    /**
     * count of contained items
     */
    @XmlElement(required = false, nillable = false)
    private List<? extends BulkItemDataType<T>> itemResults = null;
    /**
     * count of contained items
     */
    @XmlElement(required = false, nillable = false)
    private List<BulkItemErrorType> itemErrors = null;

    public BulkResultType() {
    }

    public BulkResultType(Integer itemTotalCount, List<? extends BulkItemDataType<T>> itemResults, List<BulkItemErrorType> itemErrors) {
        super();
        this.itemTotalCount = itemTotalCount;
        this.itemResults = itemResults;
        this.itemErrors = itemErrors;
        this.itemResultsCount = (this.itemResults == null ? 0 : this.itemResults.size());
        this.itemErrorsCount = (this.itemErrors == null ? 0 : this.itemErrors.size());
    }

    public Integer getItemTotalCount() {
        return itemTotalCount;
    }

    public void setItemTotalCount(Integer itemTotalCount) {
        this.itemTotalCount = itemTotalCount;
    }

    public Integer getItemResultsCount() {
        return itemResultsCount;
    }

    public void setItemResultsCount(Integer itemResultsCount) {
        this.itemResultsCount = itemResultsCount;
    }

    public Integer getItemErrorsCount() {
        return itemErrorsCount;
    }

    public void setItemErrorsCount(Integer itemErrorsCount) {
        this.itemErrorsCount = itemErrorsCount;
    }

    public List<? extends BulkItemDataType<T>> getItemResults() {
        return itemResults;
    }

    public void setItemResults(List<? extends BulkItemDataType<T>> itemResults) {
        this.itemResults = itemResults;
    }

    public List<BulkItemErrorType> getItemErrors() {
        return itemErrors;
    }

    public void setItemErrors(List<BulkItemErrorType> itemErrors) {
        this.itemErrors = itemErrors;
    }
}