/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.common;

import java.util.Collections;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * BulkType defines type to contain a bulk of items.
 * 
 * @author Jing Shi
 * @version 1.0, Mar 20, 2013
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class BulkType<T> extends BaseDataType implements WebInput {
    /**
     * count of contained items
     */
    @XmlElement(required = true, nillable = true)
    private Integer itemsCount = null;

    /**
     * list of items, can be empty list
     */
    @XmlElement(required = true, nillable = true)
    private List<T> items = null;

    public BulkType() {
    }

    public BulkType(List<T> items) {
        super();
        if (items == null) {
            this.items = Collections.emptyList();
            this.itemsCount = 0;
        } else {
            this.items = items;
            this.itemsCount = items.size();
        }
    }

    public Integer getItemsCount() {
        return itemsCount;
    }

    public void setItemsCount(Integer itemsCount) {
        this.itemsCount = itemsCount;
    }

    public List<T> getItems() {
        return items;
    }

    public void setItems(List<T> items) {
        this.items = items;
    }
}
