/*
 * NETAPP CONFIDENTIAL
 *  
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.common;

import java.util.Collections;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * ContainerType defines type to contain a set of instances.
 * 
 * @author Jing Shi
 * @version $Revision$, Mar 20, 2013
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ContainerType<T> extends BaseDataType implements WebOutput {
    /**
     * count of total instances
     */
    @XmlElement(required = true, nillable = true)
    private Integer instancesTotal = null;

    /**
     * count of contained instances
     */
    @XmlElement(required = true, nillable = true)
    private Integer instancesCount = null;

    /**
     * limit of instances per request/response, can be null
     */
    @XmlElement(required = true)
    private Integer instancesLimit = null;

    /**
     * list of instances, can be empty list
     */
    @XmlElement(required = true, nillable = true)
    private List<T> instances = null;

    public ContainerType() {
        this.instances = Collections.emptyList();
        this.instancesCount = 0;
        this.instancesTotal = 0;
        this.instancesLimit = 0;
    }

    public ContainerType(int totalCount) {
        this();
        setInstancesTotal(totalCount);
    }

    public ContainerType(T instance) {
        assert instance != null;

        this.instances = Collections.singletonList(instance);
        this.instancesCount = 1;
        this.instancesTotal = 1;
        this.instancesLimit = 0;
    }

    public ContainerType(List<T> instances, int count, int total, int limit) {
        assert instances != null;

        this.instances = Collections.unmodifiableList(instances);
        this.instancesCount = count;
        this.instancesTotal = total;
        this.instancesLimit = limit;
    }

    public Integer getInstancesCount() {
        return instancesCount;
    }

    public void setInstancesCount(Integer resultsCount) {
        this.instancesCount = resultsCount;
    }

    public Integer getInstancesLimit() {
        return instancesLimit;
    }

    public void setInstancesLimit(Integer resultsLimit) {
        this.instancesLimit = resultsLimit;
    }

    public Integer getInstancesTotal() {
        return instancesTotal;
    }

    public void setInstancesTotal(Integer resultsTotal) {
        this.instancesTotal = resultsTotal;
    }

    public List<T> getInstances() {
        return instances;
    }

    public void setInstances(List<T> instances) {
        this.instances = Collections.unmodifiableList(instances);
    }
}
