/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.common;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * ErrorType contains an error message and a cause message, if any.
 * 
 * @author Jing Shi
 * @version $Revision$, Mar 22, 2013
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ErrorType<E, C> extends BaseDataType implements WebOutput, MapiOutput {
    /**
     * error message
     */
    @XmlElement(required = true)
    private MessageType<E> errorMessage = null;

    /**
     * error cause message
     */
    @XmlElement(required = false)
    private MessageType<C> causeMessage = null;

    /**
     * Constructor empty errors with fields of NONE
     */
    public ErrorType() {

    }

    /**
     * Constructor
     * 
     * @param errorMessage
     */
    public ErrorType(MessageType<E> errorMessage) {
        this.errorMessage = errorMessage;
    }

    /**
     * Constructor
     * 
     * @param errorMessage
     * @param causeMessage
     */
    public ErrorType(MessageType<E> errorMessage, MessageType<C> causeMessage) {
        this.errorMessage = errorMessage;
        this.causeMessage = causeMessage;
    }

    /**
     * @return the errorMessage
     */
    public MessageType<E> getErrorMessage() {
        return errorMessage;
    }

    /**
     * @param errorMessage
     *            the errorMessage to set
     */
    public void setErrorMessage(MessageType<E> errorMessage) {
        this.errorMessage = errorMessage;
    }

    /**
     * @return the causeMessage
     */
    public MessageType<C> getCauseMessage() {
        return causeMessage;
    }

    /**
     * @param causeMessage
     *            the causeMessage to set
     */
    public void setCauseMessage(MessageType<C> causeMessage) {
        this.causeMessage = causeMessage;
    }
}
