/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.common;

import java.util.Collections;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * MessageType defines data transform object of I18n message. The message text is based on message language, default is in "en-US". The message key and message argument are used to support localized
 * message from additional language resource bundle. Additional output of type T is optionally provided.
 * 
 * @author Jing Shi
 * @version 1.0, Apr 5, 2013
 * @param <T>
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class MessageType<T> extends BaseDataType implements WebData {
    /**
     * message text in message language
     */
    @XmlElement(required = true)
    private String text;

    /**
     * message key used for retrieving text from message resource bundle
     */
    @XmlElement(required = true)
    private String key;

    /**
     * message args to compose the complete message text
     */
    @XmlElement(required = false)
    private List<Object> args;

    /**
     * message output used to return any additional results
     */
    @XmlElement(required = false)
    private T output;

    /**
     * message language, default is en-US
     */
    @XmlElement(required = true)
    private String language;

    public final static String DEFAULT_LANGUAGE = "en-US";

    public MessageType() {}
    

    public MessageType(String text, String key, List<Object> args) {
        this(DEFAULT_LANGUAGE, text, key, args);
    }

    public MessageType(String text, String key, List<Object> args, T output) {
        this(DEFAULT_LANGUAGE, text, key, args, output);
    }

    public MessageType(String language, String text, String key, List<Object> args, T output) {
        this(language, text, key, args);
        this.output = output;
    }

    public MessageType(String language, String text, String key, List<Object> args) {
        this.text = text;
        this.key = key;
        
        if (args != null && args.size() >0) {
            this.args = args;
        } else {
            this.args = Collections.emptyList();
        }
        
        this.language = language;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public String getKey() {
        return key;
    }

    public void setKey(String key) {
        this.key = key;
    }

    public List<Object> getArgs() {
        return args;
    }

    public void setArgs(List<Object> args) {
        this.args = args;
    }

    public String getLanguage() {
        return language;
    }

    public void setLanguage(String language) {
        this.language = language;
    }

    public T getOutput() {
        return output;
    }

    public void setOutput(T output) {
        this.output = output;
    }
}
