/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.common;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * Pair identified pair like data type
 * 
 * @author Jing Shi
 * @version $Revision$, Mar 27, 2013
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class Pair<L, R> extends BaseDataType implements WebData {

    @XmlElement(required = true)
    private L left;

    @XmlElement(required = true)
    private R right;

    public Pair() {}
    
    public Pair(L left, R right) {
        this.left = left;
        this.right = right;
    }

    @JsonProperty("left")
    public void setL(L left) {
        this.left = left;
    }

    @JsonProperty("left")
    public L getL() {
        return left;
    }
    
    @JsonProperty("right")
    public void setR(R right) {
        this.right = right;
    }

    @JsonProperty("right")
    public R getR() {
        return right;
    }
    
    public L getFirst() {
        return left;
    }

    public R getSecond() {
        return right;
    }

    /**
     * {@link @see java.lang.Object#hashCodes(java.lang.Object)}
     */
    @Override
    public int hashCode() {
        return left.hashCode() ^ right.hashCode();
    }

    /**
     * {@link @see java.lang.Object#equals(java.lang.Object)}
     */
    @SuppressWarnings("rawtypes")
    @Override
    public boolean equals(Object obj) {
        if (obj == null)
            return false;
        if (!(obj instanceof Pair))
            return false;

        Pair pair = (Pair) obj;
        return this.left.equals(pair.getL()) && this.right.equals(pair.getR());
    }

    /**
     * {@link @see java.lang.Object#toString()}
     */
    @Override
    public String toString() {
        return super.toString() + "[" + left + "," + right + "]";
    }

}
