/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.common;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * ResponseContentType defines the WebMAPI response, which includes a "status", "error" (if any), and "result" (if any).
 * 
 * @author Jing Shi
 * @version $Revision$, Feb 28, 2013
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ResponseContentType<T> extends BaseDataType implements WebData {
    /**
     * Operation context used by web service
     */
    @XmlTransient
    @JsonIgnore
    protected ContextType context;

    /**
     * Operation status
     */
    @XmlElement(required = true)
    protected ReplyStatusType status;

    /**
     * Operation error
     */
    @XmlElement(required = false)
    protected ErrorType error;

    /**
     * Operation result
     */
    @XmlElement(required = false)
    protected T result;

    public ResponseContentType() {
    }

    public ResponseContentType(ContextType context, ReplyStatusType status, ErrorType error, T result) {
        this.context = context;
        this.status = status;
        this.error = error;
        this.result = result;
    }

    @JsonIgnore
    public ContextType getContext() {
        return context;
    }

    @JsonIgnore
    public void setContext(ContextType context) {
        this.context = context;
    }

    public ReplyStatusType getStatus() {
        return status;
    }

    public void setStatus(ReplyStatusType status) {
        this.status = status;
    }

    public ErrorType getError() {
        return error;
    }

    public void setError(ErrorType error) {
        this.error = error;
    }

    public T getResult() {
        return result;
    }

    public void setResult(T result) {
        this.result = result;
    }

}
