/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.common;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
/**
 * 
 * TreeNodeType provides a type for a node object for a tree.
 * The tree contains objects of type T.
 *
 * @author tsanjeev
 * @version $Revision$, Jan 13, 2015
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class TreeNodeType <T> extends BaseDataType implements WebOutput {
    
    /**
     * current node 
     */
    @XmlElement(required = true, name = "current-node")
    private T currentNode = null;

    /**
     * Parent node id (or name).
     * 
     * */
    @XmlElement(required = false, nillable = true, name = "parent-node-id")
    private String parentNodeId = null;

    /**
     * list of child nodes, can be empty list
     */
    @XmlElement(required = false, nillable = true, name = "child-nodes")
    private List<TreeNodeType<T>> childNodes = null;

    
    public TreeNodeType() {
        super();
    }

    public TreeNodeType(T currentNode) {
        super();
        this.currentNode = currentNode;
        this.childNodes = new ArrayList<TreeNodeType<T>>();
    }
    
    public TreeNodeType(T currentNode, String parentNodeId, List<TreeNodeType<T>> childNodes) {
        super();
        this.currentNode = currentNode;
        this.parentNodeId = parentNodeId;
        this.childNodes = childNodes;
    }

    public T getCurrentNode() {
        return currentNode;
    }

    public void setCurrentNode(T currentNode) {
        this.currentNode = currentNode;
    }

    public String getParentNodeId() {
        return parentNodeId;
    }

    public void setParentNodeId(String parentNodeId) {
        this.parentNodeId = parentNodeId;
    }

    public List<TreeNodeType<T>> getChildNodes() {
        return childNodes;
    }

    public void setChildNodes(List<TreeNodeType<T>> childNodes) {
        this.childNodes = childNodes;
    }
    
    
}
