/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * Alert info type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class AlertInfoType extends BaseDataType implements MapiOutput, WebOutput {

    /** Policy which created the alert. */
    @XmlElement(required = true, name = "policy-id")
    private String policyId;
    
    /** Class of the alert. */
    @XmlElement(required = true, name = "managed-entity-class")
    private String managedEntityClass;
    
    /** Unit identifier of the alert. */
    @XmlElement(required = true, name = "managed-entity-id")
    private String managedEntityId;
    
    /** Description of the alert. */
    @XmlElement(required = true, name = "description")
    private String description;
    
    /** Corrective action to be undertaken to repair the reason for the alert. */
    @XmlElement(required = true, name = "corrective-action")
    private String correctiveAction;
    
    /** Severity of the alert. */
    @XmlElement(required = true, name = "severity")
    private SystemHealthAlertSeverity severity;
    
    /** Epoch creation time of the alert. */
    @XmlElement(required = true, name = "create-time-epoch")
    private Integer createTimeEpoch;
    
    /** Tags associated with the alert. */
    @XmlElement(required = true, name = "tags")
    private String tags;
    
    /** Indicates if this alert has been acknowledged. */
    @XmlElement(required = true)
    private Boolean acknowledged;
    
    /** Who acknowledged the alert. */
    @XmlElement(required = true)
    private String acknowledger;
    
    /** Health state. */
    @XmlElement(required = true, name = "health-state")
    private HealthState healthState;

    public AlertInfoType() {
        super();
    }

    public AlertInfoType(String policyId, String managedEntityClass,
            String managedEntityId, String description,
            String correctiveAction, SystemHealthAlertSeverity severity,
            Integer createTimeEpoch, String tags, Boolean acknowledged,
            String acknowledger) {
        super();
        this.policyId = policyId;
        this.managedEntityClass = managedEntityClass;
        this.managedEntityId = managedEntityId;
        this.description = description;
        this.correctiveAction = correctiveAction;
        setSeverity(severity);
        this.createTimeEpoch = createTimeEpoch;
        this.tags = tags;
        this.acknowledged = acknowledged;
        this.acknowledger = acknowledger;
    }



    @JsonProperty("policy-id")
    public String getPolicyId() {
        return policyId;
    }

    @JsonProperty("policy-id")
    public void setPolicyId(String policyId) {
        this.policyId = policyId;
    }

    @JsonProperty("managed-entity-class")
    public String getManagedEntityClass() {
        return managedEntityClass;
    }

    @JsonProperty("managed-entity-class")
    public void setManagedEntityClass(String managedEntityClass) {
        this.managedEntityClass = managedEntityClass;
    }

    @JsonProperty("managed-entity-id")
    public String getManagedEntityId() {
        return managedEntityId;
    }

    @JsonProperty("managed-entity-id")
    public void setManagedEntityId(String managedEntityId) {
        this.managedEntityId = managedEntityId;
    }

    @JsonProperty("description")
    public String getDescription() {
        return description;
    }

    @JsonProperty("description")
    public void setDescription(String description) {
        this.description = description;
    }

    @JsonProperty("corrective-action")
    public String getCorrectiveAction() {
        return correctiveAction;
    }

    @JsonProperty("corrective-action")
    public void setCorrectiveAction(String correctiveAction) {
        this.correctiveAction = correctiveAction;
    }

    @JsonProperty("severity")
    public SystemHealthAlertSeverity getSeverity() {
        return severity;
    }

    @JsonProperty("severity")
    public void setSeverity(SystemHealthAlertSeverity severity) {
        this.severity = severity;
        this.healthState = SystemHealthAlertSeverity.getHealthState(severity);
    }

    @JsonProperty("tags")
    public String getTags() {
        return tags;
    }

    @JsonProperty("tags")
    public void setTags(String tags) {
        this.tags = tags;
    }

    @JsonProperty("create-time-epoch")
    public Integer getCreateTimeEpoch() {
        return createTimeEpoch;
    }

    @JsonProperty("create-time-epoch")
    public void setCreateTimeEpoch(Integer createTimeEpoch) {
        this.createTimeEpoch = createTimeEpoch;
    }

    @JsonProperty("acknowledged")
    public Boolean getAcknowledged() {
        return acknowledged;
    }

    @JsonProperty("acknowledged")
    public void setAcknowledged(Boolean acknowledged) {
        this.acknowledged = acknowledged;
    }

    @JsonProperty("acknowledger")
    public String getAcknowledger() {
        return acknowledger;
    }

    @JsonProperty("acknowledger")
    public void setAcknowledger(String acknowledger) {
        this.acknowledger = acknowledger;
    }
}
