/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiData;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebData;

/**
 * Archiver counter data per sampling resolution
 *
 * @author Jing Shi
 * @version 1.0, May 9, 2014
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ArchiverCounterDataSeriesType extends BaseDataType implements WebData, MapiData {
    /** Statistics sampling resolution */
    @XmlElement(required = true, name = "result-sampling-rate")
    private ArchiverSamplingRate resultSamplingRate;
    
    /** Timestamp of first result (UTC in seconds) */
    @XmlElement(required = true, name = "result-start-time")
    private Long resultStartTime;
    
    /** Timestamp of last result (UTC in seconds) */
    @XmlElement(required = true, name = "result-end-time")
    private Long resultEndTime;
    
    /** Total number of records in this series */
    @XmlElement(required = true, name = "number-of-records")
    private Integer numberOfRecords;
    
    /** Counter data values */
    @XmlElement(required = true, name = "values")
    private ArchiverDataValuesType[] values;
    
    public ArchiverCounterDataSeriesType() {}
    
    public ArchiverCounterDataSeriesType(ArchiverSamplingRate resultSamplingRate, Long resultStartTime, Long resultEndTime, Integer numberOfRecords, ArchiverDataValuesType[] values) {
        super();
        this.resultSamplingRate = resultSamplingRate;
        this.resultStartTime = resultStartTime;
        this.resultEndTime = resultEndTime;
        this.numberOfRecords = numberOfRecords;
        this.values = values;
    }
    
    public ArchiverCounterDataSeriesType(ArchiverCounterDataSeriesType that) {
        super();
        copy(that);
    }

    @JsonProperty("result-sampling-rate")
    public ArchiverSamplingRate getResultSamplingRate() {
        return resultSamplingRate;
    }

    @JsonProperty("result-sampling-rate")
    public void setResultSamplingRate(ArchiverSamplingRate resultSamplingRate) {
        this.resultSamplingRate = resultSamplingRate;
    }
    
    @JsonProperty("result-start-time")
    public Long getResultStartTime() {
        return resultStartTime;
    }

    @JsonProperty("result-start-time")
    public void setResultStartTime(Long resultStartTime) {
        this.resultStartTime = resultStartTime;
    }

    @JsonProperty("result-end-time")
    public Long getResultEndTime() {
        return resultEndTime;
    }

    @JsonProperty("result-end-time")
    public void setResultEndTime(Long resultEndTime) {
        this.resultEndTime = resultEndTime;
    }

    @JsonProperty("number-of-records")
    public Integer getNumberOfRecords() {
        return numberOfRecords;
    }

    @JsonProperty("number-of-records")
    public void setNumberOfRecords(Integer numberOfRecords) {
        this.numberOfRecords = numberOfRecords;
    }

    @JsonProperty("values")
    public ArchiverDataValuesType[] getValues() {
        return values;
    }

    @JsonProperty("values")
    public void setValues(ArchiverDataValuesType[] values) {
        this.values = values;
    }
    
    public void copy(ArchiverCounterDataSeriesType that) {
        if (that == null) return;
        
        this.resultSamplingRate = that.resultSamplingRate;
        this.resultStartTime = that.resultStartTime;
        this.resultEndTime = that.resultEndTime;
        this.numberOfRecords = that.numberOfRecords;
        if (that.values == null) {
            this.values = null;
        } else {
            this.values = new ArchiverDataValuesType[that.values.length];
            for(int i=0; i< this.values.length; i++) {
                this.values[i] = new ArchiverDataValuesType(that.values[i]);
            }
        }
    }
}
