/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiData;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebData;

/**
 * Archiver counter data
 *
 * @author Jing Shi
 * @version 1.0, May 9, 2014
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ArchiverCounterDataType extends BaseDataType implements WebData, MapiData {
    /** Name of the counter */
    @XmlElement(required = true, name = "counter-name")
    private String counterName;
    
    /** Meta-data and data for the counters */
    @XmlElement(required = true, name = "archiver-counter-data-series-list")
    private ArchiverCounterDataSeriesType[] archiverCounterDataSeriesList;

    public ArchiverCounterDataType() {}
    
    public ArchiverCounterDataType(String counterName, ArchiverCounterDataSeriesType[] archiverCounterDataSeriesList) {
        super();
        this.counterName = counterName;
        this.archiverCounterDataSeriesList = archiverCounterDataSeriesList;
    }

    public ArchiverCounterDataType(ArchiverCounterDataType that) {
        super();
        copy(that);
    }
    
    @JsonProperty("counter-name")
    public String getCounterName() {
        return counterName;
    }

    @JsonProperty("counter-name")
    public void setCounterName(String counterName) {
        this.counterName = counterName;
    }

    @JsonProperty("archiver-counter-data-series-list")
    public ArchiverCounterDataSeriesType[] getArchiverCounterDataSeriesList() {
        return archiverCounterDataSeriesList;
    }

    @JsonProperty("archiver-counter-data-series-list")
    public void setArchiverCounterDataSeriesList(ArchiverCounterDataSeriesType[] archiverCounterDataSeriesList) {
        this.archiverCounterDataSeriesList = archiverCounterDataSeriesList;
    }    
    
    public void copy(ArchiverCounterDataType that) {
        if (that == null) return;
        
        this.counterName = that.counterName;
        if (that.archiverCounterDataSeriesList == null) {
            this.archiverCounterDataSeriesList =  null;
        } else {
            this.archiverCounterDataSeriesList = new ArchiverCounterDataSeriesType[that.archiverCounterDataSeriesList.length];
            for(int i=0; i< this.archiverCounterDataSeriesList.length; i++) {
                this.archiverCounterDataSeriesList[i] = new ArchiverCounterDataSeriesType(that.archiverCounterDataSeriesList[i]);
            }
        }
    }
}
