/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;


/**
 * Stats show operation input type
 * <p>
 * ${mapi-datatype-desc}
 * </p>
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ArchiverShowInputType extends BaseDataType implements WebInput, MapiInput {
    private static final String s_cluster = "cluster";
    
    /**
     * Begin time (UTC in seconds) to get archived counter samples.<!-- --> <br>
     * When it is absent, API returns the earliest available samples. 
     */
    @XmlElement(required = false, name = "begin-time")
    private Number beginTime;

    /**
     * End time (UTC in seconds) to get archived counter samples.<!-- --> <br>
     * When it is absent, API uses current cluster time as the end time.
     */
    @XmlElement(required = false, name = "end-time")
    private Number endTime;

    /**
     * Name of the counter manager object
     */
    @XmlElement(required = true, name = "object-name")
    protected String objectName;

    /**
     * Names of the counters requested
     */
    @XmlElement(required = true, name = "counter-names")
    protected String[] counterNames;

    /**
     * Scope the statistics across cluster.<!-- --> <br>
     * When it is specified as false or absent, API returns instances counter values
     * and instances must be specified; 
     * otherwise, API returns cluster scope counter values, and instances counter values when instances specified.
     */
    @XmlElement(required = false, name = "cluster-scope")
    protected Boolean clusterScope;
    
    /**
     * Names of the counter manager instances requested.<!-- --> <br>
     * When cluster-scope is false or absent, instances must be specified;
     * otherwise, API returns cluster scope counter values, and instances counter values when instances specified.<!-- --> <br>
     * Maximum number of allowed instances are 4. More than 4 instance names gets error.
     */
    @XmlElement(required = false, name = "instance-names")
    protected String[] instanceNames;

    public ArchiverShowInputType() {}
    
    public ArchiverShowInputType(Long beginTime, Long endTime, String objectName, String[] counterNames, Boolean clusterScope, String[] instanceNames) {
        super();
        this.beginTime = beginTime;
        this.endTime = endTime;
        this.objectName = objectName;
        this.counterNames = counterNames;
        this.clusterScope = clusterScope;
        this.instanceNames = instanceNames;
    }

    @JsonProperty("begin-time")
    public Number getBeginTime() {
        return beginTime;
    }

    @JsonProperty("begin-time")
    public void setBeginTime(Number beginTime) {
        this.beginTime = beginTime;
    }

    @JsonProperty("end-time")
    public Number getEndTime() {
        return endTime;
    }

    @JsonProperty("end-time")
    public void setEndTime(Number endTime) {
        this.endTime = endTime;
    }

    @JsonProperty("object-name")
    public String getObjectName() {
        return objectName;
    }

    @JsonProperty("object-name")
    public void setObjectName(String objectName) {
        this.objectName = objectName;
    }

    @JsonProperty("counter-names")
    public String[] getCounterNames() {
        return counterNames;
    }

    @JsonProperty("counter-names")
    public void setCounterNames(String[] counterNames) {
        this.counterNames = counterNames;
    }

    @JsonProperty("cluster-scope")
    public Boolean getClusterScope() {
        return clusterScope;
    }

    @JsonProperty("cluster-scope")
    public void setClusterScope(Boolean clusterScope) {
        this.clusterScope = clusterScope;
    }

    @JsonProperty("instance-names")
    public String[] getInstanceNames() {
        return instanceNames;
    }

    @JsonProperty("instance-names")
    public void setInstanceNames(String[] instanceNames) {
        this.instanceNames = instanceNames;
    }
}
