/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;
/**
 * Chassis info data type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ChassisInfoType extends BaseDataType implements WebOutput, MapiOutput  {
    /** Chassis name */
    @XmlElement(required = true, name = "name")
    private String name;
    
    /** Serial number.<!-- --> <br>
     * This element is not supported in this release and will always be empty. */
    @XmlElement(required = true, name = "serial-number")
    private String serialNumber = "";
    
    /**  Health state */
    @XmlElement(required = true, name = "health-state")
    private HealthState healthState = HealthState.unknown;
    
    /**  Aggregate health state of fans */
    @XmlElement(required = true, name = "fan-health-state")
    private HealthState fanHealthState = HealthState.unknown;
    
    /**  Aggregate health state of power supplies */
    @XmlElement(required = true, name = "psu-health-state")
    private HealthState psuHealthState = HealthState.unknown;
    
    /**  Part number.<!-- --> <br>
     * This element is not supported in this release and will always be empty.  */
    @XmlElement(required = true, name = "part-number")
    private String partNumber = "";
    
    /**  FRU revision.<!-- --> <br>
     * This element is not supported in this release and will always be empty.*/
    @XmlElement(required = true, name = "revision")
    private String revision = "";

    /**  Controller names */
    @XmlElement(required = true, name = "controller-names")
    private String[] controllerNames = null;
    
    /**  Chassis element status info */
    @XmlElement(required = true, name = "element-status-info")
    private ChassisElementStatusInfoType elementStatusInfo = null;

    public ChassisInfoType() {
        super();
    }


    public ChassisInfoType(String name, String serialNumber,
            HealthState healthState, HealthState fanHealthState,
            HealthState psuHealthState, String partNumber, String revision, String[] controllerNames,
            ChassisElementStatusInfoType elementStatusInfo) {
        super();
        this.name = name;
        this.serialNumber = serialNumber;
        this.healthState = healthState;
        this.fanHealthState = fanHealthState;
        this.psuHealthState = psuHealthState;
        this.partNumber = partNumber;
        this.revision = revision;
        this.controllerNames = controllerNames;
        this.elementStatusInfo = elementStatusInfo;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("serial-number")
    public String getSerialNumber() {
        return serialNumber;
    }

    @JsonProperty("serial-number")
    public void setSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }

    @JsonProperty("health-state")
    public HealthState getHealthState() {
        return healthState;
    }

    @JsonProperty("health-state")
    public void setHealthState(HealthState healthState) {
        this.healthState = healthState;
    }

    @JsonProperty("fan-health-state")
    public HealthState getFanHealthState() {
        return fanHealthState;
    }

    @JsonProperty("fan-health-state")
    public void setFanHealthState(HealthState fanHealthState) {
        this.fanHealthState = fanHealthState;
    }

    @JsonProperty("psu-health-state")
    public HealthState getPsuHealthState() {
        return psuHealthState;
    }

    @JsonProperty("psu-health-state")
    public void setPsuHealthState(HealthState psuHealthState) {
        this.psuHealthState = psuHealthState;
    }

    @JsonProperty("part-number")
    public String getPartNumber() {
        return partNumber;
    }

    @JsonProperty("part-number")
    public void setPartNumber(String partNumber) {
        this.partNumber = partNumber;
    }

    @JsonProperty("revision")
    public String getRevision() {
        return revision;
    }

    @JsonProperty("revision")
    public void setRevision(String revision) {
        this.revision = revision;
    }

    @JsonProperty("controller-names")
    public String[] getControllerNames() {
        return controllerNames;
    }

    @JsonProperty("controller-names")
    public void setControllerNames(String[] controllerNames) {
        this.controllerNames = controllerNames;
    }

    @JsonProperty("element-status-info")
    public ChassisElementStatusInfoType getElementStatusInfo() {
        return elementStatusInfo;
    }

    @JsonProperty("element-status-info")
    public void setElementStatusInfo(
            ChassisElementStatusInfoType elementStatusInfo) {
        this.elementStatusInfo = elementStatusInfo;
    }
}
