/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiInput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebInput;

/**
 * Cluster create operation input type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ClusterCreateInputType extends BaseDataType implements WebInput, MapiInput {
    /**
     * Human-readable name given to the cluster.<!-- -->This is typically the (unqualified) DNS name assigned to the management interface <br>
     * The cluster name is up to 63 characters long, and contains only ASCII letters 'a' through 'z', digits '0' through '9', and a hypen ('-'). The name must start with a letter or digit.
     */
    @XmlElement(required = true, name = "name")
    private String name;

    /**
     * Base64-encoded new password for default admin account, to replace default password.<!-- --> <br>
     * A password is between 8 and 128 characters long, and contains only ASCII letters 'a' through 'z', 'A' through 'Z', digits '0' through '9', '!', '@', '#', '$', '%', '?', '^' and '~'.
     */
    @XmlElement(required = true, name = "password")
    private String password;

    /**
     * Array of serial numbers of the nodes to form the new cluster.<!-- --> <br>
     * If this parameter is specified, the serial number of the node on which the request is being made must be included.<!-- --> <br>
     * If this parameter is not specified, or if it is specified with an empty array, <br>
     * a cluster will be created using all the eligible nodes seen on the cluster network.
     */
    @XmlElement(required = false, name = "node-serialnums")
    private String[] nodeSerialnums;

    public ClusterCreateInputType() {
    }

    public ClusterCreateInputType(String name, String password) {
        this.name = name;
        this.password = password;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("password")
    public String getPassword() {
        return password;
    }

    @JsonProperty("password")
    public void setPassword(String password) {
        this.password = password;
    }

    @JsonProperty("node-serialnums")
    public String[] getNodeSerialnums() {
        return nodeSerialnums;
    }

    @JsonProperty("node-serialnums")
    public void setNodeSerialnums(String[] nodeSerialnums) {
        this.nodeSerialnums = nodeSerialnums;
    }
}
