/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * Controller info data type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
public class ControllerInfoType extends BaseDataType implements WebOutput, MapiOutput  {
    
    /** Controller name */
    @XmlElement(required = true, name = "name")
    private String name;
    
    /** Cluster name */
    @XmlElement(required = true, name = "cluster-name")
    private String clusterName;
    
    /** Chassis name */
    @XmlElement(required = true, name = "chassis-name")
    private String chassisName;
    
    /** List of nodes running on controller. */
    @XmlElement(required = true, name = "nodes-info")
    private NodeInfoType[] nodesInfo;
    
    /** Controller serial number */
    @XmlElement(required = true, name = "serial-number")
    private String serialNumber;
    
    /** Board serial number */
    @XmlElement(required = true, name = "board-serial-number")
    private String boardSerialNumber;
    
    /** System ID */
    @XmlElement(required = true, name = "system-id")
    private String systemId;
    
    /** System ID */
    @XmlElement(required = true, name = "model-name")
    private String modelName;

    /**  Health state */
    @XmlElement(required = true, name = "health-state")
    private HealthState healthState = HealthState.unknown;
    
    /**  Aggregate health state of CPUs */
    @XmlElement(required = true, name = "cpu-health-state")
    private HealthState cpuHealthState = HealthState.unknown;
    
    /**  Aggregate health state of Temperature sensors */
    @XmlElement(required = true, name = "temp-health-state")
    private HealthState tempHealthState = HealthState.unknown;
    
    /**  Aggregate health state of NVRAM sensors */
    @XmlElement(required = true, name = "nvram-health-state")
    private HealthState nvramHealthState = HealthState.unknown;
    
    /**  Aggregate health state of SP (Service Processor) sensors */
    @XmlElement(required = true, name = "sp-health-state")
    private HealthState spHealthState = HealthState.unknown;
    
    /**  Part number */
    @XmlElement(required = true, name = "part-number")
    private String partNumber;
    
    /**  FRU revision */
    @XmlElement(required = true, name = "revision")
    private String revision;
    
    /**  NVRAM size in MB */
    @XmlElement(required = false, name = "nvram-size")
    private Integer nvramSize;
    
    public ControllerInfoType() {
        super();
    }

    public ControllerInfoType(String name, String clusterName,
            String chassisName, NodeInfoType[] nodesInfo,
            String serialNumber, String boardSerialNumber,
            String systemId, String modelName, HealthState healthState,
            HealthState cpuHealthState, HealthState tempHealthState,
            HealthState nvramHealthState, HealthState spHealthState,
            String partNumber, String revision, Integer nvramSize) {
        super();
        this.name = name;
        this.clusterName = clusterName;
        this.chassisName = chassisName;
        this.nodesInfo = nodesInfo;
        this.serialNumber = serialNumber;
        this.boardSerialNumber = boardSerialNumber;
        this.systemId = systemId;
        this.modelName = modelName;
        this.healthState = healthState;
        this.cpuHealthState = cpuHealthState;
        this.tempHealthState = tempHealthState;
        this.nvramHealthState = nvramHealthState;
        this.spHealthState = spHealthState;
        this.partNumber = partNumber;
        this.revision = revision;
        this.nvramSize = nvramSize;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("cluster-name")
    public String getClusterName() {
        return clusterName;
    }

    @JsonProperty("cluster-name")
    public void setClusterName(String clusterName) {
        this.clusterName = clusterName;
    }

    @JsonProperty("chassis-name")
    public String getChassisName() {
        return chassisName;
    }

    @JsonProperty("chassis-name")
    public void setChassisName(String chassisName) {
        this.chassisName = chassisName;
    }

    @JsonProperty("nodes-info")
    public NodeInfoType[] getNodesInfo() {
        return nodesInfo;
    }

    @JsonProperty("nodes-info")
    public void setNodesInfo(NodeInfoType[] nodesInfo) {
        this.nodesInfo = nodesInfo;
    }

    @JsonProperty("serial-number")
    public String getSerialNumber() {
        return serialNumber;
    }

    @JsonProperty("serial-number")
    public void setSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }
    
    @JsonProperty("system-serial-number")
    public void setSystemSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }

    @JsonProperty("board-serial-number")
    public String getBoardSerialNumber() {
        return boardSerialNumber;
    }

    @JsonProperty("board-serial-number")
    public void setBoardSerialNumber(String boardSerialNumber) {
        this.boardSerialNumber = boardSerialNumber;
    }

    @JsonProperty("system-id")
    public String getSystemId() {
        return systemId;
    }

    @JsonProperty("system-id")
    public void setSystemId(String systemId) {
        this.systemId = systemId;
    }

    @JsonProperty("model-name")
    public String getModelName() {
        return modelName;
    }

    @JsonProperty("model-name")
    public void setModelName(String modelName) {
        this.modelName = modelName;
    }

    @JsonProperty("health-state")
    public HealthState getHealthState() {
        return healthState;
    }

    @JsonProperty("health-state")
    public void setHealthState(HealthState healthState) {
        this.healthState = healthState;
    }

    @JsonProperty("cpu-health-state")
    public HealthState getCpuHealthState() {
        return cpuHealthState;
    }

    @JsonProperty("cpu-health-state")
    public void setCpuHealthState(HealthState cpuHealthState) {
        this.cpuHealthState = cpuHealthState;
    }

    @JsonProperty("temp-health-state")
    public HealthState getTempHealthState() {
        return tempHealthState;
    }

    @JsonProperty("temp-health-state")
    public void setTempHealthState(HealthState tempHealthState) {
        this.tempHealthState = tempHealthState;
    }

    @JsonProperty("nvram-health-state")
    public HealthState getNvramHealthState() {
        return nvramHealthState;
    }

    @JsonProperty("nvram-health-state")
    public void setNvramHealthState(HealthState nvramHealthState) {
        this.nvramHealthState = nvramHealthState;
    }

    @JsonProperty("sp-health-state")
    public HealthState getSpHealthState() {
        return spHealthState;
    }

    @JsonProperty("sp-health-state")
    public void setSpHealthState(HealthState spHealthState) {
        this.spHealthState = spHealthState;
    }

    @JsonProperty("part-number")
    public String getPartNumber() {
        return partNumber;
    }

    @JsonProperty("part-number")
    public void setPartNumber(String partNumber) {
        this.partNumber = partNumber;
    }

    @JsonProperty("revision")
    public String getRevision() {
        return revision;
    }

    @JsonProperty("revision")
    public void setRevision(String revision) {
        this.revision = revision;
    }

    @JsonProperty("nvram-size")
    public Integer getNvramSize() {
        return nvramSize;
    }

    @JsonProperty("nvram-size")
    public void setNvramSize(Integer nvramSize) {
        this.nvramSize = nvramSize;
    }
 
}
