/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;


import java.util.Arrays;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * Disk details info data type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class DiskDetailsInfoType extends BaseDataType implements WebOutput, MapiOutput {

    /** Name of the disk */
    @XmlElement(required = true, name = "name")
    private String name;

    /** Disk unique identifier.<!-- --> Maximum length of 90 characters. */
    @XmlElement(required = true, name = "uid")
    private String diskUid;

    /** Firmware revision */
    @XmlElement(required = true, name = "firmware-revision")
    private String firmwareRevision;

    /** Disk state. */
    @XmlElement(required = true, name = "state")
    private DiskState state;

    /** Physical disk size in bytes.<!-- --> Range : [0..2^64-1]. */
    @XmlElement(required = true, name = "physical-space")
    private Long physicalSpace;

    /** Vendor of this disk. */
    @XmlElement(required = true, name = "vendor-id")
    private String vendorId;

    /** Disk model */
    @XmlElement(required = true, name = "product-id")
    private String productId;

    /** Disk serial number.<!-- --> Maximum length of 129 characters. */
    @XmlElement(required = true, name = "serial-number")
    private String serialNumber;

    /** Disk shelf.<!-- --> If disk shelf can't be determined, value will be "?". */
    @XmlElement(required = true, name = "shelf-id")
    private Integer shelfId;

    /** Disk bay.<!-- --> If disk bay cant be determined, value will be "?". */
    @XmlElement(required = true, name = "bay")
    private Integer bay;

    /** Bytes per sector.<!-- --> Range : [0..2^31-1]. */
    @XmlElement(required = true, name = "bytes-per-sector")
    private Integer bytesPerSector;

    /** Info block for solid-state storage devices. */
    @XmlElement(required = false, name = "ssd-info")
    private StorageSsdInfoType ssdInfo;

    /** The percentage of copy already done if a copy is happening. */
    @XmlElement(required = false, name = "copy-percent")
    private Integer copyPercent;

    /** The percentage of copy already done if a zeroing is happening. */
    @XmlElement(required = false, name = "zeroing-percent")
    private Integer zeroingPercent;
    
    /**  The percentage of sanitizing done if it is already is happening. */
    @XmlElement(required = false, name = "sanitizing-percent")
    private Integer sanitizingPercent;

    /**  Disk health state */
    @XmlElement(required = true, name = "health-state")
    private HealthState healthState = HealthState.unknown;
    
    /**  Disk LED status */
    @XmlElement(required = true, name = "led-status")
    private DiskLedStatusEnumType ledStatus = null;

    public DiskDetailsInfoType() {
    }

    public DiskDetailsInfoType(String name, String diskUid, String firmwareRevision, DiskState state, Long physicalSpace, String vendorId, String productId, String serialNumber,
            Integer shelfId, Integer bay, Integer bytesPerSector, StorageSsdInfoType ssdInfo, Integer copyPercent, Integer zeroingPercent, Integer sanitizingPercent, DiskLedStatusEnumType ledStatus) {
        super();
        this.name = name;
        this.diskUid = diskUid;
        this.firmwareRevision = firmwareRevision;
        this.state = state;
        this.physicalSpace = physicalSpace;
        this.vendorId = vendorId;
        this.productId = productId;
        this.serialNumber = serialNumber;
        this.shelfId = shelfId;
        this.bay = bay;
        this.bytesPerSector = bytesPerSector;
        this.ssdInfo = ssdInfo;
        this.copyPercent = copyPercent;
        this.zeroingPercent = zeroingPercent;
        this.sanitizingPercent = sanitizingPercent;
        setHealthState(this.ssdInfo, this.state);
        this.ledStatus = ledStatus;
    }

    @JsonProperty("name")
    public String getName() {
        return this.name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("uid")
    public String getDiskUid() {
        return this.diskUid;
    }

    @JsonProperty("uid")
    public void setDiskUid(String diskUid) {
        this.diskUid = diskUid;
    }

    @JsonProperty("firmware-revision")
    public String getFirmwareRevision() {
        return this.firmwareRevision;
    }

    @JsonProperty("firmware-revision")
    public void setFirmwareRevision(String firmwareRevision) {
        this.firmwareRevision = firmwareRevision;
    }

    @JsonProperty("state")
    public DiskState getState() {
        return this.state;
    }

    @JsonProperty("state")
    public void setState(DiskState state) {
        this.state = state;
        setHealthState(this.ssdInfo, this.state);
    }

    @JsonProperty("physical-space")
    public Long getPhysicalSpace() {
        return this.physicalSpace;
    }

    @JsonProperty("physical-space")
    public void setPhysicalSpace(Long physicalSpace) {
        this.physicalSpace = physicalSpace;
    }

    @JsonProperty("vendor-id")
    public String getVendorId() {
        return this.vendorId;
    }

    @JsonProperty("vendor-id")
    public void setVendorId(String vendorId) {
        this.vendorId = vendorId;
    }

    @JsonProperty("product-id")
    public String getProductId() {
        return this.productId;
    }

    @JsonProperty("product-id")
    public void setProductId(String productId) {
        this.productId = productId;
    }

    @JsonProperty("serial-number")
    public String getSerialNumber() {
        return this.serialNumber;
    }

    @JsonProperty("serial-number")
    public void setSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }

    @JsonProperty("shelf-id")
    public Integer getShelfId() {
        return this.shelfId;
    }

    @JsonProperty("shelf-id")
    public void setShelfId(Integer shelfId) {
        this.shelfId = shelfId;
    }

    @JsonProperty("bay")
    public Integer getBay() {
        return this.bay;
    }

    @JsonProperty("bay")
    public void setBay(Integer bay) {
        this.bay = bay;
    }

    @JsonProperty("bytes-per-sector")
    public Integer getBytesPerSector() {
        return this.bytesPerSector;
    }

    @JsonProperty("bytes-per-sector")
    public void setBytesPerSector(Integer bytesPerSector) {
        this.bytesPerSector = bytesPerSector;
    }

    @JsonProperty("ssd-info")
    public StorageSsdInfoType getSsdInfo() {
        return this.ssdInfo;
    }

    @JsonProperty("ssd-info")
    public void setSsdInfo(StorageSsdInfoType ssdInfo) {
        this.ssdInfo = ssdInfo;
        setHealthState(ssdInfo, this.state);
    }

    @JsonProperty("copy-percent")
    public Integer getCopyPercent() {
        return this.copyPercent;
    }

    @JsonProperty("copy-percent")
    public void setCopyPercent(Integer copyPercent) {
        this.copyPercent = copyPercent;
    }

    @JsonProperty("zeroing-percent")
    public Integer getZeroingPercent() {
        return this.zeroingPercent;
    }

    @JsonProperty("zeroing-percent")
    public void setZeroingPercent(Integer zeroingPercent) {
        this.zeroingPercent = zeroingPercent;
    }
    
    @JsonProperty("sanitizing-percent")
    public Integer getSanitizingPercent() {
        return this.sanitizingPercent;
    }

    @JsonProperty("sanitizing-percent")
    public void setSanitizingPercent(Integer sanitizingPercent) {
        this.sanitizingPercent = sanitizingPercent;
    }

    @JsonProperty("health-state")
    public HealthState getHealthState() {
        return healthState;
    }

    @JsonProperty("health-state")
    public void setHealthState(HealthState healthState) {
        this.healthState = healthState;
    }
    
    @JsonProperty("led-status")
    public DiskLedStatusEnumType getLedStatus() {
        return ledStatus;
    }

    @JsonProperty("led-status")
    public void setLedStatus(DiskLedStatusEnumType ledStatus) {
        this.ledStatus = ledStatus;
    }

    /**
     * Sets the health state of the disk to the worst of the SSD info(wear level) and disk state. 
     * @param ssdInfo
     * @param diskState
     */
    private void setHealthState(StorageSsdInfoType ssdInfo, DiskState diskState) {
        if (ssdInfo != null && diskState != null) {
            if (ssdInfo.getWearLevelState().code() > diskState.getHealthState().code()) {
                this.healthState = ssdInfo.getWearLevelState();
            } else {
                this.healthState = diskState.getHealthState();
            }
        } else if (ssdInfo != null) {
            this.healthState = ssdInfo.getWearLevelState();
        } else if (diskState != null) {
            this.healthState = diskState.getHealthState();
        } else {
            this.healthState = HealthState.unknown;
        }
        
    }
    
    /**
     * Returns a DiskDetailsInfoType object for a specified shelfDeviceElementInfo for a powered off or removed/missing device/SSD.
     * @param shelfDeviceElementInfo
     * @param shelfName
     * @return
     */
    public static DiskDetailsInfoType getDiskFromShelfDeviceElement(ShelfDeviceElementInfo shelfDeviceElementInfo, String shelfName) {
        List<ShelfDeviceElementStatus> deviceStatusList = Arrays.asList(shelfDeviceElementInfo.getDeviceElementStatus());
        if ((!deviceStatusList.contains(ShelfDeviceElementStatus.DeviceOff) &&
                !shelfDeviceElementInfo.getStatus().equals(ShelfElementAggregateState.NotInstalled))) {
            return null;
        }
        String[] tokens = shelfName.split("shelf");
        Integer shelfId = Integer.parseInt(tokens[1]);        
        DiskDetailsInfoType diskDetailsInfo = new DiskDetailsInfoType();
        diskDetailsInfo.setBay(shelfDeviceElementInfo.getReading());
        diskDetailsInfo.setShelfId(shelfId);
        diskDetailsInfo.setName(tokens[0] + tokens[1] + "." + shelfDeviceElementInfo.getReading());
        if (deviceStatusList.contains(ShelfDeviceElementStatus.DeviceOff)) {
            diskDetailsInfo.setState(DiskState.DiskPoweredOff);
        } else if (shelfDeviceElementInfo.getStatus().equals(ShelfElementAggregateState.NotInstalled)) {
            diskDetailsInfo.setState(DiskState.DiskNotExist);
        }        
        diskDetailsInfo.setHealthState(HealthState.warning);
        return diskDetailsInfo;
    }
}