/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written

package com.netapp.offtap3.internal.mars.webmapi.model.domain;


import java.util.HashMap;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebEnum;

/**
 * The different states possible on each disk.
 * 
 * @mapi disk-state
 */
public enum DiskState implements WebEnum {
    DiskNotExist("not-exist", "Disk does not exists."),
    DiskNotReady("not-ready", "Disk is not ready."),
    DiskInitFailed("init-failed", "Disk initialization failed."),
    DiskReady("ready", "Disk is ready."),
    DiskOpen("open", "Disk is open."),
    DiskZeroing("zeroing", "Disk is zeroing."),
    DiskFaulted("faulted", "Disk has faulted."),
    DiskSanitizing("sanitizing", "Disk is sanitizing."),
    DiskFwUpdating("fw-updating", "Disk is updating Firmware."),
    DiskPoweredOff("disk-powered-off", "Disk is powered off.")
    ;

    
    private final String value;
    private final String desc;
    public static Map<String, DiskState> valueDiskStateMap = new HashMap<String, DiskState>();
    
    static {
        for (DiskState ds: DiskState.values()) {
            valueDiskStateMap.put(ds.value(), ds);
        }
        
    }
    private DiskState(String value, String desc) {
        this.value = value;
        this.desc = desc;
    }
    
    public String value() {
        return value;
    }
    
    public String desc() {
        return desc;
    }

    @JsonCreator
    public static DiskState forValue(String str) {
        for(DiskState ds: DiskState.values()) {
            if (ds.value().equals(str)) return ds;
        }
        return null;
    }
    
    public static DiskState getType(String name) {
        for (DiskState type : values()) {
            if (type.name().equalsIgnoreCase(name))
                return type;
        }
        return null;
    }
    
    /**
     * Returns a health state of disk derived from the disk state enum value
     * @return health state of disk
     */
    public HealthState getHealthState() {
        if (equals(DiskReady) || equals(DiskOpen)) {
            return HealthState.ok;
        } else if (equals(DiskZeroing) || equals(DiskSanitizing) || equals(DiskFwUpdating)
                || equals(DiskNotExist) || equals(DiskPoweredOff)) {
            return HealthState.warning;
        } else  if (equals(DiskFaulted) || equals(DiskNotReady) || equals(DiskInitFailed)) {
            return HealthState.critical;
        } else {
            return HealthState.unknown;
        }
    }
}