/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiData;

/**
 * Event monitor system event type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class EmsEventType extends BaseDataType implements MapiData {
    /**
     * ems event version
     */
    @XmlElement(required = true, name = "version")
    private int version;

    /**
     * ems event name separated by dot
     */
    @XmlElement(required = true, name = "name")
    private String name;

    /**
     * ems event sequence number is unique to each event instance
     */
    @XmlElement(required = true, name = "seq-num")
    private Integer seqNum;

    /**
     * ems event number, is unique to the same kind of event
     */
    @XmlElement(required = true, name = "number")
    private String number;

    /**
     * ems event severity
     */
    @XmlElement(required = true, name = "severity")
    private EmsEventSeverity severity;

    /**
     * ems event timestamp is the cluster system time in seconds with long integer format
     */
    @XmlElement(required = true, name = "timestamp")
    private Long timestamp;
    /**
     * ems event description
     */
    @XmlElement(required = true, name = "description")
    private String description;

    /**
     * ems event message param0
     */
    @XmlElement(required = false, name = "param0")
    private String param0;

    /**
     * ems event message param1
     */
    @XmlElement(required = false, name = "param1")
    private String param1;

    /**
     * ems event message param2
     */
    @XmlElement(required = false, name = "param2")
    private String param2;

    /**
     * ems event message param3
     */
    @XmlElement(required = false, name = "param3")
    private String param3;

    /**
     * ems event message param4
     */
    @XmlElement(required = false, name = "param4")
    private String param4;

    /**
     * ems event corrective action
     */
    @XmlElement(required = false, name = "corrective-action")
    private String correctiveAction;

    public EmsEventType() {
    }

    public EmsEventType(Integer version, String name, Integer seqNum, String number, EmsEventSeverity severity, Long timestamp, String description, String correctiveAction) {
        super();
        this.version = version;
        this.name = name;
        this.seqNum = seqNum;
        this.number = number;
        this.severity = severity;
        this.timestamp = timestamp;
        this.description = description;
        this.correctiveAction = correctiveAction;
    }

    @JsonProperty("version")
    public int getVersion() {
        return version;
    }

    @JsonProperty("version")
    public void setVersion(int version) {
        this.version = version;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("seq-num")
    public Integer getSeqNum() {
        return seqNum;
    }

    @JsonProperty("seq-num")
    public void setSeqNum(Integer seqNum) {
        this.seqNum = seqNum;
    }

    @JsonProperty("number")
    public String getNumber() {
        return number;
    }

    @JsonProperty("number")
    public void setNumber(String number) {
        this.number = number;
    }

    @JsonProperty("severity")
    public EmsEventSeverity getSeverity() {
        return severity;
    }

    @JsonProperty("severity")
    public void setSeverity(EmsEventSeverity severity) {
        this.severity = severity;
    }

    @JsonProperty("timestamp")
    public Long getTimestamp() {
        return timestamp;
    }

    @JsonProperty("timestamp")
    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    @JsonProperty("description")
    public String getDescription() {
        return description;
    }

    @JsonProperty("description")
    public void setDescription(String description) {
        this.description = description;
    }

    @JsonProperty("param0")
    public String getParam0() {
        return param0;
    }

    @JsonProperty("param0")
    public void setParam0(String param0) {
        this.param0 = param0;
    }

    @JsonProperty("param1")
    public String getParam1() {
        return param1;
    }

    @JsonProperty("param1")
    public void setParam1(String param1) {
        this.param1 = param1;
    }

    @JsonProperty("param2")
    public String getParam2() {
        return param2;
    }

    @JsonProperty("param2")
    public void setParam2(String param2) {
        this.param2 = param2;
    }

    @JsonProperty("param3")
    public String getParam3() {
        return param3;
    }

    @JsonProperty("param3")
    public void setParam3(String param3) {
        this.param3 = param3;
    }

    @JsonProperty("param4")
    public String getParam4() {
        return param4;
    }

    @JsonProperty("param4")
    public void setParam4(String param4) {
        this.param4 = param4;
    }

    @JsonProperty("corrective-action")
    public String getCorrectiveAction() {
        return correctiveAction;
    }

    @JsonProperty("corrective-action")
    public void setCorrectiveAction(String correctiveAction) {
        this.correctiveAction = correctiveAction;
    }
}