/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.netapp.offtap3.internal.mars.webmapi.util.MappingException;

public enum EthernetPortStatus {
    Startup("STARTUP", "Driver is allocated."),
    Uninitialized("UNINITIALIZED", "Driver is kick started."),
    InitializingFirmware("INITIALIZING_FIRMWARE", "Initializing firmware."),
    LinkNotConnected("LINK_NOT_CONNECTED", "Waiting for link."),
    Online("ONLINE", "Link is up."),
    LinkDisconnected("LINK_DISCONNECTED", "Link is down."),
    Resetting("RESETTING", "Waiting for reset."),
    OfflineHard("OFFLINE_HARD", "Error state. The adapter is offline."),
    OfflinedByUserOrSystem("OFFLINED_BY_USER_OR_SYSTEM", "User offlined the adapter."),
    
    ;
    private final String value;
    private final String desc;
    
    private EthernetPortStatus(String value, String desc) {
        this.value = value;
        this.desc = desc;
    }
    
    public String value() {
        return value;
    }
    
    private String desc() {
        return desc;
    }

    @JsonCreator
    public static EthernetPortStatus forValue(String str) {
        for(EthernetPortStatus ds: EthernetPortStatus.values()) {
            if (ds.value().equals(str)) return ds;
        }
        throw new MappingException("status", str);
    }
    
    public static EthernetPortStatus getType(String name) {
        for (EthernetPortStatus type : values()) {
            if (type.name().equalsIgnoreCase(name))
                return type;
        }
        return null;
    }
}
