/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * FC Port info data type
 */
@XmlRootElement()
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class FcPortInfoType extends PortInfoType implements WebOutput, MapiOutput {
    /** Configured speed */
    @XmlElement(required = true, name = "configured-speed")
    public FcPortConfigSpeed configuredSpeed = null;
    
    /** Port status */
    @XmlElement(required = true)
    public FcPortStatus status = null;

    public FcPortInfoType() {
        this.portType = PortType.fc;
    }
    public FcPortInfoType(String name, String controller, String node,
            PortState state, Integer currentSpeed,
            String hardwareName, HealthState healthState,
            FcPortConfigSpeed configuredSpeed, FcPortStatus status) {
        super(name, controller, node, PortType.fc, state, currentSpeed,
              hardwareName, healthState);
        this.configuredSpeed = configuredSpeed;
        setStatus(status);
    }

    @JsonProperty("configured-speed")
    public FcPortConfigSpeed getConfiguredSpeed() {
        return configuredSpeed;
    }

    @JsonProperty("configured-speed")
    public void setConfiguredSpeed(FcPortConfigSpeed configuredSpeed) {
        this.configuredSpeed = configuredSpeed;
    }

    @JsonProperty("status")
    public FcPortStatus getStatus() {
        return status;
    }

    @JsonProperty("status")
    public void setStatus(FcPortStatus status) {
        this.status = status;
        this.setHealthState(FcPortStatus.getMappedHealthState(status));
    }
    /**
     * 
     * Note: The following methods cannot simply override the base class
     * (PortInfoType) methods, since the JsonProperty names differ from
     * the ones in base class. Hence these are differentiated from base class 
     * methods by adding 'Fc' in their names (e.g. setFcName() for a base class
     * method of 'setName()'.)
     * 
     */
    @JsonProperty("port")
    public void setFcName(String name) {
        this.name = name;
    }
    
    @JsonProperty("port-name")
    public void setFcHardwareName(String hardwareName) {
        this.hardwareName = hardwareName;
    }

    @JsonProperty("data-link-rate")
    public void setFcCurrentSpeed(Integer currentSpeed) {
        this.currentSpeed = currentSpeed;
    }

    @JsonProperty("configured-state")
    public void setFcState(PortState state) {
        this.state = state;
    }
}
