/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.netapp.offtap3.internal.mars.webmapi.util.MappingException;

public enum FcPortStatus {
    Startup("Driver is allocated."),
    Uninitialized("Driver is kick started."),
    InitializingFirmware("Initializing firmware."),
    LinkNotConnected("Waiting for link."),
    Online("Link is up."),
    LinkDisconnected("Link is down."),
    Resetting("Waiting for reset."),
    OfflineHard("Error state. The adapter is offline."),
    OfflinedByUserOrSystem("User offlined the adapter.");

    private final String desc;
    
    private FcPortStatus(String desc) {
        this.desc = desc;
    }

    private String desc() {
        return desc;
    }

    @JsonCreator
    public static FcPortStatus forValue(String str) {
        for(FcPortStatus ds: FcPortStatus.values()) {
            if (ds.name().equals(str)) return ds;
        }
        throw new MappingException("status", str);
    }
    
    public static FcPortStatus getType(String name) {
        for (FcPortStatus type : values()) {
            if (type.name().equalsIgnoreCase(name))
                return type;
        }
        return null;
    }
    
    public static HealthState getMappedHealthState(FcPortStatus status) {
        if (status == FcPortStatus.OfflinedByUserOrSystem) {
            return HealthState.passive;
        } else if (status == FcPortStatus.OfflineHard || status == FcPortStatus.LinkDisconnected) {
            return HealthState.critical;
        } else if (status == FcPortStatus.Startup ||
                status == FcPortStatus.InitializingFirmware ||
                status == FcPortStatus.LinkNotConnected ||
                status == FcPortStatus.Resetting ||
                status == FcPortStatus.Uninitialized
                ) {
            return HealthState.warning;
        } else if (status == FcPortStatus.Online) {
            return HealthState.ok;
        } else {
            return HealthState.unknown;
        }
    }
}
