/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * LIF info type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class LifInfoType extends BaseDataType implements WebOutput, MapiOutput {

    /** LIF name. */
    @XmlElement(required = true, name = "name")
    private String name;
    
    /** Physical port name. */
    @XmlElement(required = true, name = "port-name")
    private String portName;
    
    /** Name of the node hosting the port with this LIF. */
    @XmlElement(required = true, name = "node-name")
    private String nodeName;
    
    /** SCSI Relative Target Port Identifier. */
    @XmlElement(required = true, name = "rtpid")
    private Integer rtpid;
    
    /**
     * Physical layer transport for this LIF.<!-- --> <br>
     * The 'fcoe' and 'iscsi' types are reserved for future use and not yet supported.
     */
    @XmlElement(required = true, name = "transport")
    private ScsiTransportType transport;
    
    /** Administrative state for this LIF. */
    @XmlElement(required = true, name = "state")
    private LifStateType state;
    
    /** Information specific to FC LIFs. */
    @XmlElement(required = false, name = "fc-lif-info")
    private FcInfoType fcLifInfo;
    
    /** Speed of the physical link, in Gb/s. */
    @XmlElement(required = true, name = "speed")
    private Integer speed;
    
    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("port-name")
    public String getPortName() {
        return portName;
    }

    @JsonProperty("port-name")
    public void setPortName(String portName) {
        this.portName = portName;
    }

    @JsonProperty("node-name")
    public String getNodeName() {
        return nodeName;
    }

    @JsonProperty("node-name")
    public void setNodeName(String nodeName) {
        this.nodeName = nodeName;
    }

    @JsonProperty("rtpid")
    public Integer getRtpid() {
        return rtpid;
    }

    @JsonProperty("rtpid")
    public void setRtpid(Integer rtpid) {
        this.rtpid = rtpid;
    }

    @JsonProperty("transport")
    public ScsiTransportType getTransport() {
        return transport;
    }

    @JsonProperty("transport")
    public void setTransport(ScsiTransportType transport) {
        this.transport = transport;
    }

    @JsonProperty("state")
    public LifStateType getState() {
        return state;
    }

    @JsonProperty("state")
    public void setState(LifStateType state) {
        this.state = state;
    }

    @JsonProperty("fc-lif-info")
    public FcInfoType getFcLifInfo() {
        return fcLifInfo;
    }

    @JsonProperty("fc-lif-info")
    public void setFcLifInfo(FcInfoType fcLifInfo) {
        this.fcLifInfo = fcLifInfo;
    }

    @JsonProperty("speed")
    public Integer getSpeed() {
        return speed;
    }

    @JsonProperty("speed")
    public void setSpeed(Integer speed) {
        this.speed = speed;
    }
}
