/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlEnumValue;
import javax.xml.bind.annotation.XmlType;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebEnum;

/**
 * LIF Administrative state
 *
 * @mapi lif-state
 * @author TSANJEEV
 * @version $Revision$, Mar 3, 2014
 */
@XmlType(name = "LifStateType")
@XmlEnum
public enum LifStateType implements WebEnum {
    @XmlEnumValue("offline") 
    offline("offline", "The LIF has been disabled by administrative action"),
    @XmlEnumValue("online") 
    online("online", "The LIF is online"),
    @XmlEnumValue("link-down") 
    linkdown("link-down", "The LIF is online but the physical port is not getting link"),
    @XmlEnumValue("error") 
    error("error", "The LIF is offline because of a problem with the adapter"),
    @XmlEnumValue("initializing") 
    initializing("initializing", "The lif or the physical port is still initializing"),
    @XmlEnumValue("stopped") 
    stopped("stopped", "The lif is offline because the fcp service has been stopped"),
    @XmlEnumValue("port-offline") 
    portoffline("port-offline", "The lif is offline because the fcp port is configured down")
    ;
    
    private final String value;
    private final String desc;

    private LifStateType(String value, String desc) {
        this.value = value;
        this.desc = desc;
    }
    
    public String value() {
        return value;
    }

    public String desc() {
        return desc;
    }
    @JsonCreator
    public static LifStateType forValue(String str) {
        if (str.equals("link-down")) {
            return linkdown;
        } else if (str.equals("port-offline")) {
            return portoffline;
        }
        return getType(str);        
    }
    
    public static LifStateType getType(String name) {
        for (LifStateType type : values()) {
            if (type.name().equalsIgnoreCase(name))
                return type;
        }
        return null;
    }
}
