/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;


import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * Logged in initiator info type for a LIF
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class LoggedInInitiatorInfoType extends BaseDataType implements WebOutput, MapiOutput {
    /** LIF name */
    @XmlElement(required = true, name = "name")
    private String name;
    
    /** Physical port name */
    @XmlElement(required = true, name = "port-name")
    private String portName;
    
    /** Name of the node hosting the port with this LIF */
    @XmlElement(required = true, name = "node-name")
    private String nodeName;
    
    /**
     * Physical layer transport for this LIF.<!-- --> <br>
     * The 'fcoe' and 'iscsi' types are reserved for future use and not yet supported.
     */
    @XmlElement(required = true, name = "transport")
    private ScsiTransportType transport;
    
    /**  List of igroups this initiator belongs to, if any */
    @XmlElement(required = false, name = "igroups")
    private String[] igroups;
    
    /** FC World-Wide Port Name, if transport is FCP */
    @XmlElement(required = false, name = "wwpn")
    private String wwpn;
    
    @JsonProperty("name")
    public String getName() {
        return name;
    }
    
    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }
    
    @JsonProperty("port-name")
    public String getPortName() {
        return portName;
    }
    
    @JsonProperty("port-name")
    public void setPortName(String portName) {
        this.portName = portName;
    }

    @JsonProperty("node-name")
    public String getNodeName() {
        return nodeName;
    }

    @JsonProperty("node-name")
    public void setNodeName(String nodeName) {
        this.nodeName = nodeName;
    }

    @JsonProperty("transport")
    public ScsiTransportType getTransport() {
        return transport;
    }

    @JsonProperty("transport")
    public void setTransport(ScsiTransportType transport) {
        this.transport = transport;
    }

    @JsonProperty("igroups")
    public String[] getIgroups() {
        return igroups;
    }

    @JsonProperty("igroups")
    public void setIgroups(String[] igroups) {
        this.igroups = igroups;
    }

    @JsonProperty("wwpn")
    public String getWwpn() {
        return wwpn;
    }

    @JsonProperty("wwpn")
    public void setWwpn(String wwpn) {
        this.wwpn = wwpn;
    }
}
