/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiInput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebInput;
import com.netapp.offtap3.mars.LunMaxExtentSize;

/**
 * Lun create operation input type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class LunCreateInputType extends BaseDataType implements WebInput, MapiInput {
    /**
     * Name for the LUN.<!-- --> <br>
     * A LUN name is up-to 255 characters long, and contains only ASCII letters 'a' through 'z', 'A' through 'Z', digits '0' through '9', '-', '.', '_', ':', '{', and '}'.
     */
    @XmlElement(required = true)
    private String name;

    /**
     * Size for LUN with an optional multiplier<!-- --> <br>
     * It starts with a digit and ends with an optional multiplier character.<!-- --> <br>
     * Allowed multiplier characters are 'k' or 'K', 'm' or 'M', 'g' or 'G' and 't' or 'T'.<!-- --> <br>
     * k/K : 1024 <br>
     * m/M : 1024k <br>
     * g/G : 1024m <br>
     * t/T : 1024g
     */
    @XmlElement(required = true)
    private String size;

    /**
     * Maximum extent size for the LUN.<!-- --> <br>
     * This setting affects how large writes are broken up
     * into smaller writes or how small writes are combined
     * into large writes before being stored.
     */
    @XmlElement(required = false, name = "max-extent-size")
    private LunMaxExtentSize maxExtentSize = null;

    public LunCreateInputType() {
    }

    public LunCreateInputType(String name, String size) {
        this.name = name;
        this.size = size;
    }

    public LunCreateInputType(String name, String size, LunMaxExtentSize maxExtentSize) {
        this.name = name;
        this.size = size;
        this.maxExtentSize = maxExtentSize;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("size")
    public String getSize() {
        return size;
    }

    @JsonProperty("size")
    public void setSize(String size) {
        this.size = size;
    }

    @JsonProperty("max-extent-size")
    public LunMaxExtentSize getMaxExtentSize() {
        return maxExtentSize;
    }

    @JsonProperty("max-extent-size")
    public void setMaxExtentSize(LunMaxExtentSize maxExtentSize) {
        this.maxExtentSize = maxExtentSize;
    }
}
