/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;


/**
 * Lun info data type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
@JsonIgnoreProperties(value = { "index" })
public class LunInfoType extends BaseDataType implements WebOutput, MapiOutput, UuidDataType, NameType {

    /** LUN uuid */
    @XmlElement(required = true)
    public String uuid = null;

    /** LUN name */
    @XmlElement(required = true)
    private String name = null;

    /** Size of the LUN */
    @XmlElement(required = true)
    private Long size = null;

    /** Maximum extent size for the LUN. <!-- --> <br>
     * This setting affects how large writes are broken up
     * into smaller writes or how small writes are combined
     * into large writes before being stored.
     */
    @XmlElement(required = true, name = "max-extent-size")
    private LunMaxExtentSizeOutput maxExtentSize = null;

    /** Size of user data stored in the LUN */
    @XmlElement(required = false, name = "space-used")
    private Long spaceUsed = null;

    /** State of the LUN */
    @XmlElement(required = false, name = "lun-state")
    private LunStateType lunState = null;
    
    /** Status of the LUN */
    @XmlElement(required = false, name = "lun-status")
    private LunStatusType lunStatus = null;
    
    /** Detailed state of the LUN */
    @XmlElement(required = false, name="lun-detailed-state")
    private LunDetailedStateType lunDetailedState = null;

    /** The serial number of the LUN */
    @XmlElement(required = false)
    private String serial = null;

    /** Size in bytes of the LUN's logical blocks */
    @XmlElement(required = false, name = "block-size")
    private Integer blockSize = null;

    /** List of igroup names that are currently mapped to the LUN */
    @XmlElement(required = false, name = "igroups")
    private String[] igroups = null;
    
    /**  List of LUN Maps for this LUN */
    @XmlElement(required = false, name = "maps")
    private LunMapInfoType[] lunMaps = null;

    /** LUN creation time in seconds since epoch. */
    @XmlElement(required = true, name = "create-time")
    private Integer createTime = null;
    
    /** Name of parent LUN if this is a clone. */
    @XmlElement(required = false, name = "parent")
    private String parent = null;
    
    public LunInfoType() {
    }

    public LunInfoType(Long index, String uuid, String name, Long size, LunMaxExtentSizeOutput maxExtentSize, Long spaceUsed, LunStateType lunState, LunStatusType lunStatus, String serial, Integer blockSize,
            String[] igroups,
            LunMapInfoType[] lunMaps, String cg, Integer createTime, String parent) {
        this.uuid = uuid;
        this.name = name;
        this.size = size;
        this.maxExtentSize = maxExtentSize;
        this.spaceUsed = spaceUsed;
        this.lunState = lunState;
        this.lunStatus = lunStatus;
        this.serial = serial;
        this.blockSize = blockSize;
        this.igroups = igroups;
        this.lunMaps = lunMaps;
        this.lunDetailedState = combineStates(lunState, lunStatus);
        this.createTime = createTime;
        this.parent = parent;
    }

    @JsonProperty("uuid")
    public String getUuid() {
        return uuid;
    }

    @JsonProperty("uuid")
    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("size")
    public Long getSize() {
        return size;
    }

    @JsonProperty("size")
    public void setSize(Long size) {
        this.size = size;
    }

    @JsonProperty("max-extent-size")
    public LunMaxExtentSizeOutput getMaxExtentSize() {
        return maxExtentSize;
    }

    @JsonProperty("max-extent-size")
    public void setMaxExtentSize(LunMaxExtentSizeOutput maxExtentSize) {
        this.maxExtentSize = maxExtentSize;
    }

    @JsonProperty("space-used")
    public Long getSpaceUsed() {
        return spaceUsed;
    }

    @JsonProperty("space-used")
    public void setSpaceUsed(Long spaceUsed) {
        this.spaceUsed = spaceUsed;
    }

    @JsonProperty("lun-state")
    public LunStateType getLunState() {
        return lunState;
    }

    @JsonProperty("lun-state")
    public void setLunState(LunStateType state) {
        this.lunState = state;
        setLunDetailedState(combineStates(state, lunStatus));
    }
    
    @JsonProperty("lun-status")
    public LunStatusType getLunStatus(){
        return lunStatus;
    }
    
    @JsonProperty("lun-status")
    public void setLunStatus(LunStatusType status){
        this.lunStatus = status;
        setLunDetailedState(combineStates(lunState, status));
    }

    @JsonProperty("lun-detailed-state")
    public LunDetailedStateType getLunDetailedState(){
      return lunDetailedState;
    }

    @JsonProperty("lun-detailed-state")
    public void setLunDetailedState(LunDetailedStateType lunDetailedState){
        this.lunDetailedState = lunDetailedState;
    }
    
    @JsonProperty("serial")
    public String getSerial() {
        return serial;
    }

    @JsonProperty("serial")
    public void setSerial(String serial) {
        this.serial = serial;
    }

    @JsonProperty("block-size")
    public Integer getBlockSize() {
        return blockSize;
    }

    @JsonProperty("block-size")
    public void setBlockSize(Integer blockSize) {
        this.blockSize = blockSize;
    }

    @JsonProperty("igroups")
    public String[] getIgroups() {
        return igroups;
    }

    @JsonProperty("igroups")
    public void setIgroups(String[] igroups) {
        this.igroups = igroups;
    }
    
    @JsonProperty("maps")
    public LunMapInfoType[] getMaps() {
        return lunMaps;
    }

    @JsonProperty("maps")
    public void setMaps(LunMapInfoType[] lunMaps) {
        this.lunMaps = lunMaps;
    }
    
    @JsonProperty("create-time")
    public Integer getCreateTime() {
        return createTime;
    }

    @JsonProperty("create-time")
    public void setCreateTime(Integer createTime) {
        this.createTime = createTime;
    }

    @JsonProperty("parent")
    public String getParent() {
        return parent;
    }

    @JsonProperty("parent")
    public void setParent(String parent) {
        this.parent = parent;
    }

    public static LunDetailedStateType combineStates(LunStateType lunState, LunStatusType lunStatus){
        
        //if one of the state/status property is not yet set, do nothing
        if(lunState == null || lunStatus == null)
            return null;
        
        if(lunState == LunStateType.online && lunStatus == LunStatusType.normal){
            return LunDetailedStateType.online;
        }else if(lunState == LunStateType.online && lunStatus == LunStatusType.nospace){
            return LunDetailedStateType.onlineNoSpace;
        }else if(lunState == LunStateType.offline && lunStatus == LunStatusType.normal){
            return LunDetailedStateType.offline;
        }else if(lunState == LunStateType.online && lunStatus == LunStatusType.nvfail){
            return LunDetailedStateType.onlineNvfail;
        }else if(lunState == LunStateType.offline && lunStatus == LunStatusType.nvfail){
            return LunDetailedStateType.offlineNvfail;
        }else if(lunState == LunStateType.offline && lunStatus == LunStatusType.corrupt){
            return LunDetailedStateType.offlineCorrupt;
        }else if(lunState == LunStateType.offline && lunStatus == LunStatusType.nospace){
            return LunDetailedStateType.offlineNoSpace;
        }else{
            return LunDetailedStateType.unknown;
        }
    }
}
