/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * Lun map advanced output type
 *
 * @author Jing Shi
 * @version 1.0, May 12, 2014
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class LunMapAdvancedOutputType extends BaseDataType implements WebOutput {
    /** LUN uuid */
    @XmlElement(required = true)
    public String lunUuid = null;

    /** LUN name */
    @XmlElement(required = true)
    private String name = null;

    /** Size of the LUN */
    @XmlElement(required = true)
    private Long size = null;

    /** Size of user data stored in the LUN */
    @XmlElement(required = false, name = "space-used")
    private Long spaceUsed = null;

    /** State of the LUN */
    @XmlElement(required = false, name = "lun-state")
    private LunStateType lunState = null;
    
    /** Status of the LUN */
    @XmlElement(required = false, name = "lun-status")
    private LunStatusType lunStatus = null;
    
    /** detailed state of the LUN */
    @XmlElement(required = false, name="lun-detailed-state")
    private LunDetailedStateType lunDetailedState = null;
    
    /** The serial number of the LUN */
    @XmlElement(required = false)
    private String serial = null;

    /** Size in bytes of the LUN's logical blocks */
    @XmlElement(required = false, name = "block-size")
    private Integer blockSize = null;
    
    /** List of igroup names that are currently mapped to the LUN */
    @XmlElement(required = false, name = "igroups")
    private String[] igroups = null;
    
    /**  List of LUN Maps for this LUN */
    @XmlElement(required = false, name = "maps")
    private LunMapInfoType[] lunMaps = null;
    
    /** Igroup name */
    @XmlElement(required = true, name = "igroup-name")
    private String igroupName;

    /** Logical unit number */
    @XmlElement(required = true, name = "lun-id")
    private Integer lunId;

   
    public LunMapAdvancedOutputType() {}
    
    public LunMapAdvancedOutputType(String lunUuid, String name, Long size, Long spaceUsed, LunStateType lunState, LunStatusType lunStatus, String serial, Integer blockSize, String[] igroups, LunMapInfoType[] lunMaps, String igroupName, Integer lunId) {
        super();
        this.lunUuid = lunUuid;
        this.name = name;
        this.size = size;
        this.spaceUsed = spaceUsed;
        this.lunState = lunState;
        this.lunStatus = lunStatus;
        this.serial = serial;
        this.blockSize = blockSize;
        this.igroups = igroups;
        this.lunMaps = lunMaps;
        this.igroupName = igroupName;
        this.lunId = lunId;
        this.lunDetailedState = LunInfoType.combineStates(lunState, lunStatus);
    }



    public LunMapAdvancedOutputType(LunInfoType lun, LunMapInfoType map) {
        if (lun != null) {
            this.lunUuid = lun.getUuid();
            this.name = lun.getName();
            this.size = lun.getSize();
            this.spaceUsed = lun.getSpaceUsed();
            this.lunState = lun.getLunState();
            this.lunStatus = lun.getLunStatus();
            this.serial = lun.getSerial();
            this.blockSize = lun.getBlockSize();
            this.igroups = lun.getIgroups();
            this.lunMaps = lun.getMaps();
            this.lunDetailedState = LunInfoType.combineStates(lunState, lunStatus);
        } 
        
        if (map != null) {
            this.igroupName = map.getIgroupName();
            this.lunId = map.getLunId();
        }
    }
    
    public String getLunUuid() {
        return lunUuid;
    }

    public void setLunUuid(String lunUuid) {
        this.lunUuid = lunUuid;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Long getSize() {
        return size;
    }

    public void setSize(Long size) {
        this.size = size;
    }

    public Long getSpaceUsed() {
        return spaceUsed;
    }

    public void setSpaceUsed(Long spaceUsed) {
        this.spaceUsed = spaceUsed;
    }

    public LunStateType getLunState(){
        return lunState;
    }
    
    public void setLunState(LunStateType state){
        this.lunState = state;
    }
    
    public LunStatusType getLunStatus(){
        return lunStatus;
    }
    
    public void setLunStatus(LunStatusType status){
        this.lunStatus = status;
    }
    
    public LunDetailedStateType getLunDetailedState(){
        return lunDetailedState;
    }

    public void setLunDetailedState(LunDetailedStateType lunDetailedState){
        this.lunDetailedState = lunDetailedState;
    }
    
    public String getSerial() {
        return serial;
    }

    public void setSerial(String serial) {
        this.serial = serial;
    }

    public Integer getBlockSize() {
        return blockSize;
    }

    public void setBlockSize(Integer blockSize) {
        this.blockSize = blockSize;
    }

    public String[] getIgroups() {
        return igroups;
    }

    public void setIgroups(String[] igroups) {
        this.igroups = igroups;
    }

    public LunMapInfoType[] getLunMaps() {
        return lunMaps;
    }

    public void setLunMaps(LunMapInfoType[] lunMaps) {
        this.lunMaps = lunMaps;
    }

    public String getIgroupName() {
        return igroupName;
    }

    public void setIgroupName(String igroupName) {
        this.igroupName = igroupName;
    }

    public Integer getLunId() {
        return lunId;
    }

    public void setLunId(Integer lunId) {
        this.lunId = lunId;
    }

}
