/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;

/**
 * Node details info data type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class NodeDetailsInfoType extends BaseDataType implements WebData, MapiData {

    /** Name of the node */
    @XmlElement(required = true)
    private String name = null;

    /** node unique identifier */
    @XmlElement(required = true, name = "uuid")
    public String uuid = null;

    /** product id of the node/controller id */
    @XmlElement(required = false, name = "product-id")
    public String productId = null;

    /** vendor id of the node */
    @XmlElement(required = false, name = "vendor-id")
    public String vendorId = null;

    /** Node Module A firmware revision */
    @XmlElement(required = true, name = "firmware")
    public String firmware = null;

    /** Modules within the node */
    @XmlElement(required = true, name = "module")
    private NodeModule module = null;

    /** Node state */
    @XmlElement(required = true, name = "node-state")
    private NodeState nodeState = null;

    /** Node led state */
    @XmlElement(required = true, name = "node-led-state")
    private NodeLedState nodeLedState = null;

    /** UIDS of shelves that are in the node */
    @XmlElement(required = true, name = "shelf-list")
    private String[] shelfList = null;

    /** Connectors information */
    @XmlElement(required = false, name = "connector-information")
    private NodeConnectorInfoType connectorInformation = null;

    /** Is Power on */
    @XmlElement(required = false, name = "power-on")
    private Boolean powerOn = null;

    /** SP status */
    @XmlElement(required = false, name = "sp-status")
    private String spStatus = null;

    /** NV battery status */
    @XmlElement(required = false, name = "nv-battery-status")
    private String nvBatteryStatus = null;

    /** Service disk status */
    @XmlElement(required = false, name = "service-disk-status")
    private String serviceDiskStatus = null;

    public NodeDetailsInfoType() {
    }

    public NodeDetailsInfoType(String name, String uuid, String productId, String vendorId, String firmware, NodeModule module, NodeState nodeState, NodeLedState nodeLedState, String[] shelfList,
            NodeConnectorInfoType connectorInformation, Boolean powerOn, String spStatus, String nsBatteryStatus, String serviceDiskStatus) {
        super();
        this.name = name;
        this.uuid = uuid;
        this.productId = productId;
        this.vendorId = vendorId;
        this.firmware = firmware;
        this.module = module;
        this.nodeState = nodeState;
        this.nodeLedState = nodeLedState;
        this.shelfList = shelfList;
        this.connectorInformation = connectorInformation;
        this.powerOn = powerOn;
        this.spStatus = spStatus;
        this.nvBatteryStatus = nsBatteryStatus;
        this.serviceDiskStatus = serviceDiskStatus;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public String getProductId() {
        return productId;
    }

    public void setProductId(String productId) {
        this.productId = productId;
    }

    public String getVendorId() {
        return vendorId;
    }

    public void setVendorId(String vendorId) {
        this.vendorId = vendorId;
    }

    public String getFirmware() {
        return firmware;
    }

    public void setFirmware(String firmware) {
        this.firmware = firmware;
    }

    public NodeModule getModule() {
        return module;
    }

    public void setModule(NodeModule module) {
        this.module = module;
    }

    public NodeState getNodeState() {
        return nodeState;
    }

    public void setNodeState(NodeState nodeState) {
        this.nodeState = nodeState;
    }

    public NodeLedState getNodeLedState() {
        return nodeLedState;
    }

    public void setNodeLedState(NodeLedState nodeLedState) {
        this.nodeLedState = nodeLedState;
    }

    public String[] getShelfList() {
        return shelfList;
    }

    public void setShelfList(String[] shelfList) {
        this.shelfList = shelfList;
    }

    public NodeConnectorInfoType getConnectorInformation() {
        return connectorInformation;
    }

    public void setConnectorInformation(NodeConnectorInfoType connectorInformation) {
        this.connectorInformation = connectorInformation;
    }

    public Boolean getPowerOn() {
        return powerOn;
    }

    public void setPowerOn(Boolean powerOn) {
        this.powerOn = powerOn;
    }

    public String getSpStatus() {
        return spStatus;
    }

    public void setSpStatus(String spStatus) {
        this.spStatus = spStatus;
    }

    public String getNvBatteryStatus() {
        return nvBatteryStatus;
    }

    public void setNvBatteryStatus(String nsBatteryStatus) {
        this.nvBatteryStatus = nsBatteryStatus;
    }

    public String getServiceDiskStatus() {
        return serviceDiskStatus;
    }

    public void setServiceDiskStatus(String serviceDiskStatus) {
        this.serviceDiskStatus = serviceDiskStatus;
    }

}