/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;


import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;

/**
 * RAID status information
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class RaidStatusInfoType extends BaseDataType implements WebOutput, MapiOutput {
    /** RAID Group ID */
    @XmlElement(required = true, name = "id")
    private Integer id;
    
    /** RAID state */
    @XmlElement(required = true, name = "state")
    private RaidState state;
    
    /** RAID Group status */
    @XmlElement(required = true, name = "status")
    private RaidStatus status;
    
    /** Percentage reconstructed */
    @XmlElement(required = false, name = "recons-progress-percent")
    private Integer reconsProgressPercent;
    
    /** Number of disks needed to return to normal RG size */
    @XmlElement(required = true, name = "num-missing-disks")
    private Integer numMissingDisks;
    
    /** Health state of RAID group */
    @XmlElement(required = true, name = "health-state")
    private HealthState healthState;
    
    /**
     * Estimated time required to complete reconstruction, in seconds.<!-- --> <br>
     * Only available when the status is 'reconstructing'.
     */
    @XmlElement(required = false, name = "estimated-remaining-time")
    private Long estimatedTimeRemaining;

    public RaidStatusInfoType(){
        
    }
    public RaidStatusInfoType(Integer id, RaidState state,
            RaidStatus status, Integer reconsProgressPercent,
            Integer numMissingDisks, HealthState healthState,
            Long stateChangeTimestamp,
            Long estimatedTimeRemaining) {
        super();
        this.id = id;
        this.state = state;
        this.status = status;
        this.reconsProgressPercent = reconsProgressPercent;
        this.numMissingDisks = numMissingDisks;
        this.healthState = healthState;
        this.estimatedTimeRemaining = estimatedTimeRemaining;
    }
    
    @JsonProperty("id")
    public Integer getId() {
        return id;
    }
    
    @JsonProperty("id")
    public void setId(Integer id) {
        this.id = id;
    }
    
    @JsonProperty("state")
    public RaidState getState() {
        return state;
    }
    
    @JsonProperty("state")
    public void setState(RaidState state) {
        this.state = state;
    }
    
    @JsonProperty("status")
    public RaidStatus getStatus() {
        return status;
    }
    
    @JsonProperty("status")
    public void setStatus(RaidStatus status) {
        this.status = status;
    }
    
    @JsonProperty("recons-progress-percent")
    public Integer getReconsProgressPercent() {
        return reconsProgressPercent;
    }
    
    @JsonProperty("recons-progress-percent")
    public void setReconsProgressPercent(Integer reconsProgressPercent) {
        this.reconsProgressPercent = reconsProgressPercent;
    }
    
    @JsonProperty("num-missing-disks")
    public Integer getNumMissingDisks() {
        return numMissingDisks;
    }
    
    @JsonProperty("num-missing-disks")
    public void setNumMissingDisks(Integer numMissingDisks) {
        this.numMissingDisks = numMissingDisks;
    }
    
    @JsonProperty("health-state")
    public HealthState getHealthState() {
        return healthState;
    }
    
    @JsonProperty("health-state")
    public void setHealthState(HealthState healthState) {
        this.healthState = healthState;
    }
    
    @JsonProperty("estimated-remaining-time")
    public Long getEstimatedTimeRemaining() {
        return estimatedTimeRemaining;
    }
    
    @JsonProperty("estimated-remaining-time")
    public void setEstimatedTimeRemaining(Long estimatedTimeRemaining) {
        this.estimatedTimeRemaining = estimatedTimeRemaining;
    }
}
