/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;

/**
 * SAS connector information
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class SasConnectorInfoType extends BaseDataType implements WebData, MapiData {

    /** Cable end identifier<!-- --> 
     * <br>Each cable has two ends.<!-- --> This field shows which end of the cable is connected to the shelf.<!-- --> This field will not be present if the information is not available or accessible, or if a cable is not connected at this connector.<!-- --> Possible values: "end_0", "end_1".
     * */
    @XmlElement(required = false, name = "cable-end-identifier")
    private String cableEndIdentifier = null;
    
    /** Cable length.<!-- -->
     * <br> This field will not be present if the information is not available or accessible, or if a cable is not connected at this connector.
     * */
    @XmlElement(required = false, name = "cable-length")
    private String cableLength = null;
    
    /** Part number of the cable as assigned by the cable manufacturer.<!-- -->
     * <br> This field will not be present if the information is not available, or if a cable is not connected at this connector.
     * */
    @XmlElement(required = false, name = "cable-part-no")
    private String cablePartNo = null;
    
    /** Serial number of the cable as assigned by the cable manufacturer.<!-- -->
     * <br> This field will not be present if the information is not available, or if a cable is not connected at this connector.
     * */
    @XmlElement(required = false, name = "cable-serial-no")
    private String cableSerialNo = null;
    
    /** Cable technology.<!-- -->
     * <br> This field will not be present if the information is not available, or if a cable is not connected at this connector.<!-- --> Possible values: "copper", "optical".
     * */
    @XmlElement(required = false, name = "cable-technology")
    private String cableTechnology = null;
    
    /** Type of the cable being used.<!-- -->
     * <br> This field will not be present if the information is not available, or if a cable is not connected at this connector.<!-- --> Possible values: "qsfp".
     * */
    @XmlElement(required = false, name = "cable-type")
    private String cableType = null;
    
    /** Connector designator.<!-- -->
     * <br> "sqr" - The port is marked with a square on the shelf connector panel.<!-- --> "cir" - The port is marked with a circle on the shelf connector panel.<!-- --> Will be missing if unknown or not available.
     * */
    @XmlElement(required = false, name = "connector-designator")
    private String connectorDesignator = null;
    
    /** Connector number*/
    @XmlElement(required = true, name = "connector-no")
    private Integer connectorNo = null;
    
    /** Indicates whether a cable is connected at this connector location.<!-- -->
     * <br> No further information will be provided if a cable is not connected at this connector location.<!-- --> Note that a cable connection does not necessarily mean that the shelf is connected to a storage controller.<!-- --> This will also depend on the other end of the cable.
     * */
    @XmlElement(required = true, name = "is-cable-connected")
    private Boolean isCableConnected = null;

    /** Indicates whether the connector element has encountered an error.*/
    @XmlElement(required = false, name = "is-connector-error")
    private Boolean isConnectorError = null;
    
    /** Shelf module for this connector.<!-- -->
     * <br> "a" - Presenting data for shelf module A.<!-- --> "b" - Presenting data for shelf module B.
     * */
    @XmlElement(required = false, name = "shelf-module")
    private String shelfModule = null;
    
    /** Number of times, since last boot, a cable has been inserted into this connector. */
    @XmlElement(required = false, name = "swap-count")
    private Integer swapCount = null;
    
    public SasConnectorInfoType(){}

    public SasConnectorInfoType(String cableEndIdentifier,
            String cableLength, String cablePartNo, String cableSerialNo,
            String cableTechnology, String cableType,
            String connectorDesignator, Integer connectorNo,
            Boolean isCableConnected, Boolean isConnectorError,
            String shelfModule, Integer swapCount) {
        super();
        this.cableEndIdentifier = cableEndIdentifier;
        this.cableLength = cableLength;
        this.cablePartNo = cablePartNo;
        this.cableSerialNo = cableSerialNo;
        this.cableTechnology = cableTechnology;
        this.cableType = cableType;
        this.connectorDesignator = connectorDesignator;
        this.connectorNo = connectorNo;
        this.isCableConnected = isCableConnected;
        this.isConnectorError = isConnectorError;
        this.shelfModule = shelfModule;
        this.swapCount = swapCount;
    }

    @JsonProperty("cable-end-identifier")
    public String getCableEndIdentifier() {
        return cableEndIdentifier;
    }

    @JsonProperty("cable-end-identifier")
    public void setCableEndIdentifier(String cableEndIdentifier) {
        this.cableEndIdentifier = cableEndIdentifier;
    }

    @JsonProperty("cable-length")
    public String getCableLength() {
        return cableLength;
    }

    @JsonProperty("cable-length")
    public void setCableLength(String cableLength) {
        this.cableLength = cableLength;
    }

    @JsonProperty("cable-part-no")
    public String getCablePartNo() {
        return cablePartNo;
    }

    @JsonProperty("cable-part-no")
    public void setCablePartNo(String cablePartNo) {
        this.cablePartNo = cablePartNo;
    }

    @JsonProperty("cable-serial-no")
    public String getCableSerialNo() {
        return cableSerialNo;
    }

    @JsonProperty("cable-serial-no")
    public void setCableSerialNo(String cableSerialNo) {
        this.cableSerialNo = cableSerialNo;
    }

    @JsonProperty("cable-technology")
    public String getCableTechnology() {
        return cableTechnology;
    }

    @JsonProperty("cable-technology")
    public void setCableTechnology(String cableTechnology) {
        this.cableTechnology = cableTechnology;
    }

    @JsonProperty("cable-type")
    public String getCableType() {
        return cableType;
    }

    @JsonProperty("cable-type")
    public void setCableType(String cableType) {
        this.cableType = cableType;
    }

    @JsonProperty("connector-designator")
    public String getConnectorDesignator() {
        return connectorDesignator;
    }

    @JsonProperty("connector-designator")
    public void setConnectorDesignator(String connectorDesignator) {
        this.connectorDesignator = connectorDesignator;
    }

    @JsonProperty("connector-no")
    public Integer getConnectorNo() {
        return connectorNo;
    }

    @JsonProperty("connector-no")
    public void setConnectorNo(Integer connectorNo) {
        this.connectorNo = connectorNo;
    }

    @JsonProperty("is-cable-connected")
    public Boolean getIsCableConnected() {
        return isCableConnected;
    }

    @JsonProperty("is-cable-connected")
    public void setIsCableConnected(Boolean isCableConnected) {
        this.isCableConnected = isCableConnected;
    }

    @JsonProperty("is-connector-error")
    public Boolean getIsConnectorError() {
        return isConnectorError;
    }

    @JsonProperty("is-connector-error")
    public void setIsConnectorError(Boolean isConnectorError) {
        this.isConnectorError = isConnectorError;
    }

    @JsonProperty("shelf-module")
    public String getShelfModule() {
        return shelfModule;
    }

    @JsonProperty("shelf-module")
    public void setShelfModule(String shelfModule) {
        this.shelfModule = shelfModule;
    }

    @JsonProperty("swap-count")
    public Integer getSwapCount() {
        return swapCount;
    }

    @JsonProperty("swap-count")
    public void setSwapCount(Integer swapCount) {
        this.swapCount = swapCount;
    }
    
    

}