/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * FC Port info data type
 */
@XmlRootElement()
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class SasPortInfoType extends PortInfoType implements WebOutput, MapiOutput {
    /** Configured speed */
    @XmlElement(required = true, name = "configured-speed")
    public SasPortConfigSpeed configuredSpeed = null;
    
    /** Port status */
    @XmlElement(required = true)
    public SasPortStatus status = null;

    public SasPortInfoType() {
        this.portType = PortType.sas;
    }
    public SasPortInfoType(String name, String controller, String node,
            PortState state, Integer currentSpeed,
            String hardwareName, HealthState healthState,
            SasPortConfigSpeed configuredSpeed, SasPortStatus status) {
        super(name, controller, node, PortType.sas, state, currentSpeed,
              hardwareName, healthState);
        this.configuredSpeed = configuredSpeed;
        this.status = status;
    }

    @JsonProperty("configured-speed")
    public SasPortConfigSpeed getConfiguredSpeed() {
        return configuredSpeed;
    }

    @JsonProperty("configured-speed")
    public void setConfiguredSpeed(SasPortConfigSpeed configuredSpeed) {
        this.configuredSpeed = configuredSpeed;
    }

    @JsonProperty("status")
    public SasPortStatus getStatus() {
        return status;
    }

    @JsonProperty("status")
    public void setStatus(SasPortStatus status) {
        this.status = status;
    }
    
    /**
     * 
     * Note: The following methods cannot simply override the base class
     * (PortInfoType) methods, since the JsonProperty names differ from
     * the ones in base class. Hence these are differentiated from base class 
     * methods by adding 'Sas' in their names (e.g. setSasHardwareName() for a base class
     * method of 'setHardwareName()'.)
     * 
     */
    
    @JsonProperty("uuid")
    public void setSasHardwareName(String hardwareName) {
        this.hardwareName = hardwareName;
    }
    
    @JsonProperty("state")
    public void setSasStatus(SasPortStatus status) {
        this.status = status;
    }
    
    @JsonProperty("node-name")
    public void setSasNode(String node) {
        this.node = node;
    }
    
    @JsonProperty("adapter-health")
    public void setSasHealthState(HealthState healthState) {
        this.healthState = healthState;
    }
    
    @JsonProperty("enabled")
    public void setSasState(PortState state) {
        this.state = state;
    }
}
