/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiData;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebData;

/**
 * Information on cooling elements (fans).
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ShelfCoolingElementInfoType extends BaseDataType implements WebData, MapiData {

    /** 
     * Indicates whether cooling elements has failed or reported any errors.<!-- -> Will not be present if cooling element is not installed.
     * 
     * */
    @XmlElement(required = false, name = "cooling-element-is-error")
    private Boolean coolingElementIsError = null;
    
    /** 
     * Indicates the cooling elements is not installed.<!-- -> This will only be present if the element is missing, in such case no further data for this element will be presented.
     * 
     * */
    @XmlElement(required = false, name = "cooling-element-is-not-installed")
    private Boolean coolingElementIsNotInstalled = null;
    
    /** Cooling element number */
    @XmlElement(required = true, name = "cooling-element-number")
    private Integer coolingElementNumber = null;
    
    /** Current RPM of the fan */
    @XmlElement(required = false, name = "rpm")
    private Integer rpm = null;
    
    public ShelfCoolingElementInfoType() {
    }
    
    public ShelfCoolingElementInfoType(
            Boolean coolingElementIsError, 
            Boolean coolingElementIsNotInstalled,
            Integer coolingElementNumber,
            Integer rpm) {
        this.coolingElementIsError = coolingElementIsError;
        this.coolingElementIsNotInstalled = coolingElementIsNotInstalled;
        this.coolingElementNumber = coolingElementNumber;
        this.rpm = rpm;
    }

    @JsonProperty("cooling-element-is-error")
    public Boolean getCoolingElementIsError() {
        return coolingElementIsError;
    }

    @JsonProperty("cooling-element-is-error")
    public void setCoolingElementIsError(Boolean coolingElementIsError) {
        this.coolingElementIsError = coolingElementIsError;
    }

    @JsonProperty("cooling-element-is-not-installed")
    public Boolean getCoolingElementIsNotInstalled() {
        return coolingElementIsNotInstalled;
    }

    @JsonProperty("cooling-element-is-not-installed")
    public void setCoolingElementIsNotInstalled(
            Boolean coolingElementIsNotInstalled) {
        this.coolingElementIsNotInstalled = coolingElementIsNotInstalled;
    }

    @JsonProperty("cooling-element-number")
    public Integer getCoolingElementNumber() {
        return coolingElementNumber;
    }

    @JsonProperty("cooling-element-number")
    public void setCoolingElementNumber(Integer coolingElementNumber) {
        this.coolingElementNumber = coolingElementNumber;
    }

    @JsonProperty("rpm")
    public Integer getRpm() {
        return rpm;
    }

    @JsonProperty("rpm")
    public void setRpm(Integer rpm) {
        this.rpm = rpm;
    }
    
}
