/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;

/**
 * Electrical sensor information
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ShelfCurrentSensorInfoType extends BaseDataType implements WebData, MapiData {
    /** Current sensor number */
    @XmlElement(required = true, name = "current-sensor-no")
    private Integer currentSensorNo = null;

    /** Will indicate whether the sensor has encountered an error */
    @XmlElement(required = false, name = "is-sensor-error")
    private Boolean isSensorError = null;
    
    /** Indicates the current sensor elements is not installed.<!-- -->
     * <br> This will only be present if the element is missing, in such case no further data for this element will be presented.
     */
    @XmlElement(required = false, name = "is-sensor-not-installed")
    private Boolean isSensorNotInstalled = null;

    /** A Text string describing whether the sensor-reading field is within normal operating range.<!-- -->
     * <br>This field is present only if sensor-reading field is also present.<!-- --> Possible values are: "overcurrent_failure", "overcurrent_warning", "normal_operating_range".
     * */
    @XmlElement(required = false, name = "sensor-condition")
    private String sensorCondition = null;
    
    /** Current reading of the sensor.<!-- -->
     * <br> This field will be not be present if unable to read the sensor value properly.
     * */
    @XmlElement(required = false, name = "sensor-reading")
    private String sensorReading = null;

    public ShelfCurrentSensorInfoType() {
    }

    public ShelfCurrentSensorInfoType(Integer currentSensorNo,
            Boolean isSensorError, Boolean isSensorNotInstalled,
            String sensorCondition, String sensorReading) {
        super();
        this.currentSensorNo = currentSensorNo;
        this.isSensorError = isSensorError;
        this.isSensorNotInstalled = isSensorNotInstalled;
        this.sensorCondition = sensorCondition;
        this.sensorReading = sensorReading;
    }

    @JsonProperty("current-sensor-no")
    public Integer getCurrentSensorNo() {
        return currentSensorNo;
    }

    @JsonProperty("current-sensor-no")
    public void setCurrentSensorNo(Integer currentSensorNo) {
        this.currentSensorNo = currentSensorNo;
    }

    @JsonProperty("is-sensor-error")
    public Boolean getIsSensorError() {
        return isSensorError;
    }

    @JsonProperty("is-sensor-error")
    public void setIsSensorError(Boolean isSensorError) {
        this.isSensorError = isSensorError;
    }

    @JsonProperty("is-sensor-not-installed")
    public Boolean getIsSensorNotInstalled() {
        return isSensorNotInstalled;
    }

    @JsonProperty("is-sensor-not-installed")
    public void setIsSensorNotInstalled(Boolean isSensorNotInstalled) {
        this.isSensorNotInstalled = isSensorNotInstalled;
    }
    
    @JsonProperty("sensor-condition")
    public String getSensorCondition() {
        return sensorCondition;
    }

    @JsonProperty("sensor-condition")
    public void setSensorCondition(String sensorCondition) {
        this.sensorCondition = sensorCondition;
    }

    @JsonProperty("sensor-reading")
    public String getSensorReading() {
        return sensorReading;
    }

    @JsonProperty("sensor-reading")
    public void setSensorReading(String sensorReading) {
        this.sensorReading = sensorReading;
    }
    
}