/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiOutput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * Shelf details info data type
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ShelfDetailsInfoType extends BaseDataType implements WebOutput, MapiOutput {
    /** Name of the shelf */
    @XmlElement(required = true)
    private String name = null;

    /** ID of the shelf */
    @XmlElement(required = true, name = "shelf-id")
    private Integer shelfId = null;
    
    /** shelf unique identifier */
    @XmlElement(required = true, name = "shelf-uid")
    public String shelfUid = null;

    /** product id of the shelf */
    @XmlElement(required = true, name = "product-id")
    public String productId = null;

    /** vendor id of the shelf */
    @XmlElement(required = true, name = "vendor-id")
    public String vendorId = null;

    /** The back plane serial number. */
    @XmlElement(required = true, name = "serial-no")
    public String  serialNo = null;

    /** Shelf Module A firmware revision */
    @XmlElement(required = true, name = "firmware-A")
    public String firmwareA = null;

    /** Shelf Module B firmware revision */
    @XmlElement(required = true, name = "firmware-B")
    public String firmwareB = null;

    /** Modules within the shelf */
    @XmlElement(required = true, name = "module-a")
    private ShelfModule moduleA = null;

    /** Modules within the shelf */
    @XmlElement(required = true, name = "module-a-state")
    private ShelfModuleState moduleAState = null;

    /** Modules within the shelf */
    @XmlElement(required = true, name = "module-b")
    private ShelfModule moduleB = null;

    /** Modules within the shelf */
    @XmlElement(required = true, name = "module-b-state")
    private ShelfModuleState moduleBState = null;

    /** Shelf state */
    @XmlElement(required = true, name = "shelf-state")
    private ShelfState shelfState = null;

    /** Shelf health state */
    @XmlElement(required = true, name = "health-state")
    private HealthState healthState = null;
    
    /** UUIDS of disk that are in the shelf */
    @XmlElement(required = true, name = "bay-list")
    private String[] bayList = null;
    
    /** Aggregate sensor status info list */
    @XmlElement(required = true, name = "aggregate-element-status-info")
    private ShelfAggregateStatusInfoType aggregateElementStatusInfo = null;
    
    /** Shelf element status info */
    @XmlElement(required = true, name = "element-status-info")
    private ShelfElementStatusInfoType shelfElementStatusInfo = null;
    
    public ShelfDetailsInfoType() {
    }

    public ShelfDetailsInfoType(String name, Integer shelfId, String shelfUid, String productId, String vendorId, String  serialNo, String firmwareA, String firmwareB, ShelfModule moduleA,
            ShelfModuleState moduleAState, ShelfModule moduleB, ShelfModuleState moduleBState, ShelfState shelfState, HealthState shelfHealthState, String[] bayList, 
            ShelfAggregateStatusInfoType aggregateElementStatusInfo, ShelfElementStatusInfoType shelfElementStatusInfo) {
        super();
        this.name = name;
        this.shelfId = shelfId;
        this.shelfUid = shelfUid;
        this.productId = productId;
        this.vendorId = vendorId;
        this.serialNo = serialNo;
        this.firmwareA = firmwareA;
        this.firmwareB = firmwareB;
        this.moduleA = moduleA;
        this.moduleAState = moduleAState;
        this.moduleB = moduleB;
        this.moduleBState = moduleBState;
        this.shelfState = shelfState;
        this.healthState = shelfHealthState;
        this.bayList = bayList;
        this.aggregateElementStatusInfo = aggregateElementStatusInfo;
        this.shelfElementStatusInfo = shelfElementStatusInfo;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

    @JsonProperty("shelf-id")
    public Integer getShelfId() {
        return shelfId;
    }

    @JsonProperty("shelf-id")
    public void setShelfId(Integer shelfId) {
        this.shelfId = shelfId;
    }
    
    @JsonProperty("shelf-uid")
    public String getShelfUid() {
        return shelfUid;
    }

    @JsonProperty("shelf-uid")
    public void setShelfUid(String shelfUid) {
        this.shelfUid = shelfUid;
    }

    @JsonProperty("product-id")
    public String getProductId() {
        return productId;
    }

    @JsonProperty("product-id")
    public void setProductId(String productId) {
        this.productId = productId;
    }

    @JsonProperty("vendor-id")
    public String getVendorId() {
        return vendorId;
    }

    @JsonProperty("vendor-id")
    public void setVendorId(String vendorId) {
        this.vendorId = vendorId;
    }

    @JsonProperty("firmware-A")
    public String getFirmwareA() {
        return firmwareA;
    }

    @JsonProperty("firmware-A")
    public void setFirmwareA(String firmwareA) {
        this.firmwareA = firmwareA;
    }

    @JsonProperty("firmware-B")
    public String getFirmwareB() {
        return firmwareB;
    }

    @JsonProperty("firmware-B")
    public void setFirmwareB(String firmwareB) {
        this.firmwareB = firmwareB;
    }

    @JsonProperty("module-a")
    public ShelfModule getModuleA() {
        return moduleA;
    }

    @JsonProperty("module-a")
    public void setModuleA(ShelfModule moduleA) {
        this.moduleA = moduleA;
    }

    @JsonProperty("module-a-state")
    public ShelfModuleState getModuleAState() {
        return moduleAState;
    }

    @JsonProperty("module-a-state")
    public void setModuleAState(ShelfModuleState moduleAState) {
        this.moduleAState = moduleAState;
    }

    @JsonProperty("module-b")
    public ShelfModule getModuleB() {
        return moduleB;
    }

    @JsonProperty("module-b")
    public void setModuleB(ShelfModule moduleB) {
        this.moduleB = moduleB;
    }

    @JsonProperty("module-b-state")
    public ShelfModuleState getModuleBState() {
        return moduleBState;
    }

    @JsonProperty("module-b-state")
    public void setModuleBState(ShelfModuleState moduleBState) {
        this.moduleBState = moduleBState;
    }

    @JsonProperty("shelf-state")
    public ShelfState getShelfState() {
        return shelfState;
    }

    @JsonProperty("shelf-state")
    public void setShelfState(ShelfState shelfState) {
        this.shelfState = shelfState;
    }

    @JsonProperty("shelf-health")
    public void setShelfHealth(ShelfElementAggregateState aggrHealthState) {
        this.healthState = ShelfElementAggregateState.getMappedHealthState(aggrHealthState);
    }

    @JsonProperty("health-state")
    public HealthState getHealthState() {
        return healthState;
    }

    @JsonProperty("health-state")
    public void setHealthState(HealthState healthState) {
        this.healthState = healthState;
    }
    
    @JsonProperty("bay-list")
    public String[] getBayList() {
        return bayList;
    }

    @JsonProperty("bay-list")
    public void setBayList(String[] bayList) {
        this.bayList = bayList;
    }
  
    @JsonProperty("serial-no")
    public String getSerialNo() {
        return serialNo;
    }
    
    @JsonProperty("serial-no")
    public void setSerialNo(String serialNo) {
        this.serialNo = serialNo;
    }

    @JsonProperty("aggregate-element-status-info")
    public ShelfAggregateStatusInfoType getAggregateElementStatusInfo() {
        return aggregateElementStatusInfo;
    }

    @JsonProperty("aggregate-element-status-info")
    public void setAggregateElementStatusInfo(
            ShelfAggregateStatusInfoType aggregateElementStatusInfo) {
        this.aggregateElementStatusInfo = aggregateElementStatusInfo;
    }

    @JsonProperty("element-status-info")
    public ShelfElementStatusInfoType getShelfElementStatusInfo() {
        return shelfElementStatusInfo;
    }

    @JsonProperty("element-status-info")
    public void setShelfElementStatusInfo(
            ShelfElementStatusInfoType shelfElementStatusInfo) {
        this.shelfElementStatusInfo = shelfElementStatusInfo;
    }
}
