/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;


import com.fasterxml.jackson.annotation.JsonCreator;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebEnum;
import com.netapp.offtap3.internal.mars.webmapi.util.MappingException;

/**
 * @mapi shelf.element-common-status
 */
public enum ShelfElementAggregateState implements WebEnum {
    Unsupported ("unsupported", "Element is unsupported" ),
    OK("ok", "Element is OK"),
    Critical("critical", "Element is in a critical state"),
    NonCritical("non-critical", "Element is in a non-critical state"),
    Unrecoverable("unrecoverable", "Element is in an unrecoverable state"),
    NotInstalled("not-installed", "Element is not installed"),
    Unknown("unknown", "Element is Element is unknown"),
    NotAvailable("not-available","Element is not available");
    
    private String value;
    private String desc;

    private ShelfElementAggregateState(String value, String desc) {
        this.value = value;
        this.desc = desc;
    }

    public String value() {
        return value;
    }

    public String desc() {
        return desc;
    }
    
    @JsonCreator
    public static ShelfElementAggregateState forValue(String str) {
        for (ShelfElementAggregateState type : values()) {
            if (type.value().equalsIgnoreCase(str))
                return type;
        }
        throw new MappingException("element-common-status", str);
    }

    public static ShelfElementAggregateState getType(String name) {
        for (ShelfElementAggregateState type : values()) {
            if (type.name().equalsIgnoreCase(name))
                return type;
        }
        return null;
    }
    
    public static HealthState getMappedHealthState(ShelfElementAggregateState aagregateState) {
        if (aagregateState == ShelfElementAggregateState.Critical || aagregateState == ShelfElementAggregateState.Unrecoverable ||
                aagregateState == ShelfElementAggregateState.NotInstalled) {
            return HealthState.critical;
        } else if (aagregateState == ShelfElementAggregateState.NonCritical) {
            return HealthState.warning;
        } else if (aagregateState == ShelfElementAggregateState.OK) {
            return HealthState.ok;
        } else {
            return HealthState.unknown;
        }
    }
}
