/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;

/**
 * Power supply information
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ShelfPowerSupplyInfoType extends BaseDataType implements WebData, MapiData {
    /** Indicates whether the auto power reset of the supply is enabled.<!-- -->
     * <br>  This field will not be present if the power supply is missing.
     *  */
    @XmlElement(required = false, name = "is-auto-power-reset-enabled")
    private Boolean isAutoPowerResetEnabled = null;

    /** Indicates whether the power supply can be reset via software control.<!-- -->
     * <br>  This field will not be present if power supply is missing.
     *  */
    @XmlElement(required = false, name = "is-power-reset-capable")
    private Boolean isPowerResetCapable = null;

    /** Element number of the power supply. */
    @XmlElement(required = true, name = "power-supply-element-number")
    private Integer powerSupplyElementNumber = null;

    /** Power supply firmware revision<!-- --> 
     *  <br> This field will not be present if firmware rev is unavailable or feature not implemented, or if the power supply is missing.
     *  */
    @XmlElement(required = false, name = "power-supply-firmware-revision")
    private String powerSupplyFirmwareRevision = null;

    /** Indicates whether the power supply has reported any errors.<!-- -->
     * <br> This field will not be present if the power supply is missing. 
     * */
    @XmlElement(required = false, name = "power-supply-is-error")
    private Boolean powerSupplyIsError = null;

    /** Indicates the power supply for this element is not installed.<!-- -->
     * <br> This will only be presented if the power supply is missing, in which case no further information for the element will be available.
     *  */
    @XmlElement(required=true, name="power-supply-is-not-installed")
    private Boolean powerSupplyIsNotInstalled = null;

    /** Power supply part number */
    @XmlElement(required = false, name = "power-supply-part-no")
    private String powerSupplyPartNo = null;

    /** Power supply serial number */
    @XmlElement(required = false, name = "power-supply-serial-no")
    private String powerSupplySerialNo = null;

    /** Number of power supply swap counts since boot */
    @XmlElement(required = false, name = "power-supply-swap-count")
    private Integer powerSupplySwapCount = null;

    /** Power supply type */
    @XmlElement(required = false, name = "power-supply-type")
    private String powerSupplyType = null;

    public ShelfPowerSupplyInfoType() {
    }

    public ShelfPowerSupplyInfoType(Boolean isAutoPowerResetEnabled,
            Boolean isPowerResetCapable, Integer powerSupplyElementNumber,
            String powerSupplyFirmwareRevision, Boolean powerSupplyIsError,
            Boolean powerSupplyIsNotInstalled, String powerSupplyPartNo,
            String powerSupplySerialNo, Integer powerSupplySwapCount,
            String powerSupplyType) {
        super();
        this.isAutoPowerResetEnabled = isAutoPowerResetEnabled;
        this.isPowerResetCapable = isPowerResetCapable;
        this.powerSupplyElementNumber = powerSupplyElementNumber;
        this.powerSupplyFirmwareRevision = powerSupplyFirmwareRevision;
        this.powerSupplyIsError = powerSupplyIsError;
        this.powerSupplyIsNotInstalled = powerSupplyIsNotInstalled;
        this.powerSupplyPartNo = powerSupplyPartNo;
        this.powerSupplySerialNo = powerSupplySerialNo;
        this.powerSupplySwapCount = powerSupplySwapCount;
        this.powerSupplyType = powerSupplyType;
    }

    @JsonProperty("is-auto-power-reset-enabled")
    public Boolean getIsAutoPowerResetEnabled() {
        return isAutoPowerResetEnabled;
    }

    @JsonProperty("is-auto-power-reset-enabled")
    public void setAutoPowerResetEnabled(Boolean isAutoPowerResetEnabled) {
        this.isAutoPowerResetEnabled = isAutoPowerResetEnabled;
    }

    @JsonProperty("is-power-reset-capable")
    public Boolean getIsPowerResetCapable() {
        return isPowerResetCapable;
    }

    @JsonProperty("is-power-reset-capable")
    public void setIsPowerResetCapable(Boolean isPowerResetCapable) {
        this.isPowerResetCapable = isPowerResetCapable;
    }

    @JsonProperty("power-supply-element-number")
    public Integer getPowerSupplyElementNumber() {
        return powerSupplyElementNumber;
    }

    @JsonProperty("power-supply-element-number")
    public void setPowerSupplyElementNumber(Integer powerSupplyElementNumber) {
        this.powerSupplyElementNumber = powerSupplyElementNumber;
    }

    @JsonProperty("power-supply-firmware-revision")
    public String getPowerSupplyFirmwareRevision() {
        return powerSupplyFirmwareRevision;
    }

    @JsonProperty("power-supply-firmware-revision")
    public void setPowerSupplyFirmwareRevision(String powerSupplyFirmwareRevision) {
        this.powerSupplyFirmwareRevision = powerSupplyFirmwareRevision;
    }

    @JsonProperty("power-supply-is-error")
    public Boolean getPowerSupplyIsError() {
        return powerSupplyIsError;
    }

    @JsonProperty("power-supply-is-error")
    public void setPowerSupplyIsError(Boolean powerSupplyIsError) {
        this.powerSupplyIsError = powerSupplyIsError;
    }

    @JsonProperty("power-supply-is-not-installed")
    public Boolean getPowerSupplyIsNotInstalled() {
        return powerSupplyIsNotInstalled;
    }
    
    @JsonProperty("power-supply-is-not-installed")
    public void setPowerSupplyIsNotInstalled(Boolean powerSupplyIsNotInstalled) {
        this.powerSupplyIsNotInstalled = powerSupplyIsNotInstalled;
    }

    @JsonProperty("power-supply-part-no")
    public String getPowerSupplyPartNo() {
        return powerSupplyPartNo;
    }

    @JsonProperty("power-supply-part-no")
    public void setPowerSupplyPartNo(String powerSupplyPartNo) {
        this.powerSupplyPartNo = powerSupplyPartNo;
    }

    @JsonProperty("power-supply-serial-no")
    public String getPowerSupplySerialNo() {
        return powerSupplySerialNo;
    }

    @JsonProperty("power-supply-serial-no")
    public void setPowerSupplySerialNo(String powerSupplySerialNo) {
        this.powerSupplySerialNo = powerSupplySerialNo;
    }

    @JsonProperty("power-supply-swap-count")
    public Integer getPowerSupplySwapCount() {
        return powerSupplySwapCount;
    }

    @JsonProperty("power-supply-swap-count")
    public void setPowerSupplySwapCount(Integer powerSupplySwapCount) {
        this.powerSupplySwapCount = powerSupplySwapCount;
    }

    @JsonProperty("power-supply-type")
    public String getPowerSupplyType() {
        return powerSupplyType;
    }

    @JsonProperty("power-supply-type")
    public void setPowerSupplyType(String powerSupplyType) {
        this.powerSupplyType = powerSupplyType;
    }

}
