/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;

/**
 * Temperature sensors installed on the shelf.
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ShelfTempSensorInfoType extends BaseDataType implements WebData, MapiData {
    /** current Temperature for the sensor */
    @XmlElement(required = false, name = "temp-sensor-current-condition")
    private String tempSensorCurrentCondition = null;

    /**  Current temperature reading in degrees Celsius. */
    @XmlElement(required = false, name = "temp-sensor-current-temperature")
    private Integer tempSensorCurrentTemperature = null;
    
    /** Element number for the temperature sensor */
    @XmlElement(required = false, name = "temp-sensor-element-no")
    private Integer tempSensorElementNo = null;
    
    /** High critical temperature in degree Celsius */
    @XmlElement(required = true, name = "temp-sensor-hi-critical")
    private Integer tempSensorHiCritical = null;
    
    /** High warning temperature in degree Celsius */
    @XmlElement(required = true, name = "temp-sensor-hi-warning")
    private Integer tempSensorHiWarning = null;
    
    /** Indicates whether the temp-sensor-current-temp is the ambient temperature */
    @XmlElement(required = false, name = "temp-sensor-is-ambient")
    private Boolean tempSensorIsAmbient = null;
    
    /** Indicates whether the sensor has indicated an error in temperature */
    @XmlElement(required = false, name = "temp-sensor-is-error")
    private Boolean tempSensorIsError = null;

    /** Temperature sensor is not installed.<!-- -->
     * <br>Indicates the sensor for this element is not installed.<!-- --> 
     * <br> This will only be presented if the sensor is missing, and no further data for this sensor will be presented.
     */
    @XmlElement(required = false, name = "temp-sensor-is-not-installed")
    private String tempSensorIsNotInstalled = null;
    
    /** Low critical temperature in degree Celsius */
    @XmlElement(required = true, name = "temp-sensor-low-critical")
    private Integer tempSensorLowCritical = null;
    
    /** Low warning temperature in degree Celsius */
    @XmlElement(required = true, name = "temp-sensor-low-warning")
    private Integer tempSensorLowWarning = null;



    public ShelfTempSensorInfoType() {
    }



    public ShelfTempSensorInfoType(String tempSensorCurrentCondition,
            Integer tempSensorCurrentTemperature,
            Integer tempSensorElementNo, Integer tempSensorHiCritical,
            Integer tempSensorHiWarning, Boolean tempSensorIsAmbient,
            Boolean tempSensorIsError, String tempSensorIsNotInstalled,
            Integer tempSensorLowCritical, Integer tempSensorLowWarning) {
        super();
        this.tempSensorCurrentCondition = tempSensorCurrentCondition;
        this.tempSensorCurrentTemperature = tempSensorCurrentTemperature;
        this.tempSensorElementNo = tempSensorElementNo;
        this.tempSensorHiCritical = tempSensorHiCritical;
        this.tempSensorHiWarning = tempSensorHiWarning;
        this.tempSensorIsAmbient = tempSensorIsAmbient;
        this.tempSensorIsError = tempSensorIsError;
        this.tempSensorIsNotInstalled = tempSensorIsNotInstalled;
        this.tempSensorLowCritical = tempSensorLowCritical;
        this.tempSensorLowWarning = tempSensorLowWarning;
    }

    @JsonProperty("temp-sensor-current-condition")
    public String getTempSensorCurrentCondition() {
        return tempSensorCurrentCondition;
    }

    @JsonProperty("temp-sensor-current-condition")
    public void setTempSensorCurrentCondition(String tempSensorCurrentCondition) {
        this.tempSensorCurrentCondition = tempSensorCurrentCondition;
    }

    @JsonProperty("temp-sensor-current-temperature")
    public Integer getTempSensorCurrentTemperature() {
        return tempSensorCurrentTemperature;
    }

    @JsonProperty("temp-sensor-current-temperature")
    public void setTempSensorCurrentTemperature(
            Integer tempSensorCurrentTemperature) {
        this.tempSensorCurrentTemperature = tempSensorCurrentTemperature;
    }

    @JsonProperty("temp-sensor-element-no")
    public Integer getTempSensorElementNo() {
        return tempSensorElementNo;
    }

    @JsonProperty("temp-sensor-element-no")
    public void setTempSensorElementNo(Integer tempSensorElementNo) {
        this.tempSensorElementNo = tempSensorElementNo;
    }

    @JsonProperty("temp-sensor-hi-critical")
    public Integer getTempSensorHiCritical() {
        return tempSensorHiCritical;
    }

    @JsonProperty("temp-sensor-hi-critical")
    public void setTempSensorHiCritical(Integer tempSensorHiCritical) {
        this.tempSensorHiCritical = tempSensorHiCritical;
    }

    @JsonProperty("temp-sensor-hi-warning")
    public Integer getTempSensorHiWarning() {
        return tempSensorHiWarning;
    }

    @JsonProperty("temp-sensor-hi-warning")
    public void setTempSensorHiWarning(Integer tempSensorHiWarning) {
        this.tempSensorHiWarning = tempSensorHiWarning;
    }

    @JsonProperty("temp-sensor-is-ambient")
    public Boolean getTempSensorIsAmbient() {
        return tempSensorIsAmbient;
    }

    @JsonProperty("temp-sensor-is-ambient")
    public void setTempSensorIsAmbient(Boolean tempSensorIsAmbient) {
        this.tempSensorIsAmbient = tempSensorIsAmbient;
    }

    @JsonProperty("temp-sensor-is-error")
    public Boolean getTempSensorIsError() {
        return tempSensorIsError;
    }

    @JsonProperty("temp-sensor-is-error")
    public void setTempSensorIsError(Boolean tempSensorIsError) {
        this.tempSensorIsError = tempSensorIsError;
    }

    @JsonProperty("temp-sensor-is-not-installed")
    public String getTempSensorIsNotInstalled() {
        return tempSensorIsNotInstalled;
    }

    @JsonProperty("temp-sensor-is-not-installed")
    public void setTempSensorIsNotInstalled(String tempSensorIsNotInstalled) {
        this.tempSensorIsNotInstalled = tempSensorIsNotInstalled;
    }

    @JsonProperty("temp-sensor-low-critical")
    public Integer getTempSensorLowCritical() {
        return tempSensorLowCritical;
    }

    @JsonProperty("temp-sensor-low-critical")
    public void setTempSensorLowCritical(Integer tempSensorLowCritical) {
        this.tempSensorLowCritical = tempSensorLowCritical;
    }

    @JsonProperty("temp-sensor-low-warning")
    public Integer getTempSensorLowWarning() {
        return tempSensorLowWarning;
    }

    @JsonProperty("temp-sensor-low-warning")
    public void setTempSensorLowWarning(Integer tempSensorLowWarning) {
        this.tempSensorLowWarning = tempSensorLowWarning;
    }
}