/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.*;

/**
 *  voltage sensor information
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class ShelfVoltageSensorInfoType extends BaseDataType implements WebData, MapiData {

    /** Indicates whether the sensor has encountered an error. */
    @XmlElement(required = false, name = "is-sensor-error")
    private Boolean isSensorError = null;
    
    /** Indicates the voltage sensor elements is not installed.<!-- -->
     * <br>This will only be present if the element is missing, in such case no further data for this element will be presented.
     * 
     * */
    @XmlElement(required = false, name = "is-sensor-not-installed")
    private Boolean isSensorNotInstalled = null;
    
    /** Sensor condition<!-- -->
     * <br> A Text string describing whether the sensor-reading field is within normal operating range.<!-- --> 
     * This field is present only if sensor-reading field is also present.<!-- -->
     * <br> Possible values are: "overvoltage_failure", "overvoltage_warning", "undervoltage_warning", "undervoltage_failure", "normal_operating_range".
     * 
     */
    @XmlElement(required = false, name = "sensor-condition")
    private String sensorCondition = null;
    
    /** Voltage reading of the sensor<!-- --> 
     * <br>This field will not be present if unable to read the sensor value properly.
     * */
    @XmlElement(required = false, name = "sensor-reading")
    private String sensorReading = null;

    /**  Voltage sensor number */
    @XmlElement(required = true, name = "voltage-sensor-no")
    private Integer voltageSensorNo = null;



    public ShelfVoltageSensorInfoType() {
    }



    public ShelfVoltageSensorInfoType(Boolean isSensorError,
            Boolean isSensorNotInstalled, String sensorCondition,
            String sensorReading, Integer voltageSensorNo) {
        super();
        this.isSensorError = isSensorError;
        this.isSensorNotInstalled = isSensorNotInstalled;
        this.sensorCondition = sensorCondition;
        this.sensorReading = sensorReading;
        this.voltageSensorNo = voltageSensorNo;
    }


    @JsonProperty("is-sensor-error")
    public Boolean getIsSensorError() {
        return isSensorError;
    }

    @JsonProperty("is-sensor-error")
    public void setIsSensorError(Boolean isSensorError) {
        this.isSensorError = isSensorError;
    }

    @JsonProperty("is-sensor-not-installed")
    public Boolean getIsSensorNotInstalled() {
        return isSensorNotInstalled;
    }

    @JsonProperty("is-sensor-not-installed")
    public void setIsSensorNotInstalled(Boolean isSensorNotInstalled) {
        this.isSensorNotInstalled = isSensorNotInstalled;
    }

    @JsonProperty("sensor-condition")
    public String getSensorCondition() {
        return sensorCondition;
    }

    @JsonProperty("sensor-condition")
    public void setSensorCondition(String sensorCondition) {
        this.sensorCondition = sensorCondition;
    }

    @JsonProperty("sensor-reading")
    public String getSensorReading() {
        return sensorReading;
    }

    @JsonProperty("sensor-reading")
    public void setSensorReading(String sensorReading) {
        this.sensorReading = sensorReading;
    }

    @JsonProperty("voltage-sensor-no")
    public Integer getVoltageSensorNo() {
        return voltageSensorNo;
    }

    @JsonProperty("voltage-sensor-no")
    public void setVoltageSensorNo(Integer voltageSensorNo) {
        this.voltageSensorNo = voltageSensorNo;
    }


}