/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiInput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebInput;

/**
 * Input settings for creating a Snapshot copy. <!-- --> <br>
 * The four options "lun", "lun-uuid", "cg" and "cg-uuid" are mutually exclusive. One and only one must be provided.
 * 
 * @author Jing Shi
 * @since 1.0.1
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class SnapCreateInputType extends BaseDataType implements WebInput, MapiInput {
    /**
     * Name of the LUN to be captured in the Snapshot copy. The LUN must not be a member
     * of any consistency group.
     */
    @XmlElement(name="lun", required = false)
    private String lun;
    
    /**
     * Universally unique ID of the LUN to be captured in the Snapshot copy. The LUN must
     * not be a member of any consistency group.
     */
    @XmlElement(name="lun-uuid", required = false)
    private String lunUuid;
    
    /**
     * Name of the consistency group to be captured in the Snapshot copy.
     */
    @XmlElement(name="cg", required = false)
    private String cg;
    
    /**
     * Universally unique ID of the consistency group to be captured in the Snapshot copy.<!-- --> <br>
     */
    @XmlElement(name="cg-uuid", required = false)
    private String cgUuid;
    
    /**
     * Name for the Snapshot copy to be created. If omitted, a default name will be used.<!-- --> <br>
     * A Snapshot copy name is up to 280 characters long, and contains only ASCII letters 'a' through 'z', 'A' through 'Z', digits '0' through '9', '-', '.', '_', ':', '{', and '}'.
     */
    @XmlElement(name="name", required = false)
    private String name;

    public SnapCreateInputType() {}
    public SnapCreateInputType(String lun, String lunUuid, String cg, String cgUuid, String name) {
        super();
        this.lun = lun;
        this.lunUuid = lunUuid;
        this.cg = cg;
        this.cgUuid = cgUuid;
        this.name = name;
    }

    @JsonProperty("lun")
    public String getLun() {
        return lun;
    }

    @JsonProperty("lun")
    public void setLun(String lun) {
        this.lun = lun;
    }

    @JsonProperty("lun-uuid")
    public String getLunUuid() {
        return lunUuid;
    }

    @JsonProperty("lun-uuid")
    public void setLunUuid(String lunUuid) {
        this.lunUuid = lunUuid;
    }

    @JsonProperty("cg")
    public String getCg() {
        return cg;
    }

    @JsonProperty("cg")
    public void setCg(String cg) {
        this.cg = cg;
    }

    @JsonProperty("cg-uuid")
    public String getCgUuid() {
        return cgUuid;
    }

    @JsonProperty("cg-uuid")
    public void setCgUuid(String cgUuid) {
        this.cgUuid = cgUuid;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }

}
