/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiData;
import com.netapp.offtap3.internal.mars.webmapi.model.common.NameType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.UuidDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebData;

/**
 * Information about a Snapshot copy
 * 
 * @author Jing Shi
 * @since 1.0.1
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class SnapInfoType extends BaseDataType implements WebData, MapiData, UuidDataType, NameType {
    /** Name of the LUN. */
    @XmlElement(required = true, name = "lun")
    private String lun;
    
    /** Name of the consistency group. */
    @XmlElement(required = true, name = "cg")
    private String cg;

    /** Name of the Snapshot copy. */
    @XmlElement(required = true, name = "name")
    private String name = null;
    
    /** Universally unique ID of the Snapshot copy. */
    @XmlElement(required = true, name = "uuid")
    public String uuid = null;

    /** Creation time of the Snapshot copy, in seconds since the epoch. */
    @XmlElement(required = true, name = "create-time")
    private Long createTime;

    /** LUN names. */
    @XmlElement(required = true, name = "member-names")
    private String[] memberNames;
    
    public SnapInfoType() {}

    public SnapInfoType(String lun, String cg, String name, String uuid, Long createTime, String[] memberNames) {
        super();
        this.lun = lun;
        this.cg = cg;
        this.name = name;
        this.uuid = uuid;
        this.createTime = createTime;
        this.memberNames = memberNames;
    }

    @JsonProperty("lun")
    public String getLun() {
        return lun;
    }
    
    @JsonProperty("lun")
    public void setLun(String lun) {
        this.lun = lun;
    }

    @JsonProperty("cg")
    public String getCg() {
        return cg;
    }

    @JsonProperty("cg")
    public void setCg(String cg) {
        this.cg = cg;
    }

    @JsonProperty("uuid")
    public String getUuid() {
        return uuid;
    }

    @JsonProperty("uuid")
    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    @JsonProperty("name")
    public String getName() {
        return name;
    }

    @JsonProperty("name")
    public void setName(String name) {
        this.name = name;
    }


    @JsonProperty("member-names")
    public String[] getMemberNames() {
        return memberNames;
    }

    @JsonProperty("member-names")
    public void setMemberNames(String[] memberNames) {
        this.memberNames = memberNames;
    }

    @JsonProperty("create-time")
    public Long getCreateTime() {
        return createTime;
    }

    @JsonProperty("create-time")
    public void setCreateTime(Long createTime) {
        this.createTime = createTime;
    }
}
