/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiInput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebInput;

/**
 * Input for showing Snapshot copies
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class SnapShowInputType extends BaseDataType implements WebInput, MapiInput {
    /**
     * Names of one or more LUNs.
     */
    @XmlElement(required = false, name = "luns")
    private String[] luns;
    
    /**
     * Universally unique IDs of one or more LUNs.
     */
    @XmlElement(required = false, name = "lun-uuids")
    private String[] lunUuids;
    
    /**
     * Names of one or more consistency groups.
     */
    @XmlElement(required = false, name = "cgs")
    private String[] cgs;
    
    /**
     * Universally unique IDs of one or more consistency groups.
     */
    @XmlElement(required = false, name = "cg-uuids")
    private String[] cgUuids;
    
    /**
     * Names of one or more Snapshot copies.
     */
    @XmlElement(required = false, name = "names")
    private String[] names;
    
    /**
     * Universally unique IDs of one or more Snapshot copies.
     */
    @XmlElement(required = false, name = "uuids")
    private String[] uuids;

    /**
     * Display verbose Snapshot copy information
     */
    @XmlElement(required = false, name = "verbose")
    private Boolean verbose;
    
    public SnapShowInputType() {}

    public SnapShowInputType(String[] luns, String[] lunUuids, String[] cgs, String[] cgUuids, String[] names, String[] uuids, Boolean verbose) {
        super();
        this.luns = luns;
        this.lunUuids = lunUuids;
        this.cgs = cgs;
        this.cgUuids = cgUuids;
        this.names = names;
        this.uuids = uuids;
        this.verbose = verbose;
    }

    @JsonProperty("luns")
    public String[] getLuns() {
        return luns;
    }

    @JsonProperty("luns")
    public void setLuns(String[] luns) {
        this.luns = luns;
    }

    @JsonProperty("lun-uuids")
    public String[] getLunUuids() {
        return lunUuids;
    }

    @JsonProperty("lun-uuids")
    public void setLunUuid(String[] lunUuids) {
        this.lunUuids = lunUuids;
    }

    @JsonProperty("cgs")
    public String[] getCgs() {
        return cgs;
    }

    @JsonProperty("cgs")
    public void setCgs(String[] cgs) {
        this.cgs = cgs;
    }

    @JsonProperty("cg-uuids")
    public String[] getCgUuids() {
        return cgUuids;
    }

    @JsonProperty("cg-uuids")
    public void setCgUuids(String[] cgUuids) {
        this.cgUuids = cgUuids;
    }

    @JsonProperty("names")
    public String[] getNames() {
        return names;
    }

    @JsonProperty("names")
    public void setNames(String[] names) {
        this.names = names;
    }

    @JsonProperty("uuids")
    public String[] getUuids() {
        return uuids;
    }

    @JsonProperty("uuids")
    public void setUuids(String[] uuids) {
        this.uuids = uuids;
    }

    @JsonProperty("verbose")
    public Boolean getVerbose() {
        return verbose;
    }

    @JsonProperty("verbose")
    public void setVerbose(Boolean verbose) {
        this.verbose = verbose;
    }

    
}
