/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiInput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebInput;

/**
 * Stats show operation input type
 * <p>
 * ${mapi-datatype-desc}
 * </p>
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class StatsShowInputType extends BaseDataType implements WebInput, MapiInput, StatsShowInput {
    /**
     * Display the raw counter values taken at the time the command is executed. Without the raw option, the command displays counter values that are a delta between the previous stop command and the
     * previous start command.
     */
    @XmlElement(required = false)
    protected Boolean raw;

    /**
     * Specify the number of iterations over which the counter values will be displayed.
     */
    @XmlElement(required = false)
    protected Integer iterations;

    /**
     * Intervals to display counter values. The interval time is in seconds.
     */
    @XmlElement(required = false)
    protected Integer interval;

    /**
     * Object to display counter values for.
     */
    @XmlElement(required = true)
    protected String object;

    /**
     * Specify the list of instances for which to display counter value.
     */
    @XmlElement(required = false)
    protected String[] instances;

    /**
     * Specify the list of counters for which to display values for.
     */
    @XmlElement(required = false)
    protected String[] counters;

    /**
     * Specify the scope for aggregation of counter values.
     */
    @XmlElement(required = false)
    protected String scope;

    /**
     * Identifier for the sample to show collected statistics.
     */
    @XmlElement(required = false)
    protected String id;

    /**
     * Specify the counter name for sorting. This must be a single counter subset of the counters specified in the counters option.
     */
    @XmlElement(required = false, name = "sort-by-counter")
    protected String sortByCounter;

    /**
     * Specify the sort order for counter values. ASC or DESC.
     */
    @XmlElement(required = false, name = "sort-order")
    protected String sortOrder;

    /**
     * Specify the top 'n' number of instances for counter specified in the sort-by-counter option.
     */
    @XmlElement(required = false)
    protected Integer top;

    /**
     * Specify the bottom 'n' number of instances for counter specified in the sort-by-counter option.
     */
    @XmlElement(required = false)
    protected Integer bottom;

    public StatsShowInputType() {
    }

    public StatsShowInputType(String object, String[] instances, String[] counters) {
        this.object = object;
        this.instances = instances;
        this.counters = counters;
    }

    public Boolean getRaw() {
        return raw;
    }

    public void setRaw(Boolean raw) {
        this.raw = raw;
    }

    public Integer getIterations() {
        return iterations;
    }

    public void setIterations(Integer iterations) {
        this.iterations = iterations;
    }

    public Integer getInterval() {
        return interval;
    }

    public void setInterval(Integer interval) {
        this.interval = interval;
    }

    public String getObject() {
        return object;
    }

    public void setObject(String object) {
        this.object = object;
    }

    public String[] getInstances() {
        return instances;
    }

    public void setInstances(String[] instances) {
        this.instances = instances;
    }

    public String[] getCounters() {
        return counters;
    }

    public void setCounters(String[] counters) {
        this.counters = counters;
    }

    public String getScope() {
        return scope;
    }

    public void setScope(String scope) {
        this.scope = scope;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    @JsonProperty("sort-by-counter")
    public String getSortByCounter() {
        return sortByCounter;
    }

    @JsonProperty("sort-by-counter")
    public void setSortByCounter(String sortByCounter) {
        this.sortByCounter = sortByCounter;
    }

    @JsonProperty("sort-order")
    public String getSortOrder() {
        return sortOrder;
    }

    @JsonProperty("sort-order")
    public void setSortOrder(String sortOrder) {
        this.sortOrder = sortOrder;
    }

    public Integer getTop() {
        return top;
    }

    public void setTop(Integer top) {
        this.top = top;
    }

    public Integer getBottom() {
        return bottom;
    }

    public void setBottom(Integer bottom) {
        this.bottom = bottom;
    }

}
