/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */

//$$WEBMAPI: manual-written 

package com.netapp.offtap3.internal.mars.webmapi.model.domain;


import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiData;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebData;

/**
 * Storage info block for solid-state-drives. *
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class StorageSsdInfoType extends BaseDataType implements WebData, MapiData {

    /** rated-life used percentage */
    @XmlElement(required = false, name = "percent-rated-life-used")
    private Integer percentRatedLifeUsed;

    /** Spares Consumed percentage */
    @XmlElement(required = false, name = "percent-spares-consumed")
    private Integer percentSparesConsumed;

    /** Total hours the SSD has been powered on. */
    @XmlElement(required = false, name = "power-on-hours")
    private Integer powerOnHours;

    /** Wear level of the disk */
    @XmlElement(required = true, name = "wear-level")
    private Integer wearLevel;
    
    /** Wear level health status of the disk */
    @XmlElement(required = false, name = "wear-level-status")
    private HealthState wearLevelState;

    /** Number of bytes read for that disk */
    @XmlElement(required = true, name = "bytes-read")
    private Integer bytesRead;

    /** Number of bytes written for that disk */
    @XmlElement(required = true, name = "sectors-written")
    private Long sectorsWritten;

    /** The most recent temperature of the disk in Centigrade */
    @XmlElement(required = true, name = "temperature")
    private Integer temperature;

    public StorageSsdInfoType() {
    }

    public StorageSsdInfoType(Integer percentRatedLifeUsed,
            Integer percentSparesConsumed, Integer powerOnHours,
            Integer wearLevel, HealthState wearLevelState, Integer bytesRead, Long sectorsWritten,
            Integer temperature) {
        super();
        this.percentRatedLifeUsed = percentRatedLifeUsed;
        this.percentSparesConsumed = percentSparesConsumed;
        this.powerOnHours = powerOnHours;
        this.wearLevel = wearLevel;
        this.wearLevelState = wearLevelState;
        this.bytesRead = bytesRead;
        this.sectorsWritten = sectorsWritten;
        this.temperature = temperature;
    }

    @JsonProperty("percent-rated-life-used")
    public Integer getPercentRatedLifeUsed() {
        return this.percentRatedLifeUsed;
    }

    @JsonProperty("percent-rated-life-used")
    public void setPercentRatedLifeUsed(Integer percentRatedLifeUsed) {
        this.percentRatedLifeUsed = percentRatedLifeUsed;
    }

    @JsonProperty("percent-spares-consumed")
    public Integer getPercentSparesConsumed() {
        return this.percentSparesConsumed;
    }

    @JsonProperty("percent-spares-consumed")
    public void setPercentSparesConsumed(Integer percentSparesConsumed) {
        this.percentSparesConsumed = percentSparesConsumed;
    }

    @JsonProperty("power-on-hours")
    public Integer getPowerOnHours() {
        return this.powerOnHours;
    }

    @JsonProperty("power-on-hours")
    public void setPowerOnHours(Integer powerOnHours) {
        this.powerOnHours = powerOnHours;
    }

    @JsonProperty("wear-level")
    public Integer getWearLevel() {
        return this.wearLevel;
    }

    @JsonProperty("wear-level")
    public void setWearLevel(Integer wearLevel) {
        this.wearLevel = wearLevel;
    }

    @JsonProperty("bytes-read")
    public Integer getBytesRead() {
        return this.bytesRead;
    }

    @JsonProperty("bytes-read")
    public void setBytesRead(Integer bytesRead) {
        this.bytesRead = bytesRead;
    }

    @JsonProperty("sectors-written")
    public Long getSectorsWritten() {
        return sectorsWritten;
    }

    @JsonProperty("sectors-written")
    public void setSectorsWritten(Long sectorsWritten) {
        this.sectorsWritten = sectorsWritten;
    }

    @JsonProperty("temperature")
    public Integer getTemperature() {
        return temperature;
    }

    @JsonProperty("temperature")
    public void setTemperature(Integer temperature) {
        this.temperature = temperature;
    }
    
    @JsonProperty("wear-level-status")
    public HealthState getWearLevelState() {
        return wearLevelState;
    }

    @JsonProperty("wear-level-status")
    public void setWearLevelState(HealthState wearLevelState) {
        this.wearLevelState = wearLevelState;
    }
}