/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebEnum;
import com.netapp.offtap3.internal.mars.webmapi.util.MappingException;

/**
 * Severity of a health alert
 * 
 * @mapi system.health-alert-severity-id
 * @author TSANJEEV
 * @version $Revision$, Aug 20, 2014
 */
public enum SystemHealthAlertSeverity implements WebEnum {

    normal,
    informational,
    warning,
    error,
    critical,
    emergency,
    ;
    
    @JsonCreator
    public static SystemHealthAlertSeverity forValue(String str) {
        for (SystemHealthAlertSeverity type : values()) {
            if (type.name().equalsIgnoreCase(str))
                return type;
        }
        throw new MappingException("severity", str);
    }
    
    public static HealthState getHealthState(SystemHealthAlertSeverity alertSeverity) {
        switch (alertSeverity) {
        case normal:
        case informational: return HealthState.ok;
        case warning: return HealthState.warning;
        case error:
        case critical:
        case emergency:
            return HealthState.critical;
        default: return HealthState.unknown;
        }
    }
}
