/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.domain;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MapiInput;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebInput;

/**
 * System service processor network modify input data type <br>
 * <br>
 * If DHCP is enabled, changes to the IPv4 settings will not be cached. <br>
 * Changes to IPv4 settings will take effect only if DHCP is disabled.
 * 
 * @author TSANJEEV
 * @version $Revision$, Dec 11, 2013
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class SystemServiceProcessorNetworkModifyInputType extends BaseDataType implements WebInput, MapiInput {

    /** IPv4 address */
    @XmlElement(required = false, name = "ip-address")
    private String ipAddress;

    /** IPv4 netmask */
    @XmlElement(required = false, name = "netmask")
    private String netmask;

    /** IPv4 gateway */
    @XmlElement(required = false, name = "gateway")
    private String gateway;

    /** DHCP enabled? */
    @XmlElement(required = false, name = "dhcp-enable")
    private Boolean dhcpEnable;

    public SystemServiceProcessorNetworkModifyInputType() {
    }

    public SystemServiceProcessorNetworkModifyInputType(String ipAddress, String netmask, String gateway, Boolean dhcpEnable, String ipv6Address, String ipv6AddressPrefix, String ipv6GatewayAddress,
            Boolean ipv6RouterAddressEnable) {
        super();
        this.ipAddress = ipAddress;
        this.netmask = netmask;
        this.gateway = gateway;
        this.dhcpEnable = dhcpEnable;
    }

    @JsonProperty("ip-address")
    public String getIpAddress() {
        return ipAddress;
    }

    @JsonProperty("ip-address")
    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
    }

    @JsonProperty("netmask")
    public String getNetmask() {
        return netmask;
    }

    @JsonProperty("netmask")
    public void setNetmask(String netmask) {
        this.netmask = netmask;
    }

    @JsonProperty("gateway")
    public String getGateway() {
        return gateway;
    }

    @JsonProperty("gateway")
    public void setGateway(String gateway) {
        this.gateway = gateway;
    }

    @JsonProperty("dhcp-enable")
    public Boolean getDhcpEnable() {
        return dhcpEnable;
    }

    @JsonProperty("dhcp-enable")
    public void setDhcpEnable(Boolean dhcpEnable) {
        this.dhcpEnable = dhcpEnable;
    }
}
