/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.service;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * Details about the WebMAPI version
 *
 * @author sjing
 * @version $Revision$, Apr 2, 2014
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class VersionType extends BaseDataType implements WebOutput {
    public final static String s_dash = "_";
    
    /**
     * Major version
     */
    @XmlElement(required = true, name = "major")
    private Integer major;
    /**
     * Minor version
     */
    @XmlElement(required = true, name = "minor")
    private Integer minor;
    /**
     * Maintain version
     */
    @XmlElement(required = false, name = "maintain")
    private Integer maintain;
    /**
     * Build id
     */
    @XmlElement(required = false, name = "build-id")
    private String buildId;
    
    public VersionType() {}

    public VersionType(Integer major, Integer minor, Integer maintain,
            String buildId) {
        super();
        this.major = (major==null? 0 : major);
        this.minor = (minor==null? 0 : minor);
        this.maintain = maintain;
        this.buildId = buildId;
    }

    public Integer getMajor() {
        return major;
    }

    public void setMajor(Integer major) {
        this.major = major;
    }

    public Integer getMinor() {
        return minor;
    }

    public void setMinor(Integer minor) {
        this.minor = minor;
    }

    public Integer getMaintain() {
        return maintain;
    }

    public void setMaintain(Integer maintain) {
        this.maintain = maintain;
    }

    public String getBuildId() {
        return buildId;
    }

    public void setBuildId(String buildId) {
        this.buildId = buildId;
    }
    
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(major).append(s_dash).append(minor);
        if (maintain != null)
            sb.append(s_dash).append(maintain);
        if (buildId != null)
            sb.append(s_dash).append(buildId);
        return sb.toString();
    }
    
    public static VersionType parse(String str) throws IllegalArgumentException {
        if (str == null)
            throw new IllegalArgumentException("Version string cannot be null");
        
        String[] segments = str.split(s_dash);
        if (segments== null || segments.length<2)
            throw new IllegalArgumentException("Version string has invalid format");
        
        try {
            Integer major = Integer.parseInt(segments[0]);
            Integer minor = Integer.parseInt(segments[1]);
            Integer maintain = null;
            String buildId = null;
            if (segments.length >2) {
                try {
                    maintain = Integer.parseInt(segments[2]);
                    if (segments.length >3)
                        buildId = segments[3];
                } catch (Exception ex) {
                    buildId = segments[2];
                }
            }
            
            return new VersionType(major, minor, maintain, buildId);
        } catch (Exception ex) {
            throw new IllegalArgumentException("Version string has invalid format");
        }
        
    }
}
