/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains
 * the property of NetApp, Inc. and its suppliers if any.
 * 
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.model.service;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.WebOutput;

/**
 * WebMapiInfoType provides information regarding to server running WEBMAPI library
 *
 * @author Jing Shi
 * @version 1.0, Apr 2, 2014
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@JsonInclude(Include.NON_NULL)
public class WebMapiInfoType extends BaseDataType implements WebOutput {
    
    /**
     * Server running WEBMAPI version<!-- --><br>
     * Version is in the string format of major.minor.maintenance<!-- --><br>
     * Major, minor and maintenance(or patch) are non-negative integers without leading zeroes.<!-- --><br>
     * Maintenance is optional, only introduced when there is incompatibility in the same major.minor versions.
     */
    @XmlElement(required = true, name = "x-webmapi-version")
    private String webmapiVersion;
    
    /**
     * Server supported WEBMAPI compatible versions<!-- --><br>
     * Each compatible version is in string format of major.minor.maintenance<!-- --><br>
     * Major, minor and maintenance(or patch) are non-negative integers without leading zeroes.<!-- --><br>
     * Maintenance is optional, only introduced when there is incompatibility in the same major.minor versions.
     */
    @XmlElement(required = true, name = "webmapi-compatible-versions")
    private String[] compatibleVersions;
    
    public WebMapiInfoType() {}

    public WebMapiInfoType(String webmapiVersion, String[] compatibleVersions) {
        super();
        this.webmapiVersion = webmapiVersion;
        this.compatibleVersions = compatibleVersions;
    }

    public WebMapiInfoType(WebMapiInfoType that) {
        super();
        this.webmapiVersion = that.webmapiVersion;
        this.compatibleVersions = that.compatibleVersions;
    }
    
    @JsonProperty("x-webmapi-version")
    public String getWebmapiVersion() {
        return webmapiVersion;
    }

    @JsonProperty("x-webmapi-version")
    public void setWebmapiVersion(String webmapiVersion) {
        this.webmapiVersion = webmapiVersion;
    }

    @JsonProperty("webmapi-compatible-versions")
    public String[] getCompatibleVersions() {
        return compatibleVersions;
    }

    @JsonProperty("webmapi-compatible-versions")
    public void setCompatibleVersions(String[] compatibleVersions) {
        this.compatibleVersions = compatibleVersions;
    }
}
