/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.util;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;
/**
 * IndexCondition defines condition type to limit data records by data global index
 * 
 * @author Jing Shi
 * @version $Revision$, May 7, 2013
 */
public class IndexCondition extends BaseDataType implements Comparable<Long> {
    public static final IndexCondition NONE_INDEX_CONDITION = new IndexCondition(0L);

    public static final String QUERY_INDEX_KEY = "index";
    public static final String QUERY_INDEX_FORMAT = "index={positive-number}";
    public static final Range<Long> INDEX_RANGE = new Range<Long>(0L, Long.MAX_VALUE);

    private static String s_Query_EQ = "=";
    private static String s_LB = "[";
    private static String s_RB = "]";

    /**
     * the global index of the data record
     */
    private Long index = 0L;

    public IndexCondition() {
    }

    public IndexCondition(Long index) {
        setIndex(index);
    }

    public Long getIndex() {
        return index;
    }

    public void setIndex(Long index) {
        if (INDEX_RANGE.inRange(index))
            this.index = index;
        else
            throw new IllegalArgumentException("Index " + index + " is out of range " + INDEX_RANGE);
    }
    public int compareTo(Long Index) {
        
        if (index == Index)
            return 0;

        if (Index == null)
            return 1;
        else
            return (int) (this.index - Index);
    }

    public boolean isAfter(Long Index) {
        return compareTo(Index) <= 0;
    }

    public String toNiceString() {
        StringBuilder sb = new StringBuilder();
        sb.append(QUERY_INDEX_KEY).append(s_Query_EQ).append(index);
        return sb.toString();
    }
}
