/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.util;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.LinkedHashMap;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * JSONParser provides ...
 * 
 * @author Jing Shi
 * @version $Revision$, May 6, 2013
 */
public class JSONParser implements ParserHelper {
    private static final Logger s_logger = LogManager.getLogger(JSONParser.class);
    private final ObjectMapper m_mapper;

    public JSONParser() {
        m_mapper = new ObjectMapper();
        m_mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        m_mapper.enable(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT);
    }

    public <T> String convertToString(T instance) throws ParseException {
        try {
            return m_mapper.writeValueAsString(instance);
        } catch (Throwable ex) {
            s_logger.debug("Failed to convert instance to string: {0}", ex);
            throw new ParseException(ex);
        }
    }

    public <T> T convertToInstance(String content, TypeReference<T> typeRef) throws ParseException {
        try {
            return m_mapper.readValue(content, typeRef);
        } catch (Throwable ex) {
            s_logger.debug("Failed to convert string to instance: {0}", ex);
            throw new ParseException(ex);
        }
    }

    @SuppressWarnings("rawtypes")
    public <T> T convertToInstance(LinkedHashMap content, TypeReference<T> typeRef) throws ParseException {
        T ret = m_mapper.convertValue(content, typeRef);
        s_logger.trace("content=>" + content + ", ret=>" + ret);
        return ret;
    }

    public <T> void writeInstance(OutputStream out, T instance) throws ParseException {
        try {
            m_mapper.writeValue(out, instance);
        } catch (Throwable ex) {
            s_logger.debug("Failed to write instance: {0}", ex);
            throw new ParseException(ex);
        }
    }

    public <T> T readInstance(InputStream in, TypeReference<T> typeRef) throws ParseException {
        try {
            return m_mapper.readValue(in, typeRef);
        } catch (Throwable ex) {
            s_logger.debug("Failed to read instance: {0}", ex);
            throw new ParseException(ex);
        }
    }
    
    public JsonNode readTree(InputStream in) throws ParseException {
        try {
            return m_mapper.readTree(in);
        } catch (Throwable ex) {
            s_logger.debug("Failed to read instance: {0}", ex);
            throw new ParseException(ex);
        }
    }
    
    public <T> T convertToObject(JsonNode node, TypeReference<T> typeRef) throws ParseException {
        try {
            InputStream is = new ByteArrayInputStream(node.toString().getBytes("UTF-8"));
            return m_mapper.readValue(is, typeRef);
        } catch (Throwable ex) {
            s_logger.debug("Failed to read instance: {0}", ex);
            throw new ParseException(ex);
        }
    }
}
