/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.util;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * LimitCondition defines the condition type from query of limit data records by number as ?limit={number}
 * 
 * @author Jing Shi
 * @version $Revision$, Mar 25, 2013
 */
public class LimitCondition extends BaseDataType {
    public static final String QUERY_LIMIT_KEY = "limit";
    public static final Range<Integer> LIMIT_RANGE = new Range<Integer>(0, Integer.MAX_VALUE);

    private Integer limit = Integer.MAX_VALUE;

    public LimitCondition() {
    }

    public LimitCondition(Integer limit) {
        setLimit(limit);
    }

    public Integer getLimit() {
        return limit;
    }

    public void setLimit(Integer limit) {
        if (LIMIT_RANGE.inRange(limit))
            this.limit = limit;
        else
            throw new IllegalArgumentException("Limit " + limit + " is out of range " + LIMIT_RANGE);
    }
    public int compareTo(Integer count) {
        if (this.limit == count)
            return 0;
        if (count == null)
            return 1;
        else
            return (int) (this.limit - count);
    }

    public boolean reachLimit(Integer count) {
        return compareTo(count) <= 0;
    }
}
