/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.util;

import com.netapp.offtap3.internal.mars.webmapi.model.common.ErrorType;

public enum MarsError {
    // Start manually entered section
    RESPONSE_DECODE_FAILED("2", "The response could not be decoded."),
    BODY_ENCODE_FAILED("3", "The body could not be encoded."),
    INVALID_URI("4", "The specified URI was invalid."),
    CONNECTION_FAILED("5", "Unable to connect to the specified host."),
    INVALID_PROTOCOL("6", "The specified connection protocol was invalid."),
    UNKNOWN("7", "The API call failed for an unknown reason."),
    UNAUTHORIZED("8", "The authentication information could not be validated."),
    // End manually entered section

    // Start generated section from Mars library
    EGENERICNOMEM("1", "The command failed because Mars OS is out of memory."),
    VOLUME_SHOW_VOLUME_NOT_FOUND("1000", "Volume not found."),
    VOLUME_OFFLINE_VOLUME_NOT_FOUND("1001", "Volume not found."),
    VOLUME_OFFLINE_VOLUME_ALREADY_OFFLINE("1002", "Volume already offline."),
    VOLUME_ONLINE_VOLUME_NOT_FOUND("1003", "Volume not found."),
    VOLUME_ONLINE_VOLUME_ALREADY_ONLINE("1004", "Volume already online."),
    TEST_CMD_ERROR_ENOVERSION("2001", "QA test"),
    TEST_CMD_ERROR_LUN_NOT_FOUND("2004", "QA test"),
    TEST_CMD_ERROR_CODE_LUN_NOT_FOUND("2005", "QA test test 1:n mapping for mapi error and mars error (n>1)."),
    CG_CREATE_EXISTS_CG("4001", "Consistency group name already in use."),
    CG_SHOW_CG_NOT_FOUND("4004", "Consistency group not found."),
    CG_MEMBER_REMOVE_CG_NOT_FOUND("4006", "Consistency group not found."),
    CG_CREATE_MAX_CG_COUNT_EXCEEDED("4010", "System's maximum consistency group count has been exceeded."),
    CG_DELETE_CG_NOT_FOUND("4011", "Consistency group not found."),
    CG_DELETE_CG_HAS_MEMBERS("4012", "Consistency group has existing members."),
    CG_DELETE_SNAPS_EXIST("4013", "Consistency group has snapshots."),
    CG_DELETE_INTERNAL_ERROR("4014", "Unable to fetch consistency group info internally."),
    CG_MEMBER_ADD_CG_NOT_FOUND("4015", "Consistency group not found."),
    CG_MEMBER_REMOVE_LUN_NOT_FOUND("4018", "Member LUN not found."),
    CG_MEMBER_REMOVE_LUN_UNGROUPED("4019",
                                   "One or more of the given member LUNs are not part of this consistency group."),
    CG_MEMBER_REMOVE_LUN_IN_OTHER_CG("4020", "Given member to be removed is part of a different consistency group."),
    CG_MEMBER_REMOVE_DISCARDED("4021",
                               "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    CG_CREATE_EXISTS_LUN("4022", "LUN name already in use."),
    CG_CREATE_LUN_NOT_FOUND("4023", "Member LUN not found."),
    CG_CREATE_LUN_IN_OTHER_CG("4024", "One of the given member LUNs is already part of another consistency group."),
    CG_CREATE_DISCARDED("4025",
                        "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    CG_SHOW_LUN_NOT_FOUND("4026", "Member LUN not found."),
    CG_DELETE_DISCARDED("4027",
                        "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    CG_MEMBER_ADD_LUN_NOT_FOUND("4028", "Member LUN not found."),
    CG_MEMBER_ADD_LUN_IN_OTHER_CG("4029", "LUN is already part of a different consistency group."),
    CG_MEMBER_ADD_DISCARDED("4030",
                            "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    SECURITY_LOGIN_DELETE_USER_NOT_FOUND("5000", "User not found."),
    SECURITY_LOGIN_DELETE_REQUEST_FAILED("5001", "Returned during a shell failure."),
    SECURITY_LOGIN_DELETE_REQUEST_IN_PROGRESS("5002", "Request in progress."),
    SECURITY_LOGIN_PASSWORD_USER_NOT_FOUND("5003", "User name not found."),
    SECURITY_LOGIN_PASSWORD_REQUEST_FAILED("5004", "Shell failure."),
    SECURITY_LOGIN_PASSWORD_REQUEST_IN_PROGRESS("5005", "Request in progress."),
    SECURITY_LOGIN_CREATE_USER_EXISTS("5006", "Username is already assigned to a user."),
    SECURITY_LOGIN_CREATE_REQUEST_FAILED("5007", "Shell failure."),
    SECURITY_LOGIN_CREATE_REQUEST_IN_PROGRESS("5008", "Request in progress."),
    SECURITY_LOGIN_PASSWORD_INVALID_PASSWORD("5009", "Invalid old password."),
    SECURITY_LOGIN_GET_USER_NOT_FOUND("5010", "User not found."),
    SECURITY_LOGIN_GET_REQUEST_FAILED("5011", "Memory corruption or unrecoverable logic error."),
    SECURITY_LOGIN_GET_SHUTDOWN_IN_PROGRESS("5012", "Security module is shutting down."),
    SECURITY_LOGIN_PASSWORD_SAME_PASSWORD("5013", "New password must be different from old password."),
    SECURITY_MARSDIAG_ENABLE_INVALID_PARAMETER("5014", "Password parameter was missing or empty."),
    SECURITY_MARSDIAG_ENABLE_REQUEST_FAILED("5015", "Shell failure."),
    SECURITY_MARSDIAG_DISABLE_REQUEST_FAILED("5016", "Shell failure."),
    SECURITY_CERTIFICATE_GENCERT_REQUEST_FAILED("5017", "Self-signed certificate generation failure"),
    SECURITY_CERTIFICATE_EXPORTCSR_REQUEST_FAILED("5018", "Certificate signing request exporting failure"),
    SECURITY_CERTIFICATE_EXPORTCERT_REQUEST_FAILED("5019", "Certificate exporting failure"),
    SECURITY_CERTIFICATE_GENKEY_REQUEST_FAILED("5020", "RSA private/public key pair generation failure"),
    SECURITY_CERTIFICATE_IMPORTCERT_REQUEST_FAILED("5021", "Certificate importing failure"),
    SECURITY_CERTIFICATE_ARCHIVEPKCS12_REQUEST_FAILED("5022", "Archiving certificate and key into pkcs12 file failure"),
    SECURITY_CERTIFICATE_GENCSR_REQUEST_FAILED("5023", "Certificate signing request generation failure"),
    SECURITY_CERTIFICATE_GENKEYSTORE_REQUEST_FAILED("5024", "Keystore generation failure"),
    SECURITY_CERTIFICATE_COMMIT_REQUEST_FAILED("5025", "PKI(Public Key Infrastructure) files commit failure"),
    SECURITY_CERTIFICATE_VERIFYCERT_REQUEST_FAILED("5026", "Certificate verification failure"),
    SECURITY_CERTIFICATE_SHOW_CERTIFICATE_NOT_FOUND("5027", "Certificate is not found"),
    SECURITY_CERTIFICATE_SHOW_REQUEST_FAILED("5028", "Memory corruption or unrecoverable logic error"),
    SECURITY_CERTIFICATE_SHOW_SHUTDOWN_IN_PROGRESS("5029", "Security service is shutting down"),
    SECURITY_CERTIFICATE_MODIFY_CERTIFICATE_NOT_FOUND("5030", "Certificate is not found"),
    SECURITY_CERTIFICATE_MODIFY_REQUEST_FAILED("5031", "Memory corruption or unrecoverable logic error"),
    SECURITY_CERTIFICATE_MODIFY_REQUEST_IN_PROGRESS("5032", "Request in progress"),
    SECURITY_CERTIFICATE_MODIFY_CERTIFICATE_GENERATION_FOUND("5033", "Failed to generate certificate"),
    SECURITY_CERTIFICATE_MODIFY_MUST_MODIFY_SOMETHING("5034", "Failed to specify something to modify"),
    IGROUP_SHOW_IGROUP_NOT_FOUND("6000", "Initiator group not found."),
    IGROUP_ADD_IGROUP_NOT_FOUND("6001", "Initiator group not found."),
    IGROUP_ADD_EXISTS("6002", "Initiator already exists in the initiator group."),
    IGROUP_ADD_INITIATOR_CONFLICTING_OS("6004",
                                        "One of the initiators being added belongs to an initiator group with a different operating system type."),
    IGROUP_REMOVE_IGROUP_NOT_FOUND("6006", "Initiator group not found."),
    IGROUP_REMOVE_INITIATOR_NOT_FOUND("6007", "Initiator not found."),
    IGROUP_CREATE_EXISTS("6008", "Initiator group already exists."),
    IGROUP_CREATE_INVALID_PARAM("6009", "Invalid parameter."),
    IGROUP_CREATE_INITIATOR_CONFLICTING_OS("6011",
                                           "One of the initiators being added to this new initiator group belongs to another initiator group with a different operating system type."),
    IGROUP_CREATE_MAX_INITIATORS("6012", "Maximum number of initiators across all initiator groups has been exceeded."),
    IGROUP_CREATE_MAX_INITIATORS_PER_IGROUP("6013", "Initiator group's maximum initiator count has been exceeded."),
    IGROUP_DESTROY_IGROUP_NOT_FOUND("6014", "Initiator group not found."),
    IGROUP_DESTROY_IGROUP_HAS_MAPS("6015", "Initiator group could not be destroyed because it has maps."),
    IGROUP_DESTROY_INTERNAL_ERROR("6016", "Unable to fetch information internally."),
    IGROUP_ADD_MAX_INITIATORS("6017", "Maximum number of initiators across all initiator groups has been exceeded."),
    IGROUP_ADD_MAX_INITIATORS_PER_IGROUP("6018",
                                         "Maximum number of initiators in the initiator group has been exceeded."),
    IGROUP_RENAME_IGROUP_NOT_FOUND("6019", "Initiator group not found."),
    IGROUP_RENAME_IGROUP_EXISTS("6020", "Initiator group name already in use."),
    IGROUP_CREATE_MAX_IGROUP_COUNT_EXCEEDED("6021", "Maximum initiator group count has been exceeded."),
    IGROUP_DESTROY_DISCARDED("6022",
                             "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    IGROUP_ADD_DUPLICATE_LUN("6023",
                             "One of the initiators being added is already mapped to one of the LUNs in the initiator groups."),
    IGROUP_ADD_LUN_ID_IN_USE("6024",
                             "One of the initiators being added already has a map that uses a LUN ID in use by this initiator group."),
    IGROUP_ADD_DUPLICATE_INPUTS("6025", "List of initiators to add contains one or more duplicates."),
    IGROUP_ADD_DISCARDED("6026",
                         "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    IGROUP_REMOVE_DUPLICATE_INPUTS("6027", "Initiator specified more than once."),
    IGROUP_REMOVE_DISCARDED("6028",
                            "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    IGROUP_RENAME_DISCARDED("6029",
                            "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    IGROUP_CREATE_DUPLICATE_INPUTS("6030", "Initiator specified more than once."),
    IGROUP_CREATE_DISCARDED("6031",
                            "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    IGROUP_MODIFY_IGROUP_NOT_FOUND("6032", "Initiator group not found."),
    IGROUP_MODIFY_DISCARDED("6033", "Internal error occurred and the operation was discarded."),
    IGROUP_MODIFY_NO_NAMES_GIVEN("6034", "You must specify names or universally unique IDs (UUIDs)."),
    IGROUP_MODIFY_INITIATOR_CONFLICTING_OS("6035",
                                           "Operation would have resulted in initiator(s) belonging to initiator groups of differing operating system types."),
    CLUSTER_CREATE_EXISTS("7000", "Cluster already exists."),
    CLUSTER_SHOW_CLUSTER_UNINITIALIZED("7001", "Cluster is uninitialized."),
    CLUSTER_STARTUP_DONE_SHOW_STARTUP_NOT_FINISHED("7008", "Startup is not finished."),
    CLUSTER_CREATE_INTERNAL_ERROR("7009", "Internal error occurred."),
    CLUSTER_CREATE_REQUEST_IN_PROGRESS("7010", "Another request is in progress."),
    CLUSTER_CREATE_SHELL_FAILURE("7011", "Cluster operation failed."),
    CLUSTER_MANAGEMENT_INTERFACE_SHOW_INTERNAL_ERROR("7017", "Internal error occurred."),
    CLUSTER_MANAGEMENT_INTERFACE_SHOW_REQUEST_IN_PROGRESS("7018", "Another request is in progress."),
    CLUSTER_MANAGEMENT_INTERFACE_SHOW_MANAGEMENT_IP_NOT_INITIALIZED("7019",
                                                                    "Cluster management IP address not initialized."),
    CLUSTER_MANAGEMENT_INTERFACE_SHOW_IPV4_UNINITIALIZED("7020", "IPv4 cluster management address not initialized."),
    CLUSTER_MANAGEMENT_INTERFACE_SHOW_IPV6_UNINITIALIZED("7021", "IPv6 cluster management address not initialized."),
    CLUSTER_MANAGEMENT_INTERFACE_SHOW_CLUSTER_SHUTDOWN("7022", "Cluster shutdown in progress."),
    CLUSTER_SHOW_INTERNAL_ERROR("7023", "Internal error occurred."),
    CLUSTER_SHOW_EXISTS("7024", "Cluster already exists."),
    CLUSTER_SHOW_SHELL_FAILURE("7025", "Cluster operation failed."),
    CLUSTER_SHOW_CLUSTER_SHUTDOWN("7026", "Cluster shutdown in progress."),
    CLUSTER_DISCOVER_NODES_INTERNAL_ERROR("7027", "Internal error occurred."),
    CLUSTER_MANAGEMENT_INTERFACE_MODIFY_INTERNAL_ERROR("7037", "Internal error occurred."),
    CLUSTER_MANAGEMENT_INTERFACE_MODIFY_REQUEST_IN_PROGRESS("7038", "Another request is in progress."),
    CLUSTER_MANAGEMENT_INTERFACE_MODIFY_SHELL_FAILURE("7039", "Cluster operation failed."),
    CLUSTER_MANAGEMENT_INTERFACE_MODIFY_CLUSTER_SHUTDOWN("7040", "Cluster shutdown in progress."),
    CLUSTER_MANAGEMENT_INTERFACE_MODIFY_CLUSTER_IPV6_NOT_SUPPORTED("7041", "IPv6 not supported."),
    SYSTEM_VOLTAGE_SHOW_DEVICE_OPEN("8012", "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_VOLTAGE_SHOW_GET_SENSOR_LIST("8013", "Failed to get sensor list from Environment Manager daemon."),
    SYSTEM_VOLTAGE_SHOW_GET_VOLT_INFO("8014", "Failed to get system voltage info from Environment Manager daemon."),
    SYSTEM_TEMP_SHOW_DEVICE_OPEN("8015", "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_TEMP_SHOW_GET_SENSOR_LIST("8016", "Failed to get sensor list from Environment Manager daemon."),
    SYSTEM_TEMP_SHOW_GET_TEMP_INFO("8017", "Failed to get system temperature info from Environment Manager daemon."),
    SYSTEM_FANS_SHOW_DEVICE_OPEN("8018", "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_FANS_SHOW_GET_FAN_INFO("8019", "Failed to get fan info from Environment Manager daemon."),
    SYSTEM_BATTERY_SHOW_DEVICE_OPEN("8020", "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_BATTERY_SHOW_GET_SENSOR_LIST("8021", "Failed to get sensor list from Environment Manager daemon."),
    SYSTEM_BATTERY_SHOW_GET_BAT_INFO("8022", "Failed to get NVRAM battery info from Environment Manager daemon."),
    SYSTEM_PSU_SHOW_DEVICE_OPEN("8023", "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_PSU_SHOW_GET_PSU_INFO("8024", "Failed to get power supply unit info from Environment Manager daemon."),
    SYSTEM_INFO_DEVICE_OPEN("8025", "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_INFO_GET_SYSTEM_INFO("8026", "Failed to get system info from Environment Manager daemon."),
    SYSTEM_CURRENT_SHOW_DEVICE_OPEN("8027", "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_CURRENT_SHOW_GET_SENSOR_LIST("8028", "Failed to get sensor list from Environment Manager daemon."),
    SYSTEM_CURRENT_SHOW_GET_CURR_INFO("8029", "Failed to get system current info from Environment Manager daemon."),
    SYSTEM_ENVIRONMENT_SHOW_DEVICE_OPEN("8030",
                                        "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_ENVIRONMENT_SHOW_GET_SENSOR_LIST("8031", "Failed to get sensor list from Environment Manager daemon."),
    SYSTEM_LED_SHOW_FAILED_TO_GET_LED_STATUS("8032", "Failed to get LED STATUS."),
    SYSTEM_FRU_SHOW_DEVICE_OPEN("8033", "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_FRU_SHOW_GET_SENSOR_LIST("8034", "Failed to get FRU list from Environment Manager daemon."),
    SYSTEM_WATCHDOG_DISABLE_DEVICE_OPEN("8035",
                                        "Failed to open socket to communicate with Environment Manager daemon."),
    SYSTEM_WATCHDOG_DISABLE_WATCHDOG_DISABLE("8036", "Failed to disable watchdog timer."),
    API_GET_VERSION_ENOVERSION("9000", "No API version is available."),
    API_GET_VERSION_EUPDATING("9001", "API is currently updating and version is very likely to change."),
    LIF_OFFLINE_LIF_NOT_FOUND("11001", "Interface not found."),
    LIF_OFFLINE_LIF_NOT_ONLINE("11002", "Offline failed because the interface is not currently online."),
    LIF_SHOW_LIF_NOT_FOUND("11003", "Interface not found."),
    LIF_ONLINE_LIF_NOT_FOUND("11004", "Interface not found."),
    LIF_ONLINE_LIF_NOT_OFFLINE("11005", "Online failed because the interface is not currently offline."),
    LIF_CREATE_EXISTS("11006", "Interface name already in use."),
    LIF_CREATE_PORT_NOT_FOUND("11007", "Port not found."),
    LIF_CREATE_INTERNAL_LIF_CREATE_FAILURE("11008", "Unexpected internal failure, check kernel traces."),
    LIF_CREATE_MAX_LIF_COUNT_EXCEEDED("11009", "Limit reached on number of interfaces."),
    LIF_INITIATOR_SHOW_LIF_NOT_FOUND("11010", "Interface not found."),
    LIF_INITIATOR_SHOW_INTERNAL_ERROR("11011", "Unable to fetch interfaces information internally."),
    LIF_INITIATOR_SHOW_IGROUP_NOT_FOUND("11012", "Initiator group not found."),
    LIF_OFFLINE_DISCARDED("11013",
                          "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LIF_OFFLINE_SERVICE_STOPPED("11014", "Interface state cannot be changed while the FCP service is stopped."),
    LIF_ONLINE_DISCARDED("11015",
                         "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LIF_ONLINE_SERVICE_STOPPED("11016", "Interface state can't be changed while the FCP service is stopped."),
    LIF_CREATE_DISCARDED("11017",
                         "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    AUTOSUPPORT_INVOKE_EVENT_MANAGER_UNAVAILABLE("12003", "Event Manager is unavailable to process this request."),
    AUTOSUPPORT_INVOKE_EVENT_MANAGER_FAILURE("12004", "Event Manager failed to process this request."),
    AUTOSUPPORT_INVOKE_AUTOSUPPORT_UNAVAILABLE("12005", "AutoSupport service is unavailable to process this request."),
    AUTOSUPPORT_DESTINATION_ADD_AUTOSUPPORT_DESTINATION_BAD("12018", "Malformed destination specified."),
    AUTOSUPPORT_DESTINATION_ADD_AUTOSUPPORT_CDB_IO_FAILURE("12019", "Failed interaction with cluster database."),
    AUTOSUPPORT_CALLHOME_ENABLE_AUTOSUPPORT_CDB_IO_FAILURE("12021", "Failed interaction with cluster database."),
    AUTOSUPPORT_DESTINATION_REMOVE_AUTOSUPPORT_TARGET_NO_MATCH("12022", "Destination not found."),
    AUTOSUPPORT_DESTINATION_REMOVE_AUTOSUPPORT_CDB_IO_FAILURE("12023", "Failed interaction with cluster database."),
    AUTOSUPPORT_DESTINATION_REMOVE_AUTOSUPPORT_NON_EDITABLE_DESTINATION("12024", "This destination is not removable."),
    AUTOSUPPORT_CALLHOME_SHOW_AUTOSUPPORT_CDB_IO_FAILURE("12025", "Failed interaction with cluster database."),
    AUTOSUPPORT_CALLHOME_DISABLE_AUTOSUPPORT_CDB_IO_FAILURE("12026", "Failed interaction with cluster database."),
    AUTOSUPPORT_DESTINATION_MODIFY_AUTOSUPPORT_TARGET_NO_MATCH("12027", "Destination not found."),
    AUTOSUPPORT_DESTINATION_MODIFY_AUTOSUPPORT_DESTINATION_BAD("12028", "Malformed destination specified."),
    AUTOSUPPORT_DESTINATION_MODIFY_AUTOSUPPORT_CDB_IO_FAILURE("12029", "Failed interaction with cluster database."),
    AUTOSUPPORT_DESTINATION_MODIFY_AUTOSUPPORT_MODIFY_DESTINATION_NO_ARGS("12030",
                                                                          "No fields were specified to be updated by the modify command."),
    AUTOSUPPORT_DESTINATION_MODIFY_AUTOSUPPORT_NON_EDITABLE_DESTINATION("12032",
                                                                        "Payload type and target URI are not editable for this destination."),
    AUTOSUPPORT_DESTINATION_SHOW_AUTOSUPPORT_CDB_IO_FAILURE("12034", "Failed interaction with cluster database."),
    AUTOSUPPORT_DESTINATION_SHOW_AUTOSUPPORT_TARGET_NO_MATCH("12035", "Destination not found."),
    AUTOSUPPORT_DESTINATION_SHOW_SUPPORTABILITY_CDB_ROW_VERSION_MISMATCH("12036", "Version mismatch in database."),
    AUTOSUPPORT_DESTINATION_ADD_AUTOSUPPORT_INVALID_URI("12037",
                                                        "Invalid target URI, must contain at least one non-whitespace character."),
    AUTOSUPPORT_CALLHOME_ENABLE_SUPPORTABILITY_CDB_ROW_VERSION_MISMATCH("12038", "Version mismatch in database."),
    AUTOSUPPORT_CALLHOME_SHOW_SUPPORTABILITY_CDB_ROW_VERSION_MISMATCH("12039", "Version mismatch in database."),
    AUTOSUPPORT_CALLHOME_DISABLE_SUPPORTABILITY_CDB_ROW_VERSION_MISMATCH("12040", "Version mismatch in database."),
    AUTOSUPPORT_HISTORY_SHOW_AUTOSUPPORT_FAILED_TO_RETRIEVE_STATUS("12041",
                                                                   "Failed to retrieve AutoSupport message history."),
    AUTOSUPPORT_DESTINATION_FACTORY_RESET_AUTOSUPPORT_BAD_DEFAULT_DATA("12042",
                                                                       "Default destination data is malformed."),
    AUTOSUPPORT_DESTINATION_FACTORY_RESET_AUTOSUPPORT_CDB_IO_FAILURE("12043",
                                                                     "Failed interaction with cluster database."),
    AUTOSUPPORT_DESTINATION_MODIFY_AUTOSUPPORT_INVALID_URI("12044",
                                                           "Invalid target URI, must contain at least one non-whitespace character."),
    AUTOSUPPORT_DESTINATION_MODIFY_SUPPORTABILITY_CDB_ROW_VERSION_MISMATCH("12045", "Version mismatch in database."),
    CLONE_CREATE_OBJ_NOT_FOUND("13000", "Object not found."),
    CLONE_CREATE_SNAP_NOT_FOUND("13001", "Snapshot not found."),
    CLONE_CREATE_EXISTS_CG("13003", "Consistency group name already in use."),
    CLONE_CREATE_NOT_FULLY_SUPPORTED("13004", "Not fully supported."),
    CLONE_CREATE_EXISTS_LUN("13005", "LUN name already in use."),
    CLONE_CREATE_MAX_CG_COUNT_EXCEEDED("13006", "System's maximum consistency group count has been exceeded."),
    CLONE_CREATE_MAX_LUN_COUNT_EXCEEDED("13007", "System's maximum LUN count has been exceeded."),
    CLONE_CREATE_DISCARDED("13008",
                           "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    DISK_FAIL_MAPI_DISK_NO_EXIST("14000", "Disk not found."),
    DISK_FAIL_MAPI_DISK_PARAMETER_ERROR("14001", "Provide only a disk name or UID, not both."),
    DISK_UNFAIL_MAPI_DISK_NO_EXIST("14003", "Disk not found."),
    DISK_UNFAIL_MAPI_DISK_PARAMETER_ERROR("14004", "Provide only a disk name or UID, not both."),
    DISK_LIST_INFO_MAPI_DISK_NO_EXIST("14006", "Disk not found."),
    DISK_LIST_INFO_MAPI_DISK_PARAMETER_ERROR("14007", "Provide only a disk name or UID, not both."),
    DISK_SET_LED_MAPI_DISK_NO_EXIST("14009", "Disk not found."),
    DISK_SET_LED_MAPI_DISK_PARAMETER_ERROR("14010", "Provide only a disk name or UID, not both."),
    DISK_SET_LED_MAPI_DISK_SET_LED_FAILED("14012", "Operation failed to set LED."),
    DISK_ZERO_MAPI_DISK_NO_EXIST("14016", "Disk not found."),
    DISK_ZERO_MAPI_DISK_PARAMETER_ERROR("14017", "Provide only a disk name or UID, not both."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_NO_EXIST("14019", "Disk not found."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_FW_FILE_INVALID("14021", "Provided disk firmware file is invalid."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_FW_UPDATE_FAILED("14022", "Disk firmware update failed."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_FW_UPTODATE("14023", "Disk firmware is already up-to-date."),
    DISK_SMART_LIST_INFO_MAPI_DISK_NO_EXIST("14024", "Disk not found."),
    DISK_SMART_LIST_INFO_MAPI_DISK_PARAMETER_ERROR("14025", "Provide only a disk name or UID, not both."),
    DISK_FAIL_MAPI_DISK_FAULTED("14027", "Disk is already failed."),
    DISK_PATH_LIST_INFO_MAPI_DISK_PATH_NO_EXIST("14028", "Disk not found."),
    DISK_PATH_LIST_INFO_MAPI_DISK_PATH_PARAMETER_ERROR("14029", "Provide only a disk name or UID, not both."),
    DISK_UNFAIL_MAPI_DISK_NOT_FAILED("14031", "Disk is not failed."),
    DISK_DD_MAPI_DISK_NO_EXIST("14032", "Name of disk provided does not exist."),
    DISK_DD_MAPI_DISK_PARAMETER_ERROR("14033", "In or out parameter is not entered."),
    DISK_DD_MAPI_DISK_FAILED("14034", "Operation failed."),
    DISK_DD_MAPI_DISK_FAULTED("14035", "Disk has been failed. Unfail the disk and try again."),
    DISK_SET_LED_MAPI_DISK_INVALID_ACTION("14036", "LED action is invalid."),
    DISK_ZERO_MAPI_DISK_ZEROING("14037", "Disk zeroing is already in progress."),
    DISK_ZERO_MAPI_DISK_SANTIZING("14038", "Disk sanitization in progress. Try again after sanitizing has completed."),
    DISK_ZERO_MAPI_DISK_FW_UPDATING("14039",
                                    "Disk firmware update in progress. Try again after the update has completed."),
    DISK_ZERO_MAPI_DISK_FAULTED("14040", "Disk has been failed. Unfail the disk and try again."),
    DISK_SANITIZE_MAPI_DISK_NO_EXIST("14041", "Disk not found."),
    DISK_SANITIZE_MAPI_DISK_PARAMETER_ERROR("14042", "Provide only a disk name or UID, not both."),
    DISK_SANITIZE_MAPI_DISK_ZEROING("14044", "Disk zeroing in progress. Try again after zeroing has completed."),
    DISK_SANITIZE_MAPI_DISK_SANTIZING("14045", "Disk sanitization is already in progress."),
    DISK_SANITIZE_MAPI_DISK_FW_UPDATING("14046",
                                        "Disk firmware update in progress. Try again after the update has completed."),
    DISK_SANITIZE_MAPI_DISK_FAULTED("14047", "Disk has been failed. Unfail the disk and try again."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_ZEROING("14048", "Disk zeroing in progress. Try again after zeroing has completed."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_SANTIZING("14049",
                                            "Disk sanitization in progress. Try again after sanitizing has completed."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_FW_UPDATING("14050", "Disk firmware update is already in progress."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_FW_FILENAME_INVALID("14051", "Disk firmware filename is invalid."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_FAULTED("14052", "Disk has been failed. Unfail the disk and try again."),
    DISK_FAIL_MAPI_DISK_SET_LED_FAILED("14053", "Failed to set disk LED to faulted state."),
    DISK_FAIL_MAPI_DISK_TRANSIENT_ERROR("14054", "Transient error occurred while faulting drive."),
    DISK_POWER_CTRL_MAPI_DISK_INVALID_ACTION("14055", "Incorrect action specified."),
    DISK_POWER_CTRL_MAPI_DISK_INVALID_UID("14056", "UID provided is invalid."),
    DISK_POWER_CTRL_MAPI_DISK_NO_EXIST("14057", "Disk not found."),
    DISK_POWER_CTRL_MAPI_DISK_PARAMETER_ERROR("14058", "Provide only a disk name or UID, not both."),
    DISK_POWER_CTRL_MAPI_DISK_POWER_CYCLE_FAILED("14059", "Failure to power cycle disk."),
    DISK_FAIL_MAPI_DISK_INVALID_UID("14060", "UID provided is invalid."),
    DISK_PATH_LIST_INFO_MAPI_DISK_PATH_INVALID_UID("14061", "UID provided is invalid."),
    DISK_UNFAIL_MAPI_DISK_INVALID_UID("14062", "UID provided is invalid."),
    DISK_LIST_INFO_MAPI_DISK_INVALID_UID("14063", "UID provided is invalid."),
    DISK_SET_LED_MAPI_DISK_INVALID_UID("14064", "UID provided is invalid."),
    DISK_ZERO_MAPI_DISK_INVALID_UID("14065", "UID provided is invalid."),
    DISK_SANITIZE_MAPI_DISK_INVALID_UID("14066", "UID provided is invalid."),
    DISK_UPDATE_DISK_FW_MAPI_DISK_INVALID_UID("14067", "UID provided is invalid."),
    DISK_SMART_LIST_INFO_MAPI_DISK_INVALID_UID("14068", "UID provided is invalid."),
    FCP_SRAMDUMP_ERROR_STATE("16002", "Adapter is in error state."),
    FCP_PORT_SHOW_ERROR_STATE("16003", "Adapter is in error state."),
    FCP_PORT_SHOW_INVALID_PORT("16004", "Adapter not found."),
    FCP_STOP_ALREADY_STOPPED("16005", "Fibre Channel target is already stopped."),
    FCP_STOP_DISCARDED("16006",
                       "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    FCP_START_ALREADY_STARTED("16007", "Fibre Channel target is already started."),
    FCP_START_DISCARDED("16008",
                        "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    FCP_PORT_MODIFY_LIFS_ONLINE("16009",
                                "All interfaces on this adapter must be down before the adapter can be offlined."),
    FCP_PORT_MODIFY_PORT_NOT_FOUND("16010", "Adapter not found."),
    FCP_PORT_MODIFY_DISCARDED("16011",
                              "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    FCP_PORT_MODIFY_SPEED_NOT_SUPPORTED("16012", "Requested speed is not supported by the adapter."),
    FCP_PORT_MODIFY_PORT_NOT_OFFLINE("16013", "Adapter must be offline to change the speed."),
    FCP_PORT_MODIFY_PORT_ALREADY_ONLINE("16014", "Adapter is already online."),
    FCP_PORT_MODIFY_PORT_ALREADY_OFFLINE("16015", "Adapter is already offline."),
    FCP_PORT_MODIFY_NO_ACTION_SPECIFIED("16016", "Must specify state or speed or both."),
    FCP_PORT_MODIFY_SPEED_NOT_CHANGED("16017", "Speed specified is the same as the current speed."),
    FCP_PING_INTERNAL_ERROR("16019", "Internal error finding the physical adapter for the interface."),
    FCP_PING_INVALID_WWPN("16020", "Invalid Fibre Channel World Wide Port Name (WWPN)."),
    FCP_PING_WWPN_NOT_FOUND("16021",
                            "Ping failed. The Fibre Channel World Wide Port Name (WWPN) specified does not exists in the Name Server database."),
    FCP_PING_INVALID_PORT_FCID("16022", "Port FCID is not a 6-digit hexadecimal number."),
    FCP_PING_INVALID_PAYLOAD("16023", "Payload specified is not valid."),
    FCP_PING_MISSING_DESTINATION_PORT_INFO("16024", "FCID/WWPN is missing."),
    FCP_PING_TOO_MANY_ARGUMENTS("16025",
                                "Too many arguments. Provide either World Wide Port Number (WWPN) or provide FCID, but not both."),
    FCP_PING_DESTINATION_UNREACHABLE("16027", "Destination did not respond."),
    FCP_PING_PAYLOAD_MISMATCH("16028", "Payload data mismatch detected. See event logs for details."),
    FCP_INTERFACE_MODIFY_INTERFACE_NOT_FOUND("16029", "The specified interface can not be found."),
    FCP_INTERFACE_MODIFY_INTERFACE_NOT_UP("16030", "Interface down operation failed because it is not currently up."),
    FCP_INTERFACE_MODIFY_INTERFACE_NOT_DOWN("16031", "Interface up operation failed because it is not currently down."),
    FCP_INTERFACE_MODIFY_NO_ACTION_SPECIFIED("16032", "Must specify -status-admin option."),
    FCP_INTERFACE_MODIFY_STATE_NOT_SUPPORTED("16033", "Requested state is not supported by the interface."),
    FCP_INTERFACE_MODIFY_DISCARDED("16034",
                                   "An internal error occurred and the operation was canceled. This is usually a transient error."),
    FCP_INTERFACE_MODIFY_SERVICE_STOPPED("16035",
                                         "FCP interface state can't be changed while the FCP service is stopped."),
    FCP_RESET_INVALID_ADAPTER("16036", "Adapter not found."),
    FCP_PING_INTERFACE_NOT_FOUND("16037", "Interface not found."),
    FCP_PING_INTERFACE_OFFLINE("16038", "Interface is not online or FCP is stopped."),
    FCP_SRAMDUMP_INVALID_ADAPTER("16039", "Adapter not found."),
    LOADGEN_START_IO_ALREADY_STARTED("17001", "Load generator is already running."),
    LOADGEN_START_IO_STOPPING("17002", "Load generator is stopping."),
    LOADGEN_START_LUN_NOT_FOUND("17003", "LUN not found."),
    SASADMIN_ADAPTER_STATE_INFO_PORT_NO_EXIST("19006",
                                              "Name or Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    SASADMIN_EXPANDER_CLI_SHELF_NO_EXIST("19007", "Shelf not found."),
    SASADMIN_EXPANDER_CLI_MODULE_NO_EXIST("19008", "Module in the shelf does not exist."),
    SASADMIN_EXPANDER_CLI_PORT_NO_EXIST("19009", "Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    SASADMIN_EXPANDER_CLI_CMD_NO_EXIST("19010", "Expander command not found."),
    SASADMIN_EXPANDER_CLI_CMD_FAILED("19011", "Expander command failed."),
    SASADMIN_MODIFY_PORT_STATE_PORT_NO_EXIST("19012",
                                             "Name or Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    SASADMIN_EXPANDER_STATE_INFO_SHELF_NO_EXIST("19013", "Shelf not found."),
    SASADMIN_EXPANDER_STATE_INFO_MODULE_NO_EXIST("19014", "Module in the shelf does not exist."),
    SASADMIN_EXPANDER_STATE_INFO_PORT_NO_EXIST("19015",
                                               "Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    DEVICE_STATS_INFO_PORT_NO_EXIST("19016", "Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    SASADMIN_EXPANDER_CTRL_SHELF_NO_EXIST("19022", "Shelf not found."),
    SASADMIN_EXPANDER_CTRL_MODULE_NO_EXIST("19023", "Module in the shelf does not exist."),
    SASADMIN_EXPANDER_CTRL_PORT_NO_EXIST("19024", "Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    SASADMIN_EXPANDER_CTRL_CMD_NO_EXIST("19025", "Expander command not found."),
    SASADMIN_EXPANDER_CTRL_CMD_FAILED("19026", "Expander command failed."),
    SAS_PORT_RESET_PORT_NO_EXIST("19028", "Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    SAS_PORT_RESCAN_PORT_NO_EXIST("19029", "Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    PORT_PHY_CTRL_MAPI_M_MAP_BAD_INPUT("19030",
                                       "Provide either disk name or UID, not both. It also takes shelf-id and phy-id."),
    PORT_PHY_CTRL_PORT_NO_EXIST("19031", "Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    PORT_PHY_CTRL_CMD_FAILED("19032", "Device phy control command failed."),
    PORT_PHY_CTRL_PHY_NO_EXIST("19033", "PHY not found."),
    DEVICE_PHY_CTRL_MAPI_DISK_NO_EXIST("19034", "Disk not found."),
    DEVICE_PHY_CTRL_MAPI_M_MAP_BAD_INPUT("19035",
                                         "Provide either disk name or UID, not both. It also takes shelf-id and phy-id."),
    DEVICE_PHY_CTRL_PORT_NO_EXIST("19037", "Serial Attached SCSI Host Bus Adapter (SAS HBA) port not found."),
    DEVICE_PHY_CTRL_CMD_FAILED("19038", "Device phy control command failed."),
    SASADMIN_EXPANDER_CTRL_PHY_NO_EXIST("19039", "PHY not found."),
    DEVICE_PHY_CTRL_MAPI_DISK_INVALID_UID("19040", "UID provided is invalid."),
    SNAP_CREATE_EXISTS("20000", "Snapshot name is already in use by an existing snapshot of that object."),
    SNAP_CREATE_INTERNAL_ERROR("20001", "Unable to fetch snapshot info internally."),
    SNAP_CREATE_OBJ_NOT_FOUND("20002", "Object not found."),
    SNAP_CREATE_SUB_CG_OP("20003",
                          "Snapshot creation disallowed since this LUN is currently part of a consistency group."),
    SNAP_CREATE_SNAP_NAMEGEN_FAILED("20004", "Default snapshot name could not be generated."),
    SNAP_CREATE_NOT_FULLY_SUPPORTED("20005",
                                    "WARNING - Limited Support: A snapshot name was created and saved in the data base, no LUN user data was actually captured in a snapshot, as this functionality is not yet supported."),
    SNAP_CREATE_MAX_SNAP_COUNT_EXCCEDED("20006", "System's maximum snapshot count has been reached."),
    SNAP_SHOW_SNAP_NOT_FOUND("20007", "Snapshot not found."),
    SNAP_SHOW_OBJ_NOT_FOUND("20008", "Object not found."),
    SNAP_DELETE_OBJ_NOT_FOUND("20009", "Consistency group/LUN not found."),
    SNAP_DELETE_SNAP_NOT_FOUND("20010", "Snapshot not found."),
    SNAP_CREATE_DISCARDED("20011",
                          "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    SNAP_DELETE_DISCARDED("20012",
                          "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    STATS_STOP_OBJECT_NOT_FOUND("21000", "Object not found."),
    STATS_STOP_INSTANCE_NOT_FOUND("21001", "Instances not found."),
    STATS_STOP_COUNTER_NOT_FOUND("21002", "Counters not found."),
    STATS_STOP_ID_NOT_FOUND("21003", "ID requested is not valid."),
    STATS_START_OBJECT_NOT_FOUND("21004", "Object not found."),
    STATS_START_INSTANCE_NOT_FOUND("21005", "Instances not found."),
    STATS_START_COUNTER_NOT_FOUND("21006", "Counters not found."),
    STATS_SYSCONFIG_OBJECT_NOT_FOUND("21007", "Object not found."),
    STATS_SYSCONFIG_INSTANCE_NOT_FOUND("21008", "Instances not found."),
    STATS_SYSCONFIG_COUNTER_NOT_FOUND("21009", "Configurations not found."),
    STATS_SYSCONFIG_INCORRECT_ARGUMENTS("21010", "Arguments specified were incorrect."),
    STATS_SHOW_OBJECT_NOT_FOUND("21011", "Object not found."),
    STATS_SHOW_INSTANCE_NOT_FOUND("21012", "Instances not found."),
    STATS_SHOW_COUNTER_NOT_FOUND("21013", "Counters not found."),
    STATS_SHOW_INCORRECT_ARGUMENTS("21014",
                                   "The arguments specified were incorrect. Possible reasons: Interval specified is greater than duration, value of interval or duration is negative, specified an interval and no duration (or vice versa)."),
    STATS_SHOW_COUNTERS_NOT_SET("21015", "Counter names were not specified in the input."),
    STATS_SHOW_INSTANCES_NOT_SET("21016", "Instances were not specified in the input."),
    STATS_SHOW_OBJECT_NOT_SET("21017", "Object was not specified in the input."),
    STATS_LIST_OBJECT_NOT_FOUND("21018", "Object not found."),
    STATS_START_DISK_NOT_AVAILABLE("21019", "There are no disks."),
    STATS_EXPLAIN_OBJECT_NOT_FOUND("21020", "Object not found."),
    STATS_EXPLAIN_COUNTER_NOT_FOUND("21021", "One of the counters requested is not valid."),
    STATS_SHOW_DISK_NOT_AVAILABLE("21022", "There are no disks."),
    STATS_LIST_LIST_NOT_SUPPORTED("21023", "List instances is not supported for this object."),
    STATS_STOP_CM_NOT_INIT("21024", "Counter manager is not initialized."),
    STATS_STOP_CM_IN_PRESHUTDOWN("21025", "Counter manager is in preshutdown; cannot service request."),
    STATS_START_CM_NOT_INIT("21026", "Counter manager is not initialized."),
    STATS_START_CM_IN_PRESHUTDOWN("21027", "Counter manager is in preshutdown; cannot service request."),
    STATS_START_CM_SCOPE_INSTANCE_SPECIFIED("21029",
                                            "Scope and instances are mutually exclusive and cannot be specified in the same request."),
    STATS_SYSCONFIG_CM_NOT_INIT("21030", "Counter manager is not initialized."),
    STATS_SYSCONFIG_CM_IN_PRESHUTDOWN("21031", "Counter manager is in preshutdown; cannot service request."),
    STATS_SHOW_CM_NOT_INIT("21032", "Counter manager is not initialized."),
    STATS_SHOW_CM_IN_PRESHUTDOWN("21033", "Counter manager is in preshutdown state; cannot service request."),
    STATS_SHOW_CM_SCOPE_INSTANCE_SPECIFIED("21035", "Scope and instances parameters are mutually exclusive."),
    STATS_SHOW_SORT_BY_COUNTER_NOT_SPECIFIED("21036",
                                             "The top and bottom parameters must be accompanied by sort-by-counter parameter."),
    STATS_SHOW_SORT_BY_STRING_COUNTER_NOT_ALLOWED("21037", "Unable to sort by a string type counter."),
    STATS_SHOW_SORT_BY_HISTOGRAM_COUNTER_NOT_ALLOWED("21038", "Unable to sort by a histogram type counter."),
    STATS_SHOW_SORT_BY_COUNTER_NOT_FOUND("21039", "Counter specified by sort-by-counter parameter not found."),
    STATS_SHOW_CM_TOP_BOTTOM_SPECIFIED("21040", "Top and bottom parameters are mutually exclusive."),
    STORAGE_ADAPTER_LIST_INFO_M_MAP_BAD_INPUT("22000", "Error in port name."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_DAEMON_TIME_OUT("23000", "Service processor daemon timed out."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_HARDWARE_TIME_OUT("23001", "Service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_HARDWARE_DOWN("23002", "Service processor unavailable."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_EINVAL("23003", "Invalid argument."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_EINTR("23004", "Communication with the service processor was interrupted."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_ETIMEDOUT("23005", "Communication with the service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_EIO("23006", "Communication error with the service processor."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_EBADF("23007", "Communication port to the service processor daemon is unusable."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_EPIPE("23008",
                                           "Communication port to the service processor was unexpectedly closed."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_EBADE("23009", "Service processor replied with unexpected data."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_ENODATA("23010", "Service processor replied with insufficient data."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_DAEMON_TIME_OUT("23011", "Service processor daemon timed out."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_HARDWARE_TIME_OUT("23012", "Service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_HARDWARE_DOWN("23013", "Service processor unavailable."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_EINVAL("23014", "Invalid argument."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_EINTR("23015",
                                                   "Communication with the service processor was interrupted."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_ETIMEDOUT("23016", "Communication with the service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_EIO("23017", "Communication error with the service processor."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_EBADF("23018",
                                                   "Communication port to the service processor daemon is unusable."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_EPIPE("23019",
                                                   "Communication port to the service processor was unexpectedly closed."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_EBADE("23020", "Service processor replied with unexpected data."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_ENODATA("23021", "Service processor replied with insufficient data."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_DAEMON_TIME_OUT("23022", "Service processor daemon timed out."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_HARDWARE_TIME_OUT("23023", "Service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_HARDWARE_DOWN("23024", "Service processor unavailable."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_EINVAL("23025", "Invalid argument."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_EINTR("23026", "Communication with the service processor was interrupted."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_ETIMEDOUT("23027", "Communication with the service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_EIO("23028", "Communication error with the service processor."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_EBADF("23029",
                                                "Communication port to the service processor daemon is unusable."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_EPIPE("23030",
                                                "Communication port to the service processor was unexpectedly closed."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_EBADE("23031", "Service processor replied with unexpected data."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_ENODATA("23032", "Service processor replied with insufficient data."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_DAEMON_TIME_OUT("23033", "Service processor daemon timed out."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_HARDWARE_TIME_OUT("23034", "Service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_HARDWARE_DOWN("23035", "Service processor unavailable."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_EINVAL("23036", "Invalid argument."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_EINTR("23037", "Communication with the service processor was interrupted."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_ETIMEDOUT("23038", "Communication with the service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_EIO("23039", "Communication error with the service processor."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_EBADF("23040",
                                                 "Communication port to the service processor daemon is unusable."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_EPIPE("23041",
                                                 "Communication port to the service processor was unexpectedly closed."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_ENODATA("23042", "Service processor replied with insufficient data."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_DAEMON_TIME_OUT("23043", "Service processor daemon timed out."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_HARDWARE_TIME_OUT("23044", "Service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_HARDWARE_DOWN("23045", "Service processor unavailable."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_EINVAL("23046", "Invalid argument."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_EINTR("23047",
                                                     "Communication with the service processor was interrupted."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_ETIMEDOUT("23048",
                                                         "Communication with the service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_EIO("23049", "Communication error with the service processor."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_EBADF("23050",
                                                     "Communication port to the service processor daemon is unusable."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_EPIPE("23051",
                                                     "Communication port to the service processor was unexpectedly closed."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_EBADE("23052", "Service processor replied with unexpected data."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_ENODATA("23053", "Service processor replied with insufficient data."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_MUST_SPECIFY_DHCP_OR_IP("23054", "DHCP or IP address not specified."),
    SYSTEM_HEALTH_ALERT_CREATE_HM_EINVAL("23056", "Invalid argument."),
    SYSTEM_HEALTH_ALERT_DELETE_HM_EINVAL("23057", "Invalid Argument."),
    SYSTEM_HEALTH_ALERT_REFRESH_HM_EINVAL("23058", "Internal error."),
    SYSTEM_HEALTH_ALERT_SHOW_HM_EINVAL("23059", "Invalid Argument."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_MUST_SPECIFY_ALL_IPV4("23060", "Missing gateway and/or netmask."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_PREFIX_BAD_VALUE("23061", "Bad IPv6 prefix length value."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_MUST_SPECIFY_ALL_IPV6("23062",
                                                                     "Missing gateway and/or router prefix length."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_IPV6_DISABLED_IN_POC_RELEASE("23063",
                                                                            "IPv6 support disabled in this release."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_DAEMON_TIME_OUT("23064", "Service processor daemon timed out."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_HARDWARE_TIME_OUT("23065", "Service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_HARDWARE_DOWN("23066", "Service processor unavailable."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_EINVAL("23067", "Invalid argument."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_EINTR("23068",
                                                    "Communication with the service processor was interrupted."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_ETIMEDOUT("23069", "Communication with the service processor timed out."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_EIO("23070", "Communication error with the service processor."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_EBADF("23071",
                                                    "Communication port to the service processor daemon is unusable."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_EPIPE("23072",
                                                    "Communication port to the service processor was unexpectedly closed."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_ENODATA("23073", "Service processor replied with insufficient data."),
    SYSTEM_HEALTH_ALERT_CREATE_HM_SOCKET_M_ENOTCONN("23074", "Connection with health monitor failed."),
    SYSTEM_HEALTH_ALERT_CREATE_HM_SOCKET_CLOSED("23075", "Connection with health monitor closed."),
    SYSTEM_HEALTH_ALERT_DELETE_HM_SOCKET_M_ENOTCONN("23076", "Connection with health monitor failed."),
    SYSTEM_HEALTH_ALERT_DELETE_HM_SOCKET_CLOSED("23077", "Connection with health monitor closed."),
    SYSTEM_HEALTH_ALERT_REFRESH_HM_SOCKET_M_ENOTCONN("23078", "Connection with health monitor failed."),
    SYSTEM_HEALTH_ALERT_REFRESH_HM_SOCKET_CLOSED("23079", "Connection with health monitor closed."),
    SYSTEM_HEALTH_ALERT_REFRESH_HM_SUBSYSTEM_INCONSISTENT_STATE("23080",
                                                                "Health monitor subsystem encountered a problem."),
    SYSTEM_HEALTH_ALERT_SHOW_HM_SOCKET_M_ENOTCONN("23081", "Connection with health monitor failed."),
    SYSTEM_HEALTH_ALERT_SHOW_HM_SOCKET_CLOSED("23082", "Connection with health monitor closed."),
    SYSTEM_COREDUMP_DELETE_SUPPORTABILITY_NO_ARGUMENTS("23083", "No arguments were specified."),
    SYSTEM_COREDUMP_DELETE_SUPPORTABILITY_INCOMPATIBLE_ARGUMENTS("23084", "Incompatible arguments."),
    SYSTEM_COREDUMP_DELETE_SUPPORTABILITY_FAILED_TO_QUERY_CORE_FOLDER_PATH("23085",
                                                                           "Failed to query the folder path for cores."),
    SYSTEM_COREDUMP_DELETE_SUPPORTABILITY_NODE_NOT_IN_CLUSTER("23086", "Node not found."),
    SYSTEM_COREDUMP_DELETE_SUPPORTABILITY_CORE_NOT_FOUND("23087", "Core not found."),
    SYSTEM_LOG_SHOW_SUPPORTABILITY_FILE_OPERATION_FAILURE("23088", "File operation failure."),
    SYSTEM_LOG_SHOW_SUPPORTABILITY_NODE_NOT_IN_CLUSTER("23089", "Node not found."),
    SYSTEM_IMAGE_PACKAGE_DESTROY_IMAGE_NOT_FOUND("23090", "Failed to find image at specified location."),
    SYSTEM_IMAGE_PACKAGE_DESTROY_DESTROY_ERROR("23091", "Failed to destroy the downloaded install image."),
    SYSTEM_IMAGE_PACKAGE_DESTROY_UPDATE_CANCEL_ERROR("23092", "Failed to cancel the pending update."),
    SYSTEM_IMAGE_GET_INVALID_RENAME("23093", "Filename is not valid."),
    SYSTEM_IMAGE_GET_INVALID_URL("23094", "Input was not a valid URL."),
    SYSTEM_IMAGE_GET_NETWORK_INIT_ERROR("23095", "Failed to initialize the network stack."),
    SYSTEM_IMAGE_GET_DOWNLOAD_ERROR("23096", "Failed to download remote file."),
    SYSTEM_IMAGE_GET_WRITE_ERROR("23097", "Failed to write to local file."),
    SYSTEM_IMAGE_GET_ACCESS_ERROR("23098", "Failed to access the remote file."),
    SYSTEM_IMAGE_GET_FILE_EXIST_ERROR("23099", "Local file already exists."),
    SYSTEM_IMAGE_GET_FILESYSTEM_ERROR("23100", "Can't access local install directory."),
    SYSTEM_IMAGE_UPDATE_IMAGE_NOT_FOUND("23101", "Failed to find image at specified location."),
    SYSTEM_IMAGE_UPDATE_IMAGE_NO_MEMORY("23102", "Failed to allocate memory."),
    SYSTEM_IMAGE_UPDATE_NETWORK_INIT_ERROR("23103", "Failed to initialize the network stack."),
    SYSTEM_IMAGE_UPDATE_ACCESS_ERROR("23104", "Failed to access the remote file."),
    SYSTEM_COREDUMP_SHOW_SUPPORTABILITY_FAILED_TO_QUERY_CORE_FOLDER_PATH("23108",
                                                                         "Failed to query the folder path for cores."),
    SYSTEM_COREDUMP_SHOW_SUPPORTABILITY_NODE_NOT_IN_CLUSTER("23109", "Node not found."),
    SYSTEM_IMAGE_UPDATECANCEL_UPDATE_CANCEL_ERROR("23110", "Failed to cancel the pending update."),
    SYSTEM_IMAGE_UPDATECANCEL_NO_UPDATE_PENDING("23111", "No pending update to cancel."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_ENOTCONN("23112", "Failed to connect to the service processor daemon."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_RETRY("23113", "Service processor replied with retry request."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SP_ERROR_INTERNAL("23114", "Service processor internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_REFRESH_USERS_SPD_ERROR_INTERNAL("23115",
                                                              "Service processor daemon internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_ENOTCONN("23116", "Failed to connect to the service processor daemon."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_RETRY("23117", "Service processor replied with retry request."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_ERROR_INTERNAL("23118", "Service processor internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SPD_ERROR_INTERNAL("23119",
                                                               "Service processor daemon internal error occurred."),
    SYSTEM_IMAGE_GET_FILE_VERIFICATION_ERROR("23120", "Invalid file was downloaded."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_ENOTCONN("23121", "Failed to connect to the service processor daemon."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_RETRY("23122", "Service processor replied with retry request."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SP_ERROR_INTERNAL("23123", "Service processor internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_SHOW_SPD_ERROR_INTERNAL("23124",
                                                             "Service processor daemon internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_ENOTCONN("23125", "Failed to connect to the service processor daemon."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_RETRY("23126", "Service processor replied with retry request."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SP_ERROR_INTERNAL("23127", "Service processor internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_REBOOT_SP_SPD_ERROR_INTERNAL("23128", "Service processor daemon internal error occurred."),
    SYSTEM_COREDUMP_SHOW_SUPPORTABILITY_FAILED_TO_QUERY_CORE_FOLDER_CONTENTS("23129",
                                                                             "Failed to query folder contents of the core dump folder."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_ENOTCONN("23130", "Failed to connect to the service processor daemon."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_RETRY("23131", "Service processor replied with retry request."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SP_ERROR_INTERNAL("23132", "Service processor internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_SHOW_SPD_ERROR_INTERNAL("23133", "Service processor daemon internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_ENOTCONN("23134", "Failed to connect to the service processor daemon."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_RETRY("23135", "Service processor replied with retry request."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SP_ERROR_INTERNAL("23136", "Service processor internal error occurred."),
    SYSTEM_SERVICE_PROCESSOR_DEBUG_MODE_SPD_ERROR_INTERNAL("23137",
                                                           "Service processor daemon internal error occurred."),
    SYSTEM_UPTIME_SHOW_SUPPORTABILITY_NODE_NOT_IN_CLUSTER("23138", "Node not found."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_DISABLE_IPV4_EXCLUSIVE("23139",
                                                                      "IPv4 disable is an exclusive parameter."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_DISABLE_IPV6_EXCLUSIVE("23140",
                                                                      "IPv6 disable is an exclusive parameter."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_MUST_SPECIFY_IPV6_INFO("23141", "Missing IPv6 address."),
    SYSTEM_IMAGE_GET_OUT_OF_SPACE("23142", "Failed to download remote file, try removing old packages"),
    SYSTEM_IMAGE_GET_UNKNOWN_VERSION("23143", "Invalid file was downloaded."),
    SYSTEM_IMAGE_UPDATE_PENDING_ACCESS_ERROR("23144", "Failed to set image as the pending update."),
    SYSTEM_IMAGE_UPDATE_UNKNOWN_VERSION("23145", "Specified image has no version."),
    SYSTEM_SERVICE_PROCESSOR_NETWORK_MODIFY_SP_NOTHING_TO_MODIFY("23146",
                                                                 "You have not specified what parameter(s) to modify."),
    SYSTEM_HEALTH_ALERT_POLICY_MODIFY_HM_EINVAL("23147", "Internal error."),
    SYSTEM_HEALTH_ALERT_POLICY_MODIFY_HM_SOCKET_M_ENOTCONN("23148", "Connection with health monitor failed."),
    SYSTEM_HEALTH_ALERT_POLICY_MODIFY_HM_SOCKET_CLOSED("23149", "Connection with health monitor closed."),
    SYSTEM_HEALTH_ALERT_POLICY_MODIFY_HM_SUBSYSTEM_INCONSISTENT_STATE("23150",
                                                                      "Health monitor subsystem encountered a problem."),
    SYSTEM_IMAGE_PACKAGE_SHOW_CACHE_ERROR("23151", "Unable to clear or update the package info cache."),
    SYSTEM_IMAGE_PACKAGE_DESTROY_CACHE_ERROR("23152", "Unable to update the package info cache."),
    SYSTEM_HEALTH_ALERT_SHOW_HM_EPROTO("23153", "Protocol error."),
    SYSTEM_IMAGE_GET_CACHE_ERROR("23154", "Unable to update the package info cache."),
    SYSTEM_HEALTH_SHOW_HM_EPROTO("23155", "Protocol error."),
    SYSTEM_HEALTH_SHOW_HM_SOCKET_M_ENOTCONN("23156", "Connection with health monitor failed."),
    SYSTEM_HEALTH_SHOW_HM_SOCKET_CLOSED("23157", "Connection with health monitor closed."),
    TRACE_DUMP_CLIENT_UNKNOWN("24000", "Client not found."),
    TRACE_CONFIG_SETTINGS_INVALID("24001",
                                  "One of the trace configuration parameters is invalid.  This is a general catch-all status."),
    TRACE_CONFIG_CLIENT_UNKNOWN("24002", "Trace client name was not recognized."),
    TRACE_CONFIG_MULTIPLE_CLIENTS_NOT_ALLOWED("24003",
                                              "If setting categories (-C,<category>) you can't specify the client as 'all' unless the category is 'all' or 'none'."),
    TRACE_CONFIG_UNKNOWN_FLAG("24004", "One of the option flags was not recognized."),
    TRACE_CONFIG_UNKNOWN_SEV("24005", "Severity (-S,<sev>) was not recognized."),
    TRACE_CONFIG_UNKNOWN_CATEGORY("24006", "Category (-C,<cat>) was not recognized."),
    TRACE_CONFIG_UNKNOWN_OPTION("24007", "Options (-O,<option>) was not recognized."),
    TRACE_CONFIG_UNKNOWN_UTILITY("24008", "Trace utility (-U,<utility>) was not recognized."),
    TRACE_CONFIG_NO_SETTINGS_FLAG_GIVEN("24009", "No option flag given after the trace client name."),
    TRACE_CATEGORIES_GET_CLIENT_UNKNOWN("24010", "Client not found."),
    TRACE_CONFIG_GET_CLIENT_UNKNOWN("24011", "Client not found."),
    TRACE_SEVERITIES_GET_CLIENT_UNKNOWN("24012", "Client not found."),
    EVENT_MARK_INVALID_PARAMETER("25000", "Invalid comment string."),
    EVENT_MARK_EVENT_MANAGER_UNAVAILABLE("25001", "Event Manager is unavailable to process this request."),
    EVENT_MARK_EVENT_MANAGER_FAILURE("25002", "Event Manager failed to process this request."),
    FIU_DISABLE_ERR("26000", "UID returned FIU error."),
    FIU_DISABLE_FIU_NOT_INIT("26001", "FIU library could not be initialized."),
    FIU_DISABLE_FIU_LIB_DISABLED("26002", "FIU library is disabled for this build."),
    FIU_ENABLE_ERR("26003", "FIU library error."),
    FIU_ENABLE_FIU_LIB_DISABLED("26004", "FIU library is disabled for this build."),
    FIU_ENABLE_PERIODIC_INTERVAL_ERROR("26005", "FIU mode PERIODIC_INTERVAL requires the -time-interval parameter."),
    FIU_ENABLE_PERIODIC_CALL_ERROR("26006", "FIU mode PERIODIC_CALLS requires the -call-interval parameter."),
    FIU_ENABLE_TARGET_CONV_ERROR("26007", "Unable to convert the parameter specified to expected type."),
    FIU_ENABLE_FIU_INVALID_NAME("26008", "Unable to find the FIU specified."),
    FIU_ENABLE_FIU_SEQUENCE_ERROR("26009", "The -start/-stop parameters are not specified correctly."),
    FIU_CHAIN_ERR("26010", "FIU library error."),
    FIU_CHAIN_FIU_LIB_DISABLED("26011", "FIU library is disabled for this build."),
    FIU_CHAIN_PERIODIC_INTERVAL_ERROR("26012", "FIU mode PERIODIC_INTERVAL requires the -time-interval parameter."),
    FIU_CHAIN_PERIODIC_CALL_ERROR("26013", "FIU mode PERIODIC_CALLS requires the -call-interval parameter."),
    FIU_CHAIN_TARGET_CONV_ERROR("26014", "Unable to convert the parameter specified to expected type."),
    FIU_CHAIN_FIU_INVALID_NAME("26015", "Unable to find the FIU specified."),
    FIU_CHAIN_FIU_SEQUENCE_ERROR("26016", "The -start/-stop parameters are not specified correctly."),
    FIU_CHAIN_FIU_CHAIN_ID_ERR("26017",
                               "Chain IDs (parent or child) are invalid (or) parent id already exists which could potentially create cyclic dependencies."),
    FIU_EXECUTE_FIU_INVALID_NAME("26018", "Function has not been registered yet or function does not exist."),
    FIU_EXECUTE_FIU_INVALID_KEY_VALUE_PAIR("26019",
                                           "The user has either specified more than 10 key-value pairs or set of duplicate keys."),
    FIU_ENABLE_FIU_INVALID_KEY_VALUE_PAIR("26020",
                                          "The user has either specified more than 10 key-value pairs or set of duplicate keys."),
    FIU_ENABLE_FIU_DUPLICATE_CONDITION("26021",
                                       "The user has already enabled an fiu with the same name and the same condition."),
    LUN_OFFLINE_LUN_NOT_FOUND("27000", "LUN not found."),
    LUN_OFFLINE_LUN_ALREADY_OFFLINE("27001", "LUN is already offline."),
    LUN_OFFLINE_LUN_VOL_CORRUPT("27002", "LUN is corrupt."),
    LUN_OFFLINE_DISCARDED("27003",
                          "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_MAP_LUN_NOT_FOUND("27004", "LUN not found."),
    LUN_MAP_IGROUP_NOT_FOUND("27005", "Initiator group not found."),
    LUN_MAP_LUN_ID_OUT_OF_RANGE("27006", "Specified LUN ID is out of range (0 - 4095)."),
    LUN_MAP_LUN_ALREADY_MAPPED_TO_IGROUP("27007", "LUN is already mapped to the initiator group."),
    LUN_MAP_LUN_ID_IN_USE("27008", "LUN ID is already used by this initiator group."),
    LUN_MAP_LUN_ALREADY_MAPPED("27009",
                               "Specified LUN is already mapped to an initiator(s) in the initiator group via another initiator group."),
    LUN_MAP_LUN_ID_COULD_NOT_BE_SELECTED("27010",
                                         "Could not find an available LUN ID from 0-4095 to use -- all have been taken by initiators in the initiator group."),
    LUN_MAP_LUN_MAP_LIMIT_EXCEEDED("27011", "LUN already mapped to the maximum number of initiator groups."),
    LUN_MAP_LUN_VOL_CORRUPT("27012", "LUN is corrupt."),
    LUN_MAP_DISCARDED("27014",
                      "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_RESIZE_LUN_NOT_FOUND("27015", "LUN not found."),
    LUN_RESIZE_DISCARDED("27016",
                         "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_RESIZE_SHRINK_NOT_SUPPORTED("27017", "Only growing the LUN is allowed at this time."),
    LUN_RESIZE_MAX_SIZE_EXCEEDED("27018", "New size exceeds maximum LUN size."),
    LUN_RESIZE_NO_CHANGE("27019", "New size is the same as old LUN size."),
    LUN_RESIZE_LUN_RESIZE_FAILED("27020", "Internal error resizing LUN."),
    LUN_RESIZE_LUN_NVFAILED("27021", "LUN status is nvfail. Use lun modify command to clear nvfail status."),
    LUN_RESIZE_LUN_CORRUPT("27022", "LUN is corrupt."),
    LUN_ONLINE_LUN_NOT_FOUND("27023", "LUN not found."),
    LUN_ONLINE_LUN_ALREADY_ONLINE("27024", "LUN is already online."),
    LUN_ONLINE_LUN_VOL_CORRUPT("27025", "LUN is corrupt."),
    LUN_ONLINE_DISCARDED("27026",
                         "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_RENAME_LUN_NOT_FOUND("27027", "LUN not found."),
    LUN_RENAME_EXISTS_LUN("27028", "New name specified for this LUN is already in use by an existing LUN."),
    LUN_RENAME_DISCARDED("27030",
                         "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_CONSISTENCY_CHECK_LUN_NOT_FOUND("27031", "LUN not found."),
    LUN_CONSISTENCY_CHECK_LUN_NOT_OFFLINE("27032", "LUN must be offline for this operation."),
    LUN_CONSISTENCY_CHECK_LUN_NVFAILED("27033", "LUN status is nvfail."),
    LUN_CONSISTENCY_CHECK_LUN_ONLINE_FAILED("27034", "LUN was not corrupt, but onlining failed in persistence layer."),
    LUN_CONSISTENCY_CHECK_LUN_OFFLINE_FAILED("27035",
                                             "LUN could not be offlined in the persistence layer, consistency check could not be performed."),
    LUN_CONSISTENCY_CHECK_DISCARDED("27036",
                                    "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_MODIFY_SERIAL_NUMBER_IN_USE("27037", "New serial number is already in use."),
    LUN_MODIFY_LUN_NOT_FOUND("27038", "LUN not found."),
    LUN_MODIFY_LUN_NOT_OFFLINE("27039", "LUN must be offline for this operation."),
    LUN_MODIFY_LUN_NVFAILED("27040", "LUN status is nvfail. Use lun modify command to clear nvfail status."),
    LUN_MODIFY_LUN_CORRUPT("27041", "LUN is corrupt."),
    LUN_MODIFY_DISCARDED("27042",
                         "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_MODIFY_TOO_MANY_MODIFICATIONS("27043", "Too many changes requested."),
    LUN_MODIFY_COMMENT_TOO_LONG("27044", "Comment is longer than 255 characters."),
    LUN_MODIFY_MUST_MODIFY_SOMETHING("27045", "No change requested."),
    LUN_DESTROY_LUN_NOT_FOUND("27046", "LUN not found."),
    LUN_DESTROY_LUN_CORRUPT("27047", "LUN is corrupt."),
    LUN_DESTROY_LUN_MAPPED_AND_ONLINE("27048", "LUN is mapped and online."),
    LUN_DESTROY_INTERNAL_ERROR("27049", "Internal error: unable to fetch LUN information."),
    LUN_DESTROY_SNAPS_EXIST("27050", "LUN has snapshots."),
    LUN_DESTROY_LUN_ONLINE_FAILED("27051", "Internal error bringing LUN online."),
    LUN_DESTROY_LUN_OFFLINE_FAILED("27052", "Internal error taking LUN offline."),
    LUN_DESTROY_DISCARDED("27053",
                          "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_UNMAP_LUN_NOT_FOUND("27054", "LUN not found."),
    LUN_UNMAP_IGROUP_NOT_FOUND("27055", "Initiator group not found."),
    LUN_UNMAP_LUN_NOT_MAPPED_TO_IGROUP("27056", "LUN is not mapped to the specified initiator group."),
    LUN_UNMAP_BAD_PARAMS("27057", "Internal parameter translation failed."),
    LUN_UNMAP_DISCARDED("27058",
                        "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_UNMAP_LUN_CORRUPT("27060", "LUN is corrupt."),
    LUN_CREATE_EXISTS_LUN("27061", "Name specified for this LUN is already in use by an existing LUN."),
    LUN_CREATE_MAX_LUN_SIZE_EXCEEDED("27064", "Size exceeds the maximum LUN size."),
    LUN_CREATE_MAX_LUN_COUNT_EXCEEDED("27065", "Maximum LUN count has been exceeded."),
    LUN_CREATE_LUN_SIZE_TOO_SMALL("27066", "Size is less than or equal to 0."),
    LUN_CREATE_DISCARDED("27067",
                         "Internal error occurred and the operation was discarded.  This is usually a transient error."),
    LUN_SHOW_LUN_NOT_FOUND("27068", "LUN not found."),
    LUN_MAP_SHOW_LUN_NOT_FOUND("27069", "LUN not found."),
    LUN_MAP_SHOW_IGROUP_NOT_FOUND("27070", "Initiator group not found."),
    LUN_MAP_SHOW_TOO_MANY_OPTIONS_SPECIFIED("27071", "Too many options specified."),
    LUN_MAP_LUN_ID_IN_USE_VIA_OTHER_IGROUP("27072",
                                           "Specified LUN ID is already used by an initiator(s) in the initiator group via another initiator group."),
    LUN_RESIZE_TOTAL_PROVISIONED_SPACE_EXCEEDED("27073",
                                                "Creating this LUN would exceed the limit on provisioned space."),
    LUN_MODIFY_LUN_ALREADY_ONLINE("27074", "LUN is already online."),
    LUN_MODIFY_LUN_ALREADY_OFFLINE("27075", "LUN is already offline."),
    LUN_MODIFY_LUN_NOT_NVFAILED("27076", "LUN does not have nvfail status."),
    LUN_MODIFY_VOLUME_NOT_FOUND("27077", "Volume not found."),
    LUN_MODIFY_INCORRECT_MAX_EXTENT_SIZE("27078", "Internal error saving the extent size."),
    LUN_MODIFY_NO_CHANGE("27079", "No change requested."),
    LUN_CREATE_TOTAL_PROVISIONED_SPACE_EXCEEDED("27080",
                                                "Creating this LUN would exceed the limit on provisioned space."),
    NVSTORE_PERS_WRITE_LUN_NOT_FOUND("28000", "LUN not found."),
    NVSTORE_PERS_WRITE_NVSTORE_INPUT_INVALID("28001", "Input offset or length invalid."),
    NVSTORE_PERS_READHDR_LUN_NOT_FOUND("28002", "LUN not found."),
    SHELF_UPDATE_SHELF_FW_MAPI_SES_NO_EXIST("29000", "Shelf UID or name provided does not match any shelves."),
    SHELF_UPDATE_SHELF_FW_MAPI_SES_INVALID_FW_FILENAME("29001", "Shelf firmware filename provided is invalid."),
    SHELF_UPDATE_SHELF_FW_MAPI_SES_INVALID_FW_FILE_NOT_EXIST("29002", "Shelf firmware filename provided is invalid."),
    SHELF_UPDATE_SHELF_FW_MAPI_SES_SFU_INIT_FAILED("29003", "Shelf firmware update failed to initialize."),
    SHELF_UPDATE_SHELF_FW_MAPI_SES_SFU_OPTION_UNSUPPORTED("29004", "Shelf firmware update option not supported."),
    SHELF_UPDATE_SHELF_FW_MAPI_SES_SFU_FW_FILE_EMPTY("29005", "Shelf firmware file provided is empty."),
    SHELF_LIST_INFO_MAPI_SES_NO_EXIST("29010", "Shelf ID or name provided does not match any shelves."),
    SHELF_LIST_INFO_STORAGE_INVALID_PARAMETER("29011", "Invalid name or UID."),
    SHELF_FAULT_LIST_INFO_STORAGE_INVALID_PARAMETER("29012", "Invalid name or UID."),
    SHELF_FAULT_LIST_INFO_STORAGE_NON_EXISTANT_DEVICE("29013", "Shelf not found."),
    SHELF_UPDATE_SHELF_FW_MAPI_SES_INVALID_UID("29014", "UID provided is invalid."),
    ES_COMPRESSION_STOP_ALREADY_STOPPED("30000", "Compression already disabled."),
    ES_COMPRESSION_START_ALREADY_STARTED("30001", "Compression already enabled."),
    NVRAM_DIMM_INJECT_UNCORR_DEV_NOT_FOUND("31000", "DDR MRI element not found."),
    NVRAM_DIMM_INJECT_CORR_DEV_NOT_FOUND("31001", "DDR MRI element not found."),
    NVRAM_FLASH_VERIFY_META_DEV_NOT_FOUND("31002", "DDR MRI element not found."),
    NVRAM_FLASH_VERIFY_META_VERIFY_META_IMAGE_FAILED("31003", "Invalid image for Meta Verification."),
    NVRAM_FLASH_VERIFY_META_VERIFY_META_GET_BOARD_FAILED("31004", "Get Board Information for Meta Verify failed."),
    NVRAM_FLASH_VERIFY_META_VERIFY_META_OPEN_FILE_FAILED("31005", "Open Firmware file for Meta Verify failed."),
    NVRAM_FLASH_VERIFY_META_VERIFY_META_FAILED("31006", "Firmware Meta Verification failed."),
    NVRAM_FLASH_SELECT_DEV_NOT_FOUND("31011", "DDR MRI element not found."),
    NVRAM_FLASH_SELECT_SELECT_INVALID_IMAGE_FAILED("31012", "Invalid image ID."),
    NVRAM_FLASH_SELECT_SELECT_FAILED("31013", "Image ID selection failed."),
    NVRAM_FLASH_PROGRAM_DEV_NOT_FOUND("31014", "DDR MRI element not found."),
    NVRAM_FLASH_PROGRAM_PROGRAM_IMAGE_FAILED("31015", "Invalid image for programming."),
    NVRAM_FLASH_PROGRAM_PROGRAM_GET_BOARD_FAILED("31016", "Get board information for programming failed."),
    NVRAM_FLASH_PROGRAM_PROGRAM_GET_CONFIG_FAILED("31017", "Get NVRAM Configuration for programming failed."),
    NVRAM_FLASH_PROGRAM_PROGRAM_OPEN_FILE_FAILED("31018", "Open firmware file for programming failed."),
    NVRAM_FLASH_PROGRAM_PROGRAM_SELECT_FAILED("31019", "Selection of image after programming failed."),
    NVRAM_FLASH_PROGRAM_PROGRAM_FAILED("31020", "Firmware update failed."),
    NVRAM_NAND_WIPE_DEV_NOT_FOUND("31022", "Message statistics is not turned on."),
    NVRAM_FLASH_PROGRAM_META_DEV_NOT_FOUND("31023", "DDR MRI element not found."),
    NVRAM_FLASH_PROGRAM_META_PROGRAM_META_IMAGE_FAILED("31024", "Invalid image for programming metadata."),
    NVRAM_FLASH_PROGRAM_META_PROGRAM_META_GET_BOARD_FAILED("31025",
                                                           "Get board information for programming metadata failed."),
    NVRAM_FLASH_PROGRAM_META_PROGRAM_META_OPEN_FILE_FAILED("31026",
                                                           "Open firmware file for programming metadata failed."),
    NVRAM_FLASH_PROGRAM_META_PROGRAM_META_FAILED("31027", "Firmware metadata update failed."),
    NVRAM_FLASH_VALIDATE_DEV_NOT_FOUND("31028", "DDR MRI element not found."),
    NVRAM_FLASH_VALIDATE_VALIDATE_IMAGE_FAILED("31029", "Invalid image for validation."),
    NVRAM_FLASH_VALIDATE_VALIDATE_GET_BOARD_FAILED("31030", "Get board information for validate failed."),
    NVRAM_FLASH_VALIDATE_VALIDATE_READ_HDR_FAILED("31031", "Read header for validate failed."),
    NVRAM_FLASH_VALIDATE_VALIDATE_HDR_FAILED("31032", "Header check for validate failed."),
    NVRAM_FLASH_VALIDATE_VALIDATE_CRC_FAILED("31033", "CRC check for validate failed."),
    NVRAM_FLASH_VALIDATE_VALIDATE_META_FAILED("31034", "Meta check for validate failed."),
    NVRAM_NAND_UNWIPE_DEV_NOT_FOUND("31035", "Message statistics is not turned on."),
    NVRAM_NAND_BAD_BLOCK_SHOW_DEV_NOT_FOUND("31041", "DDR MRI element not found."),
    NVRAM_NAND_BAD_BLOCK_SHOW_INVALID_BB_ADDR("31043", "Invalid NAND bad block address."),
    NVRAM_NAND_BAD_BLOCK_REMAP_DEV_NOT_FOUND("31044", "DDR MRI element not found."),
    NVRAM_NAND_BAD_BLOCK_SHOW_INVALID_CHIP("31048", "Invalid NAND chip ID."),
    NVRAM_DIMM_INJECT_CORR_INJECT_NO_SELECT("31049", "Specify either single or multiple errors."),
    NVRAM_DIMM_INJECT_CORR_INJECT_ALL_SELECT("31050", "Specify either single and multiple errors, but not both."),
    NVRAM_DIMM_SPD_SHOW_DEV_NOT_FOUND("31051", "MRI TWI SPD EEPROM not found."),
    NVRAM_DIMM_INJECT_UNCORR_DESTAGE_DEV_NOT_FOUND("31052", "DDR MRI element not found."),
    NVRAM_NAND_BAD_BLOCK_REMAP_INVALID_CHIP("31053", "Invalid NAND chip ID."),
    NVRAM_FLASH_PROGRAM_LIST_FW_OPEN_DIR_FAILED("31054", "Open of NVRAM firmware directory failed."),
    NVRAM_FLASH_FIRMWARE_SHOW_LIST_FW_OPEN_DIR_FAILED("31055", "Open of NVRAM firmware directory failed."),
    NVRAM_DIMM_SPD_CLEAR_DEV_NOT_FOUND("31056", "MRI TWI SPD EEPROM not found."),
    NVRAM_DIMM_INJECT_UNCORR_INJECT_NO_SELECT("31057", "Specify either single or multiple errors."),
    NVRAM_DIMM_INJECT_UNCORR_INJECT_ALL_SELECT("31058", "Specify either single or multiple errors, but not both."),
    NVRAM_FLASH_SHOW_DEV_NOT_FOUND("31059", "DDR MRI element not found."),
    NVRAM_FLASH_PACKAGE_SHOW_DEV_NOT_FOUND("31060", "DDR MRI element not found."),
    NVRAM_FLASH_PACKAGE_SHOW_LIST_OPEN_FILE_FAILED("31061", "Open firmware file for listing failed."),
    NVRAM_FLASH_PACKAGE_SHOW_LIST_GET_HDR1_FAILED("31062", "Get first header for listing failed."),
    NVRAM_FLASH_PACKAGE_SHOW_LIST_GET_HDR2_FAILED("31063", "Get second header for listing failed."),
    LOADGEN2_CREATE_TEST_EXISTS("32000", "Test already exists."),
    LOADGEN2_CREATE_LU_NOT_FOUND("32001", "LUN not found."),
    LOADGEN2_CREATE_IO_ALREADY_STARTED("32002", "Load generation is already running."),
    LOADGEN2_CREATE_IO_STOPPING("32003", "Load generation is stopping."),
    LOADGEN2_START_TEST_NOT_FOUND("32004", "Test not found."),
    LOADGEN2_START_IO_ALREADY_STARTED("32005", "This test is already running."),
    LOADGEN2_START_IO_STOPPING("32006", "Load generation test is stopping."),
    LOADGEN2_STOP_TEST_NOT_FOUND("32007", "Test not found."),
    LOADGEN2_SET_TEST_NOT_FOUND("32008", "Test not found."),
    LOADGEN2_DELETE_LUN_NOT_FOUND("32009", "LUN not found."),
    LOADGEN2_DELETE_IO_ALREADY_STARTED("32010", "Load generation is running. Try again when stopped."),
    LOADGEN2_DELETE_IO_STOPPING("32011", "Load generation is stopping. Try again when stopped."),
    LOADGEN2_CREATE_TEST_INVALID_RECSZ("32012", "Invalid iosize (recsz). Must be multiple of 512 bytes."),
    LOADGEN2_CREATE_LU_DISK_FOUND("32013", "This test only allows Luns; it does not accept disks."),
    LOADGEN2_CREATE_LU_INVALID_DISK("32014", "Disk not found."),
    LOADGEN2_CREATE_LU_FOUND("32015", "This test only accepts disks; it does not accept luns."),
    STSB_LIST_INFO_MAPI_DISK_NO_EXIST("33000", "Disk not found."),
    STSB_LIST_INFO_MAPI_DISK_PARAMETER_ERROR("33001", "Provide only a disk name or UID, not both."),
    STSB_GET_CRASH_DUMP_MAPI_DISK_NO_EXIST("33003", "Disk not found."),
    STSB_GET_CRASH_DUMP_MAPI_DISK_PARAMETER_ERROR("33004", "Provide only a disk name or UID, not both."),
    STSB_GET_EVENT_LOG_MAPI_DISK_NO_EXIST("33006", "Disk not found."),
    STSB_GET_EVENT_LOG_MAPI_DISK_PARAMETER_ERROR("33007", "Provide only a disk name or UID, not both."),
    STSB_IDENTIFY_DEVICE_MAPI_DISK_NO_EXIST("33009", "Disk not found."),
    STSB_IDENTIFY_DEVICE_MAPI_DISK_PARAMETER_ERROR("33010", "Provide only a disk name or UID, not both."),
    STSB_LIST_INFO_MAPI_DISK_INVALID_UID("33012", "UID provided is invalid."),
    STSB_GET_CRASH_DUMP_MAPI_DISK_INVALID_UID("33013", "UID provided is invalid."),
    STSB_GET_EVENT_LOG_MAPI_DISK_INVALID_UID("33014", "UID provided is invalid."),
    STSB_IDENTIFY_DEVICE_MAPI_DISK_INVALID_UID("33015", "UID provided is invalid."),
    ARCHIVER_SHOW_INTERNAL_ERROR("34000", "Internal error occurred."),
    ARCHIVER_SHOW_ARCHIVER_NEGATIVE_TIMESTAMP("34001", "Timestamp specified must be positive."),
    ARCHIVER_SHOW_ARCHIVER_TIME_BEGIN_AFTER_END("34002", "Begin time must start before or at end time."),
    ARCHIVER_SHOW_ARCHIVER_INSTANCE_NOT_FOUND("34003", "Instance not found."),
    ARCHIVER_SHOW_ARCHIVER_COUNTER_NOT_FOUND("34004", "Counter not found."),
    ARCHIVER_SHOW_ARCHIVER_OBJECT_NOT_FOUND("34005", "Object not found."),
    ARCHIVER_SHOW_ARCHIVER_SCOPE_INSTANCE_NOT_SET("34006", "Scope or instances must be set."),
    ARCHIVER_SHOW_ARCHIVER_DB_QUERY_FAILED("34007", "Error returned from database query."),
    ARCHIVER_SHOW_ARCHIVER_TOO_MANY_INSTANCES("34008", "Too many instances specified, max = 4."),
    NETWORK_PORT_SHOW_INTERNAL_ERROR("35000", "Internal error occurred."),
    NETWORK_PORT_SHOW_REQUEST_IN_PROGRESS("35001", "Another request is in progress."),
    NETWORK_PORT_SHOW_SHELL_FAILURE("35002", "Network operation failed."),
    NETWORK_PORT_SHOW_NETWORK_SHUTDOWN("35003", "Network shutdown in progress."),
    NETWORK_DNS_SERVER_CLEAR_INTERNAL_ERROR("35004", "Internal error occurred."),
    NETWORK_DNS_SERVER_CLEAR_REQUEST_IN_PROGRESS("35005", "Another request is in progress."),
    NETWORK_DNS_SERVER_CLEAR_SHELL_FAILURE("35006", "Network operation failed."),
    NETWORK_DNS_SERVER_CLEAR_NETWORK_SHUTDOWN("35007", "Network shutdown in progress."),
    NETWORK_DNS_SERVER_SHOW_INTERNAL_ERROR("35008", "Internal error occurred."),
    NETWORK_DNS_SERVER_SHOW_REQUEST_IN_PROGRESS("35009", "Another request is in progress."),
    NETWORK_DNS_SERVER_SHOW_DNS_UNINITIALIZED("35010", "Network DNS server not initialized."),
    NETWORK_DNS_SERVER_SHOW_SHELL_FAILURE("35011", "Network operation failed."),
    NETWORK_DNS_SERVER_SHOW_NETWORK_SHUTDOWN("35012", "Network shutdown in progress."),
    NETWORK_DNS_SERVER_MODIFY_INTERNAL_ERROR("35013", "Internal error occurred."),
    NETWORK_DNS_SERVER_MODIFY_REQUEST_IN_PROGRESS("35014", "Another request is in progress."),
    NETWORK_DNS_SERVER_MODIFY_SHELL_FAILURE("35015", "Network operation failed."),
    NETWORK_DNS_SERVER_MODIFY_NETWORK_SHUTDOWN("35016", "Network shutdown in progress."),
    NETWORK_DNS_SERVER_MODIFY_NETWORK_IPV6_NOT_SUPPORTED("35017", "IPV6 not supported."),
    NETWORK_DNS_SERVER_MODIFY_NUM_NETWORK_DNS_SERVERS("35018", "Incorrect number of DNS servers."),
    SERVICE_DISK_STATS_INFO_SOCKET_ERROR("36000",
                                         "Socket error occurred; unable to retrieve service disk information."),
    SERVICE_DISK_STATS_INFO_NO_EXIST("36001", "Invalid disk name."),
    SERVICE_DISK_INFO_SOCKET_ERROR("36002", "Socket error occurred. Failed to retrieve service disk information.");
    // End generated section from Mars library

    private String key;
    private String description;
    private int intKey;

    MarsError(String key, String description) {
        this.key = key;
        intKey = Integer.parseInt(key);
        this.description = description;
    }

    public boolean hasKey(String key) {
        return this.key.equals(key);
    }

    public int getError() {
        return intKey;
    }

    public String getKey() {
        return key;
    }

    public boolean isError(ErrorType errorType) {
        return (this.key.equals(errorType.getErrorMessage().getKey()));
    }

    public String getReason() {
        return description;
    }

    public static MarsError valueOf(int error) {
        for (MarsError marsError : MarsError.values()) {
            if (marsError.getError() == error) {
                return marsError;
            }
        }
        return null;
    }
}
