/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.util;

import com.netapp.offtap3.internal.mars.webmapi.model.BaseDataType;

/**
 * OffsetCondition defines condition type to limit data records by offset in the data set
 * 
 * @author Jing Shi
 * @version $Revision$, May 7, 2013
 */
public class OffsetCondition extends BaseDataType implements Comparable<Integer> {
    public static final String QUERY_OFFSET_KEY = "offset";
    public static final String QUERY_OFFSET_FORMAT = "offset={positive-number}";
    private static String s_Query_EQ = "=";
    private static String s_LB = "[";
    private static String s_RB = "]";

    private Integer offset = Integer.MAX_VALUE;

    public OffsetCondition() {
    }

    public OffsetCondition(Integer offset) {
        setOffset(offset);
    }

    public Integer getOffset() {
        return offset;
    }

    public void setOffset(Integer offset) {
        if (ConditionHelper.OFFSET_RANGE.inRange(offset))
            this.offset = offset;
        else
            throw new IllegalArgumentException("Offset " + offset + " is out of range " + ConditionHelper.OFFSET_RANGE);
    }

    public boolean checkOffset() {
        return offset != null;
    }
    public int compareTo(Integer offset) {
        if (this.offset == offset)
            return 0;

        if (offset == null)
            return 1;
        else
            return (int) (this.offset - offset);
    }

    public boolean isAfter(Integer offset) {
        return compareTo(offset) < 0;
    }

    public String toNiceString() {
        StringBuilder sb = new StringBuilder();
        sb.append(QUERY_OFFSET_KEY).append(s_Query_EQ).append(offset);
        return sb.toString();
    }
}