/*
 * NETAPP CONFIDENTIAL
 *
 * All information contained herein is, and remains 
 * the property of NetApp, Inc. and its suppliers if any.
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.mars.webmapi.util;

import java.util.Comparator;

import com.netapp.offtap3.internal.mars.webmapi.model.common.WebData;

/**
 * SortCondition defines condition to sort data records
 * 
 * @author Jing Shi
 * @version $Revision$, Mar 25, 2013
 */
public class SortCondition implements Comparable<SortCondition>, Comparator<SortCondition>, WebData {
    public static final String QUERY_SORT_KEY = "sortby";
    public static final String QUERY_SORT_FORMAT = "sortby={field}(direction)";
    public static String s_Query_LB = "(";
    public static String s_Query_RB = ")";
    public static String s_Query_EQ = "=";

    /**
     * SortDirection defines enumeration of sorting direction
     */
    public enum SortDirection {
        /** ascendent direction */
        asc,
        /** descendant direction */
        desc;

        private static SortDirection[] s_directions = SortDirection.values();

        public static SortDirection getType(String name) {
            if (name == null || name.length() == 0)
                throw new IllegalArgumentException("Invalid argument:" + name);
            String lowercase = name.toLowerCase();
            for (SortDirection op : s_directions) {
                if (lowercase.equals(op.name())) {
                    return op;
                }
            }

            throw new IllegalArgumentException("Invalid sort type for:" + name);
        }
    }

    /** sorter's priority, smaller number is higher priority */
    private Integer priority;

    /** sorted by field */
    private String field;

    /** sorted by direction */
    private SortDirection direction;

    public SortCondition(String field, SortDirection direction) {
        if (field == null || direction == null)
            throw new IllegalArgumentException("Invalid sorter condition arguments " + field + direction);

        this.field = field;
        this.direction = direction;
    }

    public Integer getPriority() {
        return priority;
    }

    public void setPriority(Integer priority) {
        this.priority = priority;
    }

    public String getField() {
        return field;
    }

    public void setField(String field) {
        this.field = field;
    }

    public SortDirection getDirection() {
        return this.direction;
    }

    public void setDirection(SortDirection direction) {
        this.direction = direction;
    }

    public boolean isAscendant() {
        return this.direction == SortDirection.asc ? true : false;
    }

    @Override
    public int hashCode() {
        return field.hashCode() ^ direction.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null)
            return false;
        if (obj instanceof SortCondition) {
            SortCondition that = (SortCondition) obj;
            return this.field.equals(that.field) && this.direction.equals(that.direction);
        }

        return false;
    }

    @Override
    public int compareTo(SortCondition o) {
        return compare(this, o);
    }

    @Override
    public int compare(SortCondition o1, SortCondition o2) {
        if (o1 == null && o2 == null)
            return 0;
        if (o1 == null)
            return -1;
        if (o2 == null)
            return 1;

        return (o1.priority - o2.priority);
    }

    public String toString() {
        return super.toString() + "[" + QUERY_SORT_KEY + this.priority + "=" + this.field + "(" + this.direction + ")";
    }

    public static SortCondition parse(String sortValue) throws Exception {
        assert sortValue != null;

        int lbIdx = sortValue.indexOf(s_Query_LB);
        int rbIdx = sortValue.indexOf(s_Query_RB);
        if (lbIdx <= 0 || rbIdx >= sortValue.length() || lbIdx >= rbIdx)
            throw new IllegalArgumentException("Invalid query string: " +  sortValue);

        String field = sortValue.substring(0, lbIdx);
        SortDirection direction = SortDirection.getType(sortValue.substring(lbIdx + 1, rbIdx));

        return new SortCondition(field, direction);
    }
}
