/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/CModeCloneSpecImpl.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.File;
import com.netapp.offtap3.ontap.CModeCloneSpec;

public class CModeCloneSpecImpl implements CModeCloneSpec {
    private File destinationFile;
    private String token;
    private boolean createToken;
    private boolean autoDelete;
    private int tokenRetryInterval;
    private int tokenRetryCount;
    private boolean fileCertainToExist;

    public CModeCloneSpecImpl(Builder builder) {
        this.destinationFile = builder.destinationFile;
        this.createToken = builder.createToken;
        this.token = builder.token;
        this.autoDelete = builder.autoDelete;
        this.tokenRetryInterval = builder.tokenRetryInterval;
        this.tokenRetryCount = builder.tokenRetryCount;
        this.fileCertainToExist = builder.fileCertainToExist;
    }

    @Override
    public String getToken() {
        return token;
    }

    @Override
    public boolean createToken() {
        return createToken;
    }

    @Override
    public boolean autoDelete() {
        return autoDelete;
    }

    @Override
    public int getTokenRetryInterval() {
        return tokenRetryInterval;
    }

    @Override
    public int getTokenRetryCount() {
        return tokenRetryCount;
    }

    @Override
    public File getDestination() {
        return destinationFile;
    }
    
    @Override
	public boolean fileCertainToExist() {
		return fileCertainToExist;
	}
    
	public static class Builder implements CModeCloneSpec.Builder {
        private static final int DEFAULT_TOKEN_RETRY_INTERVAL = 60;
        private File destinationFile;
        private String token;
        private boolean createToken;
        private boolean autoDelete;
        private int tokenRetryInterval;
        private int tokenRetryCount;
        private boolean fileCertainToExist;

        public Builder(File destinationFile) {
            this.destinationFile = destinationFile;
        }

        @Override
        public CModeCloneSpec.Builder token(String token) {
            this.token = token;
            return this;
        }

        @Override
        public CModeCloneSpec.Builder createToken(boolean createToken) {
            this.createToken = createToken;
            return this;
        }

        @Override
        public CModeCloneSpec.Builder autoDelete(boolean autoDelete) {
            this.autoDelete = autoDelete;
            return this;
        }
        
        @Override 
        public CModeCloneSpec.Builder fileCertainToExist(boolean fileCertainToExist) {
        	this.fileCertainToExist = fileCertainToExist;
        	return this;
        }

        @Override
        public CModeCloneSpec.Builder tokenRetryInterval(int intervalInSeconds) {
            if (intervalInSeconds <= 0) {
                this.tokenRetryInterval = DEFAULT_TOKEN_RETRY_INTERVAL;
            } else {
                this.tokenRetryInterval = intervalInSeconds;
            }
            return this;
        }

        @Override
        public CModeCloneSpec.Builder tokenRetryCount(int retryCount) {
            if (retryCount < 0) {
                this.tokenRetryCount = 0;
            } else {
                this.tokenRetryCount = retryCount;
            }
            return this;
        }

        @Override
        public CModeCloneSpec build() {
            return new CModeCloneSpecImpl(this);
        }
    }
}
