package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.LogOperationException;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.EmsLogLevel;
import com.netapp.offtap3.ontap.EmsLogger;
import com.netapp.offtap3.ontap.EmsLoggerSpec;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;


public class EmsLoggerImpl implements EmsLogger {

    private static final Logger LOGGER = Logger.getLogger(EmsLoggerImpl.class);

    private final EmsLoggerSpec spec;
    private final InternalZapiHandler zapiHandler;

    public EmsLoggerImpl(InternalZapiHandler zapiHandler, EmsLoggerSpec spec) {
        this.zapiHandler = zapiHandler;
        this.spec = spec;
    }

    @Override
    public void log(EmsLogLevel level, int eventId, String message) {

        NaElement xi = new NaElement("ems-autosupport-log");
        xi.addNewChild("auto-support", String.valueOf(spec.isTriggerAutoSupport()));
        xi.addNewChild("category", spec.getCategory());
        xi.addNewChild("computer-name", spec.getComputerName());
        xi.addNewChild("event-description", message);
        xi.addNewChild("app-version", spec.getVersion());
        xi.addNewChild("event-id", String.valueOf(eventId));
        xi.addNewChild("event-source", spec.getEventSource());
        xi.addNewChild("log-level", String.valueOf(level.toZapiValue()));

        LOGGER.info(zapiHandler + " is logging at level " + level + " to EMS: " + message);

        try {
            zapiHandler.invokeZapi(xi, ZapiCache.NO_CACHE);
        } catch (Exception ex) {
            LOGGER.error("Unable to add message to EMS on controller. " + message, ex);
            throw new LogOperationException(zapiHandler + " was unable to add message to EMS on controller. " + message, ex);
        }
    }

    @Override
    public void log(String message) {
        log(spec.getDefaultLogLevel(), spec.getDefaultEventId(), message);
    }

    @Override
    public void log(EmsLogLevel level, String message) {
        log(level, spec.getDefaultEventId(), message);
    }
}
