package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.utils.StringUtils;
import com.netapp.offtap3.ontap.EmsLogLevel;
import com.netapp.offtap3.ontap.EmsLoggerSpec;
import org.apache.log4j.Logger;

import java.net.InetAddress;
import java.net.UnknownHostException;

public class EmsLoggerSpecImpl implements EmsLoggerSpec {

    private static final Logger logger = Logger.getLogger(EmsLoggerSpecImpl.class);

    private String computerName;
    private final int defaultEventId;
    private final String category;
    private final String version;
    private final EmsLogLevel defaultLevel;
    private final String eventSource;
    private final boolean autoSupport;

    private EmsLoggerSpecImpl(BuilderImpl spec) {
        // sanitize inputs and assign reasonable defaults
        if (!StringUtils.isReal(spec.computerName)) {
            try {
                computerName = InetAddress.getLocalHost().getHostName();
            } catch (UnknownHostException e) {
                logger.error("Cannot find the local host address" + e.getMessage());
                computerName = "unknown";
            }
        } else {
            computerName = spec.computerName;
        }

        defaultEventId = spec.defaultEventId;
        category = spec.category == null ? "" : spec.category;
        version = spec.version;
        defaultLevel = spec.defaultLevel == null ? EmsLogLevel.info : spec.defaultLevel;
        eventSource = spec.eventSource;
        autoSupport = spec.autoSupport;
    }

    public String getComputerName() {
        return computerName;
    }

    public int getDefaultEventId() {
        return defaultEventId;
    }

    public String getCategory() {
        return category;
    }

    public String getVersion() {
        return version;
    }

    public EmsLogLevel getDefaultLogLevel() {
        return defaultLevel;
    }

    public String getEventSource() {
        return eventSource;
    }

    public boolean isTriggerAutoSupport() {
        return autoSupport;
    }

    public static class BuilderImpl implements Builder {
        String computerName;
        int defaultEventId;
        String category;
        String version;
        EmsLogLevel defaultLevel;
        String eventSource;
        boolean autoSupport;

        @Override
        public Builder triggerAutoSupport(boolean autoSupport) {
            this.autoSupport = autoSupport;
            return this;
        }

        @Override
        public Builder category(String category) {
            this.category = category;
            return this;
        }

        @Override
        public Builder defaultLogLevel(EmsLogLevel defaultLevel) {
            this.defaultLevel = defaultLevel;
            return this;
        }

        @Override
        public Builder computerName(String computerName) {
            this.computerName = computerName;
            return this;
        }

        @Override
        public Builder version(String version) {
            this.version = version;
            return this;
        }

        @Override
        public Builder defaultEventId(int eventId) {
            this.defaultEventId = eventId;
            return this;
        }

        @Override
        public Builder eventSource(String eventSource) {
            this.eventSource = eventSource;
            return this;
        }

        @Override
        public EmsLoggerSpec build() {
            return new EmsLoggerSpecImpl(this);
        }
    }
}
