/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/ExportPolicyRuleImpl.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.ExportOperationException;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.ExportPolicyRule;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.List;

public class ExportPolicyRuleImpl implements ExportPolicyRule {
    private static final Logger LOGGER = Logger.getLogger(ExportPolicyRuleImpl.class);
    private int anonymousUserId;
    private String clientMatch;
    private String changeOwnershipMode;
    private String ntfsUnixSecurityOperations;
    private boolean isAllowDeviceCreationEnabled;
    private boolean isAllowSetUuidEnabled;
    private String policyName;
    private int ruleIndex;
    private List<String> accessProtocols;
    private List<String> readOnlySecurityFlavors;
    private List<String> readWriteSecurityFlavors;
    private List<String> superUserSecurityFlavors;
    private ExportPolicyImpl exportPolicy;

    public ExportPolicyRuleImpl(ExportPolicyImpl exportPolicy, int anonymousUserId, String clientMatch, String changeOwnershipMode, String ntfsUnixSecurityOperations,
                          boolean isAllowDeviceCreationEnabled, boolean isAllowSetUuidEnabled, String policyName, int ruleIndex, List<String> accessProtocols,
                          List<String> readOnlySecurityFlavors, List<String> readWriteSecurityFlavors, List<String> superUserSecurityFlavors) {
        this.exportPolicy = exportPolicy;
        this.anonymousUserId = anonymousUserId;
        this.clientMatch = clientMatch;
        this.changeOwnershipMode = changeOwnershipMode;
        this.ntfsUnixSecurityOperations = ntfsUnixSecurityOperations;
        this.isAllowDeviceCreationEnabled = isAllowDeviceCreationEnabled;
        this.isAllowSetUuidEnabled = isAllowSetUuidEnabled;
        this.policyName = policyName;
        this.ruleIndex = ruleIndex;
        this.accessProtocols = accessProtocols;
        this.readOnlySecurityFlavors = readOnlySecurityFlavors;
        this.readWriteSecurityFlavors = readWriteSecurityFlavors;
        this.superUserSecurityFlavors = superUserSecurityFlavors;
    }

    @Override
    public int getAnonymousUserId() {
        return anonymousUserId;
    }

    @Override
    public String getClientMatch() {
        return clientMatch;
    }

    @Override
    public String getChangeOwnershipMode() {
        return changeOwnershipMode;
    }

    @Override
    public String getNtfsUnixSecurityOperations() {
        return ntfsUnixSecurityOperations;
    }

    @Override
    public boolean isAllowDeviceCreationEnabled() {
        return isAllowDeviceCreationEnabled;
    }

    @Override
    public boolean isAllowSetUuidEnabled() {
        return isAllowSetUuidEnabled;
    }

    @Override
    public String getPolicyName() {
        return policyName;
    }

    @Override
    public int getRuleIndex() {
        return ruleIndex;
    }

    @Override
    public List<String> getAccessProtocols() {
        return accessProtocols;
    }

    @Override
    public List<String> getReadOnlySecurityFlavors() {
        return readOnlySecurityFlavors;
    }

    @Override
    public List<String> getReadWriteSecurityFlavors() {
        return readWriteSecurityFlavors;
    }

    @Override
    public List<String> getSuperUserSecurityFlavors() {
        return superUserSecurityFlavors;
    }

    @Override
    public void modifyRuleIndex(int newRuleIndex) throws ExportOperationException {
        LOGGER.debug("modifyRuleIndex: newRuleIndex = " + newRuleIndex + ", policyName = " + policyName);
        NaElement setIndex = new NaElement("export-rule-set-index");

        setIndex.addNewChild("policy-name", policyName);
        setIndex.addNewChild("rule-index", Integer.toString(ruleIndex));
        setIndex.addNewChild("new-rule-index", Integer.toString(newRuleIndex));

        try {
            exportPolicy.getControllerImpl().invokeZapi(setIndex, ZapiCache.NO_CACHE);
        } catch (ApiFailedException e) {
            String message = "modifyRuleIndex: Failed to modify export policy rule index " + policyName + "-" + ruleIndex;
            LOGGER.error(message);
            throw new ExportOperationException(message, e);
        }
    }

    @Override
    public void destroy() throws ExportOperationException {
        LOGGER.debug("destroy: policyName=" + exportPolicy.getName() + ", ruleIndex=" + ruleIndex);

        NaElement xi = new NaElement("export-rule-destroy");
        xi.addNewChild("policy-name", policyName);
        xi.addNewChild("rule-index", String.valueOf(ruleIndex));

        try {
            NaElement xo = exportPolicy.getControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
            exportPolicy.getRules().remove(this);
            exportPolicy.getControllerImpl().getZapiCache().blastCacheValues("export");
            LOGGER.info("Destroyed " + policyName + "-" + ruleIndex + ":" + xo.getAttr("status") + ".");
        } catch (ApiFailedException e) {
            LOGGER.error("destroy: Failed to destroy export policy rule " + policyName + "-" + ruleIndex, e);
            throw new ExportOperationException("Failed to destroy export policy rule " + policyName + "-" + ruleIndex, e);
        }
    }

}
