/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/FileOperationProcessImpl.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.FileOperationException;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.DataTransferStatus;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

public class FileOperationProcessImpl extends DataTransferProcessImpl {
    private static final Logger LOGGER = Logger.getLogger(FileOperationProcessImpl.class);
    private final String transferType;

    public FileOperationProcessImpl(DataSvmImpl dataSvm, String jobId, TransferType transferType) {
        super(dataSvm, jobId);
        this.transferType = transferType.name();
    }

    @Override
    public List<DataTransferStatus> getStatus() throws FileOperationException {
        LOGGER.debug("Enter getStatus() for controller: " + getDataSvm().getName() + ", operation type = " +
                             transferType);
        NaElement xi = new NaElement("file-" + transferType + "-get-iter");
        NaElement query = new NaElement("query");
        NaElement jobDetails = new NaElement("file-" + transferType + "-info");
        jobDetails.addNewChild("job-uuid", getJobId());
        query.addChildElem(jobDetails);
        xi.addChildElem(query);
        List<DataTransferStatus> results = new ArrayList<DataTransferStatus>();
        try {
        	NaElement xo = null;
        	if(transferType.equals(TransferType.move.toString()) || getDataSvm().isDirectSvm()){
        		xo = getDataSvm().invokeZapi(xi, ZapiCache.NO_CACHE);
        	} else {
        		xo = getDataSvm().getParentControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
        	}        	
            if (xo != null && xo.getChildByName("attributes-list") != null) {
                List<NaElement> fileOperationInfoEl = xo.getChildByName("attributes-list").getChildren();
                for (NaElement currXML : fileOperationInfoEl) {
                    String destinationPath = currXML.getChildContent("destination-path");
                    String uuid = currXML.getChildContent("job-uuid");
                    String scannerStatus = currXML.getChildContent("scanner-status");
                    DataTransferStatusImpl fileOperationInfo = new DataTransferStatusImpl(destinationPath, uuid, scannerStatus);
                    if (scannerStatus.equalsIgnoreCase(ScannerStates.data.toString()) ||
                            scannerStatus.equalsIgnoreCase(ScannerStates.complete.toString())) {
                        fileOperationInfo.setScannerProgressPercentage(
                                currXML.getChildIntValue("scanner-progress-percentage", 0));
                        fileOperationInfo.setDestinationReady(
                                Boolean.parseBoolean(currXML.getChildContent("is-destination-ready")));
                    }
                    if (scannerStatus.equalsIgnoreCase(ScannerStates.destroyed.toString()) ||
                            scannerStatus.equalsIgnoreCase(ScannerStates.destroying.toString())) {
                        fileOperationInfo.setFailureReason(currXML.getChildContent("last-failure-reason"));
                    }
                    results.add(fileOperationInfo);
                }
            }
        } catch (ApiFailedException e) {
            LOGGER.error("ZAPI failure trying to get file " + transferType + " status: " + getDataSvm().getName(), e);
        } catch (Exception e) {
            LOGGER.error("Unexpected error processing ZAPI result for file-" + transferType + "-get-iter: " + getDataSvm().getName(), e);
        }
        if (results.size() == 0) {
            LOGGER.debug("No file " + transferType + " status information found: " + getDataSvm().getName() + " for job:" + getJobId());
        }
        return results;

    }
}
