/*
 * $Id$
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.internal.ontap.controller.OntapConnection;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.HasMcc;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.SvmOperationException;
import com.netapp.offtap3.utils.Pair;

import java.util.List;

public class HasMccImpl implements HasMcc {
    private boolean isMccSource;
    private OntapConnection connection;

    public HasMccImpl(OntapConnection connection) {
        this.connection = connection;
    }

    @Override
    public boolean isMccSource() {
        return isMccSource;
    }

    public void setMccSource(boolean isMccSource) {
        this.isMccSource = isMccSource;
    }

    public String getVServerStateWithMinimalTimeout(String vServerName) {
        Pair<String, String> pair = getVServerNameAndStateWithMinimalTimeout(vServerName);
        if (pair != null) {
            return pair.getB();
        } else {
            return null;
        }
    }

    public String getVServerNameWithMinimalTimeout(String vServerName) {
        Pair<String, String> pair = getVServerNameAndStateWithMinimalTimeout(vServerName);
        if (pair != null) {
            return pair.getA();
        } else {
            return null;
        }
    }

    public Pair<String, String> getVServerNameAndStateWithMinimalTimeout(String vServerName) {
        NaElement xi = new NaElement("vserver-get-iter");
        NaElement desiredAttributes = new NaElement("desired-attributes");
        NaElement desiredVServerInfo = new NaElement("vserver-info");
        desiredVServerInfo.addChildElem(new NaElement("operational-state"));
        xi.addChildElem(desiredAttributes);
        desiredAttributes.addChildElem(desiredVServerInfo);
        OntapConnection connection = new OntapConnection(this.connection);
        try {
            connection.setZapiTimeout(connection.getOfftapPreferences().getZapiInvocationMinimalTimeout());
            connection.setAllowRetry(false);
            NaElement xo = connection.invokeZapi(xi, ZapiCache.SHORT_CACHE);
            if (xo == null) {
                return null;
            }
            NaElement attributesList = xo.getChildByName("attributes-list");
            if (attributesList == null) {
                return null;
            }

            List<NaElement> vServerInfoList = attributesList.getChildren();
            if (vServerInfoList != null) {
                for (NaElement vServerInfo : vServerInfoList) {
                    String state = vServerInfo.getChildContent("operational-state");
                    String name = vServerInfo.getChildContent("vserver-name");
                    String withoutMc = vServerName.replaceAll("-mc", "");
                    String nameWithoutMc = name.replaceAll("-mc", "");
                    if (withoutMc.equals(nameWithoutMc)) {
                        return new Pair<String, String>(name, state);
                    }
                }
            }
            return null;
        } catch (ApiFailedException afe) {
            throw new SvmOperationException("Unable to getVServerNameAndStateWithMinimalTimeout on " + vServerName + " based on " + afe.getMessage(),
                                            afe);
        }
    }
}
