/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/LunOperationProcessImpl.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.DataTransferStatus;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

public class LunOperationProcessImpl extends DataTransferProcessImpl {
    private static final Logger LOGGER = Logger.getLogger(LunOperationProcessImpl.class);
    private final String transferType;

    public LunOperationProcessImpl(DataSvmImpl dataSvm, String jobId, TransferType transferType) {
        super(dataSvm, jobId);
        this.transferType = transferType.name();
    }

    @Override
    public List<DataTransferStatus> getStatus() {
        LOGGER.debug("Entered getStatus() for controller: " + getDataSvm().getName());
        NaElement input = new NaElement("lun-" + transferType + "-get-iter");
        NaElement query = new NaElement("query");
        NaElement jobDetails = new NaElement("lun-" + transferType + "-info");
        jobDetails.addNewChild("job-uuid", getJobId());
        query.addChildElem(jobDetails);
        input.addChildElem(query);
        List<DataTransferStatus> results = new ArrayList<DataTransferStatus>();
        try {
        	NaElement output = null;
        	if(transferType.equals(TransferType.move.toString()) || getDataSvm().isDirectSvm()){
        		output = getDataSvm().invokeZapi(input, ZapiCache.NO_CACHE);
        	} else {
        		output = getDataSvm().getParentControllerImpl().invokeZapi(input, ZapiCache.NO_CACHE);
        	}            
            if (output != null && output.getChildByName("attributes-list") != null) {
                List<NaElement> lun_move_infos = output.getChildByName("attributes-list").getChildren();
                for (NaElement currXML : lun_move_infos) {
                    String destinationPath =  currXML.getChildContent("destination-path");
                    String uuid = currXML.getChildContent("job-uuid");
                    String scannerStatus = currXML.getChildContent("job-status");
                    DataTransferStatusImpl lunMoveInfo = new DataTransferStatusImpl(destinationPath, uuid, scannerStatus);
                    if (scannerStatus.equalsIgnoreCase(ScannerStates.data.toString()) || scannerStatus.equalsIgnoreCase(ScannerStates.complete.toString())){
                        lunMoveInfo.setScannerProgressPercentage(Integer.parseInt(currXML.getChildContent("progress-percent")));
                        lunMoveInfo.setDestinationReady(Boolean.parseBoolean(currXML.getChildContent("is-destination-ready")));
                    }
                    if (scannerStatus.equalsIgnoreCase(ScannerStates.destroyed.toString()) || scannerStatus.equalsIgnoreCase(ScannerStates.destroying.toString())){
                        lunMoveInfo.setFailureReason(currXML.getChildContent("last-failure-reason"));
                    }
                    results.add(lunMoveInfo);
                }
            }
        } catch (ApiFailedException e) {
            LOGGER.error("ZAPI failure trying to get lun " + transferType + " status: " + getDataSvm().getName(), e);
        } catch (Exception e) {
            LOGGER.error("Unexpected error processing ZAPI result for lun-" + transferType + "-get-iter: " + getDataSvm().getName(), e);
        }
        if(results.size()==0){
            LOGGER.debug(
                    "No lun " + transferType + " status information found: " + getDataSvm().getName() + " for job:" +
                            getJobId());
        }
        return results;
    }
}