package com.netapp.offtap3.internal.ontap;

import java.io.IOException;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;

import org.apache.log4j.Logger;

import com.netapp.offtap3.Controller;
import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.factories.ControllerFactory;
import com.netapp.offtap3.internal.ontap.controller.OntapConnection;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.utils.StatsCollector;
import com.netapp.offtap3.ontap.AdminSvm;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.ClusterPeer;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.SvmOperationException;
import com.netapp.offtap3.utils.Pair;
import com.netapp.offtap3.utils.StringUtils;
import com.netapp.offtap3.utils.Utils;
import com.netapp.offtap3.utils.Utils.Callback;

public class MCCManager {
	private static Logger logger = Logger.getLogger(MCCManager.class);
	
    private ConcurrentHashMap<String, MCCSlapperThread> mccSlapperThreads = new ConcurrentHashMap();    
    private Map<String, Pair<String,String>> mccFailovers = new ConcurrentHashMap<String, Pair<String,String>>();
    private static MCCManager instance = null;
    private static ConcurrentHashMap<Thread, URLConnection> connectionsByThread = new ConcurrentHashMap<Thread, URLConnection>();
    private static CopyOnWriteArrayList inFlightMccFailovers = new CopyOnWriteArrayList<String>();
    private static ConcurrentHashMap<String, String> adminVsToKnownMccMode = new ConcurrentHashMap<String,String>();
    private List<MCCSwitchEventListener> mccSwitchEventListeners = new LinkedList<MCCSwitchEventListener>();
    
    private MCCManager() {
    }
    
    public static MCCManager getInstance() {
    	if (instance == null) {
    		instance = new MCCManager();
    	}
    	return instance;
    }
    
    public String getSupportabilityInformation() {
    	StringBuffer retVal = new StringBuffer();
    	retVal.append("<h3>MCC Information</h3>");
    	if (mccFailovers.size() > 0) {
    		retVal.append("<TABLE border=1><TR><TH>Controller key</TH><TH>Failover value</TH></TR>\n");
	    	for (String key : Utils.getSortedList(mccFailovers.keySet())) {
	    		retVal.append("<TR><TD>");
	    		Controller c = ControllerFactory.getInstance().getController(key);
	    		retVal.append(c.getId() + " [" + c.getName() + "]");
	    		retVal.append("</TD><TD>");
	    		Pair p = mccFailovers.get(key);
	    		retVal.append(p.getA() + ":" + p.getB());
	    		retVal.append("</TD></TR>\n");
	    	}
    		retVal.append("</TABLE><BR>\n");
    	}
    	if (mccSlapperThreads.size() > 0) {
    		retVal.append("<TABLE border=1><TR><TH>Controller key</TH><TH>Slapper information</TH></TR>\n");
    		for (String key : Utils.getSortedList(mccSlapperThreads.keySet())) {
    			MCCSlapperThread slapper = mccSlapperThreads.get(key);
    			retVal.append("<TR><TD>");
    			Controller c = ControllerFactory.getInstance().getController(key);
    			retVal.append(c.getId() + " [" + c.getName() + "]");
    			retVal.append("</TD><TD>");
    			retVal.append(slapper.getSupportabilityInformation());
    			retVal.append("</TD></TR>\n");
    		}
    		retVal.append("</TABLE><BR>\n");
    	}
    	retVal.append("<br><br>");
    	return retVal.toString();
    }

    public MCCSlapperThread getMCCSlapperThreadCanBeNull(ControllerConnection connection) {
		synchronized (this) {
			String mccKey = connection.getFilerIdOfTarget();
			if (mccKey != null) {
				Controller controller = ControllerFactory.getInstance().getController(mccKey);
				if (controller instanceof DataSvm) {
					if (!((DataSvm) controller).isMccSource()) {
						return null;
					}
				} else {
					if (controller instanceof AdminSvm) {
						AdminSvm adminVs = (AdminSvm) controller;
						if (!adminVs.isMccSource()) {
							return null;
						}
					} else {
						return null;
					}
				}

				MCCSlapperThread retVal = this.mccSlapperThreads.get(mccKey);
				if (retVal == null) {
					retVal = new MCCSlapperThread(mccKey, connection.getOfftapPreferences());
					retVal.start();
					this.mccSlapperThreads.put(mccKey, retVal);
				}
				return retVal;
			} else {
				return null;
			}
		}
	}
    
    public static void addConnectionForThread(URLConnection incoming) {
    	connectionsByThread.put(Thread.currentThread(), incoming);
    }
    
    public static URLConnection getConnectionForThread(Thread incoming) {
    	return connectionsByThread.get(incoming);
    }
    
    public static void removeConnectionForThread() {
    	connectionsByThread.remove(Thread.currentThread());
    }
    
    public void clearMCCFailover() {
		mccFailovers.clear();
	}
    
    public boolean isFailedOverSomewhere(String controllerId) {
    	Pair p = this.mccFailovers.get(controllerId);
    	if (p != null && p.getA() != null && !p.getA().equals(controllerId)) {
    		return true;
    	}
    	return false;
    }
    
    public String getFailedOverControllerKey(String controllerId) {
    	Pair p = this.mccFailovers.get(controllerId);
    	if (p != null) {
    		return (String) p.getA();
    	}
    	return null;
    }
	
	public boolean addMCCFailover(String controllerId, String newControllerId, String newvserverName) {
		// lets see if this is functionally different than what we had before
		boolean isDifferent = false;
		Pair knownValue = mccFailovers.get(controllerId);
		String oldName = null;
		if (knownValue != null) {
			isDifferent = !(knownValue.getA().equals(newControllerId) && Utils.areEqual(knownValue.getB(), newvserverName));
		} else {
			// ok - we don't have this guy in the failover map.  Lets see if this is any different than 
			// what we would have used
			Controller controller = ControllerFactory.getInstance().getController(controllerId);			
			if (controller instanceof DataSvm) {
				DataSvm dataVs = (DataSvm) controller;
				oldName = dataVs.getName();
			}
			if (!Utils.areEqual(oldName, newvserverName)) {
				isDifferent = true;
			}
			if (!Utils.areEqual(controllerId, newControllerId)) {
				// well - they are different.  But lets see if they actually resolve to different IP addresses
				isDifferent = true;
				Controller oldController = ControllerFactory.getInstance().getController(controllerId);
				if (oldController instanceof DataSvm) {
					DataSvm oldControllerDataVs = (DataSvm) oldController;
					if (oldControllerDataVs.getParentController() != null) {
						if (oldControllerDataVs.getParentController().getId().equals(newControllerId)) {
							isDifferent = false;
						}
					}
				}
			}
		}
		logger.debug("MCCManager.addMCCFailover - for " + controllerId + " redirecting to " + newControllerId + ":" + newvserverName + " was had isDifferent=" + isDifferent);
		mccFailovers.put(controllerId, new Pair(newControllerId, newvserverName));
		
		if (isDifferent) {
        	
        	// lets only notify for top level controllers
        	boolean topLevel = false;
        	Controller controller =  ControllerFactory.getInstance().getController(controllerId);
        	if (controller instanceof AdminSvm) {
        		topLevel = true;
        	}
        	if (controller instanceof DataSvm) {
        		DataSvm dataVs = (DataSvm) controller;
        		if (dataVs.isDirectSvm()) {
        			topLevel = true;
        		}
        	}
        	
        	logger.debug("MCCManager.addMCCFailover - for " + controllerId + " topLevel was " + topLevel);
        	
        	if (topLevel) {
	            MCCSwitchEvent event = new MCCSwitchEvent();
	            event.setNewControllerId(newControllerId);
	            event.setNewControllerName(newvserverName);
	            event.setOldControllerId(controllerId);
	            event.setOldControllerName(oldName);            
	            notifyListeners(event);
        	}
        }
		
		return isDifferent;
	}
	
	public String getVServerNameByUuid(DataSvm vserver) {
		String controllerId = vserver.getId();
		if (controllerId != null) {
			Pair p = mccFailovers.get(controllerId);
			if (p != null) {
				return (String) p.getB();
			}
		}
		DataSvm controller = (DataSvm) ControllerFactory.getInstance().getController(controllerId);
		if (controller != null) {
			return controller.getName();
		} else {
			return vserver.getName();
		}
	}
	
	public String getVServerNameFromObjectOrFallback(DataSvm mightBeNull, String possibleName) {
		String retVal = null;
		if (mightBeNull != null && StringUtils.isReal(mightBeNull.getId())) {
			retVal = getVServerNameByUuid(mightBeNull);
		}
		return possibleName;
	}
	
	public String getVserverNameMightBeFailedOver(String controllerId, String proposedName) {
		Pair p = mccFailovers.get(controllerId);
		String foundName = null;
		if (p != null) {
			// lets see if there's a -mc twiddle here
			foundName = (String) p.getB();
			String proposedNameWithoutMc = proposedName.replaceAll("-mc", "");
			String foundNameWithoutMc = foundName.replaceAll("-mc", "");
			if (proposedNameWithoutMc.equals(foundNameWithoutMc)) {
				return foundNameWithoutMc;
			}
		}
		return proposedName;
	}
	
	
	public OntapConnection possiblyReviseHelpfulServerForMCC(OntapConnection incoming) {
		String mccFailoverKey = incoming.getFilerIdOfTarget();
		if (mccFailoverKey != null) {
	    	// special case here.  If we are CURRENTLY checking this guy for MCC failover, meaning we suspect it might be busted because things were really slow, etc
	    	// lets give that a bit of time to process before we proceed and attempt the retry.  Otherwise we can hustle thru here and start another call to the 
	    	// MCC dead site before giving it time to see where it might really have landed.  Not fool-proof, but may help.
			int deadlockStopper = 0;
			while(inFlightMccFailovers.contains(mccFailoverKey)) {
				StatsCollector.sleep(1000, "in-flight-mcc-failovers");
				if (deadlockStopper > 30) {
					break;
				}
				deadlockStopper++;
			}

			Pair p = (Pair) mccFailovers.get(mccFailoverKey);
			if (p != null) {
				// uh-oh - someone failed this sucker over.
				String controllerIdOfTarget = (String) p.getA();
				String vserverNameOfTarget = (String) p.getB();
				Controller theController = ControllerFactory.getInstance().getController(controllerIdOfTarget);
				OntapConnection connection = new OntapConnection((OntapConnection) theController.getConnection());
				if (StringUtils.isReal(connection.getFilerIdOfTarget())) {
					connection.setFilerIdOfTarget(incoming.getFilerIdOfTarget());
				}
				// we want to set this vfiler UNLESS its a direct connect vserver, in which case we're only talking to a single filer
				// and whatever answers must be the right vserver name
				if (theController instanceof DataSvm) {
					DataSvm dataVs = (DataSvm) theController;
					if (!dataVs.isDirectSvm()) {
						connection.setContext(vserverNameOfTarget);
					}
				}
				if (theController instanceof AdminSvm && StringUtils.isReal(vserverNameOfTarget)) {
					connection.setContext(vserverNameOfTarget);
				}				
				StatsCollector.getInstance().addCounter("mcc-failover-zapi-redirection");
				logger.debug("MCCManager.possiblyReviseHelpfulServerForMCC - Looking for " + mccFailoverKey + " and found " + controllerIdOfTarget + ":" + vserverNameOfTarget);
				return connection;
			}
		}
		//logger.debug("MCCManager.possiblyReviseHelpfulServerForMCC - Looking for " + mccFailoverKey + " and nothing found");
		return incoming;
	}
	
	public void synchronouslyCheckForChangedVServerName(OntapConnection incoming) {
		MCCSlapperThread slapper = this.getMCCSlapperThreadCanBeNull(incoming);
		if (slapper != null) {
			slapper.synchronouslyExecuteCheck(false);
		}
	}
	
	Pair<String, String> getMCCTargetForClusterMinimalTimeout(AdminSvm sourceAdminVs, AdminSvm targetAdminVs) {
    	NaElement xi = new NaElement("metrocluster-get");
        try {
        	OntapConnection connection = new OntapConnection(targetAdminVs.getConnection());
        	if (connection.getContext() != null) {
        		connection.setContext(null);
        	}
        	int oldTimeout = connection.getZapiTimeout();
    		connection.setZapiTimeout(connection.getOfftapPreferences().getZapiInvocationMinimalTimeout());
    		connection.setAllowRetry(false);
    		NaElement xo = connection.invokeZapi(xi, ZapiCache.SHORT_CACHE);
            if (xo == null) {
            	return null;
            }
            NaElement attributesList = xo.getChildByName("attributes");
            if (attributesList == null) {
                return null;
            }

            List<NaElement> vServerInfoList = attributesList.getChildren();
            if (vServerInfoList != null) {
	            for (NaElement vServerInfo : vServerInfoList) {
	            	String state = vServerInfo.getChildContent("local-mode");
	            	if (StringUtils.isReal(state)) {
	            		adminVsToKnownMccMode.put(sourceAdminVs.getId(), state);
	            		adminVsToKnownMccMode.put(targetAdminVs.getId(), state);
	            	}
	            	if ("switchover".equals(state) || "waiting_for_switchback".equals(state)) {
	            		// ok - that means that a source cluster has failed over to here
	            		return new Pair(targetAdminVs.getId(), null);
	            	}
	            	if ("normal".equals(state)) {
	            		return new Pair(sourceAdminVs.getId(), null);
	            	}
	            }
            }
            return null;
        } catch (ApiFailedException e) {
        	throw new SvmOperationException("Could not get MCCTargetForClusterMinimalTimeout with minimal timeout for " + sourceAdminVs, e);
        }
    }
	
	public String getMccState(String controllerId) {
		return adminVsToKnownMccMode.get(controllerId);
	}
	    
	 /**
	     * Go see if a controller has failed over or failed back, and if so, see where the heck it went.
	     * @param controllerID
	     * @return
	     */
	    public Pair checkForMccFailover(String controllerID) throws IOException {
	    	try {
	    		inFlightMccFailovers.add(controllerID);	    	
		    	Controller controller = ControllerFactory.getInstance().getController(controllerID);
		    	logger.debug("MCCManager.checkForMccFailover for " + controllerID + " saw " + controller);
		    	if (controller instanceof DataSvm) {
		    		DataSvm other = (DataSvm) controller;
		    		if (!other.isMccSource()) {
		    			logger.debug("MCCManager.checkForMccFailover for " + controllerID + " not mcc source");
		    			// if it wasn't an MCC source, no need to even fool with this.
		    			return null;
		    		}
		    		if (other.getParentController() != null) {
		    			logger.debug("MCCManager.checkForMccFailover for " + controllerID + " had parent controller");
		    			AdminSvm parent = other.getParentController();
		    			for (ClusterPeer peer : parent.getClusterPeers()) {
		    				String farSide = peer.getUuid();
		    				AdminSvm farSideAdminVs = (AdminSvm) ControllerFactory.getInstance().getController(farSide);
		    				logger.debug("MCCManager.checkForMccFailover for " + controllerID + " farSideAdminVs was " + farSideAdminVs);
		    				if (farSideAdminVs != null) {
			    				String expectedVServerName = other.getName() + "-mc";
			    				try {
				    				String state = farSideAdminVs.getVServerStateWithMinimalTimeout(expectedVServerName);
				    				logger.debug("MCCManager.checkForMccFailover for " + controllerID + " for farSideAdminVs of " + farSideAdminVs + " see " + expectedVServerName + " and " + state);
				    				if ("running".equalsIgnoreCase(state)) {
				    					// we're in business
				    					return new Pair(farSide, expectedVServerName);
				    				}
			    				} catch (Throwable t) {
			    					t.printStackTrace();
			    					// no problem
			    				}
		    				}
		    			}
		        		// OK - Its not on one of the peers, lets see if its back where it is supposed to be
		    			try {
			    			Pair<String, String> nameAndState = parent.getVServerNameAndStateWithMinimalTimeout(controller.getName());
			    			if (nameAndState != null) {
			    				logger.debug("MCCManager.checkForMccFailover for " + controllerID + " looked at self and saw " + nameAndState.getA() + " and " + nameAndState.getB());
								if ("running".equalsIgnoreCase((String) nameAndState.getB())) {
									// we're in business
									return new Pair(parent.getId(), nameAndState.getA());
								}
			    			}
		    			} catch (Throwable t) {
		    				// no problem
		    				t.printStackTrace();
		    			}
		    		} else {
		    			// direct connect mcc vserver.  IP address will stay the same, but maybe the vserver name changed from -mc to not -mc or some such
		    			// so lets go ask the daggum thing what it is called these days and use that for the name
		    			return new Pair(controller.getId(), other.getVServerNameWithMinimalTimeout(controller.getName()));
		    		}
		    	} else if (controller instanceof AdminSvm) {	    		
		    		AdminSvm other = (AdminSvm) controller;
		    		for (ClusterPeer peer : other.getClusterPeers()) {
						String farSide = peer.getUuid();
						AdminSvm farSideAdminVs = (AdminSvm) ControllerFactory.getInstance().getController(farSide);
						logger.debug("MCCManager.checkForMccFailover for admin of " + other + " looked at " + farSideAdminVs + " checking");
						if (farSideAdminVs != null) {
							try {
								Pair p = getMCCTargetForClusterMinimalTimeout(other, farSideAdminVs);
								logger.debug("MCCManager.checkForMccFailover for admin of " + other + " returning " + p.getA() + " and " + p.getB());
								return p;
							} catch (Throwable t) {
								// no problem
							}
						}
		    		}
		    	}
		    	logger.debug("MCCManager.checkForMccFailover returning null");
		    	return null;
	    	} finally {
	    		inFlightMccFailovers.remove(controllerID);
	    	}
	    }
	    
	    public boolean isPreparedForMCCFailover(String controllerID) {
	    	DataSvm dataVs = (DataSvm) ControllerFactory.getInstance().getController(controllerID);
	    	boolean foundAny = false;
	    	boolean missedAny = false;
	    	AdminSvm adminVs = dataVs.getParentController();
	    	if (adminVs != null) {
	    		for (ClusterPeer currPeer : adminVs.getClusterPeers()) {
	    			if (ControllerFactory.getInstance().getController(currPeer.getUuid()) != null) {
	    				foundAny = true;
	    			} else {
	    				missedAny = true;
	    			}
	    		}    	
	    		if (missedAny) {
	    			return false;
	    		}
	    		return foundAny;
	    	} else {
	    		// so - a direct connect vserver is ALWAYS ready for MCC failover, since the data 
	    		// lif just fails over automatically.  The only hassle is that the vserver name will
	    		// change, which we need to account for.
	    		return true;
	    	}
	    	
	    }
	    
	    public List<MCCSwitchEventListener> getMccSwitchEventListeners() {
	        return mccSwitchEventListeners;
	    }

	    public void addMccSwitchEventListener(MCCSwitchEventListener mccSwitchEventListener) {
	        this.mccSwitchEventListeners.add(mccSwitchEventListener);
	    }

	    public void removeMccSwitchEventListener(MCCSwitchEventListener mccSwitchEventListener) {
	        this.mccSwitchEventListeners.remove(mccSwitchEventListener);
	    }

	    public void clearMccSwitchEventListeners() {
	        this.mccSwitchEventListeners.clear();
	    }

	    private void notifyListeners(final MCCSwitchEvent event) {
	    	List callbacks = new ArrayList();
	        for (MCCSwitchEventListener listener : getMccSwitchEventListeners()) {
	        	final MCCSwitchEventListener myListener = listener;
	        	Callback callback = new Callback() {
	        		public Object call() {
	        			myListener.notifySwitch(event);
	        			return null;
	        		}
	        	};        
	        	callbacks.add(callback);
	        }
	        Utils.executeCallbacksInParallelWithTimeoutEatExceptions(callbacks, 1);
	    }
}
