/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/OntapVolumeMaximumsImpl.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.Size;
import com.netapp.offtap3.ontap.Aggregate;

/**
 * Holds a set of volume maximums depending on the type of aggregate. All values are in bytes for consistency.
 */
public class OntapVolumeMaximumsImpl implements com.netapp.offtap3.ontap.OntapVolumeMaximums {
    private Size max32BitDenseVolumeSize;
    private Size max32BitDenseSharedVolumeSize;
    private Size max64BitDenseVolumeSize;
    private Size max64BitDenseSharedVolumeSize;

    public OntapVolumeMaximumsImpl() {
        max32BitDenseVolumeSize = Size.createB(0);
        max32BitDenseSharedVolumeSize = Size.createB(0);
        max64BitDenseVolumeSize = Size.createB(0);
        max64BitDenseSharedVolumeSize = Size.createB(0);
    }

    public OntapVolumeMaximumsImpl(OntapVolumeMaximumsImpl volumeMaximums) {
        max32BitDenseVolumeSize = volumeMaximums.max32BitDenseVolumeSize;
        max32BitDenseSharedVolumeSize = volumeMaximums.max32BitDenseSharedVolumeSize;
        max64BitDenseVolumeSize = volumeMaximums.max64BitDenseVolumeSize;
        max64BitDenseSharedVolumeSize = volumeMaximums.max64BitDenseSharedVolumeSize;
    }

    @Override
    public Size getMax32BitDenseVolumeSize() {
        return max32BitDenseVolumeSize;
    }

    public void setMax32BitDenseVolumeSize(Size max32BitDenseVolumeSize) {
        this.max32BitDenseVolumeSize = max32BitDenseVolumeSize;
    }

    @Override
    public Size getMax32BitDenseSharedVolumeSize() {
        return max32BitDenseSharedVolumeSize;
    }

    public void setMax32BitDenseSharedVolumeSize(Size max32BitDenseSharedVolumeSize) {
        this.max32BitDenseSharedVolumeSize = max32BitDenseSharedVolumeSize;
    }

    @Override
    public Size getMax64BitDenseVolumeSize() {
        return max64BitDenseVolumeSize;
    }

    public void setMax64BitDenseVolumeSize(Size max64BitDenseVolumeSize) {
        this.max64BitDenseVolumeSize = max64BitDenseVolumeSize;
    }

    @Override
    public Size getMax64BitDenseSharedVolumeSize() {
        return max64BitDenseSharedVolumeSize;
    }

    public void setMax64BitDenseSharedVolumeSize(Size max64BitDenseSharedVolumeSize) {
        this.max64BitDenseSharedVolumeSize = max64BitDenseSharedVolumeSize;
    }

    /**
     * Returns the maximum dense volume size based on the type of aggregate.
     * @param aggregate The aggregate for which the maximum size is needed.
     * @return The maximum size in bytes
     */
    @Override
    public Size getMaxDenseVolumeSize(Aggregate aggregate) {
        if (BIT32AGGR.equals(aggregate.getBlockType())) {
            return max32BitDenseVolumeSize;
        } else {
            return max64BitDenseVolumeSize;
        }
    }

    /**
     * Returns the maximum dense shared volume size based on the type of aggregate.
     * @param aggregate The aggregate for which the maximum size is needed.
     * @return The maximum size in bytes
     */
    @Override
    public Size getMaxDenseSharedVolumeSize(Aggregate aggregate) {
        if (BIT32AGGR.equals(aggregate.getBlockType())) {
            return max32BitDenseSharedVolumeSize;
        } else {
            return max64BitDenseSharedVolumeSize;
        }
    }

    /**
     * Returns the maximum dense volume size for a 64 bit aggregate if possible or 32 bit if 64 bit maximum was not found.
     * @return The maximum size in bytes
     */
    @Override
    public Size getMaxDenseVolumeSize() {
        if (max64BitDenseVolumeSize != null) {
            return max64BitDenseVolumeSize;
        } else {
            return max32BitDenseVolumeSize;
        }
    }

    /**
     * Returns the maximum dense shared volume size for a 64 bit aggregate if possible or 32 bit if 64 bit maximum was not found.
     * @return The maximum size in bytes
     */
    @Override
    public Size getMaxDenseSharedVolumeSize() {
        if (max64BitDenseSharedVolumeSize != null) {
            return max64BitDenseSharedVolumeSize;
        } else {
            return max32BitDenseSharedVolumeSize;
        }
    }
}
