/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/SevenModeCloneSpecImpl.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.File;
import com.netapp.offtap3.ontap.SevenModeCloneSpec;

import java.util.concurrent.LinkedBlockingQueue;

public class SevenModeCloneSpecImpl implements SevenModeCloneSpec {
    private File destinationFile;
    private boolean fallBackToCopy;
    private int fallBackThreshold;
    private boolean createSnapshot;
    private LinkedBlockingQueue<Integer> statusQueue;

    private SevenModeCloneSpecImpl(Builder builder) {
        destinationFile = builder.destinationFile;
        fallBackToCopy = builder.fallBackToCopy;
        fallBackThreshold = builder.fallBackThreshold;
        createSnapshot = builder.createSnapshot;
        statusQueue = builder.statusQueue;
    }

    @Override
    public boolean fallBackToCopy() {
        return fallBackToCopy;
    }

    @Override
    public int getFallBackThreshold() {
        return fallBackThreshold;
    }

    @Override
    public boolean createSnapshot() {
        return createSnapshot;
    }

    @Override
    public LinkedBlockingQueue<Integer> getStatusQueue() {
        return statusQueue;
    }

    @Override
    public File getDestination() {
        return destinationFile;
    }

    public static class Builder implements SevenModeCloneSpec.Builder {
        private File destinationFile;
        private boolean fallBackToCopy = true;
        private int fallBackThreshold;
        private boolean createSnapshot;
        private LinkedBlockingQueue<Integer> statusQueue;

        public Builder(File destinationFile) {
            this.destinationFile = destinationFile;
        }

        @Override
        public SevenModeCloneSpec.Builder fallBackToCopy(boolean fallBack) {
            this.fallBackToCopy = fallBack;
            return this;
        }

        @Override
        public SevenModeCloneSpec.Builder fallBackThreshold(int threshold) {
            this.fallBackThreshold = threshold;
            return this;
        }

        @Override
        public SevenModeCloneSpec.Builder createSnapshot(boolean createSnapshot) {
            this.createSnapshot = createSnapshot;
            return this;
        }

        @Override
        public SevenModeCloneSpec.Builder statusQueue(LinkedBlockingQueue<Integer> statusQueue) {
            this.statusQueue = statusQueue;
            return this;
        }

        @Override
        public SevenModeCloneSpec build() {
            return new SevenModeCloneSpecImpl(this);
        }
    }
}
