package com.netapp.offtap3.internal.ontap;

import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.Messages;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.utils.StatsCollector;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.ControllerWithSnmpSupport;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.Set;

public class SnmpHostManager  {

    private static final Logger logger = Logger.getLogger(SnmpHostManager.class);
    private final ControllerWithSnmpSupport controller;

    public SnmpHostManager(ControllerWithSnmpSupport controller) {
        this.controller = controller;
    }

    public void addSnmpTrapListener(String hostIp) throws DataLoaderException {
        try {
            Set<String> trapHosts = controller.getSnmpHosts();
            if (trapHosts.contains(hostIp)) {
                return;
            }
            if (trapHosts.size() >= 8) {
                // horrible Ontap bug that causes infinite panic reboot cycle - see 717821 for details.
                String message = controller + " already had 8 traphosts, so not adding " + hostIp;
                StatsCollector.getInstance().addCounter("error-" + message);
                logger.error(message);
                return;
            }
            NaElement xi = new NaElement("snmp-traphost-add");
            xi.addNewChild("host", hostIp);
            controller.invokeZapi(xi, ZapiCache.NO_CACHE);
            ZapiCache.getInstance().blastCacheValues("snmp-");
            logger.debug(hostIp + " is now added as traphost for " + controller);
        } catch (ApiFailedException ex) {
            if (ex.toString().contains("uplicate")) {
                return;
            }
            String msg = Messages.getInstance().getMessage("snmp.host.add.fail", controller, hostIp, ex.getMessage());
            throw new DataLoaderException(msg, ex);
        }
    }

    public void removeSnmpTrapListener(String hostIp) throws DataLoaderException {
        try {
            if (!controller.getSnmpHosts().contains(hostIp)) {
                return;
            }
            NaElement xi = new NaElement("snmp-traphost-delete");
            xi.addNewChild("host", hostIp);
            controller.invokeZapi(xi, ZapiCache.NO_CACHE);
            ZapiCache.getInstance().blastCacheValues("snmp-");
            logger.debug(hostIp + " has now been deleted as traphost from " + controller);
        } catch (ApiFailedException ex) {
            String msg = Messages.getInstance().getMessage("snmp.host.delete.fail", controller, hostIp, ex.getMessage());
            throw new DataLoaderException(msg, ex);
        }
    }
}
