/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/controller/AbstractOntapController.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.controller;

import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.DiscoveryError;
import com.netapp.offtap3.FcInterface;
import com.netapp.offtap3.IpInterface;
import com.netapp.offtap3.Version;
import com.netapp.offtap3.internal.AbstractController;
import com.netapp.offtap3.internal.ontap.EmsLoggerImpl;
import com.netapp.offtap3.internal.ontap.EmsLoggerSpecImpl;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.Aggregate;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.EmsLogger;
import com.netapp.offtap3.ontap.EmsLoggerSpec;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.OntapController;
import com.netapp.offtap3.ontap.OntapVersion;
import com.netapp.offtap3.ontap.OntapVolumeMaximums;
import com.netapp.offtap3.ontap.SvmOperationException;
import com.netapp.offtap3.ontap.ZapiAccessPermissions;
import com.netapp.offtap3.ontap.ZapiExecutionError;
import com.netapp.offtap3.utils.Pair;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;
import org.w3c.dom.Document;

import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.CopyOnWriteArrayList;

public abstract class AbstractOntapController<D extends OntapController> extends AbstractController<D> implements OntapController,
        InternalZapiHandler {

    private Map<String, ? extends Aggregate> aggregates = new SafeConcurrentHashMap<String, Aggregate>();
    private Map<String, ? extends IpInterface> interfaces = new SafeConcurrentHashMap<String, IpInterface>();
    private List<FcInterface> fcInterfaces = new CopyOnWriteArrayList<FcInterface>();
    private boolean mccSource = false;
    private ZapiAccessPermissions zapiAccessPermissions = null;
    private OntapVolumeMaximums volumeMaximums;

    public AbstractOntapController(OntapConnection connection, Version version) {
        super(new OntapConnection(connection));
        setVersion(version);
    }

    @Override
    public OntapConnection getConnection() {
        return (OntapConnection) super.getConnection();
    }

    @Override
    public OntapVersion getVersion() {
        return (OntapVersion) super.getVersion();
    }

    @Override
    public void setConnection(ControllerConnection controllerConnection) {
        super.setInternalConnection(new OntapConnection(controllerConnection));
    }

    @Override
    public EmsLoggerSpec.Builder getLoggerSpecBuilder(String application, String version) {
        EmsLoggerSpec.Builder builder = new EmsLoggerSpecImpl.BuilderImpl();
        return builder.eventSource(application).version(version);
    }

    @Override
    public EmsLogger getLogger(EmsLoggerSpec spec) {
        // this is the happy path / simple case.  see subclasses for overrides
        return new EmsLoggerImpl(getConnection(), spec);
    }

    public ZapiCache getZapiCache() {
        // todo aeonflux-next - consider injection here and whether we need to return the cache, or just expose blastCacheValues
        // what if we just had a separate instance per controller?
        return ZapiCache.getInstance();
    }

    @Override
    public ZapiAccessPermissions getZapiAccessPermissions() {
        return zapiAccessPermissions;
    }

    public void setZapiAccessPermissions(ZapiAccessPermissions zapiAccessPermissions) {
        this.zapiAccessPermissions = zapiAccessPermissions;
    }

    @Override
    public NaElement invokeZapi(NaElement xmlInput, long cacheDuration) throws ApiFailedException {
        return getConnection().invokeZapi(xmlInput, cacheDuration);
    }

    @Override
    public Document invokeZapi(Document input, long cacheDuration) throws ApiFailedException {
        return getConnection().invokeZapi(input, cacheDuration);
    }

    @Override
    public Map<String, ? extends Aggregate> getAggregates() {
        return aggregates;
    }

    public void setAggregates(Map<String, ? extends Aggregate> aggregates) {
        this.aggregates = aggregates;
    }

    @Override
    public Map<String, ? extends IpInterface> getIpInterfaces() {
        return interfaces;
    }

    public void setIpInterfaces(Map<String, ? extends IpInterface> interfaces) {
        this.interfaces = interfaces;
    }

    @Override
    public List<FcInterface> getFcInterfaces() {
        return fcInterfaces;
    }

    public void setFcInterfaces(List<FcInterface> fcInterfaces) {
        this.fcInterfaces = fcInterfaces;
    }
    public List<DiscoveryError> getDiscoveryErrors() {
        return getConnection().getDiscoveryErrors();
    }

    public List<ZapiExecutionError> getZapiExecutionErrors() {
        return getConnection().getZapiExecutionErrors();
    }

    public void addDiscoveryError(String message, Throwable ex) {
        getConnection().addDiscoveryError(message, ex);
    }

    public void addZapiExecutionError(String message, NaElement input, Throwable e) {
        getConnection().addZapiExecutionError(message, input, e);
    }

	public boolean isMccSource() {
		return mccSource;
	}

	public void setMccSource(boolean mccSource) {
		this.mccSource = mccSource;
	}

    @Override
    public OntapVolumeMaximums getVolumeMaximums() {
        return volumeMaximums;
    }

    public void setVolumeMaximums(OntapVolumeMaximums volumeMaximums) {
        this.volumeMaximums = volumeMaximums;
    }
	
	public String getVServerStateWithMinimalTimeout(String vServerName) {
    	Pair<String, String> pair = getVServerNameAndStateWithMinimalTimeout(vServerName);
    	if (pair != null) {
    		return pair.getB();
    	} else {
    		return null;
    	}
    }
    
    public String getVServerNameWithMinimalTimeout(String vServerName) {
    	Pair<String, String> pair = getVServerNameAndStateWithMinimalTimeout(vServerName);
    	if (pair != null) {
    		return pair.getA();
    	} else {
    		return null;
    	}
    }
    
    public Pair<String, String> getVServerNameAndStateWithMinimalTimeout(String vServerName) {
    	NaElement xi = new NaElement("vserver-get-iter");    	
        NaElement desiredAttributes = new NaElement("desired-attributes");
        NaElement desiredVServerInfo = new NaElement("vserver-info");
        desiredVServerInfo.addChildElem(new NaElement("operational-state"));
        xi.addChildElem(desiredAttributes);
        // dsarisky 12/8/14 - this is a hack, but this is not working on sizes > 20 vservers.
        // I will fix it next week after innovation challenge to be a more proper fix.
        xi.addNewChild("max-records", "2000");        
        desiredAttributes.addChildElem(desiredVServerInfo);
        OntapConnection connection = new OntapConnection(this.getConnection());
        try {        	
    		connection.setZapiTimeout(connection.getOfftapPreferences().getZapiInvocationMinimalTimeout());
    		connection.setAllowRetry(false);        	
            NaElement xo = connection.invokeZapi(xi, ZapiCache.SHORT_CACHE);
            if (xo == null) {
            	return null;
            }
            NaElement attributesList = xo.getChildByName("attributes-list");
            if (attributesList == null) {
                return null;
            }

            List<NaElement> vServerInfoList = attributesList.getChildren();
            if (vServerInfoList != null) {
	            for (NaElement vServerInfo : vServerInfoList) {
	            	String state = vServerInfo.getChildContent("operational-state");
	            	String name = vServerInfo.getChildContent("vserver-name");
	            	String withoutMc = vServerName.replaceAll("-mc", "");
	            	String nameWithoutMc = name.replaceAll("-mc", "");
	            	if (withoutMc.equals(nameWithoutMc)) {
	            		return new Pair<String, String>(name, state);
	            	}
	            }
            }
            return null;
        } catch (ApiFailedException afe) {
        	throw new SvmOperationException("Unable to getVServerNameAndStateWithMinimalTimeout on " + vServerName + " based on " + afe.getMessage(),
                    afe);
        }
    }

}
