/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/controller/AbstractOntapDataController.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.controller;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;

import org.apache.log4j.Logger;

import com.netapp.offtap3.CifsShare;
import com.netapp.offtap3.IGroupSpec;
import com.netapp.offtap3.IscsiAdapter;
import com.netapp.offtap3.IscsiConnection;
import com.netapp.offtap3.Lun;
import com.netapp.offtap3.Size;
import com.netapp.offtap3.StorageProtocol;
import com.netapp.offtap3.Version;
import com.netapp.offtap3.internal.HasProxy;
import com.netapp.offtap3.internal.ontap.OntapIGroupSpecImpl.BuilderImpl;
import com.netapp.offtap3.internal.ontap.intf.OntapIGroupImpl;
import com.netapp.offtap3.internal.ontap.lun.OntapLunSpecImpl;
import com.netapp.offtap3.internal.ontap.snap.ConsistencyGroupImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.volume.OntapVolumeSpecImpl;
import com.netapp.offtap3.ontap.Aggregate;
import com.netapp.offtap3.ontap.ConsistencyGroup;
import com.netapp.offtap3.ontap.Export;
import com.netapp.offtap3.ontap.IGroupOperationException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.OntapDataController;
import com.netapp.offtap3.ontap.OntapIGroup;
import com.netapp.offtap3.ontap.OntapIGroupSpec;
import com.netapp.offtap3.ontap.OntapIGroupSpec.Builder;
import com.netapp.offtap3.ontap.OntapLun;
import com.netapp.offtap3.ontap.OntapLunSpec;
import com.netapp.offtap3.ontap.OntapVolume;
import com.netapp.offtap3.ontap.OntapVolumeSpec;
import com.netapp.offtap3.ontap.SnapMirrorRelation;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

public abstract class AbstractOntapDataController<I extends OntapDataController> extends
        AbstractOntapController<I> implements OntapDataController, HasProxy<I> {

    private static final Logger LOGGER = Logger.getLogger(AbstractOntapDataController.class);

    private String rootVolume = "";

    protected Map<String, OntapLun> luns = new SafeConcurrentHashMap<>();
    private Map<String, ? extends OntapVolume> volumes = new SafeConcurrentHashMap<>();
    private List<OntapIGroup> igroups = new CopyOnWriteArrayList<>();
    private List<IscsiAdapter> iscsiAdapters = new CopyOnWriteArrayList<>();
    private List<IscsiConnection> iscsiConnections = new CopyOnWriteArrayList<>();

    private List<CifsShare> cifsShares = new CopyOnWriteArrayList<>();

    private List<? extends SnapMirrorRelation> snapMirrorRelations = new CopyOnWriteArrayList<>();
    private boolean vStorageApiEnabled;

    public AbstractOntapDataController(OntapConnection connInfo, Version version) {
        super(connInfo, version);
    }

    @Override
    public Map<String, ? extends OntapVolume> getVolumes() {
        return volumes;
    }

    @Override
    public OntapVolume getVolume(String volumeName) {
        return volumes.get(volumeName);
    }

    public void setVolumes(Map<String, ? extends OntapVolume> volumes) {
        this.volumes = volumes;
    }

    @Override
    public List<CifsShare> getCifsShares() {
        return cifsShares;
    }

    public void setCifsShares(List<CifsShare> cifsShares) {
        this.cifsShares = cifsShares;
    }

    @Override
    public boolean isVStorageApiEnabled() {
        return vStorageApiEnabled;
    }

    public void setVStorageApiEnabled(boolean vStorageApiEnabled) {
        this.vStorageApiEnabled = vStorageApiEnabled;
    }

    @Override
    public void modifyVStorageApiEnabled(boolean enabled) {
        if (!isVStorageApiSupported()) {
            throw new UnsupportedOperationException("Error: The vStorage API is not supported on controller " + getName());
        }
    }

    @Override
    public Map<String, Export> getExports() {
        Map<String, Export> exports = new SafeConcurrentHashMap<>();
        for (OntapVolume vol : volumes.values()) {
            exports.putAll(vol.getExports());
        }
        return exports;
    }

    @Override
    public String getRootVolume() {
        return rootVolume;
    }

    public void setRootVolume(String rootVolume) {
        this.rootVolume = rootVolume;
    }

    @Override
    public Export getExport(String exportedPath) {
        return getExports().get(exportedPath);
    }

    @Override
    public Map<String, OntapLun> getLuns() {
        return luns;
    }

    public void setLuns(Map<String, OntapLun> luns) {
        this.luns = luns;
    }

    @Override
    public OntapLun getLun(String path) {
        return luns.get(path);
    }

    @Override
    public List<OntapIGroup> getIGroups() {
        return igroups;
    }

    @Override
    public OntapIGroup getIGroup(String name) {
        for (OntapIGroup group : getIGroups()) {
            if (group.getName().equals(name)) {
                return group;
            }
        }
        return null;
    }

    @Override
    public Set<Lun> getLunsMappedToInitiator(String initiator) {
        Set<Lun> mappedLuns = new HashSet<>();
        for (OntapLun lun : getProxy().getLuns().values()) {
            for (OntapIGroup igroup : lun.getIgroups()) {
                if (igroup.getInitiatorIds().contains(initiator)) {
                    mappedLuns.add(lun);
                }
            }
        }
        return mappedLuns;
    }

    public void setIGroups(List<OntapIGroup> igroups) {
        this.igroups = igroups;
    }

    @Override
    public List<IscsiAdapter> getIscsiAdapters() {
        return iscsiAdapters;
    }

    public void setIscsiAdapters(List<IscsiAdapter> iscsiAdapters) {
        this.iscsiAdapters = iscsiAdapters;
    }

    @Override
    public List<IscsiConnection> getIscsiConnections() {
        return iscsiConnections;
    }

    public void setIscsiConnections(List<IscsiConnection> iscsiConnections) {
        this.iscsiConnections = iscsiConnections;
    }

    @Override
    public List<? extends SnapMirrorRelation> getSnapMirrorRelations() {
        return snapMirrorRelations;
    }

    public void setSnapMirrorRelations(List<? extends SnapMirrorRelation> snapMirrorRelations) {
        this.snapMirrorRelations = snapMirrorRelations;
    }

    @Override
    public OntapVolumeSpec.Builder getVolumeSpecBuilder(String name, Aggregate aggregate, Size size) {
        return new OntapVolumeSpecImpl.Builder(name, aggregate, size);
    }

    @Override
    public OntapLunSpec.Builder getLunSpecBuilder(String path, OntapVolume volume, String type, Size size) {
        return new OntapLunSpecImpl.BuilderImpl(path, volume, type, size);
    }

    @Override
    public ConsistencyGroup createConsistencyGroup(String snapshotName, List<OntapVolume> volumes) {
        return new ConsistencyGroupImpl(this, volumes, snapshotName);
    }

    @Override
    public Builder getIGroupSpecBuilder(String name, String osType) {
        return new BuilderImpl(name, osType);
    }

    public OntapIGroup createIGroup(IGroupSpec iGroupSpec) {
        OntapIGroupSpec ontapIGroupSpec = (OntapIGroupSpec) iGroupSpec;
        String igroupName = ontapIGroupSpec.getName();
        StorageProtocol storageProtocol = ontapIGroupSpec.getStorageProtocol();
        LOGGER.debug(this + " createIgroup: " + igroupName + " " + storageProtocol.toString() + " " + ontapIGroupSpec.getOsType() + " alua: " +
                             ontapIGroupSpec.getWantsAlua());

        String newName = igroupName;

        if (!ontapIGroupSpec.getKeepName()) {
            if (ontapIGroupSpec.getWantsAlua()) {
                newName = igroupName + "_alua";
            }

            int iNum = 0;
            while (getIGroup(newName) != null) {
                iNum++;
                newName = String.format("%s%04d", igroupName, iNum);
            }
            igroupName = newName;
        }

        LOGGER.info(this + " about to create igroup " + igroupName);
        NaElement xi;
        xi = new NaElement("igroup-create");
        xi.addNewChild("initiator-group-name", igroupName);
        xi.addNewChild("os-type", ontapIGroupSpec.getOsType());
        if (storageProtocol == StorageProtocol.ISCSI) {
            xi.addNewChild("initiator-group-type", "iscsi");
        } else if (storageProtocol == StorageProtocol.MIXED) {
            xi.addNewChild("initiator-group-type", "mixed");
        } else {
            xi.addNewChild("initiator-group-type", "fcp");
        }
        try {
            invokeZapi(xi, ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("igroup");

            // todo aeonflux-next - is this the right way to create an igroup?
            // we should probably go back to the filer and load
            @SuppressWarnings("unchecked")
            OntapIGroupImpl igroup = new OntapIGroupImpl(this);
            igroup.setName(igroupName);
            igroup.setOsType(ontapIGroupSpec.getOsType());
            igroup.setProtocol(storageProtocol.name().toLowerCase());

            // runs another ZAPI to set ALUA
            igroup.setAlua(ontapIGroupSpec.getWantsAlua());
            // now igroup is successfully created, add it to the collection
            igroups.add(igroup);
            return igroup;
        } catch (Exception e) {
            LOGGER.error("Failed to create igroup " + igroupName + " type " + ontapIGroupSpec.getOsType() + ".", e);
            throw new IGroupOperationException("Failed to create igroup " + igroupName + " type " + ontapIGroupSpec.getOsType() + "." + e.getMessage(), e);
        }

    }
}
