package com.netapp.offtap3.internal.ontap.controller;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;

import org.apache.log4j.Logger;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;
import com.netapp.offtap3.CifsShare;
import com.netapp.offtap3.CifsShareOperationException;
import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.FileOperationException;
import com.netapp.offtap3.IpInterface;
import com.netapp.offtap3.IscsiConnection;
import com.netapp.offtap3.IscsiSession;
import com.netapp.offtap3.LogOperationException;
import com.netapp.offtap3.StorageProtocol;
import com.netapp.offtap3.StorageSystem;
import com.netapp.offtap3.UnixGroup;
import com.netapp.offtap3.UnixUser;
import com.netapp.offtap3.UserAndGroupOperationException;
import com.netapp.offtap3.VolumeOperationException;
import com.netapp.offtap3.internal.CachingControllerProxy;
import com.netapp.offtap3.internal.CifsShareImpl;
import com.netapp.offtap3.internal.HasParentControllerImpl;
import com.netapp.offtap3.internal.IscsiSessionImpl;
import com.netapp.offtap3.internal.UnixGroupImpl;
import com.netapp.offtap3.internal.UnixUserImpl;
import com.netapp.offtap3.internal.ontap.DataTransferProcessImpl;
import com.netapp.offtap3.internal.ontap.DataTransferProcessImpl.TransferType;
import com.netapp.offtap3.internal.ontap.FileOperationProcessImpl;
import com.netapp.offtap3.internal.ontap.LunOperationProcessImpl;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.internal.ontap.OntapVersionImpl;
import com.netapp.offtap3.internal.ontap.dataloader.CModePerformanceObjectsLoaderDelegate;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataParser;
import com.netapp.offtap3.internal.ontap.dataloader.OntapDataCache;
import com.netapp.offtap3.internal.ontap.dataloader.SimpleCdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.lun.CModeLunImpl;
import com.netapp.offtap3.internal.ontap.proxy.DataSvmProxy;
import com.netapp.offtap3.internal.ontap.snap.CModeConsistencyGroupImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.utils.StatsCollector;
import com.netapp.offtap3.ontap.AdminSvm;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.CModeAggregate;
import com.netapp.offtap3.ontap.CModeFile;
import com.netapp.offtap3.ontap.CModeIpInterface;
import com.netapp.offtap3.ontap.CModeLun;
import com.netapp.offtap3.ontap.CModeSnapMirrorRelation;
import com.netapp.offtap3.ontap.CModeVolume;
import com.netapp.offtap3.ontap.ConsistencyGroup;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.ontap.DataTransferProcess;
import com.netapp.offtap3.ontap.EmsLogger;
import com.netapp.offtap3.ontap.EmsLoggerSpec;
import com.netapp.offtap3.ontap.ExportPolicy;
import com.netapp.offtap3.ontap.LunBinding;
import com.netapp.offtap3.ontap.LunOperationException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.NfsDetails;
import com.netapp.offtap3.ontap.OntapIGroup;
import com.netapp.offtap3.ontap.OntapLunSpec;
import com.netapp.offtap3.ontap.OntapVolume;
import com.netapp.offtap3.ontap.OntapVolumeSpec;
import com.netapp.offtap3.ontap.PortSet;
import com.netapp.offtap3.ontap.QosOperationException;
import com.netapp.offtap3.ontap.QosPolicyGroup;
import com.netapp.offtap3.ontap.QosThroughput;
import com.netapp.offtap3.ontap.SnapMirrorOperationException;
import com.netapp.offtap3.ontap.SnapMirrorRelation;
import com.netapp.offtap3.ontap.SpaceReserve;
import com.netapp.offtap3.ontap.SvmOperationException;
import com.netapp.offtap3.ontap.SvmPeer;
import com.netapp.offtap3.utils.ExpiringHashMap;
import com.netapp.offtap3.utils.Pair;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;
import com.netapp.offtap3.utils.StringUtils;
import com.netapp.offtap3.utils.Utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;


public class DataSvmImpl extends AbstractOntapDataController<DataSvm> implements DataSvm, HasParentControllerImpl {
    private static final Logger LOGGER = Logger.getLogger(DataSvmImpl.class);
    private static final Set<StorageSystem> STORAGE_SYSTEMS = Collections.singleton(StorageSystem.datasvm);
    private final AdminSvmImpl adminSvm;
    private final OntapDataCache dataCache;
    private Map<String, QosPolicyGroup> qosPolicyGroups = new SafeConcurrentHashMap<String, QosPolicyGroup>();
    private Map<String, ExportPolicy> exportPolicies = new SafeConcurrentHashMap<String, ExportPolicy>();
    private List<PortSet> portSets = new CopyOnWriteArrayList<PortSet>();
    private Map<String, UnixGroup> unixGroups = new SafeConcurrentHashMap<String, UnixGroup>();
    private Map<String, UnixUser> unixUsers = new SafeConcurrentHashMap<String, UnixUser>();
    private Map<String, CModeAggregate> aggregateList = new SafeConcurrentHashMap<String, CModeAggregate>();
    private Map<String, DataTransferProcess> dataTransfers = new ExpiringHashMap<String, DataTransferProcess>(1000*60*60l);

    private DataSvm proxy;
    private String iScsiNodeName = "";
    private State state;
    private String rootVolumeAggregate = "";
    private String snapShotPolicy = "";
    private String qosPolicyGroupName = null;
    private boolean repository = false;
    private CModePerformanceObjectsLoaderDelegate performanceObjectsLoader;
    private NfsDetails nfsDetails;
    private List<LunBinding> lunBindings;
    private List<SvmPeer> svmPeers;

    // direct svm
    public DataSvmImpl(OntapConnection connInfo, OntapVersionImpl version) {
        super(connInfo, version);
        performanceObjectsLoader = new CModePerformanceObjectsLoaderDelegate(this);
        dataCache = new OntapDataCache(this);
        adminSvm = null;
    }

    // pass-through svm
    public DataSvmImpl(OntapConnection connInfo, AdminSvmImpl adminSvm, OntapDataCache dataCache) {
        super(connInfo, adminSvm.getVersion());
        this.adminSvm = adminSvm;
        performanceObjectsLoader = new CModePerformanceObjectsLoaderDelegate(this);
        this.dataCache = dataCache;
    }

    @Override
    public EmsLogger getLogger(EmsLoggerSpec spec) {
        if (getVersion().atLeastSN()) {
            return super.getLogger(spec);
        }
        // not SN, do we know the cluster admin?
        else if (!isDirectSvm()) {
            return adminSvm.getLogger(spec);
        }
        // not SN, and we are direct connect - can't be done!
        else {
            throw new LogOperationException(this + " does not have EMS ZAPI support");
        }
    }

    @Override
    public NfsDetails getNfsDetails() {
        return nfsDetails;
    }

    public void setNfsDetails(NfsDetails nfsDetails) {
        this.nfsDetails = nfsDetails;
    }

    @Override
    public String getIscsiNodeName() {
        return iScsiNodeName;
    }

    public void setIscsiNodeName(String iScsiNodeName) {
        this.iScsiNodeName = iScsiNodeName;
    }

    @Override
    public List<SvmPeer> getSvmPeers() {
        return svmPeers;
    }

    public void setSvmPeers(List<SvmPeer> svmPeers) {
        this.svmPeers = svmPeers;
    }

    @Override
    public String getRootVolumeAggregate() {
        return rootVolumeAggregate;
    }

    public void setRootVolumeAggregate(String rootVolumeAggregate) {
        this.rootVolumeAggregate = rootVolumeAggregate;
    }

    @Override
    public boolean isRepository() {
        return repository;
    }

    public void setRepository(boolean repository) {
        this.repository = repository;
    }

    @Override
    public String getSnapShotPolicy() {
        return snapShotPolicy;
    }

    public void setSnapShotPolicy(String snapShotPolicy) {
        this.snapShotPolicy = snapShotPolicy;
    }

    @Override
    public QosPolicyGroup getQosPolicyGroup() {
    	QosPolicyGroup group = null;
    	if (qosPolicyGroupName != null && StringUtils.isReal(qosPolicyGroupName)) {
	    	group = getQosPolicyGroup(qosPolicyGroupName);
	        if (group == null) {
	            // Maybe the policy groups on the controller were out of date. Let's refresh just in case.
	            refresh(ControllerResource.qos);
	            group = getQosPolicyGroup(qosPolicyGroupName);
	        }
    	}
        return group;
    }

    @Override
    public Map<String, QosPolicyGroup> getQosPolicyGroups() {
        return qosPolicyGroups;
    }

    public void setQosPolicyGroups(Map<String, QosPolicyGroup> groups) {
        qosPolicyGroups = groups;
    }

    @Override
    public Map<String, ExportPolicy> getExportPolicies() {
        return exportPolicies;
    }

    public void setExportPolicies(Map<String, ExportPolicy> exportPolicies) {
        this.exportPolicies = exportPolicies;
    }

    @Override
    public QosPolicyGroup getQosPolicyGroup(String name) {
        return qosPolicyGroups.get(name);
    }

    public void setQosPolicyGroupName(String qosPolicyGroup) {
        qosPolicyGroupName = qosPolicyGroup;
    }

    @Override
    public List<LunBinding> getLunBindings() {
        return lunBindings;
    }

    public void setLunBindings(List<LunBinding> lunBindings) {
        this.lunBindings = lunBindings;
    }

    @Override
    public Map<String, UnixGroup> getUnixGroups() {
        return unixGroups;
    }

    public void setUnixGroups(Map<String, UnixGroup> unixGroups) {
        this.unixGroups = unixGroups;
    }

    @Override
    public Map<String, UnixUser> getUnixUsers() {
        return unixUsers;
    }

    @Override
    public boolean supportsQos() {
        return getVersion().atLeastSN() && !isDirectSvm();
    }

    public void setUnixUsers(Map<String, UnixUser> unixUsers) {
        this.unixUsers = unixUsers;
    }

    @Override
    public Map<String, CModeIpInterface> getIpInterfaces() {
        return (Map<String, CModeIpInterface>) super.getIpInterfaces();
    }

    @Override
    public UnixGroup createUnixGroup(String name, int id) throws UserAndGroupOperationException {
        LOGGER.debug("createUnixGroup: " + name);

        NaElement xi = new NaElement("name-mapping-unix-group-create");
        xi.addNewChild("group-name", name);
        xi.addNewChild("group-id", String.valueOf(id));
        try {
            NaElement xo = invokeZapi(xi, ZapiCache.NO_CACHE);
            UnixGroupImpl newGroup = new UnixGroupImpl(this, name, id);
            unixGroups.put(name, newGroup);
            LOGGER.info("Created Unix Group " + name + ":" + xo.getAttr("status") + ".");
            return newGroup;
        } catch (ApiFailedException e) {
            String message = "Unable to create unix group " + name + " on data svm " + getName() + ".";
            LOGGER.error("createUnixGroup: " + message);
            throw new UserAndGroupOperationException(message, e);
        }
    }
    
    @Override
    public DataTransferProcess getDataTransfer(String jobId, boolean isLun, boolean isMove) {
    	DataTransferProcess retVal = dataTransfers.get(jobId);
    	if (retVal == null) {
    		if (isLun) {
    			if (isMove) {
    				retVal = new LunOperationProcessImpl(this, jobId, TransferType.move);
    			} else {
    				retVal = new LunOperationProcessImpl(this, jobId, TransferType.copy);
    			}
    		} else {
    			if (isMove) {
    				retVal = new FileOperationProcessImpl(this, jobId, TransferType.move);
    			} else {
    				retVal = new FileOperationProcessImpl(this, jobId, TransferType.copy);
    			}
    		}    		
    	}
    	return retVal;
    }


    @Override
    public UnixUser createUnixUser(String name, int id) throws UserAndGroupOperationException {
        LOGGER.debug("createUnixUser: " + name);

        // No negative user ids allowed in unix.
        if (id < 0) {
            throw new UserAndGroupOperationException("The user id " + id + " must be a positive integer.");
        }

        // Check if a group with the specified name already exists.
        if (unixGroups.containsKey(name)) {
            throw new UserAndGroupOperationException("The specified user name " + name +
                                                             " already exists as a group. To add this user to that group, it must be specified.");
        }

        // Find a valid group id.
        int groupId = findNextGroupId(id);

        // Create private group for the user.
        UnixGroup group = createUnixGroup(name, groupId);

        return createUnixUser(name, id, group);
    }

    private int findNextGroupId(int id) {
        int groupId = id;
        // First put the groups in a list sorted by id to make searching easier.
        List<UnixGroup> sortedGroups = new ArrayList<UnixGroup>(unixGroups.values());
        Collections.sort(sortedGroups, new Comparator<UnixGroup>() {
            @Override
            public int compare(UnixGroup o1, UnixGroup o2) {
                return o1.getId() - o2.getId();
            }
        });
        for (UnixGroup group : sortedGroups) {
            if (group.getId() == groupId) {
                groupId++;
            }
        }
        return groupId;
    }

    @Override
    public UnixUser createUnixUser(String name, int id, UnixGroup group) throws UserAndGroupOperationException {
        LOGGER.debug("createUnixUser: " + name + ", group = " + group.getName());
        UnixGroupImpl groupImpl = (UnixGroupImpl) group;
        NaElement input = new NaElement("name-mapping-unix-user-create");
        input.addNewChild("user-name", name);
        input.addNewChild("user-id", String.valueOf(id));
        input.addNewChild("group-id", String.valueOf(group.getId()));
        input.addNewChild("return-record", "true");
        try {
            NaElement xo = invokeZapi(input, ZapiCache.NO_CACHE);
            LOGGER.info("Created " + name + ":" + xo.getAttr("status") + ".");
            UnixUserImpl newUser = new UnixUserImpl(groupImpl, name, id);
            groupImpl.addUser(newUser);
            unixUsers.put(name, newUser);
            return newUser;
        } catch (ApiFailedException e) {
            String message = "Unable to create unix user " + name + " on data svm " + getName() + ".";
            LOGGER.error("createUser: " + message);
            throw new UserAndGroupOperationException(message, e);
        }
    }

    @Override
    public ExportPolicy createExportPolicy(String policyName) {
        LOGGER.debug("createExportPolicy: " + policyName);

        NaElement input = new NaElement("export-policy-create");
        input.addNewChild("policy-name", policyName);
        try {
            NaElement output = invokeZapi(input, ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("export");
            LOGGER.info("Created export policy " + policyName + ":" + output.getAttr("status") + ".");
            getProxy().refresh(ControllerResource.export_policies);
            return getProxy().getExportPolicies().get(policyName);
        } catch (Exception ex) {
            LOGGER.error("Failed to create export policy " + policyName, ex);
            throw new SvmOperationException(
                    "Failed to create export policy " + policyName + ".  Error: " + ex.getMessage(), ex);
        }
    }

    @Override
    public DataTransferProcess copyFilesOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs, DataSvm destinationController) throws FileOperationException {
        return copyFilesOnDemand(srcDestPairs, destinationController, null);
    }

    @Override
    public DataTransferProcess copyFilesOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs, DataSvm destinationController,
                                                 CModeFile referenceFile) throws FileOperationException {
        LOGGER.debug("copyFilesOnDemand: begin");
        return transferFileDataOnDemand(srcDestPairs, referenceFile, destinationController, DataTransferProcessImpl.TransferType.copy);
    }

    @Override
    public DataTransferProcess moveFilesOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs, DataSvm destinationController) throws FileOperationException {
        return moveFilesOnDemand(srcDestPairs, destinationController, null);
    }

    @Override
    public DataTransferProcess moveFilesOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs,DataSvm destinationController, 
                                                 CModeFile referenceFile) throws FileOperationException {
        LOGGER.debug("moveFilesOnDemand: begin");
        return transferFileDataOnDemand(srcDestPairs, referenceFile, destinationController, DataTransferProcessImpl.TransferType.move);
    }

    private DataTransferProcess transferFileDataOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs,
                                                         CModeFile referenceFile, DataSvm destinationController, DataTransferProcessImpl.TransferType transferTypeEnum) {
        String transferType = transferTypeEnum.name();
        NaElement input, sourcePaths, destinationPaths;
        input = new NaElement("file-" + transferType + "-start");
        sourcePaths = new NaElement("source-paths");
        destinationPaths = new NaElement("destination-paths");

        validateFilesForOnDemand(srcDestPairs, destinationController, referenceFile);
        for (Pair<CModeFile, CModeFile> srcDestPair : srcDestPairs) {
            String srcPath = getCopyOnDemandPath(srcDestPair.getA(),MCCManager.getInstance().getVServerNameByUuid(this));
            String destPath = getCopyOnDemandPath(srcDestPair.getB(),MCCManager.getInstance().getVServerNameByUuid(destinationController));
            sourcePaths.addNewChild("sfod-operation-path", srcPath);
            destinationPaths.addNewChild("sfod-operation-path", destPath);
            LOGGER.debug("Going to " + transferType + " src file=" + srcPath + " to dest=" + destPath);
        }

        input.addChildElem(sourcePaths);
        input.addChildElem(destinationPaths);
        if (referenceFile != null) {
            input.addNewChild("reference-path", getCopyOnDemandPath(referenceFile,MCCManager.getInstance().getVServerNameByUuid(this)));
        }

        try {
        	NaElement output;
        	if(transferTypeEnum == TransferType.move || isDirectSvm()){
        		output = invokeZapi(input, ZapiCache.NO_CACHE);
        	} else {
        		output = getParentControllerImpl().invokeZapi(input, ZapiCache.NO_CACHE);
        	}            
            LOGGER.debug("Invoked Zapi file-" + transferType + "-start");
            DataTransferProcess retVal = new FileOperationProcessImpl(this, output.getChildContent("job-uuid"), transferTypeEnum);
            dataTransfers.put(retVal.getJobId(), retVal);
            return retVal;

        } catch (Exception e) {
            String message = "Unable to " + transferType + " on demand files with source and destination: " + srcDestPairs;
            LOGGER.error(message);
            throw new FileOperationException(message, e);
        }
    }

    private String getCopyOnDemandPath(CModeFile file, String svmName) {    	
        return svmName+":"+file.getVolume().getName() + "/" + file.getVolumeRelativePath();
    }

    /**
     * Performs the following validation checks on the files:
     * <ul>
     * <li>The source files are from the same volume</li>
     * <li>The destination files are from the same volume</li>
     * <li>The volumes are on the same controller</li>
     * <li>That the referenceFile is one of the source files</li>
     * </ul>
     */
    private void validateFilesForOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs, DataSvm destinationController, CModeFile referenceFile) {
        CModeVolume srcVolume = null, destVolume = null;
        // If no reference file, act like we already found it to short circuit checking.
        boolean referenceFileFound = referenceFile == null;
        for (Pair<CModeFile, CModeFile> srcDestPair : srcDestPairs) {
            if (!referenceFileFound && srcDestPair.getA().equals(referenceFile)) {
                referenceFileFound = true;
            }

            if (srcVolume == null) {
                srcVolume = srcDestPair.getA().getVolume();
            } else if (!srcDestPair.getA().getVolume().equals(srcVolume)) {
                // All source files must be on the same volume and we found one that isn't on the same volume as another.
                throw new FileOperationException(
                        "Error: All source files for the copy/move operation must be on the same volume.");
            }

            if (destVolume == null) {
                destVolume = srcDestPair.getB().getVolume();
            } else if (!srcDestPair.getB().getVolume().equals(destVolume)) {
                // All destination files must be on the same volume and we found one that isn't on the same volume as another.
                throw new FileOperationException(
                        "Error: All destination files for the copy/move operation must be on the same volume.");
            }

            if (!getVolumes().containsValue(srcVolume) || !destinationController.getVolumes().containsValue(destVolume)) {
                throw new FileOperationException(
                        "Error: The source and destination volumes must exist.");
            }
        }
        if (!referenceFileFound) {
            throw new FileOperationException("Error: The specified reference file " + referenceFile.getFullPath() +
                                                     " must be included as a source file.");
        }
    }

    @Override
    public DataTransferProcess copyLunsOnDemand(
            List<Pair<CModeLun, String>> srcDestPairs, DataSvm destinationController) throws LunOperationException {
        return copyLunsOnDemand(srcDestPairs, null, destinationController);
    }

    @Override
    public DataTransferProcess copyLunsOnDemand(
            List<Pair<CModeLun, String>> srcDestPairs, CModeLun referenceLun, DataSvm destinationController) throws LunOperationException {
        return transferLunDataOnDemand(srcDestPairs, referenceLun, destinationController,
                                       DataTransferProcessImpl.TransferType.copy);
    }

    @Override
    public DataTransferProcess moveLunsOnDemand(
            List<Pair<CModeLun, String>> srcDestPairs, DataSvm destinationController) throws LunOperationException {
        return moveLunsOnDemand(srcDestPairs, null, destinationController);
    }

    @Override
    public DataTransferProcess moveLunsOnDemand(
            List<Pair<CModeLun, String>> srcDestPairs, CModeLun referenceLun, DataSvm destinationController) throws LunOperationException {
        LOGGER.debug("Enter lunMoveStart");
        return transferLunDataOnDemand(srcDestPairs, referenceLun, destinationController,
                                DataTransferProcessImpl.TransferType.move);
    }

    private DataTransferProcess transferLunDataOnDemand(List<Pair<CModeLun, String>> srcDestPairs, CModeLun referenceLun, DataSvm destinationController, DataTransferProcessImpl.TransferType transferTypeEnum) throws LunOperationException {
        String transferType = transferTypeEnum.name();
        NaElement xi,xa,xb;
        xi = new NaElement("lun-" + transferType + "-start");
        xa = new NaElement("paths");

        validateLunsForOnDemand(srcDestPairs, referenceLun, destinationController);

        for (Pair<CModeLun, String> srcDestPair : srcDestPairs) {
            String srcSfodPath = srcDestPair.getA().getPath();
            String destSfodPath = srcDestPair.getB();
            xb = new NaElement("lun-path-pair");
            xb.addNewChild("destination-path", destSfodPath);
            xb.addNewChild("source-path", srcSfodPath);
            xa.addChildElem(xb);
            LOGGER.debug("Going to SFCod src lun="+getName()+":"+srcSfodPath + " to destination="+destinationController.getName()+":"+destSfodPath);
        }
        xi.addChildElem(xa);        

        if (referenceLun != null) {
            xi.addNewChild("reference-lun", referenceLun.getPath());
        }

        try {
			if (transferTypeEnum == TransferType.copy) {
				xi.addNewChild("destination-vserver", MCCManager.getInstance().getVServerNameByUuid(destinationController));
				xi.addNewChild("source-vserver", MCCManager.getInstance().getVServerNameByUuid(this));
			}
			NaElement output;
			if (transferTypeEnum == TransferType.move || isDirectSvm()) {
				output = invokeZapi(xi, ZapiCache.NO_CACHE);
			} else {
				output = getParentControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
			}
            LOGGER.debug("Invoking Zapi lun-" + transferType + "-start");
            return new LunOperationProcessImpl(this, output.getChildContent("job-uuid"), transferTypeEnum);
        } catch (Exception e) {
            String message = "Unable to " + transferType + " on demand luns with source and destination: " + srcDestPairs;
            LOGGER.error(message);
            throw new LunOperationException(message, e);
        }
    }

    /**
     * Performs the following validation checks on the luns:
     * <ul>
     * <li>The source files are from the same volume</li>
     * <li>The destination files are from the same volume</li>
     * <li>The destination controller is a data svm in this cluster</li>
     * <li>That the referenceLun is one of the source luns</li>
     * </ul>
     */
    private void validateLunsForOnDemand(List<Pair<CModeLun, String>> srcDestPairs, CModeLun referenceLun, DataSvm destinationController) {
        String srcVolumeName = null, destVolumeName = null;
        // If no reference file, act like we already found it to short circuit checking.
        boolean referenceLunFound = referenceLun == null;
        for (Pair<CModeLun, String> srcDestPair : srcDestPairs) {
            if (!referenceLunFound && srcDestPair.getA().equals(referenceLun)) {
                referenceLunFound = true;
            }

            if (srcVolumeName == null) {
                srcVolumeName = srcDestPair.getA().getVolumeName();
            } else if (!srcDestPair.getA().getVolumeName().equals(srcVolumeName)) {
                // All source Luns must be on the same volume and we found one that isn't on the same volume as another.
                throw new LunOperationException(
                        "Error: All source luns for the copy/move operation must be on the same volume.");
            }

            if (destVolumeName == null) {
                destVolumeName = getVolumeNameFromLunPath(srcDestPair.getB());
            } else if (!getVolumeNameFromLunPath(srcDestPair.getB()).equals(destVolumeName)) {
                // All destination files must be on the same volume and we found one that isn't on the same volume as another.
                throw new LunOperationException(
                        "Error: All destination files for the copy/move operation must be on the same volume.");
            }

            if (!getParentController().equals(destinationController.getParentController())) {
                throw new LunOperationException(
                        "Error: The source and destination Storage Virual Machines must be in the same cluster.");
            }
        }
        if (!referenceLunFound) {
            throw new LunOperationException("Error: The specified reference file " + referenceLun.getPath() +
                                                     " must be included as a source file.");
        }
    }

    private String getVolumeNameFromLunPath(String lunPath) throws LunOperationException {
        String lunPathPrefix = "/vol/";
        if (!lunPath.startsWith(lunPathPrefix)) {
            throw new LunOperationException("Error: The lun path " + lunPath + " is not valid.");
        }
        String volumeName = lunPath.substring(lunPathPrefix.length());
        volumeName = volumeName.substring(0, volumeName.indexOf("/"));
        return volumeName;
    }


    @Override
    public QosPolicyGroup createQosPolicyGroup(String inputName, QosThroughput maxThroughput) {
        String name = sanitizePolicyName(inputName);

        // we can't create QoS on a direct svm today, but maybe we will in FS.x or LB...
        if (isDirectSvm()) {
            throw new UnsupportedOperationException(this + " can't create QoS policy groups on " + getVersion());
        }

        // we can't create QoS on anything prior to SN
        if (!getVersion().atLeastSN()) {
            throw new UnsupportedOperationException(
                    this + " does not support QoS policies, as version is " + getVersion() + " and must be 8.2");
        }

        LOGGER.debug("Enter createQosPolicy");
        NaElement xi = new NaElement("qos-policy-group-create");
        xi.addNewChild("policy-group", name);
        xi.addNewChild("max-throughput", maxThroughput.toONTAPResolution().toString());
        xi.addNewChild("vserver", MCCManager.getInstance().getVServerNameByUuid(this));

        try {
            // run this against the admin
            NaElement xo = getParentControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
            LOGGER.info("Create QoS policfy '" + name + "' completed with status: " + xo.getAttr("status"));
            getProxy().refresh(ControllerResource.qos);
            return getProxy().getQosPolicyGroup(name);
        } catch (ApiFailedException e) {
            LOGGER.error(this + " failed to create QoS policy named " + name + " with max throughput: " + maxThroughput,
                         e);
            throw new QosOperationException(
                    this + " failed to create QoS policy named " + name + " with max throughput: " + maxThroughput, e);
        }
    }


    /**
     * Insures that the given policy name adheres to the ONTAP rules for policy name.
     * Here's the ONTAP errror message:<pre>
     * Policy group names must be unique and are restricted to
     * 127 alphanumeric characters, "-", and "_", and must start with an
     * alphanumeric character (a-z, A-Z, or 0-9) or "-".
     * </pre>
     *
     * @param policyName the requested policy name
     * @return an acceptable policy name.
     */
    private String sanitizePolicyName(String policyName) {
        policyName = StringUtils.replaceNonAlphanumerics(policyName, "_");
        if (policyName.startsWith("_")) {
            policyName = policyName.replace("_", "N_");
        }
        if (policyName.length() > 127) {
            policyName = policyName.substring(0, 127);
        }
        return policyName;
    }

    @Override
    public AdminSvm getParentController() {
        if (adminSvm == null) {
            return null;
        } else {
            return adminSvm.getProxy();
        }
    }

    @Override
    public AdminSvmImpl getParentControllerImpl() {
        // backdoor for accessing the parent, without using a proxy
        return adminSvm;
    }

    @Override
    public boolean isDirectSvm() {
        return adminSvm == null;
    }

    @Override
    public void assignIscsiNodeName(String iscsiName) {

        if (!getProxy().supportsProtocol(StorageProtocol.ISCSI)) {
            throw new UnsupportedOperationException("assignIscsiNodeName should only be called for iSCSI enabled svms");
        }

        NaElement xi = new NaElement("iscsi-node-set-name");
        xi.addNewChild("node-name", iscsiName);
        String message = "Setting iscsi node name to be " + iscsiName;

        try {
            disableISCSI();
            invokeZapi(xi, ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("iscsi-");
            LOGGER.info(message);
            getProxy().refresh(ControllerResource.protocols);
        } catch (ApiFailedException ex) {
            if (ex.toString().contains("Specified target name is same as current target name")) {
                return;
            }
            LOGGER.error("Failed to " + message, ex);
            throw new SvmOperationException(message + " Error: " + ex.getMessage(), ex);
        } finally {
            enableISCSI();
        }
    }

    @Override
    public State getState() {
        return state;
    }

    public void setState(State state) {
        this.state = state;
    }

    @Override
    public List<PortSet> getPortSets() {
        return portSets;
    }

    public void setPortSets(List<PortSet> portSets) {
        this.portSets = portSets;
    }


    @Override
    @SuppressWarnings("unchecked")
    public Map<String, CModeAggregate> getAggregates() {
        if (isDirectSvm()) {
            return aggregateList;
        } else if (Utils.isReal(aggregateList)) {
            Map<String, CModeAggregate> aggrMap = new HashMap(super.getAggregates());
            aggrMap.keySet().retainAll(aggregateList.keySet());
            return aggrMap;
        } else {
            return (Map<String, CModeAggregate>) super.getAggregates();
        }
    }

    public void setAggregateList(Map<String, CModeAggregate> aggregateList) {
        this.aggregateList = aggregateList;
    }

    public Map<String, CModeAggregate> getAggregateList() {
        return aggregateList;
    }

    @Override
    public boolean enableNdmp() {
        if (isDirectSvm()) {
            throw new UnsupportedOperationException("NDMP service can not be enabled on a direct SVM connection");
        }
        NaElement xi = new NaElement("system-cli");
        xi.addNewChild("priv", "admin");
        NaElement xa = new NaElement("args");

        xa.addNewChild("arg", "vserver");
        xa.addNewChild("arg", "services");
        xa.addNewChild("arg", "ndmp");
        xa.addNewChild("arg", "on");
        xa.addNewChild("arg", "-vserver");
        xa.addNewChild("arg", MCCManager.getInstance().getVServerNameByUuid(this));
        xi.addChildElem(xa);

        boolean success = true;
        try {
            NaElement xo = getParentControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
            int rc = xo.getChildIntValue("cli-result-value", 0);
            String output = xo.getChildContent("cli-output");
            if (rc != 1) {
                LOGGER.error(String.format("Unable to enable ndmp on vserver. rc = %s. output = %s", rc, output));
                success = false;
            }
        } catch (ApiFailedException e) {
            LOGGER.error("Unable to enable ndmp on vserver. - " + e.getMessage(), e);
        }
        return success;
    }

    @Override
    public void setName(String name) {
        super.setName(name);
        if (!isDirectSvm()) {
            getConnection().setContext(name);
        }
    }

    @Override
    public void setConnection(ControllerConnection controllerConnection) {
        super.setConnection(controllerConnection);
        if (!isDirectSvm()) {
            getConnection().setContext(getName());
        }
    }

    @Override
    public String toString() {
        if (isDirectSvm()) {
            return super.toString() + " [direct connect]";
        }
        return super.toString();
    }

    @Override
    public List<String> getPerformanceObjectInstanceList(String objectName) {
        return performanceObjectsLoader.getPerformanceObjectInstanceList(objectName);
    }

    @Override
    public Map<String, Long> getPerformanceCounters(String counter, String objectName, List<String> objectInstances) {
        return performanceObjectsLoader.getPerformanceCounters(counter, objectName, objectInstances);
    }

    @Override
    public ConsistencyGroup createConsistencyGroup(String snapshotName, List<OntapVolume> volumes) {
        return new CModeConsistencyGroupImpl(this, volumes, snapshotName);
    }

    @Override
    public CifsShare createCifsShare(String shareName, String sharePath) {
        NaElement xi = new NaElement("cifs-share-create");
        xi.addNewChild("share-name", shareName);
        xi.addNewChild("path", sharePath);
        try {
            invokeZapi(xi, ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("cifs-");

            CifsShareImpl share = new CifsShareImpl(this);
            share.setName(shareName);
            share.setMountPoint(sharePath);
            return share;
        } catch (ApiFailedException e) {
            LOGGER.error(this + " failed to create CIFS share named: " + shareName + " at path: " + sharePath, e);
            throw new CifsShareOperationException(
                    this + " failed to create CIFS share named: " + shareName + " at path: " + sharePath, e);
        }
    }

    @Override
    public CModeVolume getVolume(String name) {
        return (CModeVolume) super.getVolume(name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public Map<String, CModeVolume> getVolumes() {
        return (Map<String, CModeVolume>) super.getVolumes();
    }

    @Override
    public CModeVolume createVolume(OntapVolumeSpec volSpec) {
        LOGGER.debug(this + " createVolume: " + volSpec);

        String volumeName = volSpec.getVolumeName();

        NaElement xi = new NaElement("volume-create");
        xi.addNewChild("volume", volumeName);
        xi.addNewChild("containing-aggr-name", volSpec.getAggregateName());
        xi.addNewChild("size", volSpec.getSize().toMB() + "m");
        xi.addNewChild("space-reserve", volSpec.getSpaceReserve().name());
        xi.addNewChild("volume-type", volSpec.getVolumeType().name());

        if (volSpec.getSnapshotPolicy() != null) {
            xi.addNewChild("snapshot-policy", volSpec.getSnapshotPolicy());
        }

        if (volSpec.getQosPolicyGroupName() != null) {
            xi.addNewChild("qos-policy-group-name", volSpec.getQosPolicyGroupName());
        }

        if (volSpec.getVmAlignSector() > 0 && volSpec.getVmAlignSuffix() != null) {
            xi.addNewChild("vm-align-sector", Integer.toString(volSpec.getVmAlignSector()));
            xi.addNewChild("vm-align-suffix", volSpec.getVmAlignSuffix());
        }

        if (volSpec.getSecurityStyle() != null) {
            xi.addNewChild("volume-security-style", volSpec.getSecurityStyle().name());
        }

        if (volSpec.getSecurityPermissions() != null) {
            xi.addNewChild("unix-permissions", volSpec.getSecurityPermissions());
        }

        if (volSpec.getUserId() != -1) {
            xi.addNewChild("user-id", String.valueOf(volSpec.getUserId()));
        }

        if (volSpec.getGroupId() != -1) {
            xi.addNewChild("group-id", String.valueOf(volSpec.getGroupId()));
        }

        if (volSpec.getJunctionPath() != null) {
            xi.addNewChild("junction-path", volSpec.getJunctionPath());
        }

        if (volSpec.getExportPolicy() != null) {
            xi.addNewChild("export-policy", volSpec.getExportPolicy());
        }

        try {
            NaElement xo = invokeZapi(xi, ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("volume-");
            LOGGER.info("Create " + volumeName + ":" + xo.getAttr("status") + ".");
        } catch (ApiFailedException e) {
            addZapiExecutionError("Failed creating volume: " + volumeName, xi, e);
            LOGGER.error("createVolume: " + this + " Unable to create volume.", e);
            throw new VolumeOperationException(this + " failed to createVolume " + volumeName, e);
        }

        checkForLSRootVol();
        getProxy().refresh(ControllerResource.volumes);
        return getVolume(volumeName);
    }

    public boolean supportsDirectLsMirrorUpdate() {
        return !isDirectSvm() || (getVersion().atLeastFS() ||
                getVersion().compareTo(new OntapVersionImpl("ls mirror test", 8, 2, 3)) >= 0);
    }

    /**
     * If the root volume is in a LS mirror, then force the mirror to update so that this new volume
     * will be ready to export/mount.
     * This method should <b>NOT</b> be called with a direct-connect data vserver (an Exception will be thrown).
     * <p/>
     * This method does not throw any checked Exceptions, because the idea is that this is a "best effort"
     * approach, and any failure to detect or update the mirror will be reported, but normal execution will be
     * allowed to continue.
     *
     * @return true if either the root vol is not the source of an LS mirror <b>OR</b> it was successfully updated.
     */
    public boolean checkForLSRootVol() {
        boolean hasLSRootVol = false;
        if (!supportsDirectLsMirrorUpdate()) {
            // unfortunately, it is impossible to perform the check if this is a direct-connect vserver
            LOGGER.debug(
                    "not checking for LS root volume, since " + getName() + " is a direct-connect vserver running " +
                            getVersion() + " that does not support the necessary ZAPI.");
            return false;
        }

        try {
            DataSvm svm = getProxy();
            CModeVolume rootVol = svm.getVolume(svm.getRootVolume());
            for (SnapMirrorRelation relation : rootVol.getSnapMirrorRelations()) {
                if (((CModeSnapMirrorRelation) relation).getType().equals(CModeSnapMirrorRelation.Type.load_sharing)) {
                    hasLSRootVol = true;
                    LOGGER.info("Root volume " + getRootVolume() + " is in an LS mirror.  Updating relationship...");
                    rootVol.updateLSMirror(getOfftapPreferences().getRootVolMirrorUpdateTimeout());
                    return true;
                }
            }
            LOGGER.info("No LS mirror found for root volume: " + getRootVolume());
        } catch (SnapMirrorOperationException smoe) {
            if (hasLSRootVol) {
                LOGGER.error(
                        "Failed to update LS mirror relationship for the root volume of vserver: " + getName() + ". ");
            } else {
                LOGGER.error(
                        "Failed to retrieve SnapMirror relationships for the root volume of vserver: " + getName() +
                                ". ");
            }
        }
        return false;
    }

    @Override
    public CModeLun createLun(OntapLunSpec lunSpec) {
        LOGGER.debug("createLun (C-mode): " + lunSpec);
        if (lunSpec.getVolume() == null) {
            throw new IllegalArgumentException("No volume specified in spec: " + lunSpec);
        }

        String fullPath = "/vol/" + lunSpec.getVolume().getName() + "/" + lunSpec.getLunName();

        int num = 0;
        while ((getLun(fullPath) != null)) {
            num++;
            String newLunName = String.format("%s%02d", lunSpec.getLunName(), num);
            fullPath = "/vol/" + lunSpec.getVolume().getName() + "/" + newLunName;
        }

        NaElement input = new NaElement("lun-create-by-size");
        input.addNewChild("ostype", lunSpec.getType());
        input.addNewChild("path", fullPath);
        input.addNewChild("size", String.valueOf(lunSpec.getSize().toBytes()));
        if (lunSpec.getSpaceReserve() == SpaceReserve.none) {
            input.addNewChild("space-reservation-enabled", "false");
        } else {
            input.addNewChild("space-reservation-enabled", "true");
        }
        //SCSI thin provisioning, default is false
        if(lunSpec.getSpaceAllocation()){
        	input.addNewChild("space-allocation-enabled", "true");
        }
        if (StringUtils.isReal(lunSpec.getLunClass()) && getVersion().atLeastSN()) {
            input.addNewChild("class", lunSpec.getLunClass());

        }

        if (lunSpec.getPrefixSize() != null) {
            input.addNewChild("prefix-size", lunSpec.getPrefixSize().toString());
        }

        try {
            invokeZapi(input, ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("lun-", "volume-", "aggr-");
            LOGGER.info("Create lun:" + fullPath + ".");
        } catch (ApiFailedException e) {
            LOGGER.error("Failed to create lun from file " + fullPath + ".", e);
            throw new LunOperationException("Failed to createLun" + fullPath + ".  Error: " + e.getMessage(), e);
        }

        // lun created, make a dummy lun with just the path and call refresh. This will cause the dataloader to load
        // only the information for that lun when we refresh it.
        CModeLunImpl lunImpl = new CModeLunImpl(this);
        lunImpl.setPath(fullPath);
        CModeLun lun = lunImpl.refresh();        
        
        if(lun == null){        	 
        	 LOGGER.error("Create lun: Failed to refresh LUN after create " + fullPath );
        	 StatsCollector.getInstance().addCounter("aeonflux-lun-create-refresh-fail");   
        	 Utils.dumpStackTrace("Create lun: Failed to refresh LUN after create");
        	 
        	 //retry once
        	 StatsCollector.sleep(1000, "aeonflux-lun-create-refresh-fail");        	 
        	 lun = lunImpl.refresh();
        	 if(lun != null){
        		 StatsCollector.getInstance().addCounter("aeonflux-lun-create-refresh-fail-retry-success");   
        	 }
        }
        
        getLuns().put(fullPath, lun);        

        if (StringUtils.isReal(lunSpec.getComment())) {
            lun.setComment(lunSpec.getComment());
        }
        
        // dsarisky 6/26/14 - this appears to be copied over from original aeonflux, but it sure
        // seems wrong - only space reserve it if someone WANTS it space reserved
        if (lunSpec.getSpaceReserve() != SpaceReserve.none) {
            lun.setSpaceReservation(true);
        }
        return lun;
    }

    public CModeLun getLun(String path) {
        return (CModeLun) super.getLun(path);
    }

    /**
     * Waits for the specified ONTAP job to complete.
     *
     * @param jobId   - which job to query status
     * @param timeout - how long to wait until giving up on job (in seconds)
     * @param poll    - how frequently to poll for job status (in seconds)
     * @return state of job or null if not done
     * @throws com.netapp.offtap3.ontap.ApiFailedException
     */
    public String blockUntilJobCompletion(long jobId, int timeout, int poll) throws ApiFailedException {

        LOGGER.info("Waiting " + timeout + " seconds for job: " + jobId + " to complete.");
        NaElement xi = new NaElement("job-completed-get-iter");
        NaElement query = new NaElement("query");
        NaElement jobInfoQuery = new NaElement("job-info");
        jobInfoQuery.addNewChild("job-id", String.valueOf(jobId));
        query.addChildElem(jobInfoQuery);
        xi.addChildElem(query);

        long expireTime = System.currentTimeMillis() + timeout * 1000;
        String jobCompletionState = null;

        while (jobCompletionState == null && (System.currentTimeMillis() < expireTime)) {
            NaElement xo = invokeZapi(xi, ZapiCache.NO_CACHE);

            NaElement attributesList = xo.getChildByName("attributes-list");
            if (attributesList != null) {
                List<NaElement> jobInfos = attributesList.getChildren();
                NaElement jobInfo = jobInfos.get(0);
                jobCompletionState = jobInfo.getChildContent("job-state");
                LOGGER.info("Job (" + jobId + ") completed in state: " + jobCompletionState +
                                    " with message: " + jobInfo.getChildContent("job-completion"));
            } else {
                LOGGER.info(
                        "Job (" + jobId + ") has not completed.  Waiting " + poll + " seconds before polling again.");
                Utils.sleep(poll * 1000);
            }
        }
        return jobCompletionState;
    }

    @Override
    public boolean isSnapMirrorAvailable() {
        // SnapMirror is always available on cDOT, although it may not be licensed
        if (!isLicensed("snapmirror")) {
            LOGGER.warn(this + " does not have a SnapMirror license");
            return false;
        }
        return true;
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<CModeSnapMirrorRelation> getSnapMirrorRelations() {
        return (List<CModeSnapMirrorRelation>) super.getSnapMirrorRelations();
    }

    @Override
    public void enableSnapMirror() {
        LOGGER.debug("request to enable snap mirror on cDOT ignored for " + this);
    }

    @Override
    public void disableSnapMirror() {
        LOGGER.debug("request to disable snap mirror on cDOT ignored for " + this);
    }

    @Override
    public void updateSnapMirrorRelations() {
        NaElement xi = new NaElement("snapmirror-update-iter");
        NaElement query = new NaElement("query");
        NaElement snapmirrorInfo = new NaElement("snapmirror-info");
        xi.addChildElem(query);
        query.addChildElem(snapmirrorInfo);

        CdotIterativeLoader<Object> iterativeLoader = new CdotIterativeLoader<Object>(this, xi, new DataParser<Object>() {
            @Override
            public void parse(Object cummulativeResults, NaElement results) {
                String numFailed = results.getChildContent("num-failed");
                String numSucceeded = results.getChildContent("num-succeeded");
                String failureList = results.getChildByName("failure-list").toPrettyString("");
                String successList = results.getChildByName("success-list").toPrettyString("");
                LOGGER.debug("snapmirror-update-iter result = num-failed: " + numFailed + " num-succeeded: " + numSucceeded + " failure-list: " + failureList + " success-list: " + successList);
            }
        }, getOfftapPreferences().getIteratorSize(), ZapiCache.NO_CACHE);

        try {
            iterativeLoader.load(new Object());
        } catch (ApiFailedException e) {
            throw new SnapMirrorOperationException(this + " failed to update snap mirror relationships", e);
        }
    }

    @Override
    public List<IscsiSession> getIscsiSessions() {
        LOGGER.debug("getIscsiSessions: c-mode (begin)");

        ListMultimap<Integer, IscsiConnection> connections = ArrayListMultimap.create();
        for (IscsiConnection conn : getIscsiConnections()) {
            connections.put((int) conn.getSessionId(), conn);
        }

        List<IscsiSession> sessions = new ArrayList<IscsiSession>();
        SimpleCdotIterativeLoader iterativeLoader = new SimpleCdotIterativeLoader(this, new NaElement(
                "iscsi-session-get-iter"));

        try {
            for (NaElement sessionInfo : iterativeLoader.loadChildren()) {
                String nodename = sessionInfo.getChildContent("initiator-nodename");
                String alias = sessionInfo.getChildContent("initiator-alias");
                List<IscsiConnection> iscsiConnections = new ArrayList<IscsiConnection>();

                if (sessionInfo.getChildByName("iscsi-session-connection-list-entries")
                               .getChildContent("iscsi-session-connection-list-entry-info") != null) {
                    int sessionId = sessionInfo.getChildIntValue("target-session-id", 0);
                    iscsiConnections = connections.get(sessionId);
                }

                IscsiSessionImpl iscsiSession = new IscsiSessionImpl();
                iscsiSession.setInitiatorNodeName(nodename);
                iscsiSession.setInitiatorAlias(alias);

                iscsiSession.setIscsiConnections(iscsiConnections);
                sessions.add(iscsiSession);
            }
        } catch (ApiFailedException e) {
            throw new LunOperationException(this + " could not load iscsi sessions", e);
        }

        LOGGER.debug("getIscsiSessions: c-mode (end): " + sessions);

        return sessions;
    }

    @Override
    public void addInterfacesToIscsiInitiators(List<String> initiatorNames, List<IpInterface> interfaces) {
        LOGGER.debug(this + " addInterfacesToIscsiInitiators: begin");

        for (String init : initiatorNames) {
            for (IpInterface intf : interfaces) {
                NaElement xi = new NaElement("iscsi-interface-accesslist-add");
                xi.addNewChild("initiator", init);
                xi.addNewChild("interface-name", intf.getName());

                try {
                    invokeZapi(xi, ZapiCache.NO_CACHE);
                    getZapiCache().blastCacheValues("iscsi", "igroup");
                    LOGGER.debug("Added interface " + intf.getName() + " to iscsi initiator " + init);
                } catch (ApiFailedException ex) {
                    LOGGER.error(this + " failed to add interface " + intf + " to iscsi initiator " + init, ex);
                    throw new LunOperationException(
                            this + " failed to add interface " + intf + " to iscsi initiator " + init, ex);
                }
            }
        }
        LOGGER.debug(this + " addInterfacesToIscsiInitiators: end");
    }

    public OntapDataCache getDataCache() {
        return dataCache;
    }

    @Override
    public synchronized DataSvm getProxy() {
        if (proxy == null) {
            proxy = (DataSvm) CachingControllerProxy
                    .create(DataSvm.class, DataSvmProxy.class, this, dataCache);
        }
        return proxy;
    }

    @Override
    public void start() {
        requirePassThrough();
        NaElement xi = new NaElement("vserver-start");
        xi.addNewChild("vserver-name", MCCManager.getInstance().getVServerNameByUuid(this));
        try {
            // run against the admin connection
            adminSvm.invokeZapi(xi, ZapiCache.NO_CACHE);
            adminSvm.getZapiCache().blastCacheValues("vserver");

            LOGGER.info("Started vServer " + getName() + ".");
        } catch (ApiFailedException e) {
            LOGGER.error(this + " failed to start.", e);
            throw new SvmOperationException(this + " failed to start", e);
        }
    }

    @Override
    public void rename(String newName) {
        requirePassThrough();
        NaElement xi = new NaElement("vserver-rename");
        xi.addNewChild("vserver-name", MCCManager.getInstance().getVServerNameByUuid(this));
        xi.addNewChild("new-name", newName);
        try {
            // run against the admin connection
            adminSvm.invokeZapi(xi, ZapiCache.NO_CACHE);
            adminSvm.getZapiCache().blastCacheValues("vserver");
            LOGGER.info("Renamed vserver " + getName() + " to " + newName + ".");
            setName(newName);
            // map is keyed by vserver name, so that's all busted now.  refresh all.
            getParentController().refresh(ControllerResource.svms);
        } catch (ApiFailedException e) {
            LOGGER.error(this + " failed to rename to " + newName, e);
            throw new SvmOperationException(this + " failed to rename to " + newName, e);
        }
    }

    @Override
    public void stop() {
        requirePassThrough();
        NaElement xi = new NaElement("vserver-stop");
        xi.addNewChild("vserver-name", MCCManager.getInstance().getVServerNameByUuid(this));
        try {
            // run against the admin connection
            adminSvm.invokeZapi(xi, ZapiCache.NO_CACHE);
            adminSvm.getZapiCache().blastCacheValues("vserver");
            LOGGER.info("Stopped vServer " + getName() + ".");
        } catch (ApiFailedException e) {
            LOGGER.error("stopVServer: " + this + " Unable to stop vserver.", e);
            throw new SvmOperationException(this + " failed to stopVServer", e);
        }
    }

    @Override
    public void obliterate() {
        // we need to manually whack all of the volumes here.
        // have to do root volume last.
        for (OntapVolume volume : getProxy().getVolumes().values()) {
            if (!volume.getName().equals(getRootVolume())) {
                volume.destroy();
            }
        }
        OntapVolume rootVol = getProxy().getVolume(getRootVolume());
        if (rootVol != null) {
            rootVol.destroy();
        }

        // whack igroups too
        for (OntapIGroup igroup : getProxy().getIGroups()) {
            igroup.destroy();
        }
        destroy();
    }

    @Override
    public void destroy() {
        requirePassThrough();
        NaElement xi = new NaElement("vserver-destroy");
        xi.addNewChild("vserver-name", MCCManager.getInstance().getVServerNameByUuid(this));
        try {
            // run against the admin connection
            adminSvm.invokeZapi(xi, ZapiCache.NO_CACHE);
            adminSvm.getChildControllers().remove(getName());
            adminSvm.getZapiCache().blastCacheValues("");
            LOGGER.info("Destroyed vServer " + getName() + ".");
        } catch (ApiFailedException e) {
            LOGGER.error(this + " failed to destroy", e);
            throw new SvmOperationException(this + " failed to destroy.", e);
        }
    }

    private void requirePassThrough() {
        if (isDirectSvm()) {
            throw new SvmOperationException("This operation is not supported on a direct-connect SVM");
        }
    }

    @Override
    public void disableISCSI() {
        if (!getProxy().supportsProtocol(StorageProtocol.ISCSI)) {
            return;
        }
        String message = "Disabled ISCSI on " + getName();
        try {
            invokeZapi(new NaElement("iscsi-service-stop"), ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("iscsi-");
            LOGGER.info(message);
            getProxy().refresh(ControllerResource.protocols);
        } catch (ApiFailedException ex) {
            LOGGER.error("Failed to " + message, ex);
            throw new SvmOperationException(message + " Error: " + ex.getMessage(), ex);
        }
    }

    @Override
    public void enableISCSI() {
        if (getProxy().supportsProtocol(StorageProtocol.ISCSI)) {
            return;
        }

        String message = "Enabled ISCSI on " + getName();
        try {
            invokeZapi(new NaElement("iscsi-service-create"), ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("iscsi-");
            LOGGER.info(message);
            getProxy().refresh(ControllerResource.protocols);
        } catch (ApiFailedException ex) {
            if (ex.getMessage() != null && ex.getMessage().contains("exists")) {
                try {
                    invokeZapi(new NaElement("iscsi-service-start"), ZapiCache.NO_CACHE);
                } catch (ApiFailedException ex2) {
                    throw new SvmOperationException(message + " Error: " + ex.getMessage(), ex);
                }
                getZapiCache().blastCacheValues("iscsi-");
                LOGGER.info(message);
                getProxy().refresh(ControllerResource.protocols);
            } else {
                throw new SvmOperationException(message + " Error: " + ex.getMessage(), ex);
            }
        }
    }

    @Override
    public void enableFCP() {
        if (getProxy().supportsProtocol(StorageProtocol.FCP)) {
            return;
        }
        String message = "Enabled FCP on " + getName();
        try {
            invokeZapi(new NaElement("fcp-service-create"), ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("fcp-");
            LOGGER.info(message);
            getProxy().refresh(ControllerResource.protocols);
        } catch (ApiFailedException ex) {
            if (ex.getMessage() != null && ex.getMessage().contains("exists")) {
                try {
                    invokeZapi(new NaElement("fcp-service-start"), ZapiCache.NO_CACHE);
                } catch (ApiFailedException ex2) {
                    throw new SvmOperationException(message + " Error: " + ex.getMessage(), ex);
                }
                getZapiCache().blastCacheValues("fcp-");
                LOGGER.info(message);
                getProxy().refresh(ControllerResource.protocols);
            } else {
                throw new SvmOperationException(message + " Error: " + ex.getMessage(), ex);
            }
        }
    }

    @Override
    public void enableNFS() {
        if (getProxy().supportsProtocol(StorageProtocol.NFS)) {
            return;
        }

        String message = "Enabled NFS on " + getName();
        try {
            invokeZapi(new NaElement("nfs-enable"), ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("nfs-");
            LOGGER.info(message);
            getProxy().refresh(ControllerResource.protocols);
        } catch (ApiFailedException ex) {
            LOGGER.error("Failed to " + message, ex);
            throw new SvmOperationException(message + " Error: " + ex.getMessage(), ex);
        }
    }

    @Override
    public void disableNFS() {
        if (!getProxy().supportsProtocol(StorageProtocol.NFS)) {
            return;
        }
        String message = "Disabling NFS on " + getName();
        try {
            invokeZapi(new NaElement("nfs-disable"), ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("nfs-");
            LOGGER.info(message);
            getProxy().refresh(ControllerResource.protocols);
        } catch (ApiFailedException ex) {
            LOGGER.error("Failed to " + message, ex);
            throw new SvmOperationException(message + " Error: " + ex.getMessage(), ex);
        }
    }

    @Override
    public boolean isVStorageApiSupported() {
        return getVersion() != null && getVersion().compareTo(new OntapVersionImpl("8.1.0", 8, 1, 0)) >= 0;
    }

    @Override
    public void modifyVStorageApiEnabled(boolean enabled) {
        super.modifyVStorageApiEnabled(enabled);
        try {
            NaElement input = new NaElement("nfs-service-modify");
            input.addNewChild("is-vstorage-enabled", String.valueOf(enabled));
            invokeZapi(input, ZapiCache.NO_CACHE);
            getZapiCache().blastCacheValues("nfs-");
            LOGGER.info("Set vStorage enabled to " + enabled + " for controller " + getName());
            setVStorageApiEnabled(enabled);
        } catch (ApiFailedException e) {
            throw new SvmOperationException("An error occurred while setting vStorage enabled to " + enabled + " on controller " + getName() + ".", e);
        }
    }

    @Override
    public Set<StorageSystem> getStorageSystems() {
        return STORAGE_SYSTEMS;
    }
}
