package com.netapp.offtap3.internal.ontap.controller;

import com.netapp.offtap3.StorageSystem;
import com.netapp.offtap3.Version;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.OfftapProxy;
import com.netapp.offtap3.internal.ontap.GlobalStatus;
import com.netapp.offtap3.internal.ontap.SnmpHostManager;
import com.netapp.offtap3.internal.ontap.dataloader.FilerPerformanceObjectsLoaderDelegate;
import com.netapp.offtap3.internal.ontap.proxy.PFilerProxy;
import com.netapp.offtap3.ontap.ControllerWithSnmpSupport;
import com.netapp.offtap3.ontap.PFiler;
import com.netapp.offtap3.ontap.PortSet;
import com.netapp.offtap3.ontap.VFiler;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * Represents the physical 7-mode filer.
 */
public class PFilerImpl extends FilerImpl<PFiler> implements PFiler, HasChildOntapControllers,
        ControllerWithSnmpSupport {

    private static final Set<StorageSystem> storageSystems = Collections.singleton(StorageSystem.pfiler);
    private PFiler proxy;
    private GlobalStatus globalStatus = new GlobalStatus();
    private Map<String, VFiler> vfilers = new SafeConcurrentHashMap<String, VFiler>();
    private Set<String> snmpHosts = new CopyOnWriteArraySet<String>();
    private List<PortSet> portSets = new CopyOnWriteArrayList<PortSet>();
    private boolean asupEnabled;

    private SnmpHostManager snmpHostManager;
    private FilerPerformanceObjectsLoaderDelegate performanceObjectsLoader;


    public PFilerImpl(OntapConnection connInfo, Version version) {
        super(connInfo, version);
        snmpHostManager = new SnmpHostManager(this);
        performanceObjectsLoader = new FilerPerformanceObjectsLoaderDelegate(this);
    }

    @Override
    public OntapConnection getNonPassThroughConnection() {
        // this is a pfiler, so the connection is already to the pfiler (not passthrough)
        return getConnection();
    }


    public Map<String, VFiler> getVFilers() {
        return vfilers;
    }

    public void addVFiler(VFiler vfiler) {
        vfilers.put(vfiler.getName(), vfiler);
    }

    @Override
    public boolean isAsupEnabled() {
        return asupEnabled;
    }

    public void setAsupEnabled(boolean asup) {
        this.asupEnabled = asup;
    }

    @Override
    public boolean hasVFilerInterface(String ip) {
        for (VFiler vfiler : getVFilers().values()) {
            if (vfiler.getIpInterfaces().containsKey(ip)) {
                return true;
            }
        }
        return false;
    }


    @Override
    public List<PortSet> getPortSets() {
        return portSets;
    }

    public void setPortSets(List<PortSet> portSets) {
        this.portSets = portSets;
    }

    @Override
    public String getGlobalStatusMessage() {
        return globalStatus.getMessage();
    }

    public void setGlobalStatus(GlobalStatus globalStatus) {
        this.globalStatus = globalStatus;
    }

    @Override
    public int getGlobalStatusCode() {
        return globalStatus.getCode();
    }

    @Override
    public Set<String> getSnmpHosts() {
        return snmpHosts;
    }

    @Override
    public void setSnmpHosts(Set<String> snmpHosts) {
        this.snmpHosts = snmpHosts;
    }

    @Override
    public void addSnmpTrapListener(String hostIp) throws DataLoaderException {
        snmpHostManager.addSnmpTrapListener(hostIp);
    }

    @Override
    public void removeSnmpTrapListener(String hostIp) throws DataLoaderException {
        snmpHostManager.removeSnmpTrapListener(hostIp);
    }

    @Override
    public Map<String, VFiler> getChildControllers() {
        return getVFilers();
    }

    @Override
    public List<String> getPerformanceObjectInstanceList(String objectName) {
        return performanceObjectsLoader.getPerformanceObjectInstanceList(objectName);
    }

    @Override
    public Map<String, Long> getPerformanceCounters(String counter, String objectName, List<String> objectInstances) {
        return performanceObjectsLoader.getPerformanceCounters(counter, objectName, objectInstances);
    }

    @Override
    public synchronized PFiler getProxy() {
        if (proxy == null) {
            proxy = OfftapProxy.create(PFiler.class, PFilerProxy.class, this);
        }
        return proxy;
    }

    @Override
    public Set<StorageSystem> getStorageSystems() {
        return storageSystems;
    }
}
