package com.netapp.offtap3.internal.ontap.controller;

import com.netapp.offtap3.StorageSystem;
import com.netapp.offtap3.Version;
import com.netapp.offtap3.internal.OfftapProxy;
import com.netapp.offtap3.internal.ontap.proxy.VFilerProxy;
import com.netapp.offtap3.ontap.FilerVolume;
import com.netapp.offtap3.ontap.OntapVolumeSpec;
import com.netapp.offtap3.ontap.PFiler;
import com.netapp.offtap3.ontap.VFiler;
import com.netapp.offtap3.ontap.NaElement;

import java.util.Collections;
import java.util.Set;

public class VFilerImpl extends FilerImpl<VFiler> implements VFiler {

    private VFiler proxy;
    private static final Set<StorageSystem> storageSystems = Collections.singleton(StorageSystem.vfiler);

    private final PFilerImpl parent;
    private String vfilerStatus;

    // direct vfiler
    public VFilerImpl(OntapConnection connInfo, Version version) {
        super(connInfo, version);
        parent = null;
    }

    // pass-through vfiler
    public VFilerImpl(OntapConnection connInfo, PFilerImpl parent) {
        super(connInfo, parent.getVersion());
        this.parent = parent;
        setModel(parent.getModel());
        setSystemId(parent.getSystemId());
    }

    @Override
    public PFiler getParentController() {
        // this is a "public" method, so don't hand back the impl,
        // but instead return the proxy
        if (parent != null) {
            return parent.getProxy();
        }
        return null;
    }

    // convenience method - not exposed on interface until proven necessary
    @Override
    public boolean isDirectVFiler() {
        return parent == null;
    }

    // convenience method - not exposed on interface until proven necessary
    @Override
    public boolean isPassThroughVFiler() {
        return parent != null;
    }

    @Override
    public FilerVolume createVolume(OntapVolumeSpec spec) {
        if (isDirectVFiler()) {
            throw new UnsupportedOperationException("Direct vFiler connection (" + this + ") does not support volume creation.");
        }
        return super.createVolume(spec);
    }

    @Override
    public boolean enableNdmp() {
        if (isDirectVFiler()) {
            throw new UnsupportedOperationException("Direct vFiler connection (" + this + ") does not support NDMP enabling");
        }
        return super.enableNdmp();
    }

    @Override
    public OntapConnection getNonPassThroughConnection() {
        // if pass-through, then get a connection to the parent/pfiler
        if (isPassThroughVFiler()) {
            return parent.getConnection();
        } else {
            return getConnection();
        }
    }

    @Override
    public String getVFilerStatus() {
        return vfilerStatus;
    }

    public void setVFilerStatus(String status) {
        vfilerStatus = status;
    }

    @Override
    public void addSystemCliArgs(NaElement command) {
        if (isPassThroughVFiler()) {
            command.addNewChild("arg", "vfiler");
            command.addNewChild("arg", "run");
            command.addNewChild("arg", getName());
        }
    }

    @Override
    public String toString() {
        if (isDirectVFiler()) {
            return super.toString() + " [direct connect]";
        }
        return super.toString();
    }

    @Override
    public synchronized VFiler getProxy() {
        if (proxy == null) {
            proxy = OfftapProxy.create(VFiler.class, VFilerProxy.class, this);
        }
        return proxy;
    }

    @Override
    public Set<StorageSystem> getStorageSystems() {
        return storageSystems;
    }
}
