package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.AdminSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.utils.Utils;

import org.apache.log4j.Logger;

import java.util.HashSet;
import java.util.List;
import java.util.Set;


public class AdminSvmLicenseDataLoader extends AbstractDataLoader<AdminSvmImpl>  {

    private static final Logger logger = Logger.getLogger(AdminSvmLicenseDataLoader.class);

    @Override
    protected boolean doLoad(AdminSvmImpl adminSvm, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("populateLicenses: " + adminSvm + " gathering license information");
        Set<String> licenses = new HashSet<String>();
        NaElement result;
        try {            
            if (adminSvm.getVersion().atLeastSN()) {
                result = adminSvm.invokeZapi(new NaElement("license-v2-list-info"), ZapiCache.LONG_CACHE);
            } else {
                try {
                    result = adminSvm.invokeZapi(new NaElement("interim-license-list-get"), ZapiCache.LONG_CACHE);
                } catch (ApiFailedException e) {
                    result = adminSvm.invokeZapi(new NaElement("license-v2-list-info"), ZapiCache.LONG_CACHE);
                }
            }
            List<NaElement> licenseInfos = result.getChildByName("licenses").getChildren();
            for (NaElement licenseInfo : licenseInfos) {
                String service = licenseInfo.getChildContent("service");
                if (service == null) {
                    service = licenseInfo.getChildContent("package");
                }
                licenses.add(service);
            }
        }
        catch (ApiFailedException e) {
            throw new DataLoaderException(adminSvm + " failed to load licenses", e);
        }
        logger.debug("populateLicenses: for " + adminSvm + " concluded " + licenses);
        if (!Utils.isReal(licenses)) {
        	logger.error("populateLicenses: got bad looking data of " + result);
        }
        adminSvm.setLicenses(licenses);
        return true;
    }

}
