package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapController;
import com.netapp.offtap3.utils.Utils;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.ontap.OntapController;

import java.util.List;
import java.util.Map;

public class CModePerformanceObjectsLoaderDelegate extends PerformanceObjectsLoaderDelegate {

    public CModePerformanceObjectsLoaderDelegate(AbstractOntapController vserver) {
        super(vserver);
    }

    @Override
    protected String getZapiRequest() {
        return "perf-object-instance-list-info-iter";
    }

    @Override
    public Map<String, Long> getPerformanceCounters(String counter, String objectName, List<String> objectInstances) throws DataLoaderException {
        OntapController controller = getController();
        if (controller instanceof DataSvm) {
            if (((DataSvm)controller).isDirectSvm()) {
                throw new UnsupportedOperationException("get performance counters is not availabe for direct vserver " + controller);
            }
        }

        if (!Utils.isReal(objectInstances)) {
            throw new IllegalArgumentException("getPerformanceCounters in C-Mode requires an explicit list of instances, only 7-mode allows empty instances");
        }

        return super.getPerformanceCounters(counter, objectName, objectInstances);
    }

    @Override
    public List<String> getPerformanceObjectInstanceList(String objectName) throws DataLoaderException {
        OntapController controller = getController();
        if (controller instanceof DataSvm) {
            if (((DataSvm)controller).isDirectSvm()) {
                throw new UnsupportedOperationException("get performance object instance list is not availabe for direct vserver " + controller);
            }
        }
        return super.getPerformanceObjectInstanceList(objectName);
    }

}
