/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/CdotIterativeLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

public class CdotIterativeLoader<T> {
    private static final Logger logger = Logger.getLogger(CdotIterativeLoader.class);
    private DataParser<T> parser;
    private InternalZapiHandler zapiHandler;
    private NaElement input;
    private long iteratorSize;
    private long cache;

    public CdotIterativeLoader(InternalZapiHandler zapiHandler, NaElement input, DataParser<T> parser, long iteratorSize, long cache) {
        this.parser = parser;
        this.iteratorSize = iteratorSize;
        this.input = input;
        this.zapiHandler = zapiHandler;
        this.cache = cache;
    }

    public void load(T object) throws ApiFailedException {
        String nextTag = null;
        if (iteratorSize > 0 && (input.getChildByName("max-records") == null)) {
            input.addNewChild("max-records", String.valueOf(iteratorSize));
        }
        // Just in case the "tag" element was previously added, remove it.
        input.removeChildrenByName("tag");

        int numIterations = 1;
        do {
            if (nextTag != null) {
                NaElement tagElement = input.getChildByName("tag");
                if (tagElement == null) {
                    input.addChildElem(new NaElement("tag", nextTag));
                } else {
                    tagElement.setContent(nextTag);
                }
            }

            long start = System.currentTimeMillis();
            NaElement xo = zapiHandler.invokeZapi(input, cache);
            long end = System.currentTimeMillis();

            logger.debug("Retrieve of zapi " + input.getName() + " took " + (end - start) + " milliseconds for iter #" + numIterations);

            parser.parse(object, xo);

            nextTag = xo.getChildContent("next-tag");
            numIterations++;
        } while (nextTag != null);
    }
}
