package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.Messages;
import com.netapp.offtap3.internal.ontap.ClusterPeerImpl;
import com.netapp.offtap3.internal.ontap.controller.AdminSvmImpl;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.ClusterPeer;

import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

public class ClusterPeerDataLoader extends AbstractDataLoader<AdminSvmImpl> {

    private static final Logger logger = Logger.getLogger(ClusterPeerDataLoader.class);

    @Override
    protected Refreshable doRefresh(AdminSvmImpl loadTarget, Resource resource, Refreshable refreshTarget) {
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource), (ClusterPeer) refreshTarget);
        for (ClusterPeer peer : loadTarget.getClusterPeers()) {
            if (peer.getUuid().equals(((ClusterPeer) refreshTarget).getUuid())) {
                return peer;
            }
        }
        return null;
    }

    @Override
    protected boolean doLoad(AdminSvmImpl adminSvm, LoadType loadType, Set<Resource> resources) {
        return doLoad(adminSvm, loadType, resources, null);
    }

    protected boolean doLoad(AdminSvmImpl adminSvm, LoadType loadType, Set<Resource> resources, ClusterPeer reloadTarget) {
        List<ClusterPeer> peers = new ArrayList<ClusterPeer>();
        NaElement input = new NaElement("cluster-peer-get-iter");
        if (reloadTarget != null) {
            NaElement query = new NaElement("query");
            NaElement clusterPeerInfo = new NaElement("cluster-peer-info");
            clusterPeerInfo.addNewChild("cluster-uuid", reloadTarget.getUuid());
            query.addChildElem(clusterPeerInfo);
            input.addChildElem(query);

            // Initialize the list with the existing set of peers minus the one we're reloading.
            peers.addAll(adminSvm.getClusterPeers());
            peers.remove(reloadTarget);
        }
        SimpleCdotIterativeLoader iterLoader = new SimpleCdotIterativeLoader(adminSvm, input);

        try {
            for (NaElement peerXml : iterLoader.loadChildren()) {
                ClusterPeerImpl peer = new ClusterPeerImpl(adminSvm);
                peer.setName(peerXml.getChildContent("cluster-name"));
                peer.setUuid(peerXml.getChildContent("cluster-uuid"));
                peer.setAvailable("available".equals(peerXml.getChildContent("availability")));
                NaElement addressesXml = peerXml.getChildByName("peer-addresses");
                if (addressesXml != null) {
                    List<String> addresses = new ArrayList<String>();
                    for (NaElement addressXml : addressesXml.getChildren()) {
                        addresses.add(addressXml.getContent());
                    }
                    peer.setAddresses(addresses);
                }
                peers.add(peer);
            }
            adminSvm.setClusterPeers(peers);
        }
        catch (ApiFailedException e) {
            String msg = Messages.getInstance().getMessage("cluster.peer.load.fail", adminSvm, e.getMessage());
            logger.error(msg, e);
            throw new DataLoaderException(msg, e);
        }
        return true;
    }
}
