package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.CifsShare;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.CifsShareImpl;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;


public class DataSvmCifsSharesDataLoader extends AbstractDataLoader<DataSvmImpl> {

    @Override
    protected Refreshable doRefresh(DataSvmImpl loadTarget, Resource resource, Refreshable refreshTarget) {
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource));
        return getFromList(loadTarget.getCifsShares(), refreshTarget.getName());
    }

    @Override
    protected boolean doLoad(DataSvmImpl controller, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        List<CifsShare> cifsShares = new ArrayList<CifsShare>();
        try {
            SimpleCdotIterativeLoader iterativeLoader = new SimpleCdotIterativeLoader(controller, new NaElement("cifs-share-get-iter"));

            for (final NaElement cifsShare : iterativeLoader.loadChildren()) {
                CifsShareImpl share = new CifsShareImpl(controller);
                share.setName(cifsShare.getChildContent("share-name"));
                share.setMountPoint(cifsShare.getChildContent("path"));
                cifsShares.add(share);
            }

            controller.setCifsShares(cifsShares);
            return true;
        }
        catch (ApiFailedException e) {
            throw new DataLoaderException(controller + " failed to load CIFS shares", e);
        }
    }
}
