/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/DataSvmExportPolicyDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.ExportPolicyImpl;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.ExportPolicy;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class DataSvmExportPolicyDataLoader extends AbstractDataLoader<DataSvmImpl> {
    private static final Logger LOGGER = Logger.getLogger(DataSvmExportPolicyDataLoader.class);

    @Override
    protected Refreshable doRefresh(DataSvmImpl loadTarget, Resource resource, Refreshable refreshTarget) {
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource));
        return loadTarget.getExportPolicies().get(refreshTarget.getName());
    }

    @Override
    protected boolean doLoad(DataSvmImpl dataSvm, LoadType loadType,
                             Set<Resource> resources) throws DataLoaderException {
        LOGGER.debug("doLoad: begin - " + dataSvm.getName());

        NaElement input = new NaElement("export-policy-get-iter");
        CdotIterativeLoader<Map<String, ExportPolicy>> loader = new CdotIterativeLoader<Map<String, ExportPolicy>>(
                dataSvm, input, new ExportPolicyDataParser(dataSvm), dataSvm.getOfftapPreferences().getIteratorSize(),
                ZapiCache.SHORT_CACHE);
        Map<String, ExportPolicy> exportPolicies = new SafeConcurrentHashMap<String, ExportPolicy>();

        try {
            loader.load(exportPolicies);
        } catch (ApiFailedException e) {
            throw new DataLoaderException("Error occurred while loading export policies: " + e.getMessage(), e);
        }

        dataSvm.setExportPolicies(exportPolicies);
        LOGGER.debug("doLoad: end - " + dataSvm.getName());
        return true;
    }

    private class ExportPolicyDataParser implements DataParser<Map<String, ExportPolicy>> {
        private DataSvmImpl dataSvm;
        private DataCache<NaElement, InternalZapiHandler> dataCache;

        ExportPolicyDataParser(DataSvmImpl dataSvm) {
            this.dataSvm = dataSvm;
            dataCache = new OntapDataCache(dataSvm);
        }

        @Override
        public void parse(Map<String, ExportPolicy> cummulativeResults, NaElement results) {
            NaElement attributesList = results.getChildByName("attributes-list");
            if (attributesList == null) {
                return;
            }

            List<NaElement> exportsAttributes = results.getChildByName("attributes-list").getChildren();
            for (NaElement exportAttributes : exportsAttributes) {
                ExportPolicyImpl exportPolicy = new ExportPolicyImpl(dataSvm, dataCache);
                exportPolicy.setName(exportAttributes.getChildContent("policy-name"));
                exportPolicy.setId(exportAttributes.getChildLongValue("policy-id", -1));
                cummulativeResults.put(exportPolicy.getName(), exportPolicy.getProxy());
            }
        }
    }
}
