package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.NaErrno;
import org.apache.log4j.Logger;

import java.util.HashSet;
import java.util.Set;


public class DataSvmLicenseDataLoader extends AbstractDataLoader<DataSvmImpl> {

    private static final Logger logger = Logger.getLogger(DataSvmLicenseDataLoader.class);

    @Override
    protected boolean doLoad(DataSvmImpl svm, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        Set<String> licenses = new HashSet<String>();
        if (svm.isDirectSvm()) {
            logger.debug(svm + " will attempt to determine licenses.");

           	try{
           		logger.debug("Submitting dummy clone request to test if FlexClone is licensed.");
                svm.invokeZapi(new NaElement("clone-create"), ZapiCache.NO_CACHE);
          	} catch (ApiFailedException e) {
                // brianjm - you would think errno would be sufficient, but the old AeonFlux code
                // used the message, so preserving that "just in case".
                // Those that know better than me should remove it.
                if ((e.getErrno() == NaErrno.EDENSE_CLONE_NOT_LICENSED) ||
                        (e.getMessage().trim().equals("FlexClone not licensed (errno=14616)"))) {
                    logger.debug(this + " does not have a license for flex clone.  Error was: " + e.getMessage());
                } else {
                    // The one known problem with this case is that permissions are checked before licenses, so in a case
                    // where FlexClone isn't licensed and the user isn't permitted to use it, this block will execute
                    // giving a false positive for the licensing. However, the user still won't be able to FlexClone as
                    // they have no permission to do so.
                    logger.debug("FlexClone failed, but not due to a licensing issue. Adding FlexClone to license list.");
                    licenses.add("flexclone");
                }
           	}
        }
        else {
            licenses.addAll(svm.getParentControllerImpl().getLicenses());
        }
        svm.setLicenses(licenses);
        return true;
    }
}
