/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/DataSvmPortSetDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.utils.StringUtils;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.PortSet;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

public class DataSvmPortSetDataLoader extends AbstractPortSetDataLoader<DataSvmImpl> {
    private static Logger logger = Logger.getLogger(DataSvmPortSetDataLoader.class);
    private CdotIterativeLoader<NaElement> iterativeLoader;

    public DataSvmPortSetDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);
        iterativeLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(),
                                                             new NaElement("portset-get-iter"),
                                                             new CumulativeDataParser(), prefs.getIteratorSize(),
                                                             ZapiCache.SHORT_CACHE);
    }

    @Override
    public java.util.Map<Resource, NaElement> retrieve(DataSvmImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("retrieve: start");

        NaElement result = new NaElement("results");
        try {
            iterativeLoader.load(result);
        } catch (ApiFailedException e) {
            throw new DataLoaderException(e);
        }

        logger.debug("retrieve: end");
        return Collections.singletonMap(resources.iterator().next(), result);
    }

    @Override
    public boolean parse(DataSvmImpl dataSvm, NaElement data, Resource resource) {
        logger.debug("parse: begin");

        List<NaElement> pps = data.getChildren();
        List<PortSet> portSets = new ArrayList<PortSet>(pps.size());
        for (final NaElement portsetInfo : pps) {
            String vserver = portsetInfo.getChildContent("vserver");
            if (StringUtils.isReal(vserver) && MCCManager.getInstance().getVServerNameByUuid(dataSvm).equals(vserver)) {
                PortSet portSet = parsePortSet(portsetInfo);
                portSets.add(portSet);
            }
        }
        dataSvm.setPortSets(portSets);

        logger.debug("parse: end result = " + dataSvm.getPortSets());
        return true;
    }
}
