package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.qos.QosPolicyGroupImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.QosPolicyGroup;
import com.netapp.offtap3.ontap.QosThroughput;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class DataSvmQosPolicyGroupDataLoader extends OntapCachingDataLoader<DataSvmImpl> {

    private static final Logger logger = Logger.getLogger(DataSvmQosPolicyGroupDataLoader.class);

    private CdotIterativeLoader<NaElement> iterativeLoader;

    public DataSvmQosPolicyGroupDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);
        iterativeLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(),
                                                             new NaElement("qos-policy-group-get-iter"),
                                                             new CumulativeDataParser(), prefs.getIteratorSize(),
                                                             ZapiCache.SHORT_CACHE);
    }
    
    
    @Override
    protected Map<Resource, NaElement> retrieve(DataSvmImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {    	
        logger.debug("retrieve: start");
        if (object.getVersion().atLeastSN()) {
	        NaElement result = new NaElement("results");
	        try {
	            iterativeLoader.load(result);
	        } catch (ApiFailedException e) {
	            throw new DataLoaderException(e);
	        }
	
	        logger.debug("retrieve: end");
	        return Collections.singletonMap(resources.iterator().next(), result);
        } else {
        	return Collections.EMPTY_MAP;
        }
    }

    @Override
    protected boolean parse(DataSvmImpl svm, NaElement data, Resource resource) {
        logger.debug("parse: begin");

        List<NaElement> policyElems = data.getChildren();
        Map<String, QosPolicyGroup> groups = new SafeConcurrentHashMap<String, QosPolicyGroup>();
        for (NaElement policyElem : policyElems) {
            if (policyElem != null) {
                String vserverName = policyElem.getChildContent("vserver");
                if (MCCManager.getInstance().getVServerNameByUuid(svm).equals(vserverName)) {
                    QosThroughput maxThroughput = new QosThroughput(policyElem.getChildContent("max-throughput"));
                    String name = policyElem.getChildContent("policy-group");
                    QosPolicyGroupImpl group = new QosPolicyGroupImpl(svm, name, maxThroughput);
                    groups.put(name, group);
                }
            }
        }
        svm.setQosPolicyGroups(groups);
        return true;
    }
}
