/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/DataSvmUnixUserAndGroupDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.UnixGroup;
import com.netapp.offtap3.UnixUser;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.UnixGroupImpl;
import com.netapp.offtap3.internal.UnixUserImpl;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class DataSvmUnixUserAndGroupDataLoader extends AbstractDataLoader<DataSvmImpl> {
    @Override
    protected boolean doLoad(DataSvmImpl dataSvm, LoadType loadType,
                             Set<Resource> resources) throws DataLoaderException {
        Map<String, UnixGroup> unixGroups = new SafeConcurrentHashMap<String, UnixGroup>();
        NaElement input = new NaElement("name-mapping-unix-group-get-iter");
        CdotIterativeLoader<Map<String, UnixGroup>> groupsLoader = new CdotIterativeLoader<Map<String, UnixGroup>>(
                dataSvm, input, new UnixGroupParser(dataSvm), dataSvm.getOfftapPreferences().getIteratorSize(), ZapiCache.SHORT_CACHE);

        try {
            groupsLoader.load(unixGroups);
        } catch (ApiFailedException e) {
            throw new DataLoaderException("Unable to load unix groups for data svm " + dataSvm.getName(), e);
        }

        Map<String, UnixUser> unixUsers = new SafeConcurrentHashMap<String, UnixUser>();
        input = new NaElement("name-mapping-unix-user-get-iter");
        CdotIterativeLoader<Map<String, UnixUser>> usersLoader = new CdotIterativeLoader<Map<String, UnixUser>>(dataSvm,
                                                                                                                input,
                                                                                                                new UnixUserParser(unixGroups),
                                                                                                                dataSvm.getOfftapPreferences().getIteratorSize(),
                                                                                                                ZapiCache.SHORT_CACHE);

        try {
            usersLoader.load(unixUsers);
        } catch (ApiFailedException e) {
            throw new DataLoaderException("Unable to load unix users for data svm " + dataSvm.getName(), e);
        }

        updateUsers(unixGroups, unixUsers);

        dataSvm.setUnixGroups(unixGroups);
        dataSvm.setUnixUsers(unixUsers);
        return true;
    }

    /**
     * This step ensures the users in the groups are the fully populated users. Secondary users will not have their
     * id set to the proper value and this fixes that.
     */
    private void updateUsers(Map<String, UnixGroup> groups, Map<String, UnixUser> users) {
        for (UnixGroup group : groups.values()) {
            Map<String, UnixUser> populatedUsers = new SafeConcurrentHashMap<String, UnixUser>();
            for (UnixUser user : group.getUsers().values()) {
                UnixUser populatedUser = users.get(user.getName());
                if (populatedUser != null) {
                    populatedUsers.put(populatedUser.getName(), populatedUser);
                }
            }
            ((UnixGroupImpl)group).setUsers(populatedUsers);
        }
    }

    private class UnixGroupParser implements DataParser<Map<String, UnixGroup>> {
        private DataSvmImpl dataSvm;

        public UnixGroupParser(DataSvmImpl dataSvm) {
            this.dataSvm = dataSvm;
        }

        @Override
        public void parse(Map<String, UnixGroup> groups, NaElement results) {
            NaElement attributesList = results.getChildByName("attributes-list");
            if (attributesList == null) {
                return;
            }

            List<NaElement> groupsAttributes = results.getChildByName("attributes-list").getChildren();
            for (NaElement groupAttributes : groupsAttributes) {
                String groupName = groupAttributes.getChildContent("group-name");
                int groupId = groupAttributes.getChildIntValue("group-id", 0);

                UnixGroupImpl group = new UnixGroupImpl(dataSvm, groupName, groupId);
                if(groupAttributes.getChildByName("users") != null) {
                    for(NaElement userInfo : groupAttributes.getChildByName("users").getChildren()) {
                        UnixUserImpl user = new UnixUserImpl(group, userInfo.getChildContent("user-name"), -1);
                        group.addUser(user);
                    }
                }
                groups.put(groupName, group);
            }
        }
    }

    private class UnixUserParser implements DataParser<Map<String, UnixUser>> {
        private Map<String, UnixGroup> groups;

        public UnixUserParser(Map<String, UnixGroup> groups) {
            this.groups = groups;
        }

        @Override
        public void parse(Map<String, UnixUser> users, NaElement results) {
            NaElement attributesList = results.getChildByName("attributes-list");
            if (attributesList == null) {
                return;
            }

            List<NaElement> usersAttributes = results.getChildByName("attributes-list").getChildren();
            for(NaElement userAttributes : usersAttributes) {
                String name = userAttributes.getChildContent("user-name");
                int userId = userAttributes.getChildIntValue("user-id", 0);
                int groupId = userAttributes.getChildIntValue("group-id", 0);
                UnixGroupImpl group = findGroup(groupId);
                if (group != null) {
                    UnixUserImpl user = new UnixUserImpl(group, name, userId);
                    Set<UnixGroup> secondaryGroups = getAndUpdateSecondaryGroups(user);
                    user.setSecondaryGroups(secondaryGroups);
                    users.put(name, user);
                    group.addUser(user);
                }
            }
        }

        private UnixGroupImpl findGroup(int groupId) {
            for (UnixGroup group : groups.values()) {
                if (group.getId() == groupId) {
                    return (UnixGroupImpl) group;
                }
            }
            return null;
        }

        private Set<UnixGroup> getAndUpdateSecondaryGroups(UnixUserImpl user) {
            Set<UnixGroup> secondaryGroups = new HashSet<UnixGroup>();
            for (UnixGroup group : groups.values()) {
                UnixUser oldUser = group.getUsers().get(user.getName());
                if (oldUser != null) {
                    secondaryGroups.add(group);
                    ((UnixGroupImpl)group).removeUser(user.getName());
                    ((UnixGroupImpl)group).addUser(user);
                }
            }
            return secondaryGroups;
        }
    }
}