package com.netapp.offtap3.internal.ontap.dataloader;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;

import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.volume.CModeVolumeImpl;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.CModeVolume;
import com.netapp.offtap3.ontap.NaElement;

public class DataSvmVolumeDataLoader extends AbstractDataLoader<DataSvmImpl> {
    private static Logger logger = Logger.getLogger(DataSvmVolumeDataLoader.class);

    private OfftapPreferences prefs;

    public DataSvmVolumeDataLoader(OfftapPreferences prefs) {
        this.prefs = prefs;
    }

    @Override
    protected boolean doLoad(DataSvmImpl dataSvm, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        try {
            logger.debug("populateVolume: start");

            NaElement input = new NaElement("volume-get-iter");

            NaElement desiredAttributes = new NaElement("desired-attributes");
            NaElement desiredVolumeAttributes = new NaElement("volume-attributes");

            // always get volume id attributes so we can find ourselves.
            NaElement volumeIdAttributes = new NaElement("volume-id-attributes");
            volumeIdAttributes.addChildElem(new NaElement("name"));

            // note - uuid changes on vol move.  instance-uuid does not.
            // that's the one we want
            volumeIdAttributes.addChildElem(new NaElement("instance-uuid"));
            volumeIdAttributes.addChildElem(new NaElement("containing-aggregate-name"));
            volumeIdAttributes.addChildElem(new NaElement("containing-aggregate-uuid"));
            volumeIdAttributes.addChildElem(new NaElement("junction-path"));
            volumeIdAttributes.addChildElem(new NaElement("type"));
            desiredVolumeAttributes.addChildElem(volumeIdAttributes);
            desiredAttributes.addChildElem(desiredVolumeAttributes);
            input.addChildElem(desiredAttributes);
            VolumeParser volumeParser = new VolumeParser();
            CdotIterativeLoader<DataSvmImpl> loader = new CdotIterativeLoader<DataSvmImpl>(dataSvm, input, volumeParser, prefs.getIteratorSize(),
                                                                                    ZapiCache.SHORT_CACHE);
            loader.load(dataSvm);
            volumeParser.applyAccumulatedVolumes();

        } catch (ApiFailedException e) {
            throw new DataLoaderException("Unable to load volumes for: " + dataSvm, e);
        }
        return true;
    }

    private class VolumeParser implements DataParser<DataSvmImpl> {
        private DataCache<NaElement, InternalZapiHandler> dataCache;
        private DataSvmImpl dataSvm;
        private Map<String, CModeVolume> accumulatedVolumes = new ConcurrentHashMap<String, CModeVolume>();


        @Override
        public void parse(DataSvmImpl dataSvm, NaElement results) {
        	this.dataSvm = dataSvm;
            if (dataCache == null) {
                // Set up the data cache the volumes will use.
                dataCache = new OntapDataCache(dataSvm);
            }

            NaElement attrList = results.getChildByName("attributes-list");
            if (attrList != null) {
                List<NaElement> volumesAttributes = attrList.getChildren();

                for (NaElement volumeInfo : volumesAttributes) {
                    CModeVolumeImpl volume = new CModeVolumeImpl(dataSvm, dataCache);

                    NaElement volumeId = volumeInfo.getChildByName("volume-id-attributes");
                    if (volumeId != null) {
                        volume.setName(volumeId.getChildContent("name"));

                        // note - uuid changes on vol move.  instance-uuid does not.
                        // that's the one we want
                        volume.setUuid(volumeId.getChildContent("instance-uuid"));
                        volume.setContainingAggregate(volumeId.getChildContent("containing-aggregate-name"));
                        volume.setAggregateUUID(volumeId.getChildContent("containing-aggregate-uuid"));
                        volume.setJunctionPath(volumeId.getChildContent("junction-path"));
                        volume.setFlexVolType(volumeId.getChildContent("type"));
                    }

                    if (!"tmp".equals(volume.getFlexVolType())) {
                        // discard tmp types, these are temporary volumes created as part of vol move.
                        // we don't want anyone fooling with these since they disappear real quick.
                        CModeVolume proxy = volume.getProxy();
                        // have to accumulate these volumes into a data structure so that we can set them all at once.
                        // previously we just added them to the DataSvm, which meant it became a roach motel of accumulated volumes
                        // and no one ever cleaned it out when they disappeared off the filer.
                        accumulatedVolumes.put(volume.getName(), proxy);
                    }
                }
            }
        }
        
        public void applyAccumulatedVolumes() {
        	if (dataSvm != null) {
        		dataSvm.setVolumes(accumulatedVolumes);
        	}
        }
    }
}
