/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/ExportPolicyRulesDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.ExportPolicyImpl;
import com.netapp.offtap3.internal.ontap.ExportPolicyRuleImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.ExportPolicyRule;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;

public class ExportPolicyRulesDataLoader extends OntapCachingDataLoader<ExportPolicyImpl> {
    private static final Logger LOGGER = Logger.getLogger(ExportPolicyRulesDataLoader.class);
    private OfftapPreferences prefs;

    public ExportPolicyRulesDataLoader(DataCache<NaElement, InternalZapiHandler> dataCache, OfftapPreferences prefs) {
        super(dataCache);
        this.prefs = prefs;
    }

    @Override
    protected Map<Resource, NaElement> retrieve(ExportPolicyImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        LOGGER.debug("doLoad: begin");

        NaElement input = new NaElement("export-rule-get-iter");
        CdotIterativeLoader<NaElement> loader = new CdotIterativeLoader<NaElement>(getDataCache().getDataRetriever(),
                                                                                   input, new CumulativeDataParser(),
                                                                                   prefs.getIteratorSize(),
                                                                                   ZapiCache.SHORT_CACHE);
        NaElement results = new NaElement("results");

        try {
            loader.load(results);
        } catch (ApiFailedException e) {
            throw new DataLoaderException("Error occurred while loading export policies: " + e.getMessage(), e);
        }

        LOGGER.debug("doLoad: end");
        return Collections.singletonMap(resources.iterator().next(), results);
    }

    @Override
    protected boolean parse(ExportPolicyImpl exportPolicy, NaElement data, Resource resource) {
        LOGGER.debug("parse: begin");
        List<ExportPolicyRule> rules = new CopyOnWriteArrayList<>();
        List<NaElement> exportsAttributes = data.getChildren();
        for (NaElement exportAttributes : exportsAttributes) {
            String policyName = exportAttributes.getChildContent("policy-name");
            if (exportPolicy.getName().equals(policyName)) {
                // Setting this to -1 to indicate that we had no value.  0 is a valid and frequently used value,
                // so it's unclear whether we were getting something default or legitimate.
                int anonymousUserId = exportAttributes.getChildIntValue("anonymous-user-id", -1);
                String clientMatch = exportAttributes.getChildContent("client-match");
                String changeOwnershipMode = exportAttributes.getChildContent("export-chown-mode");
                String ntfsUnixSecurityOperations = exportAttributes.getChildContent("export-ntfs-unix-security-ops");

                boolean isAllowDeviceCreationEnabled = "true".equals(
                        exportAttributes.getChildContent("is-allow-dev-is-enabled"));
                boolean isAllowSetUuidEnabled = "true".equals(
                        exportAttributes.getChildContent("is-allow-set-uid-enabled"));

                int ruleIndex = exportAttributes.getChildIntValue("rule-index", 0);

                List<String> accessProtocols = new ArrayList<String>();
                if (exportAttributes.getChildContent("protocol") != null) {
                    for (NaElement accessProtocol : exportAttributes.getChildByName("protocol").getChildren()) {
                        accessProtocols.add(accessProtocol.getContent());
                    }
                }

                List<String> readOnlySecurityFlavors = new ArrayList<String>();
                if (exportAttributes.getChildContent("ro-rule") != null) {
                    for (NaElement securityFlavor : exportAttributes.getChildByName("ro-rule").getChildren()) {
                        readOnlySecurityFlavors.add(securityFlavor.getContent());
                    }
                }

                List<String> readWriteSecurityFlavors = new ArrayList<String>();
                if (exportAttributes.getChildContent("rw-rule") != null) {
                    for (NaElement securityFlavor : exportAttributes.getChildByName("rw-rule").getChildren()) {
                        readWriteSecurityFlavors.add(securityFlavor.getContent());
                    }
                }

                List<String> superUserSecurityFlavors = new ArrayList<String>();
                if (exportAttributes.getChildContent("super-user-security") != null) {
                    for (NaElement securityFlavor : exportAttributes.getChildByName("super-user-security")
                                                                    .getChildren()) {
                        superUserSecurityFlavors.add(securityFlavor.getContent());
                    }
                }

                ExportPolicyRuleImpl exportRule = new ExportPolicyRuleImpl(exportPolicy, anonymousUserId, clientMatch,
                                                                           changeOwnershipMode,
                                                                           ntfsUnixSecurityOperations,
                                                                           isAllowDeviceCreationEnabled,
                                                                           isAllowSetUuidEnabled, policyName, ruleIndex,
                                                                           accessProtocols, readOnlySecurityFlavors,
                                                                           readWriteSecurityFlavors,
                                                                           superUserSecurityFlavors);
                rules.add(exportRule);
            }
        }
        exportPolicy.setRules(rules);
        LOGGER.debug("parse: end");
        return true;
    }

}
