package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.CifsShare;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.CifsShareImpl;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.Filer;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;


public class FilerCifsSharesDataLoader extends AbstractDataLoader<FilerImpl<? extends Filer>> {

    private static final Logger logger = Logger.getLogger(FilerCifsSharesDataLoader.class);

    private final OfftapPreferences offtapPreferences;

    public FilerCifsSharesDataLoader(OfftapPreferences offtapPreferences) {
        this.offtapPreferences = offtapPreferences;
    }

    @Override
    protected boolean doLoad(FilerImpl<? extends Filer> filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        if (!filer.isLicensed("cifs")) {
            logger.info(filer + " unable to load CIFS shares due to lack of license");
            return false;
        }

        long maxGrab = offtapPreferences.getIteratorSize();

        // you'd think we could cache this, but 7-mode iteration stuff here is a frackin
        // mess, I'm just not gonna fool with it
        NaElement xo;
        try {
            xo = filer.invokeZapi(new NaElement("cifs-share-list-iter-start"), ZapiCache.NO_CACHE);
        } catch (ApiFailedException nafe) {
            if (nafe.toString() != null && nafe.toString().contains("CIFS not running")) {
                filer.setCifsShares(new ArrayList<CifsShare>());
                return false;
            } else {
                throw new DataLoaderException(filer + " failed to list CIFS shares", nafe);
            }
        }

        Long records = xo.getChildLongValue("records", 0L);
        String tag = xo.getChildContent("tag");

        boolean done = false;
        // grab is the maximum records to get per iteration.
        Long grab = records;
        List<CifsShare> cifsShares = new ArrayList<CifsShare>();

        try {
            while (!done){
                if (grab > maxGrab){
                    grab = maxGrab;
                }
                if (records < 1) {
                    break;
                }
                NaElement api = new NaElement("cifs-share-list-iter-next");
                api.addNewChild("tag", tag);
                api.addNewChild("maximum", grab+"");
                xo = filer.invokeZapi(api, ZapiCache.NO_CACHE);
                if (xo != null){
                    if (xo.getChildByName("cifs-shares") != null){
                        List<NaElement> pps = xo.getChildByName("cifs-shares").getChildren();
                        for (final NaElement cifsShare : pps) {
                            CifsShareImpl share = new CifsShareImpl(filer);
                            share.setName(cifsShare.getChildContent("share-name"));
                            share.setMountPoint(cifsShare.getChildContent("mount-point"));
                            cifsShares.add(share);
                        }
                    }
                }
                records = records - grab;
                if (records < 1) {
                    done = true;
                } else {
                    grab = records;
                }
            }

            NaElement api = new NaElement("cifs-share-list-iter-end");
            api.addNewChild("tag", tag);
            filer.invokeZapi(api, ZapiCache.NO_CACHE);
        }
        catch (ApiFailedException e) {
            throw new DataLoaderException(filer + " failed to iterate CIFS shares", e);
        }
        logger.debug(filer + " loaded " + cifsShares.size() + " CIFS shares");
        filer.setCifsShares(cifsShares);
        return true;
    }

    @Override
    protected Refreshable doRefresh(FilerImpl<? extends Filer> loadTarget, Resource resource, Refreshable refreshTarget) {
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource));
        return getFromList(loadTarget.getCifsShares(), refreshTarget.getName());
    }
}
