package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.ClusterFailoverInfoImpl;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.NaErrno;
import org.apache.log4j.Logger;

import java.util.Set;

public class FilerClusterFailoverDataLoader extends AbstractDataLoader<FilerImpl> {

    private static final Logger logger = Logger.getLogger(FilerClusterFailoverDataLoader.class);

    @Override
    public boolean doLoad(FilerImpl filer, LoadType loadType, Set<Resource> resources) {
        logger.debug("load: start " +filer);
        boolean passed = false;
        try {
            NaElement result = filer.invokeZapi(new NaElement("cf-status"), ZapiCache.SHORT_CACHE);
            ClusterFailoverInfoImpl cfInfo = new ClusterFailoverInfoImpl();
            filer.setClusterFailoverInfo(cfInfo);
            cfInfo.setState(result.getChildContent("state"));

            // only valid for SN or better filers
            cfInfo.setCurrentMode(result.getChildContent("current-mode"));

            result = filer.invokeZapi(new NaElement("cf-get-partner"), ZapiCache.SHORT_CACHE);
            cfInfo.setPartnerName(result.getChildContent("partner"));

            passed = true;

            result = filer.invokeZapi(new NaElement("fcp-get-cfmode"), ZapiCache.SHORT_CACHE);
            cfInfo.setCfMode(result.getChildContent("fcp-cfmode"));
        } catch (ApiFailedException ex) {
            if (ex.getErrno() == NaErrno.EINTERNALERROR){
                logger.warn("load: " +filer +" Failed to get cluster info. " + ex.getMessage());
            }else{
                logger.error("load: " +filer +" Failed to get cluster info. " + ex.getMessage());
            }
        }
        return passed;
    }
}
