package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.intf.OntapIGroupImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.Filer;
import com.netapp.offtap3.ontap.OntapIGroup;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;

public class FilerIGroupDataLoader extends AbstractDataLoader<FilerImpl<? extends Filer>> {

    private static final Logger LOGGER = Logger.getLogger(FilerIGroupDataLoader.class);

    @Override
    protected Refreshable doRefresh(FilerImpl<? extends Filer> filer, Resource resource, Refreshable refreshTarget) {
        doLoad(filer, LoadType.refresh, Collections.singleton(resource), refreshTarget);
        return filer.getIGroup(refreshTarget.getName());
    }

    @Override
    protected boolean doLoad(FilerImpl<? extends Filer> filer, LoadType loadType, Set<Resource> resources) {
        return doLoad(filer, loadType, resources, null);
    }

    protected <F extends Filer> boolean doLoad(FilerImpl<F> filer, LoadType loadType, Set<Resource> resources, Refreshable reloadTarget) {
        LOGGER.debug(filer + " start");

        List<OntapIGroup> iGroups = new CopyOnWriteArrayList<OntapIGroup>();

        try {
            NaElement input = new NaElement("igroup-list-info");
            if (loadType == LoadType.refresh && reloadTarget != null) {
                input.addNewChild("initiator-group-name", reloadTarget.getName());
            }
            NaElement output = filer.invokeZapi(input, ZapiCache.SHORT_CACHE);
            for (NaElement iGroupInfo : output.getChildByName("initiator-groups").getChildren()) {
                OntapIGroupImpl<FilerImpl<F>, F> iGroup = new OntapIGroupImpl<>(filer);
                iGroup.setName(iGroupInfo.getChildContent("initiator-group-name"));
                iGroup.setProtocol(iGroupInfo.getChildContent("initiator-group-type"));
                iGroup.setOsType(iGroupInfo.getChildContent("initiator-group-os-type"));
                iGroup.setAluaEnabled(Boolean.parseBoolean(iGroupInfo.getChildContent("initiator-group-alua-enabled")));
                iGroup.setPortsetName(iGroupInfo.getChildContent("initiator-group-portset-name"));

                Set<String> initiators = new HashSet<String>();
                if (iGroupInfo.getChildByName("initiators").getChildContent("initiator-info") != null) {
                    for (NaElement el1 : iGroupInfo.getChildByName("initiators").getChildren()) {
                        String iname = el1.getChildContent("initiator-name");
                        if (iname != null) {
                            initiators.add(iname);
                        }
                    }
                }
                iGroup.setInitiatorIds(initiators);

                int lunId;
                if (iGroupInfo.getChildContent("lun-id") != null) {
                    lunId = iGroupInfo.getChildIntValue("lun-id", 0);
                } else {
                    lunId = -1;
                }
                iGroup.setLunId(lunId);
                iGroups.add(iGroup);
            }
            filer.setIGroups(iGroups);
            LOGGER.debug(filer + " has " + iGroups.size() + " igroups");
        }
        catch (ApiFailedException e) {
            LOGGER.error(filer + " failed to load igroups", e);
            throw new DataLoaderException(filer + " failed to load igroups", e);
        }
        return true;
    }
}
