package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.OntapVersionImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class FilerLicenseDataLoader extends AbstractDataLoader<FilerImpl> {
    private static Logger logger = Logger.getLogger(FilerSystemInfoDataLoader.class);

    @Override
    protected boolean doLoad(FilerImpl filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("getLicenses: " +filer +" gathering license information");

        Set<String> licenses = new HashSet<String>();
        String licenseZapi = "license-list-info";
        OntapVersionImpl ntapVersion = (OntapVersionImpl) filer.getVersion();
        if (ntapVersion.atLeastSN()) {
            licenseZapi = "license-v2-list-info";
        }

        NaElement result = null;
        try {
            result = filer.invokeZapi(new NaElement(licenseZapi), ZapiCache.LONG_CACHE);
        } catch (ApiFailedException e) {
            throw new DataLoaderException("Failed to load licenses for: " + filer, e);
        }
        List<NaElement> licenseInfos = result.getChildByName("licenses").getChildren();
        for (NaElement licenseInfo : licenseInfos) {
            String service = licenseInfo.getChildContent("service");
            if (service != null) {
                if (Boolean.parseBoolean(licenseInfo.getChildContent("is-licensed"))) {
                    licenses.add(service); // original license ZAPI result
                }
            }
            else if (licenseInfo.getChildContent("package") != null) {
                licenses.add(licenseInfo.getChildContent("package").toLowerCase());  // v2 license ZAPI result
            }
        }

        filer.setLicenses(licenses);

        return true;
    }
}
