package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.PFilerImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.PortSet;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class FilerPortSetDataLoader extends AbstractDataLoader<PFilerImpl> {

    private static final Logger logger = Logger.getLogger(FilerPortSetDataLoader.class);

    @Override
    protected boolean doLoad(PFilerImpl filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {

        NaElement xo;
        try {
            xo = filer.invokeZapi(new NaElement("portset-list-info"), ZapiCache.LONG_CACHE);
        } catch (ApiFailedException nafe) {
            if (nafe.getErrno() == 9129) {
                return false;
            }
            throw new DataLoaderException(filer + " failed to get portset list", nafe);
        }

        if (xo.getChildByName("portset-sets") != null) {
            List<NaElement> pps = xo.getChildByName("portset-sets").getChildren();
            List<PortSet> portSets = new ArrayList<PortSet>(pps.size());
            for (final NaElement portsetInfo : pps) {
                PortSet portSet = AbstractPortSetDataLoader.parsePortSet(portsetInfo);
                portSets.add(portSet);
            }
            filer.setPortSets(portSets);
        }
        logger.debug(filer + " now has " + filer.getPortSets().size() + " portsets");
        return true;
    }
}
