/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader;

import java.util.Set;

import org.apache.log4j.Logger;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.OntapVersionImpl;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;

public class FilerVStorageStatusDataLoader extends AbstractDataLoader<FilerImpl> {
    private static final Logger logger = Logger.getLogger(FilerVStorageStatusDataLoader.class);

    @Override
    protected boolean doLoad(FilerImpl filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("Filer vStorage Status Load: begin");
        boolean result = true;
        if (filer.getVersion().compareTo(new OntapVersionImpl("8.1.1", 8, 1, 1)) < 0) {
            logger.debug("Skipped loading vstorage status, not supported prior to ONTAP 8.1.1 in 7-Mode.");
        } else {
            NaElement input = new NaElement("options-get");
            input.addNewChild("name", "nfs.vstorage.enable");
            try {
                NaElement output = filer.invokeZapi(input, ZapiCache.LONG_CACHE);
                String value = output.getChildContent("value");
                if (value != null) {
                    filer.setVStorageApiEnabled("on".equals(value));
                } else {
                    filer.setVStorageApiEnabled(false);
                    result = false;
                }
            } catch (ApiFailedException e) {
                throw new DataLoaderException(
                        "An error occurred while loading the vStorage status for the filer " + filer.getName() + ".", e);
            }
        }
        logger.debug("Filer vStorage Status Load: end, result = " + result);
        return result;
    }
}
