package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.GlobalStatus;
import com.netapp.offtap3.internal.ontap.controller.PFilerImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.Set;


public class GlobalStatusDataLoader extends AbstractDataLoader<PFilerImpl> {

    private static Logger logger = Logger.getLogger(GlobalStatusDataLoader.class);

    @Override
    protected boolean doLoad(PFilerImpl filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {

        boolean success = true;
        GlobalStatus globalStatus = new GlobalStatus();

        logger.debug(filer + " start");
        NaElement xi = new NaElement("snmp-get");
        NaElement xo = null;
        xi.addNewChild("object-id", ".1.3.6.1.4.1.789.1.2.2.4.0");
        /*
         * SNMP OID that represents the status
         * iso org dod internet private enterprises netapp netapp1 sysStat misc miscGlobalStatus 0
         *  1   3   6     1        4         1        789     1       2      2         4         0
         */

        try {
            xo = filer.invokeZapi(xi, ZapiCache.NO_CACHE);
        } catch (ApiFailedException e) {
            logger.warn(filer + " failed to get snmp for " + xi + " to assign global status code", e);
            success = false;
        }

        if (xo != null){
            int code = xo.getChildIntValue("value", 0);
            globalStatus.setCode(code);
            logger.debug(filer + " set status to " + code);
        }

        xi = new NaElement("snmp-get");
        xi.addNewChild("object-id", ".1.3.6.1.4.1.789.1.2.2.25.0");
        /*
         * SNMP OID that represents the description of the status
         * iso org dod internet private enterprises netapp netapp1 sysStat misc miscGlobalStatusMessage 0
         *  1   3   6     1        4         1        789     1       2      2            25            0
         */

        try {
            xo = filer.invokeZapi(xi, ZapiCache.NO_CACHE);
        } catch (ApiFailedException e) {
            logger.warn(filer + " failed to get snmp for " + xi + " to assign global status code", e);
            success = false;
        }
        if (xo != null){
            String message = xo.getChildContent("value");
            globalStatus.setMessage(message);
            logger.debug(filer + " set status reason to " + message);
        }

        filer.setGlobalStatus(globalStatus);
        return success;
    }
}
