package com.netapp.offtap3.internal.ontap.dataloader;


import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.Size;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.SpaceUsageImpl;
import com.netapp.offtap3.internal.ontap.controller.PFilerImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.NaErrno;

import org.apache.log4j.Logger;

import java.util.EnumSet;
import java.util.List;
import java.util.Set;

public class PFilerSpaceUsageDataLoader extends AbstractDataLoader<PFilerImpl> implements HasResourceDependencies {

    private static final Logger LOGGER = Logger.getLogger(PFilerSpaceUsageDataLoader.class);

    @Override
    public java.util.Set<ControllerResource> getResources(Refreshable refreshable) {
        // need aggregate information to complete the space usage calculations
        return EnumSet.of(ControllerResource.aggregates);
    }

    @Override
    protected boolean doLoad(PFilerImpl filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {

        // Is a physical filer.  Calculate total physical space.
        LOGGER.debug("populateSpaceTotals: start " + filer);
        NaElement result;

        try {
            result = filer.invokeZapi(new NaElement("disk-list-info"), ZapiCache.LONG_CACHE);
            if (result != null){
                List<NaElement> disksInfo = result.getChildByName("disk-details").getChildren();
                if (!disksInfo.isEmpty()) {
                    long capacity = 0;
                    for (NaElement diskInfo : disksInfo) {
                        // Only include PRESENT disks; skip PARTNER and SPARE disks in the calculation
                        if (diskInfo.getChildContent("raid-state").equals("present") && diskInfo.getChildContent("raid-type").equals("data")) {
                            capacity += diskInfo.getChildLongValue("physical-space", 0L);
                        }
                    }
                    if (capacity > 0) {
                        SpaceUsageImpl spaceUsage = filer.getSpaceUsage();
                        spaceUsage.setCapacity(Size.createB(capacity));
                        spaceUsage.setFree(spaceUsage.getCapacity().minus(spaceUsage.getUsed()));
                    }

                }else{
                    LOGGER.error("populateSpaceTotals: " + filer + " no disks reported!");
                }
            }
        } catch (ApiFailedException e) {
            String message = "populateSpaceTotals: " +filer +" Failed to get space totals. " + e.getMessage();
            if (e.getErrno() == NaErrno.EINTERNALERROR){
                LOGGER.warn(message);
            }else{
                LOGGER.error(message);
            }
        }
        LOGGER.debug("populateSpaceTotals: done " + filer + " TotalCapacity=" + filer.getSpaceUsage().getCapacity());

        return true;
    }
}
