package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapController;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.HasPerformanceObjects;
import com.netapp.offtap3.ontap.OntapController;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


public abstract class PerformanceObjectsLoaderDelegate implements HasPerformanceObjects {

    private final AbstractOntapController controller;

    PerformanceObjectsLoaderDelegate(AbstractOntapController controller) {
        this.controller = controller;
    }

    protected abstract String getZapiRequest();

    protected OntapController getController() {
        return controller;
    }

    @Override
    public List<String> getPerformanceObjectInstanceList(String objectName) throws DataLoaderException {
        List<String> performanceList = new ArrayList<String>();
        NaElement xi = new NaElement(getZapiRequest());

        xi.addNewChild("objectname", objectName);
        try {
            NaElement xo = controller.invokeZapi(xi, ZapiCache.LONG_CACHE);
            if (xo.getChildren() != null) {
                for (NaElement child : xo.getChildren()) {
                    for (NaElement child2 : child.getChildren()) {
                        performanceList.add(child2.getChildContent("name"));
                    }
                }
            }
        } catch (ApiFailedException e) {
            throw new DataLoaderException(controller + " failed to load object named '" + objectName + "'", e);
        }

        return performanceList;

    }

    @Override
    public Map<String, Long> getPerformanceCounters(String counter, String objectName, List<String> objectInstances) throws DataLoaderException {
        Map<String, Long> retVal = new SafeConcurrentHashMap<String, Long>();
        NaElement xi = new NaElement("perf-object-get-instances");
        xi.addNewChild("objectname", objectName);
        NaElement counters = new NaElement("counters");
        xi.addChildElem(counters);
        counters.addNewChild("counter", counter);

        NaElement instancesToSend = new NaElement("instances");
        xi.addChildElem(instancesToSend);
        for (String objectInstance : objectInstances) {
            instancesToSend.addNewChild("instance", objectInstance);
        }
        try {
            NaElement xo = controller.invokeZapi(xi, ZapiCache.LONG_CACHE);
            NaElement instances = xo.getChildByName("instances");
            if (instances != null) {
                long theVal = 0;
                for (NaElement instanceData : instances.getChildren()) {
                    NaElement countersElement = instanceData.getChildByName("counters");
                    if (countersElement != null) {
                        NaElement counterData = countersElement.getChildByName("counter-data");
                        if (counterData != null) {
                            theVal = counterData.getChildLongValue("value", 0);
                        }
                    }
                    String currObjectName = instanceData.getChildContent("name");
                    if (currObjectName != null) {
                        retVal.put(currObjectName, theVal);
                    }
                }
            }
        } catch (ApiFailedException e) {
            throw new DataLoaderException(controller + " failed to load performance stats for " + objectName + " and list " + objectInstances, e);
        }
        return retVal;
    }
}
