package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.ontap.OntapVersionImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.utils.OntapVersionGuesser;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.NaErrno;
import org.apache.log4j.Logger;

/**
 * Creates the NtapVersion instance that represents the target controller.
 */
public class SevenModeVersionBuilder {
    private static Logger logger = Logger.getLogger(SevenModeVersionBuilder.class);

    private OntapVersionGuesser versionGuesser = new OntapVersionGuesser();

    public OntapVersionImpl build(NaElement systemVersionXml, InternalZapiHandler unknownController) throws DataLoaderException {

        String rawVersion = systemVersionXml.getChildContent("version");

        // attempt to parse raw string into x.y.z format
        String guessedVersion = versionGuesser.fromVersionString(rawVersion);
        if (guessedVersion == null) {
            logger.warn("Failed to determine version from '" + unknownController + "'.  Reading system-ontapi-version...");
            guessedVersion = guessFromOntapiVersion(unknownController);
        }

        if (guessedVersion == null) {
            logger.error("Failed to determine version for 7-mode filer (" + unknownController + ") with version string: " + rawVersion);
            throw new DataLoaderException("Failed to determine version for 7-mode filer (" + unknownController + ") with version string: " + rawVersion);
        }

        logger.info("versionBuilder: " + unknownController + " assigning version of '" + guessedVersion + "'");
        return new OntapVersionImpl(rawVersion, guessedVersion.split("\\."));
    }

    private String guessFromOntapiVersion(InternalZapiHandler unknownController) throws DataLoaderException {
        int ontapiMinorVersion = getONTAPIMinorVersion(unknownController);
        String guessedVersion = versionGuesser.fromOntapiVersion(ontapiMinorVersion);
        logger.warn("Guessed ONTAP version as " + guessedVersion + " given ontapi minor version of " +
                ontapiMinorVersion);
        return guessedVersion;
    }

    int getONTAPIMinorVersion(InternalZapiHandler unknownController) throws DataLoaderException  {
        NaElement xi = new NaElement("system-get-ontapi-version");
        NaElement xo;

        try {
            xo = unknownController.invokeZapi(xi, ZapiCache.LONG_CACHE);
            int minorVersion = Integer.parseInt(xo.getChildContent("minor-version"));

            logger.debug("getONTAPIMinorVersion: ONTAPI minor version found to be: " + minorVersion);
            return minorVersion;
        } catch (ApiFailedException e) {
            if (e.getErrno() == NaErrno.EAPIPRIVILEGE) {
                String errorMsg = "getONTAPIMinorVersion: " + unknownController +
                        " insufficient privileges (system-get-ontapi-version)." + e.getMessage();
                logger.error(errorMsg);
                throw new DataLoaderException(errorMsg);
            } else {
                String errorMsg = "getONTAPIMinorVersion: " + unknownController + " unspecified error.";
                logger.error(errorMsg);
                throw new DataLoaderException(errorMsg);
            }
        }
    }

}
