package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.zapi.SevenModeZapiAccessPermissions;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.VFiler;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.NaErrno;

import org.apache.log4j.Logger;

import java.util.HashSet;
import java.util.List;
import java.util.Set;


public class SevenModeZapiAccessPermissionsDataLoader extends AbstractDataLoader<FilerImpl> {

    private static final Logger logger = Logger.getLogger(SevenModeZapiAccessPermissionsDataLoader.class);

    @Override
    protected boolean doLoad(FilerImpl filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        if ((filer instanceof VFiler) && !filer.isDirectVFiler()) {
            // don't load permissions for pass-through vfilers
            return true;
        }
        try {
            Set<String> zapis = getAvailableZapis(filer);
            SevenModeZapiAccessPermissions perms = new SevenModeZapiAccessPermissions();
            perms.allowedZapis(zapis);
            filer.setZapiAccessPermissions(perms);
        } catch (ApiFailedException e) {
            if (e.getErrno() == NaErrno.EAPIPRIVILEGE) {
                logger.error("getZapiAccessInformation: " + filer +
                        " insufficient privileges (useradmin-user-list)." + e.getMessage());
            } else if (e.getErrno() == NaErrno.EINTERNALERROR &&
                    e.getMessage().contains("contains an invalid character") &&
                    filer.getConnection().getUserName().contains("\\")) {
                logger.error("getZapiAccessInformation: " + filer +
                        "useradmin-user-list does not support domain credentials" + e.getMessage());
            }
            throw new DataLoaderException(filer + " failed to load ZAPI access permissions: " + e, e);
        }
        return true;
    }

    Set<String> getAvailableZapis(FilerImpl filer) throws ApiFailedException {
        Set<String> availableZapis = new HashSet<String>();
        String userName = filer.getConnection().getUserName();
        if ("root".equals(userName)) {
            availableZapis.add("*");
        } else {
            NaElement xi = new NaElement("useradmin-user-list");
            xi.addNewChild("user-name", userName);
            xi.addNewChild("verbose", "true");   // verbose is need so that the allowed-capabilities will be returned.

            NaElement xo = filer.invokeZapi(xi, ZapiCache.NO_CACHE);
            NaElement userInfo = xo.getChildByName("useradmin-users").getChildByName("useradmin-user-info");
            List<NaElement> capabilities = userInfo.getChildByName("allowed-capabilities").getChildren();
            for (NaElement capability : capabilities) {
                availableZapis.add(capability.getChildContent("name"));
            }
        }
        return availableZapis;
    }

}
