package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.volume.FilerVolumeImpl;
import com.netapp.offtap3.internal.ontap.snap.SnapVaultScheduleImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.SnapVaultSchedule;
import com.netapp.offtap3.ontap.NaElement;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;


public class SnapVaultScheduleDataLoader extends AbstractDataLoader<FilerVolumeImpl> {

    @Override
    protected Refreshable doRefresh(FilerVolumeImpl loadTarget, Resource resource, Refreshable refreshTarget) {
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource));
        return getFromList(loadTarget.getSnapVaultSchedules(), refreshTarget.getName());
    }

    @Override
    protected boolean doLoad(FilerVolumeImpl volume, LoadType loadType, Set<Resource> resources) throws DataLoaderException {

        NaElement xi = new NaElement("snapvault-primary-snapshot-schedule-list-info");
        xi.addNewChild("volume-name", volume.getName());
        NaElement xo;

        try {
            xo = volume.getControllerImpl().invokeZapi(xi, ZapiCache.LONG_CACHE);
        } catch (ApiFailedException ne) {
            if (ne.getErrno() == 13008) {
                return false;
            } else {
                throw new DataLoaderException(volume.getControllerImpl() +
                        " failed to get snap vault schedules for " + volume.getName(), ne);
            }
        }

        if (xo.getChildByName("snapshot-schedules") != null){
            List<NaElement> pps = xo.getChildByName("snapshot-schedules").getChildren();
            List<SnapVaultSchedule> schedules = new ArrayList<SnapVaultSchedule>();
            for (final NaElement snapvaultSchedule : pps) {
                String scheduleName = snapvaultSchedule.getChildContent("schedule-name");
            	SnapVaultScheduleImpl schedule = new SnapVaultScheduleImpl(scheduleName, volume, SnapVaultSchedule.Relation.primary);
            	schedule.setRetentionCount(snapvaultSchedule.getChildLongValue("retention-count", 0));
            	NaElement scheduleElement = snapvaultSchedule.getChildByName("schedule");
            	if (scheduleElement != null) {
            		NaElement scheduleInfoElement = scheduleElement.getChildByName("snapvault-schedule-info");
            		if (scheduleInfoElement != null) {
            			schedule.setDaysOfWeek(scheduleInfoElement.getChildContent("days-of-week"));
            			schedule.setHoursOfDay(scheduleInfoElement.getChildContent("hours-of-day"));
            		}
            	}
                schedules.add(schedule);
            }
            volume.setSnapVaultSchedules(schedules);
        }
        return true;
    }
}
