package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.ControllerWithSnmpSupport;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;

import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.HashSet;
import java.util.Set;

public class SnmpHostDataLoader extends AbstractDataLoader<ControllerWithSnmpSupport> {

    private final OfftapPreferences prefs;

    private static final Logger logger = Logger.getLogger(SnmpHostDataLoader.class);

    public SnmpHostDataLoader(OfftapPreferences prefs) {
        this.prefs = prefs;
    }

    @Override
    protected boolean doLoad(ControllerWithSnmpSupport controller, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        Set<String> hosts = new HashSet<String>();
        try {
            logger.info("Getting snmp on " + controller);
            NaElement xo = controller.invokeZapi(new NaElement("snmp-status"), ZapiCache.LONG_CACHE);
            String isEnabled = xo.getChildContent("is-trap-enabled");
            if (!"true".equals(isEnabled) && prefs.getSnmpLoadThrowsExceptionWhenDisabled()) {
                throw new ApiFailedException(911, "SNMP traps not enabled");
            }
            NaElement traphosts = xo.getChildByName("traphosts");
            if (traphosts != null && traphosts.getChildren() != null) {
                for (Object child : traphosts.getChildren()) {
                    String ipContent = ((NaElement) child).getChildContent("ip-address");
                    hosts.add(ipContent);
                }
            }
            logger.debug(controller + " had " + hosts.size() + " SNMP hosts.");
            controller.setSnmpHosts(hosts);
        } catch (ApiFailedException ex) {
            logger.error(controller + " failed to get snmp", ex);
            throw new DataLoaderException(controller + " failed to get SNMP, Error: " + ex.getMessage(), ex);
        }
        return true;
    }
}
