/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/SvmAggregateDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.Size;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.aggregate.AggregateImpl;
import com.netapp.offtap3.internal.ontap.aggregate.CModeAggregateImpl;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapController;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.utils.StatsCollector;
import com.netapp.offtap3.ontap.AdminSvm;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.CModeAggregate;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.NodeSvm;
import com.netapp.offtap3.ontap.OntapController;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;
import com.netapp.offtap3.utils.StringUtils;

public class SvmAggregateDataLoader extends OntapCachingDataLoader<AbstractOntapController<? extends OntapController>> implements HasResourceDependencies {
    private static final Logger logger = Logger.getLogger(SvmAggregateDataLoader.class);
    private OfftapPreferences prefs;

    public SvmAggregateDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);
        this.prefs = prefs;
    }

    private NaElement getIterativeAggrInput(Refreshable reloadTarget) {
        NaElement input = new NaElement("aggr-get-iter");
        if (reloadTarget != null) {
            NaElement query = new NaElement("query");
            NaElement aggrAttributes = new NaElement("aggr-attributes");
            aggrAttributes.addNewChild("aggregate-name", reloadTarget.getName());
            query.addChildElem(aggrAttributes);
            input.addChildElem(query);
            StatsCollector.getInstance().addCounter("refresh-targeted-aggr");
        } else {
        	StatsCollector.getInstance().addCounter("refresh-whole-aggr");
        }
        NaElement desiredAttributes = new NaElement("desired-attributes");
        NaElement desiredAggrAttributes = new NaElement("aggr-attributes");
        desiredAggrAttributes.addChildElem(new NaElement("aggregate-name"));
        desiredAggrAttributes.addChildElem(new NaElement("aggregate-uuid"));
        desiredAggrAttributes.addChildElem(new NaElement("striping-type"));
        desiredAggrAttributes.addChildElem(new NaElement("aggr-ownership-attributes"));

        NaElement aggrRaidAttributes = new NaElement("aggr-raid-attributes");
        aggrRaidAttributes.addChildElem(new NaElement("state"));
        aggrRaidAttributes.addChildElem(new NaElement("disk-count"));
        aggrRaidAttributes.addChildElem(new NaElement("is-hybrid"));
        aggrRaidAttributes.addChildElem(new NaElement("is-hybrid-enabled"));
        aggrRaidAttributes.addChildElem(new NaElement("is-root-aggregate"));
        desiredAggrAttributes.addChildElem(aggrRaidAttributes);

        NaElement aggrFsAttributes = new NaElement("aggr-fs-attributes");
        aggrFsAttributes.addChildElem(new NaElement("block-type"));
        desiredAggrAttributes.addChildElem(aggrFsAttributes);
        desiredAggrAttributes.addChildElem(new NaElement("aggr-space-attributes"));
        desiredAttributes.addChildElem(desiredAggrAttributes);

        input.addChildElem(desiredAttributes);
        return input;
    }

    private NaElement getIterativeDiskInput() {
        NaElement input = new NaElement("storage-disk-get-iter");

        NaElement desiredAttributes = new NaElement("desired-attributes");
        NaElement desiredStorageDiskInfoAttributes = new NaElement("storage-disk-info");
        NaElement desiredDiskInventoryInfoAttributes = new NaElement("disk-inventory-info");
        NaElement desiredDiskTypeAttribute = new NaElement("disk-type");
        NaElement desiredDiskRaidInfoAttributes = new NaElement("disk-raid-info");
        
        NaElement desiredDiskAggregateInfoAttributes = new NaElement("disk-aggregate-info");
        NaElement desiredDiskAggregateNameAttribute = new NaElement("aggregate-name");
        
        NaElement desiredDiskSharedInfoAttributes = new NaElement("disk-shared-info");
        NaElement desiredDiskSharedInfoAggrListAttributes = new NaElement("aggregate-list");           

        desiredDiskInventoryInfoAttributes.addChildElem(desiredDiskTypeAttribute);                
        
        desiredDiskSharedInfoAttributes.addChildElem(desiredDiskSharedInfoAggrListAttributes);        
        desiredDiskRaidInfoAttributes.addChildElem(desiredDiskSharedInfoAttributes);
        
        desiredDiskAggregateInfoAttributes.addChildElem(desiredDiskAggregateNameAttribute); 
        desiredDiskRaidInfoAttributes.addChildElem(desiredDiskAggregateInfoAttributes);
        
        NaElement effectiveDiskType = new NaElement("effective-disk-type");
        desiredDiskRaidInfoAttributes.addChildElem(effectiveDiskType);
        
        desiredStorageDiskInfoAttributes.addChildElem(desiredDiskInventoryInfoAttributes);
        desiredStorageDiskInfoAttributes.addChildElem(desiredDiskRaidInfoAttributes);        
       
        desiredAttributes.addChildElem(desiredStorageDiskInfoAttributes);        
        input.addChildElem(desiredAttributes);
        return input;
    }

    @Override
    protected Map<Resource, NaElement> retrieve(AbstractOntapController<? extends OntapController> object, LoadType loadType, Set<Resource> resources) {
        return retrieve(object, loadType, resources, null);
    }

    @Override
    protected NaElement retrieve(AbstractOntapController loadTarget, Resource resource, Refreshable refreshable) {
        Map<Resource, NaElement> dataMap = retrieve(loadTarget, LoadType.refresh, Collections.singleton(resource),
                                                    refreshable);
        return dataMap.get(resource);
    }

    @SuppressWarnings("unused")
    protected Map<Resource, NaElement> retrieve(AbstractOntapController object, LoadType loadType, Set<Resource> resources, Refreshable reloadTarget) {
        logger.debug("retrieve: start");
        NaElement aggrResult = new NaElement("aggr");
        NaElement diskResult = new NaElement("disk");

        CdotIterativeLoader<NaElement> iterativeAggrLoader = new CdotIterativeLoader<NaElement>(getDataCache().getDataRetriever(), getIterativeAggrInput(reloadTarget),
                                                                                       new CumulativeDataParser(), prefs.getIteratorSize(),
                                                                                       ZapiCache.SHORT_CACHE);

        boolean loadDisksToo = true;
        if (reloadTarget != null) {
        	// loading disks is really quite expensive, and traditional aeonflux was coded to not do this expensive disk load if we were reloading a 
        	// single aggregate.  So we'll maintain that behavior here.
        	loadDisksToo = false;
        }
        
        CdotIterativeLoader<NaElement> iterativeDiskLoader = null;        
        if (loadDisksToo) {
	
	        // Apparently this zapi blows chunks if it's too big.
	        long iteratorSize = Math.min(prefs.getIteratorSize(), 2000);
	        iterativeDiskLoader = new CdotIterativeLoader<NaElement>(getDataCache().getDataRetriever(), getIterativeDiskInput(),
	                                                                 new CumulativeDataParser(), iteratorSize,
	                                                                 ZapiCache.SHORT_CACHE);
        }
        // If we're not in an admin SVM, don't bother trying to get the aggregate information, it won't be accessible.
        if (getDataCache().getDataRetriever() instanceof AdminSvm) {
            try {
                iterativeAggrLoader.load(aggrResult);
                if (iterativeDiskLoader != null) {
                	iterativeDiskLoader.load(diskResult);
                }
            } catch (ApiFailedException e) {
                throw new DataLoaderException(e);
            }
        }

        NaElement result = new NaElement("results");
        result.addChildElem(aggrResult);
        result.addChildElem(diskResult);

        logger.debug("retrieve: end");
        return Collections.singletonMap(resources.iterator().next(), result);
    }

    @Override
    protected Refreshable parse(AbstractOntapController<? extends OntapController> loadTarget, NaElement data, Resource resource, Refreshable reloadTarget) {
        doParse(loadTarget, data, resource, reloadTarget);
        return loadTarget.getAggregates().get(reloadTarget.getName());
    }

    @Override
    protected boolean parse(AbstractOntapController<? extends OntapController> svm, NaElement data, Resource resource) {
        return doParse(svm, data, resource, null);
    }

    @SuppressWarnings("unused")
    protected boolean doParse(AbstractOntapController<? extends OntapController> svm, NaElement data, Resource resource, Refreshable reloadTarget) {
        logger.debug("parse: begin");
        Map<String, AggregateImpl> aggrImpls = addAggregates(svm, data.getChildByName("aggr"), reloadTarget);
        boolean result = aggrImpls != null;
        if (result) {
            result = addDiskTypes(aggrImpls, data.getChildByName("disk"));
        }
        logger.debug("parse: end - " + result);
        return result;
    }

    private Map<String, AggregateImpl> addAggregates(AbstractOntapController<? extends OntapController> svm, NaElement data, Refreshable reloadTarget) {    	
    	
        Map<String, AggregateImpl> aggrImpls = new SafeConcurrentHashMap<String, AggregateImpl>();
        Map<String, CModeAggregate> aggrs = new SafeConcurrentHashMap<String, CModeAggregate>();
        logger.debug("addAggregates called with reloadTarget of " + reloadTarget);
        if (reloadTarget != null) {
            // Since we're reloading, initialize the aggregates with the existing set minus the one we're trying to reload.
            @SuppressWarnings("unchecked")
            Map<String, CModeAggregate> tempAggrMap = (Map<String, CModeAggregate>) svm.getAggregates();
            aggrs.putAll(tempAggrMap);
            aggrs.remove(reloadTarget.getName());
        }

        long tempAggrAllocated = 0L;
        long tempAggrUsed = 0L;
        long tempAggrFree = 0L;

        if (data.getChildren().size() == 0) {
            logger.warn("Attributes list was empty.");
            return Collections.emptyMap();
        }

        Map<String, CModeAggregate> aggregateList;
        if (svm instanceof DataSvmImpl) {
            aggregateList = ((DataSvmImpl)svm).getAggregateList();
        } else {
            aggregateList = Collections.emptyMap();
        }

        List<NaElement> aggrsAttributes = data.getChildren();
        for (NaElement aggrAttributes : aggrsAttributes) {
            String name = aggrAttributes.getChildContent("aggregate-name");

            if (!aggregateList.isEmpty() && !aggregateList.containsKey(name)) {
                // The current svm has access to only a subset of the aggregates and the current aggregate was not in that list.
                continue;
            }

            String uuid = aggrAttributes.getChildContent("aggregate-uuid");
            String stripingType = aggrAttributes.getChildContent("striping-type");

            NaElement raidAttributes = aggrAttributes.getChildByName("aggr-raid-attributes");
            String state = null;
            Long diskCount = 0L;
            boolean isHybrid = false;
            if (raidAttributes != null) {
                state = raidAttributes.getChildContent("state");
                String diskCountString = raidAttributes.getChildContent("disk-count");
                isHybrid = "true".equals(raidAttributes.getChildContent("is-hybrid")) && "true".equals(
                        raidAttributes.getChildContent("is-hybrid-enabled"));
                if (StringUtils.isNumeric(diskCountString)) {
                    diskCount = Long.parseLong(diskCountString);
                }
            }

            NaElement fsAttributes = aggrAttributes.getChildByName("aggr-fs-attributes");
            String blockType = "";
            if (fsAttributes != null) {
                blockType = fsAttributes.getChildContent("block-type");
            }

            NaElement spaceAttributes = aggrAttributes.getChildByName("aggr-space-attributes");
            long sizeUsed = 0L;
            long sizeTotal = 0L;
            long sizeAvailable = 0L;
            int percentageUsed = 0;
            if (spaceAttributes != null) {
                sizeUsed = spaceAttributes.getChildLongValue("size-used", 0L);
                percentageUsed = spaceAttributes.getChildIntValue("percent-used-capacity", 0);
                sizeTotal = spaceAttributes.getChildLongValue("size-total", 0L);
                sizeAvailable = spaceAttributes.getChildLongValue("size-available", 0L);

                tempAggrAllocated += sizeTotal;
                tempAggrUsed += sizeUsed;
            }

            String homeNode = null;
            String currNode = null;
            NaElement ownershipAttributes = aggrAttributes.getChildByName("aggr-ownership-attributes");
            if (ownershipAttributes != null) {
                homeNode = ownershipAttributes.getChildContent("home-name");
                currNode = ownershipAttributes.getChildContent("owner-name");
            }

            if (svm instanceof NodeSvm && !svm.getName().equals(currNode)) {
                // If we're populating a node svm, only add the aggregates that are currently on that node.
                continue;
            }

            CModeAggregateImpl aggr = new CModeAggregateImpl(svm, name, uuid, state, sizeUsed, percentageUsed,
                                                             sizeTotal, sizeAvailable, new ArrayList<String>(),
                                                             diskCount);
            logger.debug("addAggregates-For svm of " + svm.getName() + " got aggr back of " + name);
            aggr.setStriped((stripingType != null) && "striped".equals(stripingType));
            aggr.setStripingType(stripingType);
            aggr.setBlockType(blockType);
            aggr.setHomeNodeName(homeNode);
            aggr.setCurrNodeName(currNode);
            if (raidAttributes != null) {
            	// note - only shows up on fullsteam
            	aggr.setRoot("true".equals(raidAttributes.getChildContent("is-root-aggregate")));
            }
            aggr.setLastLoad(new Date());
            aggr.setHybrid(isHybrid);
            aggrs.put(name, aggr);
            aggrImpls.put(name, aggr);
        }

        if (svm.getSpaceUsage().getCapacity().toBytes() > 0) {
            tempAggrFree = svm.getSpaceUsage().getCapacity().toBytes() - tempAggrUsed;
        }

        // Set free space calcs here.
        svm.getSpaceUsage().setAllocated(Size.createB(tempAggrAllocated));
        svm.getSpaceUsage().setUsed(Size.createB(tempAggrUsed));
        svm.getSpaceUsage().setFree(Size.createB(tempAggrFree));

        logger.debug("addAggregates-Finished loading " + svm.getName() + " have " + aggrs.size() + " entries");
        for (String key : aggrs.keySet()) {
        	logger.debug("addAggregates-For svm of " + svm.getName() + " got aggr back of " + key + " which was " + aggrs.get(key).getName());
        }
        
        svm.setAggregates(aggrs);
        //TODO tdeepak see burt 883803, Seems to me that we should also update the list of aggrs associated with each vserver(dataSvm) on an aggr refresh.
      
        return aggrImpls;
    }

    private boolean addDiskTypes(Map<String, AggregateImpl> aggregates, NaElement data) {
        Map<String, Set<String>> diskTypesByAggregate = new SafeConcurrentHashMap<String, Set<String>>();

        for (NaElement diskInfos : data.getChildren()) {
            String diskType = null;
            Set<String> localAggrNames = new HashSet<String>();
            NaElement inventoryInfo = diskInfos.getChildByName("disk-inventory-info");
            if (inventoryInfo != null) {
                diskType = inventoryInfo.getChildContent("disk-type");
            }
            NaElement diskRaidInfo = diskInfos.getChildByName("disk-raid-info");
            if (diskRaidInfo != null) {
                NaElement diskAggrInfo = diskRaidInfo.getChildByName("disk-aggregate-info");
                if (diskAggrInfo != null) {
                    String aggrName = diskAggrInfo.getChildContent("aggregate-name");
                    if(aggrName!=null){
                    	localAggrNames.add(aggrName);
                    }                    
                }
                
                /* tdeepak: see burt 879084. With AFF, Ontap supports per disk partitioning, i.e SSD disk can belong to 2 aggrs*/                
                if(localAggrNames.size()==0){
                	 NaElement diskSharedInfo = diskRaidInfo.getChildByName("disk-shared-info");
                	 if(diskSharedInfo!=null){
                		 NaElement aggrList = diskSharedInfo.getChildByName("aggregate-list");
                		 if(aggrList!=null){
                			 for(NaElement sharedAggregateInfo : aggrList.getChildren()){
                    			 String aggrName =  sharedAggregateInfo.getChildContent("aggregate-name");
                    			 localAggrNames.add(aggrName);
                    		 }
                		 }                		 
                	 }
                }                  
                
                NaElement effectiveType = diskRaidInfo.getChildByName("effective-disk-type");
                if (effectiveType != null && StringUtils.isReal(effectiveType.getContent())) {
                    diskType = effectiveType.getContent();
                }
            }
            
            for(String localAggrName : localAggrNames){
            	if (localAggrName != null && diskType != null) {
                    Set<String> aggregateDiskTypes = diskTypesByAggregate.get(localAggrName);
                    if (aggregateDiskTypes == null) {
                        aggregateDiskTypes = new HashSet<String>();
                        diskTypesByAggregate.put(localAggrName, aggregateDiskTypes);
                    }
                    aggregateDiskTypes.add(diskType);
                }
            }            
        }

        for (Map.Entry<String, Set<String>> entry : diskTypesByAggregate.entrySet()) {
            AggregateImpl aggregate = aggregates.get(entry.getKey());
            if (aggregate != null) {
                Set<String> diskTypes = entry.getValue();
                aggregate.setDiskTypes(diskTypes);
            }
        }

        return true;
    }

    @Override
    public java.util.Set<ControllerResource> getResources(Refreshable refreshable) {
   		return Collections.emptySet();
    }

}
