/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/SvmAggregateListDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.internal.ontap.aggregate.CModeAggregateImpl;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapController;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.CModeAggregate;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class SvmAggregateListDataLoader extends OntapCachingDataLoader<AbstractOntapController> {
    private static final Logger logger = Logger.getLogger(SvmAggregateListDataLoader.class);
    private OfftapPreferences prefs;
    private CdotIterativeLoader<NaElement> iterativeLoader;

    public SvmAggregateListDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);
        this.prefs = prefs;
        NaElement input = new NaElement("vserver-get-iter");
        NaElement desiredAttributes = new NaElement("desired-attributes");
        NaElement desiredVServerInfo = new NaElement("vserver-info");
        desiredVServerInfo.addChildElem(new NaElement("vserver-name"));

        NaElement desiredAggrListInfo = new NaElement("vserver-aggr-info-list");
        desiredAggrListInfo.addChildElem(new NaElement("aggr-availsize"));
        desiredAggrListInfo.addChildElem(new NaElement("aggr-name"));

        desiredVServerInfo.addChildElem(desiredAggrListInfo);
        desiredAttributes.addChildElem(desiredVServerInfo);
        input.addChildElem(desiredAttributes);
        iterativeLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(), input,
                                                             new CumulativeDataParser(), prefs.getIteratorSize(),
                                                             ZapiCache.LONG_CACHE);
    }

    @Override
    protected Map<Resource, NaElement> retrieve(AbstractOntapController object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("retrieve: start");
        NaElement result = new NaElement("results");
        try {
            iterativeLoader.load(result);
        } catch (ApiFailedException e) {
            throw new DataLoaderException(e);
        }
        logger.debug("retrieve: end");
        return Collections.singletonMap(resources.iterator().next(), result);
    }

    @Override
    protected boolean parse(AbstractOntapController svm, NaElement data, Resource resource) {
        logger.debug("parse: start");
        if (svm instanceof DataSvmImpl) {
            DataSvmImpl dataSvm = (DataSvmImpl) svm;
            List<NaElement> svmInfoList = data.getChildren();
            Map<String, CModeAggregate> aggrs = new SafeConcurrentHashMap<String, CModeAggregate>();
            for (NaElement svmInfo : svmInfoList) {
                if (MCCManager.getInstance().getVServerNameByUuid(dataSvm).equals(svmInfo.getChildContent("vserver-name"))) {
                    NaElement aggrInfoList = svmInfo.getChildByName("vserver-aggr-info-list");
                    if (aggrInfoList != null) {
                        for (NaElement aggrInfo : aggrInfoList.getChildren()) {
                            String name = aggrInfo.getChildContent("aggr-name");
                            long sizeAvailable = aggrInfo.getChildLongValue("aggr-availsize", 0L);
                            CModeAggregateImpl aggr = new CModeAggregateImpl(dataSvm, name, sizeAvailable);
                            aggrs.put(name, aggr);
                        }
                    }
                }
            }
            dataSvm.setAggregateList(aggrs);
        }
        logger.debug("parse: end");
        return true;
    }
}
