package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.internal.ontap.SvmPeerImpl;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.OntapController;
import com.netapp.offtap3.ontap.SvmPeer;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class SvmPeerDataLoader extends OntapCachingDataLoader<DataSvmImpl> {

    private static final Logger logger = Logger.getLogger(SvmPeerDataLoader.class);

    private CdotIterativeLoader<NaElement> iterativeLoader;
    private boolean isAtLeastSN = false;

    public SvmPeerDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);
        iterativeLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(), new NaElement("vserver-peer-get-iter"),
                new CumulativeDataParser(), prefs.getIteratorSize(), ZapiCache.SHORT_CACHE);
        isAtLeastSN = ((OntapController)dataCache.getDataRetriever()).getVersion().atLeastSN();
    }

    @Override
    protected Map<Resource, NaElement> retrieve(DataSvmImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("retrieve: start");

        if (!isAtLeastSN) {
            return Collections.EMPTY_MAP;
        }

        NaElement result = new NaElement("results");
        try {
            iterativeLoader.load(result);
        } catch (ApiFailedException e) {
            throw new DataLoaderException(e);
        }

        logger.debug("retrieve: end");
        return Collections.singletonMap(resources.iterator().next(), result);
    }

    @Override
    protected boolean parse(DataSvmImpl svm, NaElement data, Resource resource) {
        logger.debug("parse: begin");

        List<SvmPeer> svmPeers = new ArrayList<SvmPeer>();

        if (data != null) {
	        for (NaElement peerElem : data.getChildren()) {
	            String name = peerElem.getChildContent("vserver");
	            if (MCCManager.getInstance().getVServerNameByUuid(svm).equals(name)) {
	                SvmPeerImpl peer = new SvmPeerImpl();
	                peer.setClusterName(peerElem.getChildContent("peer-cluster"));
	                peer.setName(peerElem.getChildContent("peer-vserver"));
	                peer.setState(peerElem.getChildContent("peer-state"));
	                svmPeers.add(peer);
	            }
	        }
        }
        svm.setSvmPeers(svmPeers);
        return true;
    }

    @Override
    protected Refreshable parse(DataSvmImpl loadTarget, NaElement data, Resource resource, Refreshable reloadTarget) {
        return null;
    }
}
