package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.Size;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.SpaceUsageImpl;
import com.netapp.offtap3.internal.ontap.controller.VFilerImpl;
import com.netapp.offtap3.ontap.OntapVolume;

import java.util.EnumSet;
import java.util.Set;


public class VFilerSpaceUsageDataLoader extends AbstractDataLoader<VFilerImpl> implements HasResourceDependencies {

    @Override
    public java.util.Set<ControllerResource> getResources(Refreshable refreshable) {
        // can't compute space for the vfiler without all volume data populated
        return EnumSet.of(ControllerResource.volumes);
    }

    @Override
    protected boolean doLoad(VFilerImpl filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        Long totalAllocated = 0L;
        Long totalUsed = 0L;
        Long totalFree = 0L;
        for (OntapVolume vol : filer.getVolumes().values()){
            totalAllocated = totalAllocated + vol.getSizeTotal().toBytes();
            totalUsed = totalUsed + vol.getSizeUsed().toBytes();
            totalFree = totalFree + vol.getFreeSpace().toBytes();
        }
        SpaceUsageImpl usage = filer.getSpaceUsage();
        usage.setAllocated(Size.createB(totalAllocated));
        usage.setCapacity(usage.getAllocated());
        usage.setUsed(Size.createB(totalUsed));
        usage.setFree(Size.createB(totalFree));

        // this is cheap, so we can just execute every time
        return false;
    }
}
