package com.netapp.offtap3.internal.ontap.dataloader;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.VFilerImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;

import java.util.Set;

public class VFilerStatusDataLoader extends AbstractDataLoader<VFilerImpl> {

    @Override
    protected boolean doLoad(VFilerImpl vfiler, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        if (vfiler.isDirectVFiler()) {
            // well, if we're talking directly to the vfiler, then it must be running
            vfiler.setVFilerStatus("running");
            return true;
        }

        //run zapi call to get vFiler status
        NaElement xi = new NaElement("vfiler-get-status");
        NaElement vfilerTarget = new NaElement("vfiler", vfiler.getName());
        xi.addChildElem(vfilerTarget);
        try {
            // this ZAPI must be sent to the pfiler (throws an error if you tunnel to vfiler)
            NaElement xo = vfiler.getNonPassThroughConnection().invokeZapi(xi, ZapiCache.LONG_CACHE);
            vfiler.setVFilerStatus(xo.getChildContent("status"));
            return true;
        }
        catch (ApiFailedException e) {
            throw new DataLoaderException("Failed to get vfiler status", e);
        }
    }
}
