/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/file/FileMetadataDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader.file;

import com.netapp.offtap3.FileOperationException;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.Size;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.file.OntapFileImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.Set;

public class FileMetadataDataLoader extends AbstractDataLoader<OntapFileImpl> {

    private static final Logger logger = Logger.getLogger(FileMetadataDataLoader.class);

    @Override
    protected boolean doLoad(OntapFileImpl file, LoadType loadType,
                             Set<Resource> resources) throws DataLoaderException {
        logger.debug("load metadata: " + this);
        String path = file.getFullPath();
        NaElement xi;
        xi = new NaElement("file-get-file-info");
        xi.addNewChild("path", path);

        try {
            NaElement fileInfo = file.getControllerImpl().invokeZapi(xi, ZapiCache.SHORT_CACHE).getChildByName("file-info");
            String name = fileInfo.getChildContent("name");
            if (name == null) {
                // for unknown reasons, sometimes, ONTAP fails to return the name in the response.
                // In order for File to be consistent for the success path, fill it in for the caller.
                name = path;
            }

            int lastSlash = name.lastIndexOf("/");
            if (lastSlash > 0) {
                name = name.substring(lastSlash+1);
            }
            file.setName(name);

            file.setAclType(fileInfo.getChildContent("acl-type"));
            file.setFileType(fileInfo.getChildContent("file-type"));
            file.setAccessedTimestamp(fileInfo.getChildIntValue("accessed-timestamp", 0));
            file.setChangedTimestamp(fileInfo.getChildIntValue("changed-timestamp", 0));
            file.setCreationTimestamp(fileInfo.getChildIntValue("creation-timestamp", 0));
            file.setFileSize(Size.createB(fileInfo.getChildLongValue("file-size", 0)));
            file.setGroupID(fileInfo.getChildIntValue("group-id", 0));
            file.setHardLinksCount(fileInfo.getChildIntValue("hard-links-count", 0));
            file.setInodeNumber(fileInfo.getChildIntValue("inode-number", 0));
            file.setModifiedTimestamp(fileInfo.getChildIntValue("modified-timestamp", 0));
            file.setOwnerID(fileInfo.getChildIntValue("owner-id", 0));
            file.setPerm(fileInfo.getChildIntValue("perm", 0));
            file.setBytesUsed(fileInfo.getChildLongValue("bytes-used", 0));
            file.setExists(true);

            logger.debug("getFile (returning file with name): " +name);
        } catch (Exception e) {
            file.setExists(false);
            logger.debug("Failed to get file " + path + ". " +e.getMessage());
            throw new FileOperationException("Failed to get file " + path + ". " + e.getMessage(), e);
        }
        return true;
    }
}
