/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/lun/LunBindingsDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader.lun;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataParser;
import com.netapp.offtap3.internal.ontap.lun.LunBindingImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.LunBinding;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

public class LunBindingsDataLoader extends AbstractDataLoader<DataSvmImpl> {

    private static final Logger logger = Logger.getLogger(LunBindingsDataLoader.class);
    private OfftapPreferences prefs;

    public LunBindingsDataLoader(OfftapPreferences prefs) {
        this.prefs = prefs;
    }

    @Override
    protected boolean doLoad(DataSvmImpl dataSvm, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("doLoad: begin");
        NaElement input = new NaElement("lun-bind-get-iter");
        CdotIterativeLoader<List<LunBinding>> bindingsLoader = new CdotIterativeLoader<List<LunBinding>>(dataSvm, input,
                                                                                                         new LunBindingsParser(
                                                                                                                 dataSvm),
                                                                                                         prefs.getIteratorSize(),
                                                                                                         ZapiCache.SHORT_CACHE);
        List<LunBinding> lunBindings = new ArrayList<LunBinding>();
        try {
            bindingsLoader.load(lunBindings);
        } catch (Exception e) {
            logger.error("unable to get vvol lun bindings", e);
            throw new DataLoaderException("unable to get vvol lun bindings." + e.getMessage(), e);
        }
        dataSvm.setLunBindings(lunBindings);
        logger.debug("doLoad: end");
        return true;
    }

    @Override
    protected Refreshable doRefresh(DataSvmImpl loadTarget, Resource resource, Refreshable refreshTarget) {
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource));
        LunBinding reloadBinding = (LunBinding) refreshTarget;
        for (LunBinding binding : loadTarget.getLunBindings()) {
            if (binding.getVvolPath().equals(reloadBinding.getVvolPath()) &&
                    binding.getProtocolEndpointPath().equals(reloadBinding.getProtocolEndpointPath())) {
                return binding;
            }
        }
        return null;
    }

    private class LunBindingsParser implements DataParser<List<LunBinding>> {
        private DataSvmImpl dataSvm;

        public LunBindingsParser(DataSvmImpl dataSvm) {
            this.dataSvm = dataSvm;
        }

        @Override
        public void parse(List<LunBinding> cummulativeResults, NaElement results) {
            NaElement attrList = results.getChildByName("attributes-list");
            if (attrList != null) {
                List<NaElement> binds = attrList.getChildren();
                for (NaElement bind : binds) {
                    String pePath = bind.getChildContent("protocol-endpoint-path");
                    String vServer = bind.getChildContent("vserver");
                    int refCount = bind.getChildIntValue("reference-count", 0);
                    boolean isOptimal = Boolean.getBoolean(bind.getChildContent("is-optimal"));
                    long secLunId = bind.getChildLongValue("secondary-lun", 0L);
                    String vvolPath = bind.getChildContent("vvol-path");

                    LunBindingImpl lunBindInfo = new LunBindingImpl(isOptimal, pePath, refCount, secLunId, vServer,
                                                                    vvolPath, dataSvm);
                    cummulativeResults.add(lunBindInfo);
                }
            }
        }
    }
}
