/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/lun/LunGeometryDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader.lun;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.lun.OntapLunImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.Set;

public class LunGeometryDataLoader extends AbstractDataLoader<OntapLunImpl> {

    private static final Logger logger = Logger.getLogger(LunGeometryDataLoader.class);

    @Override
    protected boolean doLoad(OntapLunImpl lun, LoadType loadType,
                             Set<Resource> resources) throws DataLoaderException {
        logger.debug("load: begin for path " + lun.getPath());
        long maxResizeSize = -1;
        NaElement executeApi = new NaElement("lun-get-geometry");
        // get lun-get-geometry
        try {
            executeApi.addNewChild("path", lun.getPath());
            NaElement lunGeometryInfo = lun.getControllerImpl().invokeZapi(executeApi, ZapiCache.LONG_CACHE);
            maxResizeSize = lunGeometryInfo.getChildLongValue("max-resize-size", 0L);
        } catch (ApiFailedException e) {
            // TODO: What is the right error handling here? Adding a discovery error or throwing an exception or just logging?
            lun.getControllerImpl().addDiscoveryError("Problem loading lun geometry", e);
        } catch (Exception e) {
            logger.error(
                    "getLun: " + lun.getControllerImpl().getName() + " " + lun.getPath() + " failed to get maximum resize size : " + e.getMessage());
            lun.getControllerImpl().addDiscoveryError("Exception while calling lun-get-geometry for " + lun.getPath(), e);
        }
        lun.setMaxResizeSize(maxResizeSize);
        return true;
    }
}
