/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/lun/LunMapListDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader.lun;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapDataController;
import com.netapp.offtap3.internal.ontap.intf.OntapIGroupImpl;
import com.netapp.offtap3.internal.ontap.lun.OntapLunImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.OntapIGroup;
import com.netapp.offtap3.ontap.OntapDataController;

import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class LunMapListDataLoader extends AbstractDataLoader<OntapLunImpl> {
    private static final Logger logger = Logger.getLogger(LunMapListDataLoader.class);

    @Override
    protected boolean doLoad(OntapLunImpl lun, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        if (lun == null || lun.getPath() == null || "vvol".equals(lun.getLunClass())) {
            return true;
        }

        logger.debug("load begin for lun " + lun.getPath());
        NaElement lunMapListApi = new NaElement("lun-map-list-info");
        lunMapListApi.addNewChild("path", lun.getPath());
        Set<OntapIGroup> igroups = new HashSet<OntapIGroup>();

        // we found an igroup with alua enabled
        boolean foundAluaEnabled = false;
        // we found an igroup with alua disabled
        boolean foundAluaDisabled = false;

        try {
            NaElement lunMapListInfo = lun.getControllerImpl().invokeZapi(lunMapListApi, ZapiCache.LONG_CACHE);

            NaElement initiatorGroupsResponse = lunMapListInfo.getChildByName("initiator-groups");
            if (initiatorGroupsResponse != null) {
                List<NaElement> igroupsInfo = initiatorGroupsResponse.getChildren();

                OntapIGroupImpl lastIGroup = null;

                for (NaElement iGroupInfo : igroupsInfo) {
                    OntapIGroupImpl<AbstractOntapDataController<OntapDataController>, OntapDataController> igroup =
                            new OntapIGroupImpl<AbstractOntapDataController<OntapDataController>, OntapDataController>(lun.getControllerImpl());
                    igroup.setName(iGroupInfo.getChildContent("initiator-group-name"));
                    igroup.setProtocol(iGroupInfo.getChildContent("initiator-group-type"));
                    igroup.setOsType(iGroupInfo.getChildContent("initiator-group-os-type"));
                    igroup.setLunId(iGroupInfo.getChildIntValue("lun-id", 0));
                    igroup.setAluaEnabled(
                            Boolean.parseBoolean(iGroupInfo.getChildContent("initiator-group-alua-enabled")));
                    if (igroup.isAluaEnabled()) {
                        foundAluaEnabled = true;
                    } else {
                        foundAluaDisabled = true;
                    }

                    igroup.setPortsetName(iGroupInfo.getChildContent("initiator-group-portset-name"));
                    if (iGroupInfo.getChildByName("initiators") != null) {
                        for (NaElement initiatorInfo : iGroupInfo.getChildByName("initiators").getChildren()) {
                            igroup.getInitiatorIds().add(initiatorInfo.getChildContent("initiator-name"));
                        }
                    }
                    if (igroup.getName() != null) {
                        igroups.add(igroup);
                    }

                    // This is for MHC compatibility use only - it has no value otherwise!
                    lastIGroup = igroup;
                }

                if (lastIGroup != null) {
                    lun.setLastIGroup(lastIGroup);
                }
            }
        } catch (ApiFailedException e) {
            lun.getControllerImpl().addZapiExecutionError("Exception while calling lun-map-list-info" + lun.getPath(), lunMapListApi,
                                             e);
        }

        // if this lun has at least one igroup with alua disabled AND at least one igroup with alua enabled
        // then mark the lun as multistate
        lun.setMultiStateAlua(foundAluaEnabled && foundAluaDisabled);

        if (igroups.isEmpty()) {
            lun.setMapped(false);
        } else {
            lun.setMapped(true);
            lun.setIgroups(igroups);
        }
        lun.setLastLoad(new Date());
        logger.debug("load: end");
        return true;
    }
}
