package com.netapp.offtap3.internal.ontap.dataloader.node;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.NodeSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.Set;

public class NodeSvmClusterFailoverDataLoader extends AbstractDataLoader<NodeSvmImpl> {

    private static final Logger logger = Logger.getLogger(NodeSvmClusterFailoverDataLoader.class);

    @Override
    protected boolean doLoad(NodeSvmImpl node, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        loadPartner(node);
        loadHaMode(node);
        return true;
    }

    protected void loadPartner(NodeSvmImpl node) {
        try {
            NaElement xi = new NaElement("cf-get-partner");
            xi.addNewChild("node", node.getName());
            NaElement result = node.getParentControllerImpl().invokeZapi(xi, ZapiCache.LONG_CACHE);
            if (!result.getChildContent("partner").isEmpty()) {
                node.setPartner(result.getChildContent("partner"));
                xi = new NaElement("cf-status");                
                xi.addNewChild("node", node.getName());
                if (node.getParentControllerImpl() != null) {
                	result = node.getParentControllerImpl().invokeZapi(xi, ZapiCache.LONG_CACHE);                
                	node.setClusterStatus(result.getChildContent("state"));
                }
            }
        } catch (ApiFailedException ex) {
            logger.error("getClusterInfo: " + node +" Failed to get cluster info. " + ex.getMessage());
        }
    }

    protected void loadHaMode(NodeSvmImpl node) {
        NaElement api = new NaElement("cf-status");
        api.addNewChild("node", node.getName());

        try {
            NaElement result = node.getParentControllerImpl().invokeZapi(api, ZapiCache.LONG_CACHE);
            String currentMode = result.getChildContent("current-mode");

            //I know how this looks, but below DOT 8.2 <current-mode> doesn't exist, so null is returned
            //but if the node wasn't in HA Pair, invokeZAPI would produce 'Internal Error' NaAPIFailedException exception
            if (currentMode == null) {
                currentMode = "ha";
            }
            // allow user to use storage failover modify to change ha settings
            if ("false".equals(result.getChildContent("is-enabled"))) {
                currentMode = "non_ha";
            }
            node.setHighAvailabilityMode(currentMode);
        }
        catch (ApiFailedException e) {
            if (e.getMessage().equals("Internal Error (errno=13114)")) {
                logger.info("DOT version is below 8.2 and it's non_ha so get this exception");
                logger.info("in populateHighAvailabilityMode: " + e.getLocalizedMessage());
                node.setHighAvailabilityMode("non_ha");
            }
            else {
                logger.error("unknown reason for the exception in populateHighAvailabilityMode:" + e.getLocalizedMessage());
                node.setHighAvailabilityMode(null);
            }
        }
    }
}
