package com.netapp.offtap3.internal.ontap.dataloader.node;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.NodeSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.List;
import java.util.Set;


public class NodeSvmFlashCacheCapacityDataLoader extends AbstractDataLoader<NodeSvmImpl> {

    private static final Logger logger = Logger.getLogger(NodeSvmFlashCacheCapacityDataLoader.class);

    @Override
    protected boolean doLoad(NodeSvmImpl node, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        NaElement api = new NaElement("flash-device-list-info");
        api.addNewChild("node", node.getName());

        try {
            NaElement result = node.getParentControllerImpl().invokeZapi(api, ZapiCache.LONG_CACHE);
            NaElement deviceInfo = result.getChildByName("flash-device-info");
            int cacheCapacity = 0;

            if (deviceInfo != null) {
                List<NaElement> listOfDevices = deviceInfo.getChildren();
                for (NaElement device : listOfDevices) {
                    String flashCacheNode = device.getChildByName("node").getContent();

                    if (flashCacheNode.equals(node.getName())) {
                        String flashCacheCapacity = device.getChildContent("capacity");
                        if (flashCacheCapacity != null) {
                            cacheCapacity += Integer.valueOf(flashCacheCapacity);
                        }
                        else {
                            logger.debug("populateFlashCacheInfo: flashCacheCapacity was null, cacheCapacity was set to 0");
                        }
                    }
                }
            }
            node.setFlashCacheCapacity(cacheCapacity);
        }
        catch (ApiFailedException e) {
            logger.error("in populateFlashCacheInfo:" + e.getLocalizedMessage());
            logger.info("flash-device-list-info zapi was not found, cacheCapacity was set to 0");
            node.setFlashCacheCapacity(0);
        }
        return true;
    }
}
