/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/node/NodeSvmSystemModelDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader.node;

import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.controller.NodeSvmImpl;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.CumulativeDataParser;
import com.netapp.offtap3.internal.ontap.dataloader.OntapCachingDataLoader;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class NodeSvmSystemModelDataLoader extends OntapCachingDataLoader<NodeSvmImpl> {

    private static final Logger logger = Logger.getLogger(NodeSvmSystemModelDataLoader.class);

    private CdotIterativeLoader<NaElement> iterativeLoader;

    public NodeSvmSystemModelDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);
        iterativeLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(),
                                                             new NaElement("system-get-node-info-iter"),
                                                             new CumulativeDataParser(), prefs.getIteratorSize(),
                                                             ZapiCache.SHORT_CACHE);
    }

    @Override
    protected Map<Resource, NaElement> retrieve(NodeSvmImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("retrieve: start");

        NaElement result = new NaElement("results");
        try {
            iterativeLoader.load(result);
        } catch (ApiFailedException e) {
            throw new DataLoaderException(e);
        }

        logger.debug("retrieve: end");
        return Collections.singletonMap(resources.iterator().next(), result);
    }

    @Override
    protected boolean parse(NodeSvmImpl node, NaElement data, Resource resource) {
        List<NaElement> systemInfo = data.getChildren();
        for (NaElement systems : systemInfo) {
            if (systems.getChildContent("system-name").equals(node.getName())) {
                node.setModel(systems.getChildContent("system-model"));
            }
        }
        return true;
    }
}
