package com.netapp.offtap3.internal.ontap.dataloader.snapmirror;



import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.CumulativeDataParser;
import com.netapp.offtap3.internal.ontap.dataloader.OntapCachingDataLoader;
import com.netapp.offtap3.internal.ontap.snap.CModeSnapMirrorPolicyImpl;
import com.netapp.offtap3.internal.ontap.snap.CModeSnapMirrorRelationImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.CModeSnapMirrorRelation;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;




import java.util.Collections;
import java.util.Map;
import java.util.Set;



/**

 * Created by torir on 9/22/14.

 */

public class CModeSnapMirrorPolicyDataLoader extends OntapCachingDataLoader<CModeSnapMirrorRelationImpl> {

    private static final Logger logger = Logger.getLogger(CModeSnapMirrorScheduleDataLoader.class);

    private OfftapPreferences prefs;



    public CModeSnapMirrorPolicyDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> snapMirrorCache) {

        super(snapMirrorCache);

        this.prefs = prefs;

    }



    @Override

    protected Map<Resource, NaElement> retrieve(CModeSnapMirrorRelationImpl relationToReload, LoadType loadType, Set<Resource> resources) throws DataLoaderException {

        logger.debug("begin retrieve, " + "loadType = " + loadType + ", relationToReload = " + relationToReload);



        NaElement result = new NaElement("results");

        try {

            DataSvm svm = relationToReload.getDestinationSvm();

            if (svm.getVersion().atLeastSN()) {

                getIterativeDataLoader(loadType, relationToReload).load(result);

            }

        } catch (ApiFailedException e) {

            throw new DataLoaderException(e);

        }



        logger.debug("retrieve: end");

        return Collections.singletonMap(resources.iterator().next(), result);

    }



    private CdotIterativeLoader<NaElement> getIterativeDataLoader(LoadType loadType, CModeSnapMirrorRelation relationToReload) {

        NaElement input = new NaElement("snapmirror-policy-get-iter");

        NaElement query = new NaElement("query");

        NaElement info = new NaElement("snapmirror-policy-info");

        query.addChildElem(info);

        input.addChildElem(query);

        return new CdotIterativeLoader<NaElement>(getDataCache().getDataRetriever(), input, new CumulativeDataParser(),

                prefs.getIteratorSize(), ZapiCache.SHORT_CACHE);

    }



    @Override

    protected boolean parse(CModeSnapMirrorRelationImpl relation, NaElement data, Resource resource) {

        DataSvm svm = relation.getDestinationController();

        logger.debug("parse: start " + svm);



        String policy = relation.getPolicyName();

        // System.out.println("# policy info items found = " + data.getChildren().size());



        for (NaElement policyInfo : data.getChildren()) {

            // check to see if this is the policy for this relation

            String policyName = policyInfo.getChildContent("policy-name");

            if (policyName == null || !policyName.equals(policy)) {

                continue;

            }



            CModeSnapMirrorPolicyImpl policyObject = new CModeSnapMirrorPolicyImpl();
            
            policyObject.setController((DataSvmImpl) svm);
            
            policyObject.setName(policyName);

            policyObject.setComment(policyInfo.getChildContent("comment"));

            policyObject.setRestart(policyInfo.getChildContent("restart"));

            policyObject.setTransferPriority(policyInfo.getChildContent("transfer-priority"));

            policyObject.setTries(policyInfo.getChildContent("tries"));

            relation.setPolicyBean(policyObject);

            break;

        }

        return true;

    }

}

