package com.netapp.offtap3.internal.ontap.dataloader.snapmirror;


import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.CumulativeDataParser;
import com.netapp.offtap3.internal.ontap.dataloader.OntapCachingDataLoader;
import com.netapp.offtap3.internal.ontap.snap.CModeSnapMirrorRelationImpl;
import com.netapp.offtap3.internal.ontap.snap.SnapMirrorScheduleImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.Map;
import java.util.Set;

public class CModeSnapMirrorScheduleDataLoader extends OntapCachingDataLoader<CModeSnapMirrorRelationImpl> {
    private static final Logger logger = Logger.getLogger(CModeSnapMirrorScheduleDataLoader.class);

    private CdotIterativeLoader<NaElement> iterativeLoader;


    public CModeSnapMirrorScheduleDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);
        iterativeLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(),
                new NaElement("job-schedule-cron-get-iter"),
                new CumulativeDataParser(), prefs.getIteratorSize(),
                ZapiCache.SHORT_CACHE);
    }

    @Override
    protected Map<Resource, NaElement> retrieve(CModeSnapMirrorRelationImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("retrieve: start");

        NaElement result = new NaElement("results");
        try {
            iterativeLoader.load(result);
        } catch (ApiFailedException e) {
            throw new DataLoaderException(e);
        }

        logger.debug("retrieve: end");
        return Collections.singletonMap(resources.iterator().next(), result);
    }

    @Override
    protected boolean parse(CModeSnapMirrorRelationImpl relation, NaElement data, Resource resource) {

        DataSvm svm = relation.getDestinationSvm();
        logger.debug("parse: start " + svm);

        String scheduleName = relation.getScheduleName();

        for (NaElement jobScheduleCronInfo : data.getChildren()) {

            // check to see if this is the schedule for this relation
            String cronName = jobScheduleCronInfo.getChildContent("job-schedule-name");
            if (cronName == null || !cronName.equals(scheduleName)) {
                continue;
            }

            SnapMirrorScheduleImpl schedule = new SnapMirrorScheduleImpl();
            schedule.setName(cronName);
            schedule.setDestinationController(relation.getDestinationController());
            // job schedule info. collected in 0-based ONTAP format; 1-based cron values
            // calculated on the fly in getCrontabFormat()
            schedule.setMinutes(collect(jobScheduleCronInfo, "job-schedule-cron-minute"));
            schedule.setHours(collect(jobScheduleCronInfo, "job-schedule-cron-hour"));
            schedule.setDaysOfMonth(collect(jobScheduleCronInfo, "job-schedule-cron-day"));
            schedule.setMonths(collect(jobScheduleCronInfo, "job-schedule-cron-month"));
            schedule.setDaysOfWeek(collect(jobScheduleCronInfo, "job-schedule-cron-day-of-week"));
            relation.setScheduleBean(schedule);
            logger.debug("Built this crontab entry: " + schedule.getCrontabFormat());
            break;
        }
        return true;
    }

    String collect(NaElement parent, String elementName) {
        StringBuilder entry = new StringBuilder();
        try {
            if (parent.getChildByName(elementName) != null) {
                for (NaElement child : parent.getChildByName(elementName).getChildren()) {
                    if (entry.length() > 0) {
                        entry.append(",");
                    }
                    int value = Integer.parseInt(child.getContent());
                    entry.append(value);
                }
            }
            if (entry.length() > 0) {
                return entry.toString();
            }
        } catch (NumberFormatException e) {
            logger.error("Failed to parse cron content from element: " + elementName + ". Error: " + e, e);
        }
        return null;
    }

}