package com.netapp.offtap3.internal.ontap.dataloader.snapmirror;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.SnapMirrorResource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.snap.CModeSnapMirrorRelationImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.List;
import java.util.Set;

public class CModeSnapMirrorStatusDataLoader extends AbstractDataLoader<CModeSnapMirrorRelationImpl> {

    private static final Logger logger = Logger.getLogger(CModeSnapMirrorStatusDataLoader.class);

    public CModeSnapMirrorStatusDataLoader() {
        // Since this data is loaded by default with the main loader, set the isLoaded flag to true from the beginning.
        setLoaded(SnapMirrorResource.status, true);
    }

    @Override
    protected boolean doLoad(CModeSnapMirrorRelationImpl relation, LoadType loadType, Set<Resource> resources) throws DataLoaderException {

        if (loadType != LoadType.refresh) {
            // all this data has already been loaded when created, so only do this for refresh
            return true;
        }

        logger.debug("refreshing: " + relation);

        NaElement xi = new NaElement("snapmirror-get");
        xi.addNewChild("destination-location", relation.getDestinationLocation());
        xi.addNewChild("source-location", relation.getSourceLocation());

        try {
            NaElement xo = relation.getDestinationController().getParentControllerImpl().invokeZapi(xi, ZapiCache.LONG_CACHE);
            NaElement attrs = xo.getChildByName("attributes");
            List children = attrs.getChildren();
            if (!children.isEmpty()) {
                CModeSnapMirrorRelationParser.getInstance().populate(relation, (NaElement) children.get(0));
            }
        } catch (ApiFailedException e) {
            throw new DataLoaderException(relation + " failed to reload its state due to : " + e, e);
        }
        return true;
    }
}
