package com.netapp.offtap3.internal.ontap.dataloader.snapmirror;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.CumulativeDataParser;
import com.netapp.offtap3.internal.ontap.dataloader.OntapCachingDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.OntapDataCache;
import com.netapp.offtap3.internal.ontap.snap.CModeSnapMirrorRelationImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;
import com.netapp.offtap3.utils.Utils;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.CModeSnapMirrorRelation;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class DataSvmSnapMirrorRelationDataLoader extends OntapCachingDataLoader<DataSvmImpl> {
    private static final Logger LOGGER = Logger.getLogger(DataSvmSnapMirrorRelationDataLoader.class);
    private OfftapPreferences prefs;

    public DataSvmSnapMirrorRelationDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> snapMirrorCache) {
        super(snapMirrorCache);
        this.prefs = prefs;
    }

    @Override
    protected NaElement retrieve(DataSvmImpl loadTarget, Resource resource, Refreshable refreshable) {
        LOGGER.debug("begin retrieve(refresh)");
        Map<Resource, NaElement> dataMap = retrieve(loadTarget, LoadType.refresh, Collections.singleton(resource),
                                                    (CModeSnapMirrorRelation) refreshable);
        NaElement result = dataMap.get(resource);
        LOGGER.debug("end retrieve(refresh)");
        return result;
    }

    @Override
    protected Map<Resource, NaElement> retrieve(DataSvmImpl svm, LoadType loadType, Set<Resource> resources) {
        LOGGER.debug("begin retrieve(override)");
        Map<Resource, NaElement> result = retrieve(svm, loadType, resources, null);
        LOGGER.debug("end retrieve(override)");
        return result;
    }

    protected Map<Resource, NaElement> retrieve(DataSvmImpl svm, LoadType loadType, Set<Resource> resources, CModeSnapMirrorRelation relationToReload) {
        LOGGER.debug("begin retrieve, svm = " + svm + ", loadType = " + loadType + ", relationToReload = " + relationToReload);

        NaElement result = new NaElement("results");
        try {
            getIterativeDataLoader(loadType, relationToReload).load(result);
            // I have seen cases where the relationship only shows up in the
            // destination-iter.  So, let's go get those as well, and let parse()
            // discard the duplicates.
            if (svm.getVersion().atLeastSN()) {
                getDestinationIterativeLoader(loadType, relationToReload).load(result);
            }
        } catch (ApiFailedException e) {
            throw new DataLoaderException(e);
        }

        LOGGER.debug("end retrieve");
        return Collections.singletonMap(resources.iterator().next(), result);
    }

    private CdotIterativeLoader<NaElement> getIterativeDataLoader(LoadType loadType, CModeSnapMirrorRelation relationToReload) {
        NaElement input = new NaElement("snapmirror-get-iter");
        if (loadType == LoadType.refresh && relationToReload != null) {
            input.addChildElem(createQuery("snapmirror-info", relationToReload));
        }
        return new CdotIterativeLoader<NaElement>(getDataCache().getDataRetriever(), input, new CumulativeDataParser(),
                                                  prefs.getIteratorSize(), ZapiCache.SHORT_CACHE);

    }

    private CdotIterativeLoader<NaElement> getDestinationIterativeLoader(LoadType loadType, CModeSnapMirrorRelation relationToReload) {
        NaElement input = new NaElement("snapmirror-get-destination-iter");
        if (loadType == LoadType.refresh && relationToReload != null) {
            input.addChildElem(createQuery("snapmirror-destination-info", relationToReload));
        }
        return new CdotIterativeLoader<NaElement>(getDataCache().getDataRetriever(), input, new CumulativeDataParser(),
                                                  prefs.getIteratorSize(), ZapiCache.SHORT_CACHE);
    }

    private NaElement createQuery(String infoElement, CModeSnapMirrorRelation relationToReload) {
        NaElement query = new NaElement("query");
        NaElement snapMirrorInfo = new NaElement((infoElement));
        if (relationToReload.getSourceLocation() != null) {
            snapMirrorInfo.addNewChild("source-location", relationToReload.getSourceLocation());
        }
        if (relationToReload.getDestinationLocation() != null) {
            snapMirrorInfo.addNewChild("destination-location", relationToReload.getDestinationLocation());
        }
        if (relationToReload.getRelationshipId() != null) {
            snapMirrorInfo.addNewChild("relationship-id", relationToReload.getRelationshipId());
        }
        query.addChildElem(snapMirrorInfo);
        return query;
    }

    @Override
    protected boolean parse(DataSvmImpl svm, NaElement data, Resource resource) {
        parse(svm, data, resource, false);
        return true;
    }

    protected void parse(DataSvmImpl svm, NaElement data, Resource resource, boolean refresh) {
        LOGGER.debug("begin parse svm = " + svm + ", data = " + data);
        String clusterName = null;
        if (!svm.isDirectSvm()) {
            clusterName = svm.getParentControllerImpl().getName();
        }

        Map<String, CModeSnapMirrorRelation> relations = new SafeConcurrentHashMap<String, CModeSnapMirrorRelation>();

        OntapDataCache snapMirrorCache = new OntapDataCache(svm);
        for (NaElement snapMirrorElem : data.getChildren()) {
            String destCluster = snapMirrorElem.getChildContent("destination-cluster");
            String destSvm = snapMirrorElem.getChildContent("destination-vserver");
            String sourceSvm = snapMirrorElem.getChildContent("source-vserver");
            // for direct svm, clusterName is null, and we know this relation has this svm as its destination
            if ((destCluster == null || clusterName == null || destCluster.equals(clusterName)) &&
                    (MCCManager.getInstance().getVServerNameByUuid(svm).equals(destSvm) || MCCManager.getInstance().getVServerNameByUuid(svm).equals(sourceSvm))) {
                String sourceLocation = snapMirrorElem.getChildContent("source-location");
                String destinationLocation = snapMirrorElem.getChildContent("destination-location");
                String relationshipId = snapMirrorElem.getChildContent("relationship-id");
                String key = createKey(sourceLocation, destinationLocation, relationshipId);
                if (relations.containsKey(key)) {
                    continue;
                }
                CModeSnapMirrorRelationImpl relation = new CModeSnapMirrorRelationImpl(svm, snapMirrorCache);
                relation.setDestinationLocation(destinationLocation);
                relation.setSourceLocation(sourceLocation);
                relation.setRelationshipId(relationshipId);
                relation.setDestinationController(svm);
                relation.setDestinationVolume(
                        svm.getProxy().getVolume(snapMirrorElem.getChildContent("destination-volume")));
                relation.setSourceCluster(snapMirrorElem.getChildContent("source-cluster"));
                // for direct svm, clusterName is null and the source might be another svm or even outside the cluster
                // so we cannot assign the source svm reference
                if (clusterName != null && (relation.getSourceCluster() == null ||
                        relation.getSourceCluster().equals(svm.getParentController().getName()))) {
                    relation.setSourceController(svm.getParentController().getDataSvm(sourceSvm));
                }
                relation.setSourceSvm(sourceSvm);
                relation.setSourceVolume(snapMirrorElem.getChildContent("source-volume"));
                CModeSnapMirrorRelationParser.getInstance().populate(relation, snapMirrorElem);
                relations.put(key, relation.getProxy());
            }
        }

        List<CModeSnapMirrorRelation> relationList = new ArrayList<CModeSnapMirrorRelation>();
        if (refresh) {
            relationList.addAll(svm.getSnapMirrorRelations());
            removeFoundRelations(relationList, relations);
        }
        relationList.addAll(relations.values());
        svm.setSnapMirrorRelations(relationList);
        LOGGER.debug("end parse, relations size = " + relationList.size());
    }

    @Override
    protected Refreshable parse(DataSvmImpl loadTarget, NaElement data, Resource resource, Refreshable reloadTarget) {
        LOGGER.debug("begin parse(refresh)");
        parse(loadTarget, data, resource, true);
        CModeSnapMirrorRelation reloadRelation = (CModeSnapMirrorRelation) reloadTarget;
        CModeSnapMirrorRelation result = null;
        for (CModeSnapMirrorRelation relation : loadTarget.getSnapMirrorRelations()) {
            if (Utils.areEqual(relation.getSourceLocation(), reloadRelation.getSourceLocation()) &&
                    Utils.areEqual(relation.getDestinationLocation(), reloadRelation.getDestinationLocation()) &&
                    Utils.areEqual(relation.getRelationshipId(), reloadRelation.getRelationshipId())) {
                result = relation;
                break;
            }
        }
        LOGGER.debug("end parse(refresh), result = " + result);
        return result;
    }

    private void removeFoundRelations(List<CModeSnapMirrorRelation> existingRelations, Map<String, CModeSnapMirrorRelation> relationsToRemove) {
        for (Iterator<CModeSnapMirrorRelation> iter = existingRelations.iterator(); iter.hasNext();) {
            CModeSnapMirrorRelation relation = iter.next();
            String key = createKey(relation.getSourceLocation(), relation.getDestinationLocation(), relation.getRelationshipId());
            if (relationsToRemove.containsKey(key)) {
                iter.remove();
            }
        }
    }

    private String createKey(String sourceLocation, String destinationLocation, String relationshipId) {
        return sourceLocation + "||" + destinationLocation + "||" + relationshipId;
    }
}
