package com.netapp.offtap3.internal.ontap.dataloader.snapmirror;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.dataloader.OntapDataCache;
import com.netapp.offtap3.internal.ontap.snap.SevenModeSnapMirrorRelationImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.utils.Utils;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.Filer;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.SnapMirrorRelation;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class FilerSnapMirrorRelationDataLoader extends AbstractDataLoader<FilerImpl<? extends Filer>> {

    private static final Logger logger = Logger.getLogger(FilerSnapMirrorRelationDataLoader.class);

    @Override
    protected Refreshable doRefresh(FilerImpl<? extends Filer> loadTarget, Resource resource, Refreshable refreshTarget) {
        SnapMirrorRelation reloadRelation = (SnapMirrorRelation) refreshTarget;
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource), reloadRelation);
        for (SnapMirrorRelation relation : loadTarget.getSnapMirrorRelations()) {
            if (Utils.areEqual(relation.getSourceLocation(), reloadRelation.getSourceLocation()) &&
                    Utils.areEqual(relation.getDestinationLocation(), reloadRelation.getDestinationLocation())) {
                return relation;
            }
        }
        return null;
    }

    @Override
    protected boolean doLoad(FilerImpl<? extends Filer> filer, LoadType loadType, Set<Resource> resources) {
        return doLoad(filer, loadType, resources, null);
    }

    protected boolean doLoad(FilerImpl<? extends Filer> filer, LoadType loadType, Set<Resource> resources, SnapMirrorRelation reloadTarget) {
        logger.trace("enter");
        List<SnapMirrorRelation> snapMirrorRelations;
        NaElement input = new NaElement("snapmirror-get-status");
        if (loadType == LoadType.refresh && reloadTarget != null && reloadTarget.getDestinationLocation() != null) {
            // If we're reloading a specific snap mirror relation, try to restrict what we pull back to just what is needed.
            input.addNewChild("location", reloadTarget.getDestinationLocation());
            // Make sure we start with the orignal set of relations. We're only going to add the refreshed ones that
            // have the same destination as the reloadTarget.
            snapMirrorRelations = new ArrayList<SnapMirrorRelation>(filer.getSnapMirrorRelations());
            // Remove the relations that have the same destination as the target. They'll get replaced with reloaded
            // ones during the parsing.
            removeOldRelation(snapMirrorRelations, reloadTarget);
        } else {
            snapMirrorRelations = new ArrayList<SnapMirrorRelation>();
        }

        try {
            NaElement xo = filer.invokeZapi(input, ZapiCache.SHORT_CACHE);
            String isAvailable = xo.getChildContent("is-available");

            if (Boolean.parseBoolean(isAvailable)) {
                NaElement snapMirrorStatus = xo.getChildByName("snapmirror-status");

                if (snapMirrorStatus != null) {
                    SevenModeSnapMirrorRelationParser parser = SevenModeSnapMirrorRelationParser.getInstance();
                    OntapDataCache snapMirrorCache = new OntapDataCache(filer);
                    for (NaElement status : snapMirrorStatus
                            .getChildren()) { // Each info is the <snapmirror-status-info> tag
                        SevenModeSnapMirrorRelationImpl relation = new SevenModeSnapMirrorRelationImpl(filer, snapMirrorCache);
                        relation.setSourceController(filer);
                        relation.setSourceLocation(status.getChildContent("source-location"));
                        relation.setDestinationController(filer);
                        relation.setDestinationLocation(status.getChildContent("destination-location"));
                        parser.populate(relation, status);
                        snapMirrorRelations.add(relation.getProxy());
                    }
                }
            } else {
                logger.warn(filer + " Unable to get SnapMirror status: SnapMirror unavailable.");
            }

        } catch (ApiFailedException e) {
            logger.error("Unable to get SnapMirror status.", e);
            throw new DataLoaderException(filer + " unable to get SnapMirror status" + e.getMessage(), e);
        }

        filer.setSnapMirrorRelations(snapMirrorRelations);
        logger.trace("exit");
        return true;
    }

    private void removeOldRelation(List<SnapMirrorRelation> relations, SnapMirrorRelation oldRelation) {
        for (Iterator<SnapMirrorRelation> relationIter = relations.iterator(); relationIter.hasNext(); ) {
            SnapMirrorRelation relation = relationIter.next();
            if (relation.getDestinationLocation() != null &&
                    relation.getDestinationLocation().equals(oldRelation.getDestinationLocation())) {
                relationIter.remove();
            }
        }
    }
}
