package com.netapp.offtap3.internal.ontap.dataloader.snapmirror;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.dataloader.OntapCachingDataLoader;
import com.netapp.offtap3.internal.ontap.snap.SevenModeSnapMirrorRelationImpl;
import com.netapp.offtap3.internal.ontap.snap.SnapMirrorScheduleImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.Map;
import java.util.Set;


public class SevenModeSnapMirrorScheduleDataLoader extends OntapCachingDataLoader<SevenModeSnapMirrorRelationImpl> {

    private static final Logger logger = Logger.getLogger(SevenModeSnapMirrorScheduleDataLoader.class);

    public SevenModeSnapMirrorScheduleDataLoader(DataCache<NaElement, InternalZapiHandler> snapMirrorCache) {
        super(snapMirrorCache);
    }

    @Override
    protected Map<Resource, NaElement> retrieve(SevenModeSnapMirrorRelationImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        InternalZapiHandler handler = getDataCache().getDataRetriever();
        logger.debug(handler + " is retrieving all snap mirror schedules");
        try {
            NaElement results = handler.invokeZapi(new NaElement("snapmirror-list-schedule"), ZapiCache.SHORT_CACHE);
            return Collections.singletonMap(resources.iterator().next(), results.getChildByName("snapmirror-schedules"));
        }
        catch (ApiFailedException e) {
            throw new DataLoaderException(handler + " failed to load snapmirror schedules: " + e, e);
        }
    }

    @Override
    protected boolean parse(SevenModeSnapMirrorRelationImpl relation, NaElement data, Resource resource) {

        for (NaElement elem : data.getChildren()) {

            String destinationLocation = elem.getChildContent("destination-location");
            if (relation.getDestinationLocation().equals(destinationLocation)) {
                SnapMirrorScheduleImpl schedule = new SnapMirrorScheduleImpl();
                schedule.setDaysOfMonth(elem.getChildContent("days-of-month"));
                schedule.setDaysOfWeek(elem.getChildContent("days-of-week"));
                schedule.setHours(elem.getChildContent("hours"));
                schedule.setMinutes(elem.getChildContent("minutes"));
                relation.setScheduleBean(schedule);
                break;
            }
        }
        return true;
    }
}
