package com.netapp.offtap3.internal.ontap.dataloader.snapmirror;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.snap.SevenModeSnapMirrorRelationImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.List;
import java.util.Set;

public class SevenModeSnapMirrorStatusDataLoader extends AbstractDataLoader<SevenModeSnapMirrorRelationImpl> {

    private static final Logger logger = Logger.getLogger(SevenModeSnapMirrorStatusDataLoader.class);

    @Override
    protected boolean doLoad(SevenModeSnapMirrorRelationImpl relation, LoadType loadType, Set<Resource> resources) throws DataLoaderException {

        // all of this data is already present.  only need to load if it's a refresh.
        if (loadType != LoadType.refresh) {
            return true;
        }

        logger.debug("refreshing: " + relation);
        NaElement xi = new NaElement("snapmirror-get-status");

        xi.addNewChild("location", relation.getDestinationLocation());

        try {
            NaElement xo = relation.getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
            NaElement statusList = xo.getChildByName("snapmirror-status");
            List statusChildren = statusList.getChildren();
            if (!statusChildren.isEmpty()) {
                SevenModeSnapMirrorRelationParser.getInstance().populate(relation, (NaElement) statusChildren.get(0));
            }
        } catch (ApiFailedException e) {
            throw new DataLoaderException(relation + " failed to load status due to: " + e.getMessage(), e);
        }
        return true;
    }
}
