/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/volume/CModeDedupeSizeSharedDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader.volume;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.utils.Utils;
import com.netapp.offtap3.internal.ontap.volume.CModeDedupeImpl;
import com.netapp.offtap3.ontap.NaElement;

import org.apache.log4j.Logger;

import java.util.Scanner;
import java.util.Set;

public class CModeDedupeSizeSharedDataLoader extends AbstractDataLoader<CModeDedupeImpl> {

    private static final Logger logger = Logger.getLogger(CModeDedupeSizeSharedDataLoader.class);

    @Override
    protected boolean doLoad(CModeDedupeImpl dedupe, LoadType loadType,
                             Set<Resource> resources) throws DataLoaderException {
        DataSvmImpl dataSvm = dedupe.getControllerImpl();
        // This call won't work on a direct connect svm.
        if (!dataSvm.isDirectSvm()) {
            /*<system-cli>
                 <args>
                   <arg>set diag ; sis stat -vserver esx_vs -volume esx_vol1 -fields shared-data</arg>
                 </args>
            </system-cli>

            <netapp version="1.15">
             <results status="passed">
                   <cli-output>vserver volume shared-data ------- -------- ----------- esx_vs esx_vol1 "1244 KB"</cli-output>
                   <cli-result-value>1</cli-result-value>
             </results>
           </netapp>*/

            StringBuffer commandToExecute = new StringBuffer();
            NaElement input = new NaElement("system-cli");
            NaElement argsElement = new NaElement("args");
            input.addChildElem(argsElement);
            addCommandElement(commandToExecute, argsElement, "set");
            addCommandElement(commandToExecute, argsElement, "diag;");
            addCommandElement(commandToExecute, argsElement, "volume");
            addCommandElement(commandToExecute, argsElement, "efficiency");
            addCommandElement(commandToExecute, argsElement, "stat");
            addCommandElement(commandToExecute, argsElement, "-vserver");
            addCommandElement(commandToExecute, argsElement, MCCManager.getInstance().getVServerNameByUuid(dataSvm));
            addCommandElement(commandToExecute, argsElement, "-volume");
            addCommandElement(commandToExecute, argsElement, dedupe.getVolume().getName());
            addCommandElement(commandToExecute, argsElement, "-fields");
            addCommandElement(commandToExecute, argsElement, "shared-data");

            try {
                NaElement output = dataSvm.getParentControllerImpl().invokeZapi(input, ZapiCache.NO_CACHE);
                if (output != null) {
                    if (output.getChildContent("cli-result-value") != null) {
                        if (Integer.parseInt(output.getChildContent("cli-result-value")) != 0) {
                            String cliOutput = output.getChildContent("cli-output");
                            cliOutput = cliOutput.replace("\"", "");
                            Scanner scanner = new Scanner(cliOutput);
                            Long spaceShared = 0L;
                            Long multiplier = 0L;

                            while (scanner.hasNext()) {
                                if (scanner.hasNextLong()) {
                                    spaceShared = scanner.nextLong();
                                } else {
                                    String next = scanner.next();
                                    if (next.equalsIgnoreCase("kb")) {
                                        multiplier = Utils.spaceStringToBytes("kb");
                                    } else if (next.equalsIgnoreCase("mb")) {
                                        multiplier = Utils.spaceStringToBytes("mb");
                                    } else if (next.equalsIgnoreCase("gb")) {
                                        multiplier = Utils.spaceStringToBytes("gb");
                                    } else if (next.equalsIgnoreCase("tb")) {
                                        multiplier = Utils.spaceStringToBytes("tb");
                                    } else if (next.equalsIgnoreCase("pb")) {
                                        multiplier = Utils.spaceStringToBytes("pb");
                                    }
                                }
                            }
                            scanner.close();
                            dedupe.setSizeShared(spaceShared * multiplier);
                        }
                    }
                }
            } catch (Exception e) {
                logger.error(
                        "getDedupeSizeShared:Unable to get dedupe-size-shared vserver:" + dataSvm.getName() + " volume:" + dedupe.getVolume().getName());
            }
        }
        return true;
    }

    private static void addCommandElement(StringBuffer commandToExecute, NaElement argsElement, String item) {
        commandToExecute.append(item);
        commandToExecute.append(" ");
        argsElement.addNewChild("arg", item);
    }
}
