/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/volume/CModeDedupeStatusDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader.volume;

import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.volume.CModeDedupeImpl;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.text.DateFormat;
import java.util.Date;
import java.util.Set;

public class CModeDedupeStatusDataLoader extends AbstractDataLoader<CModeDedupeImpl> {

    private static final Logger LOGGER = Logger.getLogger(CModeDedupeStatusDataLoader.class);

    @Override
    protected boolean doLoad(CModeDedupeImpl dedupe, LoadType loadType,
                             Set<Resource> resources) throws DataLoaderException {
        LOGGER.debug("doLoad: begin");

        if (dedupe.isEnabled()) {
            NaElement input = new NaElement("sis-get");
            input.addNewChild("path", "/vol/" + dedupe.getVolume().getName());
            NaElement desiredAttributes = new NaElement("desired-attributes");
            NaElement sisStatusInfo = new NaElement("sis-status-info");
            sisStatusInfo.addChildElem(new NaElement("type"));
            sisStatusInfo.addChildElem(new NaElement("state"));
            sisStatusInfo.addChildElem(new NaElement("status"));
            sisStatusInfo.addChildElem(new NaElement("last-op-begin-timestamp"));
            sisStatusInfo.addChildElem(new NaElement("last-op-end-timestamp"));
            sisStatusInfo.addChildElem(new NaElement("schedule"));
            sisStatusInfo.addChildElem(new NaElement("is-compression-enabled"));
            sisStatusInfo.addChildElem(new NaElement("is-inline-compression-enabled"));
            desiredAttributes.addChildElem(sisStatusInfo);
            input.addChildElem(desiredAttributes);

            try {
                NaElement output = dedupe.getControllerImpl().invokeZapi(input, ZapiCache.SHORT_CACHE);
                NaElement attributes = output.getChildByName("attributes");
                if (attributes != null) {
                    NaElement sisInfo = attributes.getChildByName("sis-status-info");
                    dedupe.setEnabled("enabled".equals(sisInfo.getChildContent("state")));
                    dedupe.setType(sisInfo.getChildContent("type"));
                    dedupe.setState(sisInfo.getChildContent("state"));
                    dedupe.setStatus(sisInfo.getChildContent("status"));
                    dedupe.setLastBeginDate(DateFormat.getDateTimeInstance().format(new Date(
                            sisInfo.getChildLongValue("last-op-begin-timestamp", 0L) * 1000)));
                    dedupe.setLastEndDate(DateFormat.getDateTimeInstance().format(new Date(
                            sisInfo.getChildLongValue("last-op-end-timestamp", 0L) * 1000)));
                    dedupe.setSchedule(sisInfo.getChildContent("schedule"));
                    dedupe.setCompressionEnabled("true".equals(sisInfo.getChildContent("is-compression-enabled")));
                    dedupe.setInlineCompressionEnabled("true".equals(sisInfo.getChildContent("is-inline-compression-enabled")));
                }
            } catch (ApiFailedException e) {
                // lets not consider missing sis to be a serious problem, since we have filers all over
                // the unit test environment where this fails
                // dataVServer.setErrorEncountered(true);

                // but we do want to note that the volume doesn't have sis here - see BURT 788588
                dedupe.setEnabled(false);
                dedupe.setCompressionEnabled(false);
                LOGGER.trace("Error while loading the sis status for volume " + dedupe.getVolume().getName(), e);
            }
        } else {
            LOGGER.debug("Sis not enabled on volume " + dedupe.getVolume().getName() +
                                 ", skipping loading of dedupe status.");
        }
        LOGGER.debug("doLoad: end");
        return true;
    }
}
