/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/dataloader/volume/CModeVolumeDataLoader.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.dataloader.volume;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import com.netapp.offtap3.Controller;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.VolumeResource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.UnixPermissionImpl;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.internal.ontap.MCCSlapperThread;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataParser;
import com.netapp.offtap3.internal.ontap.dataloader.OntapCachingDataLoader;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.utils.StatsCollector;
import com.netapp.offtap3.internal.ontap.volume.CModeDedupeImpl;
import com.netapp.offtap3.internal.ontap.volume.CModeVolumeImpl;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;
import com.netapp.offtap3.utils.StringUtils;

public class CModeVolumeDataLoader extends OntapCachingDataLoader<CModeVolumeImpl> {

    private static final Logger logger = Logger.getLogger(CModeVolumeDataLoader.class);
    private static final Map<VolumeResource, String> resourceAttributeMap = new HashMap<VolumeResource, String>();

    static {
    	resourceAttributeMap.put(VolumeResource.id, "volume-id-attributes");
        resourceAttributeMap.put(VolumeResource.state, "volume-state-attributes");
        resourceAttributeMap.put(VolumeResource.inode, "volume-inode-attributes");
        resourceAttributeMap.put(VolumeResource.striping, "volume-striping-attributes");
        resourceAttributeMap.put(VolumeResource.space, "volume-space-attributes");
        resourceAttributeMap.put(VolumeResource.sis, "volume-sis-attributes");
        resourceAttributeMap.put(VolumeResource.vm_align, "volume-vm-align-attributes");
        resourceAttributeMap.put(VolumeResource.auto_size, "volume-autosize-attributes");
        resourceAttributeMap.put(VolumeResource.performance, "volume-performance-attributes");
        resourceAttributeMap.put(VolumeResource.language, "volume-language-attributes");
        resourceAttributeMap.put(VolumeResource.clone, "volume-clone-attributes");
        resourceAttributeMap.put(VolumeResource.qos, "volume-qos-attributes");
        resourceAttributeMap.put(VolumeResource.snapshot_autodelete, "volume-snapshot-autodelete-attributes");
        resourceAttributeMap.put(VolumeResource.snapdir, "volume-snapshot-attributes");
        resourceAttributeMap.put(VolumeResource.exports, "volume-export-attributes");
        resourceAttributeMap.put(VolumeResource.security, "volume-security-attributes");
    }

    private OfftapPreferences prefs;

    public CModeVolumeDataLoader(DataCache<NaElement, InternalZapiHandler> dataCache, OfftapPreferences prefs) {
        super(dataCache);
        this.prefs = prefs;
    }

    @Override
    protected java.util.Map<Resource, NaElement> retrieve(CModeVolumeImpl volume, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("retrieve: begin");
        NaElement input = new NaElement("volume-get-iter");
        Controller controller = ((Controller)getDataCache().getDataRetriever());
        Map<Resource, NaElement> results = loadVolumeInformation(volume,
				loadType, resources, input, controller);
        // lets see if we got a uuid.
        if (!StringUtils.isReal(volume.getUuid())) {
        	// this tends to be bad.  This happens when we are in MCC failover and we get the offlined/dead version of the volume.
        	// so lets see if perchance things have just failed over and try again.
        	MCCSlapperThread thread = MCCManager.getInstance().getMCCSlapperThreadCanBeNull(controller.getConnection());
        	logger.debug("Got crummy looking volume back - considering mcc failover check");
        	if (thread != null) {
        		if (thread.synchronouslyExecuteCheck(true)) {
        			logger.debug("Mcc failover check said yes");
        			results = loadVolumeInformation(volume,
        					loadType, resources, input, controller);
        		} else {
        			logger.debug("Mcc failover check said no");
        		}
        		
        	}
        }
        return results;
    }

	private Map<Resource, NaElement> loadVolumeInformation(
			CModeVolumeImpl volume, LoadType loadType, Set<Resource> resources,
			NaElement input, Controller controller) {
		if (loadType == LoadType.refresh) {
			StatsCollector.getInstance().addCounter("refresh-targeted-volume");
            addQuery(input, volume.getName());
        } else {
        	StatsCollector.getInstance().addCounter("refresh-whole-volume");
        }
        addInput(input, resources);
        logger.debug("Input is " + input);

        CdotIterativeLoader<Map<Resource, NaElement>> loader = new CdotIterativeLoader<Map<Resource, NaElement>>(
                getDataCache().getDataRetriever(), input, new CModeVolumeLoadParser(resources), prefs.getIteratorSize(),
                ZapiCache.SHORT_CACHE);

        Map<Resource, NaElement> results = new SafeConcurrentHashMap<Resource, NaElement>();
        try {
            loader.load(results);
        } catch (ApiFailedException e) {
            throw new DataLoaderException(controller + " failed to load volume information", e);
        }
		return results;
	}

    @Override
    protected NaElement retrieve(CModeVolumeImpl loadTarget, Resource resource, Refreshable refreshable) {
        Map<Resource, NaElement> dataMap = retrieve(loadTarget, LoadType.refresh, Collections.singleton(resource));
        return dataMap.get(resource);
    }

    private void addQuery(NaElement input, String volumeName) {
        NaElement query = new NaElement("query");
        NaElement volumeAttributes = new NaElement("volume-attributes");
        NaElement volumeIdAttributes = new NaElement("volume-id-attributes");
        volumeIdAttributes.addNewChild("name", volumeName);
        volumeAttributes.addChildElem(volumeIdAttributes);
        query.addChildElem(volumeAttributes);
        input.addChildElem(query);
    }

    private void addInput(NaElement input, Set<Resource> resources) {
        NaElement desiredVolumeAttributes = new NaElement("volume-attributes");
        NaElement volumeIdAttributes = new NaElement("volume-id-attributes");
        volumeIdAttributes.addChildElem(new NaElement("name"));       
        desiredVolumeAttributes.addChildElem(volumeIdAttributes);

        for (Resource resource : resources) {
            VolumeResource volumeResource = (VolumeResource) resource;
            switch (volumeResource) {              
				case id:
					volumeIdAttributes.addChildElem(new NaElement("containing-aggregate-name"));
					volumeIdAttributes.addChildElem(new NaElement("containing-aggregate-uuid"));
					break;
					
				case state:
                    NaElement volumeStateAttributes = new NaElement("volume-state-attributes");
                    volumeStateAttributes.addChildElem(new NaElement("state"));
                    volumeStateAttributes.addChildElem(new NaElement("is-inconsistent"));
                    volumeStateAttributes.addChildElem(new NaElement("is-vserver-root"));
                    volumeStateAttributes.addChildElem(new NaElement("is-node-root"));
                    volumeStateAttributes.addChildElem(new NaElement("is-cluster-volume"));
                    volumeStateAttributes.addChildElem(new NaElement("in-nvfailed-state"));
                    desiredVolumeAttributes.addChildElem(volumeStateAttributes);
                    break;
                    
                case inode:
                    NaElement volumeInodeAttributes = new NaElement("volume-inode-attributes");
                    volumeInodeAttributes.addChildElem(new NaElement("files-total"));
                    volumeInodeAttributes.addChildElem(new NaElement("files-used"));
                    desiredVolumeAttributes.addChildElem(volumeInodeAttributes);
                    break;

                case striping:
                    NaElement volumeStripingAttributes = new NaElement("volume-striping-attributes");
                    volumeStripingAttributes.addChildElem(new NaElement("stripe-width"));
                    desiredVolumeAttributes.addChildElem(volumeStripingAttributes);
                    break;

                case space:
                    NaElement volumeSpaceAttributes = new NaElement("volume-space-attributes");
                    volumeSpaceAttributes.addChildElem(new NaElement("percentage-snapshot-reserve"));
                    volumeSpaceAttributes.addChildElem(new NaElement("percentage-snapshot-reserve-used"));
                    volumeSpaceAttributes.addChildElem(new NaElement("snapshot-reserve-size"));
                    volumeSpaceAttributes.addChildElem(new NaElement("size-total"));
                    volumeSpaceAttributes.addChildElem(new NaElement("size-used"));
                    volumeSpaceAttributes.addChildElem(new NaElement("size-available"));
                    volumeSpaceAttributes.addChildElem(new NaElement("percentage-size-used"));
                    volumeSpaceAttributes.addChildElem(new NaElement("space-guarantee"));
                    volumeSpaceAttributes.addChildElem(new NaElement("is-space-guarantee-enabled"));
                    volumeSpaceAttributes.addChildElem(new NaElement("percentage-fractional-reserve"));
                    desiredVolumeAttributes.addChildElem(volumeSpaceAttributes);
                    break;

                case sis:
                    NaElement volumeSisAttributes = new NaElement("volume-sis-attributes");
                    volumeSisAttributes.addChildElem(new NaElement("percentage-deduplication-space-saved"));
                    volumeSisAttributes.addChildElem(new NaElement("deduplication-space-saved"));
                    volumeSisAttributes.addChildElem(new NaElement("is-sis-volume"));
                    desiredVolumeAttributes.addChildElem(volumeSisAttributes);
                    break;

                case vm_align:
                    NaElement volumeVMAlignAttributes = new NaElement("volume-vm-align-attributes");
                    volumeVMAlignAttributes.addChildElem(new NaElement("vm-align-sector"));
                    volumeVMAlignAttributes.addChildElem(new NaElement("vm-align-suffix"));
                    desiredVolumeAttributes.addChildElem(volumeVMAlignAttributes);
                    break;

                case auto_size:
                    NaElement volumeAutosizeAttributes = new NaElement("volume-autosize-attributes");
                    volumeAutosizeAttributes.addChildElem(new NaElement("increment-size"));
                    volumeAutosizeAttributes.addChildElem(new NaElement("is-enabled"));
                    volumeAutosizeAttributes.addChildElem(new NaElement("maximum-size"));
                    desiredVolumeAttributes.addChildElem(volumeAutosizeAttributes);
                    break;

                case performance:
                    NaElement volumePerformanceAttributes = new NaElement("volume-performance-attributes");
                    volumePerformanceAttributes.addChildElem(new NaElement("is-atime-update-enabled"));
                    desiredVolumeAttributes.addChildElem(volumePerformanceAttributes);
                    break;

                case language:
                    NaElement volumeLanguageAttributes = new NaElement("volume-language-attributes");
                    volumeLanguageAttributes.addChildElem(new NaElement("is-convert-ucode-enabled"));
                    volumeLanguageAttributes.addChildElem(new NaElement("is-create-ucode-enabled"));
                    desiredVolumeAttributes.addChildElem(volumeLanguageAttributes);
                    break;

                case mirror:
                    NaElement volumeMirrorAttributes = new NaElement("volume-mirror-attributes");
                    volumeMirrorAttributes.addChildElem(new NaElement("is-data-protection-mirror"));
                    volumeMirrorAttributes.addChildElem(new NaElement("is-load-sharing-mirror"));
                    desiredVolumeAttributes.addChildElem(volumeMirrorAttributes);
                    break;

                case clone:
                    NaElement volumeCloneAttributes = new NaElement("volume-clone-attributes");
                    volumeCloneAttributes.addChildElem(new NaElement("clone-child-count"));
                    volumeCloneAttributes.addChildElem(new NaElement("volume-clone-parent-attributes"));
                    desiredVolumeAttributes.addChildElem(volumeCloneAttributes);
                    break;

                case qos:
                    NaElement volumeQosAttributes = new NaElement("volume-qos-attributes");
                    volumeQosAttributes.addChildElem(new NaElement("policy-group-name"));
                    desiredVolumeAttributes.addChildElem(volumeQosAttributes);
                    break;

                case snapshot_autodelete:
                    NaElement snapshotAutodeleteAttributes = new NaElement("volume-snapshot-autodelete-attributes");
                    snapshotAutodeleteAttributes.addChildElem(new NaElement("is-autodelete-enabled"));
                    snapshotAutodeleteAttributes.addChildElem(new NaElement("trigger"));
                    snapshotAutodeleteAttributes.addChildElem(new NaElement("commitment"));
                    desiredVolumeAttributes.addChildElem(snapshotAutodeleteAttributes);
                    break;

                case snapdir:
                    NaElement snapshotAttributes = new NaElement("volume-snapshot-attributes");
                    snapshotAttributes.addChildElem(new NaElement("snapdir-access-enabled"));
                    desiredVolumeAttributes.addChildElem(snapshotAttributes);
                    break;

                case exports:
                    NaElement volumeExportAttributes = new NaElement("volume-export-attributes");
                    desiredVolumeAttributes.addChildElem(volumeExportAttributes);
                    break;

                case security:
                    NaElement securityAttributes = new NaElement("volume-security-attributes");
                    NaElement unixAttributes = new NaElement("volume-security-unix-attributes");
                    securityAttributes.addChildElem(unixAttributes);
                    desiredVolumeAttributes.addChildElem(securityAttributes);
                    break;
            }
        }
        NaElement desiredAttributes = new NaElement("desired-attributes");
        desiredAttributes.addChildElem(desiredVolumeAttributes);
        input.addChildElem(desiredAttributes);
    }

    @Override
    protected boolean parse(CModeVolumeImpl vol, NaElement data, Resource resource) {
        VolumeResource volumeResource = (VolumeResource)resource;
        NaElement volumeData = findVolumeData(data, vol.getName());
        
        if (volumeData != null) {        	
            switch (volumeResource) {      
            	case id:
            		NaElement volumeId = volumeData.getChildByName("volume-id-attributes");
            		if (volumeId != null) {
            			vol.setAggregateUUID(volumeId.getChildContent("containing-aggregate-uuid"));
               	 		vol.setContainingAggregate(volumeId.getChildContent("containing-aggregate-name"));
            		}
            		break;            
                case state:
                    NaElement volumeState = volumeData.getChildByName("volume-state-attributes");
                    if (volumeState != null) {
                        vol.setState(volumeState.getChildContent("state"));
                        vol.setInconsistent(Boolean.parseBoolean(volumeState.getChildContent("is-inconsistent")));
                        vol.setVserverRoot(Boolean.parseBoolean(volumeState.getChildContent("is-vserver-root")));
                        vol.setClusterVolume(Boolean.parseBoolean(volumeState.getChildContent("is-cluster-volume")));
                        vol.setNodeRoot(Boolean.parseBoolean(volumeState.getChildContent("is-node-root")));
                        vol.setInNvfailedState(Boolean.parseBoolean(volumeState.getChildContent("in-nvfailed-state")));
                        vol.setType("flex");
                    }
                    break;
                case inode:
                    NaElement volumeInode = volumeData.getChildByName("volume-inode-attributes");
                    if(volumeInode != null) {
                        vol.setFilesTotal(volumeInode.getChildLongValue("files-total", 0L));
                        vol.setFilesUsed(volumeInode.getChildLongValue("files-used", 0L));
                    }
                    break;
                case striping:
                    NaElement volumeStripingAttributes = volumeData.getChildByName("volume-striping-attributes");
                    if (volumeStripingAttributes != null) {
                        String stripeWidth = volumeStripingAttributes.getChildContent("stripe-width");
                        try {
                            if (stripeWidth != null) {
                                vol.setStripeWidth(Integer.parseInt(stripeWidth));
                            } else {
                                vol.setStripeWidth(-1);
                            }
                        } catch (NumberFormatException e) {
                            logger.error("parse: " + vol.getControllerImpl().getName() + " " + vol.getName() +
                                                 " Unable to determine stripe width.  Error parsing value \"" + stripeWidth + "\".");
                        }
                    } else {
                        vol.setStripeWidth(-1);
                    }
                    break;
                case space:
                    NaElement volumeSpace = volumeData.getChildByName("volume-space-attributes");
                    if(volumeSpace != null) {                    	
                        vol.setSnapshotPercentReserved(volumeSpace.getChildIntValue("percentage-snapshot-reserve", 0));
                        vol.setSnapshotReservePercentUsed(
                                volumeSpace.getChildIntValue("percentage-snapshot-reserve-used", 0));
                        vol.setSnapshotSpaceReserved(volumeSpace.getChildLongValue("snapshot-reserve-size", 0L));
                        // Aproximate this value with a calculation for cDOT as it doesn't seem to be available.
                        if (vol.getSnapshotReservePercentUsed() > 0) {
                            long value = ((long) (vol.getSnapshotReservePercentUsed() / 100)) * vol.getSnapshotSpaceReserved().toBytes();
                            vol.setSnapshotReserveUsed(value);
                        }
                        vol.setSizeTotal(volumeSpace.getChildLongValue("size-total", 0L));
                        vol.setSizeUsed(volumeSpace.getChildLongValue("size-used", 0L));
                        long sizeAvailable = volumeSpace.getChildLongValue("size-available", 0L);
                        vol.setSizeAvailable(sizeAvailable);
                        int percentageUsed = volumeSpace.getChildIntValue("percentage-size-used", 0);
                        vol.setPercentageUsed(percentageUsed);
                        vol.setSpaceReserveEnabled(Boolean.parseBoolean(volumeSpace.getChildContent("is-space-guarantee-enabled")));
                        vol.setSpaceReserve(volumeSpace.getChildContent("space-guarantee"));
                        vol.setFractionalReserve(volumeSpace.getChildIntValue("percentage-fractional-reserve", 0));
                    }
                    break;
                case sis:
                    NaElement sisInfoAttrs = volumeData.getChildByName("volume-sis-attributes");
                    if (sisInfoAttrs != null) {
                        CModeDedupeImpl dedupe = new CModeDedupeImpl(vol, prefs);
                        dedupe.setSizeSaved(sisInfoAttrs.getChildLongValue("deduplication-space-saved", 0L));
                        dedupe.setPercentSaved(sisInfoAttrs.getChildIntValue("percentage-deduplication-space-saved", 0));
                        dedupe.setEnabled("true".equals(sisInfoAttrs.getChildContent("is-sis-volume")));
                        dedupe.setVolume(vol);
                        vol.setDedupe(dedupe.getProxy());
                    }
                    break;
                case vm_align:
                    NaElement vmAlignInfoAttrs = volumeData.getChildByName("volume-vm-align-attributes");
                    if (vmAlignInfoAttrs != null){
                        vol.setVmAlignSector(vmAlignInfoAttrs.getChildIntValue("vm-align-sector", 0));
                        vol.setVmAlignSuffix(vmAlignInfoAttrs.getChildContent("vm-align-suffix"));
                    }
                    break;
                case auto_size:
                    NaElement autosizeInfo = volumeData.getChildByName("volume-autosize-attributes");
                    if(autosizeInfo != null) {
                        vol.setAutoSizeEnabled(Boolean.parseBoolean(autosizeInfo.getChildContent("is-enabled")));
                        vol.setAutoSizeIncrement(autosizeInfo.getChildLongValue("increment-size", 0L));
                        vol.setAutoSizeMax(autosizeInfo.getChildLongValue("maximum-size", 0L));
                    } else {
                        vol.setAutoSizeEnabled(false);
                    }
                    break;
                case performance:
                    NaElement performance = volumeData.getChildByName("volume-performance-attributes");
                    if(performance != null) {
                        vol.setAtimeUpdateEnabled(Boolean.parseBoolean(performance.getChildContent("is-atime-update-enabled")));
                    }
                    break;
                case language:
                    NaElement volumeLanguage = volumeData.getChildByName("volume-language-attributes");
                    if (volumeLanguage != null) {
                        vol.setConvertUcodeEnabled(Boolean.parseBoolean(volumeLanguage.getChildContent("is-convert-ucode-enabled")));
                        vol.setCreateUcodeEnabled(Boolean.parseBoolean(volumeLanguage.getChildContent("is-create-ucode-enabled")));
                    }
                    break;
                case clone:
                    NaElement cloneInfo = volumeData.getChildByName("volume-clone-attributes");
                    if(cloneInfo != null) {
                        vol.setCloneChildCount(cloneInfo.getChildIntValue("clone-child-count", 0));
                        NaElement parentInfo = cloneInfo.getChildByName("volume-clone-parent-attributes");
                        if(parentInfo != null) {
                            vol.setParentVolumeName(parentInfo.getChildContent("name"));
                            vol.setParentSnapshotName(parentInfo.getChildContent("snapshot-name"));
                        }
                    }
                    break;
                case qos:
                    NaElement qosInfo = volumeData.getChildByName("volume-qos-attributes");
                    if (qosInfo != null && qosInfo.getChildContent("policy-group-name") != null) {
                        vol.setQosPolicyGroup(qosInfo.getChildContent("policy-group-name"));
                    }
                    break;
                case snapshot_autodelete:
                    NaElement snapshotAutodeleteInfo = volumeData.getChildByName("volume-snapshot-autodelete-attributes");
                    if (snapshotAutodeleteInfo != null) {
                        boolean isAutoDeleteEnabled = Boolean.parseBoolean(snapshotAutodeleteInfo.getChildContent("is-autodelete-enabled"));
                        vol.setAutoDeleteEnabled(isAutoDeleteEnabled);
                        if(isAutoDeleteEnabled){
                            vol.setAutoDeleteTrigger(snapshotAutodeleteInfo.getChildContent("trigger"));
                            vol.setAutoDeleteCommitment(snapshotAutodeleteInfo.getChildContent("commitment"));
                        }
                    }
                    break;
                case snapdir:
                    NaElement snapshotInfo = volumeData.getChildByName("volume-snapshot-attributes");
                    if (snapshotInfo != null) {
                        boolean isSnapDirAccessEnabled = Boolean.parseBoolean(snapshotInfo.getChildContent("snapdir-access-enabled"));
                        vol.setSnapDirAccessEnabled(isSnapDirAccessEnabled);
                    }
                    break;
                case exports:
                    NaElement exportInfo = volumeData.getChildByName("volume-export-attributes");
                    if(exportInfo != null) {
                        vol.setExportPolicyName(exportInfo.getChildContent("policy"));
                    }
                    break;
                case security:
                    NaElement securityInfo = volumeData.getChildByName("volume-security-attributes");
                    if (securityInfo != null) {
                        NaElement unixPermissions = securityInfo.getChildByName("volume-security-unix-attributes");
                        UnixPermissionImpl unixPermission = new UnixPermissionImpl(vol);
                        unixPermission.setPermissions(unixPermissions.getChildContent("permissions"));
                        unixPermission.setUserId(unixPermissions.getChildIntValue("user-id", -1));
                        unixPermission.setGroupId(unixPermissions.getChildIntValue("group-id", -1));
                        vol.setUnixPermissions(unixPermission);
                    }
                    break;
            }
        } else {
            logger.debug("Unable to find volumeData for " + vol.getName() + " and resource of " + volumeResource);
            // dsarisky 6/30/14 - I see the occasional Ontap bug here where sections aren't returned for autosize under certain circumstances.
            // Slapping a check in here for now and we'll see if we see similar instances in other places.
            if (VolumeResource.auto_size.equals(volumeResource)) {
                vol.setAutoSizeEnabled(false);
                vol.setAutoSizeIncrement(0);
                vol.setAutoSizeMax(0);
            }
            if (VolumeResource.space.equals(volumeResource)) {
            	logger.error("Effectively blanking out volume information for " + vol.getName() + " and resource of " + volumeResource);
                vol.setSnapshotPercentReserved(0);
                vol.setSnapshotReservePercentUsed(0);
                vol.setSnapshotSpaceReserved(0);                
                vol.setSizeTotal(0);
                vol.setSizeUsed(0);
                vol.setSizeAvailable(0);
                vol.setPercentageUsed(0);
                vol.setSpaceReserveEnabled(false);
                vol.setSpaceReserve("none");
                vol.setFractionalReserve(0);
            }
        }
        return true;
    }

    @Override
    protected Refreshable parse(CModeVolumeImpl loadTarget, NaElement data, Resource resource, Refreshable reloadTarget) {
        if (parse(loadTarget, data, resource)) {
            if (resource == VolumeResource.sis) {
                return loadTarget.getDedupe();
            } else if (resource == VolumeResource.security) {
                return loadTarget.getUnixPermission();
            }
        }
        return null;
    }

    /**
     * Returns the volume data element associated with the specified volume.
     */
    private NaElement findVolumeData(NaElement data, String volumeName) {
        if (data != null) {
            List<NaElement> volumes = data.getChildren();
            if (volumes != null) {
                for (NaElement volume : volumes) {
                    if (volumeName.equals(volume.getName())) {
                        return volume;
                    }
                }
            }
        }
        return null;
    }

    private class CModeVolumeLoadParser implements DataParser<Map<Resource, NaElement>> {
        private Set<Resource> resources;

        CModeVolumeLoadParser(Set<Resource> resources) {
            this.resources = resources;
        }

        /**
         * Parse through the next set of volume results and add them to the cummulativeResults.
         */
        @Override 
        public void parse(Map<Resource, NaElement> cummulativeResults, NaElement newResults) {
            NaElement attrList = newResults.getChildByName("attributes-list");
            if (attrList != null) {
                List<NaElement> volumeInfos = attrList.getChildren();
                for (NaElement volumeInfo : volumeInfos) {
                    String name = volumeInfo.getChildByName("volume-id-attributes").getChildContent("name");
                    parseVolumeResources(cummulativeResults, volumeInfo, name);
                }
            }
        }

        /**
         * Parses the attributes and puts the pieces associated with each resource into the cummulativeResults. Each volume
         * will have its own sub element hanging off the results element for each resource.
         */
        private void parseVolumeResources(Map<Resource, NaElement> cummulativeResults, NaElement attributes, String volumeName) {
            for (VolumeResource resource : resourceAttributeMap.keySet()) {
                if (resources.contains(resource)) {
                    String attributeName = resourceAttributeMap.get(resource);
                    NaElement results = getResults(resource, cummulativeResults);
                    NaElement attribute = attributes.getChildByName(attributeName);
                    if (attribute != null) {
                        NaElement volumeResults = new NaElement(volumeName);
                        volumeResults.addChildElem(attribute);
                        results.addChildElem(volumeResults);
                    }
                }
            }
        }

        /**
         * Returns the results element for the specified resource. If none exists, it is created and added to the
         * cummulative results.
         */
        private NaElement getResults(VolumeResource resource, Map<Resource, NaElement> cummulativeResults) {
            NaElement results = cummulativeResults.get(resource);
            if (results == null) {
                results = new NaElement(resource.name() + "_results");
                cummulativeResults.put(resource, results);
            }
            return results;
        }
    }
}
