package com.netapp.offtap3.internal.ontap.dataloader.volume;

import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.dataloader.AbstractExportDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.CumulativeDataParser;
import com.netapp.offtap3.internal.ontap.exports.QtreeImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.volume.CModeVolumeImpl;
import com.netapp.offtap3.internal.ontap.volume.OntapVolumeImpl;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.Set;


public class CModeVolumeExportDataLoader extends AbstractExportDataLoader<CModeVolumeImpl> {

    private static final Logger LOGGER = Logger.getLogger(CModeVolumeExportDataLoader.class);

    private final CdotIterativeLoader<NaElement> quotaReportLoader;
    private final CdotIterativeLoader<NaElement> qtreeListLoader;

    public CModeVolumeExportDataLoader(OfftapPreferences prefs, DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);

        quotaReportLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(),
                                                               new NaElement("quota-report-iter"),
                                                               new CumulativeDataParser(), prefs.getIteratorSize(),
                                                               ZapiCache.SHORT_CACHE);


        qtreeListLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(),
                                                             new NaElement("qtree-list-iter"),
                                                             new CumulativeDataParser(), prefs.getIteratorSize(),
                                                             ZapiCache.SHORT_CACHE);

    }

    @Override
    protected QtreeImpl createQtree(OntapVolumeImpl volume, String name) {
        return new QtreeImpl(name, volume, false);
    }

    @Override
    protected java.util.Map<Resource, NaElement> retrieve(CModeVolumeImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        InternalZapiHandler connection = getDataCache().getDataRetriever();

        LOGGER.debug(connection + " retrieving qtrees and exports");
        NaElement container = new NaElement("container");

        try {
            // get ALL of the qtrees for this controller.  we'll share the results with each volume
            NaElement qtrees = new NaElement("qtrees");
            qtreeListLoader.load(qtrees);
            container.addChildElem(qtrees);

            // get ALL of the exports for this controller. we'll share the results with each volume
            NaElement exportList = connection.invokeZapi(new NaElement("nfs-exportfs-list-rules-2"),
                                                         ZapiCache.SHORT_CACHE);
            container.addChildElem(exportList.getChildByName("rules"));

            NaElement quotas = new NaElement("quotas");
            quotaReportLoader.load(quotas);
            container.addChildElem(quotas);
        } catch (ApiFailedException e) {
            if ((e.getErrno() == 13001) && e.getReason().contains("quotas are off")) {
                LOGGER.warn(connection + " does not have any quotas");
            } else {
                throw new DataLoaderException(connection + " could not load qtrees", e);
            }
        }
        return Collections.singletonMap(resources.iterator().next(), container);
    }

}
