package com.netapp.offtap3.internal.ontap.dataloader.volume;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.dataloader.CdotIterativeLoader;
import com.netapp.offtap3.internal.ontap.dataloader.CumulativeDataParser;
import com.netapp.offtap3.internal.ontap.dataloader.OntapCachingDataLoader;
import com.netapp.offtap3.internal.ontap.snap.CModeSnapshotImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.volume.CModeVolumeImpl;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.Snapshot;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class CModeVolumeSnapshotDataLoader extends OntapCachingDataLoader<CModeVolumeImpl> {

    private static final Logger logger = Logger.getLogger(CModeVolumeSnapshotDataLoader.class);

    private CdotIterativeLoader<NaElement> iterativeLoader;

    public CModeVolumeSnapshotDataLoader(DataCache<NaElement, InternalZapiHandler> dataCache, OfftapPreferences prefs) {
        super(dataCache);
        NaElement input = new NaElement("snapshot-get-iter");
        NaElement desiredAttributes = new NaElement("desired-attributes");
        NaElement snapshotInfo = new NaElement("snapshot-info");
        snapshotInfo.addChildElem(new NaElement("name"));
        snapshotInfo.addChildElem(new NaElement("access-time"));
        snapshotInfo.addChildElem(new NaElement("volume"));
        desiredAttributes.addChildElem(snapshotInfo);
        input.addChildElem(desiredAttributes);

        iterativeLoader = new CdotIterativeLoader<NaElement>(dataCache.getDataRetriever(), input,
                                                             new CumulativeDataParser(), prefs.getIteratorSize(),
                                                             ZapiCache.SHORT_CACHE);
    }

    @Override
    protected Map<Resource, NaElement> retrieve(CModeVolumeImpl object, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        logger.debug("retrieve: start");
        NaElement result = new NaElement("results");
        try {
            iterativeLoader.load(result);
        } catch (ApiFailedException e) {
            throw new DataLoaderException(e);
        }
        logger.debug("retrieve: end");
        return Collections.singletonMap(resources.iterator().next(), result);
    }

    @Override
    protected boolean parse(CModeVolumeImpl volume, NaElement data, Resource resource) {
        List<Snapshot> snapshots = new ArrayList<Snapshot>();
        List<NaElement> snapshotInfos = data.getChildren();
        for (NaElement snapshotInfo : snapshotInfos) {
            // pull the info from the XML record
            // make a snapshot bean
            // find the right volume
            // add the snapshot to it.
            String volumeName = snapshotInfo.getChildContent("volume");
            if (volumeName.equals(volume.getName())) {
                CModeSnapshotImpl snapshot = new CModeSnapshotImpl(volume);
                snapshot.setDate(snapshotInfo.getChildLongValue("access-time", 0));
                snapshot.setName(snapshotInfo.getChildContent("name"));
                snapshots.add(snapshot);
            }
        }
        volume.setSnapshots(snapshots);

        return true;
    }

    @Override
    protected Refreshable doRefresh(CModeVolumeImpl loadTarget, Resource resource, Refreshable refreshTarget) {
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource));
        return loadTarget.getSnapshotByName(refreshTarget.getName());
    }
}
