package com.netapp.offtap3.internal.ontap.dataloader.volume;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.DataCache;
import com.netapp.offtap3.internal.ontap.dataloader.AbstractExportDataLoader;
import com.netapp.offtap3.internal.ontap.volume.FilerVolumeImpl;
import com.netapp.offtap3.internal.ontap.volume.OntapVolumeImpl;
import com.netapp.offtap3.internal.ontap.exports.QtreeImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.Map;
import java.util.Set;

public class FilerExportDataLoader extends AbstractExportDataLoader<FilerVolumeImpl> {

    private static final Logger logger = Logger.getLogger(FilerExportDataLoader.class);

    public FilerExportDataLoader(DataCache<NaElement, InternalZapiHandler> dataCache) {
        super(dataCache);
    }

    @Override
    protected QtreeImpl createQtree(OntapVolumeImpl volume, String name) {
        return new QtreeImpl(name, volume, true);
    }

    @Override
    protected NaElement retrieve(FilerVolumeImpl loadTarget, Resource resource, Refreshable refreshable) {
        Map<Resource, NaElement> dataMap = retrieve(loadTarget, LoadType.refresh, Collections.singleton(resource), refreshable);
        return dataMap.get(resource);
    }

    @Override
    protected Map<Resource, NaElement> retrieve(FilerVolumeImpl filerVolume, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        return retrieve(filerVolume, loadType, resources, null);
    }

    protected Map<Resource, NaElement> retrieve(FilerVolumeImpl filerVolume, LoadType loadType, Set<Resource> resources, Refreshable refreshable) {
        InternalZapiHandler connection = getDataCache().getDataRetriever();

        logger.debug(connection + " retrieving qtrees");
        NaElement container = new NaElement("container");

        try {
            // get ALL of the qtrees for this controller.  we'll share the results with each volume
            NaElement input = new NaElement("qtree-list");
            if (refreshable != null) {
                // During refresh of a qtree in this volume, we only get the qtrees for this volume.
                input.addNewChild("volume", filerVolume.getName());
            }

            NaElement qtreeList = connection.invokeZapi(input, ZapiCache.SHORT_CACHE);
            container.addChildElem(qtreeList.getChildByName("qtrees"));

            // get ALL of the exports for this controller. we'll share the results with each volume
            NaElement exportList = connection.invokeZapi(new NaElement("nfs-exportfs-list-rules-2"),
                                                         ZapiCache.SHORT_CACHE);
            container.addChildElem(exportList.getChildByName("rules"));

            input = new NaElement("quota-report");
            if (loadType == LoadType.refresh) {
                // During refresh, we only get the quota reports for this volume.
                input.addNewChild("volume", filerVolume.getName());
            }
            NaElement quotas = connection.invokeZapi(input, ZapiCache.SHORT_CACHE);
            container.addChildElem(quotas.getChildByName("quotas"));

        }
        catch (ApiFailedException e) {
            if ((e.getErrno() == 13001) && e.getReason().contains("quotas are off")) {
                logger.warn(connection + " does not have any quotas");
            }
            else {
                throw new DataLoaderException(connection + " could not load qtrees", e);
            }
        }
        return Collections.singletonMap(resources.iterator().next(), container);
    }

}
