package com.netapp.offtap3.internal.ontap.dataloader.volume;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.snap.SnapshotImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.volume.FilerVolumeImpl;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.Filer;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.Snapshot;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

public class FilerVolumeSnapshotDataLoader<F extends Filer> extends AbstractDataLoader<FilerVolumeImpl<F>> {

    private static final Logger logger = Logger.getLogger(FilerVolumeSnapshotDataLoader.class);

    @Override
    protected boolean doLoad(FilerVolumeImpl<F> volume, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
        FilerImpl filer = volume.getControllerImpl();

        try {
            // This is the implementation of the calculateSnapshotReserveUsedActual7Mode from
            // MHC's ControllerUtil7Mode.  Adding this metric as MHC uses it.
            NaElement xi = new NaElement("snapshot-list-info");
            xi.addNewChild("volume", volume.getName());
            NaElement xo = filer.invokeZapi(xi, ZapiCache.LONG_CACHE);

            long totalUsed = 0;

            if (xo.getChildByName("snapshots") != null) {
                List<NaElement> snapshotInfos = xo.getChildByName("snapshots").getChildren();
                List<Snapshot> snapshots = new ArrayList<Snapshot>(snapshotInfos.size());

                for (NaElement snapshotInfo : snapshotInfos) {
                    totalUsed += snapshotInfo.getChildLongValue("total", 0);
                    SnapshotImpl snapshot = new SnapshotImpl<FilerVolumeImpl<F>, F, FilerImpl<F>>(volume);
                    snapshot.setDate(snapshotInfo.getChildLongValue("access-time", 0));
                    snapshot.setName(snapshotInfo.getChildContent("name"));
                    snapshots.add(snapshot);
                }
                volume.setSnapshots(snapshots);
                logger.debug(filer + " loaded " + snapshots.size() + " snapshots for volume " + volume.getName());
            }

            volume.setSnapshotReserveUsed(1024L * totalUsed);
        } catch (ApiFailedException e) {
            logger.error("Exception while calling snapshot-list-info to calculate snapshot used actual", e);
        }
        return true;
    }

    @Override
    protected Refreshable doRefresh(FilerVolumeImpl<F> loadTarget, Resource resource, Refreshable refreshTarget) {
        doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource));
        return loadTarget.getSnapshotByName(refreshTarget.getName());
    }
}
