package com.netapp.offtap3.internal.ontap.dataloader.volume;

import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.snap.SnapVaultImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.SnapVault;
import com.netapp.offtap3.ontap.NaElement;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;


public class SnapVaultDataLoader extends AbstractDataLoader<FilerImpl> {

    @Override
    protected boolean doLoad(FilerImpl filer, LoadType loadType, Set<Resource> resources) throws DataLoaderException {

        try {
            if (resources.contains(ControllerResource.primary_snapvaults)) {
                filer.setSnapVaultsPrimary(loadSnapVaults(filer, "primary"));
            }
            if (resources.contains(ControllerResource.secondary_snapvaults)) {
                filer.setSnapVaultsSecondary(loadSnapVaults(filer, "secondary"));
            }
        } catch (ApiFailedException nafe) {
            if (nafe.getErrno() == 13008) {
                // not licensed
                return true;
            } else {
                throw new DataLoaderException(filer + " failed to load snap vault data", nafe);
            }
        }

        return true;
    }

    protected List<SnapVault> loadSnapVaults(FilerImpl controller, String primarySecondary)  throws ApiFailedException {
        List<SnapVault> snapVaults = new ArrayList<SnapVault>();

        long maxGrab = controller.getOfftapPreferences().getIteratorSize();

        NaElement api = new NaElement("snapvault-" + primarySecondary + "-relationship-status-list-iter-start");
        NaElement xo = controller.invokeZapi(api, ZapiCache.NO_CACHE);

        // you'd think we could cache this, but 7-mode iteration stuff here is a frackin
        // mess, I'm just not gonna fool with it
        Long records = xo.getChildLongValue("records", 0L);
        String tag = xo.getChildContent("tag");

        boolean done = false;
        // grab is the maximum records to get per iteration.
        Long grab = records;

        while (!done) {
            if (grab > maxGrab) {
                grab = maxGrab;
            }
            if (records < 1) {
                break;
            }
            api = new NaElement("snapvault-" + primarySecondary + "-relationship-status-list-iter-next");
            api.addNewChild("tag", tag);
            api.addNewChild("maximum", grab + "");
            xo = controller.invokeZapi(api, ZapiCache.NO_CACHE);
            if (xo != null) {
                if (xo.getChildByName("status-list") != null) {
                    List<NaElement> pps = xo.getChildByName("status-list").getChildren();
                    for (final NaElement currXml : pps) {
                        SnapVaultImpl snapVault = new SnapVaultImpl(controller);
                        snapVault.setSourceSystem(currXml.getChildContent("source-system"));
                        snapVault.setSourcePath(currXml.getChildContent("source-path"));
                        snapVault.setDestinationSystem(currXml.getChildContent("destination-system"));
                        snapVault.setDestinationPath(currXml.getChildContent("destination-path"));
                        snapVault.setLagTime(currXml.getChildLongValue("lag-time", 0));
                        snapVault.setMirrorTimestamp(currXml.getChildLongValue("mirror-timestamp", 0));
                        snapVault.setContents(currXml.getChildContent("contents"));
                        snapVault.setLastTransferType(currXml.getChildContent("last-transfer-type"));
                        snapVault.setLastTransferSize(currXml.getChildLongValue("last-transfer-size", 0));
                        snapVault.setLastTransferFromSystem(currXml.getChildContent("last-transfer-from-system"));
                        snapVault.setLastTransferFromPath(currXml.getChildContent("last-transfer-from-path"));
                        snapVault.setStatus(currXml.getChildContent("status"));
                        snapVault.setState(currXml.getChildContent("state"));
                        snapVault.setBaseSnapshot(currXml.getChildContent("base-snapshot"));
                        snapVaults.add(snapVault);
                    }
                }
            }
            records = records - grab;
            if (records < 1) {
                done = true;
            } else {
                grab = records;
            }
        }

        api = new NaElement("snapvault-" + primarySecondary + "-relationship-status-list-iter-end");
        api.addNewChild("tag", tag);
        controller.invokeZapi(api, ZapiCache.NO_CACHE);
        return snapVaults;
    }

}
