package com.netapp.offtap3.internal.ontap.exports;

import com.google.common.base.Objects;
import com.netapp.offtap3.internal.AbstractRefreshable;
import com.netapp.offtap3.internal.ontap.volume.OntapVolumeImpl;
import com.netapp.offtap3.ontap.Export;
import com.netapp.offtap3.ontap.Qtree;
import com.netapp.offtap3.ontap.SecurityStyle;
import com.netapp.offtap3.utils.SafeConcurrentHashMap;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Are associated to a volume.
 */
public class QtreeImpl extends AbstractRefreshable implements Qtree {

    private final String owningVolumeName;
    private final String path;

    private int id;
    private String status;
    private SecurityStyle securityStyle;
    private boolean hasQuota;
    private Map<String, Export> exports = new SafeConcurrentHashMap<String, Export>();

    public QtreeImpl(String name, OntapVolumeImpl volume, boolean volPrefix) {
        super(name, new Date(), volume);
        this.owningVolumeName = volume.getName();
        StringBuilder builder = new StringBuilder(volPrefix ? "/vol/" : "/").append(owningVolumeName);
        if (!getName().isEmpty()) {
            builder.append("/").append(getName());
        }
        path = builder.toString();
    }


    @Override
    public String getPath() {
        return path;
    }

    @Override
    public String getOwningVolume() {
        return owningVolumeName;
    }

    @Override
    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    @Override
    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    @Override
    public SecurityStyle getSecurityStyle() {
        return securityStyle;
    }

    public void setSecurityStyle(SecurityStyle securityStyle) {
        this.securityStyle = securityStyle;
    }

    @Override
    public boolean hasQuota() {
        return hasQuota;
    }

    public void setHasQuota(boolean hasQuota) {
        this.hasQuota = hasQuota;
    }

    public Map<String, Export> getExports() {
        return exports;
    }

    public void setExports(Map<String, Export> exports) {
        this.exports = exports;
    }

    @Override
    public Qtree refresh() {
        return (Qtree) super.refresh();
    }

    public String toString() {
        return Objects.toStringHelper(this).add("path", getPath())
                .add("status", getStatus())
                .add("security-style", getSecurityStyle())
                .add("hasQuota", hasQuota())
                .add("exports", getExports())
                .toString();
    }
}
