package com.netapp.offtap3.internal.ontap.file;

import java.util.Map;

/**
 * This class describes the $Bitmap file found in
 * an NTFS volume.
 */
public class NtfsBitmapFile {

    /**
     * The starting offset (in bytes) of the NTFS partition from the beginning of the file
     */
    private final long startingOffset;

    /**
     * The run list of the bitmap from the beginning of the file.  The key is the offset and the value is the length (in bytes)
     */
    private final Map<Long, Long> bitmapRunList;

    /**
     * The length (in bytes) of the bitmap file
     */
    private final long bitmapLength;

    /**
     * The cluster size configured for the NTFS partition
     */
    private final long clusterSize;

    private final String errorReason;

    public NtfsBitmapFile(String error) {
        this.startingOffset = 0;
        this.bitmapRunList = null;
        this.bitmapLength = 0;
        this.clusterSize = 0;
        this.errorReason = error;
    }

    public NtfsBitmapFile(long startingOffset, Map<Long, Long> bitmapRunList, long bitmapLength, long clusterSize) {
        this.startingOffset = startingOffset;
        this.bitmapRunList = bitmapRunList;
        this.bitmapLength = bitmapLength;
        this.clusterSize = clusterSize;
        this.errorReason = null;
    }

    public long getStartingOffset() {
        return startingOffset;
    }

    public Map<Long, Long> getBitmapRunList() {
        return bitmapRunList;
    }

    public long getBitmapLength() {
        return bitmapLength;
    }

    public long getClusterSize() {
        return clusterSize;
    }

    public String getErrorReason() {
        return errorReason;
    }

    public boolean isValid() {
        return (errorReason == null);
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("NtfsBitmapFile [");
        if (isValid()) {
            builder.append("startingOffset=");
            builder.append(startingOffset);
            builder.append(", bitmapRunList=");
            builder.append(bitmapRunList);
            builder.append(", bitmapLength=");
            builder.append(bitmapLength);
            builder.append(", clusterSize=");
            builder.append(clusterSize);
        } else {
            builder.append("errorReason=");
            builder.append(errorReason);
        }
        builder.append("]");
        return builder.toString();
    }
}
