/*
 *  
 * Copyright (c) 2010 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.file.ndmp;

/**
 * Typesafe holder for a value.
 * Typically used when working with anonymous inner classes to compensate for lack of closures in Java.
 * <pre>
 * {@code
 * final Holder<Foo> fooHolder = Holder.create();
 * bar.invoke(new Runnable() {
 *     public void run() {
 *         ...
 *         fooHolder.value = ...;
 *         ...
 *     }
 * });
 * // Do something with fooHolder.value
 * }
 * </pre>
 *
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2015/07/22 $
 */
public class Holder<T> {

   
    public T value;

    private Holder() { }

    private Holder(T value) {
        this.value = value;
    }

    public static <T> Holder<T> create() {
        return new Holder<T>();
    }

    public static <T> Holder<T> create(T value) {
        return new Holder<T>(value);
    }

}
