package com.netapp.offtap3.internal.ontap.intf;

import com.google.common.base.Objects;
import com.netapp.offtap3.IpInterface;
import com.netapp.offtap3.internal.AbstractRefreshable;
import com.netapp.offtap3.internal.HasProxy;

import java.util.ArrayList;
import java.util.List;

/**
 * Describes an interface on the storage controller
 */
public class IpInterfaceImpl extends AbstractRefreshable implements IpInterface {
    private String ipAddress;
    private String netmask;
    private String network;
    private String name;
    private List<String> protocols = new ArrayList<String>();

    public IpInterfaceImpl(String ipAddress, HasProxy loader) {
        super(loader);
        this.ipAddress = ipAddress;
    }

    @Override
    public String getIpAddress() {
        return ipAddress;
    }

    public void setNetmask(String netmask) {
        this.netmask = netmask;
    }

    @Override
    public String getNetmask() {
        return netmask;
    }

    public void setNetwork(String network) {
        this.network = network;
    }

    @Override
    public String getNetwork() {
        return network;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Override
    public String getName() {
        return name;
    }


    public void setProtocols(List<String> protocols) {
        this.protocols = protocols;
    }

    @Override
    public List<String> getProtocols() {
        return protocols;
    }


    @Override
    public boolean isIscsiEnabled() {
        return protocols.contains("iscsi");
    }

    @Override
    public IpInterface refresh() {
        return (IpInterface) super.refresh();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = super.hashCode();
        result = prime * result + ((name == null) ? 0 : name.hashCode());
        result = prime * result + ((ipAddress == null) ? 0 : ipAddress.hashCode());
        result = prime * result + ((netmask == null) ? 0 : netmask.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof IpInterfaceImpl) {
            IpInterfaceImpl other = (IpInterfaceImpl) obj;
            return Objects.equal(getName(), other.getName()) &&
                    Objects.equal(getIpAddress(), other.getIpAddress()) &&
                    Objects.equal(getNetmask(), other.getNetmask());
        }
        return false;
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this).add("ipAddress", ipAddress).toString();
    }
}
