package com.netapp.offtap3.internal.ontap.intf;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Set;

import org.apache.log4j.Logger;

import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.internal.IGroupImpl;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapDataController;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.IGroupOperationException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.OntapDataController;
import com.netapp.offtap3.ontap.OntapIGroup;

/**
 * This describes an iGroup
 */
public class OntapIGroupImpl<T extends AbstractOntapDataController<C>, C extends OntapDataController> extends
        IGroupImpl<T, C> implements OntapIGroup {
    private static final Logger LOGGER = Logger.getLogger(OntapIGroupImpl.class);

    private int lunId;
    private String portsetName;
    private boolean aluaEnabled;

    public OntapIGroupImpl(T controller) {
        super(controller);
    }

    @Override
	public int getLunId() {
        return lunId;
    }
    public void setLunId(int i) {
        this.lunId = i;
    }

    @Override
    public String getPortsetName() {
        return portsetName;
    }
    public void setPortsetName(String portsetName) {
        this.portsetName = portsetName;
    }

    @Override
    public boolean isAluaEnabled() {
        return aluaEnabled;
    }
    public void setAluaEnabled(boolean aluaEnabled) {
        this.aluaEnabled = aluaEnabled;
    }


    @Override
    public void addInitiators(Set<String> initiators) {
        for (String currInitiator : initiators) {
            LOGGER.debug("addToIGroup named " + getName() + " initiator named " + currInitiator);

            NaElement xi;
            xi = new NaElement("igroup-add");
            xi.addNewChild("initiator", currInitiator);
            xi.addNewChild("initiator-group-name", getName());
            try {
                controller.invokeZapi(xi, ZapiCache.NO_CACHE);
                initiatorIds.add(currInitiator);
                controller.getZapiCache().blastCacheValues("igroup");
            } catch (Exception ex) {
                OntapIGroupImpl.LOGGER.error(
                        controller + " failed to add initiator " + currInitiator + " to the igroup " + getName() + ".",
                        ex);
                throw new IGroupOperationException(controller + " failed to add initiator " + currInitiator + " to the igroup " + getName() + ", specifically " + ex, ex);
            }
        }
    }

    @Override
    public void addInitiator(String initiator) {
        addInitiators(Collections.singleton(initiator));
    }

    @Override
    public void rename(String newName) {
        NaElement xi = new NaElement("igroup-rename");
        xi.addNewChild("initiator-group-name", getName());
        xi.addNewChild("initiator-group-new-name", newName);
        try {
            controller.invokeZapi(xi, ZapiCache.NO_CACHE);
            controller.getZapiCache().blastCacheValues("igroup");
            LOGGER.info("Renamed igroup " + getName() + " to " + newName + ".");
            setName(newName);
        } catch (ApiFailedException e) {
            LOGGER.error(
                    "igroupRename: " + controller + " Unable to rename igroup from " + getName() + " to " + newName, e);
            throw new IGroupOperationException("Rename: " + controller
                    +" Unable to rename igroup from " + getName() + " to " + newName, e);
        }
    }

    @Override
    public void bind(String requestedPortSetName) {
        NaElement xi = new NaElement("igroup-bind-portset");
       	xi.addNewChild("initiator-group-name", getName());
       	xi.addNewChild("portset-name", requestedPortSetName);
        try {
            controller.invokeZapi(xi, ZapiCache.NO_CACHE);
            controller.getZapiCache().blastCacheValues("portset-");
            setPortsetName(requestedPortSetName);
        }
        catch (ApiFailedException e) {
            throw new IGroupOperationException(controller + " failed to bind " + this + " to portset: " + requestedPortSetName, e);
        }
    }

    @Override
    public void unbind() {
        NaElement xi = new NaElement("igroup-unbind-portset");
       	xi.addNewChild("initiator-group-name", getName());
        try {
            controller.invokeZapi(xi, ZapiCache.NO_CACHE);
            controller.getZapiCache().blastCacheValues("portset-");
            setPortsetName(null);
            // TODO aeonflux-next
            // unbind impacts the igroup, which is not necessarily obvious to the caller.
            // we can either refresh for them, or expect them to refresh
            controller.getProxy().refresh(ControllerResource.igroups);
        }
        catch (ApiFailedException e) {
            throw new IGroupOperationException(controller + " failed to unBind " +  this, e);
        }
    }

    @Override
    public void destroy() {
        destroy(false);
    }

    @Override
    public void destroy(boolean force) {
        LOGGER.info("Removing igroup " + getName());
        NaElement xi;
        xi = new NaElement("igroup-destroy");
        xi.addNewChild("initiator-group-name", getName());
        if (force) {
            xi.addNewChild("force", "true");
        }

        try {
            controller.invokeZapi(xi, ZapiCache.NO_CACHE);
            controller.getZapiCache().blastCacheValues("igroup");
            for (OntapIGroup curr : new ArrayList<>(controller.getIGroups())) {
                if (curr.getName().equals(getName())) {
                    controller.getIGroups().remove(curr);
                    break;
                }
            }
        }
        catch (ApiFailedException e) {
            LOGGER.error(controller + " failed to destroy igroup named " + getName(), e);
            throw new IGroupOperationException(controller + " failed to destroy igroup named " + getName(), e);
        }
    }

    @Override
    public void removeInitiators(Set<String> initiators, boolean force) {
        for (String currInitiator : initiators) {
            LOGGER.debug("addToIGroup named " + getName() + " initiator named " + currInitiator);

            NaElement xi;
            xi = new NaElement("igroup-remove");
            xi.addNewChild("initiator", currInitiator);
            xi.addNewChild("initiator-group-name", getName());
            if (force) {
                xi.addNewChild("force", "true");
            }
            try {
                controller.invokeZapi(xi, ZapiCache.NO_CACHE);
                controller.getZapiCache().blastCacheValues("igroup");
                this.initiatorIds.remove(currInitiator);
            } catch (Exception ex) {
                LOGGER.error(
                        controller + " failed to remove initiator " + currInitiator + " from the igroup " + getName()
                                + ".", ex);
                throw new IGroupOperationException(controller + " failed to remove initiator " + currInitiator + " to the igroup " + getName() + ".", ex);
            }
        }
    }

    @Override
    public void setAlua(boolean enabled) {
        LOGGER.info("About to set igroup attribute alua to " + enabled + " on " + getName());
        NaElement xi = new NaElement("igroup-set-attribute");
        xi.addNewChild("initiator-group-name", getName());
        xi.addNewChild("attribute", "alua");
        xi.addNewChild("value", String.valueOf(enabled));
        try {
            controller.invokeZapi(xi, ZapiCache.NO_CACHE);
            aluaEnabled = enabled;
            controller.getZapiCache().blastCacheValues("igroup");
        }
        catch (ApiFailedException e) {
            throw new IGroupOperationException(controller + " failed to set alua for " + this + " to " + enabled, e);
        }
    }

    @Override
    public OntapIGroup refresh() {
        return (OntapIGroup) super.refresh();
    }
}
