package com.netapp.offtap3.internal.ontap.lun;

import com.netapp.offtap3.Size;
import com.netapp.offtap3.internal.LunSpecImpl;
import com.netapp.offtap3.ontap.OntapLunSpec;
import com.netapp.offtap3.ontap.OntapVolume;
import com.netapp.offtap3.ontap.SpaceReserve;

/**
 * Describes the parameters used to create a LUN.  Consists of required parameters that
 * are passed into the constructor of the Builder, as well as optional parameters that
 * are specified on the fluent interface of the Builder.
 */
public class OntapLunSpecImpl extends LunSpecImpl implements OntapLunSpec {
    private final OntapVolume volume;
    private final String type;
    private final SpaceReserve spaceReserve;
    private final String comment;
    private final Integer prefixSize;
    private final String lunClass;
    private final boolean spaceAllocation;

    private OntapLunSpecImpl(BuilderImpl builder) {
        super(builder);
        this.volume = builder.volume;
        this.type = builder.type;
        this.spaceReserve = builder.spaceReserve;
        this.comment = builder.comment;
        this.prefixSize = builder.prefixSize;
        this.lunClass = builder.lunClass;
        this.spaceAllocation = builder.spaceAllocation;
    }

    public OntapVolume getVolume() {
        return volume;
    }

    public String getType() {
        return type;
    }

    public SpaceReserve getSpaceReserve() {
        return spaceReserve;
    }

    public boolean getSpaceAllocation() {
        return spaceAllocation;
    }
    
    public String getComment() {
        return comment;
    }

    public Integer getPrefixSize() {
        return prefixSize;
    }

    public String getLunClass() {
        return lunClass;
    }

    public String toString() {
        return getLunName() + " in volume " + volume.getName() + " of " + getSize().toMB() + " MB"
                + " with space reserve: " + spaceReserve + " with space allocation: " + spaceAllocation;
    }

    public static class BuilderImpl extends LunSpecImpl.BuilderImpl implements OntapLunSpec.Builder {
        // required fields
        private final OntapVolume volume;
        private final String type;

        // if we want to have default values for the optional fields, they should be assigned here
        private SpaceReserve spaceReserve;
        private String comment;
        private Integer prefixSize;
        private String lunClass;
        private boolean spaceAllocation;

        public BuilderImpl(String lunName, OntapVolume volume, String type, Size size) {
            super(lunName, size);
            if (lunName != null && lunName.startsWith("/vol/" + volume.getName())) {
                setLunName(lunName.substring(("/vol/" + volume.getName()).length() + 1));
            }
            this.volume = volume;
            this.type = type;
        }

        @Override
        public OntapLunSpec.Builder spaceReserve(SpaceReserve val) {
            spaceReserve = val;
            return this;
        }

        @Override
        public OntapLunSpec.Builder comment(String val) {
            comment = val;
            return this;
        }

        @Override
        public OntapLunSpec.Builder prefixSize(Integer val) {
            prefixSize = val;
            return this;
        }

        @Override
        public OntapLunSpec.Builder lunClass(String val){
            lunClass = val;
            return this;
        }
        
        //This will work only on FS and above. Pre-FS needs this to be set post creation.
        @Override
        public OntapLunSpec.Builder spaceAllocation(boolean val){
            spaceAllocation = val;
            return this;
        }

        @Override
        public OntapLunSpec build() {
            return new OntapLunSpecImpl(this);
        }
    }
}
