/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/proxy/AdminSvmProxy.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.proxy;

import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.FcpAdapter;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoader;
import com.netapp.offtap3.factories.DataLoader.LoadType;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.CachingControllerProxy;
import com.netapp.offtap3.internal.ontap.MCCManager;
import com.netapp.offtap3.internal.ontap.dataloader.AdminSvmZapiAccessPermissionsDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.ClusterPeerDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SnmpHostDataLoader;
import com.netapp.offtap3.internal.ontap.controller.AdminSvmImpl;
import com.netapp.offtap3.internal.ontap.dataloader.AdminSvmClusterStatusDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.AdminSvmClusterStatusMessageDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.AdminSvmFcpAdapterDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.AdminSvmLicenseDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SvmsDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SvmAggregateDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SvmAggregateListDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SvmInterfaceDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.VolumeMaximumDataLoader;
import com.netapp.offtap3.ontap.AdminSvm;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.Aggregate;
import com.netapp.offtap3.ontap.ClusterPeer;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.ontap.NaElement;

import java.util.Collections;

public class AdminSvmProxy extends CachingControllerProxy<AdminSvm, AdminSvmImpl, NaElement, InternalZapiHandler> {

    @Override
    protected void addDataLoaders() {
        addDataLoader(ControllerResource.svms, new SvmsDataLoader(getOfftapPreferences()));
        addDataLoader(ControllerResource.aggregates, new SvmAggregateDataLoader(getOfftapPreferences(), getDataCache()));
        addDataLoader(ControllerResource.interfaces, new SvmInterfaceDataLoader(getOfftapPreferences()));
        addDataLoader(ControllerResource.snmp, new SnmpHostDataLoader(getOfftapPreferences()));
        addDataLoader(ControllerResource.licenses, new AdminSvmLicenseDataLoader());
        addDataLoader(ControllerResource.cluster_status, new AdminSvmClusterStatusDataLoader());
        addDataLoader(ControllerResource.cluster_state_reason, new AdminSvmClusterStatusMessageDataLoader());
        addDataLoader(ControllerResource.fcp, new AdminSvmFcpAdapterDataLoader(getOfftapPreferences()));
        addDataLoader(ControllerResource.zapi_access, new AdminSvmZapiAccessPermissionsDataLoader());
        addDataLoader(ControllerResource.cluster_peers, new ClusterPeerDataLoader());
        addDataLoader(ControllerResource.volume_maximums, new VolumeMaximumDataLoader());
    }

    @Override
    protected void initialLoad() throws DataLoaderException {
        super.initialLoad();
        // If we are an mcc source, ensure the cluster peers are loaded and MCC slappers are slapping.
        if (getProxiedObject().isMccSource()) {
            executeDataLoader(LoadType.initial, Collections.<Resource>singleton(ControllerResource.cluster_peers));
            // make sure slappers are running for all of this mess
            executeDataLoader(LoadType.initial, Collections.<Resource>singleton(ControllerResource.svms));
            for (DataSvm dataSvm : getProxiedObject().getDataSvms().values()) {
                MCCManager.getInstance().getMCCSlapperThreadCanBeNull(dataSvm.getConnection());
            }
            MCCManager.getInstance().getMCCSlapperThreadCanBeNull(getProxiedObject().getConnection());
        }
    }



    @Override
    protected Resource getResource(Refreshable refreshable) {
        if (refreshable instanceof FcpAdapter) {
            return ControllerResource.fcp;
        } else if (refreshable instanceof ClusterPeer) {
            return ControllerResource.cluster_peers;
        } else if (refreshable instanceof Aggregate) {
        	return ControllerResource.aggregates;        
        } else {
            return null;
        }
    }
}
