package com.netapp.offtap3.internal.ontap.proxy;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.ProxyType;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.UnixPermission;
import com.netapp.offtap3.VolumeLoader;
import com.netapp.offtap3.VolumeResource;
import com.netapp.offtap3.internal.CachingOfftapProxy;
import com.netapp.offtap3.internal.ontap.dataloader.volume.CModeVolumeExportDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.CModeVolumeSnapshotDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.CModeVolumeDataLoader;
import com.netapp.offtap3.internal.ontap.volume.CModeVolumeImpl;
import com.netapp.offtap3.ontap.CModeDedupe;
import com.netapp.offtap3.ontap.CModeVolume;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.Export;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.Qtree;
import com.netapp.offtap3.ontap.Snapshot;

import java.lang.reflect.Method;


public class CModeVolumeProxy extends CachingOfftapProxy<NaElement, InternalZapiHandler, CModeVolume, CModeVolumeImpl> {
    public CModeVolumeProxy() {
        super(ProxyType.VOLUME);
    }

    @Override
    protected void addDataLoaders() {
        CModeVolumeDataLoader mainDataLoader = new CModeVolumeDataLoader(getDataCache(), getOfftapPreferences());
        addDataLoader(VolumeResource.id, mainDataLoader);
        addDataLoader(VolumeResource.state, mainDataLoader);
        addDataLoader(VolumeResource.inode, mainDataLoader);
        addDataLoader(VolumeResource.striping, mainDataLoader);
        addDataLoader(VolumeResource.space, mainDataLoader);
        addDataLoader(VolumeResource.sis, mainDataLoader);
        addDataLoader(VolumeResource.vm_align, mainDataLoader);
        addDataLoader(VolumeResource.auto_size, mainDataLoader);
        addDataLoader(VolumeResource.performance, mainDataLoader);
        addDataLoader(VolumeResource.language, mainDataLoader);
        addDataLoader(VolumeResource.clone, mainDataLoader);
        addDataLoader(VolumeResource.qos, mainDataLoader);
        addDataLoader(VolumeResource.snapshot_autodelete, mainDataLoader);
        addDataLoader(VolumeResource.snapdir, mainDataLoader);
        addDataLoader(VolumeResource.exports, mainDataLoader);
        addDataLoader(VolumeResource.security, mainDataLoader);

        addDataLoader(VolumeResource.qtree, new CModeVolumeExportDataLoader(getOfftapPreferences(), getDataCache()));
        addDataLoader(VolumeResource.snapshots, new CModeVolumeSnapshotDataLoader(getDataCache(), getOfftapPreferences()));

    }

    @Override
    protected Resource getResourceAnnotation(Method method) {
        VolumeLoader loader = method.getAnnotation(VolumeLoader.class);
        return (loader == null) ? null : loader.value();
    }

    @Override
    protected Resource getResource(Refreshable refreshable) {
        if (refreshable instanceof CModeDedupe) {
            return VolumeResource.sis;
        } else if (refreshable instanceof UnixPermission) {
            return VolumeResource.security;
        } else if (refreshable instanceof Qtree || refreshable instanceof Export) {
            return VolumeResource.qtree;
        } else if (refreshable instanceof Snapshot) {
            return VolumeResource.snapshots;
        } else if (refreshable instanceof UnixPermission) {
            return VolumeResource.security;
        } else {
            return null;
        }
    }
}
