/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/proxy/DataSvmProxy.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.proxy;

import com.netapp.offtap3.CifsShare;
import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.FcInterface;
import com.netapp.offtap3.IpInterface;
import com.netapp.offtap3.IscsiAdapter;
import com.netapp.offtap3.IscsiConnection;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.internal.CachingControllerProxy;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.dataloader.ControllerRootVolumeDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmCifsSharesDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmExportPolicyDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmIscsiDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmLicenseDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmProtocolDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmQosPolicyGroupDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmUnixUserAndGroupDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SvmPeerDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.snapmirror.DataSvmSnapMirrorRelationDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SvmAggregateDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmIGroupDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.lun.DataSvmLunDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmPortSetDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.DataSvmVolumeDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SvmInterfaceDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.lun.LunBindingsDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.VolumeMaximumDataLoader;
import com.netapp.offtap3.ontap.Aggregate;
import com.netapp.offtap3.ontap.CModeLun;
import com.netapp.offtap3.ontap.CModeSnapMirrorRelation;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.ExportPolicy;
import com.netapp.offtap3.ontap.OntapIGroup;
import com.netapp.offtap3.ontap.LunBinding;
import com.netapp.offtap3.ontap.NaElement;

public class DataSvmProxy extends CachingControllerProxy<DataSvm, DataSvmImpl, NaElement, InternalZapiHandler> {

    @Override
    protected void addDataLoaders() {
        OfftapPreferences prefs = getOfftapPreferences();
        addDataLoader(ControllerResource.licenses, new DataSvmLicenseDataLoader());
        addDataLoader(ControllerResource.port_sets, new DataSvmPortSetDataLoader(prefs, getDataCache()));
        addDataLoader(ControllerResource.aggregates, new SvmAggregateDataLoader(prefs, getDataCache()));
        addDataLoader(ControllerResource.volumes, new DataSvmVolumeDataLoader(prefs));
        addDataLoader(ControllerResource.interfaces, new SvmInterfaceDataLoader(prefs));
        addDataLoader(ControllerResource.luns, new DataSvmLunDataLoader(prefs));
        addDataLoader(ControllerResource.lun_bindings, new LunBindingsDataLoader(prefs));
        addDataLoader(ControllerResource.igroups, new DataSvmIGroupDataLoader(prefs));
        addDataLoader(ControllerResource.cifs_shares, new DataSvmCifsSharesDataLoader());
        DataSvmProtocolDataLoader protocolDataLoader = new DataSvmProtocolDataLoader();
        addDataLoader(ControllerResource.protocols, protocolDataLoader);
        addDataLoader(ControllerResource.vstorage_status, protocolDataLoader);
        addDataLoader(ControllerResource.snapmirror, new DataSvmSnapMirrorRelationDataLoader(prefs, getDataCache()));
        addDataLoader(ControllerResource.qos, new DataSvmQosPolicyGroupDataLoader(prefs, getDataCache()));
        addDataLoader(ControllerResource.root_volume, new ControllerRootVolumeDataLoader());
        addDataLoader(ControllerResource.svm_peers, new SvmPeerDataLoader(prefs, getDataCache()));
        addDataLoader(ControllerResource.export_policies, new DataSvmExportPolicyDataLoader());
        addDataLoader(ControllerResource.unix_users_groups, new DataSvmUnixUserAndGroupDataLoader());
        addDataLoader(ControllerResource.iscsi, new DataSvmIscsiDataLoader());
        addDataLoader(ControllerResource.volume_maximums, new VolumeMaximumDataLoader());
    }

    @Override
    protected Resource getResource(Refreshable refreshable) {
        if (refreshable instanceof CModeSnapMirrorRelation) {
            return ControllerResource.snapmirror;
        } else if (refreshable instanceof IscsiAdapter || refreshable instanceof IscsiConnection) {
            return ControllerResource.iscsi;
        } else if (refreshable instanceof OntapIGroup) {
            return ControllerResource.igroups;
        } else if (refreshable instanceof ExportPolicy) {
            return ControllerResource.export_policies;
        } else if (refreshable instanceof CModeLun) {
            return ControllerResource.luns;
        } else if (refreshable instanceof LunBinding) {
            return ControllerResource.lun_bindings;
        } else if (refreshable instanceof CifsShare) {
            return ControllerResource.cifs_shares;
        } else if (refreshable instanceof Aggregate) {
            return ControllerResource.aggregates;
        } else if (refreshable instanceof FcInterface || refreshable instanceof IpInterface) {
            return ControllerResource.interfaces;
        } else {
            return null;
        }
    }
}
