package com.netapp.offtap3.internal.ontap.proxy;

import com.netapp.offtap3.CifsShare;
import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.IpInterface;
import com.netapp.offtap3.IscsiAdapter;
import com.netapp.offtap3.IscsiConnection;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.internal.ControllerProxy;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.dataloader.ControllerRootVolumeDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerAggregateDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerCifsSharesDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerClusterFailoverDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerFcpDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerIGroupDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerIscsiDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerLicenseDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerProtocolDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerSystemInfoDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.FilerVStorageStatusDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SevenModeZapiAccessPermissionsDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.VFilerDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.lun.FilerLunDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.snapmirror.FilerSnapMirrorRelationDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.FilerVolumeDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.SnapVaultDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.VolumeMaximumDataLoader;
import com.netapp.offtap3.ontap.Aggregate;
import com.netapp.offtap3.ontap.Dedupe;
import com.netapp.offtap3.ontap.Filer;
import com.netapp.offtap3.ontap.OntapIGroup;
import com.netapp.offtap3.ontap.OntapLun;
import com.netapp.offtap3.ontap.SnapMirrorRelation;

public abstract class FilerProxy<I extends Filer, F extends FilerImpl<I>> extends ControllerProxy<I, F> {

    @Override
    protected void addDataLoaders() {
        OfftapPreferences prefs = getOfftapPreferences();

        addDataLoader(ControllerResource.system_info, new FilerSystemInfoDataLoader());
        addDataLoader(ControllerResource.licenses, new FilerLicenseDataLoader());
        addDataLoader(ControllerResource.protocols, new FilerProtocolDataLoader());
        addDataLoader(ControllerResource.vstorage_status, new FilerVStorageStatusDataLoader());
        addDataLoader(ControllerResource.cf_info, new FilerClusterFailoverDataLoader());
        addDataLoader(ControllerResource.vfilers, new VFilerDataLoader());
        addDataLoader(ControllerResource.aggregates, new FilerAggregateDataLoader());
        addDataLoader(ControllerResource.volumes, new FilerVolumeDataLoader(prefs));
        addDataLoader(ControllerResource.luns, new FilerLunDataLoader(prefs));
        addDataLoader(ControllerResource.iscsi, new FilerIscsiDataLoader());
        addDataLoader(ControllerResource.fcp, new FilerFcpDataLoader());
        addDataLoader(ControllerResource.igroups, new FilerIGroupDataLoader());
        addDataLoader(ControllerResource.cifs_shares, new FilerCifsSharesDataLoader(prefs));
        addDataLoader(ControllerResource.root_volume, new ControllerRootVolumeDataLoader());
        addDataLoader(ControllerResource.zapi_access, new SevenModeZapiAccessPermissionsDataLoader());
        addDataLoader(ControllerResource.volume_maximums, new VolumeMaximumDataLoader());

        SnapVaultDataLoader snapVaultDataLoader = new SnapVaultDataLoader();
        addDataLoader(ControllerResource.primary_snapvaults, snapVaultDataLoader);
        addDataLoader(ControllerResource.secondary_snapvaults, snapVaultDataLoader);
        addDataLoader(ControllerResource.snapmirror, new FilerSnapMirrorRelationDataLoader());
    }

    @Override
    protected Resource getResource(Refreshable refreshable) {
        if (refreshable instanceof SnapMirrorRelation) {
            return ControllerResource.snapmirror;
        } else if (refreshable instanceof IscsiAdapter || refreshable instanceof IscsiConnection) {
            return ControllerResource.iscsi;
        } else if (refreshable instanceof OntapIGroup) {
            return ControllerResource.igroups;
        } else if (refreshable instanceof OntapLun) {
            return ControllerResource.luns;
        } else if (refreshable instanceof CifsShare) {
            return ControllerResource.cifs_shares;
        } else if (refreshable instanceof Dedupe) {
            return ControllerResource.volumes;
        } else if (refreshable instanceof Aggregate) {
            return ControllerResource.aggregates;
        } else if (refreshable instanceof IpInterface) {
            return ControllerResource.interfaces;
        } else {
            return null;
        }
    }
}
