package com.netapp.offtap3.internal.ontap.proxy;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.ProxyType;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.VolumeLoader;
import com.netapp.offtap3.VolumeResource;
import com.netapp.offtap3.internal.CachingOfftapProxy;
import com.netapp.offtap3.internal.ontap.dataloader.volume.FilerExportDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.FilerVolumeSnapMirrorStatusDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.volume.FilerVolumeSnapshotDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.SnapVaultScheduleDataLoader;
import com.netapp.offtap3.internal.ontap.volume.FilerVolumeImpl;
import com.netapp.offtap3.ontap.Export;
import com.netapp.offtap3.ontap.Filer;
import com.netapp.offtap3.ontap.FilerVolume;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;

import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.Qtree;
import com.netapp.offtap3.ontap.SnapVaultSchedule;
import com.netapp.offtap3.ontap.Snapshot;

import java.lang.reflect.Method;

public class FilerVolumeProxy<F extends Filer> extends CachingOfftapProxy<NaElement, InternalZapiHandler, FilerVolume, FilerVolumeImpl<F>> {

    public FilerVolumeProxy() {
        super(ProxyType.VOLUME);
    }

    @Override
    protected void addDataLoaders() {
        addDataLoader(VolumeResource.qtree, new FilerExportDataLoader(getDataCache()));
        addDataLoader(VolumeResource.snapvault_schedule, new SnapVaultScheduleDataLoader());
        addDataLoader(VolumeResource.snapshots, new FilerVolumeSnapshotDataLoader());
        addDataLoader(VolumeResource.snapmirror_status, new FilerVolumeSnapMirrorStatusDataLoader());
    }

    @Override
    protected Resource getResourceAnnotation(Method method) {
        VolumeLoader loader = method.getAnnotation(VolumeLoader.class);
        return (loader == null) ? null : loader.value();
    }

    @Override
    protected Resource getResource(Refreshable refreshable) {
        if (refreshable instanceof Qtree || refreshable instanceof Export) {
            return VolumeResource.qtree;
        } else if (refreshable instanceof Snapshot) {
            return VolumeResource.snapshots;
        } else if (refreshable instanceof SnapVaultSchedule) {
            return VolumeResource.snapvault_schedule;
        } else {
            return null;
        }
    }
}
