package com.netapp.offtap3.internal.ontap.proxy;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.LunLoader;
import com.netapp.offtap3.LunResource;
import com.netapp.offtap3.ProxyType;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.factories.DataLoaderException;
import com.netapp.offtap3.internal.AbstractDataLoader;
import com.netapp.offtap3.internal.OfftapProxy;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapDataController;
import com.netapp.offtap3.internal.ontap.dataloader.lun.LunGeometryDataLoader;
import com.netapp.offtap3.internal.ontap.dataloader.lun.LunMapListDataLoader;
import com.netapp.offtap3.internal.ontap.lun.OntapLunImpl;
import com.netapp.offtap3.ontap.LunStats;
import com.netapp.offtap3.ontap.OntapLun;
import com.netapp.offtap3.ontap.OntapDataController;

import java.lang.reflect.Method;
import java.util.Collections;
import java.util.Set;

public class LunProxy<I extends OntapLun, L extends OntapLunImpl<I, C, M>, C extends OntapDataController, M extends AbstractOntapDataController<C>> extends OfftapProxy<I, L> {
    public LunProxy() {
        super(ProxyType.LUN);
    }

    @Override
    protected void addDataLoaders() {
        addDataLoader(LunResource.lun_stats, new AbstractDataLoader<L>() {
            @Override
            protected Refreshable doRefresh(L loadTarget, Resource resource, Refreshable refreshTarget) {
                doLoad(loadTarget, LoadType.refresh, Collections.singleton(resource));
                return loadTarget.getLunStats();
            }

            @Override
            protected boolean doLoad(L lun, LoadType loadType, Set<Resource> resources) throws DataLoaderException {
                lun.loadStats();
                return true;
            }
        });
        addDataLoader(LunResource.geometry, new LunGeometryDataLoader());
        addDataLoader(LunResource.map_list, new LunMapListDataLoader());
    }

    @Override
    protected Resource getResourceAnnotation(Method method) {
        LunLoader loader = method.getAnnotation(LunLoader.class);
        return (loader == null) ? null : loader.value();
    }

    @Override
    protected Resource getResource(Refreshable refreshable) {
        if (refreshable instanceof LunStats) {
            return LunResource.lun_stats;
        } else {
            return null;
        }
    }
}
