package com.netapp.offtap3.internal.ontap.qos;

import com.google.common.base.Objects;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.QosOperationException;
import com.netapp.offtap3.ontap.QosPolicyGroup;
import com.netapp.offtap3.ontap.QosThroughput;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

public class QosPolicyGroupImpl implements QosPolicyGroup {

    private static final Logger logger = Logger.getLogger(QosPolicyGroupImpl.class);

    public static final QosPolicyGroup NONE = new QosPolicyGroupImpl(null, "none", new QosThroughput(0.0, QosThroughput.Unit.INF));

    private final DataSvmImpl svm;
    private String name;
    private QosThroughput maxThroughput = new QosThroughput(0.0, QosThroughput.Unit.NONE);

    public QosPolicyGroupImpl(DataSvmImpl svm, String name, QosThroughput max) {
        this.svm = svm;
        this.name = name;
        this.maxThroughput = max;
    }

    @Override
    public void delete() {
        logger.debug("Enter deleteQosPolicy");
        NaElement xi = new NaElement("qos-policy-group-delete");
        xi.addNewChild("policy-group", getName());

        try {
            NaElement xo = svm.getParentControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
            logger.info("Deletion of QoS policy '" + getName() + "' completed with status: " + xo.getAttr("status"));
        } catch (ApiFailedException e) {
            logger.error(svm + " failed to delete policy: " + this + " due to: " + e, e);
            throw new QosOperationException(svm + " failed to delete policy: " + this + " due to: " + e, e);
        }
    }

    @Override
    public QosThroughput getMaxThroughput() {
        return maxThroughput;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(svm.getId(), name);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof QosPolicyGroupImpl) {
            QosPolicyGroupImpl other = (QosPolicyGroupImpl) obj;
            return name.equals(other.getName())
                    && maxThroughput.equals(other.getMaxThroughput())
                    && svm.equals(other.svm);
        }
        return false;
    }

    public String toString() {
        return Objects.toStringHelper(this)
                .add("name", name)
                .add("max throughput", maxThroughput)
                .toString();
    }
}
