package com.netapp.offtap3.internal.ontap.snap;

import com.google.common.base.Objects;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.internal.AbstractRefreshable;
import com.netapp.offtap3.internal.HasProxy;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapController;
import com.netapp.offtap3.internal.ontap.dataloader.OntapDataCache;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.utils.StatsCollector;
import com.netapp.offtap3.ontap.*;
import org.apache.log4j.Logger;

public abstract class AbstractSnapMirrorRelationImpl<I extends SnapMirrorRelation>
        extends AbstractRefreshable implements SnapMirrorRelation, HasProxy<I> {

    private static final Logger logger = Logger.getLogger(AbstractSnapMirrorRelationImpl.class);

    private final OntapDataCache dataCache;
    
    private String  sourceLocation;
    private String destinationLocation;

    private SnapMirrorSchedule schedule;

    private String  mirrorState;
    private String  status;
    private long    lagTime;
    private long    transferProgress;
    private long    mirrorTimestamp;
    private String  baseSnapshot;
    private String  currentTransferType;
    private String  currentTransferError;
    private String  contents;

    private String  lastTransferType;
    private long    lastTransferSize;
    private long    lastTransferDuration;
    private String  lastTransferFrom;

    public AbstractSnapMirrorRelationImpl(AbstractOntapController loader, OntapDataCache dataCache) {
        super(loader);
        this.dataCache = dataCache;
    }

    public abstract AbstractOntapController getDestinationController();

    protected OntapDataCache getDataCache() {
        return dataCache;
    }

    @Override
    public String getSourceLocation() {
        return sourceLocation;
    }

    public void setSourceLocation(String sourceLocation) {
        this.sourceLocation = sourceLocation;
    }

    @Override
    public String getDestinationLocation() {
        return destinationLocation;
    }

    public void setDestinationLocation(String destinationLocation) {
        this.destinationLocation = destinationLocation;
    }

    @Override
    public String getMirrorState() {
        return mirrorState;
    }

    public void setMirrorState(String mirrorState) {
        this.mirrorState = mirrorState;
    }

    @Override
    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    @Override
    public long getLagTime() {
        return lagTime;
    }

    public void setLagTime(long lagTime) {
        this.lagTime = lagTime;
    }

    @Override
    public long getTransferProgress() {
        return transferProgress;
    }
    public void setTransferProgress(long transferProgress) {
        this.transferProgress = transferProgress;
    }

    @Override
    public long getMirrorTimestamp() {
        return mirrorTimestamp;
    }

    public void setMirrorTimestamp(long mirrorTimestamp) {
        this.mirrorTimestamp = mirrorTimestamp;
    }

    public String getBaseSnapshot() {
        return baseSnapshot;
    }

    public void setBaseSnapshot(String baseSnapshot) {
        this.baseSnapshot = baseSnapshot;
    }

    @Override
    public String getCurrentTransferType() {
        return currentTransferType;
    }

    public void setCurrentTransferType(String currentTransferType) {
        this.currentTransferType = currentTransferType;
    }

    @Override
    public String getCurrentTransferError() {
        return currentTransferError;
    }

    public void setCurrentTransferError(String currentTransferError) {
        this.currentTransferError = currentTransferError;
    }

    public String getContents() {
        return contents;
    }

    public void setContents(String contents) {
        this.contents = contents;
    }

    @Override
    public String getLastTransferType() {
        return lastTransferType;
    }

    public void setLastTransferType(String lastTransferType) {
        this.lastTransferType = lastTransferType;
    }

    @Override
    public long getLastTransferSize() {
        return lastTransferSize;
    }

    public void setLastTransferSize(long lastTransferSize) {
        this.lastTransferSize = lastTransferSize;
    }

    @Override
    public long getLastTransferDuration() {
        return lastTransferDuration;
    }

    public void setLastTransferDuration(long lastTransferDuration) {
        this.lastTransferDuration = lastTransferDuration;
    }

    @Override
    public String getLastTransferFrom() {
        return lastTransferFrom;
    }

    public void setLastTransferFrom(String lastTransferFrom) {
        this.lastTransferFrom = lastTransferFrom;
    }

    @Override
    public SnapMirrorSchedule getSchedule() {
        return schedule;
    }

    @Override
    public SnapMirrorScheduleBuilder getSnapMirrorScheduleBuilder() {
        return new SnapMirrorScheduleBuilderImpl();
    }

    public void setScheduleBean(SnapMirrorScheduleImpl schedule) {
        this.schedule = schedule;
    }

    @Override
    public void abortTransfer() {
        logger.debug("Enter abortTransfer: " + destinationLocation);
        int secondsToRetry = getDestinationController().getOfftapPreferences().getSnapMirrorAbortRetryTimeout();
        NaElement xi = new NaElement("snapmirror-abort");
        xi.addNewChild("destination-location", destinationLocation);
        boolean aborted = false;
        long startTime = System.currentTimeMillis();
        while (!aborted && System.currentTimeMillis() < startTime + secondsToRetry*1000) {
            try {
                getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
                aborted = true;
                getDestinationController().getZapiCache().blastCacheValues("snap");
                logger.debug(destinationLocation + " SnapMirror abort issued.");
            } catch (Exception e) {
                if (e.getMessage().contains("17130")) { // ENOTRANSFER_IN_PROGRESS
                    logger.debug("No transfers in progress; invalid snapmirror-abort call.");
                    aborted = true;
                }
                else if (e.getMessage().contains("17131")) { // EANOTHER_OP_ACTIVE
                    logger.debug("Another operation is active. Unable to abort SnapMirror transfers at this time.");
                    StatsCollector.sleep(5000,"snapmirror-abort"); // 5 seconds
                }
                else {
                    logger.error("Unable to abort SnapMirror transfers.");
                    throw new SnapMirrorOperationException("Unable to abort SnapMirror transfers: + " + e.getMessage(), e);
                }
            }
        }
    }

    @Override
    public void breakMirror() throws SnapMirrorOperationException {
        logger.debug("Enter breakRelation: " + destinationLocation);
        int secondsToRetry = getDestinationController().getOfftapPreferences().getSnapMirrorBreakRetryTimeout();
        NaElement xi = new NaElement("snapmirror-break");
        xi.addNewChild("destination-location", destinationLocation);
        boolean brokenOff = false;
        long startTime = System.currentTimeMillis();
        while (!brokenOff) {
            try {
                getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
                brokenOff = true;
                getDestinationController().getZapiCache().blastCacheValues("snap");
                logger.debug(destinationLocation + " SnapMirror break issued.");
            } catch (Exception e) {
                // If an error occurs that isn't 'Another Snapmirror operation is in progress', throw the exception
                if (e.getMessage() == null || !e.getMessage().contains("17131") ||
                        startTime + (secondsToRetry*1000) <= System.currentTimeMillis()) { // EANOTHER_OP_ACTIVE
                    logger.error("Unable to break SnapMirror relation.");
                    throw new SnapMirrorOperationException("Unable to break SnapMirror relation: " + e.getMessage(), e);
                } else {
                    StatsCollector.sleep(5000,"snapmirror-break"); // 5 seconds
                }
            }
        }
    }
    
    @Override
    public void resumeMirror() throws SnapMirrorOperationException {
        logger.debug("Enter resumeMirror: " + destinationLocation);
        int secondsToRetry = getDestinationController().getOfftapPreferences().getSnapMirrorResumeRetryTimeout();
        NaElement xi = new NaElement("snapmirror-resume");
        xi.addNewChild("destination-location", destinationLocation);
        boolean resumed = false;
        long startTime = System.currentTimeMillis();
        while (!resumed) {
            try {
                getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
                resumed = true;
                getDestinationController().getZapiCache().blastCacheValues("snap");
                logger.debug(destinationLocation + " SnapMirror resume issued.");
            } catch (Exception e) {
                // If an error occurs that isn't 'Another Snapmirror operation is in progress', throw the exception
                if (e.getMessage() == null || !e.getMessage().contains("17131") ||
                        startTime + (secondsToRetry*1000) <= System.currentTimeMillis()) { // EANOTHER_OP_ACTIVE
                    logger.error("Unable to resume SnapMirror relation.");
                    throw new SnapMirrorOperationException("Unable to resume SnapMirror relation: " + e.getMessage(), e);
                } else {
                    StatsCollector.sleep(5000,"snapmirror-resume"); // 5 seconds
                }
            }
        }
    }

    @Override
    public void quiesceMirror() throws SnapMirrorOperationException {
        logger.debug("Enter quiesceRelation: " + destinationLocation);
        int secondsToRetry = getDestinationController().getOfftapPreferences().getSnapMirrorQuiesceRetryTimeout();
        NaElement xi = new NaElement("snapmirror-quiesce");
        xi.addNewChild("destination-location", destinationLocation);
        boolean quiesced = false;
        long startTime = System.currentTimeMillis();
        while (!quiesced) {
            try {
                getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
                quiesced = true;
                getDestinationController().getZapiCache().blastCacheValues("snap");
                logger.debug(destinationLocation + " SnapMirror quiesce issued.");
            } catch (Exception e) {
                // If an error occurs that isn't 'Another Snapmirror operation is in progress', throw the exception
                if (e.getMessage() == null || !e.getMessage().contains("17131") ||
                        startTime + (secondsToRetry*1000) <= System.currentTimeMillis()) { // EANOTHER_OP_ACTIVE
                    logger.error("Unable to quiesce SnapMirror relation.");
                    throw new SnapMirrorOperationException("Unable to quiesce SnapMirror relation: " + e.getMessage(), e);
                } else {
                    StatsCollector.sleep(5000,"snapmirror-quiesce"); // 5 seconds
                }
            }
        }
    }

    @Override
    public void releaseSource() {
        logger.debug("Enter release: " + sourceLocation + " destination: " + destinationLocation);
        NaElement xi;

        xi = new NaElement("snapmirror-release");
        xi.addNewChild("source-location", sourceLocation);
        xi.addNewChild("destination-location", destinationLocation);
        try{
            getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
            getDestinationController().getZapiCache().blastCacheValues("snap");
            logger.debug("SnapMirror release issued to: " + sourceLocation);
        } catch (ApiFailedException e) {
            if (e.getErrno() == 13001) {
                logger.debug("Unable to release SnapMirror relation, because it no longer exists.");
            }
            else {
                logger.error("Unable to release SnapMirror relation.");
                throw new SnapMirrorOperationException("Unable to break SnapMirror relation: " + e.getMessage(), e);
            }
        }

    }

    @Override
    public String update() {
        logger.debug("updateSnapMirror: " + destinationLocation);
        NaElement xi = new NaElement("snapmirror-update");
        xi.addNewChild("destination-location", destinationLocation);
        if (sourceLocation != null) {
            xi.addNewChild("source-location", sourceLocation);
        }
        try {
            AbstractOntapController controller = getDestinationController();
            NaElement xo = controller.invokeZapi(xi, ZapiCache.NO_CACHE);
            controller.getZapiCache().blastCacheValues("snap");
            logger.debug("SnapMirror update started for " + destinationLocation);
            String status = xo.getChildContent("result-status");
            logger.debug("SnapMirror update returned status " + status);
            return status;
        } catch (ApiFailedException e) {
            logger.error("Unable to start SnapMirror update for " + destinationLocation);
            throw new SnapMirrorOperationException("Unable to update SnapMirror: " + e.getMessage(), e);
        }
    }

    @Override
    public void resync() {
        logger.debug("Enter resyncSnapMirror: " + destinationLocation);
        NaElement xi;

        xi = new NaElement("snapmirror-resync");
        xi.addNewChild("destination-location", destinationLocation);

        try {
            getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
            getDestinationController().getZapiCache().blastCacheValues("snap");
            logger.debug("Resyncing SnapMirror for " + destinationLocation);
        } catch (ApiFailedException e) {
            logger.error("Unable to resync SnapMirror.");
            throw new SnapMirrorOperationException("Unable to resync SnapMirror: " +e.getMessage(), e);
        }
    }

    @Override
    public void obliterate() {
        logger.info("Completely tearing down this relation: " + this);

        try {
            breakMirror();
        }
        catch (SnapMirrorOperationException e) {
            logger.debug("Failed to break mirror, proceeding with release.  Error was: " + e, e);
        }

        try {
            releaseSource();
        }
        catch (SnapMirrorOperationException e) {
            logger.debug("Failed to release source, proceeding with destroy.  Error was: " + e, e);
        }

        try {
            destroy();
        }
        catch (SnapMirrorOperationException e) {
            logger.debug("Failed to destroy relationship, proceeding with delete schedule. Error was: " + e, e);
        }

        try {
            deleteSchedule();
        }
        catch (SnapMirrorOperationException e) {
            logger.debug("Failed to delete schedule for relationship. Error was: " + e, e);
        }
    }

    @Override
    public void refresh(Resource... resources) {
        // Handled by the proxy.
    }

    @Override
    public Refreshable refresh(Refreshable refreshable) {
        // Handled by the proxy
        return null;
    }

    @Override
    public int hashCode() {
        return toString().hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof SnapMirrorRelation && toString().equals(obj.toString());
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
                .add("sourceLocation", sourceLocation)
                .add("destinationLocation", destinationLocation)
                .add("schedule", schedule)
                .add("status", status)
                .add("mirrorState", mirrorState)
                .toString();
    }

    @Override
    public SnapMirrorRelation refresh() {
        return (SnapMirrorRelation) super.refresh();
    }
}
