package com.netapp.offtap3.internal.ontap.snap;

import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.StorageSystem;
import com.netapp.offtap3.internal.CachingOfftapProxy;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.controller.OntapConnection;
import com.netapp.offtap3.internal.ontap.dataloader.OntapDataCache;
import com.netapp.offtap3.internal.ontap.proxy.CModeSnapMirrorRelationProxy;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.*;
import com.netapp.offtap3.utils.StringUtils;
import org.apache.log4j.Logger;

import java.util.EnumSet;
import java.util.Set;

public class CModeSnapMirrorRelationImpl extends AbstractSnapMirrorRelationImpl<CModeSnapMirrorRelation> implements CModeSnapMirrorRelation {
    private static final Logger logger = Logger.getLogger(CModeSnapMirrorRelationImpl.class);

    private CModeSnapMirrorRelation proxy;
    private DataSvm sourceController;

    private DataSvmImpl destinationController;

    private String sourceCluster;
    private String sourceVServer;
    private String sourceVolume;

    private CModeVolume destinationVolume;

    private String scheduleName;

    private String relationshipType;
    private String relationshipId;

    // SnapMirror policy info.
    private String policyName;
    private CModeSnapMirrorPolicy policy;

    private String maxTransferRate;

    private boolean ishealthy = false;

    private String newestSnapshot;
    private int newestSnapshotTimestamp;
    private String transferSnapshot;

    @Override
    public CModeSnapMirrorRelation refresh() {
        return (CModeSnapMirrorRelation) super.refresh();
    }

    private static final Set<StorageSystem> storageSystems = EnumSet.of(StorageSystem.datasvm);

    public CModeSnapMirrorRelationImpl(DataSvmImpl svm, OntapDataCache dataCache) {
        super(svm, dataCache);
    }

    public DataSvm getSourceController() {
        return sourceController;
    }

    @Override
    public CModeSnapMirrorRelation.Type getType() {
        return CModeSnapMirrorRelation.Type.valueOf(relationshipType);
    }

    public void setScheduleName(String scheduleName) {
        this.scheduleName = scheduleName;
    }

    @Override
    public String getScheduleName() {
        return scheduleName;
    }

    public void setSourceController(DataSvm controller) {
        this.sourceController = controller;
    }

    @Override
    public String getSourceCluster() {
        return sourceCluster;
    }

    public void setSourceCluster(String sourceCluster) {
        this.sourceCluster = sourceCluster;
    }

    @Override
    public String getSourceSvm() {
        return sourceVServer;
    }

    public void setSourceSvm(String sourceVServer) {
        this.sourceVServer = sourceVServer;
    }

    @Override
    public String getSourceVolume() {
        return sourceVolume;
    }

    public void setSourceVolume(String sourceVolume) {
        this.sourceVolume = sourceVolume;
    }

    @Override
    public DataSvmImpl getDestinationController() {
        return destinationController;
    }

    public void setDestinationController(DataSvmImpl controller) {
        this.destinationController = controller;
    }

    @Override
    public DataSvm getDestinationSvm() {
        return destinationController;
    }

    @Override
    public CModeVolume getDestinationVolume() {
        return destinationVolume;
    }

    public void setDestinationVolume(CModeVolume destinationVolume) {
        this.destinationVolume = destinationVolume;
    }

    public void setRelationshipType(String relationshipType) {
        this.relationshipType = relationshipType;
    }

    @Override
    public boolean isHealthy() {
        return ishealthy;
    }

    public void setIsHealthy(boolean ishealthy) {
        this.ishealthy = ishealthy;
    }

    @Override
    public String getPolicyName() {
        return policyName;
    }

    public void setPolicyName(String policyName) {
        this.policyName = policyName;
    }

    @Override
    public CModeSnapMirrorPolicy getPolicy() {
        return policy;
    }

    public void setPolicyBean(CModeSnapMirrorPolicy policy) {
        this.policy = policy;
    }

    @Override
    public CModeSnapMirrorPolicyBuilder createSnapMirrorPolicyBuilder(String name) {
        return new CModeSnapMirrorPolicyBuilderImpl(name);
    }

    @Override
    public String getMaxTransferRate() {
        return maxTransferRate;
    }

    public void setMaxTransferRate(String maxTransferRate) {
        this.maxTransferRate = maxTransferRate;
    }

    @Override
    public String getRelationshipId() {
        return relationshipId;
    }

    public void setRelationshipId(String relationshipId) {
        this.relationshipId = relationshipId;
    }

    @Override
    public String getNewestSnapshot() {
        return newestSnapshot;
    }

    public void setNewestSnapshot(String newestSnapshot) {
        this.newestSnapshot = newestSnapshot;
    }

    @Override
    public String getTransferSnapshot() {
        return transferSnapshot;
    }

    public void setTransferSnapshot(String transferSnapshot) {
        this.transferSnapshot = transferSnapshot;
    }

    @Override
    public int getNewestSnapshotTimestamp() {
        return newestSnapshotTimestamp;
    }

    public void setNewestSnapshotTimestamp(int newestSnapshotTimestamp) {
        this.newestSnapshotTimestamp = newestSnapshotTimestamp;

    }

    @Override
    public void restore(CModeVolume vol) {
        restore(vol, null);
    }

    @Override
    public void restore(CModeVolume vol, CModeSnapshot optionalSourceSnapshot) {
        NaElement xi = new NaElement("snapmirror-restore");
        xi.addNewChild("source-location", getDestinationLocation());
        xi.addNewChild("destination-location", vol.getSnapMirrorFormat());
        if (optionalSourceSnapshot != null) {
            xi.addNewChild("source-snapshot", optionalSourceSnapshot.getName());
        }
        xi.addNewChild("clean-up-failure", "false");
        try {
            NaElement xo = getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
            long jobID = xo.getChildLongValue("result-jobid", -1);
            // TODO aeonflux-next - no idea how to handle this.
            // according to ZAPI doc, we can use the Job API to figure out if
            // this thing
            // is done or not. However, in my experience, I have never seen it
            // complete (waited 10 min).
            // so, we're going to wait a little while, since that seems to help
            // downstream operations
            // like reading files from the volume and unmounting/destroying the
            // volume.
            logger.debug("restore result = "
                    + xo.getChildContent("result-status") + " for " + this);
            logger.debug("now waiting on job: " + jobID);
            getDestinationController().blockUntilJobCompletion(jobID, 20, 3);
        } catch (ApiFailedException e) {
            throw new SnapMirrorOperationException(this
                    + " failed to restore to volume: " + vol, e);
        }
    }

    @Override
    public void breakMirror() {
        if (getType() == Type.vault) {
            logger.debug("skipping break mirror, since this is a vault SnapMirror");
        } else {
            super.breakMirror();
        }
    }

    @Override
    public void deleteSchedule() {
        logger.debug("Enter deleteSchedule: " + scheduleName);
        if (scheduleName == null) {
            return;
        }
        NaElement xi = new NaElement("job-schedule-cron-destroy");
        xi.addNewChild("job-schedule-name", scheduleName);
        try {
            getDestinationController().getParentControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
            getDestinationController().getParentControllerImpl().getZapiCache().blastCacheValues("snap");
        } catch (ApiFailedException e) {
            if (e.getErrno() == 15661) {
                logger.info("No schedule named '"
                        + scheduleName
                        + "' will be deleted.  It does not exist, but was used by: "
                        + this);
            } else {
                logger.error("Unable to delete cron schedule for SnapMirror.", e);
                throw new SnapMirrorOperationException("Unable to delete cron schedule for SnapMirror. "
                        + e.getMessage(), e);
            }
        }
    }

    @Override
    public void destroy() throws SnapMirrorOperationException {
        logger.debug("Enter destroy: " + getSourceLocation() + " destination: "
                + getDestinationLocation());
        NaElement xi;

        xi = new NaElement("snapmirror-destroy");
        xi.addNewChild("source-location", getSourceLocation());
        xi.addNewChild("destination-location", getDestinationLocation());
        try {
            getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
            getDestinationController().getSnapMirrorRelations().remove(this);
            getDestinationController().getZapiCache().blastCacheValues("snap");
            logger.debug("SnapMirror destroy issued to: "
                    + getDestinationLocation());
        } catch (ApiFailedException e) {
            if (e.getErrno() == 15661) {
                // not found, maybe already destroyed.
                logger.debug("Relation was not found (error = " + e + ")");
            } else {
                logger.error("Unable to destroy SnapMirror relation.");
                throw new SnapMirrorOperationException("Unable to destroy SnapMirror relation. "
                        + e.getMessage(), e);
            }
        }

    }

    @Override
    public void setPolicy(CModeSnapMirrorPolicy policy) {
        logger.debug("setSnapmirrorPolicy(" + getSourceLocation() + " -> "
                + getDestinationLocation() + "): enter");
        NaElement xi;
        if (!policy.getName().equals("DPDefault") && !policy.getName().equals("XDPDefault")) {
            if (policyName == null || !policyName.equals(policy.getName())) {
                try {
                    createPolicy(policy);
                } catch (SnapMirrorOperationException ex) {
                    if (ex.getCause() instanceof ApiFailedException) {
                        ApiFailedException e = (ApiFailedException) ex.getCause();
                        if (e.getErrno() == 13001 && e.getReason().contains("A policy with this name already exists.")) {
                            modifyPolicy(policy);
                        } else {
                            throw ex;
                        }
                    }
                }
            }
            else {
                modifyPolicy(policy);
            }
        }

        generateSnapMirrorModifyPolicyCall(policy);
        logger.debug("setSnapmirrorPolicy(" + getSourceLocation() + " -> "
                + getDestinationLocation() + "): exit");
    }

    public boolean setPolicyAndCheckFields(CModeSnapMirrorPolicy policy) {
        logger.debug("setSnapmirrorPolicyAndCheckFields(" + getSourceLocation() + " -> "
                + getDestinationLocation() + "): enter");

        if (!policy.getName().equals("DPDefault") && !policy.getName().equals("XDPDefault")) {
            if (policyName == null || !policyName.equals(policy.getName())) {
                try {
                    createPolicy(policy);
                } catch (SnapMirrorOperationException ex) {
                    if (ex.getCause() instanceof ApiFailedException) {
                        ApiFailedException e = (ApiFailedException) ex.getCause();
                        if (e.getErrno() == 13001 && e.getReason().contains("A policy with this name already exists.")) {
                            logger.debug("The policy: " + policy.getName() + " already exists. Checking whether fields are the same as provided.");
                        } else {
                            throw ex;
                        }
                    }
                }
            }
        }
        generateSnapMirrorModifyPolicyCall(policy);
        CModeSnapMirrorPolicy setPolicy = this.refresh().getPolicy();
        logger.debug("setSnapmirrorPolicyAndCheckFields(" + getSourceLocation() + " -> "
                + getDestinationLocation() + "): exit");
        return StringUtils.nullSafeCompare(setPolicy.getName(), policy.getName()) && StringUtils.nullSafeCompare(setPolicy.getComment(),policy.getComment()) && StringUtils.nullSafeCompare(setPolicy.getRestart(), policy.getRestart()) && StringUtils.nullSafeCompare(setPolicy.getTransferPriority(), policy.getTransferPriority()) && StringUtils.nullSafeCompare(setPolicy.getTries(), policy.getTries());
    }

    void generateSnapMirrorModifyPolicyCall(CModeSnapMirrorPolicy policy) {
        NaElement xi = new NaElement("snapmirror-modify");
        xi.addNewChild("destination-location", getDestinationLocation());
        xi.addNewChild("source-location", getSourceLocation());
        xi.addNewChild("policy", policy.getName());

        try {
            getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
            getDestinationController().getZapiCache().blastCacheValues("snap");
            policyName = policy.getName();
        } catch (ApiFailedException e) {
            logger.error("Unable to set SnapMirror policy.", e);
            throw new SnapMirrorOperationException("Unable to set SnapMirror policy. "
                    + e.getMessage(), e);
        }
    }

    void createPolicy(CModeSnapMirrorPolicy policy) {
        logger.debug("createPolicy(" + policy.getName() + ", " + policy
                + "): enter");
        generateCreateOrModifyPolicyCall("snapmirror-policy-create", policy);
        logger.debug("createPolicy(" + policy.getName() + ", "
                + policy + "): exit");
    }

    void modifyPolicy(CModeSnapMirrorPolicy policy) {
        logger.debug("modifyPolicy(" + policy.getName() + ", " + policy
                + "): enter");
        generateCreateOrModifyPolicyCall("snapmirror-policy-modify", policy);
        logger.debug("modifyPolicy(" + policy.getName() + ", "
                + policy + "): exit");
    }

    void generateCreateOrModifyPolicyCall(String query, CModeSnapMirrorPolicy policy) {
        NaElement xi = new NaElement(query);
        xi.addNewChild("policy-name", policy.getName());
        // These are optional
        if (policy.getComment() != null) {
            xi.addNewChild("comment", policy.getComment());
        }
        if (policy.getTries() != null) {
            xi.addNewChild("tries", policy.getTries());
        }
        if (policy.getTransferPriority() != null) {
            xi.addNewChild("transfer-priority", policy.getTransferPriority());
        }
        if (policy.getRestart() != null) {
            xi.addNewChild("restart", policy.getRestart());
        }

        try {
            getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
        } catch (ApiFailedException e) {
            logger.error("Unable to create policy for SnapMirror.", e);
            throw new SnapMirrorOperationException("Unable to create policy for SnapMirror. "
                    + e.getMessage(), e);
        }
    }

    @Override
    public void setSchedule(SnapMirrorSchedule schedule) {
        logger.debug("setSnapmirrorSchedule(" + getSourceLocation() + " -> "
                + getDestinationLocation() + "): enter");
        if (scheduleName == null || !scheduleName.equals(schedule.getName())) {
            try {
                createJobScheduleCron(schedule);
            } catch (SnapMirrorOperationException ex) {
                if (ex.getCause() instanceof ApiFailedException) {
                    ApiFailedException e = (ApiFailedException) ex.getCause();
                    if (e.getErrno() == 13130 && e.getReason().contains("duplicate entry")) {
                        modifyJobScheduleCron(schedule);
                    } else {
                        throw ex;
                    }
                }
            }
        }
        else {
            modifyJobScheduleCron(schedule);
        }


        generateSnapMirrorModifyScheduleCall(schedule);
        logger.debug("setSnapmirrorSchedule(" + getSourceLocation() + " -> "
                + getDestinationLocation() + "): exit");
    }

    // return true if the schedule was created / matches an existing schedule
    // return false if the schedule did not match the existing schedule
    public boolean setScheduleAndCheckFields(SnapMirrorSchedule schedule) {
        logger.debug("setSnapmirrorScheduleAndCheckFields(" + getSourceLocation() + " -> "
                + getDestinationLocation() + "): enter");

        if (scheduleName == null || !scheduleName.equals(schedule.getName())) {
            try {
                createJobScheduleCron(schedule);
            } catch (SnapMirrorOperationException ex) {
                if (ex.getCause() instanceof ApiFailedException) {
                    ApiFailedException e = (ApiFailedException) ex.getCause();
                    if (e.getErrno() == 13130 && e.getReason().contains("duplicate entry")) {
                        logger.debug("The schedule: " + schedule.getName() + " already exists. Checking whether fields are the same as provided.");
                    } else {
                        throw ex;
                    }
                }
            }
        }

        generateSnapMirrorModifyScheduleCall(schedule);
        SnapMirrorSchedule setSchedule = this.refresh().getSchedule();
        logger.debug("setSnapmirrorScheduleAndCheckFields(" + getSourceLocation() + " -> " + getDestinationLocation() + "): exit");
        return StringUtils.nullSafeCompare(setSchedule.getName(), schedule.getName()) && StringUtils.nullSafeCompare(setSchedule.getMinutes(), schedule.getMinutes()) && StringUtils.nullSafeCompare(setSchedule.getHours(), schedule.getHours()) && StringUtils.nullSafeCompare(setSchedule.getMonths(), schedule.getMonths()) && StringUtils.nullSafeCompare(setSchedule.getDaysOfMonth(), schedule.getDaysOfMonth()) && StringUtils.nullSafeCompare(setSchedule.getDaysOfWeek(), schedule.getDaysOfWeek());
    }

    void generateSnapMirrorModifyScheduleCall(SnapMirrorSchedule schedule) {
        logger.debug("generateSnapMirrorModifyScheduleCall(" + schedule.getName() + ", "
                + schedule + "): enter");
        NaElement xi = new NaElement("snapmirror-modify");
        xi.addNewChild("destination-location", getDestinationLocation());
        xi.addNewChild("source-location", getSourceLocation());
        xi.addNewChild("schedule", schedule.getName());

        try {
            getDestinationController().invokeZapi(xi, ZapiCache.NO_CACHE);
            getDestinationController().getZapiCache().blastCacheValues("snap");
        } catch (ApiFailedException e) {
            logger.error("Unable to set SnapMirror schedule.", e);
            throw new SnapMirrorOperationException("Unable to set SnapMirror schedule. "
                    + e.getMessage(), e);
        }

        logger.debug("generateSnapMirrorModifyScheduleCall(" + schedule.getName() + "): exit");
    }

    void createJobScheduleCron(SnapMirrorSchedule schedule) {
        logger.debug("createOrModifyJobScheduleCron(" + schedule.getName() + ", "
                + schedule + "): enter");
        generateJobScheduleCronCall("job-schedule-cron-create", schedule);
        logger.debug("createOrModifyJobScheduleCron(" + schedule.getName() + "): exit");
    }

    void modifyJobScheduleCron(SnapMirrorSchedule schedule) {
        logger.debug("createOrModifyJobScheduleCron(" + schedule.getName() + ", "
                + schedule + "): enter");
        generateJobScheduleCronCall("job-schedule-cron-modify", schedule);
        logger.debug("createOrModifyJobScheduleCron(" + schedule.getName() + "): exit");
    }

    void generateJobScheduleCronCall(String query, SnapMirrorSchedule schedule) {
        NaElement xi = new NaElement(query);
        // These are required
        xi.addNewChild("job-schedule-name", schedule.getName());
        {
            NaElement jobTag = new NaElement("job-schedule-cron-minute");
            addMultiParams(schedule.getMinutes(), jobTag, "cron-minute");
            xi.addChildElem(jobTag);
        }

        // These are optional
        if (isValidCronParam(schedule.getMonths())) {
            NaElement jobTag = new NaElement("job-schedule-cron-month");
            addMultiParams(schedule.getMonths(), jobTag, "cron-month");
            xi.addChildElem(jobTag);
        }
        if (isValidCronParam(schedule.getDaysOfWeek())) {
            NaElement jobTag = new NaElement("job-schedule-cron-day-of-week");
            addMultiParams(schedule.getDaysOfWeek(), jobTag, "cron-day-of-week");
            xi.addChildElem(jobTag);
        }
        if (isValidCronParam(schedule.getDaysOfMonth())) {
            NaElement jobTag = new NaElement("job-schedule-cron-day");
            addMultiParams(schedule.getDaysOfMonth(), jobTag, "cron-day-of-month");
            xi.addChildElem(jobTag);
        }
        if (isValidCronParam(schedule.getHours())) {
            NaElement jobTag = new NaElement("job-schedule-cron-hour");
            addMultiParams(schedule.getHours(), jobTag, "cron-hour");
            xi.addChildElem(jobTag);
        }

        try {
            getDestinationController().getParentControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
        } catch (ApiFailedException e) {
            logger.error("Unable to create cron schedule for SnapMirror.", e);
            throw new SnapMirrorOperationException("Unable to create cron schedule for SnapMirror. "
                    + e.getMessage(), e);
        }
    }


    @Override
    public synchronized CModeSnapMirrorRelation getProxy() {
        if (proxy == null) {
            proxy = CachingOfftapProxy.create(CModeSnapMirrorRelation.class, CModeSnapMirrorRelationProxy.class, this, getDataCache());
        }
        return proxy;
    }

    @Override
    public Set<StorageSystem> getStorageSystems() {
        return storageSystems;
    }

    @Override
    public OntapConnection getConnection() {
        return getDestinationController().getConnection();
    }

    private boolean isValidCronParam(String cronParam) {
        return !(cronParam == null || cronParam.length() == 0 || cronParam.equals("*"));
    }

    private void addMultiParams(String cronParam, NaElement jobTag, String tagName) {
        String[] params = cronParam.split(",");
        for (String param : params) {
            jobTag.addNewChild(tagName, param.trim());
        }
    }

    @Override
    public String toString() {
        return "CModeSnapMirrorRelationImpl{" + "destinationController="
                + destinationController + ", sourceController="
                + sourceController + ", sourceCluster='" + sourceCluster + '\''
                + ", sourceVServer='" + sourceVServer + '\''
                + ", sourceVolume='" + sourceVolume + '\''
                + ", destinationVolume=" + destinationVolume
                + ", scheduleName='" + scheduleName + '\''
                + ", relationshipType='" + relationshipType + '\''
                + ", relationshipId='" + relationshipId + '\'' + '}';
    }

}
