package com.netapp.offtap3.internal.ontap.snap;

import com.netapp.offtap3.StorageSystem;
import com.netapp.offtap3.internal.CachingOfftapProxy;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapController;
import com.netapp.offtap3.internal.ontap.controller.FilerImpl;
import com.netapp.offtap3.internal.ontap.controller.OntapConnection;
import com.netapp.offtap3.internal.ontap.dataloader.OntapDataCache;
import com.netapp.offtap3.internal.ontap.proxy.SevenModeSnapMirrorRelationProxy;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.Filer;
import com.netapp.offtap3.ontap.SnapMirrorOperationException;
import com.netapp.offtap3.ontap.SnapMirrorRelation;
import com.netapp.offtap3.ontap.SnapMirrorSchedule;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.EnumSet;
import java.util.Set;

public class SevenModeSnapMirrorRelationImpl extends AbstractSnapMirrorRelationImpl<SnapMirrorRelation> implements SnapMirrorRelation {

    private static final Logger logger = Logger.getLogger(SevenModeSnapMirrorRelationImpl.class);

    private SnapMirrorRelation proxy;

    private FilerImpl<Filer> sourceController;
    private FilerImpl<Filer> destinationController;

    private static final Set<StorageSystem> storageSystems = EnumSet.of(StorageSystem.pfiler, StorageSystem.vfiler);

    public SevenModeSnapMirrorRelationImpl(FilerImpl filer, OntapDataCache dataCache) {
        super(filer, dataCache);
    }

    @Override
    public void destroy() {
        // TODO aeonflux-next - not sure if this has any meaning in 7-mode
        // since in 7-mode, the snap mirror schedule exists solely with the relationship
        // (as opposed to cDOT, where it is shared).
        try {
            deleteSchedule();
        }
        catch (SnapMirrorOperationException e) {
            logger.debug("Failed to delete schedule as part of destroy on: " + this);
        }
    }

    @Override
    public void deleteSchedule() {
        String destinationLocation = getDestinationLocation();
        logger.debug("Enter deleteSchedule: " + destinationLocation);

        NaElement xi = new NaElement("snapmirror-delete-schedule");
        xi.addNewChild("destination-location", destinationLocation);
        try {
            destinationController.invokeZapi(xi, ZapiCache.NO_CACHE);
            destinationController.getSnapMirrorRelations().remove(this);
            destinationController.getZapiCache().blastCacheValues("snap");
        } catch (ApiFailedException e){
            logger.error("Unable to delete SnapMirror schedule.", e);
            throw new SnapMirrorOperationException(destinationController +
                    " was unable to delete SnapMirror schedule for " + destinationLocation, e);
        }
    }

    public void setSchedule(SnapMirrorSchedule schedule) {
        String sourceLocation = getSourceLocation();
        String destinationLocation = getDestinationLocation();

        logger.debug("setSnapmirrorSchedule(" + sourceLocation + " -> " + destinationLocation + "): enter");
        NaElement xi = new NaElement("snapmirror-set-schedule");
        xi.addNewChild("destination-location", destinationLocation);
        xi.addNewChild("source-location", sourceLocation);
        xi.addNewChild("minutes", schedule.getMinutes());
        xi.addNewChild("hours", schedule.getHours());
        xi.addNewChild("days-of-month", schedule.getDaysOfMonth());
        xi.addNewChild("days-of-week", schedule.getDaysOfWeek());

        AbstractOntapController filer = getDestinationController();
        try {
            filer.invokeZapi(xi, ZapiCache.NO_CACHE);
            filer.getZapiCache().blastCacheValues("snap");
        } catch (ApiFailedException e) {
            logger.error("Unable to set SnapMirror schedule.", e);
            throw new SnapMirrorOperationException(filer + " was unable to set SnapMirror schedule." + e.getMessage(), e);
        }
        logger.debug("setSnapmirrorSchedule(" + sourceLocation + " -> " + destinationLocation + "): exit");
    }

    public void setSourceController(FilerImpl sourceController) {
        this.sourceController = sourceController;
    }

    public void setDestinationController(FilerImpl destinationController) {
        this.destinationController = destinationController;
    }

    @Override
    public FilerImpl getDestinationController() {
        return destinationController;
    }

    @Override
    public synchronized SnapMirrorRelation getProxy() {
        if (proxy == null) {
            proxy = CachingOfftapProxy.create(SnapMirrorRelation.class,
                    SevenModeSnapMirrorRelationProxy.class,
                    this,
                    getDataCache());
        }
        return proxy;
    }

    @Override
    public Set<StorageSystem> getStorageSystems() {
        return storageSystems;
    }

    @Override
    public OntapConnection getConnection() {
        return sourceController.getConnection();
    }
}
