package com.netapp.offtap3.internal.ontap.snap;

import com.google.common.base.Objects;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.SnapMirrorOperationException;
import com.netapp.offtap3.ontap.SnapMirrorSchedule;
import org.apache.log4j.Logger;

public class SnapMirrorScheduleImpl implements SnapMirrorSchedule {
    private static final Logger logger = Logger.getLogger(SnapMirrorScheduleImpl.class);
    private DataSvmImpl controller;

    private String name = "";
    // all values are 0-based except for day-of-month, which is 1-based
    // all values align with cron except for month, which must be converted to 1-based
    // null by default; the only ONTAP-required field is minutes
    private String minutes = null;
    private String hours = null;
    private String months = null;
    private String daysOfMonth = null;
    private String daysOfWeek = null;

    public DataSvmImpl getDestinationController() {
        return controller;
    }

    public void setDestinationController(DataSvmImpl controller) {
        this.controller = controller;
    }

    @Override
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Override
    public String getMinutes() {
        return minutes;
    }

    public void setMinutes(String minutes) {
        this.minutes = minutes;
    }

    @Override
    public String getHours() {
        return hours;
    }

    public void setHours(String hours) {
        this.hours = hours;
    }

    @Override
    public String getMonths() {
        return months;
    }

    public void setMonths(String months) {
        this.months = months;
    }

    @Override
    public String getDaysOfMonth() {
        return daysOfMonth;
    }

    public void setDaysOfMonth(String daysOfMonth) {
        this.daysOfMonth = daysOfMonth;
    }

    @Override
    public String getDaysOfWeek() {
        return daysOfWeek;
    }

    public void setDaysOfWeek(String daysOfWeek) {
        this.daysOfWeek = daysOfWeek;
    }

    @Override   // get months in 1-based cron format
    public String getCronMonths() {
        if (months != null) {
            String[] monthList = months.split(",");
            if (monthList.length == 12) {
                return "*";
            }
            else {
                try {
                    String cron = "";
                    for (int i = 0; i < monthList.length; i++) {
                        int numToAdd = Integer.parseInt(monthList[i].trim())+1;
                        if (i < monthList.length-1) cron += (numToAdd + ",");
                        else cron += numToAdd;
                    }
                    return cron;
                } catch(NumberFormatException e) {
                    logger.error("Failed to parse cron content from SnapMirrorSchedule field: months. " + "Error: " + e, e);
                }
            }
        }
        return "*";
    }

    // convert to cron special characters as necessary (question mark is only valid for day-of-month & day-of-week)
    // in full crontab format, you can only have one ? for EITHER day-of-month OR day-of-week; we default to use it
    // with day-of-month
    public String convertToCron(String field, int maxNumValues, boolean questionMarkValid) {
        if (field != null) {
            String[] values = field.split(",");
            if (values.length == maxNumValues) {
                return "*";
            }
            else return field;
        }
        else if (questionMarkValid) {
            return "?";
        }
        else {
            return "*";
        }
    }

    @Override
    public String getCrontabFormat() {
        // ONTAP doesn't do seconds, so fill in 0
        return "0 " + convertToCron(minutes,60,false) + " " + convertToCron(hours,24,false) + " " + convertToCron(daysOfMonth,31,true) + " " + getCronMonths() + " " + convertToCron(daysOfWeek,7,false);
    }

    @Override
    public int hashCode() {
        return toString().hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof SnapMirrorSchedule &&
                getCrontabFormat().equals(((SnapMirrorSchedule) obj).getCrontabFormat());
    }

    @Override
    public void destroy() {
        logger.debug("Enter destroy: SnapMirror schedule: " + name);
        NaElement xi = new NaElement("job-schedule-cron-destroy");
        xi.addNewChild("job-schedule-name", name);
        try {
            getDestinationController().getParentControllerImpl().invokeZapi(xi, ZapiCache.NO_CACHE);
            getDestinationController().getParentControllerImpl().getZapiCache().blastCacheValues("snap");
        } catch (ApiFailedException e) {
            if (e.getErrno() == 15661) {
                logger.info("No schedule named '"
                        + name
                        + "' will be deleted.  It does not exist, but was used by: "
                        + this);
            } else {
                logger.error("Unable to delete cron schedule for SnapMirror.", e);
                throw new SnapMirrorOperationException("Unable to delete cron schedule for SnapMirror. "
                        + e.getMessage(), e);
            }
        }
    }

    @Override
    public String toString() {
        return Objects.toStringHelper(this)
                .add("daysOfMonth", daysOfMonth)
                .add("daysOfWeek", daysOfWeek)
                .add("months", months)
                .add("hours", hours)
                .add("minutes", minutes)
                .toString();
    }
}
