package com.netapp.offtap3.internal.ontap.snap;

import com.netapp.offtap3.internal.AbstractRefreshable;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapController;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.utils.StringUtils;
import com.netapp.offtap3.utils.Utils;
import com.netapp.offtap3.internal.ontap.InternalZapiHandler;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.SnapVault;
import com.netapp.offtap3.ontap.SnapVaultOperationException;
import com.netapp.offtap3.ontap.NaElement;

public class SnapVaultImpl extends AbstractRefreshable implements SnapVault {
    private String sourceSystem;
    private String sourcePath;
    private String destinationSystem;
    private String destinationPath;
    private long lagTime;
    private long mirrorTimestamp;
    private String contents;
    private String lastTransferType;
    private long lastTransferSize;
    private String lastTransferFromSystem;
    private String lastTransferFromPath;
    private String status;
    private String state;
    private String baseSnapshot;

    private final InternalZapiHandler zapiHandler;

    public SnapVaultImpl(AbstractOntapController zapiHandler) {
        super(zapiHandler);
        this.zapiHandler = zapiHandler;
    }

    @Override
    public String getSourceSystem() {
        return sourceSystem;
    }

    public void setSourceSystem(String sourceSystem) {
        this.sourceSystem = sourceSystem;
    }

    @Override
    public String getSourcePath() {
        return sourcePath;
    }

    public void setSourcePath(String sourcePath) {
        this.sourcePath = sourcePath;
    }

    @Override
    public String getDestinationSystem() {
        return destinationSystem;
    }

    public void setDestinationSystem(String destinationSystem) {
        this.destinationSystem = destinationSystem;
    }

    @Override
    public String getDestinationPath() {
        return destinationPath;
    }

    public void setDestinationPath(String destinationPath) {
        this.destinationPath = destinationPath;
    }

    @Override
    public long getLagTime() {
        return lagTime;
    }

    public void setLagTime(long lagTime) {
        this.lagTime = lagTime;
    }

    @Override
    public long getMirrorTimestamp() {
        return mirrorTimestamp;
    }

    public void setMirrorTimestamp(long mirrorTimestamp) {
        this.mirrorTimestamp = mirrorTimestamp;
    }

    @Override
    public String getContents() {
        return contents;
    }

    public void setContents(String contents) {
        this.contents = contents;
    }

    @Override
    public String getLastTransferType() {
        return lastTransferType;
    }

    public void setLastTransferType(String lastTransferType) {
        this.lastTransferType = lastTransferType;
    }

    @Override
    public long getLastTransferSize() {
        return lastTransferSize;
    }

    public void setLastTransferSize(long lastTransferSize) {
        this.lastTransferSize = lastTransferSize;
    }

    @Override
    public String getLastTransferFromSystem() {
        return lastTransferFromSystem;
    }

    public void setLastTransferFromSystem(String lastTransferFromSystem) {
        this.lastTransferFromSystem = lastTransferFromSystem;
    }

    @Override
    public String getLastTransferFromPath() {
        return lastTransferFromPath;
    }

    public void setLastTransferFromPath(String lastTransferFromPath) {
        this.lastTransferFromPath = lastTransferFromPath;
    }

    @Override
    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    @Override
    public String getState() {
        return state;
    }

    public void setState(String state) {
        this.state = state;
    }

    @Override
    public String getBaseSnapshot() {
        return baseSnapshot;
    }

    public void setBaseSnapshot(String baseSnapshot) {
        this.baseSnapshot = baseSnapshot;
    }

    public void secondaryInitiateIncrementalTransfer() throws SnapVaultOperationException {
   		NaElement xi = new NaElement("snapvault-secondary-initiate-incremental-transfer");
   		xi.addNewChild("secondary-path", getDestinationPath());
        try {
            zapiHandler.invokeZapi(xi, ZapiCache.NO_CACHE);
        } catch (ApiFailedException e) {
            throw new SnapVaultOperationException(zapiHandler +
                    " failed on secondary initiate incremental transfer for " + getDestinationPath(), e);
        }
    }

    public void primaryInitiateIncrementalRestoreTransfer(String optionalSecondarySnapshot) {
        NaElement xi = new NaElement("snapvault-primary-initiate-incremental-restore-transfer");
        executeRestoreTransfer(xi, optionalSecondarySnapshot);
    }

   	public void primaryInitiateRestoreTransfer(String optionalSecondarySnapshot) throws SnapVaultOperationException {
        NaElement xi = new NaElement("snapvault-primary-initiate-restore-transfer");
        executeRestoreTransfer(xi, optionalSecondarySnapshot);
    }

    protected void executeRestoreTransfer(NaElement xi, String optionalSecondarySnapshot) {
   		xi.addNewChild("secondary-system", getDestinationSystem());
   		xi.addNewChild("secondary-path", getDestinationPath());
   		xi.addNewChild("primary-path", getSourcePath());
   		if (StringUtils.isReal(optionalSecondarySnapshot)) {
   			xi.addNewChild("secondary-snapshot", optionalSecondarySnapshot);
   		}
        try {
            zapiHandler.invokeZapi(xi, ZapiCache.NO_CACHE);
        } catch (ApiFailedException e) {
        	if (14450 == e.getErrno()) {
        		// com.netapp.offtap3.ontap.ApiFailedException: transfer attempted for busy destination (errno=14450)
        		// see this failure in unit tests - seeing if we can make it a little more robust.
        		Utils.sleep(10000);        	
	        	try {
	        		zapiHandler.invokeZapi(xi, ZapiCache.NO_CACHE);
	        		return;
	        	} catch (Throwable t2) {
	        		// don't worry about this, we'll just give up at this point.
	        	}
        	}
        	
            throw new SnapVaultOperationException(zapiHandler + " failed to execute restore transfer: " + this, e);
        }
    }

    @Override
    public SnapVault refresh() {
        return (SnapVault) super.refresh();
    }
}
