package com.netapp.offtap3.internal.ontap.snap;

import com.netapp.offtap3.internal.AbstractRefreshable;
import com.netapp.offtap3.internal.ontap.controller.AbstractOntapDataController;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.internal.ontap.volume.OntapVolumeImpl;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.NaElement;
import com.netapp.offtap3.ontap.SnapVaultOperationException;
import com.netapp.offtap3.ontap.SnapVaultSchedule;

import java.util.Date;

public class SnapVaultScheduleImpl extends AbstractRefreshable implements SnapVaultSchedule {

    private final AbstractOntapDataController filer;
    private final String volumeName;
    private final Relation relation;

    private long retentionCount;
    private String daysOfWeek;
    private String hoursOfDay;

    public SnapVaultScheduleImpl(String name, OntapVolumeImpl volume, Relation relation) {
        super(name, new Date(), volume);
        this.filer = volume.getControllerImpl();
        this.volumeName = volume.getName();
        this.relation = relation;
    }

    public long getRetentionCount() {
        return retentionCount;
    }

    public void setRetentionCount(int retentionCount) throws SnapVaultOperationException {
        NaElement xi = new NaElement("snapvault-primary-set-snapshot-schedule");
        NaElement snapshotSchedule = new NaElement("snapshot-schedule");
        xi.addChildElem(snapshotSchedule);
        NaElement theRealSnapshotSchedule = new NaElement("snapvault-primary-snapshot-schedule-info");
        snapshotSchedule.addChildElem(theRealSnapshotSchedule);
        theRealSnapshotSchedule.addNewChild("volume-name", volumeName);
        theRealSnapshotSchedule.addNewChild("schedule-name", getName());
        theRealSnapshotSchedule.addNewChild("retention-count", "" + retentionCount);
        try {
            filer.invokeZapi(xi, ZapiCache.NO_CACHE);
            filer.getZapiCache().blastCacheValues("snapvault-");
        } catch (ApiFailedException e) {
            throw new SnapVaultOperationException(filer + " failed to set primary snapshot schedule for " + getName(),
                                                  e);
        }
    }

    public void setRetentionCount(long retentionCount) {
        this.retentionCount = retentionCount;
    }

    public String getDaysOfWeek() {
        return daysOfWeek;
    }

    public void setDaysOfWeek(String daysOfWeek) {
        this.daysOfWeek = daysOfWeek;
    }

    public String getHoursOfDay() {
        return hoursOfDay;
    }

    public void setHoursOfDay(String hoursOfDay) {
        this.hoursOfDay = hoursOfDay;
    }

    @Override
    public SnapVaultSchedule refresh() {
        return (SnapVaultSchedule) super.refresh();
    }

    public void delete() throws SnapVaultOperationException {
        NaElement xi = new NaElement("snapvault-primary-delete-snapshot-schedule");
        xi.addNewChild("volume-name", volumeName);
        xi.addNewChild("schedule-name", getName());

        try {
            filer.invokeZapi(xi, ZapiCache.NO_CACHE);
            filer.getZapiCache().blastCacheValues("snapvault-");
        } catch (ApiFailedException e) {
            throw new SnapVaultOperationException(
                    filer + " failed to delete primary snapshot schedule for " + getName(), e);
        }
    }

    public void initiateSnapshot() throws SnapVaultOperationException {
        NaElement xi = new NaElement("snapvault-primary-initiate-snapshot-create");
        xi.addNewChild("volume-name", volumeName);
        xi.addNewChild("schedule-name", getName());
        try {
            filer.invokeZapi(xi, ZapiCache.NO_CACHE);
            filer.getZapiCache().blastCacheValues("snapshot-");
        } catch (ApiFailedException e) {
            throw new SnapVaultOperationException(filer + " failed to initiate primary snapshot for " + getName(), e);
        }
    }

}
