package com.netapp.offtap3.internal.ontap.support;

import com.netapp.offtap3.ontap.NaElement;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.concurrent.atomic.AtomicLong;

/**
 * Provides basic sanity testing for a ZAPI request.
 */
public class ZapiValidator {

    private AtomicLong validations = new AtomicLong(0);
    static final String[] problemTags = new String[] {"next-tag", "max-records"};
            
    private List<String> errors = new ArrayList<String>();
    
    public void isValid(NaElement naElement) throws IllegalArgumentException {
        validations.getAndIncrement();
        // I'd love to see a ZAPI XSD and validate with that.
        // This is meant to be better than nothing.
        // Background: There is a repeated bug in how -iter ZAPIs were formulated in MHC,
        // and this check is here to keep it from resurfacing in Aeonflux.
        HashSet<String> tagCounter = new HashSet<String>();
        for (NaElement child : (List<NaElement>) naElement.getChildren()) {
            for (String tag : problemTags) {
                if (child.getName().equals(tag) && tagCounter.contains(tag)) {
                    String err = "request: " + naElement + "\ncontained multiple " + tag + " elements";
                    errors.add(err);
                    throw new IllegalArgumentException(err);
                }
                if (child.getName().equals(tag)) {
                    tagCounter.add(tag);
                }
            }
        }
        String xml = naElement.toPrettyString(" ");
        if (xml.length() > (1024 * 50)) {
            String err = "request XML: " + xml + "\n was bigger than 50kB";
            errors.add(err);
            throw new IllegalArgumentException(err);
        }

    }
    
    public List<String> getErrors() {
        return errors;
    }

    public long getNumValidations() {
        return validations.longValue();
    }
}
