package com.netapp.offtap3.internal.ontap.utils;

import java.util.HashMap;
import java.util.Map;

/**
 * Helper class that looks all wall-clocks and helps to calculate engagement percentages.
 * Engagement percentage is defined as the actual wall clock we did hit divided by the
 * theoretical maximum wall clock we could have hit.  So, basically, this will give us a nice
 * crisp breakdown of "We were 20% engaged with VMware, 50% engaged with Ontap".
 * 
 * The same thing can be derived with some spreadsheet logic in a performance.csv, but I've 
 * found myself doing that about 4 times on recent cases, and figure we'll just code it up
 * to be in there to start with so everyone can see it quickly and see where problems lie
 *
 */
public class EngagementPercentageCalculator {
	private long lastCalculationTime = 0;
	private Map<String, Long> previousSetOfWallClocks = new HashMap<String, Long>();
	
	public Map<String, Long> calculateEngagementPercentages(Map<String, Long> currentValuesIncludingWallClocks) {
		Map<String, Long> currentSetOfWallClocks = calculateCurrentWallClocks(currentValuesIncludingWallClocks);
		Map<String, Long> engagementPercentages = calclulateEngagementPercentages(currentSetOfWallClocks);
		previousSetOfWallClocks = currentSetOfWallClocks;
		lastCalculationTime = System.currentTimeMillis();
		return engagementPercentages;
	}

	private Map<String, Long> calculateCurrentWallClocks(
			Map<String, Long> currentValuesIncludingWallClocks) {
		Map<String, Long> currentSetOfWallClocks = new HashMap<String, Long>();
		if (currentValuesIncludingWallClocks == null) {
			return currentSetOfWallClocks;
		}
		for (String key : currentValuesIncludingWallClocks.keySet()) {
			if (key.startsWith("wall-clock") && key.endsWith("total")) {
				// its a wall clock.  Lets get the name of this sucker.
				String wallClockKey = key.substring(11, key.length()-6);
				Number value = (Number) currentValuesIncludingWallClocks.get(key);
				if (value != null && !(value instanceof Long)) {
					value = new Long("" + value);
				}
				currentSetOfWallClocks.put(wallClockKey, (Long) value);
			}
		}
		return currentSetOfWallClocks;
	}
	
	private Map<String, Long> calclulateEngagementPercentages(Map<String, Long> currentSetOfWallClocks) {
		Map<String, Long> engagementPercentages = new HashMap<String, Long>();
		long durationSinceLastCalculation = System.currentTimeMillis() - lastCalculationTime;
		if (lastCalculationTime == 0) {
			return engagementPercentages;
		}
		for (String key : currentSetOfWallClocks.keySet()) {
			Long currentWallClock = currentSetOfWallClocks.get(key);
			Long previousWallClock = previousSetOfWallClocks.get(key);
			if (previousWallClock == null) {
				continue;
			}
			long elapsedWallClock = currentWallClock - previousWallClock;
			if (elapsedWallClock < 0) {
				// this can happen if someone resets the stats page, lets just go zero for this so we don't
				// go all negative.
				elapsedWallClock = 0;
			}
			double percentage = ((double) elapsedWallClock) / (double) durationSinceLastCalculation;
			int asPercentage = (int) Math.round(percentage*100);
			engagementPercentages.put("engagement-percentage-" + key, new Long(asPercentage));
		}
		return engagementPercentages;
	}
}
