package com.netapp.offtap3.internal.ontap.utils;

import com.netapp.offtap3.internal.ontap.utils.StatsCollector;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Class that automatically expires things that are put into it.
 * Not the world's greatest implementation - we only check every 150 inserts, 
 * and we don't override methods that might do bulk operations.
 */
public class ExpiringArrayList<E> extends ArrayList<E> {
	private Map<E, Long> insertionTimes = new HashMap<E, Long>();
	private AtomicInteger insertionCount = new AtomicInteger(0);
	private long keepDuration = 0;
	
	public ExpiringArrayList(long durationToKeep) {
		keepDuration = durationToKeep;
	}

	@Override
	public synchronized boolean add(E arg0) {
		if (arg0 != null) {
			insertionTimes.put(arg0, System.currentTimeMillis());
			insertionCount.incrementAndGet();
			if (insertionCount.get() % 150 == 0) {
				// lets clean the sucker out.
				long whackEverythingBeforeDate = System.currentTimeMillis() - keepDuration;
				int whackCount = 0;
				for(Iterator<Map.Entry<E, Long>> entryIter = insertionTimes.entrySet().iterator(); entryIter.hasNext();) {
					Map.Entry<E, Long> entry = entryIter.next();
					if (entry.getValue() < whackEverythingBeforeDate) {
						whackCount++;
						super.remove(entry.getKey());
						entryIter.remove();						
					}
				}
				StatsCollector.getInstance().addCounter("expiring-array-list-removal", whackCount);
			}
		}
		return super.add(arg0);
	}

	@Override
	public synchronized boolean remove(Object o) {
		insertionTimes.remove(o);
		return super.remove(o);
	}

	@Override
	public synchronized void clear() {
		insertionTimes.clear();
		super.clear();
	}
}
