/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/internal/ontap/volume/CModeDedupeImpl.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.ontap.volume;

import com.netapp.offtap3.ControllerConnection;
import com.netapp.offtap3.OfftapPreferences;
import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Resource;
import com.netapp.offtap3.StorageSystem;
import com.netapp.offtap3.VolumeOperationException;
import com.netapp.offtap3.internal.HasProxy;
import com.netapp.offtap3.internal.OfftapProxy;
import com.netapp.offtap3.internal.ontap.HasOntapController;
import com.netapp.offtap3.internal.ontap.controller.DataSvmImpl;
import com.netapp.offtap3.internal.ontap.proxy.CModeDedupeProxy;
import com.netapp.offtap3.internal.ontap.support.ZapiCache;
import com.netapp.offtap3.ontap.ApiFailedException;
import com.netapp.offtap3.ontap.CModeDedupe;
import com.netapp.offtap3.ontap.CModeVolume;
import com.netapp.offtap3.ontap.DataSvm;
import com.netapp.offtap3.ontap.NaElement;
import org.apache.log4j.Logger;

import java.util.Collections;
import java.util.Set;

public class CModeDedupeImpl extends DedupeImpl implements CModeDedupe, HasProxy<CModeDedupe>, HasOntapController<DataSvm, DataSvmImpl> {
    private static final Logger LOGGER = Logger.getLogger(CModeDedupeImpl.class);
    private static final Set<StorageSystem> STORAGE_SYSTEMS = Collections.singleton(StorageSystem.datasvm);
    private boolean compressionEnabled;
    private boolean inlineCompressionEnabled;
    private CModeDedupe proxy;
    private CModeVolumeImpl volume;
    private OfftapPreferences prefs;

    public CModeDedupeImpl(CModeVolumeImpl volume, OfftapPreferences prefs) {
        super(volume);
        this.volume = volume;
        this.prefs = prefs;
    }

    @Override
    public DataSvm getController() {
        return volume.getController();
    }

    @Override
    public DataSvmImpl getControllerImpl() { return volume.getControllerImpl(); }

    public void setCompressionEnabled(boolean compressionEnabled) {
        this.compressionEnabled = compressionEnabled;
    }

    @Override
    public boolean isCompressionEnabled() {
        return compressionEnabled;
    }

    public void setInlineCompressionEnabled(boolean inlineCompressionEnabled) {
        this.inlineCompressionEnabled = inlineCompressionEnabled;
    }

    @Override
    public boolean isInlineCompressionEnabled() { return inlineCompressionEnabled; }

    public void setCompression(boolean enabled, boolean inline) {
        LOGGER.debug("setCompression: start, volume = " + volume.getName() + ", enabled = " + enabled + ", inline = " + inline);
        if (!isEnabled()) {
            String message = "Deduplication compression can only be modified for a volume in which deduplication is enabled.";
            LOGGER.error(message);
            throw new VolumeOperationException(message);
        }
        if (!enabled && inline) {
            String message = "Deduplication compression must be enabled if enabling inline compression.";
            LOGGER.error(message);
            throw new VolumeOperationException(message);
        }

        NaElement input = new NaElement("sis-set-config");
        input.addNewChild("path", "/vol/" + volume.getName());
        input.addNewChild("enable-compression", String.valueOf(enabled));
        input.addNewChild("enable-inline-compression", String.valueOf(inlineCompressionEnabled));
        try {
            getControllerImpl().invokeZapi(input, ZapiCache.NO_CACHE);
            ZapiCache.getInstance().blastCacheValues("volume-");
            ZapiCache.getInstance().blastCacheValues("sis-");
        } catch (ApiFailedException e) {
            String message = "Unable to modified deduplication compression settings on volume " + volume.getName();
            LOGGER.error(message);
            throw new VolumeOperationException(message, e);
        }
        LOGGER.debug("setCompression: end");
    }

    @Override
    public CModeVolume getVolume() {
        return volume;
    }

    public CModeDedupe getProxy() {
        if (proxy == null) {
            proxy = OfftapProxy.create(CModeDedupe.class, CModeDedupeProxy.class, this);
        }
        return proxy;
    }



    @Override
    public Set<StorageSystem> getStorageSystems() {
        return STORAGE_SYSTEMS;
    }

    @Override
    public ControllerConnection getConnection() {
        return volume.getConnection();
    }

    @Override
    public void refresh(Resource... resources) {
        // Handled by the proxy.
    }

    @Override
    public Refreshable refresh(Refreshable refreshable) {
        // Handled by the proxy
        return null;
    }

    @Override
    public CModeDedupe refresh() {
        return (CModeDedupe) super.refresh();
    }
}
