package com.netapp.offtap3.internal.ontap.zapi;

import com.netapp.offtap3.ontap.ZapiAccessPermissions;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

public class CModeZapiAccessPermissions implements ZapiAccessPermissions {

     private Map<String, Collection<String>> roZapisPerVServer = new HashMap<String, Collection<String>>();
     private Map<String, Collection<String>> rwZapisPerVServer = new HashMap<String, Collection<String>>();
     private Map<String, List<Map>> accessLevelsForVServer = new HashMap<String, List<Map>>();

     public CModeZapiAccessPermissions() {
         super();
     }

     public void setAccessLevelsForVServer(String vServer, List<Map> pairs) {
         accessLevelsForVServer.put(vServer, pairs);
     }

     public List<Map> getAccessLevelsForVServer(String vServer) {
         List<Map> retVal = accessLevelsForVServer.get(vServer);
         if (retVal == null) {
             retVal = new ArrayList<Map>();
         }
         return retVal;
     }

     public void addRWZapi(String vServer, String zapi) {
         Collection<String> existing = rwZapisPerVServer.get(vServer);
         if (existing == null) {
             existing = new ArrayList<String>();
             rwZapisPerVServer.put(vServer, existing);
         }
         existing.add(zapi);
     }

     public void addROZapi(String vServer, String zapi) {
         Collection<String> existing = roZapisPerVServer.get(vServer);
         if (existing == null) {
             existing = new ArrayList<String>();
             roZapisPerVServer.put(vServer, existing);
         }
         existing.add(zapi);
     }

     public Collection<String> getROZapis(String vServer) {
         Collection<String> retVal = roZapisPerVServer.get(vServer);
         if (retVal == null) {
             retVal = new HashSet<String>();
         }
         return retVal;
     }

     public Collection<String> getRWZapis(String vServer) {
         Collection<String> retVal = rwZapisPerVServer.get(vServer);
         if (retVal == null) {
             retVal = new HashSet<String>();
         }
         return retVal;
     }

     public Collection<String> getVServers() {
         HashSet<String> toReturn = new HashSet<String>(roZapisPerVServer.keySet());
         toReturn.addAll(accessLevelsForVServer.keySet());
         return toReturn;
     }

     /**
      * Get the most permissive vserver in the cluster
      * @return
      */
     public String getBestVServer() {
         String bestIdea = "";
         int bestSize = -1;
         for (String key : rwZapisPerVServer.keySet()) {
             int currentSize = rwZapisPerVServer.get(key).size();
             if (currentSize > bestSize) {
                 bestSize = currentSize;
                 bestIdea = key;
             }
         }
         return bestIdea;
     }

     public static String determineAccessLevel(String commandPattern, List<Map> accessLevelsToCommandTreesForThisVServer) {
         commandPattern = commandPattern.trim();
         for (Map accessLevelPatternPair : accessLevelsToCommandTreesForThisVServer) {
             if (accessLevelPatternPair.get(commandPattern) != null) {
                 return (String) accessLevelPatternPair.get(commandPattern);
             }
         }
         // didn't return, still here.
         while(true) {
             char lastChar = commandPattern.charAt(commandPattern.length()-1);
             commandPattern = commandPattern.substring(0, commandPattern.length()-1);
             if (lastChar == ' ') {
                 return determineAccessLevel(commandPattern, accessLevelsToCommandTreesForThisVServer);
             }
             if (commandPattern.length() == 0) {
                 // lets go find DEFAULT
                 for (Map p : accessLevelsToCommandTreesForThisVServer) {
                     if (p.get("DEFAULT") != null) {
                         return (String) p.get("DEFAULT");
                     }
                 }

                 return "none";
             }
         }
     }

     public String determineAccessLevel(String commandPattern, String vServer) {
         // lets see if the precise commandPattern is there.  If it is not, then lets start chopping backwards.
         // if we don't find it all the way at the end, lets return none.
         List<Map> accessLevelsToCommandTreesForThisVServer = this.getAccessLevelsForVServer(vServer);
         return determineAccessLevel(commandPattern, accessLevelsToCommandTreesForThisVServer);
     }

}
