package com.netapp.offtap3.internal.ontap.zapi;


import com.netapp.offtap3.utils.Utils;
import com.netapp.offtap3.ontap.OntapController;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * In the original implementation that Dan implemented, we relied on the system-cli zapi
 * to generate the ZapitoCommand mapping table. However while testing this proved to be a not
 * so reliable method. Hence we always rely on the static table.
 */
public class ZapiCommandMap {

    private static final String DEFAULT_NAME = "zapiToCommand.mapping";
    private static final ZapiCommandMap singleton = new ZapiCommandMap();

    private final Map<String, String> factoryBuiltZapis = new HashMap<String, String>();
    private final Map<String, String> fullSteamAliases = new HashMap<String, String>();

    private ZapiCommandMap() {
        InputStream is = null;
        InputStreamReader isr = null;
        BufferedReader reader = null;
        try {
            // this should work for running out of the jar
            is = getClass().getResourceAsStream("/" + DEFAULT_NAME);
            if (is == null) {
                // this should work for dev environment
                is = new FileInputStream("src/main/resources/" + DEFAULT_NAME);
            }
            isr = new InputStreamReader(is);
            reader = new BufferedReader(isr);
            String line = reader.readLine();
            while (line != null) {
                String[] pieces = line.split(",");
                factoryBuiltZapis.put(pieces[0], pieces[1]);
                line = reader.readLine();
            }
        }
        catch (IOException ioe) {
            System.err.println("INTERNAL FAILURE when loading required resource: " + DEFAULT_NAME + " due to: " + ioe );
            ioe.printStackTrace();
            throw new RuntimeException("INTERNAL FAILURE when loading required resource: " + DEFAULT_NAME, ioe);
        }
        finally {
            Utils.safeClose(reader);
            Utils.safeClose(isr);
            Utils.safeClose(is);
        }

        loadAliases();
    }

    private void loadAliases() {
        fullSteamAliases.put("lun initiatorListMap show", "lun mapping show-initiator");
        fullSteamAliases.put("lun map", "lun mapping create");
        fullSteamAliases.put("lun mapped show", "lun mapping show");
        fullSteamAliases.put("lun unmap", "lun mapping delete");
        fullSteamAliases.put("vserver services unix-group show", "vserver services name-service unix-group show");
        fullSteamAliases.put("vserver services unix-group adduser", "vserver services name-service unix-group adduser");
        fullSteamAliases.put("vserver services unix-group create", "vserver services name-service unix-group create");
        fullSteamAliases.put("vserver services unix-user create", "vserver services name-service unix-user create");
        fullSteamAliases.put("vserver services unix-user show", "vserver services name-service unix-user show");
    }

    public static ZapiCommandMap getInstance() {
        return singleton;
    }

    public String getCommand(String zapi, OntapController controller) {
        String command = factoryBuiltZapis.get(zapi);
        if (controller.getVersion().atLeastFS() && fullSteamAliases.containsKey(command)) {
            command = fullSteamAliases.get(command);
        }
        return command;
    }

    public Set<String> getZapis() {
        return factoryBuiltZapis.keySet();
    }

    public Map<String, String> getMap() {
        return factoryBuiltZapis;
    }

}
