/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.internal.utils;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;

public class FilteredSslSocketFactory  extends SSLSocketFactory {
    SSLSocketFactory factory;
    private boolean useTls;
    private boolean useSslv3;

    public FilteredSslSocketFactory(SSLSocketFactory wrappedFactory, boolean useTls, boolean useSslv3) {
        this.factory = wrappedFactory;
        this.useTls = useTls;
        this.useSslv3 = useSslv3;
    }


    @Override
    public String[] getDefaultCipherSuites() {
        return factory.getDefaultCipherSuites();
    }

    @Override
    public String[] getSupportedCipherSuites() {
        return factory.getSupportedCipherSuites();
    }

    @Override
    public Socket createSocket(Socket socket, String s, int i, boolean b) throws IOException {
        Socket newSocket = factory.createSocket(socket, s, i, b);
        updateProtocols(newSocket);
        return newSocket;
    }

    @Override
    public Socket createSocket(String s, int i) throws IOException {
        Socket newSocket = factory.createSocket(s, i);
        updateProtocols(newSocket);
        return newSocket;
    }

    @Override
    public Socket createSocket(String s, int i, InetAddress inetAddress, int i2) throws IOException {
        Socket newSocket = factory.createSocket(s, i, inetAddress, i2);
        updateProtocols(newSocket);
        return newSocket;
    }

    @Override
    public Socket createSocket(InetAddress inetAddress, int i) throws IOException {
        Socket newSocket = factory.createSocket(inetAddress, i);
        updateProtocols(newSocket);
        return newSocket;
    }

    @Override
    public Socket createSocket(InetAddress inetAddress, int i, InetAddress inetAddress2,
                               int i2) throws IOException {
        Socket newSocket = factory.createSocket(inetAddress, i, inetAddress2, i2);
        updateProtocols(newSocket);
        return newSocket;
    }

    private void updateProtocols(Socket socket) {
        if (socket instanceof SSLSocket) {
            SSLSocket sslsock = (SSLSocket) socket;

            String[] protocols = sslsock.getEnabledProtocols();
            List<String> protocolList = new ArrayList<String>(Arrays.asList(protocols));
            /**
             *  Remove SSLv2 protocol to enforce stronger communication.
             */
            protocolList.remove("SSLv2Hello");

            for (String proto : protocols) {
                if (!useTls && proto.toLowerCase().startsWith("tls")) {
                    protocolList.remove(proto);
                }
                if (!useSslv3 && proto.toLowerCase().startsWith("ssl")) {
                    protocolList.remove(proto);
                }
            }
            protocols = protocolList.toArray(new String[protocolList.size()]);
            sslsock.setEnabledProtocols(protocols);

            List<String> cipherList = new ArrayList<String>();
            String[] ciphers = sslsock.getEnabledCipherSuites();
            for (String cipher : ciphers) {
                /**
                 *  Removing MD5 enabled ciphers due to
                 *  known vulnerability (CVE-2004-2761).
                 */
                if (!cipher.contains("MD5")) {
                    cipherList.add(cipher);
                }
            }
            ciphers = cipherList.toArray(new String[cipherList.size()]);
            sslsock.setEnabledCipherSuites(ciphers);
        }
    }
}
