/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.mars;

import org.apache.log4j.Logger;

import com.netapp.offtap3.internal.mars.webmapi.model.common.ErrorType;
import com.netapp.offtap3.internal.mars.webmapi.model.common.MessageType;
import com.netapp.offtap3.internal.mars.webmapi.util.MarsError;

public class MarsApiFailedException extends Exception {
    private static final Logger LOGGER = Logger.getLogger(MarsApiFailedException.class);
    private MarsError marsError;
    private String messageTxt;

    public MarsApiFailedException(ErrorType errorType) {
        MessageType errorMessage = errorType.getErrorMessage();
        MessageType causeMessage = errorType.getCauseMessage();
        if (errorMessage != null || causeMessage != null) {
            int error = -1;
            MessageType message;
            if (causeMessage != null) {
                message = causeMessage;
            } else {
                message = errorMessage;
            }
            try {
                error = Integer.parseInt(message.getKey());
            } catch (NumberFormatException e) {
                LOGGER.error("Got number format exception when parsing error message number.", e);
            }
            messageTxt = message.getText();
            marsError = MarsError.valueOf(error);
        }
        if (marsError == null) {
            marsError = MarsError.UNKNOWN;
        }
    }

    public MarsApiFailedException(MarsError error) {
        this.marsError = error;
    }

    public MarsApiFailedException(MarsError error, Throwable cause) {
        super(cause);
        this.marsError = error;
    }

    public MarsError getError() {
        return marsError;
    }

    @Override
    public String getMessage() {
        if (messageTxt != null) {
            return messageTxt;
        } else {
            return marsError.getReason();
        }
    }
}
