package com.netapp.offtap3.ontap;


import com.netapp.offtap3.Size;
import com.netapp.offtap3.UnixPermission;
import com.netapp.offtap3.VolumeLoader;
import com.netapp.offtap3.VolumeOperationException;
import com.netapp.offtap3.VolumeResource;

public interface CModeVolume extends OntapVolume {

    DataSvm getController();

    @Override
    @VolumeLoader(VolumeResource.snapshots)
    CModeSnapshot getSnapshotByName(String name);

    void mount(String exportPath) throws VolumeOperationException;

    void unmount() throws VolumeOperationException;

    @Override
    @VolumeLoader(VolumeResource.state)
    String getState();

    @Override
    @VolumeLoader(VolumeResource.state)
    boolean isInconsistent();

    @VolumeLoader(VolumeResource.state)
    boolean isVserverRoot();

    @VolumeLoader(VolumeResource.state)
    boolean isClusterVolume();

    @Override
    @VolumeLoader(VolumeResource.state)
    String getType();

    @VolumeLoader(VolumeResource.striping)
    int getStripeWidth();

    boolean isNodeRoot();
    
    @VolumeLoader(VolumeResource.state)
    boolean isInNvfailedState();

    @VolumeLoader(VolumeResource.inode)
    long getFilesTotal();

    @VolumeLoader(VolumeResource.inode)
    long getFilesUsed();

    @VolumeLoader(VolumeResource.snapshot_autodelete)
    boolean isAutoDeleteEnabled();

    @Override
    @VolumeLoader(VolumeResource.snapshot_autodelete)
    String getAutoDeleteTrigger();

    @VolumeLoader(VolumeResource.snapshot_autodelete)
    String getAutoDeleteCommitment();

    @Override
    @VolumeLoader(VolumeResource.space)
    int getSnapshotPercentReserved();

    @Override
    @VolumeLoader(VolumeResource.space)
    Size getSnapshotSpaceReserved();

    @Override
    @VolumeLoader(VolumeResource.space)
    int getSnapshotReservePercentUsed();

    @Override
    @VolumeLoader(VolumeResource.space)
    Size getSnapshotReserveUsed();

    @Override
    @VolumeLoader(VolumeResource.space)
    Size getSizeUsed();

    @Override
    @VolumeLoader(VolumeResource.space)
    Size getSizeTotal();

    @Override
    @VolumeLoader(VolumeResource.space)
    int getPercentageUsed();

    @Override
    @VolumeLoader(VolumeResource.space)
    Size getSizeAvailable();

    @Override
    @VolumeLoader(VolumeResource.space)
    boolean isSpaceReserveEnabled();

    @Override
    @VolumeLoader(VolumeResource.space)
    String getSpaceReserve();

    @Override
    @VolumeLoader(VolumeResource.space)
    int getFractionalReserve();

    @Override
    @VolumeLoader(VolumeResource.vm_align)
    int getVmAlignSector();

    @Override
    @VolumeLoader(VolumeResource.vm_align)
    String getVmAlignSuffix();

    @Override
    @VolumeLoader(VolumeResource.auto_size)
    Size getAutoSizeIncrement();

    @Override
    @VolumeLoader(VolumeResource.auto_size)
    Size getAutoSizeMax();

    @Override
    @VolumeLoader(VolumeResource.auto_size)
    boolean isAutoSizeEnabled();

    @Override
    @VolumeLoader(VolumeResource.performance)
    boolean isAtimeUpdateEnabled();

    @Override
    @VolumeLoader(VolumeResource.language)
    boolean isConvertUcodeEnabled();

    @Override
    @VolumeLoader(VolumeResource.language)
    boolean isCreateUcodeEnabled();

    @Override
    @VolumeLoader(VolumeResource.clone)
    int getCloneChildCount();

    @Override
    @VolumeLoader(VolumeResource.clone)
    String getParentSnapshotName();

    @Override
    @VolumeLoader(VolumeResource.clone)
    String getParentVolumeName();

    String getFlexVolType();    
    
    @Override
    @VolumeLoader(VolumeResource.id)
    String getContainingAggregateName();
    
    @VolumeLoader(VolumeResource.id)
    String getAggregateUUID();

    @Override
    CModeAggregate getContainingAggregate();

    boolean isMoving();
    
    void createAndInitializeSnapMirror(String sourceVserver, String sourceVolume, CModeSnapMirrorRelation.Type type);

    void initializeSnapMirror(CModeVolume sourceVol, CModeSnapMirrorRelation.Type type);

    @Override
    CModeFile getFile(String relativePath);

    @Override
    @VolumeLoader(VolumeResource.sis)
    CModeDedupe getDedupe();

    @Override
    CModeLun getLun(String path);

    @VolumeLoader(VolumeResource.snapdir)
    boolean isSnapDirAccessEnabled();

    @VolumeLoader(VolumeResource.exports)
    String getExportPolicyName();

    void setExportPolicy(ExportPolicy exportPolicy);

    @VolumeLoader(VolumeResource.qos)
    QosPolicyGroup getQosPolicyGroup();

    void updateLSMirror(int timeout);

    @VolumeLoader(VolumeResource.security)
    UnixPermission getUnixPermission();

    /**
     * Assigns new permissions to this volume.
     * @param newUserId the new user ID.  if null, the existing userId is maintained.
     * @param newGroupId the new group ID.  if null, the existing groupId is maintained.
     * @param newPermissions the new permissions. if null, the existing permissions are maintained.
     */
    void setUnixPermission(Integer newUserId, Integer newGroupId, String newPermissions);

    void setSnapDirAccess(boolean enabled);
    
    void setInNvfailedState(boolean inNvfailedState);

    @Override
    CModeVolume refresh();
}

