/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/ontap/DataSvm.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.ontap;

import com.netapp.offtap3.ControllerLoader;
import com.netapp.offtap3.ControllerResource;
import com.netapp.offtap3.FileOperationException;
import com.netapp.offtap3.HasInterfaces;
import com.netapp.offtap3.HasParentController;
import com.netapp.offtap3.HasUnixUsersAndGroups;
import com.netapp.offtap3.UnixGroup;
import com.netapp.offtap3.UnixUser;
import com.netapp.offtap3.utils.Pair;

import java.util.List;
import java.util.Map;

public interface DataSvm extends OntapDataController, Svm, HasFcInterfaces, HasPerformanceObjects,
        HasCModeAggregates, HasPortSets, SupportsNdmp, HasUnixUsersAndGroups, HasParentController {
    @ControllerLoader(ControllerResource.protocols)
    String getIscsiNodeName();

    @ControllerLoader(ControllerResource.svm_peers)
    List<SvmPeer> getSvmPeers();

    @Override
    @ControllerLoader(ControllerResource.volumes)
    public CModeVolume getVolume(String name);

    @Override
    @ControllerLoader(ControllerResource.volumes)
    Map<String, CModeVolume> getVolumes();

    @Override
    CModeVolume createVolume(OntapVolumeSpec volumeSpec);

    @Override
    CModeLun createLun(OntapLunSpec lunSpec);

    @Override
    @ControllerLoader(ControllerResource.luns)
    CModeLun getLun(String path);

    @ControllerLoader(ControllerResource.lun_bindings)
    List<LunBinding> getLunBindings();

    String getRootVolumeAggregate();    

    String getSnapShotPolicy();

    void updateSnapMirrorRelations();

    @Override
    @ControllerLoader(ControllerResource.snapmirror)
    List<CModeSnapMirrorRelation> getSnapMirrorRelations();

    @Override
    @ControllerLoader(ControllerResource.unix_users_groups)
    Map<String, UnixGroup> getUnixGroups();

    @Override
    @ControllerLoader(ControllerResource.unix_users_groups)
    Map<String, UnixUser> getUnixUsers();

    boolean supportsQos();
    
    public DataTransferProcess getDataTransfer(String theVal, boolean isLun, boolean isMove);

    /**
     * QOS policy group associated with this svm, could be default.
     *
     * @return the QoS policy group applied to this svm.
     */
    @ControllerLoader(ControllerResource.qos)
    QosPolicyGroup getQosPolicyGroup();

    /**
     * Shows all of the QoS policy groups known to this svm.
     *
     * @return all qos policy groups on the svm
     */
    @ControllerLoader(ControllerResource.qos)
    Map<String, QosPolicyGroup> getQosPolicyGroups();

    /**
     * QOS policy group with the specified name, within this svm's scope.
     *
     * @param name which QoS policy group to retrieve
     * @return the QoS policy group with the given name.
     */
    @ControllerLoader(ControllerResource.qos)
    QosPolicyGroup getQosPolicyGroup(String name);

    /**
     * Create a new QoS policy group that is initially not associated to any
     * storage object.
     *
     * @param name          name for new policy group
     * @param maxThroughput max throughput
     * @return the new policy group
     */
    QosPolicyGroup createQosPolicyGroup(String name, QosThroughput maxThroughput);

    @ControllerLoader(ControllerResource.export_policies)
    Map<String, ExportPolicy> getExportPolicies();

    ExportPolicy createExportPolicy(String policyName);

    DataTransferProcess copyFilesOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs, DataSvm destinationController) throws FileOperationException;
    DataTransferProcess copyFilesOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs, DataSvm destinationController, CModeFile referenceFile) throws FileOperationException;

    DataTransferProcess moveFilesOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs, DataSvm destinationController) throws FileOperationException;
    DataTransferProcess moveFilesOnDemand(List<Pair<CModeFile, CModeFile>> srcDestPairs, DataSvm destinationController, CModeFile referenceFile) throws FileOperationException;

    DataTransferProcess copyLunsOnDemand(List<Pair<CModeLun, String>> srcDestPairs, DataSvm destinationController) throws FileOperationException;
    DataTransferProcess copyLunsOnDemand(List<Pair<CModeLun, String>> srcDestPairs, CModeLun referenceLun, DataSvm destinationController) throws FileOperationException;

    DataTransferProcess moveLunsOnDemand(List<Pair<CModeLun, String>> srcDestPairs, DataSvm destinationController) throws FileOperationException;
    DataTransferProcess moveLunsOnDemand(List<Pair<CModeLun, String>> srcDestPairs, CModeLun referenceLun, DataSvm destinationController) throws FileOperationException;

    AdminSvm getParentController();

    boolean isDirectSvm();

    void assignIscsiNodeName(String iscsiName);

    State getState();

    void start();

    void stop();

    void rename(String newName);

    /**
     * Destroys this svm.
     */
    void destroy();

    /**
     * Destroys all volumes and igroups, and then destroys this svm.
     */
    void obliterate();

    void enableISCSI();

    void disableISCSI();

    void enableFCP();

    void enableNFS();

    void disableNFS();

    @ControllerLoader(ControllerResource.protocols)
    NfsDetails getNfsDetails();

    boolean isRepository();

    enum State {RUNNING, STOPPED, STARTING, STOPPING, INITIALIZING, DELETING}
}
