package com.netapp.offtap3.ontap;


import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Size;

public interface Dedupe extends Refreshable {

    /**
     * Returns <code>true</code> if SIS is enabled on this volume;
     * <code>false</code> otherwise
     *
     * @return <code>true</code> if SIS is enabled on this volume;
     * <code>false</code> otherwise
     */
    boolean isEnabled() ;

    /**
     * Returns <code>true</code> if SIS is active on this volume;
     * <code>false</code> otherwise
     *
     * @return <code>true</code> if SIS is active on this volume;
     * <code>false</code> otherwise
     */
    boolean isActive() ;

    /**
     * Returns the SIS type on this volume
     *
     * @return the SIS type on this volume
     */
    String getType() ;

    /**
     * Returns the SIS status of this volume
     *
     * @return the SIS status of this volume
     */
    String getStatus();

    /**
     * Returns the SIS shared size of this volume
     *
     * @return the SIS shared size of this volume
     */
    Size getSizeShared() ;

    /**
     * Get current SIS state.
     *
     * @return Current SIS state.
     */
    String getState() ;

    /**
     * Returns the SIS saved size of this volume
     *
     * @return the SIS saved size of this volume
     */
    Size getSizeSaved();

    /**
     * Returns the SIS saved percentage
     *
     * @return the SIS saved percentage
     */
    int getPercentSaved();

    /**
     * Returns the last SIS begin date
     *
     * @return the last SIS begin date
     */
    String getLastBeginDate();

    /**
     * Returns the last SIS end date
     *
     * @return the last SIS end date
     */
    String getLastEndDate();

    /**
     * Returns SIS schedule.
     *
     * @return The SIS schedule.
     */
    String getSchedule();

    OntapVolume getVolume();

    @Override
    Dedupe refresh();
}
