package com.netapp.offtap3.ontap;

import com.netapp.offtap3.Refreshable;
import com.netapp.offtap3.Volume;

import java.util.List;
import java.util.Set;

public interface Export extends Refreshable {

    enum Permission {
            read_only("read-only"), read_write("read-write"), root("root");
            private final String zapiValue;
            private Permission(String value){ this.zapiValue = value;}
            public String zapiValue(){ return zapiValue;}
        }

    /**
     * The path that is exported (may be different from the underlying volume/qtree).
     * @return exported path
     */
    String getPath();

    List<ExportSecurityRule> getSecurityRules();

    /**
     * Determine if the given client IP address can access this export with the
     * requested permissions.
     * @return true if the IP address has the requested access
     */
    boolean allowsClient(String clientIpAddress, Permission permission);

    void delete();

    /**
     * Returns a Set of hosts/client matching strings which are associated with this export as allowed to connect
     * and perform both reads and writes. Note that these strings could either be an IP address, IP subnet in the form
     * x.x.x.x/y.y.y.y where y.y.y.y is the netmask, host name, net group (e.g. @netgroup), or domain (e.g. .example.com).
     */
    Set<String> getReadWriteHosts();

    /**
     * Returns a Set of hosts/client matching strings which are associated with this export as allowed to connect
     * and perform both reads only. Note that these strings could either be an IP address, IP subnet in the form
     * x.x.x.x/y.y.y.y where y.y.y.y is the netmask, host name, net group (e.g. @netgroup), or domain (e.g. .example.com).
     */
    Set<String> getReadOnlyHosts();

    /**
     * Returns a Set of hosts/client matching strings which are associated with this export as allowed to connect
     * as a super user. Note that these strings could either be an IP address, IP subnet in the form
     * x.x.x.x/y.y.y.y where y.y.y.y is the netmask, host name, net group (e.g. @netgroup), or domain (e.g. .example.com).
     */
    Set<String> getRootAccessHosts();

    /**
     * Returns true if this export can be mounted in read/write format by all hosts.
     */
    boolean isReadWriteForAllHosts();

    /**
     * Returns the volume object associated with this export.
     */
    Volume getVolume();

    @Override
    Export refresh();
}
