package com.netapp.offtap3.ontap;

import com.google.common.base.Objects;

import java.util.ArrayList;
import java.util.List;

public class FileRange implements Comparable<FileRange> {

    // the maximum hole size that can be punched is 4MB
    public static final long DEFAULT_MAX_HOLE_SIZE = 4194304L; // the maximum hole size that can be punched is 4MB

    private final long offset;
    private final long length;

    public FileRange(long offset, long length) {
        this.offset = offset;
        this.length = length;
    }

    public long getOffset() {
        return offset;
    }

    public long getLength() {
        return length;
    }

    public List<FileRange> toBiteSizedList() {
        return getBiteSizedList(DEFAULT_MAX_HOLE_SIZE);
    }

    public List<FileRange> getBiteSizedList(long maxHoleSize) {
        long numberOfBytes = length;
        long byteOffset = offset;
        List<FileRange> biteSizeHoles = new ArrayList<FileRange>();
        while (numberOfBytes > 0) {
            biteSizeHoles.add(new FileRange(byteOffset, Math.min(numberOfBytes, maxHoleSize)));
            numberOfBytes -= maxHoleSize;
            byteOffset += maxHoleSize;
        }
        return biteSizeHoles;
    }

    /**
     * NtapHoles are ordered by their offset.
     *
     * @param other
     * @return
     */
    @Override
    public int compareTo(FileRange other) {
        return offset < other.getOffset() ? -1 : (offset == other.getOffset()) ? 0 : 1;
    }

    public boolean equals(Object obj) {
        if (obj instanceof FileRange) {
            FileRange other = (FileRange) obj;
            return Objects.equal(offset, other.getOffset())
                    && Objects.equal(length, other.getLength());
        }
        return false;
    }

    public int hashCode() {
        return (int) (offset ^ length);
    }

    public String toString() {
        return "(offset: " + Long.toHexString(offset) + ", length: " + Long.toHexString(length) + ")";
    }

}
