package com.netapp.offtap3.ontap;

import com.netapp.offtap3.File;
import com.netapp.offtap3.FileOperationException;

import java.util.List;

public interface OntapFile extends File {
    long getReserveSize();

    void punchHoles(List<FileRange> holes) throws FileOperationException;

    /**
     * Initiates an NDMP copy process for this file to the specified location. The process is synchronous and will
     * return when the copy process completes. Several timeout preferences can be set in OfftapPreferences.
     * @param newLocation The location of the copy.
     * @return An NdmpCopyProcess object that contains the results of the copy including the return code and the
     * exception if one occurred.
     * @throws FileOperationException
     * @see com.netapp.offtap3.OfftapPreferences#getNdmpProgressTimeout()
     * @see com.netapp.offtap3.OfftapPreferences#getNdmpBeginTimeout()
     */
    NdmpCopyProcess ndmpCopy(OntapFile newLocation) throws FileOperationException;

    /**
     * Initiates an NDMP copy process for this file to the specified location. The process is asynchronous and will
     * return immediately. The copy process can be monitored through the NdmpCopyProcess object. Several timeout
     * preferences can be set in OfftapPreferences that change how the NdmpCopyProcess determines when there is a
     * timeout failure.
     * @param newLocation The location of the copy.
     * @return An NdmpCopyProcess object that contains the results of the copy including the return code and the
     * exception if one occurred.
     * @throws FileOperationException
     * @see com.netapp.offtap3.OfftapPreferences#getNdmpProgressTimeout()
     * @see com.netapp.offtap3.OfftapPreferences#getNdmpBeginTimeout()
     */
    NdmpCopyProcess asyncNdmpCopy(OntapFile newLocation) throws FileOperationException;

    void sisClone(OntapFile newLocation) throws FileOperationException;
    void sisClone(CloneSpec cloneSpec) throws FileOperationException;

    OntapVolume getVolume();

    /**
     * Create a lun based on this file at the specified path
     * @param lunPath The volume relative path to the newly created lun.
     * @param type os type
     * @param spaceReserve whether or not space reservation is enabled
     * @param comment optional comment for the new lun
     * @return the new lun
     */
    OntapLun createLun(String lunPath, String type, boolean spaceReserve, String comment);

    @Override
    List<? extends OntapFile> listFiles();

    @Override
    OntapFile createFile(String relativePath);

    @Override
    OntapFile getParentDir();

    String getVolumeRelativePath();

    CloneSpec.Builder getCloneSpecBuilder(File destinationFile);

    @Override
    OntapFile refresh();
}
