package com.netapp.offtap3.ontap;


import com.netapp.offtap3.utils.Utils;
import org.apache.log4j.Logger;
import org.quartz.CronExpression;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;


/**
 * Helper class for dealing with Ontap schedules - can be refactored as needed
 *
 * @author tdeepak
 */
public class OntapScheduleUtils {

    private static final int Sample_Size = 10;
    private static Logger logger = Logger.getLogger(OntapScheduleUtils.class);

    public static String constructCronJobString(List<Integer> mins, List<Integer> hours, List<Integer> daysOfMonth, List<Integer> months, List<Integer> daysOfWeek) {
        StringBuffer cronJobString = new StringBuffer("");

        //ontap doesnt support cron at second granularity
        cronJobString = cronJobString.append("0" + " ");


        if (!Utils.isReal(mins)) {
            cronJobString = cronJobString.append("0" + " ");
        } else {
            cronJobString = cronJobString.append(convertListToDelimiterSeparated(mins, ',')).append(" ");
        }

        if (!Utils.isReal(hours)) {
            cronJobString = cronJobString.append("*" + " ");
        } else {
            cronJobString = cronJobString.append(convertListToDelimiterSeparated(hours, ',')).append(" ");
        }

        if (!Utils.isReal(daysOfMonth)) {
            cronJobString = cronJobString.append("?" + " ");
        } else {
            cronJobString = cronJobString.append(convertListToDelimiterSeparated(daysOfMonth, ',')).append(" ");
        }

        if (!Utils.isReal(months)) {
            cronJobString = cronJobString.append("*" + " ");
        } else {
            //ontap month start from zero , cron parsing library starts from 1
            List<Integer> monthsIncremented = new ArrayList<Integer>();
            for (Integer temp : months) {
                int increment = temp + 1;
                monthsIncremented.add(increment);
            }
            cronJobString = cronJobString.append(convertListToDelimiterSeparated(monthsIncremented, ',')).append(" ");
        }

        if (!Utils.isReal(daysOfWeek)) {
            cronJobString = cronJobString.append("*" + " ");
        } else {
            //ontap days start from zero , cron parsing library starts from 1
            List<Integer> dayOfWeekIncremented = new ArrayList<Integer>();
            for (Integer temp : daysOfWeek) {
                int increment = temp + 1;
                dayOfWeekIncremented.add(increment);
            }
            cronJobString = cronJobString.append(convertListToDelimiterSeparated(dayOfWeekIncremented, ','))
                                         .append(" ");
        }

        return cronJobString.toString();
    }

    public static String convertListToDelimiterSeparated(List<?> list, char delimiter) {
        StringBuilder cronJobString = new StringBuilder("");
        for (Object o : list) {
            cronJobString.append(o.toString());
            cronJobString.append(delimiter);
        }
        String toReturn = cronJobString.toString();
        //trim trailing delim
        toReturn = toReturn.substring(0, toReturn.length() - 1);
        return toReturn;
    }

    public static Long parseCronJobStringAndReturnScheduleInMins(String cronJobString) {
        Long timeInMins = null;
        CronExpression expr = null;
        try {
            expr = new CronExpression(cronJobString);
        } catch (ParseException e) {
            logger.error("Unable to parse crob job string:" + e.getMessage());
            e.printStackTrace();
        }
        if (expr != null) {
            Long timeInMilliSecs = returnLargestTimeBetweenScheduledTimesInMilliSecs(expr);
            timeInMins = TimeUnit.MILLISECONDS.toMinutes(timeInMilliSecs);
        }
        return timeInMins;
    }


    public static Long returnLargestTimeBetweenScheduledTimesInMilliSecs(CronExpression expr) {
        Long time = 0L;
        Date t1 = expr.getNextValidTimeAfter(new Date());
        for (int i = 0; i < Sample_Size; i++) {
            Date t2 = expr.getNextValidTimeAfter(t1);
            Long difference = t2.getTime() - t1.getTime();
            if (difference > time) {
                time = difference;
            }
            t1 = t2;
        }
        return time;
    }

}
