/*
 * $Id: //depot/prod/aeonflux/R3.0.1/src/main/java/com/netapp/offtap3/ontap/OntapVolume.java#1 $
 *
 * Copyright (c) 2014 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.offtap3.ontap;

import com.netapp.offtap3.Size;
import com.netapp.offtap3.Volume;
import com.netapp.offtap3.VolumeLoader;
import com.netapp.offtap3.VolumeOperationException;
import com.netapp.offtap3.VolumeResource;

import java.util.List;
import java.util.Map;
import java.util.Set;

public interface OntapVolume extends Volume {

    OntapController getController();

    /**
     * Return a file handle using the path provided.
     * The path is assumed to be relative to the volume root dir.
     * If you mess that up and provide a complete absolute path
     * (e.g., starts with /vol/...), that's ok.
     * @param path relative to the volume root dir
     * @return handle representing that file path (even if it does not exist)
     */
    @Override
    OntapFile getFile(String path);

    String toFullPath(String path);

    OntapLun getLun(String path);

    Size getFreeSpace();

    String getUuid();

    String getType();

    String getState();

    boolean isSpaceReserveEnabled();

    String getSpaceReserve();

    boolean isThinProvisioned();

    Size getSizeUsed();

    Size getSizeTotal();

    boolean isInconsistent();

    Aggregate getContainingAggregate();
    
    String getContainingAggregateName();

    int getPercentageUsed();

    Size getSizeAvailable();

    String getParentSnapshotName();

    String getParentVolumeName();

    String getAutoDeleteStatus();

    String getAutoDeleteTrigger();

    Size getTotalSnapshotSpace();

    int getSnapshotPercentReserved();

    Size getSnapshotSpaceReserved();

    int getSnapshotReservePercentUsed();

    Size getSnapshotReserveUsed();

    int getCloneChildCount();

    @VolumeLoader(VolumeResource.sis)
    Dedupe getDedupe();

    Size getSharedSize();

    Size getAutoSizeIncrement();

    Size getAutoSizeMax();

    boolean isAutoSizeEnabled();
    
    void inititalizeSnapMirror(String sourceVserver, String sourceVolume);

    void initializeSnapMirror(OntapVolume sourceVol);

    @VolumeLoader(VolumeResource.snapmirror_status)
    VolumeSnapMirrorStatus getSnapMirrorStatus();

    /**
     * Get all SnapMirror relationships in which this volume participates (as either source or destination).
     */
    List<SnapMirrorRelation> getSnapMirrorRelations();

    /**
     * Get all SnapMirror relationships in which this volume is the destination.
     */
    List<SnapMirrorRelation> getSnapMirrorDestinationRelations();

    /**
     * Get all SnapMirror relationships in which this volume is the destination, and the
     * given volume is the source.
     * @param source the source for the SnapMirror relationship
     */
    List<SnapMirrorRelation> getSnapMirrorDestinationRelations(OntapVolume source);

    boolean matchesDestination(SnapMirrorRelation relation);

    boolean matchesSource(SnapMirrorRelation relation);

    String getSnapMirrorFormat();

    boolean isConvertUcodeEnabled();

    boolean isCreateUcodeEnabled();

    boolean isAtimeUpdateEnabled();

    int getFractionalReserve();

    /**
     * Qtrees on this volume, keyed by absolute path to the qtree.
     * (e.g., /vol/volume-name/qtree-name
     * @return map of qtrees, keyed by absolute path to the qtree
     */
    @VolumeLoader(VolumeResource.qtree)
    Map<String, Qtree> getQtrees();

    Qtree getQtree(String relativePath);

    /**
     * Exports for this volume, keyed by absolute path of the export exposed.
     * @return map of exports, keyed by exposed path
     */
    @VolumeLoader(VolumeResource.qtree)
    Map<String, Export> getExports();

    String getJunctionPath();

    /**
     * Helper method to find a snapshot whose name may (or may not)
     * end with ".0"
     */
    @VolumeLoader(VolumeResource.snapshots)
    Snapshot getSnapshotByName(String name);

    @VolumeLoader(VolumeResource.snapshots)
    List<Snapshot> getSnapshots();

    /**
     * creates a snapshot
     * @param name the requested name for the snapshot
     * @return the snapshot taken (Snapshot.getName() may not match name parameter if requested name was already in use)
     */
    Snapshot createSnapshot(String name);

    int getVmAlignSector();

    String getVmAlignSuffix();

    void setAutosize(Size increment, Size max, boolean enabled) throws VolumeOperationException;

    void setSis(boolean dedupeEnabled, boolean dedupeOn, boolean rescan) throws VolumeOperationException;

    void setOptions(String[][] options) throws VolumeOperationException;

    void setSnapReserve(int percent) throws VolumeOperationException;

    OntapLunSpec.Builder getLunSpecBuilder(String path, String type, Size size);

    void move(Aggregate newAggr, boolean sync);

    void export(Set<String> hosts);

    void export(String exportPath, boolean writeEnable, Set<String> hosts);

    void export(String exportPath, boolean writeEnable, boolean readWriteAccess, boolean rootAccess,
                Set<String> hosts);

    void export(String exportPath, boolean writeEnable, boolean readWriteAccess, boolean rootAccess,
                boolean exportPathIsActual, Set<String> hosts);
}
