package com.netapp.offtap3.ontap;


import com.netapp.offtap3.VolumeSpec;

/**
 * Specifies the parameters for creating a volume.
 * Options include:<ul>
 * <li> volumeName - Name for the new volume.
 * <li> aggrName - Name of the aggragate to create the volume on
 * <li> size - The size of the volume to be created
 * <li> spaceReserve - none, file or volume
 * <li> securityStyle - the security stype to apply to qtree0 (the volume)
 * <li> shimSize - The size in sectors of the "shim" to be placed in files that match shimSuffix. (vm-align-sector)
 * <li> shimSuffix - The suffix of file names that WAFL should apply the shim to. (vm-align-suffix)
 * <li> snapshotPolicy - The name of the snapshot policy to use (snapshot-policy).
 *                       If null, the parameter will not be set and 'default' will be used.
 *                       </ul>
 */
public interface OntapVolumeSpec extends VolumeSpec {

     String getAggregateName();

     SpaceReserve getSpaceReserve();

     SecurityStyle getSecurityStyle();

     String getSecurityPermissions();

     int getUserId();

     int getGroupId();

     VolumeType getVolumeType();

     String getJunctionPath();

     int getVmAlignSector();

     String getVmAlignSuffix();

     String getSnapshotPolicy();

     String getQosPolicyGroupName();

     String getExportPolicy();

    public interface Builder {

        Builder securityStyle(SecurityStyle style);

        Builder securityPermissions(String permissions);

        Builder userId(int userId);

        Builder groupId(int groupId);

        Builder spaceReserve(SpaceReserve val);

        Builder volumeType(VolumeType type);

        Builder junctionPath(String val);

        Builder vmAlign(int sector, String suffix);

        Builder snapshotPolicy(String val);

        Builder qosPolicy(String policyName);

        Builder exportPolicy(String exportPolicy);

        OntapVolumeSpec build();
    }

}
