package com.netapp.offtap3.utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class StringUtils {
    /** matches any character that is not alphanumeric or underscore or hyphen */
    private static final String NON_ALPHANUMERIC = "[^A-Za-z0-9_\\-]";

    
    /**
     * Nullsafe contains method
     * 
     * @param base
     * @param lookingFor
     * @return
     */
    public static boolean contains(String base, String lookingFor) {
        if (base != null && lookingFor != null) {
            return base.contains(lookingFor);
        } else {
            return false;
        }
    }

    public static String replaceNonAlphanumerics(String input, String replacement) {
        return input.replaceAll(NON_ALPHANUMERIC, replacement);
    }

    public static boolean isReal(String incoming) {
        if (incoming == null || incoming.length() == 0 || incoming.trim().length() == 0) {
            return false;
        }
        return true;
    }
    
    public static String getNewLine() {
    	return System.getProperty("line.separator");
    }

    public static boolean isNumeric(String incoming) {
        if (incoming == null || incoming.length() == 0 || incoming.trim().length() == 0) {
            return false;
        }

        try {
            Long.parseLong(incoming);
            return true;
        } catch (NumberFormatException e) {
            return false;
        }
    }
    
    public static String frontPad(String incoming, String pad, int length) {
        if (isReal(incoming) && isReal(pad) && length > 0) {
            while (incoming.length() < length) {
                incoming = pad + incoming;
            }
        }
        return incoming;
    }

    public static String getStringOfLength(int length) {
        String retVal = "";
        for (int i=0; i<length; i++) {
            retVal += i % 10;
        }
        return retVal;
    }
    
    public static void failOnAnyEmpty(String... incoming) {
        for (String curr : incoming) {
            if (!isReal(curr)) {
                StringBuffer message = new StringBuffer();
                message.append("Insufficient arguments of: ");
                for (String curr2 : incoming) {
                    message.append(curr2);
                    message.append(", ");
                }
                if (incoming.length > 0) {
                    message.setLength(message.length() - 2);
                }
                throw new IllegalArgumentException(message.toString());
            }
        }
    }


    public static String getPartAfterLast(String fullString, String delimiter) {
        if (fullString != null && delimiter != null) {
            int lastIndexOf = fullString.lastIndexOf(delimiter);
            if (lastIndexOf != -1) {
                return fullString.substring(lastIndexOf+1);
            }
        }
        return fullString;
    }

    public static boolean isEmpty(String incoming) {
        return !isReal(incoming);
    }

    public static int count(String theString, char toFind) {
        int retVal = 0;
        if (isEmpty(theString)) {
            return 0;
        }
        for (char c : theString.toCharArray()) {
            if (c == toFind) {
                retVal++;
            }
        }
        return retVal;
    }
    
    public static List<String> splitIntoChunks(String theValue, int size) {        
        if (theValue == null || theValue.length() < size) {
            return Arrays.asList(theValue);
        }
        StringBuffer retVal = new StringBuffer();
        List<String> strings = new ArrayList<String>();
        for (int i=0; i<theValue.length(); i++) {
            retVal.append(theValue.charAt(i));
            if (retVal.length() >= size) {
                strings.add(retVal.toString());
                retVal.setLength(0);
            }               
        }
        if (retVal.length() > 0) {
            strings.add(retVal.toString());
        }
        return strings;
    }
    
    public static List<String> splitIntoList(String incoming, String delimiter) {
        List<String> retVal = new ArrayList<String>();
        if (incoming == null || delimiter == null) {
            return retVal;            
        }
        String[] asArray = incoming.split(delimiter);
        for (String curr : asArray) {
            retVal.add(curr);
        }
        return retVal;
    }

    public static boolean nullSafeCompare(String snapMirrorScheduleInfo, String scheduleInfo) {
        return (snapMirrorScheduleInfo == null ? scheduleInfo == null : snapMirrorScheduleInfo.equals(scheduleInfo));
    }
}
