package com.netapp.vee.security;


import java.io.File;
import java.io.IOException;
import java.util.Set;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

import org.apache.log4j.Logger;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

public class CryptUtils  {
    public static final String SECURITY_DIR = getConfDir();
    // if we ever change these from .bin, then PartiallyCreatedDatabaseRemediator needs to change to copy 
    // over things that aren't .bin.
	public static final String SECURITY_FILENAME = "credentials.bin";	
    private static Logger logger = Logger.getLogger(SecurityManager.class);
    
	/**
	 * Debug logging - wrap Dev-ony code in this variable
	 */
	private static final boolean DEBUG = false;
    
     static {
        if (DEBUG) {
            // This is to make sure that we don't leave DEBUG enabled
            Logger.getLogger(SecurityManager.class).warn("WARNING: Developer (NON-PRODUCTION) debug enabled!");
        }
    }
    
	// TODO Change this to dymamic key - Clif
    /** Unique 16-byte key for the AES encryption/decryption */
    private static byte[] keyBytes = new byte[] { 0x01, 0x0c, 0x05, 0x04, 0x0c, 0x04, 0x06, 0x07, 0x0d, 0x03, 0x0f,
        0x08, 0x04, 0x0b, 0x0c, 0x0d };
    
    public static String getSecurityFileName() {
    	return SecurityUtils.addFS(SECURITY_DIR) + SECURITY_FILENAME;
    }
    
	static String getConfDir() {
		if (SecurityUtils.isReal(System.getProperty("conf.dir"))) {
			return System.getProperty("conf.dir");
		}
		return "conf";
	}
    
    
    /**
     * Method used to generate an AES encrypted, Base64 encoded string given a
     * text string.
     *
     * @param string
     *            The string to be encrypted
     * @return The encrypted string
     */
    public static String encryptString(String string) {
        String result = null;
        if (string == null || string.trim().equals("")) {
            return "";
        }

        try {
            SecretKeySpec key = new SecretKeySpec(keyBytes, "AES");
            Cipher cipher = Cipher.getInstance("AES");
            cipher.init(Cipher.ENCRYPT_MODE, key);

            byte[] input = string.getBytes();
            byte[] cipherText = cipher.doFinal(input);

            result = new BASE64Encoder().encode(cipherText);
        }
        catch (Throwable t) {
        	throw new RuntimeException("Error encrypting", t);
        }
        return result;
    }

    /**
     * Method used to decrypt an AES encrypted, Base64 encoded string
     *
     * @param string
     *            The encrypted string
     * @return The decrypted string     
     */
    public static String decryptString(String string) {
        String result = null;
        if (string == null || string.trim().equals("")) {
            return "";
        }

        try {
            SecretKeySpec key = new SecretKeySpec(keyBytes, "AES");
            Cipher cipher = Cipher.getInstance("AES");
            cipher.init(Cipher.DECRYPT_MODE, key);

            byte[] input = new BASE64Decoder().decodeBuffer(string);
            byte[] decipherText = cipher.doFinal(input);

            result = new String(decipherText);
        }
        catch (Throwable t) {
        	throw new RuntimeException("Error decrypting", t);
        }
        return result;
    }
    
    public static void writeEncryptedFile(String contents, String filename) throws IOException {
    	String encryptedString = encryptString(contents);
    	SecurityUtils.writeFile(filename, encryptedString);
    }
    
    public static String readEncryptedFile(String filename) throws IOException {
    	return decryptString(new String(SecurityUtils.readFileBinary(filename)));    	
    }
    
    
    static String readStringFromFile(String lock, String filename, int lineCount) {
        synchronized(lock) {
            try {
                String contents = readEncryptedFile(filename);
                if (DEBUG) {
                    System.out.println("Dump of " + filename + ": " + contents);
                }
                return SecurityUtils.splitIntoList(contents, "\n").get(lineCount);
            } catch (Exception e) {
                return null;
            }
        }
    } 
}
