package com.netapp.vee.security;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import org.apache.log4j.Logger;




public final class SecurityConfig {

	protected static Logger logger = Logger.getLogger(SecurityConfig.class);
    private static Set<String> missingProperties = Collections.synchronizedSet(new HashSet<String>());
    private static String FILENAME = "vee-security.properties";

    private Properties props;

    private static final SecurityConfig INSTANCE = new SecurityConfig();
    
    public static SecurityConfig getInstance() {
        return INSTANCE;
    }
 
    private SecurityConfig() {
        readConfig();
    }

    public Properties getProps() {
		return props;
	}
    
    public String getFileName() {
		return FILENAME;
	}


    public String showConfig() throws Exception {
    	BufferedReader br = new BufferedReader(new InputStreamReader(getClass().getClassLoader().getResourceAsStream(getFileName())));
    	List<String> toReturn = new ArrayList<String>();
    	String line = null;
    	while ((line = br.readLine()) != null) {
    		toReturn.add(line);
    	}
    	br.close();
    	return SecurityUtils.join(toReturn, "\n");
    }

    Properties loadSecurityPropertiesFromDisk() {
        Properties localProps = new Properties();
        try {
            InputStream in = getClass().getClassLoader().getResourceAsStream(getFileName());
            if (in != null) {
            	localProps.load(in);
	            in.close();
            } else {
            	System.out.println(getFileName() + " file not found - using defaults");
            }
        } catch (Throwable t) {
        	t.printStackTrace();
            // whatever - will just have defaults
        }
        return localProps;
    }

    //e.g. turn volume.nearly.full.threshold = 80 into volume.nearly.full.threshold=30
    //
    public void writeToVvolPropertiesFile(List<Pair<String, String>> newValues, String modifiedBy) {
        URL url = getClass().getClassLoader().getResource(getFileName());
        logger.debug("Saw value of " + url + " for location of "+getFileName());
        Date date = new Date();
        String commentExisting = "# Modified by " + modifiedBy + " @ " + date.toString();
        String commentNew = "# Added by " + modifiedBy + " @ " + date.toString();

        try {
            List<String> safeLines = SecurityUtils.getLinesSafe(url.getPath());
            if (safeLines != null) {
                for (Pair<String, String> newPair : newValues) {
                    for (int i = 0; i < safeLines.size(); i++) {
                        //skip comments, #
                        if (!safeLines.get(i).startsWith("#")) {
                            String currentPair[] = safeLines.get(i).split("=");
                            String currentKey = currentPair[0].trim();

                            if (currentKey.equals(newPair.getA())) {
                                String modifiedLine = currentKey + "=" + newPair.getB();
                                safeLines.set(i, modifiedLine);
                                //if there is already a comment replace it, if not, add a new comment
                                if (safeLines.get(i-1).startsWith("# Modified by ") ) {
                                    safeLines.set(i-1, commentExisting);
                                }
                                else {
                                    safeLines.add(i, commentExisting);
                                }
                                break;
                            }
                        }
                            //came to the end and didn't find a key match, appending new pair
                        if (i == (safeLines.size() - 1)) {
                            safeLines.add(commentNew);
                            safeLines.add(newPair.getA() + "=" + newPair.getB());
                            break;//needed since safeLines size just increased by two
                        }
                    }
                }
            }
            //clear the old vvol.properties file
            PrintWriter writer = new PrintWriter(url.getPath());
            writer.print("");
            writer.close();

            SecurityUtils.writeListToFile(url.getPath(), safeLines, true);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void readConfig() {
    	Properties localProps = loadSecurityPropertiesFromDisk();
        props = localProps;
    }
   
	/**
	Just used for unit tests
	*/
    public void setProperties(Properties incoming) {
        props = incoming;
    }

    public void setProperty(String key, String value) {
        props.setProperty(key, value);
    }
    
    /*
     * Getters
     */
    private boolean getBoolean(String key, boolean theDefault) {
    	if ("true".equalsIgnoreCase(""+get(key, null))) {
    		return true;
    	} else if ("false".equalsIgnoreCase(""+get(key, null))) {
    		return false;
    	} else {
    		return theDefault;
    	}
    }

    private long getNumber(String key, long theDefault) {
        try {
            return Long.parseLong(get(key, null));
        } catch (Throwable t){
            // whatever
        }
        return theDefault;
    }
    
    private long getLong(String key, long theDefault) {
         try {
            return Long.parseLong(get(key, null));
        } catch (Throwable t){
            // whatever
        }
        return theDefault;
    }

    private int getInt(String key, int theDefault) {
        try {
            return Integer.parseInt(get(key, null));
        } catch (Throwable t){
            // whatever
        }
        return theDefault;
    }

    private double getDouble(String key, double theDefault) {
    	try {
    		return Double.parseDouble(get(key, null));
    	} catch (Throwable t) {
    		// whatever
    	}
    	return theDefault;
    }

    public String get(String key) {
        return get(key, null);
    }

    public String get(String key, String theDefault) {
        String result = props.getProperty(key);

    	if(result != null) {
    		result = result.trim();
    	} else if (theDefault != null) {
    	    result = theDefault;
    	} else {
    	    if (missingProperties.contains(key)) {
    	        // already barked about this, just chill.
    	    } else {
    	        missingProperties.add(key);
    	        logger.debug("Unable to find vvol config property: " + key + " using default of " + theDefault);
    	    }

    	}

    	return result;
    }


    /*
     * Start config parameters
     */
    public boolean isWebServicesSslv3Enabled() {
        return getBoolean("security.vp.webservices.sslv3.enabled", false);
    }   
	
}
