package com.netapp.vee.security;

public interface SecurityConstants {
    public static final String DEFAULT_PROVIDER_NAME = "BC";
    
    public static final String DEFAULT_KEYSTORE_PROVIDER_NAME = "SUN";
    
    /**
     * Type of keystore that we will create
     */
    public static final String DEFAULT_KEY_STORE_TYPE = "JKS";
    
    public static final String DEFAULT_KEY_PAIR_TYPE = "RSA";
    
    public static final String DEFAULT_RANDOM_NUMBER_GEN = "SHA1PRNG";
    
    public static final int DEFAULT_KEY_SIZE = 2048;
    
    public static final String DEFAULT_SIGNATURE_ALGORITHM = "SHA1withRSA";
    
    public static final String DEFAULT_DIGEST_ALGORITHM = "SHA-1";
    
    /**
     * 1 second in millis
     */
    public static final long MULT_SECOND = 1000;
    
    /**
     * 1 minute in millis
     */
    public static final long MULT_MINUTE = MULT_SECOND * 60;
    
    /**
     * 1 hour in millis
     */
    public static final long MULT_HOUR = MULT_MINUTE * 60;
    
    /**
     * 1 day in millis
     */
    public static final long MULT_DAY = MULT_HOUR * 24;
    
    /**
     * 1 week in millis
     */
    public static final long MULT_WEEK = MULT_DAY * 7;
    
    /**
     * 1 year in millis rounded to nearest day.
     * Fudge in 0.25 days per year to get a closer value
     */
    public static final long MULT_YEAR = MULT_DAY * 365;
    
    /**
     * Default validity for keys, certs, etc.
     */
    public static final long DEFAULT_VALIDITY_MILLIS = MULT_YEAR * 10;
    
    public static final String DEFAULT_OWN_CERT_ALIAS = "own_certificate";
    
    public enum PEMType {
        CERTIFICATE(PEM_TYPE_CERTIFICATE),
        CSR(PEM_TYPE_CSR),
        PRIVATE_KEY(PEM_TYPE_PRIVATE_KEY),
        CRL(PEM_TYPE_CRL);
        
        private final String text;
        
        PEMType(String text) {
            this.text = text;
        }
        
        public String getText() {
            return text;
        }
    };
    
    public static final String PEM_TYPE_CSR = "CERTIFICATE REQUEST";
    public static final String PEM_TYPE_CERTIFICATE = "CERTIFICATE";
    public static final String PEM_TYPE_PRIVATE_KEY = "RSA PRIVATE KEY";
    public static final String PEM_TYPE_CRL = "X509 CRL";
    
    public static final String PEM_DASHES = "-----";
    public static final String PEM_BEGIN = "BEGIN";
    public static final String PEM_END = "END";
    
    public static final String PEM_BEGIN_CERTIFICATE = PEM_DASHES  + PEM_BEGIN + ' ' + PEM_TYPE_CERTIFICATE + PEM_DASHES;
    public static final String PEM_END_CERTIFICATE = PEM_DASHES + PEM_END + ' ' + PEM_TYPE_CERTIFICATE + PEM_DASHES;
    
    public static final String PEM_BEGIN_CSR = PEM_DASHES  + PEM_BEGIN + ' ' + PEM_TYPE_CSR + PEM_DASHES;
    public static final String PEM_END_CSR = PEM_DASHES + PEM_END + ' ' + PEM_TYPE_CSR + PEM_DASHES;
    
    public static final String PEM_BEGIN_CRL = PEM_DASHES  + PEM_BEGIN + ' ' + PEM_TYPE_CRL + PEM_DASHES;
    public static final String PEM_END_CRL = PEM_DASHES + PEM_END + ' ' + PEM_TYPE_CRL + PEM_DASHES;
    
}
