package com.netapp.vee.security;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// TODO clif - Future-proof this code by doing exclusion instead of inclusion
public class TlsServerSocketFactory extends SSLServerSocketFactory {
    private static Log log = LogFactory.getLog(TlsServerSocketFactory.class);
    private SSLServerSocketFactory delegate;
    private boolean useDelegateAlways = false;
    
    public TlsServerSocketFactory( SSLServerSocketFactory delegate ) {
        this.delegate = delegate;
        
        String version = System.getProperty("java.version");
        log.info("Runing on JRE version: " + version);
        // Check for old JRE used on Hudson server
        // No longer needed
        //useDelegateAlways = version.startsWith("1.6") ? true : false;
    }

    @Override
    public String[] getDefaultCipherSuites() {
        return delegate.getDefaultCipherSuites();
    }

    @Override
    public String[] getSupportedCipherSuites() {
        return delegate.getSupportedCipherSuites();
    }

    @Override
    public ServerSocket createServerSocket(int arg0) throws IOException {
        SSLServerSocket socket = (SSLServerSocket) delegate.createServerSocket();
        if (!useDelegateAlways) {
            updateProtocols(socket);
        } else {
            log.warn("TlsServerSocketFactory: WARNING: Skipping updateProtocols!");
        }
        
        return socket;
    }

    @Override
    public ServerSocket createServerSocket(int arg0, int arg1) throws IOException {
        SSLServerSocket socket = (SSLServerSocket) delegate.createServerSocket(arg0, arg1);
        if (!useDelegateAlways) {
            updateProtocols(socket);
        } else {
            log.warn("TlSServerSocketFactory: WARNING: Skipping updateProtocols!");
        }
        
        return socket;
    }

    private void updateProtocols(SSLServerSocket socket) {
       String[] protocols = socket.getEnabledProtocols();
        List<String> protocolList = new ArrayList<String>(Arrays.asList(protocols));
        /**
         *  Remove SSLv2 protocol to enforce stronger communication.
         */
        protocolList.remove("SSLv2Hello");
        
        for (String proto : protocols) {
            if (SecurityConfig.getInstance().isWebServicesSslv3Enabled() &&
                proto.toLowerCase().startsWith("ssl")) {
                protocolList.remove(proto);
            }
        }
        protocols = protocolList.toArray(new String[protocolList.size()]);
        for (String proto : protocolList) {
            log.debug("TLSServerSocketFactory: Supported protocol: " + proto);
        }
        socket.setEnabledProtocols(protocols);
    }
    
    @Override
    public ServerSocket createServerSocket(int arg0, int arg1, InetAddress arg2) throws IOException {
        SSLServerSocket socket = (SSLServerSocket) delegate.createServerSocket(arg0, arg1, arg2);
        if (!useDelegateAlways) {
            updateProtocols(socket);
        } else {
            log.warn("TlSServerSocketFactory: WARNING: Skipping updateProtocols!");
        }
        
        return socket;
    }

}
